import { computed, watch, defineComponent, inject, normalizeClass, createVNode, mergeProps, provide } from "vue";
import { useGlobalConfig, useDateConfig } from "@idux/components/config";
import { useThemeToken } from "@idux/components/theme";
import { useControlledProp, useState, callEmit, convertArray } from "@idux/cdk/utils";
import { IxTooltip } from "@idux/components/tooltip";
import { IxIcon } from "@idux/components/icon";
function useActiveDate(props, dateConfig, activeType) {
  const [activeDate, setActiveDate] = useControlledProp(props, "activeDate", () => dateConfig.now());
  const startActiveDate = computed(() => {
    const currDate = activeDate.value;
    const currType = activeType.value;
    const { startOf, set, get } = dateConfig;
    switch (currType) {
      case "date":
      case "week":
        return startOf(startOf(currDate, "month"), "week");
      case "month":
        return startOf(startOf(currDate, "year"), "month");
      case "quarter":
        return startOf(startOf(currDate, "year"), "quarter");
      case "year": {
        return set(currDate, parseInt(`${get(currDate, "year") / 10}`, 10) * 10 - 1, "year");
      }
      default:
        return currDate;
    }
  });
  return {
    activeDate,
    setActiveDate,
    startActiveDate
  };
}
function useActiveType(props) {
  const [activeType, setActiveType] = useState(props.type);
  watch(() => props.type, setActiveType);
  watch(
    () => props.visible,
    (visible) => visible && setActiveType(props.type)
  );
  return {
    activeType,
    setActiveType
  };
}
function useMaxIndex(activeType) {
  const maxRowIndex = computed(() => {
    const currType = activeType.value;
    switch (currType) {
      case "date":
      case "week":
        return 6;
      case "month":
        return 4;
      case "quarter":
        return 1;
      case "year":
        return 4;
      default:
        return 0;
    }
  });
  const maxCellIndex = computed(() => {
    const currType = activeType.value;
    switch (currType) {
      case "date":
      case "week":
        return 7;
      case "month":
        return 3;
      case "quarter":
        return 4;
      case "year":
        return 3;
      default:
        return 0;
    }
  });
  return {
    maxRowIndex,
    maxCellIndex
  };
}
const datePanelToken = Symbol("datePanelToken");
const datePanelProps = {
  cellTooltip: Function,
  disabledDate: Function,
  type: {
    type: String,
    default: "date"
  },
  value: [Date, Array],
  activeDate: Date,
  visible: {
    type: Boolean,
    default: void 0
  },
  isSelecting: {
    type: Boolean,
    default: void 0
  },
  onCellClick: [Function, Array],
  onCellMouseenter: [Function, Array],
  "onUpdate:activeDate": [Function, Array]
};
const panelRowProps = {
  rowIndex: {
    type: Number,
    required: true
  }
};
const panelCellProps = {
  rowIndex: {
    type: Number,
    required: true
  },
  cellIndex: {
    type: Number,
    required: true
  }
};
function getPanelCellType(type) {
  return type === "week" ? "date" : type;
}
function getPanelCellDisabled(cellDate, panelType, dateConfig, disabledDate) {
  const cellType = getPanelCellType(panelType);
  if (cellType === "date") {
    return disabledDate(cellDate);
  }
  const scopesMap = {
    date: void 0,
    month: "date",
    quarter: "month",
    year: "quarter"
  };
  const stack = [];
  const pushStacks = (date, type) => {
    const scope = scopesMap[type];
    if (!scope) {
      return;
    }
    const [start, end] = ["startOf", "endOf"].map((fn) => dateConfig.get(dateConfig[fn](date, type), scope));
    for (let i = start; i <= end; i++) {
      stack.push({
        date: dateConfig.set(date, i, scope),
        type: scope
      });
    }
  };
  pushStacks(cellDate, cellType);
  while (stack.length) {
    const item = stack.pop();
    if (!disabledDate(item.date)) {
      return false;
    }
    pushStacks(item.date, item.type);
  }
  return true;
}
const labelFormat = {
  date: "d",
  week: "d",
  month: "MMM",
  quarter: "'Q'Q",
  year: "yyyy"
};
var PanelCell = /* @__PURE__ */ defineComponent({
  props: panelCellProps,
  setup(props) {
    const {
      props: panelProps,
      slots,
      mergedPrefixCls,
      dateConfig,
      activeDate,
      setActiveDate,
      startActiveDate,
      activeType,
      setActiveType,
      maxRowIndex,
      maxCellIndex
    } = inject(datePanelToken);
    const offsetIndex = computed(() => props.rowIndex * maxCellIndex.value + props.cellIndex);
    const cellDate = computed(() => {
      return dateConfig.add(startActiveDate.value, offsetIndex.value, getPanelCellType(activeType.value));
    });
    const startDate = computed(() => getDateValue(dateConfig, panelProps.value, activeType.value, true));
    const endDate = computed(() => getDateValue(dateConfig, panelProps.value, activeType.value, false));
    const isDisabled = computed(() => {
      const disabledDate = panelProps.disabledDate;
      if (!disabledDate) {
        return false;
      }
      return getPanelCellDisabled(cellDate.value, activeType.value, dateConfig, disabledDate);
    });
    const cellTooltip = computed(() => {
      var _a;
      return (_a = panelProps.cellTooltip) == null ? void 0 : _a.call(panelProps, {
        value: cellDate.value,
        disabled: !!isDisabled.value
      });
    });
    const isStart = computed(() => startDate.value && dateConfig.isSame(startDate.value, cellDate.value, getPanelCellType(activeType.value)));
    const isEnd = computed(() => endDate.value && dateConfig.isSame(endDate.value, cellDate.value, getPanelCellType(activeType.value)));
    const isCurrent = computed(() => dateConfig.isSame(cellDate.value, dateConfig.now(), getPanelCellType(activeType.value)));
    const outView = computed(() => {
      const currType = activeType.value;
      if (getPanelCellType(activeType.value) === "date") {
        return !dateConfig.isSame(cellDate.value, activeDate.value, "month");
      }
      if (currType === "year") {
        const offset = offsetIndex.value;
        return offset === 0 || offset === maxRowIndex.value * maxCellIndex.value;
      }
      return false;
    });
    const isSelected = computed(() => {
      if (outView.value) {
        return false;
      }
      const compareType = getPanelCellType(activeType.value);
      if (panelProps.isSelecting) {
        return startDate.value && dateConfig.isSame(startDate.value, cellDate.value, compareType);
      }
      return startDate.value && dateConfig.isSame(startDate.value, cellDate.value, compareType) || endDate.value && dateConfig.isSame(endDate.value, cellDate.value, compareType);
    });
    const isInRange = computed(() => {
      const compareType = getPanelCellType(activeType.value);
      const cellDateValue = dateConfig.startOf(cellDate.value, compareType).valueOf();
      return !!startDate.value && !!endDate.value && cellDateValue >= dateConfig.startOf(startDate.value, compareType).valueOf() && cellDateValue <= dateConfig.startOf(endDate.value, compareType).valueOf();
    });
    const classes = computed(() => {
      const prefixCls = `${mergedPrefixCls.value}-cell`;
      return normalizeClass({
        [prefixCls]: true,
        [`${prefixCls}-disabled`]: isDisabled.value,
        [`${prefixCls}-selected`]: isSelected.value,
        [`${prefixCls}-in-range`]: isInRange.value,
        [`${prefixCls}-current`]: isCurrent.value,
        [`${prefixCls}-out-view`]: outView.value,
        [`${prefixCls}-start`]: isStart.value,
        [`${prefixCls}-end`]: isEnd.value
      });
    });
    const handleClick = (evt) => {
      evt.stopPropagation();
      const currDate = cellDate.value;
      if (panelProps.type !== activeType.value) {
        setActiveType(panelProps.type);
        setActiveDate(currDate);
      } else {
        callEmit(panelProps.onCellClick, currDate);
      }
    };
    const handleMouseenter = () => {
      callEmit(panelProps.onCellMouseenter, cellDate.value);
    };
    return () => {
      var _a, _b;
      const currDate = cellDate.value;
      const {
        format
      } = dateConfig;
      const cellNode = (_b = (_a = slots.cell) == null ? void 0 : _a.call(slots, {
        date: currDate
      })) != null ? _b : createVNode("div", {
        "class": `${mergedPrefixCls.value}-cell-inner`
      }, [createVNode("div", {
        "class": `${mergedPrefixCls.value}-cell-trigger`
      }, [format(currDate, labelFormat[activeType.value])])]);
      return createVNode("td", {
        "class": classes.value,
        "role": "gridcell",
        "onClick": isDisabled.value ? void 0 : handleClick,
        "onMouseenter": isDisabled.value ? void 0 : handleMouseenter
      }, [cellTooltip.value ? createVNode(IxTooltip, {
        "title": cellTooltip.value
      }, {
        default: () => [cellNode]
      }) : cellNode]);
    };
  }
});
function getDateValue(dateConfig, value, type, isStart) {
  var _a;
  const valueArr = convertArray(value);
  if (type === "week") {
    return isStart ? dateConfig.startOf(valueArr[0], "week") : dateConfig.endOf((_a = valueArr[1]) != null ? _a : valueArr[0], "week");
  }
  return valueArr[isStart ? 0 : 1];
}
var PanelRow = /* @__PURE__ */ defineComponent({
  props: panelRowProps,
  setup(props) {
    const {
      mergedPrefixCls,
      activeType,
      maxCellIndex
    } = inject(datePanelToken);
    const cells = computed(() => {
      const {
        rowIndex
      } = props;
      const currType = activeType.value;
      const cells2 = [];
      const maxIndex = maxCellIndex.value;
      for (let cellIndex = 0; cellIndex < maxIndex; cellIndex++) {
        cells2.push({
          key: `${currType}-${cellIndex}`,
          rowIndex,
          cellIndex
        });
      }
      return cells2;
    });
    return () => {
      const children = cells.value.map((cell) => createVNode(PanelCell, cell, null));
      return createVNode("tr", {
        "role": "row",
        "class": `${mergedPrefixCls.value}-row`
      }, [children]);
    };
  }
});
var PanelBody = /* @__PURE__ */ defineComponent({
  setup() {
    const {
      mergedPrefixCls,
      dateConfig,
      activeType,
      maxRowIndex,
      maxCellIndex
    } = inject(datePanelToken);
    const theadCells = useTheadCells(dateConfig, activeType, maxCellIndex);
    return () => {
      const prefixCls = `${mergedPrefixCls.value}-body`;
      const thNodes = theadCells.value.map((cell) => {
        const {
          key,
          label
        } = cell;
        return createVNode("th", {
          "role": "columnheader",
          "key": key
        }, [createVNode("div", {
          "class": `${prefixCls}-header-cell`
        }, [label])]);
      });
      const maxIndex = maxRowIndex.value;
      const rows = [];
      for (let rowIndex = 0; rowIndex < maxIndex; rowIndex++) {
        rows.push(createVNode(PanelRow, {
          "key": rowIndex,
          "rowIndex": rowIndex
        }, null));
      }
      return createVNode("div", {
        "class": prefixCls
      }, [createVNode("table", {
        "role": "grid"
      }, [thNodes.length > 0 && createVNode("thead", null, [createVNode("tr", {
        "role": "row"
      }, [thNodes])]), createVNode("tbody", null, [rows])])]);
    };
  }
});
function useTheadCells(dateConfig, activeType, maxCellIndex) {
  return computed(() => {
    const currType = activeType.value;
    const cols = [];
    if (currType === "date" || currType === "week") {
      const maxIndex = maxCellIndex.value;
      const labels = dateConfig.getLocalizedLabels("day", maxIndex, "narrow");
      const weekStarts = dateConfig.weekStartsOn();
      for (let colIndex = 0; colIndex < maxIndex; colIndex++) {
        cols.push({
          key: colIndex,
          label: labels[(colIndex + weekStarts) % maxIndex]
        });
      }
    }
    return cols;
  });
}
const hidePrevNextTypes = ["month", "quarter", "year"];
var PanelHeader = /* @__PURE__ */ defineComponent({
  setup() {
    const {
      locale,
      mergedPrefixCls,
      dateConfig,
      activeDate,
      setActiveDate,
      activeType,
      setActiveType
    } = inject(datePanelToken);
    const contents = useContents(activeType, activeDate, locale, dateConfig, setActiveType);
    const handleSuperPrevClick = () => {
      const offsetYear = activeType.value === "year" ? -10 : -1;
      setActiveDate(dateConfig.add(activeDate.value, offsetYear, "year"));
    };
    const handlePrevClick = () => {
      setActiveDate(dateConfig.add(activeDate.value, -1, "month"));
    };
    const handleSuperNextClick = () => {
      const offsetYear = activeType.value === "year" ? 10 : 1;
      setActiveDate(dateConfig.add(activeDate.value, offsetYear, "year"));
    };
    const handleNextClick = () => {
      setActiveDate(dateConfig.add(activeDate.value, 1, "month"));
    };
    return () => {
      const {
        previousDecade,
        previousYear,
        previousMonth,
        nextDecade,
        nextYear,
        nextMonth
      } = locale.datePicker;
      const currType = activeType.value;
      const prev = !hidePrevNextTypes.includes(activeType.value);
      const next = !hidePrevNextTypes.includes(activeType.value);
      const contentNodes = contents.value.map((item) => {
        const {
          label,
          ...rest
        } = item;
        return createVNode("button", mergeProps({
          "type": "button",
          "tabindex": "-1"
        }, rest), [label]);
      });
      const prefixCls = `${mergedPrefixCls.value}-header`;
      return createVNode("div", {
        "class": prefixCls
      }, [createVNode("button", {
        "class": `${prefixCls}-super-prev${" visible"}`,
        "type": "button",
        "tabindex": "-1",
        "title": currType === "year" ? previousDecade : previousYear,
        "onClick": handleSuperPrevClick
      }, [createVNode(IxIcon, {
        "name": "double-left"
      }, null)]), createVNode("button", {
        "class": `${prefixCls}-prev${prev ? " visible" : ""}`,
        "type": "button",
        "tabindex": "-1",
        "title": previousMonth,
        "onClick": handlePrevClick
      }, [createVNode(IxIcon, {
        "name": "left"
      }, null)]), createVNode("div", {
        "class": `${prefixCls}-content`
      }, [contentNodes]), createVNode("button", {
        "class": `${prefixCls}-super-next${next ? " visible" : ""}`,
        "type": "button",
        "tabindex": "-1",
        "title": nextMonth,
        "onClick": handleNextClick
      }, [createVNode(IxIcon, {
        "name": "right"
      }, null)]), createVNode("button", {
        "class": `${prefixCls}-super-prev${" visible"}`,
        "type": "button",
        "tabindex": "-1",
        "title": currType === "year" ? nextDecade : nextYear,
        "onClick": handleSuperNextClick
      }, [createVNode(IxIcon, {
        "name": "double-right"
      }, null)])]);
    };
  }
});
function useContents(activeType, activeDate, locale, dateConfig, setActiveType) {
  const handleClick = (evt, type) => {
    evt.stopPropagation();
    setActiveType(type);
  };
  return computed(() => {
    const currType = activeType.value;
    const currDate = activeDate.value;
    const {
      yearSelect,
      monthSelect,
      yearFormat,
      monthFormat
    } = locale.datePicker;
    const {
      format,
      get
    } = dateConfig;
    switch (currType) {
      case "date":
      case "week":
        return [{
          key: "year",
          title: yearSelect,
          onClick: (evt) => handleClick(evt, "year"),
          label: format(currDate, yearFormat)
        }, {
          key: "month",
          title: monthSelect,
          onClick: (evt) => handleClick(evt, "month"),
          label: format(currDate, monthFormat)
        }];
      case "month":
      case "quarter":
        return [{
          key: "year",
          title: yearSelect,
          onClick: (evt) => handleClick(evt, "year"),
          label: format(currDate, yearFormat)
        }];
      case "year": {
        const startYear = parseInt(`${get(currDate, "year") / 10}`, 10) * 10;
        const endYear = startYear + 9;
        return [{
          key: "decade",
          label: `${startYear}-${endYear}`
        }];
      }
      default:
        return [];
    }
  });
}
var DatePanel = /* @__PURE__ */ defineComponent({
  name: "\u0275DatePanel",
  props: datePanelProps,
  setup(props, {
    slots
  }) {
    const common = useGlobalConfig("common");
    const {
      globalHashId
    } = useThemeToken();
    const locale = useGlobalConfig("locale");
    const mergedPrefixCls = computed(() => `${common.prefixCls}-date-panel`);
    const dateConfig = useDateConfig();
    const {
      activeType,
      setActiveType
    } = useActiveType(props);
    const {
      activeDate,
      setActiveDate,
      startActiveDate
    } = useActiveDate(props, dateConfig, activeType);
    const {
      maxRowIndex,
      maxCellIndex
    } = useMaxIndex(activeType);
    provide(datePanelToken, {
      props,
      slots,
      locale,
      mergedPrefixCls,
      dateConfig,
      activeType,
      setActiveType,
      activeDate,
      setActiveDate,
      startActiveDate,
      maxRowIndex,
      maxCellIndex
    });
    return () => {
      const prefixCls = mergedPrefixCls.value;
      return createVNode("div", {
        "class": `${prefixCls} ${prefixCls}-${activeType.value} ${globalHashId.value}`
      }, [createVNode(PanelHeader, null, null), createVNode(PanelBody, null, null)]);
    };
  }
});
const \u0275DatePanel = DatePanel;
export { \u0275DatePanel };
