import { defineComponent, ref, onUnmounted, withDirectives, createVNode, vShow, computed, watch, mergeProps } from "vue";
import { isNumber } from "lodash-es";
import { useResizeObserver } from "@idux/cdk/resize";
import { callEmit, throwError, getBoxSizingData } from "@idux/cdk/utils";
import { useGlobalConfig } from "@idux/components/config";
import { useThemeToken } from "@idux/components/theme";
const overflowItemProps = {
  prefixCls: {
    type: String,
    required: true
  },
  display: {
    type: Boolean,
    default: true
  },
  itemKey: {
    type: [Number, String, Symbol],
    required: true
  },
  data: Object,
  onSizeChange: Function
};
const overflowProps = {
  maxLabel: {
    type: [Number, String],
    default: Number.MAX_SAFE_INTEGER
  },
  getKey: {
    type: Function,
    required: true
  },
  prefixCls: {
    type: String,
    required: true
  },
  dataSource: {
    type: Array,
    default: () => []
  }
};
var Item = /* @__PURE__ */ defineComponent({
  name: "IxOverflowItem",
  props: overflowItemProps,
  setup(props, {
    slots
  }) {
    const itemElRef = ref();
    const handleResize = (entry) => {
      var _a;
      return callEmit(props.onSizeChange, entry, (_a = props.itemKey) != null ? _a : "");
    };
    useResizeObserver(itemElRef, handleResize);
    onUnmounted(() => {
      var _a;
      callEmit(props.onSizeChange, void 0, (_a = props.itemKey) != null ? _a : "");
    });
    return () => {
      var _a;
      return withDirectives(createVNode("div", {
        "class": `${props.prefixCls}-item`,
        "ref": itemElRef
      }, [(_a = slots.default) == null ? void 0 : _a.call(slots)]), [[vShow, props.display]]);
    };
  }
});
const restNodeKey = "__IDUX_OVERFLOW_REST";
const suffixNodeKey = "__IDUX_OVERFLOW_SUFFIX";
const responsive = "responsive";
var Overflow = /* @__PURE__ */ defineComponent({
  name: "\u0275Overflow",
  props: overflowProps,
  setup(props, {
    slots
  }) {
    const common = useGlobalConfig("common");
    const {
      globalHashId
    } = useThemeToken();
    const mergedPrefixCls = computed(() => `${common.prefixCls}-overflow`);
    const containerElRef = ref();
    const {
      containerWidth,
      setContainerWidth
    } = useContainerSize(containerElRef);
    const {
      itemsWidthMap,
      setItemWidth
    } = useItemSize();
    const restWidth = ref(0);
    const suffixWidth = ref(0);
    const displayCount = ref(props.dataSource.length);
    const isResponsive = computed(() => props.maxLabel === responsive);
    const restReady = ref(false);
    const showRest = computed(() => isResponsive.value || isNumber(props.maxLabel) && props.dataSource.length > props.maxLabel);
    const mergedData = computed(() => {
      if (!isResponsive.value) {
        return props.dataSource.slice(0, props.maxLabel);
      }
      return props.dataSource;
    });
    const restData = computed(() => props.dataSource.slice(displayCount.value));
    const displayRest = computed(() => restReady.value && !!restData.value.length);
    watch([itemsWidthMap, containerWidth, restWidth, suffixWidth, mergedData], () => {
      var _a;
      const len = props.dataSource.length;
      const lastIndex = len - 1;
      const data = (_a = props.dataSource) != null ? _a : [];
      let totalWidth = 0;
      if (!len) {
        displayCount.value = 0;
        return;
      }
      if (!isResponsive.value) {
        displayCount.value = Math.min(props.maxLabel, len);
        restReady.value = true;
        return;
      }
      const internalContainerWidth = containerWidth.value - suffixWidth.value;
      for (let i = 0; i < len; i++) {
        const getItemWidth = (index) => {
          var _a2;
          return (_a2 = itemsWidthMap.value.get(props.getKey(data[index]))) != null ? _a2 : 0;
        };
        const curItemWidth = getItemWidth(i);
        if (!curItemWidth) {
          displayCount.value = i + 1;
          break;
        }
        restReady.value = true;
        totalWidth += curItemWidth;
        if (i === lastIndex && totalWidth <= internalContainerWidth) {
          displayCount.value = i + 1;
          break;
        } else if (totalWidth + restWidth.value > internalContainerWidth) {
          displayCount.value = i;
          break;
        }
        displayCount.value = i + 1;
      }
    }, {
      deep: true,
      immediate: true,
      flush: "post"
    });
    useResizeObserver(containerElRef, setContainerWidth);
    const itemSharedProps = {
      prefixCls: mergedPrefixCls.value
    };
    const internalRenderItem = (item, index) => {
      var _a, _b;
      if (!slots.item) {
        throwError("components/_private/overflow", "item slot must be provided");
      }
      const nodeContent = (_b = (_a = slots.item) == null ? void 0 : _a.call(slots, item)) != null ? _b : "";
      const key = props.getKey(item);
      return createVNode(Item, mergeProps(itemSharedProps, {
        "key": key,
        "itemKey": key,
        "display": index < displayCount.value,
        "onSizeChange": (entry, key2) => setItemWidth(key2, entry)
      }), {
        default: () => [nodeContent]
      });
    };
    const internalRenderRest = (rest) => {
      var _a, _b;
      const nodeContent = (_b = (_a = slots.rest) == null ? void 0 : _a.call(slots, rest)) != null ? _b : `+ ${rest.length} ...`;
      return createVNode(Item, mergeProps(itemSharedProps, {
        "class": `${mergedPrefixCls.value}-rest`,
        "itemKey": restNodeKey,
        "display": displayRest.value,
        "onSizeChange": (entry) => {
          var _a2;
          return restWidth.value = (_a2 = entry == null ? void 0 : entry.contentRect.width) != null ? _a2 : 0;
        }
      }), {
        default: () => [nodeContent]
      });
    };
    const internalRenderSuffix = () => {
      var _a, _b;
      const nodeContent = (_b = (_a = slots.suffix) == null ? void 0 : _a.call(slots)) != null ? _b : null;
      return nodeContent ? createVNode(Item, mergeProps(itemSharedProps, {
        "class": `${mergedPrefixCls.value}-suffix`,
        "itemKey": suffixNodeKey,
        "onSizeChange": (entry) => {
          var _a2;
          return suffixWidth.value = (_a2 = entry == null ? void 0 : entry.contentRect.width) != null ? _a2 : 0;
        }
      }), {
        default: () => [nodeContent]
      }) : null;
    };
    return () => {
      return createVNode("div", {
        "class": `${mergedPrefixCls.value} ${props.prefixCls}-overflow ${globalHashId.value}`,
        "ref": containerElRef
      }, [mergedData.value.map(internalRenderItem), showRest.value && internalRenderRest(restData.value), internalRenderSuffix()]);
    };
  }
});
const useContainerSize = (containerElRef) => {
  const containerWidth = ref(0);
  const _getContainerWidth = () => {
    const containerEl = containerElRef.value;
    if (!containerEl) {
      return 0;
    }
    const {
      width
    } = containerEl.getBoundingClientRect();
    const {
      paddingLeft,
      paddingRight
    } = getBoxSizingData(containerEl);
    return width - paddingLeft - paddingRight;
  };
  const setContainerWidth = () => {
    containerWidth.value = _getContainerWidth();
  };
  return {
    containerWidth,
    setContainerWidth
  };
};
const useItemSize = () => {
  const itemsWidthMap = ref(/* @__PURE__ */ new Map());
  const setItemWidth = (key, entry) => {
    if (!(entry == null ? void 0 : entry.target) && itemsWidthMap.value.get(key)) {
      itemsWidthMap.value.delete(key);
    } else if (entry == null ? void 0 : entry.contentRect.width) {
      itemsWidthMap.value.set(key, entry.contentRect.width);
    }
  };
  return {
    itemsWidthMap,
    setItemWidth
  };
};
const \u0275Overflow = Overflow;
export { \u0275Overflow };
