import { defineComponent, inject, computed, createVNode, ref, watchEffect, nextTick, watch, provide, mergeProps } from "vue";
import { useGlobalConfig, useDateConfig } from "@idux/components/config";
import { useThemeToken } from "@idux/components/theme";
import { callEmit, isNumeric, cancelRAF, rAF, easeInOutCubic, useControlledProp } from "@idux/cdk/utils";
import { getScroll } from "@idux/cdk/scroll";
const timePanelContext = Symbol("timePanelContext");
const baseTimePanelProps = {
  disabledHours: {
    type: Function,
    default: () => []
  },
  disabledMinutes: {
    type: Function,
    default: () => []
  },
  disabledSeconds: {
    type: Function,
    default: () => []
  },
  hideDisabledOptions: {
    type: Boolean,
    default: false
  },
  hourStep: {
    type: Number,
    default: 1
  },
  minuteStep: {
    type: Number,
    default: 1
  },
  secondStep: {
    type: Number,
    default: 1
  }
};
const timePanelProps = {
  ...baseTimePanelProps,
  value: Date,
  activeValue: Date,
  visible: {
    type: Boolean,
    default: false
  },
  hourEnabled: {
    type: Boolean,
    default: true
  },
  minuteEnabled: {
    type: Boolean,
    default: true
  },
  secondEnabled: {
    type: Boolean,
    default: true
  },
  use12Hours: {
    type: Boolean,
    default: false
  },
  "onUpdate:value": [Function, Array],
  "onUpdate:activeValue": [Function, Array],
  onChange: [Function, Array]
};
const timePanelColumnProps = {
  selectedValue: [String, Number],
  activeValue: {
    type: [String, Number],
    required: true
  },
  options: {
    type: Array,
    require: true
  },
  visible: {
    type: Boolean,
    default: false
  },
  onActiveChange: [Function, Array]
};
const timePanelCellProps = {
  disabled: {
    type: Boolean,
    default: false
  },
  selected: {
    type: Boolean,
    default: false
  },
  value: {
    type: [String, Number],
    required: true
  },
  onActive: [Function, Array]
};
var PanelCell = /* @__PURE__ */ defineComponent({
  props: timePanelCellProps,
  setup(props) {
    const {
      mergedPrefixCls
    } = inject(timePanelContext);
    const classes = computed(() => {
      const prefixCls = `${mergedPrefixCls.value}-cell`;
      return {
        [prefixCls]: true,
        [`${prefixCls}-disabled`]: props.disabled,
        [`${prefixCls}-selected`]: props.selected
      };
    });
    const onClick = (evt) => {
      evt.stopPropagation();
      if (!props.disabled && !props.selected) {
        callEmit(props.onActive, {
          value: props.value,
          disabled: props.disabled
        });
      }
    };
    const displayValue = computed(() => displayFormat(props.value));
    return () => createVNode("li", {
      "class": classes.value,
      "onClick": onClick
    }, [displayValue.value]);
  }
});
function displayFormat(val) {
  return isNumeric(val) ? val.toString().padStart(2, "0") : val;
}
function useColumnEvents(props, listRef, getTargetByScrollTop, frameRunning) {
  const activeIndex = computed(() => {
    var _a, _b;
    return (_b = (_a = props.options) == null ? void 0 : _a.findIndex((option) => option.value === props.activeValue)) != null ? _b : -1;
  });
  const handleWheel = (evt) => {
    var _a;
    evt.preventDefault();
    if (evt.shiftKey || !listRef.value || !props.options || activeIndex.value >= props.options.length) {
      return;
    }
    const target = (_a = props.options) == null ? void 0 : _a[activeIndex.value + (evt.deltaY > 0 ? 1 : -1)];
    if (target) {
      callEmit(props.onActiveChange, target);
    }
  };
  const handleScroll = () => {
    if (frameRunning.value) {
      return;
    }
    const target = getTargetByScrollTop();
    if (target) {
      callEmit(props.onActiveChange, target);
    }
  };
  const handleClick = (evt) => {
    var _a, _b;
    const children = (_a = listRef.value) == null ? void 0 : _a.children;
    if (!children) {
      return;
    }
    evt.preventDefault();
    for (let i = 0; i < children.length; i++) {
      const currentLiY = children[i].getBoundingClientRect().y;
      const nextLiY = (_b = children[i + 1]) == null ? void 0 : _b.getBoundingClientRect().y;
      if (currentLiY < evt.clientY && (!nextLiY || nextLiY > evt.clientY)) {
        children[i].dispatchEvent(new Event("click"));
        return;
      }
    }
  };
  return {
    handleWheel,
    handleClick,
    handleScroll
  };
}
function usePanelScroll(props, listRef, mergedPrefixCls) {
  const { frameRunning, scrollTo } = useScrollTo(listRef);
  const getCellHeight = () => {
    var _a, _b, _c;
    return (_c = (_b = (_a = listRef.value) == null ? void 0 : _a.querySelector(`li.${mergedPrefixCls.value}-cell`)) == null ? void 0 : _b.offsetHeight) != null ? _c : 0;
  };
  const adjustPanel = (selectedIndex, duration = 200) => {
    const target = listRef.value;
    if (!target) {
      return;
    }
    const top = Math.max(selectedIndex * getCellHeight(), 0);
    if (top === getScroll(target).scrollTop) {
      return;
    }
    scrollTo(top, duration);
  };
  const scrollToActive = (duration) => {
    const activeIndex = props.options.findIndex((item) => item.value === props.activeValue);
    adjustPanel(activeIndex, duration);
  };
  const getTargetByScrollTop = () => {
    const target = listRef.value;
    if (!target) {
      return;
    }
    const scrollTargetIndex = Math.min(
      Math.round(getScroll(target).scrollTop / getCellHeight()),
      props.options.length - 1
    );
    return props.options[scrollTargetIndex];
  };
  return {
    adjustPanel,
    scrollToActive,
    getTargetByScrollTop,
    frameRunning
  };
}
function useScrollTo(target) {
  const frameRunning = ref(false);
  let rafId;
  let currentTop;
  let currentScrollTop;
  let currentDuration;
  let currentStartTime;
  const frameFunc = (time) => {
    if (!target.value) {
      return;
    }
    const elapsed = time > currentDuration ? currentDuration : time;
    const amountOfChange = currentTop - currentScrollTop;
    const nextScrollTop = easeInOutCubic(elapsed, currentScrollTop, amountOfChange, currentDuration);
    target.value.scrollTop = nextScrollTop;
  };
  const execFrame = () => {
    cancelRAF(rafId);
    rafId = rAF(() => {
      frameRunning.value = true;
      const time = Date.now() - currentStartTime;
      frameFunc(time);
      if (time < currentDuration) {
        execFrame();
      } else {
        currentTop = 0;
        currentDuration = 0;
        currentStartTime = 0;
        rAF(() => {
          frameRunning.value = false;
        });
      }
    });
  };
  const scrollTo = (top, duration = 100) => {
    if (!target.value) {
      return;
    }
    currentDuration = duration;
    currentTop = top;
    currentScrollTop = getScroll(target.value).scrollTop;
    if (!frameRunning.value) {
      currentStartTime = Date.now();
    }
    execFrame();
  };
  return {
    frameRunning,
    scrollTo
  };
}
var PanelColumn = /* @__PURE__ */ defineComponent({
  props: timePanelColumnProps,
  setup(props) {
    const {
      mergedPrefixCls
    } = inject(timePanelContext);
    const listRef = ref(void 0);
    const {
      scrollToActive,
      getTargetByScrollTop,
      frameRunning
    } = usePanelScroll(props, listRef, mergedPrefixCls);
    function onActive(cell) {
      callEmit(props.onActiveChange, cell);
    }
    function handleScrollAdjust() {
      scrollToActive();
    }
    const {
      handleWheel,
      handleClick,
      handleScroll: _handleScroll
    } = useColumnEvents(props, listRef, getTargetByScrollTop, frameRunning);
    const handleListWheel = (evt) => {
      evt.preventDefault();
    };
    let isScrolling = false;
    const handleScroll = () => {
      isScrolling = true;
      _handleScroll();
      nextTick(() => {
        isScrolling = false;
      });
    };
    watchEffect(() => {
      if (props.visible) {
        nextTick(() => scrollToActive(0));
      }
    });
    watch(() => props.activeValue, () => {
      !isScrolling && nextTick(scrollToActive);
    });
    return () => createVNode("div", {
      "class": `${mergedPrefixCls.value}-column`,
      "onMouseleave": handleScrollAdjust,
      "onMouseenter": handleScrollAdjust,
      "onWheel": handleWheel,
      "onClick": handleClick
    }, [createVNode("ul", {
      "ref": listRef,
      "class": `${mergedPrefixCls.value}-column-inner`,
      "onScroll": handleScroll,
      "onWheel": handleListWheel
    }, [props.options.map((item, index) => {
      const {
        disabled,
        value
      } = item;
      return createVNode(PanelCell, {
        "key": index,
        "disabled": disabled,
        "selected": props.selectedValue === value,
        "value": value,
        "onActive": onActive
      }, null);
    })])]);
  }
});
function normalizeAmPm(hour, is12Hours = false) {
  if (!is12Hours) {
    return;
  }
  return hour >= 12 ? "pm" : "am";
}
function calculateViewHour(hour, is12Hours) {
  if (is12Hours) {
    hour > 12 && (hour -= 12);
    hour === 0 && (hour = 12);
  }
  return hour;
}
function getHourValue(hour, ampm) {
  if (ampm) {
    ampm = ampm.toLowerCase();
    if (ampm === "am") {
      hour >= 12 && (hour -= 12);
    }
    if (ampm === "pm") {
      hour < 12 && (hour += 12);
    }
  }
  return hour;
}
function calculateValue(dateConfig, dateNow, type, is12Hours, value) {
  const { get, set } = dateConfig;
  const selectNumber = Number(value);
  const newDate = new Date(dateNow);
  switch (type) {
    case "hour":
      return set(newDate, getHourValue(selectNumber, normalizeAmPm(get(dateNow, "hour"), is12Hours)), "hour");
    case "minute":
      return set(newDate, selectNumber, "minute");
    case "second":
      return set(newDate, selectNumber, "second");
    case "AM/PM":
      return set(newDate, getHourValue(get(newDate, "hour"), value.toString()), "hour");
  }
}
function useOptions(props, dateConfig) {
  const { get } = dateConfig;
  const [activeValue, setActiveValue] = useControlledProp(props, "activeValue", () => dateConfig.now());
  const [selectedValue, setSelectedValue] = useControlledProp(props, "value");
  const viewHours = computed(
    () => selectedValue.value && calculateViewHour(get(selectedValue.value, "hour"), props.use12Hours)
  );
  const getAmPm = (value) => value && normalizeAmPm(get(value, "hour"), props.use12Hours);
  const ampm = computed(() => getAmPm(selectedValue.value));
  function getOptions(type) {
    const getHourOptions = () => {
      const options = generateNumericOptions(
        props.use12Hours ? 12 : 24,
        props.hourStep,
        props.disabledHours(ampm.value),
        props.hideDisabledOptions
      );
      if (props.use12Hours) {
        const zeroOpt = options.find((opt) => opt.value === 0);
        zeroOpt && (zeroOpt.value = 12);
      }
      return options;
    };
    switch (type) {
      case "AM/PM":
        return generateAmPmOptions("", props.hideDisabledOptions);
      case "second":
        return generateNumericOptions(
          60,
          props.secondStep,
          props.disabledSeconds(
            viewHours.value,
            selectedValue.value && get(selectedValue.value, "minute"),
            ampm.value
          ),
          props.hideDisabledOptions
        );
      case "minute":
        return generateNumericOptions(
          60,
          props.minuteStep,
          props.disabledMinutes(viewHours.value, ampm.value),
          props.hideDisabledOptions
        );
      case "hour":
        return getHourOptions();
      default:
        return [];
    }
  }
  function getColumnValue(value, type) {
    switch (type) {
      case "AM/PM":
        return getAmPm(value);
      case "hour":
      default:
        return calculateViewHour(get(value, "hour"), props.use12Hours);
      case "minute":
        return get(value, "minute");
      case "second":
        return get(value, "second");
    }
  }
  function getOnActiveChange(type) {
    const onActiveChange = (type2, cell) => {
      var _a;
      const newValue = calculateValue(
        dateConfig,
        (_a = selectedValue.value) != null ? _a : activeValue.value,
        type2,
        props.use12Hours,
        cell.value
      );
      setActiveValue(newValue);
      if (!cell.disabled) {
        setSelectedValue(newValue);
        callEmit(props.onChange, newValue);
      }
    };
    return (cell) => onActiveChange(type, cell);
  }
  const getProps = (type) => {
    return computed(
      () => ({
        activeValue: getColumnValue(activeValue.value, type),
        selectedValue: selectedValue.value && getColumnValue(selectedValue.value, type),
        options: getOptions(type),
        onActiveChange: getOnActiveChange(type)
      })
    );
  };
  return {
    hourOptionsProps: getProps("hour"),
    minuteOptionsProps: getProps("minute"),
    secondOptionsProps: getProps("second"),
    amPmOptionsProps: getProps("AM/PM")
  };
}
function generateNumericOptions(total, step, disabledOption, hideDisabledOptions) {
  const options = [];
  for (let index = 0; index < total; index += step) {
    const isDisabled = disabledOption.includes(index);
    if (!isDisabled || !hideDisabledOptions) {
      options.push({
        value: index,
        disabled: isDisabled
      });
    }
  }
  return options;
}
function generateAmPmOptions(disabledOption, hideDisabledOptions) {
  disabledOption = disabledOption.toLowerCase();
  return ["am", "pm"].map((item) => ({
    disabled: disabledOption === item,
    value: item
  })).filter((item) => !hideDisabledOptions || !item.disabled);
}
var TimePanel = /* @__PURE__ */ defineComponent({
  name: "\u0275TimePanel",
  props: timePanelProps,
  setup(props) {
    const common = useGlobalConfig("common");
    const {
      globalHashId,
      hashId
    } = useThemeToken("timePicker");
    const mergedPrefixCls = computed(() => `${common.prefixCls}-time-panel`);
    const dateConfig = useDateConfig();
    const {
      hourOptionsProps,
      minuteOptionsProps,
      secondOptionsProps,
      amPmOptionsProps
    } = useOptions(props, dateConfig);
    provide(timePanelContext, {
      mergedPrefixCls
    });
    const columns = computed(() => {
      const result = [];
      props.hourEnabled && result.push(hourOptionsProps.value);
      props.minuteEnabled && result.push(minuteOptionsProps.value);
      props.secondEnabled && result.push(secondOptionsProps.value);
      props.use12Hours && result.push(amPmOptionsProps.value);
      return result;
    });
    return () => createVNode("div", {
      "class": [mergedPrefixCls.value, globalHashId.value, hashId.value]
    }, [columns.value.map((item, index) => createVNode(PanelColumn, mergeProps({
      "key": index
    }, item, {
      "visible": props.visible
    }), null))]);
  }
});
const \u0275TimePanel = TimePanel;
export { calculateViewHour as \u0275CalculateViewHour, normalizeAmPm as \u0275NormalizeAmPm, \u0275TimePanel, baseTimePanelProps as \u0275baseTimePanelProps };
