import { defineComponent, computed, ref, onMounted, watch, onBeforeUnmount, normalizeClass, createVNode } from "vue";
import { isArray, toString, isNil } from "lodash-es";
import { useSharedFocusMonitor } from "@idux/cdk/a11y";
import { getFirstValidNode, useState, isEmptyNode, callEmit, isFocusable } from "@idux/cdk/utils";
import { useGlobalConfig } from "@idux/components/config";
import { IxIcon } from "@idux/components/icon";
const ProxyNode = (_, {
  slots
}) => {
  var _a;
  return getFirstValidNode((_a = slots.default) == null ? void 0 : _a.call(slots));
};
const triggerProps = {
  value: { type: null, default: void 0 },
  raw: { type: Boolean, default: false },
  borderless: { type: Boolean, default: void 0 },
  clearable: { type: Boolean, default: void 0 },
  clearIcon: { type: String, default: "close-circle" },
  disabled: { type: Boolean, default: void 0 },
  focused: { type: Boolean, default: void 0 },
  monitorFocus: { type: Boolean, default: true },
  paddingless: { type: Boolean, default: false },
  placeholder: String,
  readonly: { type: Boolean, default: void 0 },
  size: { type: String, default: "md" },
  status: String,
  suffix: String,
  suffixRotate: { type: [Boolean, Number, String], default: void 0 },
  onClear: [Array, Function],
  onFocus: [Array, Function],
  onBlur: [Array, Function]
};
const hiddenBoxStyle = {
  width: 0,
  height: 0,
  position: "absolute",
  overflow: "hidden",
  opacity: 0
};
var Trigger = /* @__PURE__ */ defineComponent({
  props: triggerProps,
  setup(props, {
    expose,
    slots: _slots
  }) {
    const slots = _slots;
    const common = useGlobalConfig("common");
    const mergedPrefixCls = computed(() => `${common.prefixCls}-trigger`);
    const ariaLivePoliteValue = computed(() => isArray(props.value) ? props.value.join(",") : toString(props.value));
    const mergedClearable = computed(() => {
      return !props.disabled && !props.readonly && props.clearable && (isArray(props.value) ? !!props.value.length : !isNil(props.value));
    });
    const [focused, setFocused] = useState(false);
    const mergedFocused = computed(() => {
      var _a;
      return (_a = props.focused) != null ? _a : focused.value;
    });
    const focusMonitor = useSharedFocusMonitor();
    const triggerRef = ref();
    const focus = (options) => focusMonitor.focusVia(triggerRef.value, "program", options);
    const blur = () => focusMonitor.blurVia(triggerRef.value);
    expose({
      focus,
      blur
    });
    let monitorStop;
    const initMonitor = () => {
      monitorStop == null ? void 0 : monitorStop();
      if (props.monitorFocus) {
        monitorStop = watch(focusMonitor.monitor(triggerRef.value, true), (evt) => {
          const {
            origin,
            event
          } = evt;
          if (event) {
            if (origin) {
              setFocused(true);
              callEmit(props.onFocus, event);
            } else {
              setFocused(false);
              callEmit(props.onBlur, event);
            }
          }
        });
      }
    };
    onMounted(() => {
      watch(() => props.monitorFocus, initMonitor, {
        immediate: true
      });
    });
    onBeforeUnmount(() => {
      monitorStop == null ? void 0 : monitorStop();
      focusMonitor.stopMonitoring(triggerRef.value);
    });
    const classes = computed(() => {
      const prefixCls = mergedPrefixCls.value;
      const {
        size,
        status,
        borderless,
        paddingless,
        disabled,
        readonly,
        raw
      } = props;
      if (raw) {
        return;
      }
      return normalizeClass({
        [prefixCls]: true,
        [`${prefixCls}-${size}`]: true,
        [`${prefixCls}-${status}`]: !!status,
        [`${prefixCls}-borderless`]: borderless,
        [`${prefixCls}-paddingless`]: paddingless,
        [`${prefixCls}-with-suffix`]: slots.suffix || props.suffix || mergedClearable.value,
        [`${prefixCls}-disabled`]: disabled,
        [`${prefixCls}-focused`]: mergedFocused.value,
        [`${prefixCls}-readonly`]: readonly
      });
    });
    const handleMouseDown = (evt) => {
      if (isFocusable(evt.target)) {
        return;
      }
      const {
        disabled,
        readonly
      } = props;
      if (disabled || readonly || props.focused) {
        evt.preventDefault();
      }
    };
    const handleClear = (evt) => {
      if (props.disabled) {
        return;
      }
      evt.stopPropagation();
      callEmit(props.onClear, evt);
    };
    const renderSuffix = () => {
      var _a, _b;
      if (!(slots.suffix || props.suffix)) {
        return null;
      }
      return createVNode("div", {
        "key": "__suffix",
        "class": `${mergedPrefixCls.value}-suffix`
      }, [(_b = (_a = slots.suffix) == null ? void 0 : _a.call(slots)) != null ? _b : props.suffix && createVNode(IxIcon, {
        "rotate": props.suffixRotate,
        "name": props.suffix
      }, null)]);
    };
    const renderClearIcon = () => {
      if (!mergedClearable.value) {
        return null;
      }
      return createVNode("span", {
        "key": "__clear",
        "class": `${mergedPrefixCls.value}-clear-icon`,
        "onClick": handleClear
      }, [slots.clearIcon ? slots.clearIcon() : props.clearIcon && createVNode(IxIcon, {
        "name": props.clearIcon
      }, null)]);
    };
    return () => {
      var _a, _b, _c;
      const {
        value,
        borderless,
        disabled,
        readonly,
        raw,
        size,
        status,
        suffix,
        suffixRotate,
        clearIcon
      } = props;
      const defaultSlotParams = {
        value,
        borderless,
        disabled,
        readonly,
        focused: mergedFocused.value,
        size,
        status,
        suffix,
        suffixRotate,
        clearable: mergedClearable.value,
        clearIcon
      };
      const defaultSlotNodes = (_a = slots.default) == null ? void 0 : _a.call(slots, defaultSlotParams);
      if (raw && (defaultSlotNodes == null ? void 0 : defaultSlotNodes.length) === 1) {
        const node = defaultSlotNodes[0];
        return createVNode(ProxyNode, {
          "ref": triggerRef
        }, {
          default: () => [node]
        });
      }
      const defaultSlotEmpty = isEmptyNode(defaultSlotNodes);
      return createVNode("div", {
        "ref": triggerRef,
        "class": classes.value,
        "tabindex": -1,
        "onMousedown": handleMouseDown
      }, [props.focused && createVNode("span", {
        "style": hiddenBoxStyle,
        "aria-live": "polite"
      }, [ariaLivePoliteValue.value]), defaultSlotEmpty ? createVNode("span", {
        "class": `${mergedPrefixCls.value}-placeholder`
      }, [(_c = (_b = slots.placeholder) == null ? void 0 : _b.call(slots)) != null ? _c : props.placeholder]) : defaultSlotNodes, !raw && renderSuffix(), !raw && renderClearIcon()]);
    };
  }
});
const \u0275Trigger = Trigger;
const \u0275TriggerPropsDefs = triggerProps;
export { \u0275Trigger, \u0275TriggerPropsDefs };
