import { defineComponent, computed, ref, watch, nextTick, onMounted, onUnmounted, createVNode } from "vue";
import { convertNumber, off, isHTMLElement, on, throttleRAF } from "@idux/cdk/utils";
import { useGlobalConfig } from "@idux/components/config";
import { convertTarget } from "@idux/components/utils";
import { isObject, isUndefined } from "lodash-es";
const affixProps = {
  offset: {
    type: [Number, String, Object],
    default: 0
  },
  target: [String, HTMLElement, Function],
  onChange: Function
};
const events = ["scroll", "resize"];
const directions = ["top", "bottom", "left", "right"];
function normalizeOffset(offset) {
  if (!isObject(offset)) {
    return { top: convertNumber(offset) };
  }
  const _offset = {};
  Object.keys(offset).forEach((dire) => {
    _offset[dire] = convertNumber(offset[dire]);
  });
  return _offset;
}
function getTargetRect(target, container) {
  const targetRect = target.getBoundingClientRect();
  const containerRect = isHTMLElement(container) ? container.getBoundingClientRect() : { top: 0, bottom: window.innerHeight, left: 0, right: window.innerWidth };
  if (targetRect.width === 0 && targetRect.height === 0) {
    return null;
  }
  return {
    top: targetRect.top - containerRect.top,
    left: targetRect.left - containerRect.left,
    bottom: containerRect.bottom - targetRect.bottom,
    right: containerRect.right - targetRect.right
  };
}
function getTargetSize(target) {
  if (target === window) {
    return {
      width: target.innerWidth,
      height: target.innerHeight
    };
  }
  return {
    width: target.offsetWidth,
    height: target.offsetHeight
  };
}
function observeTarget(target, cb) {
  events.forEach((event) => {
    on(target, event, cb);
  });
}
function removeObserveTarget(target, cb) {
  events.forEach((event) => {
    off(target, event, cb);
  });
}
function isDireSticky(dire, affixRect, offsetOption) {
  return !isUndefined(offsetOption[dire]) && affixRect[dire] <= offsetOption[dire];
}
function isSticky(target, container, offsetOption) {
  const targetRect = getTargetRect(target, container);
  return !!targetRect && directions.some((dire) => isDireSticky(dire, targetRect, offsetOption));
}
function calcStickyPosition(target, container, offsetOption) {
  const targetRect = getTargetRect(target, container);
  if (!targetRect) {
    return {};
  }
  const style = {};
  style.position = container === window ? "fixed" : "absolute";
  const _directions = [
    isDireSticky("bottom", targetRect, offsetOption) ? "bottom" : "top",
    isDireSticky("right", targetRect, offsetOption) ? "right" : "left"
  ];
  _directions.forEach((dire) => {
    if (isDireSticky(dire, targetRect, offsetOption)) {
      style[dire] = `${offsetOption[dire] - (container === window ? 0 : targetRect[dire])}px`;
    } else {
      style[dire] = `${container === window ? targetRect[dire] : 0}px`;
    }
  });
  return style;
}
var Affix = /* @__PURE__ */ defineComponent({
  name: "IxAffix",
  props: affixProps,
  setup(props, {
    slots,
    expose
  }) {
    const common = useGlobalConfig("common");
    const mergedPrefixCls = computed(() => `${common.prefixCls}-affix`);
    const contentStyle = ref({});
    const affixStyle = ref({});
    const targetRef = ref(null);
    const affixRef = ref(null);
    const contentRef = ref(null);
    const offset = computed(() => normalizeOffset(props.offset));
    const isStickyRef = ref(false);
    watch(isStickyRef, (value) => {
      var _a;
      return (_a = props.onChange) == null ? void 0 : _a.call(props, value);
    });
    const throttleMeasure = throttleRAF(measure);
    function measure(event) {
      if (!affixRef.value || !targetRef.value || !contentRef.value) {
        clearStyle();
        return;
      }
      isStickyRef.value = isSticky(affixRef.value, targetRef.value, offset.value);
      if (!isStickyRef.value) {
        clearStyle();
        return;
      }
      if (event && event.type === "resize") {
        clearStyle();
        nextTick(measure);
        return;
      }
      const {
        width,
        height
      } = getTargetSize(contentRef.value);
      contentStyle.value = {
        ...calcStickyPosition(affixRef.value, targetRef.value, offset.value),
        width: `${width}px`,
        height: `${height}px`
      };
      affixStyle.value = {
        width: `${width}px`,
        height: `${height}px`
      };
      if (targetRef.value !== window) {
        affixStyle.value.position = "relative";
      }
    }
    function clearStyle() {
      affixStyle.value = {};
      contentStyle.value = {};
    }
    function initContainer() {
      targetRef.value = convertTarget(props.target);
      observeTarget(targetRef.value, throttleMeasure);
    }
    onMounted(() => {
      nextTick(() => {
        initContainer();
        measure();
      });
    });
    onUnmounted(() => {
      var _a;
      removeObserveTarget(targetRef.value, throttleMeasure);
      (_a = throttleMeasure.cancel) == null ? void 0 : _a.call(throttleMeasure);
    });
    watch(() => props.offset, measure);
    watch(() => props.target, () => {
      removeObserveTarget(targetRef.value, throttleMeasure);
      initContainer();
      measure();
    });
    expose({
      update: throttleMeasure
    });
    return () => {
      var _a;
      const prefixCls = mergedPrefixCls.value;
      return createVNode("div", {
        "ref": affixRef,
        "style": affixStyle.value,
        "class": prefixCls
      }, [createVNode("div", {
        "ref": contentRef,
        "class": `${prefixCls}-content`,
        "style": contentStyle.value
      }, [(_a = slots.default) == null ? void 0 : _a.call(slots)])]);
    };
  }
});
const IxAffix = Affix;
export { IxAffix };
