import { computed, defineComponent, watch, normalizeClass, createVNode, Transition } from "vue";
import { isObject, isNil } from "lodash-es";
import { useState, callEmit, flattenNode, convertArray } from "@idux/cdk/utils";
import { useGlobalConfig } from "@idux/components/config";
import { IxIcon } from "@idux/components/icon";
import { IxPagination } from "@idux/components/pagination";
import { getAlphaColor, useThemeToken } from "@idux/components/theme";
import { convertStringVNode } from "@idux/components/utils";
function useCloseable(props, config) {
  const closeable = computed(() => {
    var _a;
    return (_a = props.closable) != null ? _a : config.closable;
  });
  const [visible, setVisible] = useState(true);
  const handleClose = async () => {
    const result = await callEmit(props.onBeforeClose);
    if (result === false) {
      return;
    }
    setVisible(false);
    callEmit(props.onClose);
  };
  return { closeable, visible, handleClose };
}
const alertProps = {
  banner: { type: Boolean, default: false },
  closable: { type: Boolean, default: void 0 },
  closeIcon: { type: String, default: "close" },
  description: String,
  icon: String,
  title: [String, Array],
  pagination: {
    type: [Boolean, Object],
    default: false
  },
  type: {
    type: String,
    default: "warning"
  },
  onAfterClose: [Function, Array],
  onBeforeClose: [Function, Array],
  onClose: [Function, Array]
};
function getDefaultThemeTokens(tokens, algorithms) {
  const {
    heightMd,
    borderRadiusSm,
    alertCompColorAlpha,
    colorContainerBg,
    colorSuccessText,
    colorSuccessBg,
    colorInfoText,
    colorInfoBg,
    colorErrorText,
    colorErrorBg,
    colorOfflineText,
    colorOffline
  } = tokens;
  const { getColorPalette, getBaseColors } = algorithms;
  const bronzeColorPalette = getColorPalette(getBaseColors().bronze);
  return {
    height: heightMd,
    borderRadius: borderRadiusSm,
    successTextColor: colorSuccessText,
    successBgColor: getAlphaColor(colorSuccessBg, alertCompColorAlpha, colorContainerBg),
    infoTextColor: colorInfoText,
    infoBgColor: getAlphaColor(colorInfoBg, alertCompColorAlpha, colorContainerBg),
    warningTextColor: bronzeColorPalette.d10,
    warningBgColor: getAlphaColor(getBaseColors().gold, alertCompColorAlpha, colorContainerBg),
    errorTextColor: colorErrorText,
    errorBgColor: getAlphaColor(colorErrorBg, alertCompColorAlpha, colorContainerBg),
    offlineTextColor: colorOfflineText,
    offlineBgColor: getAlphaColor(colorOffline, alertCompColorAlpha, colorContainerBg)
  };
}
const getThemeTokens = (tokens, presetTheme, algorithms) => {
  return presetTheme === "default" ? getDefaultThemeTokens(tokens, algorithms) : getDefaultThemeTokens(tokens, algorithms);
};
var Alert = /* @__PURE__ */ defineComponent({
  name: "IxAlert",
  props: alertProps,
  setup(props, {
    slots
  }) {
    const common = useGlobalConfig("common");
    const {
      globalHashId,
      hashId,
      registerToken
    } = useThemeToken("alert");
    registerToken(getThemeTokens);
    const mergedPrefixCls = computed(() => `${common.prefixCls}-alert`);
    const config = useGlobalConfig("alert");
    const [pageIndex, setPageIndex] = useState(1);
    const mergedIcon = computed(() => {
      if (props.icon !== void 0) {
        return props.icon;
      }
      const iconConfig = config.icon;
      return isObject(iconConfig) ? iconConfig[props.type] : iconConfig;
    });
    const {
      closeable,
      visible,
      handleClose
    } = useCloseable(props, config);
    const handlePageChange = (index) => {
      setPageIndex(index);
    };
    watch(() => props.pagination, (pagination) => {
      if (isObject(pagination) && !isNil(pagination.pageIndex)) {
        setPageIndex(pagination.pageIndex);
      }
    }, {
      deep: true,
      immediate: true
    });
    watch(pageIndex, (index) => {
      const {
        pagination
      } = props;
      isObject(pagination) && callEmit(pagination.onChange, index);
    });
    const classes = computed(() => {
      const prefixCls = mergedPrefixCls.value;
      return normalizeClass({
        [globalHashId.value]: !!globalHashId.value,
        [hashId.value]: !!hashId.value,
        [prefixCls]: true,
        [`${prefixCls}-${props.type}`]: true,
        [`${prefixCls}-banner`]: props.banner,
        [`${prefixCls}-with-description`]: slots.description || props.description
      });
    });
    const handleAfterLeave = () => callEmit(props.onAfterClose);
    return () => {
      const {
        title,
        pagination
      } = props;
      const titleChildren = slots.default ? flattenNode(slots.default()) : convertArray(title);
      const isPagination = pagination && titleChildren.length > 1;
      const titleNodes = isPagination ? titleChildren[pageIndex.value - 1] : titleChildren;
      const iconNode = slots.icon ? slots.icon() : mergedIcon.value && createVNode(IxIcon, {
        "name": mergedIcon.value
      }, null);
      const descriptionNode = convertStringVNode(slots, props, "description");
      const prefixCls = mergedPrefixCls.value;
      return createVNode(Transition, {
        "name": prefixCls,
        "onAfterLeave": handleAfterLeave
      }, {
        default: () => [visible.value && createVNode("div", {
          "class": classes.value
        }, [iconNode && createVNode("span", {
          "class": `${prefixCls}-icon`
        }, [iconNode]), createVNode("div", {
          "class": `${prefixCls}-content`
        }, [createVNode("div", {
          "class": `${prefixCls}-title`
        }, [titleNodes]), descriptionNode && createVNode("div", {
          "class": `${prefixCls}-description`
        }, [descriptionNode])]), isPagination && createVNode(IxPagination, {
          "class": `${prefixCls}-pagination`,
          "total": titleChildren.length,
          "simple": true,
          "pageIndex": pageIndex.value,
          "showTotal": false,
          "showQuickJumper": false,
          "pageSize": 1,
          "onChange": handlePageChange
        }, null), closeable.value && createVNode("span", {
          "class": `${prefixCls}-close-icon`,
          "onClick": handleClose
        }, [slots.closeIcon ? slots.closeIcon() : createVNode(IxIcon, {
          "name": props.closeIcon
        }, null)])])]
      });
    };
  }
});
const IxAlert = Alert;
export { IxAlert, getThemeTokens as getAlertThemeTokens };
