import { defineComponent, computed, ref, watchEffect, createVNode, isVNode, onMounted, watch } from "vue";
import { isString, isNumber, isObject } from "lodash-es";
import { useSharedBreakpoints } from "@idux/cdk/breakpoint";
import { useResizeObserver } from "@idux/cdk/resize";
import { hasSlot, callEmit, convertCssPixel } from "@idux/cdk/utils";
import { useGlobalConfig } from "@idux/components/config";
import { IxIcon } from "@idux/components/icon";
import { useThemeToken } from "@idux/components/theme";
const avatarProps = {
  alt: String,
  gap: Number,
  icon: [String, Object],
  shape: String,
  size: [Number, String, Object],
  src: String,
  srcset: String,
  text: String,
  onError: [Function, Array]
};
function getDefaultThemeTokens(tokens) {
  const { heightSm, heightMd, heightLg, componentBorderRadius } = tokens;
  return {
    sizeSm: heightSm,
    sizeMd: heightMd,
    sizeLg: heightLg,
    borderRadiusSquare: componentBorderRadius
  };
}
const getThemeTokens = (tokens, presetTheme) => {
  return presetTheme === "default" ? getDefaultThemeTokens(tokens) : getDefaultThemeTokens(tokens);
};
var Avatar = /* @__PURE__ */ defineComponent({
  name: "IxAvatar",
  props: avatarProps,
  setup(props, {
    slots
  }) {
    const common = useGlobalConfig("common");
    const {
      globalHashId,
      hashId,
      registerToken
    } = useThemeToken("avatar");
    registerToken(getThemeTokens);
    const mergedPrefixCls = computed(() => `${common.prefixCls}-avatar`);
    const config = useGlobalConfig("avatar");
    const showText = ref(false);
    watchEffect(() => {
      showText.value = !props.src && (!!props.text || hasSlot(slots));
    });
    const showIcon = ref(false);
    watchEffect(() => {
      showIcon.value = !props.src && !showText.value;
    });
    const icon$$ = computed(() => {
      var _a;
      return (_a = props.icon) != null ? _a : config.icon;
    });
    const {
      avatarRef,
      textRef,
      size,
      avatarStyle,
      textStyle
    } = useSize(props, config);
    const classes = useClasses(props, config, size, showText, showIcon, mergedPrefixCls, globalHashId, hashId);
    const handleError = (evt) => {
      const result = callEmit(props.onError, evt);
      if (result !== false && !evt.defaultPrevented) {
        if (!!props.text || hasSlot(slots)) {
          showText.value = true;
        }
        showIcon.value = !showText.value;
      }
    };
    return () => {
      var _a, _b;
      let child;
      const prefixCls = mergedPrefixCls.value;
      if (showIcon.value) {
        child = renderIcon(slots.icon, icon$$.value);
      } else if (showText.value) {
        const textChild = (_b = (_a = slots.default) == null ? void 0 : _a.call(slots)) != null ? _b : props.text;
        child = createVNode("span", {
          "ref": textRef,
          "class": `${prefixCls}-text`,
          "style": textStyle.value
        }, [textChild]);
      } else {
        child = createVNode("img", {
          "src": props.src,
          "srcset": props.srcset,
          "alt": props.alt,
          "onError": handleError
        }, null);
      }
      return createVNode("span", {
        "ref": avatarRef,
        "class": classes.value,
        "style": avatarStyle.value
      }, [child]);
    };
  }
});
const useClasses = (props, config, size, showText, showIcon, mergedPrefixCls, globalHashId, hashId) => {
  return computed(() => {
    var _a;
    const shape = (_a = props.shape) != null ? _a : config.shape;
    const sizeValue = size.value;
    const prefixCls = mergedPrefixCls.value;
    return {
      [globalHashId.value]: !!globalHashId.value,
      [hashId.value]: !!hashId.value,
      [`${prefixCls}`]: true,
      [`${prefixCls}-image`]: !!props.src && !showText.value && !showIcon.value,
      [`${prefixCls}-${shape}`]: true,
      [`${prefixCls}-${sizeValue}`]: isString(sizeValue)
    };
  });
};
const useSizeStyle = (size) => {
  const breakpoints = useSharedBreakpoints();
  return computed(() => {
    let currSize;
    const sizeValue = size.value;
    if (isNumber(sizeValue)) {
      currSize = sizeValue;
    } else if (isObject(sizeValue)) {
      const currBreakpoint = Object.keys(breakpoints).find((key) => breakpoints[key]);
      currSize = sizeValue[currBreakpoint];
    }
    return getSizeStyle(currSize);
  });
};
const getSizeStyle = (size) => {
  if (!size) {
    return {};
  }
  const sizePixel = convertCssPixel(size);
  return {
    width: sizePixel,
    height: sizePixel,
    lineHeight: sizePixel,
    fontSize: convertCssPixel(size / 2)
  };
};
const renderIcon = (iconSlot, icon) => {
  if (iconSlot) {
    return iconSlot();
  }
  return isVNode(icon) ? icon : createVNode(IxIcon, {
    "name": icon
  }, null);
};
const useSize = (props, config) => {
  const avatarRef = ref();
  const textRef = ref();
  const size = computed(() => {
    var _a;
    return (_a = props.size) != null ? _a : config.size;
  });
  const gap = computed(() => {
    var _a;
    return (_a = props.gap) != null ? _a : config.gap;
  });
  const avatarStyle = useSizeStyle(size);
  const textScale = ref(1);
  const textStyle = computed(() => ({
    transform: `scale(${textScale.value}) translateX(-50%)`,
    lineHeight: avatarStyle.value.lineHeight
  }));
  const calcTextSize = () => {
    const avatarElement = avatarRef.value;
    const textElement = textRef.value;
    if (!avatarElement || !textElement) {
      return;
    }
    const avatarWidth = avatarElement.offsetWidth;
    const textWidth = textElement.offsetWidth;
    const gapWidth = gap.value * 2;
    if (avatarWidth !== 0 && textWidth !== 0 && gapWidth < avatarWidth) {
      textScale.value = avatarWidth - gapWidth < textWidth ? (avatarWidth - gapWidth) / textWidth : 1;
    }
  };
  useResizeObserver(textRef, calcTextSize);
  onMounted(() => {
    watch(gap, () => calcTextSize(), {
      immediate: true
    });
  });
  return {
    avatarRef,
    textRef,
    size,
    calcTextSize,
    avatarStyle,
    textStyle
  };
};
const IxAvatar = Avatar;
export { IxAvatar, getThemeTokens as getAvatarThemeTokens };
