import { defineComponent, computed, normalizeClass, createVNode, Transition } from "vue";
import { convertNumber, isNumeric } from "@idux/cdk/utils";
import { useGlobalConfig } from "@idux/components/config";
import { useThemeToken } from "@idux/components/theme";
const badgeProps = {
  color: { type: String },
  count: { type: [String, Number], default: 0 },
  dot: { type: [Boolean, String], default: void 0 },
  overflowCount: { type: [String, Number] },
  showZero: { type: Boolean, default: void 0 },
  status: { type: String, default: "error" },
  processing: { type: Boolean, default: false },
  title: { type: String }
};
const badgeSubProps = {
  count: { type: Number, required: true },
  dot: { type: Boolean, required: true },
  overflowCount: { type: Number, required: true },
  prefixCls: { type: String, required: true },
  showZero: { type: Boolean, required: true },
  status: { type: String, required: true },
  text: { type: String },
  title: { type: String }
};
var BadgeSub = /* @__PURE__ */ defineComponent({
  name: "IxBadgeSub",
  props: badgeSubProps,
  setup(props, {
    slots
  }) {
    const mergedCountArray = computed(() => props.count.toString().split("").map((char) => +char));
    const mergedOverflowCountArray = computed(() => props.overflowCount.toString().split(""));
    const classes = computed(() => {
      const {
        prefixCls,
        dot
      } = props;
      return normalizeClass({
        [`${prefixCls}-sub`]: true,
        [`${prefixCls}-dot`]: dot,
        [`${prefixCls}-count`]: !dot,
        [`${prefixCls}-count-multiple`]: !dot && mergedCountArray.value.length > 1
      });
    });
    return () => {
      const {
        prefixCls,
        dot,
        text
      } = props;
      if (dot || text) {
        return createVNode(Transition, {
          "name": `${prefixCls}-sub`,
          "appear": true
        }, {
          default: () => [createVNode("sup", {
            "class": classes.value,
            "title": props.title
          }, [text && createVNode("span", {
            "class": `${prefixCls}-sub-text`
          }, [text])])]
        });
      }
      const count = props.count;
      const {
        title = count,
        showZero,
        overflowCount
      } = props;
      let countNode;
      if (slots.default) {
        countNode = slots.default();
      } else if (count !== 0 || showZero) {
        countNode = renderNumbers(prefixCls, count, overflowCount, mergedCountArray, mergedOverflowCountArray);
      }
      return createVNode(Transition, {
        "name": `${prefixCls}-sub`,
        "appear": true
      }, {
        default: () => [countNode && createVNode("sup", {
          "class": classes.value,
          "title": title
        }, [createVNode("span", {
          "class": `${prefixCls}-count-track-wrapper`
        }, [countNode])])]
      });
    };
  }
});
const chars = [0, 1, 2, 3, 4, 5, 6, 7, 8, 9];
function renderNumbers(prefixCls, count, overflowCount, mergedCountArray, mergedOverflowCountArray) {
  if (count > overflowCount) {
    return `${overflowCount}+`;
  }
  const countArray = mergedCountArray.value;
  return mergedOverflowCountArray.value.map((_, index) => {
    const currentNumber = countArray[index];
    if (currentNumber === void 0) {
      return void 0;
    }
    return createVNode("span", {
      "key": index,
      "class": `${prefixCls}-count-track`,
      "style": `transform: translateY(${-currentNumber * 100}%);`
    }, [chars.map((char) => {
      const classes = normalizeClass({
        [`${prefixCls}-count-unit`]: true,
        [`${prefixCls}-count-current`]: char === currentNumber
      });
      return createVNode("span", {
        "key": char,
        "class": classes
      }, [char]);
    })]);
  });
}
function getDefaultThemeTokens() {
  return {
    dotSize: 6,
    countSize: 16
  };
}
const getThemeTokens = (tokens, presetTheme) => {
  return presetTheme === "default" ? getDefaultThemeTokens() : getDefaultThemeTokens();
};
var Badge = /* @__PURE__ */ defineComponent({
  name: "IxBadge",
  props: badgeProps,
  setup(props, {
    slots
  }) {
    const common = useGlobalConfig("common");
    const {
      globalHashId,
      hashId,
      registerToken
    } = useThemeToken("badge");
    registerToken(getThemeTokens);
    const config = useGlobalConfig("badge");
    const mergedPrefixCls = computed(() => `${common.prefixCls}-badge`);
    const mergedDot = computed(() => {
      var _a;
      return (_a = props.dot) != null ? _a : config.dot;
    });
    const mergedShowZero = computed(() => {
      var _a;
      return (_a = props.showZero) != null ? _a : config.showZero;
    });
    const mergedCount = computed(() => Math.max(0, convertNumber(props.count, 0)));
    const mergedOverflowCount = computed(() => {
      var _a;
      return Math.max(0, convertNumber((_a = props.overflowCount) != null ? _a : config.overflowCount, Number.MAX_VALUE));
    });
    const mergedText = computed(() => {
      const {
        count
      } = props;
      return isNumeric(count) ? void 0 : count;
    });
    const classes = computed(() => {
      const prefixCls = mergedPrefixCls.value;
      const {
        status,
        processing
      } = props;
      return normalizeClass({
        [globalHashId.value]: !!globalHashId.value,
        [hashId.value]: !!hashId.value,
        [prefixCls]: true,
        [`${prefixCls}-${status}`]: true,
        [`${prefixCls}-processing`]: processing
      });
    });
    const style = computed(() => {
      const {
        color
      } = props;
      return color ? `background-color: ${color}` : void 0;
    });
    return () => {
      const dot = mergedDot.value;
      const prefixCls = mergedPrefixCls.value;
      const contentNode = slots.default && slots.default();
      return createVNode("span", {
        "class": classes.value
      }, [dot === "inline" && createVNode("span", {
        "class": `${prefixCls}-dot`,
        "style": style.value
      }, null), contentNode && createVNode("span", {
        "class": `${prefixCls}-content`
      }, [contentNode]), dot !== "inline" && createVNode(BadgeSub, {
        "style": style.value,
        "count": mergedCount.value,
        "dot": dot,
        "overflowCount": mergedOverflowCount.value,
        "prefixCls": prefixCls,
        "showZero": mergedShowZero.value,
        "status": props.status,
        "text": mergedText.value,
        "title": props.title
      }, {
        default: slots.count
      })]);
    };
  }
});
const IxBadge = Badge;
export { IxBadge, getThemeTokens as getBadgeThemeTokens };
