import { defineComponent, computed, provide, normalizeClass, createVNode, isVNode, mergeProps, inject } from "vue";
import { isString } from "lodash-es";
import { useControlledProp, callEmit } from "@idux/cdk/utils";
import { \u0275Header as _Header } from "@idux/components/_private/header";
import { IxButton } from "@idux/components/button";
import { useGlobalConfig } from "@idux/components/config";
import { IxRow, IxCol } from "@idux/components/grid";
import { IxIcon } from "@idux/components/icon";
import { useThemeToken } from "@idux/components/theme";
const cardToken = Symbol("cardToken");
const cardProps = {
  borderless: {
    type: Boolean,
    default: void 0
  },
  cover: [String, Object],
  header: [String, Object],
  hoverable: {
    type: Boolean,
    default: void 0
  },
  shadow: {
    type: Boolean,
    default: void 0
  },
  loading: {
    type: Boolean,
    default: false
  },
  size: String,
  footer: Array,
  disabled: {
    type: Boolean,
    default: false
  },
  selected: {
    type: Boolean,
    default: false
  },
  selectable: {
    type: Boolean,
    default: false
  },
  "onUpdate:selected": [Function, Array],
  onSelectedChange: [Function, Array]
};
const cardGridProps = {
  hoverable: {
    type: Boolean,
    default: void 0
  }
};
function getDefaultThemeTokens(tokens, algorithms) {
  const { getGreyColors } = algorithms;
  const { fontSizeSm, fontSizeMd, fontSizeXl, fontSizeLg, marginSizeXs, paddingSizeSm, paddingSizeLg, paddingSizeXl } = tokens;
  const greyColors = getGreyColors();
  return {
    fontSizeSm,
    fontSizeMd: fontSizeSm,
    fontSizeLg: fontSizeMd,
    paddingSizeSm: paddingSizeLg,
    paddingSizeMd: paddingSizeLg,
    paddingSizeLg: paddingSizeXl,
    showShadowBorderWidth: 0,
    markWidth: fontSizeXl,
    markHeight: fontSizeLg,
    headerPadding: paddingSizeSm,
    loadingSpacing: marginSizeXs,
    loadingHeight: fontSizeMd,
    loadingBgSize: `600%`,
    loadingStartColor: greyColors.l40,
    loadingEndColor: greyColors.l30,
    gridWidth: `25%`
  };
}
function getDarkThemeTokens(tokens, algorithms) {
  const { lineWidth } = tokens;
  const defaultTokens = getDefaultThemeTokens(tokens, algorithms);
  return {
    ...defaultTokens,
    showShadowBorderWidth: lineWidth
  };
}
const getThemeTokens = (tokens, presetTheme, algorithms) => {
  return presetTheme === "default" ? getDefaultThemeTokens(tokens, algorithms) : getDarkThemeTokens(tokens, algorithms);
};
var Card = /* @__PURE__ */ defineComponent({
  name: "IxCard",
  props: cardProps,
  setup(props, {
    slots
  }) {
    const common = useGlobalConfig("common");
    const {
      globalHashId,
      hashId,
      registerToken
    } = useThemeToken("card");
    registerToken(getThemeTokens);
    const mergedPrefixCls = computed(() => `${common.prefixCls}-card`);
    const config = useGlobalConfig("card");
    const hoverable = computed(() => {
      var _a;
      return (_a = props.hoverable) != null ? _a : config.hoverable;
    });
    provide(cardToken, {
      hoverable
    });
    const [selected, setSelected] = useControlledProp(props, "selected", false);
    const isDisabled = computed(() => props.disabled);
    const isSelectable = computed(() => props.selectable);
    const handleClick = () => {
      if (isDisabled.value || !isSelectable.value) {
        return;
      }
      const newSelected = !selected.value;
      setSelected(newSelected);
      callEmit(props.onSelectedChange, newSelected);
    };
    return () => {
      var _a, _b;
      const prefixCls = mergedPrefixCls.value;
      const {
        borderless = config.borderless,
        loading,
        size = config.size,
        shadow = config.shadow
      } = props;
      const children = (_b = (_a = slots.default) == null ? void 0 : _a.call(slots)) != null ? _b : [];
      const hasGrid = children.some((node) => node.type && node.type.name === "IxCardGrid");
      const classes = normalizeClass({
        [globalHashId.value]: !!globalHashId.value,
        [hashId.value]: !!hashId.value,
        [prefixCls]: true,
        [`${prefixCls}-borderless`]: borderless,
        [`${prefixCls}-hoverable`]: !hasGrid && hoverable.value,
        [`${prefixCls}-loading`]: loading,
        [`${prefixCls}-has-shadow`]: shadow,
        [`${prefixCls}-has-grid`]: hasGrid,
        [`${prefixCls}-${size}`]: true,
        [`${prefixCls}-selectable`]: isSelectable.value,
        [`${prefixCls}-selected`]: isSelectable.value && selected.value,
        [`${prefixCls}-disabled`]: isDisabled.value
      });
      return createVNode("div", {
        "class": classes,
        "onClick": handleClick
      }, [renderCover(props, slots, prefixCls), createVNode(_Header, {
        "size": size,
        "header": props.header
      }, slots), renderBody(props, children, hasGrid, prefixCls), renderFooter(props, slots, prefixCls), renderCornerMark(props, prefixCls)]);
    };
  }
});
const renderCornerMark = (props, prefixCls) => {
  const isSelectable = computed(() => {
    var _a;
    return (_a = props.selectable) != null ? _a : false;
  });
  if (!isSelectable.value) {
    return void 0;
  }
  return createVNode("div", {
    "class": `${prefixCls}-mark-wrap`
  }, [createVNode("div", {
    "class": `${prefixCls}-mark`
  }, null), createVNode(IxIcon, {
    "name": "check-filled",
    "class": `${prefixCls}-mark-icon`,
    "color": "white"
  }, null)]);
};
const renderCover = (props, slots, prefixCls) => {
  let coverNode;
  if (slots.cover) {
    coverNode = slots.cover();
  } else if (props.cover) {
    const {
      cover
    } = props;
    const imgProps = isString(cover) ? {
      src: cover
    } : cover;
    coverNode = createVNode("img", imgProps, null);
  }
  return coverNode ? createVNode("div", {
    "class": `${prefixCls}-cover`
  }, [coverNode]) : void 0;
};
const renderBody = (props, children, hasGrid, prefixCls) => {
  let bodyNode;
  if (props.loading) {
    bodyNode = renderLoading(prefixCls);
  } else if (children.length) {
    bodyNode = hasGrid ? createVNode(IxRow, null, {
      default: () => [children]
    }) : children;
  }
  return bodyNode ? createVNode("div", {
    "class": `${prefixCls}-body`
  }, [bodyNode]) : void 0;
};
const loadingSpans = [[22], [8, 15], [6, 18], [13, 9], [4, 3, 16], [8, 6, 8]];
const renderLoading = (prefixCls) => {
  const loadingChild = loadingSpans.map((spans) => {
    const cols = spans.map((span) => createVNode(IxCol, {
      "span": span,
      "class": `${prefixCls}-loading-col`
    }, {
      default: () => [createVNode("div", {
        "class": `${prefixCls}-loading-block`
      }, null)]
    }));
    return createVNode(IxRow, null, {
      default: () => [cols]
    });
  });
  return createVNode("div", {
    "class": `${prefixCls}-loading`
  }, [loadingChild]);
};
const renderFooter = (props, slots, prefixCls) => {
  if (!slots.footer && !props.footer) {
    return void 0;
  }
  let footerNode;
  if (slots.footer) {
    footerNode = slots.footer();
  } else {
    footerNode = props.footer.map((item) => {
      let itemChild;
      if (isVNode(item)) {
        itemChild = item;
      } else {
        const {
          text,
          ...rest
        } = item;
        itemChild = createVNode(IxButton, mergeProps({
          "class": `${prefixCls}-footer-button`,
          "waveless": true
        }, rest), {
          default: () => [text]
        });
      }
      return itemChild ? createVNode("li", null, [itemChild]) : null;
    });
  }
  return createVNode("ul", {
    "class": `${prefixCls}-footer`
  }, [footerNode]);
};
var CardGrid = /* @__PURE__ */ defineComponent({
  name: "IxCardGrid",
  props: cardGridProps,
  setup(props, {
    slots
  }) {
    const common = useGlobalConfig("common");
    const {
      globalHashId,
      hashId,
      registerToken
    } = useThemeToken("card");
    registerToken(getThemeTokens);
    const mergedPrefixCls = computed(() => `${common.prefixCls}-card-grid`);
    const {
      hoverable
    } = inject(cardToken);
    const classes = computed(() => {
      var _a;
      const prefixCls = mergedPrefixCls.value;
      return normalizeClass({
        [globalHashId.value]: !!globalHashId.value,
        [hashId.value]: !!hashId.value,
        [prefixCls]: true,
        [`${prefixCls}-hoverable`]: (_a = props.hoverable) != null ? _a : hoverable.value
      });
    });
    return () => createVNode(IxCol, {
      "class": classes.value
    }, {
      default: () => {
        var _a;
        return [(_a = slots.default) == null ? void 0 : _a.call(slots)];
      }
    });
  }
});
const IxCard = Card;
const IxCardGrid = CardGrid;
export { IxCard, IxCardGrid, getThemeTokens as getCardThemeTokens };
