import { computed, onMounted, watch, onBeforeUnmount, ref, defineComponent, inject, normalizeClass, createVNode, mergeProps, onBeforeUpdate, provide, Fragment } from "vue";
import { callEmit, useState, convertElement, getCssDimensions, flattenNode } from "@idux/cdk/utils";
import { useGlobalConfig } from "@idux/components/config";
import { IxIcon } from "@idux/components/icon";
import { useThemeToken } from "@idux/components/theme";
import { useResizeObserver } from "@idux/cdk/resize";
function useAutoplay(props, config, activeIndex, goTo) {
  const mergedAutoplayTime = computed(() => {
    var _a;
    return (_a = props.autoplayTime) != null ? _a : config.autoplayTime;
  });
  let timer = null;
  const cleanAutoplay = () => {
    if (timer !== null) {
      clearTimeout(timer);
      timer = null;
    }
  };
  const startAutoplay = () => {
    cleanAutoplay();
    const autoplayTime = mergedAutoplayTime.value;
    if (autoplayTime > 0) {
      timer = setTimeout(() => {
        goTo(activeIndex.value + 1);
      }, mergedAutoplayTime.value);
    }
  };
  onMounted(() => {
    watch(mergedAutoplayTime, startAutoplay, { immediate: true });
  });
  onBeforeUnmount(cleanAutoplay);
  return { startAutoplay, cleanAutoplay };
}
function useStrategy(props, carouselRef, sliderTrackRef, sliderRefs, mergedVertical, mergedLength) {
  const activeIndex = ref(0);
  watch(activeIndex, (newVal, oldVal) => {
    callEmit(props.onChange, newVal, oldVal);
  });
  const runningIndex = ref(-1);
  const nextIndex = ref(-1);
  const [unitWidth, setUnitWidth] = useState(0);
  const [unitHeight, setUnitHeight] = useState(0);
  const layout = () => {
    const carouselElement = carouselRef.value;
    if (!carouselElement) {
      return;
    }
    const { width, height } = getCssDimensions(carouselElement);
    setUnitWidth(width);
    setUnitHeight(height);
  };
  useResizeObserver(carouselRef, layout);
  onMounted(() => {
    watch(
      mergedLength,
      () => {
        layout();
        runningIndex.value = -1;
        goTo(0);
      },
      { flush: "post" }
    );
  });
  const goTo = (index) => {
    const length = mergedLength.value;
    const sliderTrackElement = sliderTrackRef.value;
    const firstSliderElement = convertElement(sliderRefs.value[0]);
    const lastSliderElement = convertElement(sliderRefs.value[length - 1]);
    if (length < 1 || runningIndex.value !== -1 || index === activeIndex.value || !sliderTrackElement || !firstSliderElement || !lastSliderElement) {
      return;
    }
    const { from, to } = getBoundary(activeIndex.value, index, length);
    if (from === to) {
      return;
    }
    nextIndex.value = index;
    runningIndex.value = to;
    sliderTrackElement.style.transition = `transform 0.5s ease`;
    const needToAdjust = index !== to;
    if (needToAdjust) {
      const stylePropertyName = mergedVertical.value ? "top" : "left";
      const unit = mergedVertical.value ? unitHeight.value : unitWidth.value;
      if (to < from) {
        firstSliderElement.style[stylePropertyName] = `${length * unit}px`;
        lastSliderElement.style[stylePropertyName] = "";
      } else {
        firstSliderElement.style[stylePropertyName] = "";
        lastSliderElement.style[stylePropertyName] = `${-length * unit}px`;
      }
    }
  };
  const next = () => {
    goTo(activeIndex.value + 1);
  };
  const prev = () => {
    goTo(activeIndex.value - 1);
  };
  return {
    activeIndex,
    runningIndex,
    nextIndex,
    unitHeight,
    unitWidth,
    goTo,
    next,
    prev
  };
}
function getBoundary(f, t, length) {
  return { from: (f + length) % length, to: (t + length) % length };
}
const carouselToken = Symbol("carousel");
var Dot = /* @__PURE__ */ defineComponent({
  props: {
    isActive: {
      type: Boolean,
      required: true
    },
    index: {
      type: Number,
      required: true
    }
  },
  setup(props) {
    const {
      slots,
      mergedPrefixCls,
      mergedTrigger,
      cleanAutoplay,
      goTo
    } = inject(carouselToken);
    const classes = computed(() => {
      const prefixCls = `${mergedPrefixCls.value}-dot`;
      return normalizeClass({
        [prefixCls]: true,
        [`${prefixCls}-active`]: props.isActive
      });
    });
    const handleTrigger = () => {
      cleanAutoplay();
      goTo(props.index);
    };
    const events = computed(() => {
      if (mergedTrigger.value === "click") {
        return {
          onClick: handleTrigger
        };
      }
      return {
        onMouseenter: handleTrigger
      };
    });
    return () => {
      return createVNode("li", mergeProps({
        "class": classes.value
      }, events.value), [slots.dot ? slots.dot(props) : createVNode("button", {
        "type": "button"
      }, null)]);
    };
  }
});
var Slider = /* @__PURE__ */ defineComponent({
  props: {
    isActive: {
      type: Boolean,
      required: true
    },
    index: {
      type: Number,
      required: true
    }
  },
  setup(props, {
    slots
  }) {
    const {
      mergedPrefixCls,
      unitWidth,
      unitHeight
    } = inject(carouselToken);
    const classes = computed(() => {
      const prefixCls = `${mergedPrefixCls.value}-slider`;
      return normalizeClass({
        [prefixCls]: true,
        [`${prefixCls}-active`]: props.isActive
      });
    });
    const style = computed(() => {
      const width = unitWidth.value;
      const height = unitHeight.value;
      if (width <= 0 || height <= 0) {
        return void 0;
      }
      return `position: relative;width: ${width}px;height: ${height}px;`;
    });
    return () => {
      return createVNode("div", {
        "class": classes.value,
        "style": style.value
      }, [slots.default()]);
    };
  }
});
const carouselProps = {
  autoplayTime: { type: Number, default: void 0 },
  dotPlacement: { type: String, default: void 0 },
  showArrow: { type: Boolean, default: void 0 },
  trigger: { type: String, default: void 0 },
  onChange: [Function, Array]
};
function getDefaultThemeTokens(tokens) {
  const { colorWhite } = tokens;
  return {
    arrowSize: 32,
    arrowColor: colorWhite,
    horizontalDotWidth: 32,
    horizontalDotHeight: 2,
    verticalDotWidth: 2,
    verticalDotHeight: 16,
    dotBgColor: colorWhite,
    dotBorderRadius: 2,
    iconOpacity: 0.3,
    iconOpacityHover: 0.6,
    iconOpacityActive: 0.8
  };
}
const getThemeTokens = (tokens, presetTheme) => {
  return presetTheme === "default" ? getDefaultThemeTokens(tokens) : getDefaultThemeTokens(tokens);
};
var Carousel = /* @__PURE__ */ defineComponent({
  name: "IxCarousel",
  props: carouselProps,
  setup(props, {
    slots,
    expose
  }) {
    const common = useGlobalConfig("common");
    const {
      globalHashId,
      hashId,
      registerToken
    } = useThemeToken("carousel");
    registerToken(getThemeTokens);
    const config = useGlobalConfig("carousel");
    const mergedPrefixCls = computed(() => `${common.prefixCls}-carousel`);
    const mergedDotPlacement = computed(() => {
      var _a;
      return (_a = props.dotPlacement) != null ? _a : config.dotPlacement;
    });
    const mergedShowArrow = computed(() => {
      var _a;
      return (_a = props.showArrow) != null ? _a : config.showArrow;
    });
    const mergedTrigger = computed(() => {
      var _a;
      return (_a = props.trigger) != null ? _a : config.trigger;
    });
    const mergedVertical = computed(() => mergedDotPlacement.value === "start" || mergedDotPlacement.value === "end");
    const mergedChildren = computed(() => {
      var _a;
      return flattenNode((_a = slots.default) == null ? void 0 : _a.call(slots));
    });
    const mergedLength = computed(() => mergedChildren.value.length);
    const carouselRef = ref();
    const slidersRef = ref();
    const sliderTrackRef = ref();
    const sliderRefs = ref([]);
    const setRef = (instance, index) => {
      if (instance) {
        sliderRefs.value[index] = instance;
      }
    };
    onBeforeUpdate(() => {
      sliderRefs.value = [];
    });
    const {
      activeIndex,
      runningIndex,
      nextIndex,
      unitHeight,
      unitWidth,
      goTo,
      next,
      prev
    } = useStrategy(props, carouselRef, sliderTrackRef, sliderRefs, mergedVertical, mergedLength);
    const {
      startAutoplay,
      cleanAutoplay
    } = useAutoplay(props, config, activeIndex, goTo);
    expose({
      goTo,
      next,
      prev
    });
    provide(carouselToken, {
      slots,
      mergedPrefixCls,
      mergedTrigger,
      unitHeight,
      unitWidth,
      goTo,
      cleanAutoplay
    });
    const classes = computed(() => {
      const prefixCls = mergedPrefixCls.value;
      return normalizeClass({
        [globalHashId.value]: !!globalHashId.value,
        [hashId.value]: !!hashId.value,
        [prefixCls]: true,
        [`${prefixCls}-horizontal`]: !mergedVertical.value,
        [`${prefixCls}-vertical`]: mergedVertical.value
      });
    });
    const slidersStyle = computed(() => {
      const height = unitHeight.value;
      return height > 0 ? `height: ${height}px;` : void 0;
    });
    const sliderTrackStyle = computed(() => {
      const width = unitWidth.value;
      const height = unitHeight.value;
      const length = mergedLength.value;
      if (width <= 0 || height <= 0 || length <= 0) {
        return void 0;
      }
      const index = runningIndex.value > -1 ? nextIndex.value : activeIndex.value;
      return mergedVertical.value ? {
        width: `${width}px`,
        height: `${height * length}px`,
        transform: `translate3d(0, ${-index * unitHeight.value}px, 0)`
      } : {
        width: `${width * length}px`,
        height: `${height}px`,
        transform: `translate3d(${-index * unitWidth.value}px, 0, 0)`
      };
    });
    const dotsClasses = computed(() => {
      const prefixCls = mergedPrefixCls.value;
      return normalizeClass({
        [`${prefixCls}-dots`]: true,
        [`${prefixCls}-dots-${mergedDotPlacement.value}`]: true
      });
    });
    const handleKeydown = (evt) => {
      switch (evt.code) {
        case "ArrowRight":
        case "ArrowDown":
          next();
          break;
        case "ArrowLeft":
        case "ArrowUp":
          prev();
          break;
      }
    };
    const onTransitionend = () => {
      const currRunningIndex = runningIndex.value;
      if (currRunningIndex === -1) {
        return;
      }
      const sliderTrackElement = sliderTrackRef.value;
      sliderTrackElement.style.transition = "";
      sliderRefs.value.forEach((slider) => {
        const element = convertElement(slider);
        if (element) {
          element.style.top = "";
          element.style.left = "";
        }
      });
      activeIndex.value = currRunningIndex;
      runningIndex.value = -1;
      startAutoplay();
    };
    return () => {
      const prefixCls = mergedPrefixCls.value;
      const children = mergedChildren.value;
      const currActiveIndex = activeIndex.value;
      const currRunningIndex = runningIndex.value;
      return createVNode("div", {
        "ref": carouselRef,
        "class": classes.value
      }, [createVNode("div", {
        "ref": slidersRef,
        "class": `${prefixCls}-sliders`,
        "style": slidersStyle.value,
        "tabindex": "-1",
        "onKeydown": handleKeydown
      }, [createVNode("div", {
        "ref": sliderTrackRef,
        "class": `${prefixCls}-slider-track`,
        "style": sliderTrackStyle.value,
        "onTransitionend": onTransitionend
      }, [children.map((node, index) => createVNode(Slider, {
        "ref": (instance) => setRef(instance, index),
        "key": node.key,
        "isActive": index === currActiveIndex,
        "index": index
      }, {
        default: () => [node]
      }))])]), mergedShowArrow.value && children.length > 1 && createVNode(Fragment, null, [createVNode("div", {
        "key": "__arrow-prev",
        "class": `${prefixCls}-arrow ${prefixCls}-arrow-prev`,
        "onClick": prev
      }, [slots.arrow ? slots.arrow({
        type: "prev"
      }) : createVNode(IxIcon, {
        "name": "left-filled"
      }, null)]), createVNode("div", {
        "key": "__arrow-next",
        "class": `${prefixCls}-arrow ${prefixCls}-arrow-next`,
        "onClick": next
      }, [slots.arrow ? slots.arrow({
        type: "next"
      }) : createVNode(IxIcon, {
        "name": "right-filled"
      }, null)])]), mergedDotPlacement.value !== "none" && createVNode("ul", {
        "class": dotsClasses.value
      }, [children.map((node, index) => createVNode(Dot, {
        "key": node.key,
        "isActive": currRunningIndex !== -1 ? index === currRunningIndex : index === currActiveIndex,
        "index": index
      }, null))])]);
    };
  }
});
const IxCarousel = Carousel;
export { IxCarousel, getThemeTokens as getCarouselThemeTokens };
