import { defineComponent, computed, provide, normalizeClass, createVNode, inject, withDirectives, vShow, mergeProps } from "vue";
import { useControlledProp, useKey, getFirstValidNode } from "@idux/cdk/utils";
import { useGlobalConfig } from "@idux/components/config";
import { useThemeToken } from "@idux/components/theme";
import { isString } from "lodash-es";
import { \u0275CollapseTransition as _CollapseTransition } from "@idux/components/_private/collapse-transition";
import { IxHeader } from "@idux/components/header";
import { IxIcon } from "@idux/components/icon";
const collapseToken = Symbol("collapseToken");
const collapseProps = {
  expandedKeys: Array,
  accordion: {
    type: Boolean,
    default: void 0
  },
  borderless: {
    type: Boolean,
    default: void 0
  },
  expandIcon: String,
  ghost: {
    type: Boolean,
    default: void 0
  },
  size: {
    type: String,
    default: void 0
  },
  "onUpdate:expandedKeys": [Function, Array]
};
const collapsePanelProps = {
  disabled: {
    type: Boolean,
    default: false
  },
  header: [String, Object]
};
function getDefaultThemeTokens(tokens) {
  const {
    colorContainerBg,
    colorInfoContainerBg,
    fontSizeHeaderSm,
    fontSizeXl,
    fontSize2xl,
    paddingSizeMd,
    paddingSizeLg
  } = tokens;
  return {
    fontSizeSm: fontSizeHeaderSm,
    fontSizeMd: fontSizeHeaderSm,
    expandIconSizeSm: fontSizeXl,
    expandIconSizeMd: fontSize2xl,
    paddingHorizontalSm: paddingSizeMd,
    paddingHorizontalMd: paddingSizeLg,
    panelHeaderBgColor: colorContainerBg,
    panelContentBgColor: colorInfoContainerBg
  };
}
function getDarkThemeTokens(tokens) {
  const { colorContainerBg, colorInfoContainerBg } = tokens;
  const defaultTokens = getDefaultThemeTokens(tokens);
  return {
    ...defaultTokens,
    panelHeaderBgColor: colorInfoContainerBg,
    panelContentBgColor: colorContainerBg
  };
}
const getThemeTokens = (tokens, presetTheme) => {
  return presetTheme === "default" ? getDefaultThemeTokens(tokens) : getDarkThemeTokens(tokens);
};
var Collapse = /* @__PURE__ */ defineComponent({
  name: "IxCollapse",
  props: collapseProps,
  setup(props, {
    slots
  }) {
    const common = useGlobalConfig("common");
    const {
      globalHashId,
      hashId,
      registerToken
    } = useThemeToken("collapse");
    registerToken(getThemeTokens);
    const mergedPrefixCls = computed(() => `${common.prefixCls}-collapse`);
    const config = useGlobalConfig("collapse");
    const accordion = computed(() => {
      var _a;
      return (_a = props.accordion) != null ? _a : config.accordion;
    });
    const borderless = computed(() => {
      var _a;
      return (_a = props.borderless) != null ? _a : config.borderless;
    });
    const expandIcon = computed(() => {
      var _a;
      return (_a = props.expandIcon) != null ? _a : config.expandIcon;
    });
    const ghost = computed(() => {
      var _a;
      return (_a = props.ghost) != null ? _a : config.ghost;
    });
    const mergedSize = computed(() => {
      var _a;
      return (_a = props.size) != null ? _a : config.size;
    });
    const [expandedKeys, setExpandedKeys] = useControlledProp(props, "expandedKeys", () => []);
    const handleExpand = (key) => {
      let tempKeys = [...expandedKeys.value];
      const index = tempKeys.indexOf(key);
      if (accordion.value) {
        tempKeys = index > -1 ? [] : [key];
      } else {
        index > -1 ? tempKeys.splice(index, 1) : tempKeys.push(key);
      }
      setExpandedKeys(tempKeys);
    };
    provide(collapseToken, {
      props,
      slots,
      mergedSize,
      expandedKeys,
      expandIcon,
      handleExpand
    });
    const classes = computed(() => {
      const prefixCls = mergedPrefixCls.value;
      return normalizeClass({
        [globalHashId.value]: !!globalHashId.value,
        [hashId.value]: !!hashId.value,
        [prefixCls]: true,
        [`${prefixCls}-${mergedSize.value}`]: true,
        [`${prefixCls}-borderless`]: borderless.value,
        [`${prefixCls}-ghost`]: ghost.value
      });
    });
    return () => {
      var _a;
      return createVNode("div", {
        "class": classes.value
      }, [(_a = slots.default) == null ? void 0 : _a.call(slots)]);
    };
  }
});
var CollapsePanel = /* @__PURE__ */ defineComponent({
  name: "IxCollapsePanel",
  props: collapsePanelProps,
  setup(props, {
    slots
  }) {
    const common = useGlobalConfig("common");
    const {
      globalHashId,
      hashId,
      registerToken
    } = useThemeToken("collapse");
    registerToken(getThemeTokens);
    const mergedPrefixCls = computed(() => `${common.prefixCls}-collapse-panel`);
    const {
      slots: collapseSlots,
      mergedSize,
      expandedKeys,
      expandIcon,
      handleExpand
    } = inject(collapseToken);
    const key = useKey();
    const isExpanded = computed(() => expandedKeys.value.includes(key));
    const classes = computed(() => {
      const prefixCls = mergedPrefixCls.value;
      return normalizeClass({
        [globalHashId.value]: !!globalHashId.value,
        [hashId.value]: !!hashId.value,
        [`${prefixCls}`]: true,
        [`${prefixCls}-disabled`]: props.disabled,
        [`${prefixCls}-expanded`]: isExpanded.value
      });
    });
    const handleClick = () => {
      if (props.disabled) {
        return;
      }
      handleExpand(key);
    };
    return () => {
      const expanded = isExpanded.value;
      const prefixCls = mergedPrefixCls.value;
      const headerNode = renderHeader(props, slots, prefixCls, collapseSlots, key, mergedSize, expanded, expandIcon, handleClick);
      return createVNode("div", {
        "class": classes.value
      }, [headerNode, createVNode(_CollapseTransition, {
        "appear": true
      }, {
        default: () => {
          var _a;
          return [withDirectives(createVNode("div", {
            "class": `${prefixCls}-content`
          }, [createVNode("div", {
            "class": `${prefixCls}-content-box`
          }, [(_a = slots.default) == null ? void 0 : _a.call(slots)])]), [[vShow, expanded]])];
        }
      })]);
    };
  }
});
function renderHeader(props, slots, prefixCls, collapseSlots, key, mergedSize, expanded, expandIcon, changeExpand) {
  if (slots.header) {
    return slots.header({
      expanded,
      onClick: changeExpand,
      changeExpand
    });
  }
  let iconNode;
  if (collapseSlots.expandIcon) {
    const tempNode = collapseSlots.expandIcon({
      key,
      expanded
    });
    if (getFirstValidNode(tempNode)) {
      iconNode = tempNode;
    }
  } else {
    const iconName = expandIcon.value;
    iconNode = iconName ? createVNode(IxIcon, {
      "class": `${prefixCls}-expand-icon`,
      "name": iconName,
      "rotate": expanded ? 90 : 0
    }, null) : void 0;
  }
  const headerSlots = iconNode ? {
    prefix: () => iconNode
  } : void 0;
  const {
    header,
    disabled
  } = props;
  const headerProps = isString(header) ? {
    title: header
  } : header;
  return createVNode(IxHeader, mergeProps({
    "disabled": disabled,
    "size": mergedSize.value,
    "onClick": changeExpand
  }, headerProps), headerSlots);
}
const IxCollapse = Collapse;
const IxCollapsePanel = CollapsePanel;
export { IxCollapse, IxCollapsePanel, getThemeTokens as getCollapseThemeTokens };
