import { defineComponent, computed, createVNode } from "vue";
import { isString } from "lodash-es";
import { IxAvatar } from "@idux/components/avatar";
import { useGlobalConfig } from "@idux/components/config";
import { useThemeToken } from "@idux/components/theme";
import { convertStringVNode } from "@idux/components/utils";
const commentProps = {
  author: {
    type: String,
    default: void 0
  },
  avatar: {
    type: [String, Object],
    default: void 0
  },
  content: {
    type: String,
    default: void 0
  },
  datetime: {
    type: String,
    default: void 0
  }
};
function getDefaultThemeTokens(tokens) {
  const { colorTextPlaceholder, colorTextInfo, colorText } = tokens;
  return {
    nestIndent: 44,
    authorNameColor: colorTextPlaceholder,
    authorTimeColor: colorTextInfo,
    actionColor: colorTextInfo,
    actionColorHover: colorText
  };
}
const getThemeTokens = (tokens, presetTheme) => {
  return presetTheme === "default" ? getDefaultThemeTokens(tokens) : getDefaultThemeTokens(tokens);
};
var Comment = /* @__PURE__ */ defineComponent({
  name: "IxComment",
  props: commentProps,
  setup(props, {
    slots
  }) {
    const common = useGlobalConfig("common");
    const {
      globalHashId,
      hashId,
      registerToken
    } = useThemeToken("comment");
    registerToken(getThemeTokens);
    const mergedPrefixCls = computed(() => `${common.prefixCls}-comment`);
    return () => {
      const prefixCls = mergedPrefixCls.value;
      return createVNode("div", {
        "class": [prefixCls, globalHashId.value, hashId.value]
      }, [createVNode("div", {
        "class": `${prefixCls}-inner`
      }, [renderAvatar(props, slots, prefixCls), renderContent(props, slots, prefixCls)]), slots.default && createVNode("div", {
        "class": `${prefixCls}-nested`
      }, [slots.default()])]);
    };
  }
});
function renderAvatar(props, slots, prefixCls) {
  let avatarNode;
  if (slots.avatar) {
    avatarNode = slots.avatar();
  } else if (props.avatar) {
    const {
      avatar
    } = props;
    const avatarProps = isString(avatar) ? {
      src: avatar
    } : avatar;
    avatarNode = createVNode(IxAvatar, avatarProps, null);
  }
  return avatarNode && createVNode("div", {
    "class": `${prefixCls}-avatar`
  }, [avatarNode]);
}
function renderContent(props, slots, prefixCls) {
  const contentPrefixCls = `${prefixCls}-content`;
  const contentNode = convertStringVNode(slots, props, "content");
  const authorNode = convertStringVNode(slots, props, "author");
  const datetimeNode = convertStringVNode(slots, props, "datetime");
  return createVNode("div", {
    "class": contentPrefixCls
  }, [(authorNode || datetimeNode) && createVNode("div", {
    "class": `${contentPrefixCls}-author`
  }, [authorNode && createVNode("span", {
    "class": `${contentPrefixCls}-author-name`
  }, [authorNode]), datetimeNode && createVNode("span", {
    "class": `${contentPrefixCls}-author-time`
  }, [datetimeNode])]), contentNode && createVNode("div", {
    "class": `${contentPrefixCls}-detail`
  }, [contentNode]), slots.actions && createVNode("ul", {
    "class": `${contentPrefixCls}-actions`
  }, [slots.actions().map((action, index) => {
    return createVNode("li", {
      "key": `action-${index}`
    }, [action]);
  })])]);
}
const IxComment = Comment;
export { IxComment, getThemeTokens as getCommentThemeTokens };
