import { computed, watch, onMounted, toRaw, defineComponent, inject, createVNode, ref, mergeProps, withDirectives, vShow, toRef, provide, normalizeClass, h } from "vue";
import { useGlobalConfig, useDateConfig } from "@idux/components/config";
import { IxControlTrigger } from "@idux/components/control-trigger";
import { useFormItemRegister, useFormSize, useFormStatus, useFormElement } from "@idux/components/form";
import { useThemeToken } from "@idux/components/theme";
import { useState, useControlledProp, convertArray, callEmit } from "@idux/cdk/utils";
import { convertToDate, isSameDateTime, applyDateTime, sortRangeValue, adjustRangeValue, compareDateTime } from "@idux/components/utils";
import { isArray, isFunction, isString } from "lodash-es";
import { useAccessorAndControl } from "@idux/cdk/forms";
import { \u0275Footer as _Footer } from "@idux/components/_private/footer";
import { \u0275Input as _Input } from "@idux/components/_private/input";
import { \u0275DatePanel as _DatePanel } from "@idux/components/_private/date-panel";
import { \u0275TimePanel as _TimePanel } from "@idux/components/_private/time-panel";
import { getTimePickerThemeTokens } from "@idux/components/time-picker";
function getDefaultThemeTokens(tokens, algorithms) {
  const { getColorPalette } = algorithms;
  const {
    colorPrimary,
    colorPrimaryTextHover,
    colorText,
    colorTextInverse,
    colorTextDisabled,
    colorContainerBgHover,
    colorInfoContainerBg,
    marginSizeLg,
    paddingSizeSm,
    paddingSizeMd,
    paddingSizeLg,
    fontSizeSm,
    fontWeightMd,
    heightMd
  } = tokens;
  const primaryColorPalette = getColorPalette(colorPrimary);
  return {
    separatorMarginHorizontal: marginSizeLg,
    overlayWidth: 252,
    overlayPadding: paddingSizeLg,
    overlayFooterPadding: `${paddingSizeSm}px ${paddingSizeLg}px`,
    overlayDateInputWidth: 120,
    overlayTimeInputWidth: 96,
    boardWidth: 220,
    boardMaxHeight: 260,
    rangeBoardBorder: "none",
    rangeOverlayPadding: `${paddingSizeLg}px ${paddingSizeLg}px 0 ${paddingSizeLg}px`,
    rangeOverlayFooterPadding: `${paddingSizeSm}px 0`,
    rangeOverlaySeparatorWidth: 32,
    rangeOverlaySeparatorFontSize: fontSizeSm,
    rangeShortcutsItemHeight: heightMd,
    rangeShortcutsItemFontSize: fontSizeSm,
    rangeShortcutsItemPadding: `${paddingSizeSm}px ${paddingSizeMd}px`,
    panelFontSize: fontSizeSm,
    panelColor: colorText,
    panelCellWidth: 28,
    panelCellHeight: 28,
    panelCellWidthLg: 52,
    panelCellHeightLg: 24,
    panelCellColorActive: colorTextInverse,
    panelCellColorHover: colorPrimaryTextHover,
    panelCellColorDisabled: colorTextDisabled,
    panelCellBgColorHover: colorContainerBgHover,
    panelCellBgColorActive: colorPrimary,
    panelCellBgColorInRange: primaryColorPalette.l50,
    panelCellBgColorDisabled: colorInfoContainerBg,
    panelCellTriggerWidth: 20,
    panelCellTriggerHeight: 20,
    panelCellTriggerWidthLg: 52,
    panelCellTriggerHeightLg: 24,
    panelCellCurrentColor: colorPrimary,
    panelCellCurrentBorderColor: primaryColorPalette.l40,
    panelCellCurrentBgColor: colorPrimary,
    panelHeaerHeight: heightMd,
    panelHeaderFontSize: fontSizeSm,
    panelHeaderFontWeight: fontWeightMd,
    panelHeaderSpacing: marginSizeLg,
    panelBodyHeaderBgColor: colorInfoContainerBg
  };
}
const getThemeTokens = (tokens, presetTheme, algorithms) => {
  return presetTheme === "default" ? getDefaultThemeTokens(tokens, algorithms) : getDefaultThemeTokens(tokens, algorithms);
};
const defaultFormat = {
  date: "yyyy-MM-dd",
  week: "RRRR-II",
  month: "yyyy-MM",
  quarter: "yyyy-'Q'Q",
  year: "yyyy",
  datetime: "yyyy-MM-dd HH:mm:ss"
};
function useFormat(props, config) {
  const formatRef = computed(() => {
    var _a, _b, _c;
    const type = props.type;
    return (_c = (_b = props.format) != null ? _b : (_a = config.format) == null ? void 0 : _a[type]) != null ? _c : defaultFormat[type];
  });
  const { hourEnabled, hourUse12Hours, minuteEnabled, secondEnabled, use12Hours } = useTimePanelEnabledStatus(
    props,
    formatRef
  );
  const dateFormatRef = computed(() => {
    var _a;
    if (props.type !== "datetime") {
      return formatRef.value;
    }
    return (_a = props.dateFormat) != null ? _a : defaultFormat.date;
  });
  const timeFormatRef = computed(() => {
    if (props.timeFormat) {
      return props.timeFormat;
    }
    const hourFormat = hourEnabled.value && (hourUse12Hours.value ? "hh" : "HH");
    const timeFormatBase = [hourFormat, minuteEnabled.value && "mm", secondEnabled.value && "ss"].filter(Boolean).join(":");
    return use12Hours.value ? `${timeFormatBase} a` : timeFormatBase;
  });
  return {
    formatRef,
    dateFormatRef,
    timeFormatRef,
    hourEnabled,
    minuteEnabled,
    secondEnabled,
    use12Hours
  };
}
function useTimePanelEnabledStatus(props, formatRef) {
  const _formatRef = computed(() => {
    var _a;
    return (_a = props.timeFormat) != null ? _a : formatRef.value;
  });
  return {
    hourEnabled: computed(() => /[hH]/.test(_formatRef.value)),
    hourUse12Hours: computed(() => /h/.test(_formatRef.value)),
    minuteEnabled: computed(() => /m/.test(_formatRef.value)),
    secondEnabled: computed(() => /s/.test(_formatRef.value)),
    use12Hours: computed(() => /[aA]/.test(_formatRef.value))
  };
}
function useDateInputEnabledStatus(formatRef) {
  return computed(() => {
    const format = formatRef.value;
    return {
      yearEnabled: /[yYR]/.test(format),
      monthEnabled: /[MQID]/.test(format),
      dateEnabled: /[dDiEec]/.test(format)
    };
  });
}
function useControl(dateConfig, formatContext, inputEnableStatus, valueRef, handleChange) {
  const { formatRef, dateFormatRef, timeFormatRef, hourEnabled, minuteEnabled, secondEnabled, use12Hours } = formatContext;
  const [inputValue, setInputValue] = useState("");
  const [dateInputValue, setDateInputValue] = useState("");
  const [timeInputValue, setTimeInputValue] = useState("");
  const [panelValue, setPanelValue] = useState(void 0);
  const [visiblePanel, setVisiblePanel] = useState("datePanel");
  const [dateInputFocused, setDateInputFocused] = useState(false);
  const [timeInputFocused, setTimeInputFocused] = useState(false);
  const dateInputEnableStatus = useDateInputEnabledStatus(formatRef);
  const innerDateInputEnableStatus = useDateInputEnabledStatus(dateFormatRef);
  function initInputValue(currValue, force = false) {
    if (!currValue) {
      setInputValue("");
      return;
    }
    const { parse, format } = dateConfig;
    if (force || parse(inputValue.value, formatRef.value).valueOf() !== parse(format(currValue, formatRef.value), formatRef.value).valueOf()) {
      setInputValue(format(currValue, formatRef.value));
    }
  }
  function initDateInputValue(currValue, force = false) {
    if (!currValue) {
      setDateInputValue("");
      return;
    }
    const { parse, format } = dateConfig;
    const parsedValue = parse(dateInputValue.value, dateFormatRef.value);
    if (force || !isSameDateTime(dateConfig, parsedValue, currValue, ["year", "month", "date"])) {
      setDateInputValue(format(currValue, dateFormatRef.value));
    }
  }
  function initTimeInputValue(currValue, force = false) {
    if (!currValue) {
      setTimeInputValue("");
      return;
    }
    const { parse, format } = dateConfig;
    const parsedValue = parse(timeInputValue.value, timeFormatRef.value);
    if (force || !isSameDateTime(dateConfig, parsedValue, currValue, ["hour", "minute", "second"])) {
      setTimeInputValue(format(currValue, timeFormatRef.value));
    }
  }
  function init(force = false) {
    const currDateValue = convertToDate(dateConfig, valueRef.value, formatRef.value);
    initInputValue(currDateValue, force);
    inputEnableStatus.value.enableOverlayDateInput && initDateInputValue(currDateValue, force);
    inputEnableStatus.value.enableOverlayTimeInput && initTimeInputValue(currDateValue, force);
    setPanelValue(currDateValue);
  }
  watch([valueRef, formatRef], () => init(), { immediate: true });
  watch(inputEnableStatus, () => init());
  function parseInput(value, format, referenceDate) {
    return value ? dateConfig.parse(value, format, referenceDate) : void 0;
  }
  function checkInputValid(date) {
    return !date || dateConfig.isValid(date);
  }
  let DateTimeAdjustType;
  ((DateTimeAdjustType2) => {
    DateTimeAdjustType2[DateTimeAdjustType2["input"] = 0] = "input";
    DateTimeAdjustType2[DateTimeAdjustType2["timeInput"] = 1] = "timeInput";
    DateTimeAdjustType2[DateTimeAdjustType2["dateInput"] = 2] = "dateInput";
  })(DateTimeAdjustType || (DateTimeAdjustType = {}));
  function getDatePartAdjustTypes(enableStatus) {
    const { yearEnabled, monthEnabled, dateEnabled } = enableStatus;
    return [!yearEnabled && "year", !monthEnabled && "month", !dateEnabled && "date"].filter(Boolean);
  }
  function getTimePartAdjustTypes() {
    const _hourEnabled = hourEnabled.value || use12Hours.value;
    return [
      !_hourEnabled && "hour",
      !minuteEnabled.value && "minute",
      !secondEnabled.value && "second",
      "millisecond"
    ].filter(Boolean);
  }
  function adjustDateTimeOfInput(value, propValue, type) {
    if (!value || !propValue) {
      return value;
    }
    let typesToApply;
    if (type === 2) {
      typesToApply = [
        ...getDatePartAdjustTypes(innerDateInputEnableStatus.value),
        "hour",
        "minute",
        "second",
        "millisecond"
      ];
    } else if (type === 1) {
      typesToApply = ["year", "month", "date", ...getTimePartAdjustTypes()];
    } else {
      typesToApply = [...getDatePartAdjustTypes(dateInputEnableStatus.value), ...getTimePartAdjustTypes()];
    }
    return applyDateTime(dateConfig, propValue, value, typesToApply);
  }
  function handleInput(evt) {
    const value = evt.target.value;
    const referenceValue = convertToDate(dateConfig, valueRef.value, formatRef.value);
    setInputValue(value);
    let currDate = parseInput(value, formatRef.value, referenceValue);
    if (!checkInputValid(currDate)) {
      return;
    }
    currDate = adjustDateTimeOfInput(currDate, referenceValue, 0);
    handleChange(currDate);
  }
  function handleDateInput(evt) {
    const value = evt.target.value;
    const referenceValue = convertToDate(dateConfig, valueRef.value, formatRef.value);
    setDateInputValue(value);
    let currDate = parseInput(value, dateFormatRef.value, referenceValue);
    if (!checkInputValid(currDate)) {
      return;
    }
    currDate = adjustDateTimeOfInput(currDate, referenceValue, 2);
    handleChange(currDate);
    setVisiblePanel("datePanel");
  }
  function handleTimeInput(evt) {
    const value = evt.target.value;
    const referenceValue = convertToDate(dateConfig, valueRef.value, formatRef.value);
    setTimeInputValue(value);
    let currDate = parseInput(value, timeFormatRef.value, referenceValue);
    if (!checkInputValid(currDate)) {
      return;
    }
    currDate = adjustDateTimeOfInput(currDate, referenceValue, 1);
    handleChange(currDate);
    setVisiblePanel("timePanel");
  }
  function handleDateInputClear() {
    setDateInputValue("");
  }
  function handleTimeInputClear() {
    setTimeInputValue("");
  }
  function handlePanelChange(value) {
    handleChange(value);
  }
  function handleDateInputFocus() {
    setVisiblePanel("datePanel");
    setDateInputFocused(true);
  }
  function handleTimeInputFocus() {
    setVisiblePanel("timePanel");
    setTimeInputFocused(true);
  }
  function handleDateInputBlur() {
    setDateInputFocused(false);
  }
  function handleTimeInputBlur() {
    setTimeInputFocused(false);
  }
  return {
    inputValue,
    dateInputValue,
    timeInputValue,
    dateInputFocused,
    timeInputFocused,
    panelValue,
    visiblePanel,
    setVisiblePanel,
    init,
    handleInput,
    handleDateInput,
    handleTimeInput,
    handleDateInputClear,
    handleTimeInputClear,
    handlePanelChange,
    handleDateInputFocus,
    handleTimeInputFocus,
    handleDateInputBlur,
    handleTimeInputBlur
  };
}
function useInputEnableStatus(props, config) {
  return computed(() => {
    var _a;
    const allowInput = (_a = props.allowInput) != null ? _a : config.allowInput;
    return {
      allowInput,
      enableInput: allowInput === true,
      enableOverlayDateInput: allowInput === "overlay" || props.type === "datetime",
      enableOverlayTimeInput: props.type === "datetime"
    };
  });
}
function useKeyboardEvents(overlayOpened, setOverlayOpened) {
  return (evt) => {
    if (evt.code === "Escape") {
      setOverlayOpened(false);
      return;
    }
    if (!overlayOpened.value && !["Backspace", "Tab"].includes(evt.code)) {
      evt.preventDefault();
      setOverlayOpened(true);
      return;
    }
    if (evt.code === "Enter") {
      setOverlayOpened(false);
    }
  };
}
function useRangeKeyboardEvents(rangeControl, overlayOpened, setOverlayOpened, handleChange) {
  const { bufferUpdated, buffer } = rangeControl;
  return (evt) => {
    if (evt.code === "Escape") {
      setOverlayOpened(false);
      return;
    }
    if (!overlayOpened.value && !["Backspace", "Tab"].includes(evt.code)) {
      evt.preventDefault();
      setOverlayOpened(true);
      return;
    }
    if (evt.code === "Enter") {
      if (bufferUpdated.value) {
        handleChange(buffer.value);
      }
      setOverlayOpened(false);
    }
  };
}
function useOverlayState(props) {
  const [overlayOpened, setOverlayOpened] = useControlledProp(props, "open", false);
  const [overlayVisible, setOverlayVisible] = useState(false);
  watch(
    overlayOpened,
    (open) => {
      if (open) {
        setOverlayVisible(true);
      }
    },
    { immediate: true }
  );
  const onAfterLeave = () => {
    if (!overlayOpened.value) {
      setOverlayVisible(false);
    }
  };
  onMounted(() => {
    if (props.autofocus) {
      setOverlayOpened(true);
    }
  });
  return { overlayOpened, overlayVisible, setOverlayOpened, onAfterLeave };
}
function useTimePanelProps(props, hourEnabled, minuteEnabled, secondEnabled, use12Hours) {
  return computed(() => {
    var _a;
    return {
      ...getTimePanelProps((_a = props.timePanelOptions) != null ? _a : {}),
      hourEnabled: hourEnabled.value,
      minuteEnabled: minuteEnabled.value,
      secondEnabled: secondEnabled.value,
      use12Hours: use12Hours.value
    };
  });
}
function useRangeTimePanelProps(props, hourEnabled, minuteEnabled, secondEnabled, use12Hours) {
  const getOptions = (isFrom) => {
    var _a;
    return (_a = isArray(props.timePanelOptions) ? props.timePanelOptions[isFrom ? 0 : 1] : props.timePanelOptions) != null ? _a : {};
  };
  const rangeTimePanelProps = computed(() => {
    const enabledStatus = {
      hourEnabled: hourEnabled.value,
      minuteEnabled: minuteEnabled.value,
      secondEnabled: secondEnabled.value,
      use12Hours: use12Hours.value
    };
    return [
      { ...getTimePanelProps(getOptions(true)), ...enabledStatus },
      { ...getTimePanelProps(getOptions(false)), ...enabledStatus }
    ];
  });
  return rangeTimePanelProps;
}
function getTimePanelProps(timePanelOptions) {
  const { disabledHours, disabledMinutes, disabledSeconds, hideDisabledOptions, hourStep, minuteStep, secondStep } = timePanelOptions;
  return {
    disabledHours,
    disabledMinutes,
    disabledSeconds,
    hideDisabledOptions,
    hourStep,
    minuteStep,
    secondStep
  };
}
function usePickerPanelProps(context) {
  const {
    props,
    hourEnabled,
    minuteEnabled,
    secondEnabled,
    use12Hours,
    inputEnableStatus,
    controlContext: { visiblePanel, panelValue, handlePanelChange },
    overlayVisible,
    setOverlayOpened
  } = context;
  const _handlePanelChange = (value) => {
    handlePanelChange(value);
    if (!inputEnableStatus.value.enableOverlayTimeInput) {
      setOverlayOpened(false);
    }
  };
  const timePanelProps = useTimePanelProps(props, hourEnabled, minuteEnabled, secondEnabled, use12Hours);
  const datePanelProps2 = computed(() => ({
    cellTooltip: props.cellTooltip,
    disabledDate: props.disabledDate,
    value: panelValue.value,
    type: props.type,
    timePanelOptions: timePanelProps.value,
    visible: overlayVisible.value && visiblePanel.value,
    onChange: _handlePanelChange
  }));
  return datePanelProps2;
}
function usePickerState(props, config, dateConfig, formatRef) {
  const { accessor, control } = useAccessorAndControl();
  useFormItemRegister(control);
  const mergedSize = useFormSize(props, config);
  const mergedStatus = useFormStatus(props, control);
  const [focused, setFocused] = useState(false);
  const convertedValue = computed(() => {
    if (isArray(accessor.value)) {
      return accessor.value.map((v) => convertToDate(dateConfig, v, formatRef.value));
    }
    return convertToDate(dateConfig, accessor.value, formatRef.value);
  });
  function handleChange(value) {
    const newValue = isArray(value) ? sortRangeValue(dateConfig, value) : value;
    if (convertArray(newValue).some((v) => {
      var _a;
      return (_a = props.disabledDate) == null ? void 0 : _a.call(props, v);
    })) {
      return;
    }
    const oldValue = toRaw(convertedValue.value);
    accessor.setValue(newValue);
    callEmit(props.onChange, newValue, oldValue);
  }
  function handleClear(evt) {
    callEmit(props.onClear, evt);
    handleChange(void 0);
  }
  function handleFocus(evt) {
    setFocused(true);
    callEmit(props.onFocus, evt);
  }
  function handleBlur(evt) {
    setFocused(false);
    accessor.markAsBlurred();
    callEmit(props.onBlur, evt);
  }
  return {
    accessor,
    convertedValue,
    mergedSize,
    mergedStatus,
    focused,
    handleChange,
    handleClear,
    handleFocus,
    handleBlur
  };
}
function useTriggerProps(context) {
  const {
    props,
    config,
    accessor,
    mergedSize,
    mergedStatus,
    mergedPrefixCls,
    overlayOpened,
    onAfterLeave,
    setOverlayOpened,
    handleFocus,
    handleBlur,
    handleClear,
    handleKeyDown
  } = context;
  return computed(() => {
    var _a, _b, _c, _d, _e;
    return {
      autofocus: props.autofocus,
      borderless: props.borderless,
      value: accessor.value,
      clearable: (_a = props.clearable) != null ? _a : config.clearable,
      clearIcon: (_b = props.clearIcon) != null ? _b : config.clearIcon,
      disabled: accessor.disabled,
      open: overlayOpened.value,
      overlayContainer: (_c = props.overlayContainer) != null ? _c : config.overlayContainer,
      overlayContainerFallback: `.${mergedPrefixCls.value}-overlay-container`,
      overlayTabindex: (_d = props.overlayTabindex) != null ? _d : config.overlayTabindex,
      readonly: props.readonly,
      size: mergedSize.value,
      status: mergedStatus.value,
      suffix: (_e = props.suffix) != null ? _e : config.suffix,
      suffixRotate: false,
      "onUpdate:open": setOverlayOpened,
      onClear: handleClear,
      onFocus: handleFocus,
      onBlur: handleBlur,
      onKeydown: handleKeyDown,
      onOverlayAfterLeave: onAfterLeave
    };
  });
}
const datePickerToken = Symbol("datePickerToken");
const dateRangePickerToken = Symbol("dateRangePickerToken");
const datePickerPanelPropsToken = Symbol("datePickerPanelPropsToken");
const dateRangePickerPanelPropsToken = Symbol(
  "dateRangePickerPanelPropsToken"
);
const dateRangePickerPanelSlotsToken = Symbol("dateRangePickerPanelSlotsToken");
var PickerOverlayFooter = /* @__PURE__ */ defineComponent({
  name: "IxDatePickerOverlayFooter",
  setup(_, {
    slots
  }) {
    const {
      mergedPrefixCls,
      props
    } = inject(datePickerToken);
    return () => createVNode(_Footer, {
      "class": `${mergedPrefixCls.value}-overlay-footer`,
      "footer": props.footer
    }, slots);
  }
});
function useInputProps(context) {
  return computed(() => {
    var _a, _b;
    const { props, config, accessor } = context;
    return {
      borderless: false,
      clearable: (_a = props.clearable) != null ? _a : config.clearable,
      clearIcon: (_b = props.clearIcon) != null ? _b : config.clearIcon,
      disabled: accessor.disabled,
      size: "sm"
    };
  });
}
var PickerOverlayInputs = /* @__PURE__ */ defineComponent({
  name: "IxDatePickerOverlayInputs",
  setup(_, {
    slots
  }) {
    const context = inject(datePickerToken);
    const {
      mergedPrefixCls,
      dateFormatRef,
      timeFormatRef,
      inputEnableStatus,
      inputRef,
      controlContext: {
        dateInputValue,
        timeInputValue,
        dateInputFocused,
        timeInputFocused,
        handleDateInput,
        handleTimeInput,
        handleDateInputClear,
        handleTimeInputClear,
        handleDateInputFocus,
        handleTimeInputFocus,
        handleDateInputBlur,
        handleTimeInputBlur
      },
      handleKeyDown,
      handleClear
    } = context;
    const inputInstance = ref();
    onMounted(() => {
      var _a;
      if (inputEnableStatus.value.allowInput !== "overlay") {
        return;
      }
      inputRef.value = (_a = inputInstance.value) == null ? void 0 : _a.getInputElement();
    });
    const _handleDateInputFocus = (evt) => {
      handleDateInputFocus();
      if (inputEnableStatus.value.allowInput === "overlay") {
        inputRef.value = evt.target;
      }
    };
    const _handleTimeInputFocus = (evt) => {
      handleTimeInputFocus();
      if (inputEnableStatus.value.allowInput === "overlay") {
        inputRef.value = evt.target;
      }
    };
    const _handleDateInputClear = (evt) => {
      if (!inputEnableStatus.value.enableOverlayTimeInput) {
        handleClear(evt);
      }
      handleDateInputClear();
    };
    const inputProps = useInputProps(context);
    return () => {
      if (!inputEnableStatus.value.enableOverlayDateInput) {
        return null;
      }
      const prefixCls = `${mergedPrefixCls.value}-overlay-inputs`;
      return createVNode("div", {
        "class": prefixCls
      }, [createVNode(_Input, mergeProps({
        "ref": inputInstance
      }, inputProps.value, {
        "class": `${prefixCls}-date-input`,
        "value": dateInputValue.value,
        "clearVisible": !!dateInputValue.value && !!inputEnableStatus.value.allowInput,
        "focused": dateInputFocused.value,
        "placeholder": dateFormatRef.value,
        "readonly": !inputEnableStatus.value.allowInput,
        "onInput": handleDateInput,
        "onFocus": _handleDateInputFocus,
        "onBlur": handleDateInputBlur,
        "onKeydown": handleKeyDown,
        "onClear": _handleDateInputClear
      }), slots), inputEnableStatus.value.enableOverlayTimeInput && createVNode(_Input, mergeProps(inputProps.value, {
        "class": `${prefixCls}-time-input`,
        "value": timeInputValue.value,
        "clearVisible": !!timeInputValue.value && !!inputEnableStatus.value.allowInput,
        "focused": timeInputFocused.value,
        "placeholder": timeFormatRef.value,
        "readonly": !inputEnableStatus.value.allowInput,
        "onInput": handleTimeInput,
        "onFocus": _handleTimeInputFocus,
        "onBlur": handleTimeInputBlur,
        "onKeydown": handleKeyDown,
        "onClear": handleTimeInputClear
      }), slots)]);
    };
  }
});
function useActiveValue(dateConfig, mergedProps) {
  var _a;
  const [activeValue, setActiveValue] = useControlledProp(
    mergedProps,
    "activeValue",
    (_a = mergedProps.value.value) != null ? _a : dateConfig.now()
  );
  watch(
    () => mergedProps.value.value,
    (value) => setActiveValue(value != null ? value : dateConfig.now())
  );
  return {
    activeValue,
    setActiveValue
  };
}
const viewTypeMap = {
  date: "month",
  datetime: "month",
  week: "month",
  month: "year",
  quarter: "year",
  year: "year"
};
function useRangeActiveValue(dateConfig, mergedProps, panelValue, isSelecting) {
  var _a, _b;
  const { set, get } = dateConfig;
  const now = dateConfig.now();
  const viewType = computed(() => viewTypeMap[mergedProps.value.type]);
  const viewSpan = computed(() => mergedProps.value.type === "year" ? 12 : 1);
  const fromPanelValue = computed(() => {
    var _a2;
    return (_a2 = panelValue.value) == null ? void 0 : _a2[0];
  });
  const toPanelValue = computed(() => {
    var _a2;
    return (_a2 = panelValue.value) == null ? void 0 : _a2[1];
  });
  const checkActiveDateValid = (from, to) => {
    if (!from || !to) {
      return false;
    }
    const fromViewValue = get(from, viewType.value);
    const toViewValue = get(to, viewType.value);
    return mergedProps.value.type === "year" ? fromViewValue < toViewValue - viewSpan.value : (() => {
      const fromViewYearValue = get(from, "year");
      const toViewYearValue = get(to, "year");
      if (fromViewYearValue > toViewYearValue) {
        return false;
      }
      return fromViewYearValue < toViewYearValue || fromViewValue < toViewValue;
    })();
  };
  const calcValidActiveDate = (from, to, type) => {
    const getViewDate = (value) => set(value, get(value, viewType.value) + (type === "from" ? -viewSpan.value : viewSpan.value), viewType.value);
    if (!from) {
      return type === "from" ? now : getViewDate(now);
    }
    if (!to) {
      return type === "from" ? from : getViewDate(from);
    }
    const valid = checkActiveDateValid(from, to);
    if (type === "from") {
      return valid ? from : getViewDate(to);
    }
    return valid ? to : getViewDate(from);
  };
  const initActiveDate = () => {
    var _a2, _b2, _c;
    if (isSelecting.value) {
      return;
    }
    const fromValue = (_b2 = (_a2 = panelValue.value) == null ? void 0 : _a2[0]) != null ? _b2 : now;
    const toValue = (_c = panelValue.value) == null ? void 0 : _c[1];
    const valueAlreadyInView = [fromValue, toValue].every((value) => {
      if (!value) {
        return true;
      }
      const yearValue = get(value, "year");
      if (viewTypeMap[mergedProps.value.type] === "year") {
        return [fromActiveValue, toActiveValue].map((activeDate) => get(activeDate.value, "year")).includes(yearValue);
      }
      const monthValue = get(value, "month");
      return [fromActiveValue, toActiveValue].some(
        (activeDate) => yearValue === get(activeDate.value, "year") && monthValue === get(activeDate.value, "month")
      );
    });
    if (valueAlreadyInView && checkActiveDateValid(fromActiveValue.value, toActiveValue.value)) {
      return;
    }
    setActiveValue([fromValue, calcValidActiveDate(fromValue, toValue, "to")]);
  };
  const [activeValue, setActiveValue] = useControlledProp(mergedProps, "activeValue", [
    (_a = fromPanelValue.value) != null ? _a : now,
    calcValidActiveDate((_b = fromPanelValue.value) != null ? _b : now, toPanelValue.value, "to")
  ]);
  const fromActiveValue = computed(() => activeValue.value[0]);
  const toActiveValue = computed(() => activeValue.value[1]);
  watch([panelValue, () => mergedProps.value.visible, () => mergedProps.value.type], initActiveDate);
  const handleFromActiveValueUpdate = (value) => {
    setActiveValue([value, calcValidActiveDate(value, toActiveValue.value, "to")]);
  };
  const handleToActiveValueUpdate = (value) => {
    setActiveValue([calcValidActiveDate(fromActiveValue.value, value, "from"), value]);
  };
  return {
    fromActiveValue,
    toActiveValue,
    setFromActiveValue: handleFromActiveValueUpdate,
    setToActiveValue: handleToActiveValueUpdate
  };
}
const datePickerCommonProps = {
  control: {
    type: [String, Number, Object, Array],
    default: void 0
  },
  cellTooltip: Function,
  open: {
    type: Boolean,
    default: void 0
  },
  allowInput: {
    type: [Boolean, String],
    default: void 0
  },
  autofocus: {
    type: Boolean,
    default: false
  },
  borderless: {
    type: Boolean,
    default: void 0
  },
  clearable: {
    type: Boolean,
    default: void 0
  },
  clearIcon: String,
  disabled: {
    type: Boolean,
    default: false
  },
  disabledDate: Function,
  format: String,
  dateFormat: String,
  timeFormat: String,
  overlayClassName: String,
  overlayContainer: {
    type: [String, HTMLElement, Function],
    default: void 0
  },
  overlayTabindex: { type: Number, default: void 0 },
  overlayRender: Function,
  readonly: {
    type: Boolean,
    default: false
  },
  size: String,
  status: String,
  suffix: String,
  type: {
    type: String,
    default: "date"
  },
  "onUpdate:open": [Function, Array],
  onClear: [Function, Array],
  onFocus: [Function, Array],
  onBlur: [Function, Array]
};
const datePickerProps = {
  ...datePickerCommonProps,
  value: [String, Date, Number],
  footer: { type: [Boolean, Array, Object], default: false },
  placeholder: String,
  timePanelOptions: Object,
  onChange: [Function, Array],
  onInput: [Function, Array],
  "onUpdate:value": [Function, Array]
};
const dateRangePickerProps = {
  ...datePickerCommonProps,
  value: Array,
  footer: { type: [Boolean, Array, Object], default: true },
  placeholder: Array,
  separator: [String, Object],
  shortcuts: [Array, Object],
  timePanelOptions: [Object, Array],
  onChange: [Function, Array],
  onInput: [Function, Array],
  onSelect: [Function, Array],
  "onUpdate:value": [Function, Array]
};
const datePanelProps = {
  activeValue: Date,
  cellTooltip: Function,
  disabledDate: Function,
  value: Date,
  type: {
    type: String,
    default: void 0
  },
  timePanelOptions: {
    type: Object,
    default: void 0
  },
  visible: {
    type: [String, Boolean],
    default: void 0
  },
  onChange: [Function, Array],
  "onUpdate:activeValue": [Function, Array]
};
const dateRangePanelProps = {
  activeValue: Array,
  cellTooltip: Function,
  disabledDate: Function,
  value: Array,
  type: {
    type: String,
    default: void 0
  },
  timePanelOptions: {
    type: Array,
    default: void 0
  },
  visible: {
    type: [String, Boolean],
    default: void 0
  },
  onChange: [Function, Array],
  onSelect: [Function, Array],
  "onUpdate:activeValue": [Function, Array]
};
function convertPickerTypeToConfigType(type) {
  return type === "datetime" ? "date" : type;
}
const oneDayDuration = 24 * 60 * 60 * 1e3;
function getPresetRangeShortcutValue(dateConfig, presetShortcut) {
  const { now, startOf, endOf } = dateConfig;
  switch (presetShortcut) {
    case "today":
    default:
      return () => {
        const currentDate = now();
        return [startOf(currentDate, "date"), endOf(currentDate, "date")];
      };
    case "yesterday": {
      return () => {
        const currentDate = now();
        const yesterday = new Date(currentDate.valueOf() - oneDayDuration);
        return [startOf(yesterday, "date"), endOf(yesterday, "date")];
      };
    }
    case "last24h": {
      return () => {
        const currentDate = now();
        const currentHour = endOf(currentDate, "hour");
        return [new Date(currentHour.valueOf() - oneDayDuration), currentHour];
      };
    }
    case "last7d": {
      return () => {
        const currentDate = now();
        const endOfToday = endOf(currentDate, "date");
        return [new Date(endOfToday.valueOf() - oneDayDuration * 7), endOfToday];
      };
    }
    case "last30d": {
      return () => {
        const currentDate = now();
        const endOfToday = endOf(currentDate, "date");
        return [new Date(endOfToday.valueOf() - oneDayDuration * 30), endOfToday];
      };
    }
    case "last180d": {
      return () => {
        const currentDate = now();
        const endOfToday = endOf(currentDate, "date");
        return [new Date(endOfToday.valueOf() - oneDayDuration * 180), endOfToday];
      };
    }
  }
}
function extractShortcutValue(shortcut) {
  const value = shortcut.value;
  if (!value) {
    return;
  }
  return isFunction(value) ? value() : value;
}
var DatePanel = /* @__PURE__ */ defineComponent({
  name: "IxDatePanel",
  props: datePanelProps,
  setup(props, {
    slots
  }) {
    const inherittedProps = inject(datePickerPanelPropsToken, null);
    const common = useGlobalConfig("common");
    const {
      globalHashId,
      hashId,
      registerToken
    } = useThemeToken("datePicker");
    const {
      registerToken: registerTimePickerToken
    } = useThemeToken("timePicker");
    registerToken(getThemeTokens);
    registerTimePickerToken(getTimePickerThemeTokens);
    const mergedPrefixCls = computed(() => `${common.prefixCls}-date-picker-panel`);
    const dateConfig = useDateConfig();
    const mergedProps = computed(() => {
      var _a;
      if (!inherittedProps) {
        return props;
      }
      const res = {};
      Object.keys(props).forEach((key) => {
        var _a2;
        res[key] = (_a2 = props[key]) != null ? _a2 : inherittedProps.value[key];
      });
      res.type = (_a = res.type) != null ? _a : "date";
      return res;
    });
    const {
      activeValue,
      setActiveValue
    } = useActiveValue(dateConfig, mergedProps);
    function handleChange(value) {
      callEmit(mergedProps.value.onChange, value);
    }
    function handleDatePanelChange(value) {
      handleChange(mergedProps.value.value ? applyDateTime(dateConfig, mergedProps.value.value, value, ["hour", "minute", "second", "millisecond"]) : value);
    }
    function handleTimePanelChange(value) {
      handleChange(mergedProps.value.value ? applyDateTime(dateConfig, mergedProps.value.value, value, ["year", "month", "date"]) : value);
    }
    const handleDatePanelCellClick = (value) => {
      handleDatePanelChange(value);
    };
    return () => {
      const _props = mergedProps.value;
      const datePanelType = convertPickerTypeToConfigType(_props.type);
      const datePanelProps2 = {
        cellTooltip: _props.cellTooltip,
        disabledDate: _props.disabledDate,
        type: datePanelType,
        value: _props.value,
        visible: _props.type === "datetime" ? _props.visible === "datePanel" : !!_props.visible,
        activeDate: activeValue.value,
        onCellClick: handleDatePanelCellClick,
        "onUpdate:activeDate": setActiveValue
      };
      const _timePanelProps = {
        ..._props.timePanelOptions,
        activeValue: activeValue.value,
        value: _props.value,
        visible: _props.type === "datetime" ? _props.visible === "timePanel" : false,
        onChange: handleTimePanelChange,
        "onUpdate:activeValue": setActiveValue
      };
      return createVNode("div", {
        "class": [mergedPrefixCls.value, globalHashId.value, hashId.value]
      }, [withDirectives(createVNode(_DatePanel, datePanelProps2, slots), [[vShow, _props.visible !== "timePanel"]]), _props.type === "datetime" && withDirectives(createVNode(_TimePanel, _timePanelProps, null), [[vShow, _props.visible === "timePanel"]])]);
    };
  }
});
var Content = /* @__PURE__ */ defineComponent({
  setup(_, {
    slots
  }) {
    const context = inject(datePickerToken);
    const {
      props,
      mergedPrefixCls
    } = context;
    return () => {
      if (slots.overlay) {
        return slots.overlay();
      }
      const children = [createVNode(PickerOverlayInputs, null, slots), createVNode(DatePanel, null, slots), createVNode(PickerOverlayFooter, null, slots)];
      return props.overlayRender ? props.overlayRender(children) : createVNode("div", {
        "class": `${mergedPrefixCls.value}-overlay-content`
      }, [children]);
    };
  }
});
var Trigger = /* @__PURE__ */ defineComponent({
  inheritAttrs: false,
  setup(_, {
    expose,
    slots
  }) {
    const context = inject(datePickerToken);
    const {
      accessor,
      props,
      locale,
      controlContext: {
        inputValue,
        handleInput: _handleInput
      },
      mergedPrefixCls,
      formatRef,
      inputEnableStatus,
      inputRef
    } = context;
    const triggerInputRef = ref();
    const placeholder = computed(() => {
      var _a;
      return (_a = props.placeholder) != null ? _a : locale.datePicker[`${props.type}Placeholder`];
    });
    const inputSize = computed(() => Math.max(10, formatRef.value.length) + 2);
    const focus = () => {
      var _a;
      (_a = (inputEnableStatus.value.allowInput === "overlay" ? triggerInputRef : inputRef).value) == null ? void 0 : _a.focus();
    };
    expose({
      focus
    });
    const handleInput = (evt) => {
      _handleInput(evt);
      callEmit(props.onInput, evt);
    };
    return () => {
      var _a, _b;
      return createVNode("div", {
        "class": `${mergedPrefixCls.value}-input`
      }, [(_b = (_a = slots.triggerContent) == null ? void 0 : _a.call(slots, {
        inputValue: inputValue.value,
        placeholder: placeholder.value,
        readonly: props.readonly || inputEnableStatus.value.enableInput === false,
        disabled: accessor.disabled,
        handleInput
      })) != null ? _b : createVNode("input", {
        "ref": inputEnableStatus.value.allowInput === "overlay" ? triggerInputRef : inputRef,
        "class": `${mergedPrefixCls.value}-input-inner`,
        "autocomplete": "off",
        "disabled": accessor.disabled,
        "placeholder": placeholder.value,
        "readonly": props.readonly || inputEnableStatus.value.enableInput === false,
        "size": inputSize.value,
        "value": inputValue.value,
        "onInput": handleInput
      }, null)]);
    };
  }
});
var DatePicker = /* @__PURE__ */ defineComponent({
  name: "IxDatePicker",
  inheritAttrs: false,
  props: datePickerProps,
  setup(props, {
    attrs,
    expose,
    slots
  }) {
    const common = useGlobalConfig("common");
    const locale = useGlobalConfig("locale");
    const {
      globalHashId,
      hashId,
      registerToken
    } = useThemeToken("datePicker");
    registerToken(getThemeTokens);
    const mergedPrefixCls = computed(() => `${common.prefixCls}-date-picker`);
    const config = useGlobalConfig("datePicker");
    const dateConfig = useDateConfig();
    const triggerRef = ref();
    const {
      elementRef: inputRef,
      focus,
      blur
    } = useFormElement();
    expose({
      focus,
      blur
    });
    const {
      overlayOpened,
      overlayVisible,
      onAfterLeave,
      setOverlayOpened
    } = useOverlayState(props);
    const inputEnableStatus = useInputEnableStatus(props, config);
    const formatContext = useFormat(props, config);
    const pickerStateContext = usePickerState(props, config, dateConfig, formatContext.formatRef);
    const {
      accessor,
      focused,
      handleFocus,
      handleBlur,
      handleChange
    } = pickerStateContext;
    const controlContext = useControl(dateConfig, formatContext, inputEnableStatus, toRef(accessor, "value"), handleChange);
    const handleKeyDown = useKeyboardEvents(overlayOpened, setOverlayOpened);
    const context = {
      props,
      slots,
      common,
      locale,
      config,
      mergedPrefixCls,
      dateConfig,
      inputRef,
      inputEnableStatus,
      overlayOpened,
      overlayVisible,
      onAfterLeave,
      setOverlayOpened,
      handleKeyDown,
      controlContext,
      ...formatContext,
      ...pickerStateContext,
      handleFocus,
      handleBlur
    };
    const triggerProps = useTriggerProps(context);
    const datePanelProps2 = usePickerPanelProps(context);
    provide(datePickerToken, context);
    provide(datePickerPanelPropsToken, datePanelProps2);
    watch(overlayOpened, (opened) => {
      var _a;
      if (opened) {
        setTimeout(() => {
          var _a2;
          (_a2 = inputRef.value) == null ? void 0 : _a2.focus();
        });
      } else {
        controlContext.init(true);
        if (focused.value) {
          (_a = triggerRef.value) == null ? void 0 : _a.focus();
        }
      }
    });
    const renderTrigger = () => createVNode(Trigger, {
      "ref": triggerRef
    }, slots);
    const renderContent = () => createVNode(Content, null, slots);
    const overlayClass = computed(() => normalizeClass([`${mergedPrefixCls.value}-overlay`, globalHashId.value, hashId.value, props.overlayClassName]));
    return () => createVNode(IxControlTrigger, mergeProps(triggerProps.value, {
      "class": `${mergedPrefixCls.value} ${globalHashId.value} ${hashId.value}`,
      "overlayClassName": overlayClass.value
    }, attrs), {
      default: renderTrigger,
      overlay: renderContent,
      suffix: slots.suffix,
      clearIcon: slots.clearIcon
    });
  }
});
function useRangeControl(dateConfig, formatContext, inputEnableStatus, valueRef, typeRef) {
  const { formatRef, hourEnabled, secondEnabled, minuteEnabled } = formatContext;
  const convertedValue = computed(
    () => convertArray(valueRef.value).map((v) => convertToDate(dateConfig, v, formatRef.value))
  );
  const [buffer, setBuffer] = useState(convertedValue.value);
  const [bufferUpdated, setBufferUpdated] = useState(false);
  const handleBufferUpdate = (values) => {
    setBuffer(sortRangeValue(dateConfig, getRangeValue(dateConfig, values, formatRef.value), "date"));
    setBufferUpdated(true);
  };
  const [visiblePanel, setVisiblePanel] = useState("datePanel");
  const handleVisiblePanelUpdate = (value) => {
    visiblePanel.value !== value && setVisiblePanel(value);
    fromControl.visiblePanel.value !== value && fromControl.setVisiblePanel(value);
    toControl.visiblePanel.value !== value && toControl.setVisiblePanel(value);
  };
  const rangeValueRef = computed(() => convertArray(buffer.value));
  const fromDateRef = computed(() => rangeValueRef.value[0]);
  const toDateRef = computed(() => rangeValueRef.value[1]);
  watch([valueRef, formatRef], ([value]) => {
    handleBufferUpdate(value);
  });
  const getValidBufferValue = (value, isFrom) => {
    var _a, _b, _c, _d;
    if (isFrom) {
      return compareDateTime(dateConfig, value, (_a = buffer.value) == null ? void 0 : _a[1], "date") > 0 ? [value, value] : [value, (_b = buffer.value) == null ? void 0 : _b[1]];
    }
    return compareDateTime(dateConfig, value, (_c = buffer.value) == null ? void 0 : _c[0], "date") < 0 ? [value, value] : [(_d = buffer.value) == null ? void 0 : _d[0], value];
  };
  const getAdjustedBufferValue = (value, isFrom) => {
    const adjustType = secondEnabled.value ? "second" : minuteEnabled.value ? "minute" : hourEnabled.value ? "hour" : convertPickerTypeToConfigType(typeRef.value);
    return adjustRangeValue(dateConfig, getValidBufferValue(value, isFrom), convertedValue.value, adjustType);
  };
  const fromControl = useControl(dateConfig, formatContext, inputEnableStatus, fromDateRef, (value) => {
    setBuffer(getAdjustedBufferValue(value, true));
    setBufferUpdated(true);
  });
  const toControl = useControl(dateConfig, formatContext, inputEnableStatus, toDateRef, (value) => {
    setBuffer(getAdjustedBufferValue(value, false));
    setBufferUpdated(true);
  });
  watch(fromControl.visiblePanel, handleVisiblePanelUpdate);
  watch(toControl.visiblePanel, handleVisiblePanelUpdate);
  const init = (force = false) => {
    handleBufferUpdate(valueRef.value);
    setBufferUpdated(false);
    fromControl.init(force);
    toControl.init(force);
  };
  return {
    buffer,
    bufferUpdated,
    visiblePanel,
    setVisiblePanel: handleVisiblePanelUpdate,
    fromControl,
    toControl,
    init,
    handlePanelChange: handleBufferUpdate
  };
}
function getRangeValue(dateConfig, values, format) {
  return convertArray(values).map((v) => convertToDate(dateConfig, v, format));
}
function useRangePickerPanelProps(context) {
  const {
    props,
    hourEnabled,
    minuteEnabled,
    secondEnabled,
    use12Hours,
    rangeControlContext: { buffer, visiblePanel, handlePanelChange },
    overlayVisible
  } = context;
  const timePanelProps = useRangeTimePanelProps(props, hourEnabled, minuteEnabled, secondEnabled, use12Hours);
  const datePanelProps2 = computed(() => {
    return {
      value: buffer.value,
      cellTooltip: props.cellTooltip,
      disabledDate: props.disabledDate,
      type: props.type,
      timePanelOptions: timePanelProps.value,
      visible: overlayVisible.value && visiblePanel.value,
      onChange: handlePanelChange,
      onSelect: props.onSelect
    };
  });
  return datePanelProps2;
}
var RangePickerOverlayFooter = /* @__PURE__ */ defineComponent({
  name: "IxDateRangePickerFooter",
  setup(_, {
    slots
  }) {
    const {
      locale,
      mergedPrefixCls,
      props,
      rangeControlContext: {
        buffer,
        bufferUpdated
      },
      handleChange,
      setOverlayOpened
    } = inject(dateRangePickerToken);
    const handleConfirm = () => {
      if (bufferUpdated.value) {
        handleChange(buffer.value);
      }
      setOverlayOpened(false);
    };
    return () => createVNode(_Footer, {
      "class": `${mergedPrefixCls.value}-overlay-footer`,
      "footer": props.footer,
      "okText": locale.dateRangePicker.okText,
      "okButton": {
        size: "xs",
        mode: "primary"
      },
      "cancelVisible": false,
      "ok": handleConfirm
    }, slots);
  }
});
var RangePickerOverlayInputs = /* @__PURE__ */ defineComponent({
  name: "IxDateRangePickerOverlayInputs",
  setup(_, {
    slots
  }) {
    const context = inject(dateRangePickerToken);
    const {
      dateFormatRef,
      timeFormatRef,
      rangeControlContext: {
        fromControl,
        toControl
      },
      mergedPrefixCls,
      inputEnableStatus,
      inputRef,
      handleKeyDown,
      renderSeparator
    } = context;
    const inputInstance = ref();
    onMounted(() => {
      var _a;
      if (inputEnableStatus.value.allowInput !== "overlay") {
        return;
      }
      inputRef.value = (_a = inputInstance.value) == null ? void 0 : _a.getInputElement();
    });
    const inputProps = useInputProps(context);
    const renderInputsSide = (prefixCls, isFrom) => {
      const {
        enableOverlayTimeInput
      } = inputEnableStatus.value;
      const {
        dateInputValue,
        timeInputValue,
        dateInputFocused,
        timeInputFocused,
        handleDateInput,
        handleTimeInput,
        handleDateInputClear,
        handleTimeInputClear,
        handleDateInputFocus,
        handleTimeInputFocus,
        handleDateInputBlur,
        handleTimeInputBlur
      } = isFrom ? fromControl : toControl;
      const _handleDateInputFocus = (evt) => {
        handleDateInputFocus();
        if (inputEnableStatus.value.allowInput === "overlay") {
          inputRef.value = evt.target;
        }
      };
      const _handleTimeInputFocus = (evt) => {
        handleTimeInputFocus();
        if (inputEnableStatus.value.allowInput === "overlay") {
          inputRef.value = evt.target;
        }
      };
      return createVNode("div", {
        "class": `${prefixCls}-side`
      }, [createVNode(_Input, mergeProps({
        "ref": isFrom ? inputInstance : void 0
      }, inputProps.value, {
        "class": `${prefixCls}-date-input`,
        "value": dateInputValue.value,
        "clearVisible": !!dateInputValue.value && !!inputEnableStatus.value.allowInput,
        "focused": dateInputFocused.value,
        "placeholder": dateFormatRef.value,
        "readonly": !inputEnableStatus.value.allowInput,
        "onInput": handleDateInput,
        "onFocus": _handleDateInputFocus,
        "onBlur": handleDateInputBlur,
        "onKeydown": handleKeyDown,
        "onClear": handleDateInputClear
      }), slots), enableOverlayTimeInput && createVNode(_Input, mergeProps(inputProps.value, {
        "class": `${prefixCls}-time-input`,
        "value": timeInputValue.value,
        "clearVisible": !!timeInputValue.value && !!inputEnableStatus.value.allowInput,
        "focused": timeInputFocused.value,
        "placeholder": timeFormatRef.value,
        "readonly": !inputEnableStatus.value.allowInput,
        "onInput": handleTimeInput,
        "onFocus": _handleTimeInputFocus,
        "onBlur": handleTimeInputBlur,
        "onKeydown": handleKeyDown,
        "onClear": handleTimeInputClear
      }), slots)]);
    };
    return () => {
      const prefixCls = `${mergedPrefixCls.value}-overlay`;
      const inputsCls = `${prefixCls}-inputs`;
      if (!inputEnableStatus.value.enableOverlayDateInput) {
        return null;
      }
      return createVNode("div", {
        "class": inputsCls
      }, [renderInputsSide(inputsCls, true), createVNode("div", {
        "class": `${prefixCls}-separator`
      }, [renderSeparator()]), renderInputsSide(inputsCls, false)]);
    };
  }
});
function useRangePanelState(mergedProps, dateConfig) {
  const [selectingDates, setSelectingDates] = useState(mergedProps.value.value);
  const [isSelecting, setIsSelecting] = useState(false);
  watch(
    () => mergedProps.value.visible,
    (visible) => {
      setIsSelecting(false);
      if (!visible) {
        callEmit(mergedProps.value.onSelect, mergedProps.value.value);
      }
    }
  );
  const panelValue = computed(() => {
    if (isSelecting.value) {
      return sortRangeValue(dateConfig, [...convertArray(selectingDates.value)], "date");
    }
    return convertArray(mergedProps.value.value);
  });
  const handleChange = (value) => {
    callEmit(mergedProps.value.onChange, value);
    callEmit(mergedProps.value.onSelect, value);
  };
  const handleDatePanelCellClick = (value) => {
    if (!isSelecting.value) {
      setIsSelecting(true);
      setSelectingDates([value, void 0]);
      callEmit(mergedProps.value.onSelect, [value, void 0]);
    } else {
      const propsValue = convertArray(mergedProps.value.value);
      const sortedValue = sortRangeValue(dateConfig, [selectingDates.value[0], value], "date");
      handleChange(
        adjustRangeValue(
          dateConfig,
          sortedValue,
          propsValue,
          convertPickerTypeToConfigType(mergedProps.value.type)
        )
      );
      setIsSelecting(false);
    }
  };
  const handleDatePanelCellMouseenter = (value) => {
    var _a;
    if (!isSelecting.value) {
      return;
    }
    setSelectingDates([(_a = selectingDates.value) == null ? void 0 : _a[0], value]);
  };
  return {
    panelValue,
    isSelecting,
    handleChange,
    handleDatePanelCellClick,
    handleDatePanelCellMouseenter
  };
}
var DateRangePanel = /* @__PURE__ */ defineComponent({
  name: "IxDateRangePanel",
  props: dateRangePanelProps,
  setup(props, {
    slots
  }) {
    const inherittedProps = inject(dateRangePickerPanelPropsToken, null);
    const common = useGlobalConfig("common");
    const {
      globalHashId,
      hashId,
      registerToken
    } = useThemeToken("datePicker");
    const {
      registerToken: registerTimePickerToken
    } = useThemeToken("timePicker");
    registerToken(getThemeTokens);
    registerTimePickerToken(getTimePickerThemeTokens);
    const mergedPrefixCls = computed(() => `${common.prefixCls}-date-range-picker-panel`);
    const dateConfig = useDateConfig();
    const mergedProps = computed(() => {
      var _a;
      if (!inherittedProps) {
        return props;
      }
      const res = {};
      Object.keys(props).forEach((key) => {
        var _a2;
        res[key] = (_a2 = props[key]) != null ? _a2 : inherittedProps.value[key];
      });
      res.type = (_a = res.type) != null ? _a : "date";
      return res;
    });
    const {
      handleChange,
      handleDatePanelCellClick,
      handleDatePanelCellMouseenter,
      panelValue,
      isSelecting
    } = useRangePanelState(mergedProps, dateConfig);
    const {
      fromActiveValue,
      toActiveValue,
      setFromActiveValue,
      setToActiveValue
    } = useRangeActiveValue(dateConfig, mergedProps, panelValue, isSelecting);
    const renderSide = (isFrom) => {
      var _a, _b;
      const _props = mergedProps.value;
      const timeValue = (_a = panelValue.value) == null ? void 0 : _a[isFrom ? 0 : 1];
      const datePanelType = convertPickerTypeToConfigType(_props.type);
      const activeValue = isFrom ? fromActiveValue.value : toActiveValue.value;
      const handleTimePanelChange = (value) => {
        var _a2, _b2;
        handleChange(isFrom ? [value, (_a2 = panelValue.value) == null ? void 0 : _a2[1]] : [(_b2 = panelValue.value) == null ? void 0 : _b2[0], value]);
      };
      const datePanelProps2 = {
        cellTooltip: _props.cellTooltip,
        disabledDate: _props.disabledDate,
        type: datePanelType,
        value: panelValue.value,
        visible: _props.type === "datetime" ? _props.visible === "datePanel" : !!_props.visible,
        activeDate: activeValue,
        onCellClick: handleDatePanelCellClick,
        onCellMouseenter: handleDatePanelCellMouseenter,
        "onUpdate:activeDate": isFrom ? setFromActiveValue : setToActiveValue
      };
      const _timePanelProps = {
        ...(_b = isArray(_props.timePanelOptions) ? _props.timePanelOptions[isFrom ? 0 : 1] : _props.timePanelOptions) != null ? _b : {},
        activeValue: timeValue != null ? timeValue : activeValue,
        value: timeValue,
        visible: _props.type === "datetime" ? _props.visible === "timePanel" : false,
        onChange: handleTimePanelChange,
        "onUpdate:activeValue": isFrom ? setFromActiveValue : setToActiveValue
      };
      return createVNode("div", {
        "class": `${mergedPrefixCls.value}-side`
      }, [withDirectives(createVNode(_DatePanel, datePanelProps2, slots), [[vShow, _props.visible !== "timePanel"]]), _props.type === "datetime" && withDirectives(createVNode(_TimePanel, _timePanelProps, null), [[vShow, _props.visible === "timePanel"]])]);
    };
    return () => {
      var _a, _b;
      const prefixCls = mergedPrefixCls.value;
      return createVNode("div", {
        "class": [prefixCls, globalHashId.value, hashId.value]
      }, [renderSide(true), (_b = (_a = slots.separator) == null ? void 0 : _a.call(slots)) != null ? _b : createVNode("div", {
        "class": `${prefixCls}-separator`
      }, null), renderSide(false)]);
    };
  }
});
const rangePickerPanelRenderer = (context) => {
  return [
    h(RangePickerOverlayInputs, null, context.slots),
    h(DateRangePanel),
    h(RangePickerOverlayFooter, null, context.slots)
  ];
};
function useRangeShortcuts(shortcutsProp, valueRef, visibleRef, presetPanelRenderer = rangePickerPanelRenderer) {
  const dateConfig = useDateConfig();
  const { dateRangePicker: dateRangePickerLocale } = useGlobalConfig("locale");
  const [selectedShortcut, setSelectedShortcut] = useState(void 0);
  const showShortcutPanel = computed(() => {
    var _a;
    if (isArray(shortcutsProp.value)) {
      return false;
    }
    return !!((_a = shortcutsProp.value) == null ? void 0 : _a.showPanel);
  });
  const shortcuts = computed(() => {
    var _a;
    if (!shortcutsProp.value) {
      return [];
    }
    let shortcuts2 = [];
    let showPanel = false;
    if (isArray(shortcutsProp.value)) {
      shortcuts2 = shortcutsProp.value;
      showPanel = false;
    } else {
      shortcuts2 = shortcutsProp.value.shortcuts;
      showPanel = (_a = shortcutsProp.value.showPanel) != null ? _a : showPanel;
    }
    return shortcuts2.map((shortcut) => {
      var _a2;
      if (isString(shortcut)) {
        return getResolvedPresetShortcutOption(
          dateConfig,
          shortcut,
          dateRangePickerLocale,
          showPanel,
          presetPanelRenderer
        );
      }
      const panelRenderer = (_a2 = shortcut.panelRenderer) != null ? _a2 : showPanel && shortcut.value ? rangePickerPanelRenderer : void 0;
      return {
        ...shortcut,
        confirmOnSelect: !panelRenderer ? true : shortcut.confirmOnSelect,
        panelRenderer
      };
    }).filter(Boolean);
  });
  const customShortcut = computed(() => shortcuts.value.find((s) => s.key === "custom"));
  const getSelectedShortcutByValue = () => {
    var _a;
    const value = valueRef.value;
    if (!value) {
      return;
    }
    return (_a = shortcuts.value.find(
      (shortcut) => shortcut.value && (value == null ? void 0 : value.every((v, i) => {
        var _a2;
        return shortcut && ((_a2 = extractShortcutValue(shortcut)) == null ? void 0 : _a2[i].valueOf()) === (v == null ? void 0 : v.valueOf());
      }))
    )) != null ? _a : customShortcut.value;
  };
  watch(
    valueRef,
    () => {
      const selectedShortcutByValue = getSelectedShortcutByValue();
      if ((selectedShortcutByValue == null ? void 0 : selectedShortcutByValue.value) || selectedShortcut.value) {
        setSelectedShortcut(selectedShortcutByValue);
      }
    },
    {
      flush: "pre",
      immediate: true
    }
  );
  watch(
    visibleRef,
    (visible) => {
      var _a;
      if (visible) {
        return;
      }
      const selectedShortcutByValue = getSelectedShortcutByValue();
      if (((_a = selectedShortcut.value) == null ? void 0 : _a.key) !== (selectedShortcutByValue == null ? void 0 : selectedShortcutByValue.key)) {
        setSelectedShortcut(selectedShortcutByValue);
      }
    },
    { flush: "pre" }
  );
  return {
    shortcuts,
    selectedShortcut,
    showShortcutPanel,
    setSelectedShortcut
  };
}
function getResolvedPresetShortcutOption(dateConfig, presetShortcut, locales, showPanel, presetPanelRenderer) {
  if (presetShortcut === "custom") {
    return {
      key: "custom",
      label: locales.shortcuts["custom"],
      panelRenderer: presetPanelRenderer
    };
  }
  const value = getPresetRangeShortcutValue(dateConfig, presetShortcut);
  if (!value) {
    return;
  }
  return {
    key: presetShortcut,
    value,
    label: locales.shortcuts[presetShortcut],
    confirmOnSelect: !showPanel,
    selectedLabel: locales.shortcuts[presetShortcut],
    panelRenderer: showPanel ? presetPanelRenderer : void 0
  };
}
var RangeShortcuts = /* @__PURE__ */ defineComponent({
  props: {
    prefixCls: String,
    selectedShortcut: [String, Number, Symbol],
    shortcuts: Array,
    onChange: Function
  },
  setup(props) {
    return () => {
      var _a;
      const prefixCls = `${props.prefixCls}-shortcuts`;
      const handleShortcutClick = (evt, shortcut) => {
        var _a2, _b;
        (_a2 = shortcut.onClick) == null ? void 0 : _a2.call(shortcut, evt);
        (_b = props.onChange) == null ? void 0 : _b.call(props, shortcut);
      };
      return createVNode("ul", {
        "class": prefixCls
      }, [(_a = props.shortcuts) == null ? void 0 : _a.map((shortcut) => createVNode("li", {
        "key": shortcut.key,
        "class": [`${prefixCls}-item`, props.selectedShortcut === shortcut.key ? `${prefixCls}-item-selected` : void 0],
        "onClick": (evt) => handleShortcutClick(evt, shortcut)
      }, [createVNode("span", {
        "class": `${prefixCls}-item-label`
      }, [shortcut.label])]))]);
    };
  }
});
var RangeContent = /* @__PURE__ */ defineComponent({
  setup(_, {
    slots
  }) {
    const context = inject(dateRangePickerToken);
    const {
      props,
      mergedPrefixCls,
      shortcuts,
      selectedShortcut,
      showShortcutPanel,
      setSelectedShortcut,
      rangeControlContext: {
        buffer,
        bufferUpdated,
        handlePanelChange
      },
      handleChange,
      setOverlayOpened
    } = context;
    const handleShortcutChange = (shortcut) => {
      setSelectedShortcut(shortcut);
      if (!shortcut.value) {
        return;
      }
      if (shortcut.confirmOnSelect) {
        handleChange(extractShortcutValue(shortcut));
        setOverlayOpened(false);
      } else {
        handlePanelChange(extractShortcutValue(shortcut));
      }
    };
    const shortcutOk = () => {
      if (bufferUpdated.value) {
        handleChange(buffer.value);
      }
      setOverlayOpened(false);
    };
    const shorcutCancel = () => {
      setOverlayOpened(false);
    };
    return () => {
      var _a, _b, _c;
      if (slots.overlay) {
        return slots.overlay();
      }
      let children;
      const contentClasses = computed(() => {
        const prefixCls = `${mergedPrefixCls.value}-overlay-content`;
        return {
          [prefixCls]: true,
          [`${prefixCls}-with-shortcuts`]: !!shortcuts.value.length
        };
      });
      const shortcutPanelRenderContext = {
        slots,
        setBuffer: handlePanelChange,
        setValue: handleChange,
        ok: shortcutOk,
        cancel: shorcutCancel
      };
      if (shortcuts.value.length) {
        children = [createVNode(RangeShortcuts, {
          "class": {
            [`${mergedPrefixCls.value}-shortcuts-with-panel`]: showShortcutPanel.value || !!((_a = selectedShortcut.value) == null ? void 0 : _a.panelRenderer)
          },
          "prefixCls": mergedPrefixCls.value,
          "shortcuts": shortcuts.value,
          "selectedShortcut": (_b = selectedShortcut.value) == null ? void 0 : _b.key,
          "onChange": handleShortcutChange
        }, null), ((_c = selectedShortcut.value) == null ? void 0 : _c.panelRenderer) ? createVNode("div", {
          "class": `${mergedPrefixCls.value}-shortcuts-panel`
        }, [selectedShortcut.value.panelRenderer(shortcutPanelRenderContext)]) : showShortcutPanel.value ? createVNode("div", {
          "class": `${mergedPrefixCls.value}-shortcuts-panel`
        }, [createVNode(RangePickerOverlayInputs, null, slots), createVNode(DateRangePanel, null, null), createVNode(RangePickerOverlayFooter, null, slots)]) : void 0];
      } else {
        children = [createVNode(RangePickerOverlayInputs, null, slots), createVNode(DateRangePanel, null, null), createVNode(RangePickerOverlayFooter, null, slots)];
      }
      return props.overlayRender ? props.overlayRender(children) : createVNode("div", {
        "class": contentClasses.value
      }, [children]);
    };
  }
});
var RangeTrigger = /* @__PURE__ */ defineComponent({
  inheritAttrs: false,
  setup(_, {
    expose,
    slots
  }) {
    const context = inject(dateRangePickerToken);
    const {
      accessor,
      props,
      locale,
      rangeControlContext: {
        fromControl,
        toControl
      },
      mergedPrefixCls,
      formatRef,
      inputRef,
      inputEnableStatus,
      selectedShortcut,
      renderSeparator
    } = context;
    const triggerInputRef = ref();
    const triggerToInputRef = ref();
    const triggerRef = ref();
    const placeholders = computed(() => {
      var _a, _b, _c, _d;
      return [(_b = (_a = props.placeholder) == null ? void 0 : _a[0]) != null ? _b : locale.dateRangePicker[`${props.type}Placeholder`][0], (_d = (_c = props.placeholder) == null ? void 0 : _c[1]) != null ? _d : locale.dateRangePicker[`${props.type}Placeholder`][1]];
    });
    const inputSize = computed(() => Math.max(10, formatRef.value.length) + 2);
    const handleFromInput = (evt) => {
      fromControl.handleInput(evt);
      callEmit(props.onInput, true, evt);
    };
    const handleToInput = (evt) => {
      toControl.handleInput(evt);
      callEmit(props.onInput, false, evt);
    };
    const focus = () => {
      var _a;
      const inputEl = (inputEnableStatus.value.allowInput === "overlay" ? triggerInputRef : inputRef).value;
      if (inputEl) {
        inputEl.focus();
      } else {
        (_a = triggerRef.value) == null ? void 0 : _a.focus();
      }
    };
    expose({
      focus
    });
    watch(() => {
      var _a;
      return (_a = selectedShortcut.value) == null ? void 0 : _a.selectedLabel;
    }, (label) => {
      var _a;
      if (!label) {
        return;
      }
      if (document.activeElement === triggerInputRef.value || document.activeElement === triggerToInputRef.value || document.activeElement === inputRef.value) {
        (_a = triggerRef.value) == null ? void 0 : _a.focus();
      }
    });
    const renderSide = (isFrom) => {
      const prefixCls = mergedPrefixCls.value;
      const {
        inputValue
      } = isFrom ? fromControl : toControl;
      const placeholder = placeholders.value[isFrom ? 0 : 1];
      const handleInput = isFrom ? handleFromInput : handleToInput;
      return createVNode("input", {
        "ref": isFrom ? inputEnableStatus.value.allowInput === "overlay" ? triggerInputRef : inputRef : triggerToInputRef,
        "class": `${prefixCls}-input-inner`,
        "autocomplete": "off",
        "disabled": accessor.disabled,
        "placeholder": placeholder,
        "readonly": props.readonly || inputEnableStatus.value.enableInput === false,
        "size": inputSize.value,
        "value": inputValue.value,
        "onInput": handleInput
      }, null);
    };
    return () => {
      var _a, _b, _c, _d;
      return createVNode("div", {
        "ref": triggerRef,
        "class": `${mergedPrefixCls.value}-input`,
        "tabindex": -1
      }, [(_d = (_c = (_a = slots.triggerContent) == null ? void 0 : _a.call(slots, {
        selectedShortcut: selectedShortcut.value,
        inputValue: [fromControl.inputValue.value, toControl.inputValue.value],
        placeholder: placeholders.value,
        readonly: props.readonly || inputEnableStatus.value.enableInput === false,
        disabled: accessor.disabled,
        handleFromInput,
        handleToInput
      })) != null ? _c : (_b = selectedShortcut.value) == null ? void 0 : _b.selectedLabel) != null ? _d : [renderSide(true), createVNode("span", {
        "class": `${mergedPrefixCls.value}-input-separator`
      }, [renderSeparator()]), renderSide(false)]]);
    };
  }
});
var DateRangePicker = /* @__PURE__ */ defineComponent({
  name: "IxDateRangePicker",
  inheritAttrs: false,
  props: dateRangePickerProps,
  setup(props, {
    attrs,
    expose,
    slots
  }) {
    const common = useGlobalConfig("common");
    const locale = useGlobalConfig("locale");
    const {
      globalHashId,
      hashId,
      registerToken
    } = useThemeToken("datePicker");
    registerToken(getThemeTokens);
    const mergedPrefixCls = computed(() => `${common.prefixCls}-date-range-picker`);
    const config = useGlobalConfig("datePicker");
    const dateConfig = useDateConfig();
    const triggerRef = ref();
    const {
      elementRef: inputRef,
      focus,
      blur
    } = useFormElement();
    expose({
      focus,
      blur
    });
    const {
      overlayOpened,
      overlayVisible,
      onAfterLeave,
      setOverlayOpened
    } = useOverlayState(props);
    const inputEnableStatus = useInputEnableStatus(props, config);
    const formatContext = useFormat(props, config);
    const pickerStateContext = usePickerState(props, config, dateConfig, formatContext.formatRef);
    const {
      accessor,
      convertedValue,
      focused,
      handleFocus,
      handleBlur,
      handleChange
    } = pickerStateContext;
    const rangeControlContext = useRangeControl(dateConfig, formatContext, inputEnableStatus, toRef(accessor, "value"), toRef(props, "type"));
    const rangeShortcutsContext = useRangeShortcuts(computed(() => props.shortcuts), convertedValue, overlayOpened);
    const handleKeyDown = useRangeKeyboardEvents(rangeControlContext, overlayOpened, setOverlayOpened, handleChange);
    const renderSeparator = () => {
      var _a, _b, _c;
      return (_c = (_b = (_a = slots.separator) == null ? void 0 : _a.call(slots)) != null ? _b : props.separator) != null ? _c : locale.dateRangePicker.separator;
    };
    const context = {
      props,
      slots,
      common,
      locale,
      config,
      mergedPrefixCls,
      dateConfig,
      inputRef,
      inputEnableStatus,
      overlayOpened,
      overlayVisible,
      onAfterLeave,
      setOverlayOpened,
      rangeControlContext,
      renderSeparator,
      handleKeyDown,
      ...formatContext,
      ...pickerStateContext,
      ...rangeShortcutsContext,
      handleFocus,
      handleBlur
    };
    const {
      shortcuts,
      selectedShortcut,
      showShortcutPanel
    } = rangeShortcutsContext;
    const triggerProps = useTriggerProps(context);
    const dateRangePanelProps2 = useRangePickerPanelProps(context);
    const dateRangePanelSlots = {
      cell: slots.cell,
      separator: inputEnableStatus.value.enableOverlayDateInput ? () => createVNode("div", {
        "class": `${mergedPrefixCls.value}-overlay-separator`
      }, null) : void 0
    };
    provide(dateRangePickerToken, context);
    provide(dateRangePickerPanelPropsToken, dateRangePanelProps2);
    provide(dateRangePickerPanelSlotsToken, dateRangePanelSlots);
    watch(overlayOpened, (opened) => {
      var _a;
      if (opened) {
        setTimeout(() => {
          var _a2;
          if (!focused.value) {
            (_a2 = inputRef.value) == null ? void 0 : _a2.focus();
          }
        });
      } else {
        rangeControlContext.init(true);
        if (focused.value) {
          (_a = triggerRef.value) == null ? void 0 : _a.focus();
        }
      }
    });
    const renderTrigger = () => createVNode(RangeTrigger, {
      "ref": triggerRef
    }, slots);
    const renderContent = () => createVNode(RangeContent, null, slots);
    const overlayClass = computed(() => normalizeClass([`${mergedPrefixCls.value}-overlay`, globalHashId.value, hashId.value, props.overlayClassName]));
    return () => {
      var _a;
      return createVNode(IxControlTrigger, mergeProps(triggerProps.value, {
        "class": `${mergedPrefixCls.value} ${globalHashId.value} ${hashId.value}`,
        "overlayClassName": overlayClass.value,
        "overlayMatchWidth": !!shortcuts.value.length && !showShortcutPanel.value && !((_a = selectedShortcut.value) == null ? void 0 : _a.panelRenderer)
      }, attrs), {
        default: renderTrigger,
        overlay: renderContent,
        suffix: slots.suffix,
        clearIcon: slots.clearIcon
      });
    };
  }
});
const IxDatePicker = DatePicker;
const IxDatePanel = DatePanel;
const IxDateRangePicker = DateRangePicker;
const IxDateRangePanel = DateRangePanel;
export { IxDatePanel, IxDatePicker, PickerOverlayFooter as IxDatePickerOverlayFooter, PickerOverlayInputs as IxDatePickerOverlayInputs, IxDateRangePanel, IxDateRangePicker, RangePickerOverlayFooter as IxDateRangePickerOverlayFooter, RangePickerOverlayInputs as IxDateRangePickerOverlayInputs, extractShortcutValue, getThemeTokens as getDatePickerThemeTokens, getPresetRangeShortcutValue, RangeShortcuts as \u0275RangeShortcuts, useRangeShortcuts as \u0275UseRangeShortcuts };
