import { defineComponent, computed, provide, normalizeClass, createVNode, inject } from "vue";
import { \u0275Header as _Header } from "@idux/components/_private/header";
import { useGlobalConfig } from "@idux/components/config";
import { IxRow, IxCol } from "@idux/components/grid";
import { useThemeToken } from "@idux/components/theme";
import { isObject } from "lodash-es";
import { useSharedBreakpoints, BREAKPOINTS_KEYS } from "@idux/cdk/breakpoint";
import { Logger, convertCssPixel, convertNumber } from "@idux/cdk/utils";
import { convertStringVNode } from "@idux/components/utils";
const descToken = Symbol("descToken");
const descProps = {
  col: {
    type: [Number, String, Object],
    default: void 0
  },
  colonless: { type: Boolean, default: void 0 },
  header: { type: [String, Object] },
  labelAlign: String,
  labelWidth: { type: [String, Number], default: void 0 },
  layout: String,
  size: String
};
const descItemProps = {
  col: {
    type: [Number, String, Object],
    default: 1
  },
  colonless: { type: Boolean, default: void 0 },
  label: { type: [String, Number], default: void 0 },
  labelAlign: { type: String, default: void 0 },
  labelWidth: { type: [String, Number], default: void 0 }
};
function getDefaultThemeTokens(tokens) {
  const { colorTextInfo, heightSm, heightMd, heightLg, fontSizeSm } = tokens;
  return {
    itemHeightSm: heightSm,
    itemHeightMd: heightMd,
    itemHeightLg: heightLg,
    fontSizeSm,
    fontSizeMd: fontSizeSm,
    fontSizeLg: fontSizeSm,
    labelColor: colorTextInfo
  };
}
const getThemeTokens = (tokens, presetTheme) => {
  return presetTheme === "default" ? getDefaultThemeTokens(tokens) : getDefaultThemeTokens(tokens);
};
var Desc = /* @__PURE__ */ defineComponent({
  name: "IxDesc",
  props: descProps,
  setup(props, {
    slots
  }) {
    const common = useGlobalConfig("common");
    const {
      globalHashId,
      hashId,
      registerToken
    } = useThemeToken("desc");
    registerToken(getThemeTokens);
    const config = useGlobalConfig("desc");
    const mergedPrefixCls = computed(() => `${common.prefixCls}-desc`);
    const mergedCol = computed(() => {
      var _a;
      return (_a = props.col) != null ? _a : config.col;
    });
    const mergedColonless = computed(() => {
      var _a;
      return (_a = props.colonless) != null ? _a : config.colonless;
    });
    const mergedLabelAlign = computed(() => {
      var _a;
      return (_a = props.labelAlign) != null ? _a : config.labelAlign;
    });
    const mergedLabelWidth = computed(() => props.labelWidth);
    const mergedSize = computed(() => {
      var _a;
      return (_a = props.size) != null ? _a : config.size;
    });
    provide(descToken, {
      mergedPrefixCls,
      mergedCol,
      mergedColonless,
      mergedLabelAlign,
      mergedLabelWidth
    });
    const classes = computed(() => {
      const prefixCls = mergedPrefixCls.value;
      const {
        layout = config.layout
      } = props;
      return normalizeClass({
        [globalHashId.value]: !!globalHashId.value,
        [hashId.value]: !!hashId.value,
        [prefixCls]: true,
        [`${prefixCls}-${mergedSize.value}`]: true,
        [`${prefixCls}-vertical`]: layout === "vertical"
      });
    });
    const headerClasses = computed(() => `${mergedPrefixCls.value}-header`);
    return () => {
      return createVNode(IxRow, {
        "class": classes.value
      }, {
        default: () => [createVNode("div", {
          "class": headerClasses.value
        }, [createVNode(_Header, {
          "header": props.header,
          "size": mergedSize.value === "lg" ? "md" : "sm"
        }, {
          header: slots.header,
          closeIcon: slots.suffix
        })]), slots.default && slots.default()]
      });
    };
  }
});
var DescItem = /* @__PURE__ */ defineComponent({
  name: "IxDescItem",
  props: descItemProps,
  setup(props, {
    slots
  }) {
    const descContext = inject(descToken);
    if (!descContext) {
      process.env.NODE_ENV !== "production" && Logger.warn("components/desc", "The IxDescItem must be within IxDesc component");
      return;
    }
    const {
      mergedPrefixCls,
      mergedCol,
      mergedColonless,
      mergedLabelAlign,
      mergedLabelWidth
    } = descContext;
    const mergedColSpan = useMergedColSpan(props, mergedCol);
    const labelClasses = computed(() => {
      const prefixCls = `${mergedPrefixCls.value}-item-label`;
      const {
        colonless = mergedColonless.value,
        labelAlign = mergedLabelAlign.value
      } = props;
      return normalizeClass({
        [prefixCls]: true,
        [`${prefixCls}-colon`]: !colonless,
        [`${prefixCls}-start`]: labelAlign === "start"
      });
    });
    const labelStyle = computed(() => {
      var _a;
      const labelWidth = convertCssPixel((_a = props.labelWidth) != null ? _a : mergedLabelWidth.value);
      return labelWidth ? `width: ${labelWidth}` : void 0;
    });
    return () => {
      const labelNode = convertStringVNode(slots, props, "label");
      return createVNode(IxCol, {
        "class": `${mergedPrefixCls.value}-item`,
        "span": mergedColSpan.value
      }, {
        default: () => [labelNode && createVNode("div", {
          "class": labelClasses.value
        }, [createVNode("label", {
          "style": labelStyle.value
        }, [labelNode])]), createVNode("div", {
          "class": `${mergedPrefixCls.value}-content`
        }, [createVNode("span", null, [slots.default && slots.default()])])]
      });
    };
  }
});
const defaultSelfCol = 1;
const defaultParentCol = 3;
const defaultRowSpan = 24;
function useMergedColSpan(props, mergedCol) {
  const breakpoints = useSharedBreakpoints();
  return computed(() => {
    let selfCol = props.col;
    let parentCol = mergedCol.value;
    const currBreakpoint = BREAKPOINTS_KEYS.find((key) => breakpoints[key]);
    selfCol = convertNumber(isObject(selfCol) ? selfCol[currBreakpoint] : selfCol, defaultSelfCol);
    parentCol = convertNumber(isObject(parentCol) ? parentCol[currBreakpoint] : parentCol, defaultParentCol);
    return defaultRowSpan / parentCol * selfCol;
  });
}
const IxDesc = Desc;
const IxDescItem = DescItem;
export { IxDesc, IxDescItem, getThemeTokens as getDescThemeTokens };
