import { defineComponent, provide, computed, normalizeClass, createVNode, inject, onBeforeUnmount, ref, watch, shallowRef, watchEffect, mergeProps, Transition } from "vue";
import { useControl as useControl$1, FORMS_CONTROL_TOKEN } from "@idux/cdk/forms";
import { useGlobalConfig } from "@idux/components/config";
import { useThemeToken } from "@idux/components/theme";
import { isFunction, isObject, isString, isNumber, isBoolean } from "lodash-es";
import { IxRow, IxCol } from "@idux/components/grid";
import { IxIcon } from "@idux/components/icon";
import { IxTooltip } from "@idux/components/tooltip";
import { useKey, hasOwnProperty } from "@idux/cdk/utils";
import { useSharedFocusMonitor } from "@idux/cdk/a11y";
const formToken = Symbol("formToken");
const FORM_TOKEN = Symbol("FORM_TOKEN");
const FORM_ITEM_TOKEN = Symbol("FORM_ITEM_TOKEN");
const colProp = [Number, String, Object];
const formProps = {
  control: {
    type: [String, Number, Object, Array],
    default: void 0
  },
  colonless: { type: Boolean, default: void 0 },
  controlCol: colProp,
  controlTooltipIcon: String,
  labelAlign: String,
  labelCol: colProp,
  labelTooltipIcon: String,
  layout: String,
  messageTooltip: { type: [Boolean, Object], default: false },
  size: String,
  onSubmit: { type: Function, default: (evt) => evt.preventDefault() }
};
const formItemProps = {
  colonless: { type: Boolean, default: void 0 },
  control: {
    type: [String, Number, Object, Array],
    default: void 0
  },
  controlCol: colProp,
  controlTooltip: String,
  controlTooltipIcon: String,
  description: String,
  label: [String, Number],
  labelAlign: String,
  labelCol: colProp,
  labelFor: [String, Number],
  labelTooltip: String,
  labelTooltipIcon: String,
  required: { type: Boolean, default: false },
  message: [String, Function, Object],
  messageTooltip: { type: [Boolean, Object], default: void 0 },
  status: String
};
const formWrapperProps = {
  control: {
    type: [String, Number, Object, Array],
    default: void 0
  }
};
function getDefaultThemeTokens(tokens) {
  const { colorTextInfo, colorErrorBg, colorErrorText, fontSizeSm, marginSize2Xs, marginSizeSm, paddingSizeSm } = tokens;
  return {
    labelColor: colorTextInfo,
    labelRequiredColor: colorErrorText,
    labelFontSize: fontSizeSm,
    labelColonMarginLeft: marginSize2Xs,
    labelColonMarginRight: marginSizeSm,
    messageTooltipBgColor: colorErrorBg,
    itemMarginBottom: marginSizeSm,
    verticalItemLabelPadding: `0 0 ${paddingSizeSm}px`,
    verticalItemLabelMargin: 0
  };
}
const getThemeTokens = (tokens) => getDefaultThemeTokens(tokens);
var Form = /* @__PURE__ */ defineComponent({
  name: "IxForm",
  props: formProps,
  setup(props, {
    slots
  }) {
    const control = useControl$1();
    const {
      globalHashId,
      hashId,
      registerToken
    } = useThemeToken("form");
    registerToken(getThemeTokens);
    provide(FORMS_CONTROL_TOKEN, control);
    const common = useGlobalConfig("common");
    const mergedPrefixCls = computed(() => `${common.prefixCls}-form`);
    const config = useGlobalConfig("form");
    const layout = computed(() => {
      var _a;
      return (_a = props.layout) != null ? _a : config.layout;
    });
    const size = computed(() => {
      var _a;
      return (_a = props.size) != null ? _a : config.size;
    });
    provide(formToken, {
      props,
      config
    });
    provide(FORM_TOKEN, {
      size
    });
    const classes = computed(() => {
      const prefixCls = mergedPrefixCls.value;
      return normalizeClass({
        [globalHashId.value]: !!globalHashId,
        [hashId.value]: !!hashId.value,
        [prefixCls]: true,
        [`${prefixCls}-${layout.value}`]: true,
        [`${prefixCls}-${size.value}`]: true
      });
    });
    return () => createVNode("form", {
      "class": classes.value,
      "onSubmit": props.onSubmit
    }, [slots.default && slots.default()]);
  }
});
function useFormItemRegister(control) {
  const context = inject(FORM_ITEM_TOKEN, null);
  if (context) {
    const key = useKey();
    const { registerControl, unregisterControl } = context;
    registerControl(key, control);
    onBeforeUnmount(() => unregisterControl(key));
  }
}
function useFormElement() {
  const elementRef = ref();
  const focus = (options) => {
    var _a;
    (_a = elementRef.value) == null ? void 0 : _a.focus(options);
  };
  const blur = () => {
    var _a;
    return (_a = elementRef.value) == null ? void 0 : _a.blur();
  };
  return { elementRef, focus, blur };
}
function useFormFocusMonitor(options) {
  const focusMonitor = useSharedFocusMonitor();
  const elementRef = ref();
  let watchStopHandle;
  watch(elementRef, (currElement, prevElement) => {
    if (watchStopHandle) {
      watchStopHandle();
    }
    if (prevElement) {
      focusMonitor.stopMonitoring(prevElement);
    }
    watchStopHandle = watch(focusMonitor.monitor(currElement, options.checkChildren), (evt) => {
      const { origin, event } = evt;
      if (event) {
        origin ? options.handleFocus(event) : options.handleBlur(event);
      }
    });
  });
  onBeforeUnmount(() => {
    if (watchStopHandle) {
      watchStopHandle();
    }
    focusMonitor.stopMonitoring(elementRef.value);
  });
  const focus = (options2) => focusMonitor.focusVia(elementRef.value, "program", options2);
  const blur = () => focusMonitor.blurVia(elementRef.value);
  return { elementRef, focus, blur };
}
function useFormSize(props, config) {
  const formContext = inject(FORM_TOKEN, null);
  return computed(() => {
    var _a, _b;
    return (_b = (_a = props.size) != null ? _a : formContext == null ? void 0 : formContext.size.value) != null ? _b : config.size;
  });
}
function useFormStatus(props, control) {
  return computed(() => {
    if (props.status) {
      return props.status;
    }
    const currControl = control.value;
    if (!currControl || currControl.disabled.value) {
      return void 0;
    }
    const { trigger, dirty, blurred, status, validated } = currControl;
    if (trigger === "change") {
      return dirty.value ? status.value : void 0;
    }
    if (trigger === "blur") {
      return blurred.value ? status.value : void 0;
    }
    return validated.value ? status.value : void 0;
  });
}
function useFormItem(props) {
  const control = useControl();
  const status = useFormStatus(props, control);
  const message = useMessage(props, control, status);
  return { status, message };
}
function useControl() {
  const firstChildControl = shallowRef();
  let firstChildKey;
  let firstChildWatchStop;
  const registerControl = (key, control2) => {
    if (!firstChildWatchStop) {
      firstChildKey = key;
      firstChildWatchStop = watchEffect(() => firstChildControl.value = control2.value);
    }
  };
  const unregisterControl = (key) => {
    if (key === firstChildKey) {
      if (firstChildWatchStop) {
        firstChildWatchStop();
        firstChildWatchStop = void 0;
      }
      firstChildControl.value = void 0;
    }
  };
  provide(FORM_ITEM_TOKEN, { registerControl, unregisterControl });
  const selfControl = useControl$1();
  const control = shallowRef();
  watch(
    [selfControl, firstChildControl],
    ([self, child]) => {
      const target = self != null ? self : child;
      if (control.value !== target) {
        control.value = target;
      }
    },
    { immediate: true }
  );
  return control;
}
function useMessage(props, control, status) {
  const locale = useGlobalConfig("locale");
  return computed(() => {
    const currStatus = status.value;
    if (!currStatus) {
      return void 0;
    }
    const currMessage = props.message;
    if (currMessage) {
      if (isFunction(currMessage)) {
        return currMessage(control.value);
      }
      if (isObject(currMessage)) {
        return currMessage[currStatus];
      }
      return currStatus === "invalid" ? currMessage : void 0;
    }
    return getMessageByControl(control, locale);
  });
}
function getMessageByControl(control, locale) {
  const currControl = control.value;
  if (!currControl) {
    return void 0;
  }
  const error = currControl.errors.value;
  for (const key in error) {
    if (!hasOwnProperty(error, key)) {
      continue;
    }
    const { message, ...rest } = error[key];
    if (message) {
      if (isString(message)) {
        return message;
      }
      if (isFunction(message)) {
        return message(rest, currControl);
      }
      const i18nMessage = message[locale.type];
      if (i18nMessage) {
        return isFunction(i18nMessage) ? i18nMessage(rest, currControl) : i18nMessage;
      }
    }
  }
  return void 0;
}
var FormItem = /* @__PURE__ */ defineComponent({
  name: "IxFormItem",
  props: formItemProps,
  setup(props, {
    slots
  }) {
    const common = useGlobalConfig("common");
    const {
      globalHashId,
      hashId,
      registerToken
    } = useThemeToken("form");
    registerToken(getThemeTokens);
    const mergedPrefixCls = computed(() => `${common.prefixCls}-form-item`);
    const {
      props: formProps2,
      config
    } = inject(formToken, null) || {};
    const colonless = computed(() => {
      var _a, _b;
      return (_b = (_a = props.colonless) != null ? _a : formProps2 == null ? void 0 : formProps2.colonless) != null ? _b : config == null ? void 0 : config.colonless;
    });
    const labelAlign = computed(() => {
      var _a, _b;
      return (_b = (_a = props.labelAlign) != null ? _a : formProps2 == null ? void 0 : formProps2.labelAlign) != null ? _b : config == null ? void 0 : config.labelAlign;
    });
    const labelColConfig = computed(() => {
      var _a;
      return normalizeColConfig((_a = props.labelCol) != null ? _a : formProps2 == null ? void 0 : formProps2.labelCol);
    });
    const labelTooltipIcon = computed(() => {
      var _a, _b;
      return (_b = (_a = props.labelTooltipIcon) != null ? _a : formProps2 == null ? void 0 : formProps2.labelTooltipIcon) != null ? _b : config == null ? void 0 : config.labelTooltipIcon;
    });
    const controlColConfig = computed(() => {
      var _a;
      return normalizeColConfig((_a = props.controlCol) != null ? _a : formProps2 == null ? void 0 : formProps2.controlCol);
    });
    const controlTooltipIcon = computed(() => {
      var _a, _b;
      return (_b = (_a = props.controlTooltipIcon) != null ? _a : formProps2 == null ? void 0 : formProps2.controlTooltipIcon) != null ? _b : config == null ? void 0 : config.controlTooltipIcon;
    });
    const mergedMessageTooltip = computed(() => {
      var _a;
      return (_a = props.messageTooltip) != null ? _a : formProps2 == null ? void 0 : formProps2.messageTooltip;
    });
    const {
      status,
      message
    } = useFormItem(props);
    const classes = computed(() => {
      const prefixCls = mergedPrefixCls.value;
      const currStatus = status.value;
      return normalizeClass({
        [globalHashId.value]: !!globalHashId.value,
        [hashId.value]: !!hashId.value,
        [prefixCls]: true,
        [`${prefixCls}-${currStatus}`]: !!currStatus
      });
    });
    const labelClasses = computed(() => {
      const prefixCls = mergedPrefixCls.value;
      return normalizeClass({
        [`${prefixCls}-label`]: true,
        [`${prefixCls}-label-colon`]: !colonless.value,
        [`${prefixCls}-label-required`]: props.required,
        [`${prefixCls}-label-start`]: labelAlign.value === "start"
      });
    });
    return () => {
      const prefixCls = mergedPrefixCls.value;
      return createVNode(IxRow, {
        "class": classes.value
      }, {
        default: () => [renderLabel(props, slots, labelClasses, labelColConfig, labelTooltipIcon, prefixCls, hashId.value), renderControl(props, slots, common, controlColConfig, controlTooltipIcon, status, message, mergedMessageTooltip, prefixCls, hashId.value)]
      });
    };
  }
});
function normalizeColConfig(col) {
  return isNumber(col) || isString(col) ? {
    span: col
  } : col;
}
function renderLabel(props, slots, classes, labelColConfig, labelTooltipIcon, prefixCls, hashId) {
  const {
    label,
    labelFor,
    labelTooltip
  } = props;
  const {
    label: labelSlot,
    labelTooltip: labelTooltipSlot
  } = slots;
  if (!(label || labelSlot)) {
    return void 0;
  }
  const tooltipNode = renderTooltip(labelTooltipSlot, labelTooltip, labelTooltipIcon.value, hashId);
  return createVNode(IxCol, mergeProps({
    "class": classes.value
  }, labelColConfig.value), {
    default: () => [createVNode("label", {
      "for": labelFor
    }, [labelSlot ? labelSlot() : label, tooltipNode && createVNode("span", {
      "class": `${prefixCls}-label-tooltip`
    }, [tooltipNode])])]
  });
}
function renderControl(props, slots, common, controlColConfig, controlTooltipIcon, status, message, mergedMessageTooltip, prefixCls, hashId) {
  const {
    controlTooltip,
    description
  } = props;
  const {
    controlTooltip: controlTooltipSlot,
    description: descriptionSlot,
    message: messageSlot
  } = slots;
  const tooltipNode = renderTooltip(controlTooltipSlot, controlTooltip, controlTooltipIcon.value, hashId);
  const inputNode = createVNode("div", {
    "class": `${prefixCls}-control-input`
  }, [createVNode("div", {
    "class": `${prefixCls}-control-input-content`
  }, [slots.default && slots.default()]), tooltipNode && createVNode("span", {
    "class": `${prefixCls}-control-tooltip`
  }, [tooltipNode])]);
  const tooltipProps = convertTooltipProps(mergedMessageTooltip.value, message.value, prefixCls, hashId);
  const descriptionNode = descriptionSlot ? descriptionSlot() : description;
  const children = [];
  if (tooltipProps) {
    const tooltipSlots = {
      default: () => inputNode,
      title: messageSlot
    };
    children.push(createVNode(IxTooltip, tooltipProps, tooltipSlots));
  } else {
    children.push(inputNode);
    const messageNode = messageSlot ? messageSlot() : message.value;
    if (!descriptionNode || messageNode) {
      children.push(createVNode("div", {
        "class": `${prefixCls}-message`
      }, [createVNode(Transition, {
        "name": `${common.prefixCls}-fade-down`
      }, {
        default: () => [messageNode && createVNode("div", {
          "class": `${prefixCls}-message-${status.value}`
        }, [messageNode])]
      })]));
    }
  }
  if (descriptionNode) {
    children.push(createVNode("div", {
      "class": `${prefixCls}-description`
    }, [descriptionNode]));
  }
  return createVNode(IxCol, mergeProps({
    "class": `${prefixCls}-control`
  }, controlColConfig.value), {
    default: () => [children]
  });
}
function convertTooltipProps(messageTooltip, title, prefixCls, hashId) {
  if (!messageTooltip) {
    return void 0;
  }
  const tooltipProps = {
    class: [`${prefixCls}-message-tooltip`, hashId],
    offset: [0, 6],
    placement: "bottomStart",
    title
  };
  return isBoolean(messageTooltip) ? tooltipProps : {
    ...tooltipProps,
    ...messageTooltip
  };
}
function renderTooltip(slot, tooltip, iconName, hashId) {
  if (slot) {
    return slot();
  }
  return tooltip && createVNode(IxTooltip, {
    "class": hashId,
    "title": tooltip
  }, {
    default: () => [createVNode(IxIcon, {
      "name": iconName
    }, null)]
  });
}
var FormWrapper = /* @__PURE__ */ defineComponent({
  name: "IxFormWrapper",
  props: formWrapperProps,
  setup(_, {
    slots
  }) {
    const control = useControl$1();
    provide(FORMS_CONTROL_TOKEN, control);
    return () => {
      var _a;
      return (_a = slots.default) == null ? void 0 : _a.call(slots);
    };
  }
});
const IxForm = Form;
const IxFormItem = FormItem;
const IxFormWrapper = FormWrapper;
export { FORM_ITEM_TOKEN, FORM_TOKEN, IxForm, IxFormItem, IxFormWrapper, getThemeTokens as getFormThemeTokens, useFormElement, useFormFocusMonitor, useFormItemRegister, useFormSize, useFormStatus };
