import { defineComponent, computed, inject, createVNode, normalizeClass, provide } from "vue";
import { isUndefined, isNumber, isString, isArray, isObject } from "lodash-es";
import { BREAKPOINTS_KEYS, useSharedBreakpoints } from "@idux/cdk/breakpoint";
import { isNumeric, convertNumber } from "@idux/cdk/utils";
import { useGlobalConfig } from "@idux/components/config";
import { useThemeToken } from "@idux/components/theme";
const rowToken = Symbol("rowToken");
const rowProps = {
  align: String,
  justify: String,
  gutter: [Number, String, Array, Object],
  wrap: {
    type: Boolean,
    default: void 0
  }
};
const singleProp = [Number, String];
const breakpointConfig = [Number, String, Object];
const colProps = {
  flex: singleProp,
  span: singleProp,
  order: singleProp,
  offset: singleProp,
  push: singleProp,
  pull: singleProp,
  xs: breakpointConfig,
  sm: breakpointConfig,
  md: breakpointConfig,
  lg: breakpointConfig,
  xl: breakpointConfig
};
var Col = /* @__PURE__ */ defineComponent({
  name: "IxCol",
  props: colProps,
  setup(props, {
    slots
  }) {
    const common = useGlobalConfig("common");
    const {
      globalHashId
    } = useThemeToken();
    const mergedPrefixCls = computed(() => `${common.prefixCls}-col`);
    const {
      mergedGutters
    } = inject(rowToken);
    const classes = computed(() => generateAllCls(props, mergedPrefixCls.value, globalHashId));
    const style = computed(() => {
      const style2 = {};
      const [verticalGutter, horizontalGutter] = mergedGutters.value;
      if (verticalGutter > 0) {
        style2.paddingTop = `${verticalGutter / 2}px`;
        style2.paddingBottom = style2.paddingTop;
      }
      if (horizontalGutter > 0) {
        style2.paddingLeft = `${horizontalGutter / 2}px`;
        style2.paddingRight = style2.paddingLeft;
      }
      if (props.flex) {
        style2.flex = parseFlex(props.flex);
      }
      return style2;
    });
    return () => createVNode("div", {
      "class": classes.value,
      "style": style.value
    }, [slots.default && slots.default()]);
  }
});
const attrKeys = ["span", "order", "offset", "push", "pull"];
function generateAllCls(props, prefixCLs, globalHashId) {
  const clsMap = /* @__PURE__ */ new Map([[prefixCLs, true]]);
  const generateSizeCls = (sizeConfig, size) => {
    attrKeys.forEach((attrKey) => {
      const attr = sizeConfig[attrKey];
      const cls = `${prefixCLs}${size ? `-${size}` : ""}-${attrKey}-${attr}`;
      clsMap.set(cls, !isUndefined(attr));
    });
  };
  generateSizeCls(props);
  BREAKPOINTS_KEYS.forEach((size) => {
    const sizeValue = props[size];
    if (!isUndefined(sizeValue)) {
      generateSizeCls(normalizeSizeProps(sizeValue), size);
    }
  });
  const allCls = {
    [globalHashId.value]: !!globalHashId.value
  };
  clsMap.forEach((value, key) => {
    allCls[key] = value;
  });
  return normalizeClass(allCls);
}
function normalizeSizeProps(rawProps) {
  return isNumber(rawProps) || isString(rawProps) ? {
    span: rawProps
  } : rawProps;
}
function parseFlex(flex) {
  if (isNumeric(flex)) {
    return `${flex} ${flex} auto`;
  }
  if (/^\d+(\.\d+)?(px|em|rem|%)$/.test(flex)) {
    return `0 0 ${flex}`;
  }
  return flex;
}
var Row = /* @__PURE__ */ defineComponent({
  name: "IxRow",
  props: rowProps,
  setup(props, {
    slots
  }) {
    const common = useGlobalConfig("common");
    const {
      globalHashId
    } = useThemeToken();
    const mergedPrefixCls = computed(() => `${common.prefixCls}-row`);
    const config = useGlobalConfig("row");
    const breakpoints = useSharedBreakpoints();
    const mergedGutters = useGutters(props, breakpoints);
    provide(rowToken, {
      mergedGutters
    });
    const classes = computed(() => {
      const {
        align,
        justify,
        wrap = config.wrap
      } = props;
      const prefixCls = mergedPrefixCls.value;
      return normalizeClass({
        [globalHashId.value]: !!globalHashId.value,
        [prefixCls]: true,
        [`${prefixCls}-align-${align}`]: align,
        [`${prefixCls}-justify-${justify}`]: justify,
        [`${prefixCls}-nowrap`]: !wrap
      });
    });
    const style = computed(() => {
      const [verticalGutter, horizontalGutter] = mergedGutters.value;
      const style2 = {};
      if (verticalGutter > 0) {
        style2.marginTop = `${verticalGutter / -2}px`;
        style2.marginBottom = style2.marginTop;
      }
      if (horizontalGutter > 0) {
        style2.marginLeft = `${horizontalGutter / -2}px`;
        style2.marginRight = style2.marginLeft;
      }
      return style2;
    });
    return () => createVNode("div", {
      "class": classes.value,
      "style": style.value
    }, [slots.default && slots.default()]);
  }
});
function useGutters(props, breakpoints) {
  return computed(() => {
    const {
      gutter
    } = props;
    const results = [0, 0];
    const gutters = isArray(gutter) ? gutter : [gutter, gutter];
    gutters.forEach((item, index) => {
      if (isObject(item)) {
        BREAKPOINTS_KEYS.some((key) => {
          const currGutter = item[key];
          const isActive = currGutter != null && breakpoints[key];
          if (isActive) {
            results[index] = convertNumber(currGutter);
          }
          return isActive;
        });
      } else {
        results[index] = convertNumber(item);
      }
    });
    return results;
  });
}
const IxRow = Row;
const IxCol = Col;
export { IxCol, IxRow };
