import { defineComponent, computed, normalizeClass, createVNode, mergeProps } from "vue";
import { isString } from "lodash-es";
import { callEmit } from "@idux/cdk/utils";
import { IxAvatar } from "@idux/components/avatar";
import { useGlobalConfig } from "@idux/components/config";
import { useThemeToken } from "@idux/components/theme";
import { convertIconVNode, convertStringVNode } from "@idux/components/utils";
const headerProps = {
  avatar: { type: [String, Object], default: void 0 },
  description: { type: String, default: void 0 },
  disabled: { type: Boolean, default: false },
  prefix: {
    type: [String, Object, Function],
    default: void 0
  },
  size: { type: String, default: "md" },
  showBar: { type: Boolean, default: false },
  subTitle: { type: String, default: void 0 },
  suffix: {
    type: [String, Object, Function],
    default: void 0
  },
  title: { type: String, default: void 0 },
  onPrefixClick: [Function, Array],
  onSuffixClick: [Function, Array]
};
function getDefaultThemeTokens(tokens) {
  const {
    colorPrimary,
    colorPrimaryText,
    colorIcon,
    colorText,
    colorTextDisabled,
    colorIconHover,
    colorIconActive,
    colorIconDisabled,
    heightLg,
    heightXl,
    height2xl,
    fontSizeMd
  } = tokens;
  return {
    heightSm: heightLg,
    heightMd: heightXl,
    heightLg: heightXl,
    heightXl: height2xl,
    barWidth: 2,
    barBgColor: colorPrimary,
    barBorderRadius: 0,
    barMarginRight: 6,
    prefixColor: colorText,
    prefixColorHover: colorPrimaryText,
    prefixColorActive: colorPrimaryText,
    prefixColorDisabled: colorTextDisabled,
    suffixColor: colorIcon,
    suffixColorHover: colorIconHover,
    suffixColorActive: colorIconActive,
    suffixColorDisabled: colorIconDisabled,
    subTitleFontSize: fontSizeMd
  };
}
const getThemeTokens = (tokens, presetTheme) => {
  return presetTheme === "default" ? getDefaultThemeTokens(tokens) : getDefaultThemeTokens(tokens);
};
var Header = /* @__PURE__ */ defineComponent({
  name: "IxHeader",
  props: headerProps,
  setup(props, {
    slots
  }) {
    const common = useGlobalConfig("common");
    const {
      globalHashId,
      hashId,
      registerToken
    } = useThemeToken("header");
    registerToken(getThemeTokens);
    const mergedPrefixCls = computed(() => `${common.prefixCls}-header`);
    const classes = computed(() => {
      const prefixCls = mergedPrefixCls.value;
      return normalizeClass({
        [globalHashId.value]: !!globalHashId.value,
        [hashId.value]: !!hashId.value,
        [prefixCls]: true,
        [`${prefixCls}-${props.size}`]: true,
        [`${prefixCls}-disabled`]: props.disabled,
        [`${prefixCls}-with-bar`]: props.showBar,
        [`${prefixCls}-with-description`]: props.description || slots.description
      });
    });
    const onPrefixClick = (evt) => !props.disabled && callEmit(props.onPrefixClick, evt);
    const onSuffixClick = (evt) => !props.disabled && callEmit(props.onSuffixClick, evt);
    return () => {
      const prefixCls = mergedPrefixCls.value;
      const prefixIconNode = convertIconVNode(slots, props, "prefix");
      const suffixIconNode = convertIconVNode(slots, props, "suffix");
      const titleNode = convertStringVNode(slots.default, props.title);
      const subTitleNode = convertStringVNode(slots, props, "subTitle");
      const descriptionNode = convertStringVNode(slots, props, "description");
      return createVNode("div", {
        "class": classes.value
      }, [prefixIconNode && createVNode("span", {
        "class": `${prefixCls}-prefix`,
        "onClick": onPrefixClick
      }, [prefixIconNode]), renderAvatar(slots.avatar, props.avatar, props.size), createVNode("div", {
        "class": `${prefixCls}-content`
      }, [createVNode("div", {
        "class": `${prefixCls}-title-wrapper`
      }, [titleNode && createVNode("span", {
        "class": `${prefixCls}-title`
      }, [titleNode]), subTitleNode && createVNode("span", {
        "class": `${prefixCls}-sub-title`
      }, [subTitleNode])]), descriptionNode && createVNode("div", {
        "class": `${prefixCls}-description`
      }, [descriptionNode])]), suffixIconNode && createVNode("span", {
        "class": `${prefixCls}-suffix`,
        "onClick": onSuffixClick
      }, [suffixIconNode])]);
    };
  }
});
const renderAvatar = (slot, avatar, size) => {
  if (slot) {
    return slot();
  }
  if (!avatar) {
    return null;
  }
  const avatarProps = isString(avatar) ? {
    icon: avatar
  } : avatar;
  return createVNode(IxAvatar, mergeProps({
    "size": size
  }, avatarProps), null);
};
const IxHeader = Header;
export { IxHeader, getThemeTokens as getHeaderThemeTokens };
