import { computed, ref, watchEffect, normalizeClass, createVNode, defineComponent, toRef, onMounted, onBeforeUnmount, watch, Transition, mergeProps } from "vue";
import { useControlledProp, useKey, callEmit } from "@idux/cdk/utils";
import { useGlobalConfig } from "@idux/components/config";
import { IxIcon } from "@idux/components/icon";
import { useThemeToken } from "@idux/components/theme";
import { CdkDndMovable } from "@idux/cdk/dnd";
import { isFirefox } from "@idux/cdk/platform";
import { CdkPortal } from "@idux/cdk/portal";
import { usePortalTarget, useZIndex } from "@idux/components/utils";
import { debounce } from "lodash-es";
const debounceTime = 10;
const defaultScale = 1;
const defaultRotate = 0;
const defaultScaleStep = 0.2;
const defaultRotateStep = 90;
function useOprList(goHandle, rotateHandle, scaleHandle, setVisible, scaleDisabled, switchDisabled, switchVisible) {
  const goNext = () => goHandle("next");
  const goPrevious = () => goHandle("previous");
  const rotateLeft = () => rotateHandle("left");
  const rotateRight = () => rotateHandle("right");
  const zoomOut = () => scaleHandle("out");
  const zoomIn = () => scaleHandle("in");
  const close = () => setVisible(false);
  return computed(() => [
    {
      key: "goPrevious",
      icon: "left",
      opr: goPrevious,
      disabled: switchDisabled.value.previous,
      visible: switchVisible.value
    },
    { key: "goNext", icon: "right", opr: goNext, disabled: switchDisabled.value.next, visible: switchVisible.value },
    { key: "rotateLeft", icon: "rotate-left", opr: rotateLeft, visible: true },
    { key: "rotateRight", icon: "rotate-right", opr: rotateRight, visible: true },
    { key: "zoomOut", icon: "zoom-out", opr: zoomOut, disabled: scaleDisabled.value.out, visible: true },
    { key: "zoomIn", icon: "zoom-in", opr: zoomIn, disabled: scaleDisabled.value.in, visible: true },
    { key: "close", icon: "close", opr: close, visible: true }
  ]);
}
function useImgStyleOpr(zoom) {
  const scale = ref(defaultScale);
  const rotate = ref(defaultRotate);
  const rotateFactor = {
    left: -1,
    right: 1
  };
  const scaleFactor = {
    in: 1,
    out: -1
  };
  const initScale = computed(() => {
    const [zoomOut, zoomIn] = zoom.value;
    if (zoomOut > defaultScale) {
      return zoomOut;
    }
    if (zoomIn < defaultScale) {
      return zoomIn;
    }
    return defaultScale;
  });
  watchEffect(() => scale.value = initScale.value);
  const scaleDisabled = computed(() => {
    const [zoomOut, zoomIn] = zoom.value;
    return {
      in: scale.value >= zoomIn,
      out: scale.value <= zoomOut
    };
  });
  const transformStyle = computed(() => ({ transform: `scale(${scale.value}) rotate(${rotate.value}deg)` }));
  const rotateHandle = debounce((direction = "left", rotateStep = defaultRotateStep) => {
    rotate.value = rotate.value + rotateStep * rotateFactor[direction];
  }, debounceTime);
  const scaleHandle = debounce((direction, scaleStep = defaultScaleStep) => {
    if (scaleDisabled.value[direction]) {
      return;
    }
    scale.value = scale.value + scaleStep * scaleFactor[direction];
  }, debounceTime);
  const resetTransform = () => {
    scale.value = initScale.value;
    rotate.value = defaultRotate;
  };
  return {
    transformStyle,
    scaleDisabled,
    rotateHandle,
    scaleHandle,
    resetTransform
  };
}
function useImgSwitch(props, loop) {
  const [activeIndex, setIndex] = useControlledProp(props, "activeIndex", 0);
  const switchDisabled = computed(() => ({
    previous: !loop.value && activeIndex.value === 0,
    next: !loop.value && activeIndex.value === props.images.length - 1
  }));
  const switchVisible = computed(() => props.images.length > 1);
  const goHandle = debounce((direction = "next") => {
    if (direction === "next") {
      if (switchDisabled.value.next || switchDisabled.value.previous) {
        return;
      }
      setIndex(activeIndex.value >= props.images.length - 1 ? 0 : activeIndex.value + 1);
      return;
    }
    setIndex(activeIndex.value <= 0 ? props.images.length - 1 : activeIndex.value - 1);
  }, debounceTime);
  return {
    activeIndex,
    switchDisabled,
    switchVisible,
    goHandle
  };
}
const OprIcon = (props) => {
  const {
    disabled,
    icon,
    prefixCls,
    opr
  } = props;
  const key = useKey();
  const classes = computed(() => normalizeClass({
    [`${prefixCls}-opr-item`]: true,
    [`${prefixCls}-opr-item-disabled`]: disabled
  }));
  return createVNode(IxIcon, {
    "class": classes.value,
    "name": icon,
    "onClick": opr,
    "key": key
  }, null);
};
const imageViewerProps = {
  visible: {
    type: Boolean,
    default: void 0
  },
  activeIndex: Number,
  container: {
    type: [String, HTMLElement, Function],
    default: void 0
  },
  draggable: { type: Boolean, default: void 0 },
  images: {
    type: Array,
    default: () => []
  },
  zoom: {
    type: Array,
    validator: (val) => val.length === 2
  },
  loop: {
    type: Boolean,
    default: void 0
  },
  maskClosable: {
    type: Boolean,
    default: void 0
  },
  zIndex: Number,
  "onUpdate:visible": [Function, Array],
  "onUpdate:activeIndex": [Function, Array]
};
const imageProps = {
  src: {
    type: String,
    required: true
  },
  preview: {
    type: Boolean,
    default: void 0
  },
  imageViewer: Object,
  onLoad: [Function, Array],
  onError: [Function, Array]
};
function getDefaultThemeTokens(tokens) {
  const { colorWhite, fontSizeXl } = tokens;
  return {
    minWidth: 96,
    minHeight: 96,
    objectFit: "contain",
    previewBgColor: `rgba(0, 0, 0, 0.5)`,
    previewIconColor: colorWhite,
    previewIconSize: fontSizeXl,
    viewerBgColor: `rgba(0, 0, 0, 0.45)`,
    viewerOprColor: colorWhite,
    viewerOprColorDisabled: `rgba(255, 255, 255, 0.35)`,
    viewerOprHeight: 48,
    viewerOprMarginBottom: 48,
    viewerOprFontSize: fontSizeXl,
    viewerOprBgColor: `rgba(0, 0, 0, 0.1)`,
    viewerOprItemMargin: `0 24px`
  };
}
const getThemeTokens = (tokens, presetTheme) => {
  return presetTheme === "default" ? getDefaultThemeTokens(tokens) : getDefaultThemeTokens(tokens);
};
const mousewheelEventName = isFirefox ? "DOMMouseScroll" : "mousewheel";
var ImageViewer = /* @__PURE__ */ defineComponent({
  name: "IxImageViewer",
  props: imageViewerProps,
  setup(props) {
    const common = useGlobalConfig("common");
    const {
      globalHashId,
      hashId,
      registerToken
    } = useThemeToken("image");
    registerToken(getThemeTokens);
    const config = useGlobalConfig("imageViewer");
    const mergedPrefixCls = computed(() => `${common.prefixCls}-image-viewer`);
    const mergedPortalTarget = usePortalTarget(props, config, common, mergedPrefixCls);
    const mergedDraggable = computed(() => {
      var _a;
      return (_a = props.draggable) != null ? _a : config.draggable;
    });
    const [visible, setVisible] = useControlledProp(props, "visible", false);
    const currentZIndex = useZIndex(toRef(props, "zIndex"), toRef(common, "overlayZIndex"), visible);
    const zoom = computed(() => {
      var _a;
      return (_a = props.zoom) != null ? _a : config.zoom;
    });
    const loop = computed(() => {
      var _a;
      return (_a = props.loop) != null ? _a : config.loop;
    });
    const maskClosable = computed(() => {
      var _a;
      return (_a = props.maskClosable) != null ? _a : config.maskClosable;
    });
    const {
      transformStyle,
      scaleDisabled,
      rotateHandle,
      scaleHandle,
      resetTransform
    } = useImgStyleOpr(zoom);
    const {
      activeIndex,
      switchDisabled,
      switchVisible,
      goHandle
    } = useImgSwitch(props, loop);
    const onClickLayer = () => maskClosable.value && setVisible(false);
    const oprList = useOprList(goHandle, rotateHandle, scaleHandle, setVisible, scaleDisabled, switchDisabled, switchVisible);
    const {
      onWheelScroll,
      onKeydown
    } = getImageEvent(visible, setVisible, scaleHandle, goHandle);
    onMounted(() => {
      window.addEventListener(mousewheelEventName, onWheelScroll, {
        passive: false,
        capture: false
      });
      window.addEventListener("keydown", onKeydown, false);
    });
    onBeforeUnmount(() => {
      window.removeEventListener(mousewheelEventName, onWheelScroll);
      window.removeEventListener("keydown", onKeydown);
    });
    watch([visible, activeIndex], ([visible$$]) => {
      visible$$ && resetTransform();
    });
    const style = computed(() => `z-index: ${currentZIndex.value}`);
    const renderImg = () => {
      const prefixCls = mergedPrefixCls.value;
      const curImgSrc = props.images[activeIndex.value];
      return createVNode("img", {
        "draggable": "false",
        "onClick": (evt) => evt.stopImmediatePropagation(),
        "class": `${prefixCls}-preview-img`,
        "src": curImgSrc,
        "style": transformStyle.value
      }, null);
    };
    return () => {
      const prefixCls = mergedPrefixCls.value;
      return createVNode(CdkPortal, {
        "target": mergedPortalTarget.value,
        "load": visible.value
      }, {
        default: () => [createVNode(Transition, {
          "name": `${common.prefixCls}-zoom`,
          "appear": true
        }, {
          default: () => [visible.value && createVNode("div", {
            "class": [prefixCls, globalHashId.value, hashId.value],
            "style": style.value
          }, [createVNode("div", {
            "class": `${prefixCls}-opr`
          }, [oprList.value.filter((item) => item.visible).map((item) => {
            const oprIconProps = {
              ...item,
              prefixCls
            };
            return createVNode(OprIcon, oprIconProps, null);
          })]), createVNode("div", {
            "class": `${prefixCls}-preview`,
            "onClick": onClickLayer
          }, [mergedDraggable.value ? createVNode(CdkDndMovable, {
            "mode": "immediate"
          }, {
            default: () => [renderImg()]
          }) : renderImg()])])]
        })]
      });
    };
  }
});
function getImageEvent(visible, setVisible, scaleHandle, goHandle) {
  const onWheelScroll = (e) => {
    var _a;
    if (!visible.value) {
      return;
    }
    const event = e;
    event.preventDefault();
    const delta = (_a = event.wheelDelta) != null ? _a : -event.detail;
    if (delta > 0) {
      scaleHandle("in", 0.2);
    } else {
      scaleHandle("out", 0.2);
    }
  };
  const keyHandle = {
    ArrowUp: () => scaleHandle("in", 0.2),
    ArrowDown: () => scaleHandle("out", 0.2),
    ArrowLeft: () => goHandle("previous"),
    ArrowRight: () => goHandle("next"),
    Escape: () => setVisible(false)
  };
  const onKeydown = (e) => {
    if (!visible.value) {
      return;
    }
    e.preventDefault();
    if (e.code in keyHandle) {
      keyHandle[e.code]();
    }
  };
  return {
    onWheelScroll,
    onKeydown
  };
}
var Image = /* @__PURE__ */ defineComponent({
  name: "IxImage",
  inheritAttrs: false,
  props: imageProps,
  setup(props, {
    attrs,
    slots
  }) {
    const {
      class: className,
      style,
      ...rest
    } = attrs;
    const common = useGlobalConfig("common");
    const {
      globalHashId,
      hashId,
      registerToken
    } = useThemeToken("image");
    registerToken(getThemeTokens);
    const mergedPrefixCls = computed(() => `${common.prefixCls}-image`);
    const config = useGlobalConfig("image");
    const preview = usePreview(props, config);
    const {
      status,
      setFailed,
      setLoaded
    } = useStatus(props);
    const [viewerVisible, setVisible] = useViewerVisible();
    const {
      outerClasses,
      overLayerClasses,
      imageClasses
    } = useClasses(mergedPrefixCls, globalHashId, hashId, className, status, preview);
    return () => {
      var _a;
      const imageViewerProps2 = {
        visible: viewerVisible.value,
        "onUpdate:visible": setVisible,
        images: [props.src],
        ...(_a = props.imageViewer) != null ? _a : {}
      };
      return createVNode("div", {
        "class": outerClasses.value,
        "style": style
      }, [createVNode("div", {
        "class": overLayerClasses.value
      }, [renderPreviewIcon(props, slots, mergedPrefixCls, status, preview, setVisible), renderPlaceholder(slots, mergedPrefixCls, status), renderFallback(slots, mergedPrefixCls, status), createVNode(ImageViewer, imageViewerProps2, null)]), createVNode("img", mergeProps(rest, {
        "src": props.src,
        "class": imageClasses.value,
        "onLoad": setLoaded,
        "onError": setFailed
      }), null)]);
    };
  }
});
function renderPreviewIcon(props, slots, mergedPrefixCls, status, preview, setVisible) {
  var _a, _b;
  return status.value === "loaded" && preview.value && createVNode("span", {
    "class": `${mergedPrefixCls.value}-preview-wrapper`,
    "onClick": () => setVisible(true)
  }, [(_b = (_a = slots.previewIcon) == null ? void 0 : _a.call(slots)) != null ? _b : createVNode(IxIcon, {
    "class": `${mergedPrefixCls.value}-preview-icon`,
    "name": "zoom-in"
  }, null)]);
}
function renderPlaceholder(slots, mergedPrefixCls, status) {
  var _a, _b;
  return status.value === "loading" && ((_b = (_a = slots.placeholder) == null ? void 0 : _a.call(slots)) != null ? _b : createVNode(IxIcon, {
    "class": `${mergedPrefixCls.value}-placeholder`,
    "name": "loading"
  }, null));
}
function renderFallback(slots, mergedPrefixCls, status) {
  var _a, _b;
  return status.value === "failed" && ((_b = (_a = slots.fallback) == null ? void 0 : _a.call(slots)) != null ? _b : createVNode(IxIcon, {
    "class": `${mergedPrefixCls.value}-fallback`,
    "name": "file-image"
  }, null));
}
function useViewerVisible() {
  const viewerVisible = ref(false);
  const setVisible = (visible) => {
    viewerVisible.value = visible;
  };
  return [viewerVisible, setVisible];
}
function useClasses(mergedPrefixCls, globalHashId, hashId, className, status, preview) {
  const outerClasses = computed(() => normalizeClass([mergedPrefixCls.value, globalHashId.value, hashId.value, className, `${mergedPrefixCls.value}-${status.value}`, {
    [`${mergedPrefixCls.value}-preview`]: preview.value
  }]));
  const overLayerClasses = computed(() => normalizeClass(`${mergedPrefixCls.value}-layer`));
  const imageClasses = computed(() => normalizeClass([`${mergedPrefixCls.value}-inner`, {
    [`${mergedPrefixCls.value}-inner-hidden`]: status.value !== "loaded"
  }]));
  return {
    outerClasses,
    overLayerClasses,
    imageClasses
  };
}
function usePreview(props, config) {
  return computed(() => {
    var _a;
    return (_a = props.preview) != null ? _a : config.preview;
  });
}
function useStatus(props) {
  const status = ref("loading");
  const setLoaded = (e) => {
    status.value = "loaded";
    callEmit(props.onLoad, e);
  };
  const setFailed = (e) => {
    status.value = "failed";
    callEmit(props.onError, e);
  };
  watch(() => props.src, () => {
    status.value = "loading";
  }, {
    immediate: true
  });
  return {
    status,
    setLoaded,
    setFailed
  };
}
const IxImage = Image;
const IxImageViewer = ImageViewer;
export { IxImage, IxImageViewer, getThemeTokens as getImageThemeTokens };
