import { ref, computed, watch, toRaw, nextTick, defineComponent, normalizeClass, onMounted, createVNode } from "vue";
import { \u0275Input as _Input } from "@idux/components/_private/input";
import { useGlobalConfig } from "@idux/components/config";
import { useFormItemRegister, useFormSize, useFormStatus, useFormFocusMonitor } from "@idux/components/form";
import { IxIcon } from "@idux/components/icon";
import { useAccessorAndControl } from "@idux/cdk/forms";
import { Logger, callEmit } from "@idux/cdk/utils";
const inputNumberProps = {
  value: [Number, null],
  control: {
    type: [String, Number, Object, Array],
    default: void 0
  },
  disabled: {
    type: Boolean,
    default: false
  },
  formatter: Function,
  keyboard: {
    type: Boolean,
    default: void 0
  },
  max: {
    type: Number,
    default: Infinity
  },
  min: {
    type: Number,
    default: -Infinity
  },
  placeholder: String,
  precision: Number,
  readonly: {
    type: Boolean,
    default: false
  },
  size: String,
  status: String,
  step: {
    type: Number,
    default: 1
  },
  "onUpdate:value": [Function, Array],
  onChange: [Function, Array],
  onFocus: [Function, Array],
  onBlur: [Function, Array]
};
function useInputNumber(props, config) {
  const { accessor, control } = useAccessorAndControl();
  useFormItemRegister(control);
  const mergedSize = useFormSize(props, config);
  const mergedStatus = useFormStatus(props, control);
  const rawDisplayValue = ref("");
  const displayValue = ref("");
  const isIllegal = ref(true);
  const nowValue = computed(() => {
    var _a;
    return (_a = accessor.value) != null ? _a : void 0;
  });
  const isKeyboard = computed(() => {
    var _a;
    return (_a = props.keyboard) != null ? _a : config.keyboard;
  });
  const isDisabled = computed(() => accessor.disabled);
  const isDisabledDec = computed(() => props.readonly || !!nowValue.value && nowValue.value <= props.min);
  const isDisabledInc = computed(() => props.readonly || !!nowValue.value && nowValue.value >= props.max);
  const precision = computed(() => {
    const stepPrecision = getPrecision(props.step);
    if (props.precision !== void 0) {
      if (process.env.NODE_ENV !== "production" && stepPrecision > props.precision) {
        Logger.warn(
          "components/input-number",
          `The precision(${props.precision}) should not be less than the decimal point of the step(${props.step}).`
        );
      }
      return props.precision;
    }
    return Math.max(getPrecision(accessor.value), stepPrecision);
  });
  const disabledDec = computed(() => getIncValueFormAccessor(-props.step) < props.min);
  const disabledInc = computed(() => getIncValueFormAccessor(props.step) > props.max);
  const updateDisplayValue = (v) => {
    rawDisplayValue.value = v;
    if (!v) {
      displayValue.value = v;
      return;
    }
    if (!props.formatter) {
      displayValue.value = v;
    } else {
      displayValue.value = props.formatter(Number(v));
    }
  };
  function getIncValueFormAccessor(step) {
    const { value } = accessor;
    let newVal = step;
    if (typeof value === "number" && !Number.isNaN(value)) {
      newVal = parseFloat((value + step).toFixed(precision.value));
    }
    return Math.max(props.min, Math.min(props.max, newVal));
  }
  function updateDisplayValueFromAccessor() {
    const { value } = accessor;
    if (!value && value !== 0 || !String(value).trim()) {
      updateDisplayValue("");
    } else if (Number.isNaN(Number(value)) || typeof value !== "number" && typeof value !== "string") {
      updateDisplayValue("");
      if (process.env.NODE_ENV !== "production") {
        Logger.warn("components/input-number", `model value(${value}) is not a number.`);
      }
    } else {
      const newValue = Number(value);
      if (displayValue.value === "" || newValue !== Number(displayValue.value)) {
        updateDisplayValue(newValue.toFixed(precision.value));
      }
    }
  }
  function updateModelValueFromDisplayValue() {
    const { value: strVal } = displayValue;
    const numberVal = parseFloat(Number(strVal).toFixed(precision.value));
    if (strVal === "") {
      updateModelValue(null);
    } else if (Number.isNaN(numberVal)) {
      updateDisplayValueFromAccessor();
    } else {
      const newVal = Math.max(props.min, Math.min(props.max, numberVal));
      updateDisplayValue(newVal.toFixed(precision.value));
      updateModelValue(newVal);
    }
  }
  function updateModelValue(newVal) {
    const oldVal = toRaw(accessor.value);
    if (newVal !== oldVal) {
      accessor.setValue(newVal);
      callEmit(props.onChange, newVal, oldVal);
      nextTick(() => {
        if (newVal !== accessor.value) {
          updateDisplayValueFromAccessor();
        }
      });
    }
  }
  function handleInput(evt) {
    const { value: inputVal } = evt.target;
    const strVal = inputVal.trim().replace(/。/g, ".");
    updateDisplayValue(strVal);
    if (strVal === "") {
      updateModelValue(null);
      return;
    }
    const numberVal = Number(strVal);
    if (!Number.isNaN(numberVal)) {
      if (numberVal >= props.min && numberVal <= props.max) {
        updateModelValue(numberVal);
      }
    }
  }
  function handleDec() {
    if (props.readonly || isDisabled.value || disabledDec.value) {
      return;
    }
    const newVal = getIncValueFormAccessor(-props.step);
    updateModelValue(newVal);
  }
  function handleInc() {
    if (props.readonly || isDisabled.value || disabledInc.value) {
      return;
    }
    const newVal = getIncValueFormAccessor(props.step);
    updateModelValue(newVal);
  }
  function handleKeyDown(evt) {
    if (isKeyboard.value) {
      if (evt.code === "Enter" || evt.code === "NumpadEnter") {
        updateModelValueFromDisplayValue();
      } else if (evt.code === "ArrowUp") {
        evt.preventDefault();
        handleInc();
      } else if (evt.code === "ArrowDown") {
        evt.preventDefault();
        handleDec();
      }
    }
  }
  const isFocused = ref(false);
  function handleFocus(evt) {
    isFocused.value = true;
    callEmit(props.onFocus, evt);
  }
  function handleBlur(evt) {
    isFocused.value = false;
    updateModelValueFromDisplayValue();
    accessor.markAsBlurred();
    callEmit(props.onBlur, evt);
  }
  watch(
    rawDisplayValue,
    (value) => {
      if (value !== "") {
        const numberVal = Number(value);
        isIllegal.value = Number.isNaN(numberVal) || numberVal < props.min || numberVal > props.max;
      } else {
        isIllegal.value = false;
      }
    },
    { immediate: true }
  );
  watch(
    () => accessor.value,
    () => updateDisplayValueFromAccessor(),
    { immediate: true }
  );
  return {
    mergedSize,
    mergedStatus,
    displayValue,
    isIllegal,
    isDisabled,
    isDisabledDec,
    isDisabledInc,
    isFocused,
    nowValue,
    handleKeyDown,
    handleDec,
    handleInc,
    handleInput,
    handleFocus,
    handleBlur
  };
}
function getPrecision(value) {
  if (value === void 0 || value === null) {
    return 0;
  }
  const decimal = String(value).split(".")[1];
  return decimal ? decimal.length : 0;
}
var InputNumber = /* @__PURE__ */ defineComponent({
  name: "IxInputNumber",
  props: inputNumberProps,
  setup(props, {
    expose,
    slots
  }) {
    const common = useGlobalConfig("common");
    const config = useGlobalConfig("inputNumber");
    const {
      mergedSize,
      mergedStatus,
      displayValue,
      nowValue,
      isIllegal,
      isDisabled,
      isDisabledDec,
      isDisabledInc,
      isFocused,
      handleInput,
      handleFocus,
      handleBlur,
      handleKeyDown,
      handleDec,
      handleInc
    } = useInputNumber(props, config);
    const {
      elementRef,
      focus,
      blur
    } = useFormFocusMonitor({
      handleBlur,
      handleFocus
    });
    expose({
      focus,
      blur
    });
    const mergedPrefixCls = computed(() => `${common.prefixCls}-input-number`);
    const classes = computed(() => {
      const prefixCls = mergedPrefixCls.value;
      return normalizeClass({
        [prefixCls]: true,
        [`${prefixCls}-illegal`]: isIllegal.value
      });
    });
    const inputRef = ref();
    onMounted(() => {
      elementRef.value = inputRef.value.getInputElement();
    });
    const handleIncrease = (evt) => {
      evt.stopPropagation();
      handleInc();
    };
    const handleDecrease = (evt) => {
      evt.stopPropagation();
      handleDec();
    };
    return () => {
      const prefixCls = mergedPrefixCls.value;
      const vSlots = {
        ...slots,
        suffix: () => [createVNode("span", {
          "key": "increase",
          "class": normalizeClass({
            [`${prefixCls}-increase`]: true,
            [`${prefixCls}-increase-disabled`]: isDisabled.value || isDisabledInc.value
          }),
          "role": "button",
          "onClick": handleIncrease
        }, [createVNode(IxIcon, {
          "name": "up"
        }, null)]), createVNode("span", {
          "key": "decrease",
          "class": normalizeClass({
            [`${prefixCls}-decrease`]: true,
            [`${prefixCls}-decrease-disabled`]: isDisabled.value || isDisabledDec.value
          }),
          "role": "button",
          "onClick": handleDecrease
        }, [createVNode(IxIcon, {
          "name": "down"
        }, null)])]
      };
      return createVNode(_Input, {
        "class": classes.value,
        "ref": inputRef,
        "type": "text",
        "autocomplete": "off",
        "aria-valuemin": props.min,
        "aria-valuemax": props.max,
        "aria-valuenow": nowValue.value,
        "disabled": isDisabled.value,
        "focused": isFocused.value,
        "readonly": props.readonly,
        "placeholder": props.placeholder,
        "size": mergedSize.value,
        "status": mergedStatus.value,
        "value": displayValue.value,
        "onInput": handleInput,
        "onKeydown": handleKeyDown
      }, vSlots);
    };
  }
});
const IxInputNumber = InputNumber;
export { IxInputNumber };
