import { computed, ref, watch, toRaw, nextTick, defineComponent, onMounted, createVNode } from "vue";
import { \u0275Input as _Input } from "@idux/components/_private/input";
import { useGlobalConfig } from "@idux/components/config";
import { useAccessorAndControl } from "@idux/cdk/forms";
import { callEmit } from "@idux/cdk/utils";
import { useFormItemRegister, useFormSize, useFormStatus, useFormFocusMonitor } from "@idux/components/form";
const inputCommonProps = {
  control: {
    type: [String, Number, Object, Array],
    default: void 0
  },
  value: { type: String, default: void 0 },
  clearable: { type: Boolean, default: void 0 },
  clearIcon: { type: String, default: void 0 },
  disabled: { type: Boolean, default: false },
  readonly: { type: Boolean, default: false },
  size: { type: String, default: void 0 },
  status: String,
  trim: { type: Boolean, default: void 0 },
  "onUpdate:value": [Function, Array],
  onChange: [Function, Array],
  onClear: [Function, Array],
  onCompositionStart: [Function, Array],
  onCompositionEnd: [Function, Array],
  onInput: [Function, Array],
  onFocus: [Function, Array],
  onBlur: [Function, Array]
};
const inputProps = {
  ...inputCommonProps,
  addonAfter: { type: String, default: void 0 },
  addonBefore: { type: String, default: void 0 },
  borderless: { type: Boolean, default: void 0 },
  prefix: { type: String, default: void 0 },
  suffix: { type: String, default: void 0 }
};
function useInput(props, config) {
  const { accessor, control } = useAccessorAndControl();
  useFormItemRegister(control);
  const mergedSize = useFormSize(props, config);
  const mergedStatus = useFormStatus(props, control);
  const clearable = computed(() => {
    var _a;
    return (_a = props.clearable) != null ? _a : config.clearable;
  });
  const clearIcon = computed(() => {
    var _a;
    return (_a = props.clearIcon) != null ? _a : config.clearIcon;
  });
  const clearVisible = computed(() => !accessor.disabled && !props.readonly && !!accessor.value);
  const mergedTrim = computed(() => {
    var _a;
    return (_a = props.trim) != null ? _a : config.trim;
  });
  const isFocused = ref(false);
  const handleFocus = (evt) => {
    isFocused.value = true;
    callEmit(props.onFocus, evt);
  };
  const handleBlur = (evt) => {
    isFocused.value = false;
    accessor.markAsBlurred();
    callEmit(props.onBlur, evt);
    if (mergedTrim.value) {
      setValue(evt.target.value.trim());
    }
  };
  const { elementRef, focus, blur } = useFormFocusMonitor({
    handleFocus,
    handleBlur
  });
  const setValue = (value) => {
    const oldValue = toRaw(accessor.value);
    if (value !== oldValue) {
      accessor.setValue(value);
      callEmit(props.onChange, value, oldValue);
      nextTick(() => syncValue());
    }
  };
  const syncValue = () => {
    var _a;
    const element = elementRef.value;
    const value = (_a = accessor.value) != null ? _a : "";
    if (element && element.value !== value) {
      element.value = value;
    }
  };
  watch(
    () => accessor.value,
    () => syncValue()
  );
  const isComposing = ref(false);
  const handleInput = (evt, emitInput = true) => {
    emitInput && callEmit(props.onInput, evt);
    if (isComposing.value) {
      return;
    }
    setValue(evt.target.value);
  };
  const handleCompositionStart = (evt) => {
    isComposing.value = true;
    callEmit(props.onCompositionStart, evt);
  };
  const handleCompositionEnd = (evt) => {
    callEmit(props.onCompositionEnd, evt);
    if (isComposing.value) {
      isComposing.value = false;
      handleInput(evt, false);
    }
  };
  const handleClear = (evt) => {
    accessor.setValue("");
    callEmit(props.onClear, evt);
  };
  return {
    elementRef,
    accessor,
    mergedSize,
    mergedStatus,
    clearable,
    clearIcon,
    clearVisible,
    isFocused,
    isComposing,
    focus,
    blur,
    handleInput,
    handleCompositionStart,
    handleCompositionEnd,
    handleFocus,
    handleBlur,
    handleClear,
    syncValue
  };
}
var Input = /* @__PURE__ */ defineComponent({
  name: "IxInput",
  props: inputProps,
  setup(props, {
    slots,
    expose
  }) {
    const config = useGlobalConfig("input");
    const {
      elementRef,
      accessor,
      mergedSize,
      mergedStatus,
      clearable,
      clearIcon,
      clearVisible,
      isFocused,
      focus,
      blur,
      handleInput,
      handleCompositionStart,
      handleCompositionEnd,
      handleClear,
      syncValue
    } = useInput(props, config);
    expose({
      focus,
      blur
    });
    const inputRef = ref();
    onMounted(() => {
      elementRef.value = inputRef.value.getInputElement();
      syncValue();
    });
    return () => {
      const {
        addonAfter,
        addonBefore,
        borderless,
        prefix,
        suffix
      } = props;
      return createVNode(_Input, {
        "ref": inputRef,
        "addonAfter": addonAfter,
        "addonBefore": addonBefore,
        "borderless": borderless,
        "clearable": clearable.value,
        "clearIcon": clearIcon.value,
        "clearVisible": clearVisible.value,
        "disabled": accessor.disabled,
        "focused": isFocused.value,
        "prefix": prefix,
        "readonly": props.readonly,
        "size": mergedSize.value,
        "status": mergedStatus.value,
        "suffix": suffix,
        "value": accessor.value,
        "onClear": handleClear,
        "onInput": handleInput,
        "onCompositionstart": handleCompositionStart,
        "onCompositionend": handleCompositionEnd
      }, slots);
    };
  }
});
const IxInput = Input;
export { IxInput, inputCommonProps as \u0275InputCommonProps, useInput as \u0275UseInput };
