import { defineComponent, computed, normalizeClass, createVNode, provide, watch, watchEffect, mergeProps, inject } from "vue";
import { isObject, isString } from "lodash-es";
import { useGlobalConfig } from "@idux/components/config";
import { useThemeToken } from "@idux/components/theme";
import { BREAKPOINTS_KEYS, useSharedBreakpoints } from "@idux/cdk/breakpoint";
import { useControlledProp } from "@idux/cdk/utils";
import { IxButton } from "@idux/components/button";
import { IxIcon } from "@idux/components/icon";
const layoutProps = {
  fixed: { type: [Boolean, Object], default: false },
  floatSider: { type: Boolean, default: false }
};
const layoutHeaderProps = {};
const layoutContentProps = {};
const layoutFooterProps = {};
const layoutSiderProps = {
  collapsed: { type: Boolean, default: void 0 },
  breakpoint: String,
  pointer: { type: Boolean, default: false },
  pointerDelay: { type: [Number, Array], default: () => [0, 100] },
  "onUpdate:collapsed": [Function, Array]
};
const layoutSiderTriggerProps = {
  icon: {
    type: [String, Array],
    default: void 0
  }
};
function getDefaultThemeTokens() {
  return {
    siderWidth: 224,
    siderCollapsedWidth: 44
  };
}
const getThemeTokens = (tokens, presetTheme) => {
  return presetTheme === "default" ? getDefaultThemeTokens() : getDefaultThemeTokens();
};
var Layout = /* @__PURE__ */ defineComponent({
  name: "IxLayout",
  props: layoutProps,
  setup(props, {
    slots
  }) {
    const common = useGlobalConfig("common");
    const {
      globalHashId,
      hashId,
      registerToken
    } = useThemeToken("layout");
    registerToken(getThemeTokens);
    const mergedPrefixCls = computed(() => `${common.prefixCls}-layout`);
    const mergedFixed = computed(() => {
      const {
        fixed
      } = props;
      if (isObject(fixed)) {
        const {
          header = false,
          sider = false
        } = fixed;
        return {
          header,
          sider
        };
      }
      return {
        header: fixed,
        sider: fixed
      };
    });
    const classes = computed(() => {
      const prefixCls = mergedPrefixCls.value;
      const {
        header,
        sider
      } = mergedFixed.value;
      return normalizeClass({
        [globalHashId.value]: !!globalHashId.value,
        [hashId.value]: !!hashId.value,
        [prefixCls]: true,
        [`${prefixCls}-fixed-header`]: header,
        [`${prefixCls}-fixed-sider`]: sider,
        [`${prefixCls}-float-sider`]: props.floatSider
      });
    });
    return () => {
      var _a;
      return createVNode("section", {
        "class": classes.value
      }, [(_a = slots.default) == null ? void 0 : _a.call(slots)]);
    };
  }
});
var LayoutContent = /* @__PURE__ */ defineComponent({
  name: "IxLayoutContent",
  props: layoutContentProps,
  setup(_, {
    slots
  }) {
    const common = useGlobalConfig("common");
    const {
      globalHashId,
      hashId
    } = useThemeToken("layout");
    const mergedPrefixCls = computed(() => `${common.prefixCls}-layout-content`);
    return () => {
      var _a;
      const prefixCls = mergedPrefixCls.value;
      return createVNode("main", {
        "class": [prefixCls, globalHashId.value, hashId.value]
      }, [(_a = slots.default) == null ? void 0 : _a.call(slots)]);
    };
  }
});
var LayoutFooter = /* @__PURE__ */ defineComponent({
  name: "IxLayoutFooter",
  props: layoutFooterProps,
  setup(_, {
    slots
  }) {
    const common = useGlobalConfig("common");
    const {
      globalHashId,
      hashId
    } = useThemeToken("layout");
    const mergedPrefixCls = computed(() => `${common.prefixCls}-layout-footer`);
    return () => {
      var _a;
      const prefixCls = mergedPrefixCls.value;
      return createVNode("footer", {
        "class": [prefixCls, globalHashId.value, hashId.value]
      }, [(_a = slots.default) == null ? void 0 : _a.call(slots)]);
    };
  }
});
var LayoutHeader = /* @__PURE__ */ defineComponent({
  name: "IxLayoutHeader",
  props: layoutHeaderProps,
  setup(_, {
    slots
  }) {
    const common = useGlobalConfig("common");
    const {
      globalHashId,
      hashId
    } = useThemeToken("layout");
    const mergedPrefixCls = computed(() => `${common.prefixCls}-layout-header`);
    return () => {
      var _a;
      const prefixCls = mergedPrefixCls.value;
      return createVNode("header", {
        "class": [prefixCls, globalHashId.value, hashId.value]
      }, [(_a = slots.default) == null ? void 0 : _a.call(slots)]);
    };
  }
});
const layoutSiderToken = Symbol("layoutSider");
var LayoutSider = /* @__PURE__ */ defineComponent({
  name: "IxLayoutSider",
  props: layoutSiderProps,
  setup(props, {
    slots
  }) {
    const common = useGlobalConfig("common");
    const {
      globalHashId,
      hashId
    } = useThemeToken("layout");
    const mergedPrefixCls = computed(() => `${common.prefixCls}-layout-sider`);
    const [collapsed, setCollapsed] = useControlledProp(props, "collapsed", false);
    provide(layoutSiderToken, {
      mergedPrefixCls,
      collapsed,
      setCollapsed
    });
    const breakpointIndex = computed(() => {
      const {
        breakpoint
      } = props;
      return breakpoint ? BREAKPOINTS_KEYS.indexOf(breakpoint) : -1;
    });
    const useBreakpoint = computed(() => breakpointIndex.value > -1);
    const breakpoints = useSharedBreakpoints();
    let stopBreakpoints;
    watch(useBreakpoint, (breakpoint) => {
      stopBreakpoints == null ? void 0 : stopBreakpoints();
      if (breakpoint) {
        stopBreakpoints = watchEffect(() => {
          const currBreakpointIndex = BREAKPOINTS_KEYS.findIndex((key) => breakpoints[key]);
          setCollapsed(currBreakpointIndex <= breakpointIndex.value, "breakpoint");
        });
      }
    }, {
      immediate: true
    });
    let timer;
    const setCollapsedWithDelay = (collapsed2) => {
      const {
        pointerDelay
      } = props;
      const [showDelay, hideDelay] = Array.isArray(pointerDelay) ? pointerDelay : [pointerDelay, pointerDelay];
      if (timer) {
        clearTimeout(timer);
      }
      timer = setTimeout(() => {
        setCollapsed(collapsed2, "pointer");
        timer = void 0;
      }, collapsed2 ? showDelay : hideDelay);
    };
    const pointerEvents = computed(() => {
      if (!props.pointer) {
        return void 0;
      }
      return {
        onPointerenter: () => setCollapsedWithDelay(false),
        onPointerleave: () => setCollapsedWithDelay(true)
      };
    });
    const classes = computed(() => {
      const prefixCls = mergedPrefixCls.value;
      return normalizeClass({
        [globalHashId.value]: !!globalHashId.value,
        [hashId.value]: !!hashId.value,
        [prefixCls]: true,
        [`${prefixCls}-collapsed`]: collapsed.value
      });
    });
    return () => {
      var _a;
      return createVNode("aside", mergeProps({
        "class": classes.value
      }, pointerEvents.value), [(_a = slots.default) == null ? void 0 : _a.call(slots)]);
    };
  }
});
var LayoutSiderTrigger = /* @__PURE__ */ defineComponent({
  name: "IxLayoutSiderTrigger",
  props: layoutSiderTriggerProps,
  setup(props, {
    slots
  }) {
    const {
      mergedPrefixCls,
      collapsed,
      setCollapsed
    } = inject(layoutSiderToken);
    const {
      globalHashId,
      hashId
    } = useThemeToken("layout");
    const mergedIcon = computed(() => {
      const {
        icon
      } = props;
      const [fold = "menu-fold", unfold = "menu-unfold"] = Array.isArray(icon) ? icon : [icon, icon];
      return collapsed.value ? fold : unfold;
    });
    const classes = computed(() => {
      const prefixCls = `${mergedPrefixCls.value}-trigger`;
      return {
        [globalHashId.value]: !!globalHashId.value,
        [hashId.value]: !!hashId.value,
        [prefixCls]: true,
        [`${prefixCls}-collapsed`]: collapsed.value
      };
    });
    const handleClick = () => {
      setCollapsed(!collapsed.value, "trigger");
    };
    return () => {
      var _a;
      let iconNode;
      if (slots.icon) {
        iconNode = slots.icon({
          collapsed: collapsed.value
        });
      } else {
        const iconValue = mergedIcon.value;
        iconNode = isString(iconValue) ? createVNode(IxIcon, {
          "name": iconValue
        }, null) : iconValue;
      }
      return createVNode("div", {
        "class": classes.value
      }, [createVNode("span", null, [(_a = slots.default) == null ? void 0 : _a.call(slots)]), createVNode(IxButton, {
        "mode": "text",
        "onClick": handleClick
      }, {
        icon: () => iconNode
      })]);
    };
  }
});
const IxLayout = Layout;
const IxLayoutContent = LayoutContent;
const IxLayoutFooter = LayoutFooter;
const IxLayoutHeader = LayoutHeader;
const IxLayoutSider = LayoutSider;
const IxLayoutSiderTrigger = LayoutSiderTrigger;
export { IxLayout, IxLayoutContent, IxLayoutFooter, IxLayoutHeader, IxLayoutSider, IxLayoutSiderTrigger, getThemeTokens as getLayoutThemeTokens };
