import { defineComponent, computed, normalizeClass, provide, createVNode, Fragment, Transition, withDirectives, vShow, ref, nextTick, inject } from "vue";
import { CdkPortal } from "@idux/cdk/portal";
import { rAF, cancelRAF, throwError } from "@idux/cdk/utils";
import { \u0275Mask as _Mask } from "@idux/components/_private/mask";
import { useGlobalConfig } from "@idux/components/config";
import { useThemeToken } from "@idux/components/theme";
import { usePortalTarget } from "@idux/components/utils";
const LOADING_BAR_PROVIDER_TOKEN = Symbol("LOADING_BAR_PROVIDER_TOKEN");
const loadingBarProviderProps = {
  container: {
    type: [String, HTMLElement, Function],
    default: void 0
  },
  mask: {
    type: Boolean,
    default: false
  }
};
function getDefaultThemeTokens(tokens) {
  const { colorPrimary } = tokens;
  return {
    height: 2,
    loadingColor: colorPrimary
  };
}
const getThemeTokens = (tokens, presetTheme) => {
  return presetTheme === "default" ? getDefaultThemeTokens(tokens) : getDefaultThemeTokens(tokens);
};
var LoadingBarProvider = /* @__PURE__ */ defineComponent({
  name: "IxLoadingBarProvider",
  inheritAttrs: false,
  props: loadingBarProviderProps,
  setup(props, {
    slots,
    expose
  }) {
    const common = useGlobalConfig("common");
    const {
      globalHashId,
      hashId,
      registerToken
    } = useThemeToken("loadingBar");
    registerToken(getThemeTokens);
    const config = useGlobalConfig("loadingBar");
    const mergedPrefixCls = computed(() => `${common.prefixCls}-loading-bar`);
    const mergedPortalTarget = usePortalTarget(props, config, common, mergedPrefixCls);
    const {
      start,
      finish,
      error,
      status,
      visible,
      progress,
      mask
    } = useLoadingBarProvider(props, config);
    const classes = computed(() => {
      const prefixCls = mergedPrefixCls.value;
      return normalizeClass({
        [globalHashId.value]: !!globalHashId.value,
        [hashId.value]: !!hashId.value,
        [`${prefixCls}`]: true,
        [`${prefixCls}--${status.value}`]: !!status.value
      });
    });
    const apis = {
      start,
      finish,
      error
    };
    provide(LOADING_BAR_PROVIDER_TOKEN, apis);
    expose(apis);
    return () => {
      var _a;
      return createVNode(Fragment, null, [(_a = slots.default) == null ? void 0 : _a.call(slots), createVNode(CdkPortal, {
        "target": mergedPortalTarget.value
      }, {
        default: () => [createVNode(_Mask, {
          "class": `${mergedPrefixCls.value}-mask`,
          "mask": mask.value,
          "visible": visible.value
        }, null), createVNode(Transition, {
          "appear": true,
          "name": `${common.prefixCls}-move-up`
        }, {
          default: () => [withDirectives(createVNode("div", {
            "class": classes.value,
            "style": {
              maxWidth: `${progress.value}%`
            }
          }, null), [[vShow, visible.value]])]
        })]
      })]);
    };
  }
});
const useLoadingBarProvider = (props, config) => {
  const {
    loading: loadingConfig,
    error: errorConfig,
    finish: finishConfig
  } = config.animation;
  const maskEnable = computed(() => {
    var _a;
    return (_a = props.mask) != null ? _a : config.mask;
  });
  const animation = (callback) => {
    let start2 = 0;
    return async function(t) {
      if (start2 === 0) {
        start2 = t;
      }
      callback(t - start2);
    };
  };
  const visible = ref(false);
  const status = ref("finish");
  const mask = ref(false);
  const progress = ref(0);
  let raf;
  const start = (options) => {
    var _a;
    raf && cancelRAF(raf);
    progress.value = 0;
    status.value = "loading";
    visible.value = true;
    mask.value = (_a = options == null ? void 0 : options.mask) != null ? _a : maskEnable.value;
    const animationFn = animation((elapsed) => {
      if (elapsed < loadingConfig.duration) {
        progress.value = elapsed / loadingConfig.duration * loadingConfig.progress;
        raf = rAF(animationFn);
      } else {
        cancelRAF(raf);
        progress.value = loadingConfig.progress;
      }
    });
    raf = rAF(animationFn);
  };
  const error = () => {
    raf && cancelRAF(raf);
    progress.value = 0;
    status.value = "error";
    visible.value = true;
    mask.value = maskEnable.value;
    const animationFn = animation(async (elapsed) => {
      if (elapsed < errorConfig.duration) {
        progress.value = elapsed / errorConfig.duration * errorConfig.progress;
        raf = rAF(animationFn);
      } else {
        cancelRAF(raf);
        progress.value = errorConfig.progress;
        await nextTick();
        visible.value = false;
      }
    });
    raf = rAF(animationFn);
  };
  const finish = () => {
    if (status.value !== "loading") {
      return;
    }
    raf && cancelRAF(raf);
    const loadingProgress = Math.min(progress.value, loadingConfig.progress);
    const animationFn = animation(async (elapsed) => {
      if (elapsed < finishConfig.duration) {
        progress.value = loadingProgress + elapsed / finishConfig.duration * (finishConfig.progress - loadingProgress);
        raf = rAF(animationFn);
      } else {
        cancelRAF(raf);
        progress.value = finishConfig.progress;
        await nextTick();
        mask.value = maskEnable.value;
        visible.value = false;
        status.value = "finish";
      }
    });
    raf = rAF(animationFn);
  };
  return {
    start,
    error,
    finish,
    status,
    visible,
    progress,
    mask
  };
};
const useLoadingBar = () => {
  const providerRef = inject(LOADING_BAR_PROVIDER_TOKEN, null);
  if (providerRef === null) {
    return throwError("cbomponents/loading-bar", "<IxLoadingBarProvider> not found.");
  }
  return providerRef;
};
const IxLoadingBarProvider = LoadingBarProvider;
export { IxLoadingBarProvider, LOADING_BAR_PROVIDER_TOKEN, getThemeTokens as getLoadingBarThemeTokens, useLoadingBar };
