import { defineComponent, computed, withDirectives, createVNode, vShow, onMounted, watchEffect, onBeforeUnmount, provide, mergeProps, Fragment, TransitionGroup, ref, inject } from "vue";
import { useControlledProp, callEmit, convertCssPixel, convertArray, uniqueId, throwError } from "@idux/cdk/utils";
import { useGlobalConfig } from "@idux/components/config";
import { numUnitTransform, useThemeToken } from "@idux/components/theme";
import { convertIconVNode, usePortalTarget, convertStringVNode } from "@idux/components/utils";
import { CdkPortal } from "@idux/cdk/portal";
const messageProps = {
  visible: {
    type: Boolean,
    default: void 0
  },
  destroyOnHover: {
    type: Boolean,
    default: void 0
  },
  duration: Number,
  icon: [String, Object, Function],
  type: {
    type: String,
    default: "info"
  },
  "onUpdate:visible": [Function, Array],
  onClose: [Function, Array]
};
const messageProviderProps = {
  container: {
    type: [String, HTMLElement, Function],
    default: void 0
  },
  maxCount: Number,
  top: [String, Number]
};
function getDefaultThemeTokens(tokens) {
  const {
    marginSizeSm,
    fontSizeMd,
    paddingSizeLg,
    paddingSizeSm,
    colorContainerBg,
    borderRadiusMd,
    overlayBorderWidth,
    overlayBorderType,
    overlayBorderColor
  } = tokens;
  return {
    margin: marginSizeSm,
    fontSize: fontSizeMd,
    borderWidth: overlayBorderWidth,
    borderType: overlayBorderType,
    borderColor: overlayBorderColor,
    contentMinWidth: 128,
    contentMaxWidth: 480,
    contentPaddingVertical: paddingSizeSm,
    contentPaddingHorizontal: paddingSizeLg,
    bgColor: colorContainerBg,
    borderRadius: borderRadiusMd,
    iconMarginRight: marginSizeSm,
    wrapperTop: `15%`
  };
}
const transforms = {
  wrapperTop: (value) => numUnitTransform(value, "px")
};
const getThemeTokens = (tokens, presetTheme) => {
  return presetTheme === "default" ? getDefaultThemeTokens(tokens) : getDefaultThemeTokens(tokens);
};
var Message = /* @__PURE__ */ defineComponent({
  name: "IxMessage",
  props: messageProps,
  setup(props, {
    slots
  }) {
    const common = useGlobalConfig("common");
    const {
      globalHashId,
      hashId,
      registerToken
    } = useThemeToken("message");
    registerToken(getThemeTokens, transforms);
    const mergedPrefixCls = computed(() => `${common.prefixCls}-message`);
    const config = useGlobalConfig("message");
    const classes = computed(() => {
      const prefixCls = mergedPrefixCls.value;
      return [prefixCls, globalHashId.value, hashId.value, `${prefixCls}-${props.type}`];
    });
    const mergedIcon = computed(() => {
      const {
        icon,
        type
      } = props;
      return icon != null ? icon : config.icon[type];
    });
    const {
      visible,
      onMouseEnter,
      onMouseLeave
    } = useEvents(props, config);
    return () => {
      var _a;
      const icon = mergedIcon.value;
      const iconNode = convertIconVNode(slots.icon, icon);
      const prefixCls = mergedPrefixCls.value;
      return withDirectives(createVNode("div", {
        "class": classes.value,
        "onMouseenter": onMouseEnter,
        "onMouseleave": onMouseLeave
      }, [createVNode("div", {
        "class": `${prefixCls}-content`
      }, [createVNode("span", {
        "class": `${prefixCls}-content-icon`
      }, [iconNode]), createVNode("span", {
        "class": `${prefixCls}-content-text`
      }, [(_a = slots.default) == null ? void 0 : _a.call(slots)])])]), [[vShow, visible.value]]);
    };
  }
});
const useEvents = (props, config) => {
  const duration = computed(() => {
    var _a;
    return (_a = props.duration) != null ? _a : config.duration;
  });
  const destroyOnHover = computed(() => {
    var _a;
    return (_a = props.destroyOnHover) != null ? _a : config.destroyOnHover;
  });
  const autoClose = computed(() => duration.value > 0);
  const [visible, setVisible] = useControlledProp(props, "visible", false);
  let timer = null;
  const startTimer = () => {
    timer = setTimeout(() => close(), duration.value);
  };
  const clearTimer = () => {
    if (timer) {
      clearTimeout(timer);
      timer = null;
    }
  };
  const close = () => {
    clearTimer();
    setVisible(false);
    callEmit(props.onClose);
  };
  const onMouseEnter = () => {
    if (autoClose.value && !destroyOnHover.value) {
      clearTimer();
    }
  };
  const onMouseLeave = () => {
    if (autoClose.value && !destroyOnHover.value) {
      startTimer();
    }
  };
  onMounted(() => {
    watchEffect(() => {
      clearTimer();
      if (visible.value && autoClose.value) {
        startTimer();
      }
    });
  });
  onBeforeUnmount(() => clearTimer());
  return {
    visible,
    onMouseEnter,
    onMouseLeave
  };
};
const MESSAGE_PROVIDER_TOKEN = Symbol("MESSAGE_PROVIDER_TOKEN");
var MessageProvider = /* @__PURE__ */ defineComponent({
  name: "IxMessageProvider",
  inheritAttrs: false,
  props: messageProviderProps,
  setup(props, {
    expose,
    slots,
    attrs
  }) {
    const common = useGlobalConfig("common");
    const {
      globalHashId,
      hashId,
      registerToken
    } = useThemeToken("message");
    registerToken(getThemeTokens, transforms);
    const config = useGlobalConfig("message");
    const mergedPrefixCls = computed(() => `${common.prefixCls}-message`);
    const mergedPortalTarget = usePortalTarget(props, config, common, mergedPrefixCls);
    const style = computed(() => {
      var _a;
      return {
        top: convertCssPixel((_a = props.top) != null ? _a : config.top)
      };
    });
    const maxCount = computed(() => {
      var _a;
      return (_a = props.maxCount) != null ? _a : config.maxCount;
    });
    const {
      messages,
      loadContainer,
      open,
      info,
      success,
      warning,
      error,
      loading,
      update,
      destroy,
      destroyAll
    } = useMessage$1(maxCount);
    const apis = {
      open,
      info,
      success,
      warning,
      error,
      loading,
      update,
      destroy,
      destroyAll
    };
    provide(MESSAGE_PROVIDER_TOKEN, apis);
    expose(apis);
    return () => {
      var _a;
      const child = messages.value.map((item) => {
        const {
          key,
          content,
          visible = true,
          onDestroy,
          ...restProps
        } = item;
        const onClose = () => destroy(key);
        const mergedProps = {
          key,
          visible,
          onClose
        };
        const contentNode = convertStringVNode(void 0, content);
        return createVNode(Message, mergeProps(mergedProps, restProps), {
          default: () => [contentNode]
        });
      });
      return createVNode(Fragment, null, [(_a = slots.default) == null ? void 0 : _a.call(slots), createVNode(CdkPortal, {
        "target": mergedPortalTarget.value,
        "load": loadContainer.value
      }, {
        default: () => [createVNode(TransitionGroup, mergeProps({
          "tag": "div",
          "appear": true,
          "name": `${common.prefixCls}-move-up`,
          "class": [`${mergedPrefixCls.value}-wrapper`, globalHashId.value, hashId.value],
          "style": style.value
        }, attrs), {
          default: () => [child]
        })]
      })]);
    };
  }
});
const useMessage$1 = (maxCount) => {
  const messages = ref([]);
  const getCurrIndex = (key) => {
    return messages.value.findIndex((message) => message.key === key);
  };
  const add = (item) => {
    var _a;
    const currIndex = item.key ? getCurrIndex(item.key) : -1;
    if (currIndex !== -1) {
      messages.value.splice(currIndex, 1, item);
      return item.key;
    }
    if (messages.value.length >= maxCount.value) {
      messages.value = messages.value.slice(-maxCount.value + 1);
    }
    const key = (_a = item.key) != null ? _a : uniqueId("ix-message");
    messages.value.push({
      ...item,
      key
    });
    return key;
  };
  const update = (key, item) => {
    const currIndex = getCurrIndex(key);
    if (currIndex !== -1) {
      const newItem = {
        ...messages.value[currIndex],
        ...item
      };
      messages.value.splice(currIndex, 1, newItem);
    }
  };
  const destroy = (key) => {
    const keys = convertArray(key);
    keys.forEach((key2) => {
      const currIndex = getCurrIndex(key2);
      if (currIndex !== -1) {
        const item = messages.value.splice(currIndex, 1);
        callEmit(item[0].onDestroy, key2);
      }
    });
  };
  const destroyAll = () => {
    messages.value = [];
  };
  const loadContainer = ref(false);
  const open = (options) => {
    const key = add(options);
    const ref2 = {
      key,
      update: (options2) => update(key, options2),
      destroy: () => destroy(key)
    };
    if (!loadContainer.value) {
      loadContainer.value = true;
    }
    return ref2;
  };
  const messageTypes = ["info", "success", "warning", "error", "loading"];
  const [info, success, warning, error, loading] = messageTypes.map((type) => {
    return (content, options) => open({
      ...options,
      content,
      type
    });
  });
  return {
    messages,
    loadContainer,
    open,
    info,
    success,
    warning,
    error,
    loading,
    update,
    destroy,
    destroyAll
  };
};
const useMessage = () => {
  const modalProviderRef = inject(MESSAGE_PROVIDER_TOKEN, null);
  if (modalProviderRef === null) {
    return throwError("components/message", "<IxMessageProvider> not found.");
  }
  return modalProviderRef;
};
const IxMessage = Message;
const IxMessageProvider = MessageProvider;
export { IxMessage, IxMessageProvider, MESSAGE_PROVIDER_TOKEN, getThemeTokens as getMessageThemeTokens, useMessage };
