import { defineComponent, computed, withDirectives, createVNode, vShow, onMounted, watchEffect, onBeforeUnmount, provide, TransitionGroup, Fragment, isVNode, cloneVNode, mergeProps, ref, inject } from "vue";
import { isFunction, isArray, pickBy, isUndefined } from "lodash-es";
import { useControlledProp, callEmit, convertCssPixel, convertArray, uniqueId, throwError } from "@idux/cdk/utils";
import { IxButton } from "@idux/components/button";
import { useGlobalConfig } from "@idux/components/config";
import { IxSpace } from "@idux/components/space";
import { numUnitTransform, useThemeToken } from "@idux/components/theme";
import { convertIconVNode, convertStringVNode, usePortalTarget } from "@idux/components/utils";
import { CdkPortal } from "@idux/cdk/portal";
const notificationType = ["info", "success", "warning", "error"];
const notificationPlacement = ["topStart", "topEnd", "bottomStart", "bottomEnd"];
const notificationProps = {
  visible: {
    type: Boolean,
    default: void 0
  },
  destroyOnHover: {
    type: Boolean,
    default: void 0
  },
  duration: Number,
  icon: [String, Object, Function],
  closeIcon: [String, Object, Function],
  type: String,
  title: [String, Object, Function],
  content: [String, Object, Function],
  footer: [String, Array, Object],
  placement: String,
  "onUpdate:visible": [Function, Array],
  onClose: [Function, Array]
};
const notificationProviderProps = {
  container: {
    type: [String, HTMLElement, Function],
    default: void 0
  },
  maxCount: Number,
  offset: [String, Number, Array]
};
function getDefaultThemeTokens(tokens) {
  const {
    paddingSizeSm,
    paddingSizeLg,
    marginSizeXs,
    marginSizeSm,
    marginSizeLg,
    fontSizeMd,
    fontSizeSm,
    colorContainerBg,
    colorTextInfo,
    borderRadiusSm,
    overlayBorderWidth,
    overlayBorderType,
    overlayBorderColor
  } = tokens;
  return {
    padding: `${paddingSizeSm}px ${paddingSizeSm}px ${paddingSizeSm}px ${paddingSizeLg}px`,
    margin: `0 0 ${marginSizeSm}px 0`,
    width: 384,
    maxWidth: `calc(100vw - 48px)`,
    borderWidth: overlayBorderWidth,
    borderType: overlayBorderType,
    borderColor: overlayBorderColor,
    fontSize: fontSizeMd,
    bgColor: colorContainerBg,
    borderRadius: borderRadiusSm,
    iconMargin: `${marginSizeXs}px ${marginSizeSm}px`,
    iconWrapperWidth: 44,
    titleFontSize: fontSizeMd,
    titleLineHeight: fontSizeMd + 10,
    titleMargin: `0 24px 8px 0`,
    contentFontSize: fontSizeSm,
    contentColor: colorTextInfo,
    contentMargin: `0 ${marginSizeLg}px 0 0`,
    closeIconTop: marginSizeSm,
    closeIconRight: marginSizeSm,
    footerMargin: `${marginSizeSm}px 0 0 0`
  };
}
const transforms = {
  closeIconTop: (value) => numUnitTransform(value, "px"),
  closeIconRight: (value) => numUnitTransform(value, "px")
};
const getThemeTokens = (tokens, presetTheme) => {
  return presetTheme === "default" ? getDefaultThemeTokens(tokens) : getDefaultThemeTokens(tokens);
};
const defaultCloseIcon = "close";
const defaultIcon = {
  success: "check-circle-filled",
  error: "close-circle-filled",
  info: "info-circle-filled",
  warning: "exclamation-circle-filled"
};
var Notification = /* @__PURE__ */ defineComponent({
  name: "IxNotification",
  props: notificationProps,
  setup(props, {
    slots
  }) {
    const commonCfg = useGlobalConfig("common");
    const {
      globalHashId,
      hashId,
      registerToken
    } = useThemeToken("notification");
    registerToken(getThemeTokens, transforms);
    const comPrefix = computed(() => `${commonCfg.prefixCls}-notification`);
    const config = useGlobalConfig("notification");
    const wrapCls = useClasses(props, comPrefix, globalHashId, hashId);
    const icon = useIcon(props, config);
    const closeIcon = useCloseIcon(props, config);
    const {
      visible,
      close,
      onMouseEnter,
      onMouseLeave
    } = useVisible(props, config);
    return () => {
      const iconNode = convertIconVNode(void 0, icon.value);
      const closeIconNode = convertIconVNode(void 0, closeIcon.value);
      const titleNode = convertStringVNode(slots, props, "title");
      const contentNode = convertStringVNode(slots.default, props.content);
      const footerNode = getFooterNode(slots, props.footer, {
        visible: visible.value,
        close
      });
      let mainCls = `${comPrefix.value}-main`;
      if (contentNode) {
        mainCls += ` ${comPrefix.value}-main-with-content`;
      }
      return withDirectives(createVNode("div", null, [createVNode("div", {
        "class": wrapCls.value,
        "onMouseenter": onMouseEnter,
        "onMouseleave": onMouseLeave
      }, [withDirectives(createVNode("div", {
        "class": `${comPrefix.value}-icon`
      }, [iconNode]), [[vShow, icon.value]]), createVNode("span", {
        "class": `${comPrefix.value}-close-icon`,
        "onClick": close
      }, [closeIconNode]), createVNode("div", {
        "class": mainCls
      }, [createVNode("div", {
        "class": `${comPrefix.value}-title`
      }, [titleNode]), contentNode && createVNode("div", {
        "class": `${comPrefix.value}-content`
      }, [contentNode])]), createVNode("div", {
        "class": `${comPrefix.value}-footer`
      }, [footerNode && createVNode("div", {
        "class": `${comPrefix.value}-footer`
      }, [footerNode])])])]), [[vShow, visible.value]]);
    };
  }
});
function useClasses(props, comPrefix, globalHashId, hashId) {
  return computed(() => [comPrefix.value, globalHashId.value, hashId.value, {
    [`${comPrefix.value}-${props.type}`]: props.type
  }]);
}
function useIcon(props, config) {
  return computed(() => {
    var _a;
    const iconMap = {
      ...defaultIcon,
      ...config.icon
    };
    return (_a = props.icon) != null ? _a : props.type && iconMap[props.type];
  });
}
function useCloseIcon(props, config) {
  return computed(() => {
    var _a, _b;
    return (_b = (_a = props.closeIcon) != null ? _a : config.closeIcon) != null ? _b : defaultCloseIcon;
  });
}
function useVisible(props, config) {
  const duration = computed(() => {
    var _a;
    return (_a = props.duration) != null ? _a : config.duration;
  });
  const destroyOnHover = computed(() => {
    var _a;
    return (_a = props.destroyOnHover) != null ? _a : config.destroyOnHover;
  });
  const autoClose = computed(() => duration.value > 0);
  const [visible, setVisible] = useControlledProp(props, "visible", false);
  let timer = null;
  const startTimer = () => {
    timer = setTimeout(() => close(), duration.value);
  };
  const clearTimer = () => {
    if (timer) {
      clearTimeout(timer);
      timer = null;
    }
  };
  const close = () => {
    clearTimer();
    setVisible(false);
    callEmit(props.onClose);
  };
  const onMouseEnter = () => {
    if (autoClose.value && !destroyOnHover.value) {
      clearTimer();
    }
  };
  const onMouseLeave = () => {
    if (autoClose.value && !destroyOnHover.value) {
      startTimer();
    }
  };
  onMounted(() => {
    watchEffect(() => {
      clearTimer();
      if (visible.value && autoClose.value) {
        startTimer();
      }
    });
  });
  onBeforeUnmount(() => clearTimer());
  return {
    visible,
    close,
    onMouseEnter,
    onMouseLeave
  };
}
function getFooterNode(slots, footer, params) {
  if (slots.footer) {
    return slots.footer(params);
  }
  if (!footer) {
    return "";
  }
  if (isFunction(footer)) {
    return footer(params);
  }
  if (isArray(footer)) {
    const footerButtons = footer.map((item) => {
      const {
        text,
        ...rest
      } = item;
      return createVNode(IxButton, rest, {
        default: () => [text]
      });
    });
    return createVNode(IxSpace, null, {
      default: () => [footerButtons]
    });
  }
  return footer;
}
const NOTIFICATION_PROVIDER_TOKEN = Symbol("NOTIFICATION_PROVIDER_TOKEN");
const groupPositions = {
  topStart: ["top", "left"],
  topEnd: ["top", "right"],
  bottomStart: ["bottom", "left"],
  bottomEnd: ["bottom", "right"]
};
var NotificationProvider = /* @__PURE__ */ defineComponent({
  name: "IxNotificationProvider",
  inheritAttrs: false,
  props: notificationProviderProps,
  setup(props, {
    slots,
    expose
  }) {
    const commonCfg = useGlobalConfig("common");
    const {
      globalHashId,
      hashId,
      registerToken
    } = useThemeToken("notification");
    registerToken(getThemeTokens, transforms);
    const config = useGlobalConfig("notification");
    const mergedPrefixCls = computed(() => `${commonCfg.prefixCls}-notification`);
    const mergedPortalTarget = usePortalTarget(props, config, commonCfg, mergedPrefixCls);
    const maxCount = computed(() => {
      var _a;
      return (_a = props.maxCount) != null ? _a : config.maxCount;
    });
    const {
      notifications,
      loadContainer,
      open,
      info,
      success,
      warning,
      error,
      update,
      destroy,
      destroyAll
    } = useNotification$1(maxCount);
    const apis = {
      open,
      info,
      success,
      warning,
      error,
      update,
      destroy,
      destroyAll
    };
    const placementNotifications = usePlacementNotifications(props, config, notifications);
    provide(NOTIFICATION_PROVIDER_TOKEN, apis);
    expose(apis);
    return () => {
      var _a;
      const getChild = (notifications2) => {
        return notifications2.map((item) => {
          const {
            key,
            visible = true,
            content,
            contentProps,
            ...restProps
          } = item;
          const onClose = () => destroy(key);
          const mergedProps = {
            key,
            visible,
            onClose
          };
          const contentNode = isVNode(content) ? cloneVNode(content, contentProps, true) : content;
          return createVNode(Notification, mergeProps(mergedProps, restProps), {
            default: () => [contentNode]
          });
        });
      };
      const placementGroup = Object.entries(placementNotifications.value).map(([key, notificationsRecord]) => {
        const placement = key;
        const child = getChild(notificationsRecord);
        const moveName = getMoveName(placement, commonCfg);
        const position = getGroupPosition(props, config, placement);
        return createVNode(TransitionGroup, {
          "key": placement,
          "tag": "div",
          "appear": true,
          "name": moveName,
          "class": [`${mergedPrefixCls.value}-wrapper`, `${mergedPrefixCls.value}-wrapper-${placement}`, globalHashId.value, hashId.value],
          "style": position
        }, {
          default: () => [child]
        });
      });
      return createVNode(Fragment, null, [(_a = slots.default) == null ? void 0 : _a.call(slots), createVNode(CdkPortal, {
        "target": mergedPortalTarget.value,
        "load": loadContainer.value
      }, {
        default: () => [placementGroup]
      })]);
    };
  }
});
function usePlacementNotifications(props, config, notifications) {
  return computed(() => {
    const initValue = getPlacementInitValue(() => []);
    const {
      destroyOnHover,
      duration,
      closeIcon,
      placement
    } = config;
    if (notifications.value.length === 0) {
      return initValue;
    }
    return notifications.value.reduce((acc, item) => {
      const notificationProps2 = getRealObj({
        destroyOnHover,
        duration,
        closeIcon,
        placement,
        ...item
      });
      const curPlacement = notificationProps2.placement;
      acc[curPlacement].push(notificationProps2);
      return acc;
    }, initValue);
  });
}
function useNotification$1(maxCount) {
  const notifications = ref([]);
  const getCurrIndex = (key) => {
    return notifications.value.findIndex((notification) => notification.key === key);
  };
  const add = (item) => {
    var _a;
    const currIndex = item.key ? getCurrIndex(item.key) : -1;
    if (currIndex !== -1) {
      notifications.value.splice(currIndex, 1, item);
      return item.key;
    }
    if (notifications.value.length >= maxCount.value) {
      notifications.value = notifications.value.slice(-maxCount.value + 1);
    }
    const key = (_a = item.key) != null ? _a : uniqueId("ix-notification");
    notifications.value.push({
      ...item,
      key
    });
    return key;
  };
  const update = (key, item) => {
    const currIndex = getCurrIndex(key);
    if (currIndex !== -1) {
      const newItem = {
        ...notifications.value[currIndex],
        ...item
      };
      notifications.value.splice(currIndex, 1, newItem);
    }
  };
  const destroy = (key) => {
    const keys = convertArray(key);
    keys.forEach((key2) => {
      const currIndex = getCurrIndex(key2);
      if (currIndex !== -1) {
        const item = notifications.value.splice(currIndex, 1);
        callEmit(item[0].onDestroy, key2);
      }
    });
  };
  const destroyAll = () => {
    notifications.value = [];
  };
  const loadContainer = ref(false);
  const open = (options) => {
    const key = add(options);
    const ref2 = {
      key,
      update: (options2) => update(key, options2),
      destroy: () => destroy(key)
    };
    if (!loadContainer.value) {
      loadContainer.value = true;
    }
    return ref2;
  };
  const [info, success, warning, error] = notificationType.map((type) => (options) => open({
    ...options,
    type
  }));
  return {
    notifications,
    loadContainer,
    open,
    info,
    success,
    warning,
    error,
    update,
    destroy,
    destroyAll
  };
}
function getGroupPosition(props, config, placement) {
  var _a;
  const realPlacement = placement != null ? placement : config.placement;
  const realOffset = (_a = props.offset) != null ? _a : config.offset;
  return getPosition(realOffset, realPlacement);
}
function getPosition(offset, placement) {
  const offsets = isArray(offset) ? offset : [offset, offset];
  const [verticalOffset, horizontalOffset] = offsets.map(convertCssPixel);
  const [verticalPosition, horizontalPosition] = groupPositions[placement];
  return {
    [verticalPosition]: verticalOffset,
    [horizontalPosition]: horizontalOffset
  };
}
function getMoveName(placement, commonCfg) {
  const prefixCls = commonCfg.prefixCls;
  const moveNameMap = {
    topEnd: `${prefixCls}-move-end`,
    bottomEnd: `${prefixCls}-move-end`,
    topStart: `${prefixCls}-move-start`,
    bottomStart: `${prefixCls}-move-start`
  };
  return moveNameMap[placement];
}
function getRealObj(obj) {
  return pickBy(obj, (val) => !isUndefined(val));
}
function getPlacementInitValue(initValueFn) {
  return notificationPlacement.reduce((acc, item) => {
    const cloneValue = initValueFn();
    return {
      ...acc,
      [item]: cloneValue
    };
  }, {});
}
const useNotification = () => {
  const notificationProviderRef = inject(NOTIFICATION_PROVIDER_TOKEN, null);
  if (notificationProviderRef === null) {
    return throwError("components/notification", "<IxNotificationProvider> not found.");
  }
  return notificationProviderRef;
};
const IxNotification = Notification;
const IxNotificationProvider = NotificationProvider;
export { IxNotification, IxNotificationProvider, NOTIFICATION_PROVIDER_TOKEN, getThemeTokens as getNotificationThemeTokens, useNotification };
