import { computed, withKeys, watch, defineComponent, inject, createVNode, createTextVNode, provide, normalizeClass, mergeProps } from "vue";
import { \u0275Input as _Input } from "@idux/components/_private/input";
import { useGlobalConfig } from "@idux/components/config";
import { useThemeToken } from "@idux/components/theme";
import { convertNumber, useControlledProp, callEmit } from "@idux/cdk/utils";
import { isNil } from "lodash-es";
import { IxIcon } from "@idux/components/icon";
import { IxSelect } from "@idux/components/select";
function useItems(activeIndex, lastIndex) {
  return computed(() => {
    const _activeIndex = activeIndex.value;
    const _lastIndex = lastIndex.value;
    let items;
    if (_lastIndex <= 9) {
      items = generatePage(1, _lastIndex);
    } else {
      items = generateRangeItem(_activeIndex, _lastIndex);
    }
    const prevItem = { type: "prev", disabled: _activeIndex === 1 };
    const nextItem = { type: "next", disabled: _activeIndex === _lastIndex };
    return [prevItem, ...items, nextItem];
  });
}
const generatePage = (start, end) => {
  const list = [];
  for (let index = start; index <= end; index++) {
    list.push({ index, type: "page" });
  }
  return list;
};
const generateRangeItem = (pageIndex, lastIndex) => {
  let listOfRange = [];
  const prevFiveItem = { type: "prev5" };
  const nextFiveItem = { type: "next5" };
  if (pageIndex < 5) {
    listOfRange = [...generatePage(2, 5), nextFiveItem];
  } else if (pageIndex < lastIndex - 3) {
    listOfRange = [prevFiveItem, ...generatePage(pageIndex - 2, pageIndex + 2), nextFiveItem];
  } else {
    listOfRange = [prevFiveItem, ...generatePage(lastIndex - 4, lastIndex - 1)];
  }
  const firstPageItem = generatePage(1, 1);
  const lastPageItem = generatePage(lastIndex, lastIndex);
  return [...firstPageItem, ...listOfRange, ...lastPageItem];
};
function useJumpToIndex(activeIndex, changePageIndex, simple) {
  const jumpToIndex = (evt) => {
    const target = evt.target;
    const index = Math.floor(convertNumber(target.value, activeIndex.value));
    changePageIndex(index);
    target.value = simple.value ? `${activeIndex.value}` : "";
  };
  return withKeys(jumpToIndex, ["enter"]);
}
function usePages(props, config) {
  var _a;
  const [activeIndex, setActiveIndex] = useControlledProp(props, "pageIndex", 1);
  const [activeSize, setActiveSize] = useControlledProp(props, "pageSize", (_a = props.pageSize) != null ? _a : config.pageSize);
  const lastIndex = computed(() => Math.max(Math.ceil(props.total / activeSize.value), 1));
  const changePageIndex = (index) => {
    const validIndex = validatePageIndex(index, lastIndex.value);
    if (validIndex !== activeIndex.value) {
      setActiveIndex(validIndex);
      callEmit(props.onChange, validIndex, activeSize.value);
    }
  };
  const changePageSize = (size) => {
    setActiveSize(size);
    callEmit(props.onChange, activeIndex.value, size);
  };
  watch(
    [activeIndex, lastIndex],
    ([currPageIndex, currLastIndex]) => {
      if (currPageIndex > currLastIndex) {
        changePageIndex(currLastIndex);
      }
    },
    { immediate: true }
  );
  return { activeIndex, activeSize, lastIndex, changePageIndex, changePageSize };
}
const validatePageIndex = (index, lastIndex) => {
  if (index > lastIndex) {
    return lastIndex;
  } else if (index < 1) {
    return 1;
  } else {
    return index;
  }
};
const paginationToken = Symbol("paginationToken");
const paginationProps = {
  pageIndex: Number,
  pageSize: Number,
  disabled: {
    type: Boolean,
    default: false
  },
  pageSizes: Array,
  showQuickJumper: {
    type: Boolean,
    default: void 0
  },
  showSizeChanger: {
    type: Boolean,
    default: void 0
  },
  showTitle: {
    type: Boolean,
    default: void 0
  },
  showTotal: {
    type: Boolean,
    default: void 0
  },
  simple: {
    type: Boolean,
    default: void 0
  },
  size: String,
  total: {
    type: Number,
    default: 0
  },
  "onUpdate:pageIndex": [Function, Array],
  "onUpdate:pageSize": [Function, Array],
  onChange: [Function, Array]
};
const paginationItemProps = {
  disabled: {
    type: Boolean,
    default: void 0
  },
  index: Number,
  type: {
    type: String,
    required: true
  }
};
const iconMap = {
  prev: "left",
  next: "right",
  prev5: "double-left",
  next5: "double-right",
  page: ""
};
const indexDiffMap = {
  next: 1,
  prev: -1,
  prev5: -5,
  next5: 5
};
var Item = /* @__PURE__ */ defineComponent({
  props: paginationItemProps,
  setup(props) {
    const {
      props: paginationProps2,
      slots,
      config,
      locale,
      mergedPrefixCls,
      activeIndex,
      changePageIndex
    } = inject(paginationToken);
    const isActive = computed(() => activeIndex.value === props.index);
    const isDisabled = computed(() => props.disabled || paginationProps2.disabled);
    const showTitle = computed(() => {
      var _a;
      return (_a = paginationProps2.showTitle) != null ? _a : config.showTitle;
    });
    const classes = computed(() => {
      const prefixCls = `${mergedPrefixCls.value}-item`;
      return {
        [prefixCls]: true,
        [`${prefixCls}-${props.type}`]: true,
        [`${prefixCls}-active`]: isActive.value,
        [`${prefixCls}-disabled`]: isDisabled.value
      };
    });
    const title = computed(() => {
      if (!showTitle.value) {
        return void 0;
      }
      const {
        type,
        index
      } = props;
      if (type === "page") {
        return index;
      }
      return locale.pagination[type];
    });
    const handleClick = () => {
      if (isDisabled.value) {
        return;
      }
      const {
        type,
        index
      } = props;
      let newIndex;
      if (type === "page") {
        newIndex = index;
      } else {
        newIndex = activeIndex.value + indexDiffMap[type];
      }
      changePageIndex(newIndex);
    };
    const handleKeydown = (evt) => {
      if (isDisabled.value || evt.code !== "Enter") {
        return;
      }
      handleClick();
    };
    return () => {
      const prefixCls = `${mergedPrefixCls.value}-item`;
      const {
        index,
        type
      } = props;
      const disabled = isDisabled.value;
      let original;
      if (props.type === "prev5" || type === "next5") {
        original = createVNode("span", {
          "class": ` ${prefixCls}-jumper`
        }, [createVNode(IxIcon, {
          "name": iconMap[type]
        }, null), createVNode(IxIcon, {
          "class": `${prefixCls}-ellipsis`,
          "name": "ellipsis"
        }, null)]);
      } else if (!isNil(index)) {
        original = createTextVNode(index);
      } else {
        original = createVNode(IxIcon, {
          "name": iconMap[type]
        }, null);
      }
      const children = slots.item ? slots.item({
        index,
        type,
        active: isActive.value,
        disabled,
        original
      }) : original;
      return createVNode("li", {
        "class": classes.value,
        "tabindex": disabled ? -1 : 0,
        "title": title.value,
        "onClick": handleClick,
        "onKeydown": handleKeydown
      }, [createVNode("span", {
        "class": `${prefixCls}-content`
      }, [children])]);
    };
  }
});
var Jumper = /* @__PURE__ */ defineComponent({
  setup() {
    const {
      props,
      locale,
      mergedPrefixCls,
      mergedSize,
      jumpToIndex
    } = inject(paginationToken);
    return () => {
      const prefixCls = `${mergedPrefixCls.value}-jumper`;
      const {
        disabled
      } = props;
      const {
        jumpTo,
        page
      } = locale.pagination;
      return createVNode("li", {
        "class": prefixCls
      }, [jumpTo, createVNode(_Input, {
        "disabled": disabled,
        "size": mergedSize.value === "lg" ? "md" : "sm",
        "onKeydown": jumpToIndex
      }, null), page]);
    };
  }
});
var Sizes = /* @__PURE__ */ defineComponent({
  setup() {
    const {
      props,
      config,
      locale,
      mergedPrefixCls,
      mergedSize,
      activeSize,
      changePageSize
    } = inject(paginationToken);
    const sizeData = computed(() => {
      const {
        pageSizes = config.pageSizes
      } = props;
      return pageSizes.map((size) => {
        return {
          key: size,
          label: size
        };
      });
    });
    return () => {
      const prefixCls = `${mergedPrefixCls.value}-sizes`;
      const {
        itemsPerPage,
        itemsSuffix
      } = locale.pagination;
      return createVNode("li", {
        "class": prefixCls
      }, [itemsPerPage, createVNode(IxSelect, {
        "disabled": props.disabled,
        "dataSource": sizeData.value,
        "size": mergedSize.value === "lg" ? "md" : "sm",
        "value": activeSize.value,
        "onChange": changePageSize
      }, null), itemsSuffix]);
    };
  }
});
var Total = /* @__PURE__ */ defineComponent({
  setup() {
    const {
      props,
      slots,
      locale,
      activeIndex,
      activeSize,
      mergedPrefixCls
    } = inject(paginationToken);
    const range = computed(() => {
      const currIndex = activeIndex.value;
      const currSize = activeSize.value;
      const firstIndex = (currIndex - 1) * currSize + 1;
      const lastIndex = Math.min(currIndex * currSize, props.total);
      return [firstIndex, lastIndex];
    });
    return () => {
      const prefixCls = `${mergedPrefixCls.value}-total`;
      const {
        total
      } = props;
      const {
        totalPrefix,
        totalSuffix
      } = locale.pagination;
      const children = slots.total ? slots.total({
        total,
        range: range.value,
        prefix: totalPrefix,
        suffix: totalSuffix
      }) : `${totalPrefix} ${total} ${totalSuffix}`;
      return createVNode("li", {
        "class": prefixCls
      }, [children]);
    };
  }
});
function getDefaultThemeTokens(tokens, algorithms) {
  const { getColorPalette } = algorithms;
  const { colorPrimary, fontSizeSm, paddingSizeXs, heightSm, heightMd } = tokens;
  const primaryColorPalette = getColorPalette(colorPrimary);
  return {
    fontSizeSm,
    fontSizeMd: fontSizeSm,
    fontSizeLg: fontSizeSm,
    itemPaddingSm: 0,
    itemPaddingMd: paddingSizeXs,
    itemPaddingLg: paddingSizeXs,
    itemContentSizeSm: heightSm,
    itemContentSizeMd: heightSm,
    itemContentSizeLg: heightMd,
    outLineColor: primaryColorPalette.l10
  };
}
const getThemeTokens = (tokens, presetTheme, algorithms) => {
  return presetTheme === "default" ? getDefaultThemeTokens(tokens, algorithms) : getDefaultThemeTokens(tokens, algorithms);
};
var Pagination = /* @__PURE__ */ defineComponent({
  name: "IxPagination",
  props: paginationProps,
  setup(props, {
    slots
  }) {
    const common = useGlobalConfig("common");
    const {
      globalHashId,
      hashId,
      registerToken
    } = useThemeToken("pagination");
    registerToken(getThemeTokens);
    const mergedPrefixCls = computed(() => `${common.prefixCls}-pagination`);
    const locale = useGlobalConfig("locale");
    const config = useGlobalConfig("pagination");
    const showTotal = computed(() => {
      var _a;
      return (_a = props.showTotal) != null ? _a : config.showTotal;
    });
    const simple = computed(() => {
      var _a;
      return (_a = props.simple) != null ? _a : config.simple;
    });
    const mergedSize = computed(() => {
      var _a;
      return (_a = props.size) != null ? _a : config.size;
    });
    const showQuickJumper = computed(() => {
      var _a;
      return (_a = props.showQuickJumper) != null ? _a : config.showQuickJumper;
    });
    const showSizeChanger = computed(() => {
      var _a;
      return (_a = props.showSizeChanger) != null ? _a : config.showSizeChanger;
    });
    const {
      activeIndex,
      activeSize,
      lastIndex,
      changePageIndex,
      changePageSize
    } = usePages(props, config);
    const items = useItems(activeIndex, lastIndex);
    const jumpToIndex = useJumpToIndex(activeIndex, changePageIndex, simple);
    provide(paginationToken, {
      props,
      slots,
      config,
      locale,
      mergedPrefixCls,
      mergedSize,
      activeIndex,
      activeSize,
      lastIndex,
      changePageIndex,
      changePageSize,
      jumpToIndex
    });
    const classes = computed(() => {
      const prefixCls = mergedPrefixCls.value;
      return normalizeClass({
        [globalHashId.value]: !!globalHashId.value,
        [hashId.value]: !!hashId.value,
        [prefixCls]: true,
        [`${prefixCls}-disabled`]: props.disabled,
        [`${prefixCls}-simple`]: simple.value,
        [`${prefixCls}-${mergedSize.value}`]: true
      });
    });
    return () => {
      const prefixCls = mergedPrefixCls.value;
      const children = showTotal.value ? [createVNode(Total, null, null)] : [];
      if (simple.value) {
        const _activeIndex = activeIndex.value;
        const _lastIndex = lastIndex.value;
        children.push(createVNode(Item, {
          "disabled": _activeIndex === 1,
          "type": "prev"
        }, null));
        children.push(createVNode("li", {
          "class": `${prefixCls}-item`
        }, [showQuickJumper.value ? createVNode(_Input, {
          "disabled": props.disabled,
          "size": mergedSize.value === "lg" ? "md" : "sm",
          "value": _activeIndex.toString(),
          "onKeydown": jumpToIndex
        }, null) : _activeIndex, createVNode("span", {
          "class": `${prefixCls}-item-slash`
        }, [createTextVNode("/")]), createVNode("span", null, [_lastIndex])]));
        children.push(createVNode(Item, {
          "disabled": _activeIndex === _lastIndex,
          "type": "next"
        }, null));
        showSizeChanger.value && children.push(createVNode(Sizes, null, null));
      } else {
        items.value.forEach((item) => children.push(createVNode(Item, mergeProps({
          "key": item.type + "-" + item.index
        }, item), null)));
        showSizeChanger.value && children.push(createVNode(Sizes, null, null));
        showQuickJumper.value && children.push(createVNode(Jumper, null, null));
      }
      return createVNode("ul", {
        "class": classes.value
      }, [children]);
    };
  }
});
const IxPagination = Pagination;
export { IxPagination, getThemeTokens as getPaginationThemeTokens };
