import { defineComponent, inject, computed, createVNode, mergeProps, ref, provide } from "vue";
import { useGlobalConfig } from "@idux/components/config";
import { useThemeToken } from "@idux/components/theme";
import { isFunction, isString, isObject } from "lodash-es";
import { convertNumber, uniqueId } from "@idux/cdk/utils";
import { IxIcon } from "@idux/components/icon";
const progressContext = Symbol("progressContext");
const fullPercent = 100;
function convertPercent(percent) {
  if (!percent || percent < 0) {
    return 0;
  }
  if (percent > fullPercent) {
    return fullPercent;
  }
  return percent;
}
const sortGradient = (gradients) => {
  const tempArr = [];
  Object.keys(gradients).forEach((key) => {
    const formattedKey = parseFloat(key.replace(/%/g, ""));
    if (!isNaN(formattedKey)) {
      tempArr.push({
        key: formattedKey,
        value: gradients[key]
      });
    }
  });
  return tempArr.sort((a, b) => a.key - b.key);
};
const handleGradient = (strokeColor, element) => {
  const {
    from = element ? getComputedStyle(element).getPropertyValue("--ix-color-primary") : "#1c6eff",
    to = element ? getComputedStyle(element).getPropertyValue("--ix-color-primary") : "#1c6eff",
    direction = "to right",
    ...rest
  } = strokeColor;
  if (Object.keys(rest).length !== 0) {
    const sortedGradients = sortGradient(rest).map(({ key, value }) => `${value} ${key}%`).join(", ");
    return `linear-gradient(${direction}, ${sortedGradients})`;
  }
  return `linear-gradient(${direction}, ${from}, ${to})`;
};
const handleCircleGradient = (strokeColor) => {
  return sortGradient(strokeColor).map(({ key, value }) => ({ offset: `${key}%`, color: value }));
};
var ProgressInfo = /* @__PURE__ */ defineComponent({
  setup(_, {
    slots
  }) {
    const {
      props,
      config,
      mergedPrefixCls,
      status,
      percent,
      successPercent
    } = inject(progressContext);
    const formatFn = computed(() => {
      var _a;
      return (_a = props.format) != null ? _a : config.format;
    });
    const formattedText = computed(() => formatFn.value(percent.value, successPercent.value));
    const showSuccessIcon = computed(() => status.value === "success" || status.value === "normal" && percent.value === fullPercent);
    const showExceptionIcon = computed(() => status.value === "exception");
    const showFormat = computed(() => isFunction(props.format) || !(showSuccessIcon.value || showExceptionIcon.value));
    const mergedIconSuccess = computed(() => {
      var _a, _b;
      return (_b = (_a = props.icons) == null ? void 0 : _a.success) != null ? _b : config.icon.success;
    });
    const mergedIconException = computed(() => {
      var _a, _b;
      return (_b = (_a = props.icons) == null ? void 0 : _a.exception) != null ? _b : config.icon.exception;
    });
    const renderInfo = () => {
      if (showFormat.value) {
        return formattedText.value;
      }
      const prefixCls = mergedPrefixCls.value;
      if (showSuccessIcon.value && mergedIconSuccess.value) {
        return renderIcon(mergedIconSuccess.value, `${prefixCls}-success-icon`);
      }
      if (showExceptionIcon.value && mergedIconException.value) {
        return renderIcon(mergedIconException.value, `${prefixCls}-exception-icon`);
      }
      return null;
    };
    return () => {
      var _a;
      if (props.hideInfo) {
        return null;
      }
      const slot = (_a = slots.format) != null ? _a : slots.default;
      return createVNode("div", {
        "class": `${mergedPrefixCls.value}-info`
      }, [slot ? slot({
        percent: percent.value,
        successPercent: successPercent.value
      }) : renderInfo()]);
    };
  }
});
function renderIcon(icon, cls) {
  if (isString(icon)) {
    return createVNode(IxIcon, {
      "class": cls,
      "name": icon
    }, null);
  }
  return icon;
}
const defaultStrokeWidth = 6;
var Circle = /* @__PURE__ */ defineComponent({
  name: "IxProgressCircle",
  setup(_, {
    slots
  }) {
    const {
      props,
      mergedPrefixCls,
      mergedStrokeLinecap,
      percent,
      successPercent
    } = inject(progressContext);
    const circleMergedPrefixCls = computed(() => `${mergedPrefixCls.value}-circle`);
    const strokeWidth = computed(() => convertNumber(props.strokeWidth, defaultStrokeWidth));
    const isGradient = computed(() => isObject(props.strokeColor));
    const linearGradientId = computed(() => `${mergedPrefixCls.value}-gradient-${uniqueId()}`);
    const calcSharedProperties = computed(() => {
      var _a, _b;
      const isCircle = props.type === "circle";
      const radius = 50 - strokeWidth.value / 2;
      return {
        isGradient: isGradient.value,
        percent: percent.value,
        linearGradientId: linearGradientId.value,
        radius,
        gapPosition: (_a = props.gapPosition) != null ? _a : isCircle ? "top" : "bottom",
        len: Math.PI * 2 * radius,
        gapDegree: convertNumber((_b = props.gapDegree) != null ? _b : isCircle ? 0 : 75)
      };
    });
    const circleGradient = computed(() => {
      return isGradient.value ? handleCircleGradient(props.strokeColor) : [];
    });
    const pathString = usePathString(calcSharedProperties);
    const trailPathStyle = useTrailPathStyle(calcSharedProperties);
    const strokePath = useCirclePath(calcSharedProperties, circleMergedPrefixCls, props, percent, successPercent);
    const trailPathAttr = computed(() => ({
      stroke: props.trailColor,
      "fill-opacity": "0",
      "stroke-linecap": mergedStrokeLinecap.value,
      "stroke-width": strokeWidth.value,
      d: pathString.value
    }));
    const strokePathAttr = computed(() => ({
      "fill-opacity": "0",
      "stroke-linecap": mergedStrokeLinecap.value,
      "stroke-width": percent.value ? strokeWidth.value : 0,
      d: pathString.value
    }));
    const circleClasses = computed(() => {
      const prefixCls = circleMergedPrefixCls.value;
      return {
        [prefixCls]: true,
        [`${prefixCls}-gradient`]: isGradient.value
      };
    });
    const circleStyle = computed(() => ({
      width: props.width && `${props.width}px`,
      height: props.width && `${props.width}px`,
      fontSize: props.width && `${convertNumber(props.width) * 0.15 + 6}px`
    }));
    const renderDefs = () => {
      if (!isGradient.value) {
        return null;
      }
      return createVNode("defs", null, [createVNode("linearGradient", {
        "id": linearGradientId.value,
        "x1": "100%",
        "y1": "0%",
        "x2": "0%",
        "y2": "0%"
      }, [circleGradient.value.map((gradient) => createVNode("stop", {
        "offset": gradient.offset,
        "stop-color": gradient.color
      }, null))])]);
    };
    const renderStrokePath = () => strokePath.value.map((p) => createVNode("path", mergeProps({
      "class": [`${circleMergedPrefixCls.value}-path`, ...p.strokeClasses],
      "style": p.strokePathStyle,
      "stroke": p.stroke
    }, strokePathAttr.value), null));
    return () => createVNode("div", {
      "class": circleClasses.value,
      "style": circleStyle.value
    }, [createVNode("svg", {
      "viewBox": "0 0 100 100"
    }, [renderDefs(), createVNode("path", mergeProps({
      "class": `${circleMergedPrefixCls.value}-trail`,
      "style": trailPathStyle.value
    }, trailPathAttr.value), null), renderStrokePath()]), createVNode(ProgressInfo, null, slots)]);
  }
});
function usePathString(calcSharedProperties) {
  return computed(() => {
    const {
      radius,
      gapPosition
    } = calcSharedProperties.value;
    let beginPositionX = 0;
    let beginPositionY = -radius;
    let endPositionX = 0;
    let endPositionY = radius * -2;
    switch (gapPosition) {
      case "left":
        beginPositionX = -radius;
        beginPositionY = 0;
        endPositionX = radius * 2;
        endPositionY = 0;
        break;
      case "right":
        beginPositionX = radius;
        beginPositionY = 0;
        endPositionX = radius * -2;
        endPositionY = 0;
        break;
      case "bottom":
        beginPositionY = radius;
        endPositionY = radius * 2;
        break;
    }
    return `M 50,50 m ${beginPositionX},${beginPositionY}
       a ${radius},${radius} 0 1 1 ${endPositionX},${-endPositionY}
       a ${radius},${radius} 0 1 1 ${-endPositionX},${endPositionY}`;
  });
}
function useTrailPathStyle(calcSharedProperties) {
  return computed(() => {
    const {
      len,
      gapDegree
    } = calcSharedProperties.value;
    return {
      strokeDasharray: `${len - gapDegree}px ${len}px`,
      strokeDashoffset: `-${gapDegree / 2}px`,
      transition: "stroke-dashoffset .3s ease 0s, stroke-dasharray .3s ease 0s, stroke .3s"
    };
  });
}
function useCirclePath(calcSharedProperties, circleMergedPrefixCls, props, percent, successPercent) {
  return computed(() => {
    var _a;
    const {
      gapDegree,
      len,
      isGradient,
      linearGradientId
    } = calcSharedProperties.value;
    const strokeProgress = successPercent.value > 0 ? [successPercent.value, percent.value] : [percent.value];
    const successColor = (_a = props.success) == null ? void 0 : _a.strokeColor;
    return strokeProgress.map((value, index) => {
      const hasSuccessPercent = strokeProgress.length > 1 && index === 0;
      return {
        stroke: isGradient && !hasSuccessPercent ? `url(#${linearGradientId})` : void 0,
        strokeClasses: [!isGradient && hasSuccessPercent ? `${circleMergedPrefixCls.value}-success` : "", isGradient ? "" : `${circleMergedPrefixCls.value}-bg`],
        strokePathStyle: {
          stroke: !isGradient ? hasSuccessPercent ? successColor : props.strokeColor : void 0,
          transition: "stroke-dashoffset .3s ease 0s, stroke-dasharray .3s ease 0s, stroke .3s, stroke-width .06s ease .3s",
          strokeDasharray: `${value / 100 * (len - gapDegree)}px ${len}px`,
          strokeDashoffset: `-${gapDegree / 2}px`
        }
      };
    }).reverse();
  });
}
var Line = /* @__PURE__ */ defineComponent({
  name: "IxProgressLine",
  setup(_, {
    slots
  }) {
    const {
      props,
      mergedPrefixCls,
      mergedSize,
      mergedStrokeLinecap,
      percent,
      successPercent
    } = inject(progressContext);
    const lineMergedPrefixCls = computed(() => `${mergedPrefixCls.value}-line`);
    const elementRef = ref();
    const classes = computed(() => {
      const prefixCls = lineMergedPrefixCls.value;
      return {
        [prefixCls]: true,
        [`${prefixCls}-${mergedSize.value}`]: true,
        [`${prefixCls}-round`]: mergedStrokeLinecap.value === "round"
      };
    });
    const innerStyle = computed(() => ({
      background: props.trailColor
    }));
    const successStyle = computed(() => {
      var _a;
      return {
        height: `${props.strokeWidth}px`,
        width: `${successPercent.value}%`,
        background: (_a = props.success) == null ? void 0 : _a.strokeColor
      };
    });
    const bgStyle = computed(() => ({
      height: `${props.strokeWidth}px`,
      width: `${percent.value}%`,
      background: isObject(props.strokeColor) ? handleGradient(props.strokeColor, elementRef.value) : props.strokeColor
    }));
    return () => {
      const prefixCls = lineMergedPrefixCls.value;
      return createVNode("div", {
        "ref": elementRef,
        "class": classes.value
      }, [createVNode("div", {
        "class": `${prefixCls}-outer`
      }, [createVNode("div", {
        "class": `${prefixCls}-inner`,
        "style": innerStyle.value
      }, [!!successPercent.value && createVNode("div", {
        "class": `${prefixCls}-success-bg`,
        "style": successStyle.value
      }, null), createVNode("div", {
        "class": `${prefixCls}-bg`,
        "style": bgStyle.value
      }, null)])]), createVNode(ProgressInfo, null, slots)]);
    };
  }
});
const progressStatus = ["normal", "success", "exception", "active"];
const progressProps = {
  type: {
    type: String,
    default: "line"
  },
  format: Function,
  percent: {
    type: Number,
    default: 0
  },
  status: String,
  hideInfo: {
    type: Boolean,
    default: false
  },
  success: Object,
  trailColor: String,
  strokeColor: [String, Object],
  strokeLinecap: String,
  strokeWidth: Number,
  gapDegree: Number,
  gapPosition: String,
  width: {
    type: Number,
    default: 132
  },
  size: String,
  icons: Object
};
function getDefaultThemeTokens(tokens) {
  const { colorEmphasizedContainerBg, fontSizeSm, fontSizeMd, fontSize2xl, paddingSizeSm } = tokens;
  return {
    trailBgColor: colorEmphasizedContainerBg,
    textWidth: 36,
    borderRadius: 100,
    lineSizeSm: 2,
    lineSizeMd: 6,
    lineSizeLg: 8,
    lineFontSizeSm: fontSizeSm,
    lineFontSizeMd: fontSizeSm,
    lineFontSizeLg: fontSizeMd,
    lineTextPadding: `0 0 0 ${paddingSizeSm}px`,
    circleWidth: 120,
    circleFontSize: fontSize2xl
  };
}
const getThemeTokens = (tokens, presetTheme) => {
  return presetTheme === "default" ? getDefaultThemeTokens(tokens) : getDefaultThemeTokens(tokens);
};
var Progress = /* @__PURE__ */ defineComponent({
  name: "IxProgress",
  props: progressProps,
  setup(props, {
    slots
  }) {
    const {
      globalHashId,
      hashId,
      registerToken
    } = useThemeToken("progress");
    registerToken(getThemeTokens);
    const config = useGlobalConfig("progress");
    const common = useGlobalConfig("common");
    const mergedPrefixCls = computed(() => `${common.prefixCls}-progress`);
    const percent = computed(() => convertPercent(props.percent));
    const successPercent = computed(() => {
      var _a;
      return convertPercent((_a = props.success) == null ? void 0 : _a.percent);
    });
    const mergedSize = computed(() => {
      var _a;
      return (_a = props.size) != null ? _a : config.size;
    });
    const mergedStrokeLinecap = computed(() => {
      var _a;
      return (_a = props.strokeLinecap) != null ? _a : config.strokeLinecap;
    });
    const status = computed(() => {
      var _a;
      if (!progressStatus.includes(props.status) && (percent.value >= fullPercent || successPercent.value >= fullPercent)) {
        return "success";
      }
      return (_a = props.status) != null ? _a : "normal";
    });
    const classes = computed(() => {
      const prefixCls = mergedPrefixCls.value;
      return {
        [globalHashId.value]: !!globalHashId.value,
        [hashId.value]: !!hashId.value,
        [prefixCls]: true,
        [`${prefixCls}-${status.value}`]: !!status.value
      };
    });
    provide(progressContext, {
      props,
      config,
      mergedPrefixCls,
      mergedSize,
      mergedStrokeLinecap,
      percent,
      successPercent,
      status
    });
    return () => {
      const ProgressComponent = props.type === "line" ? Line : Circle;
      return createVNode(ProgressComponent, {
        "class": classes.value
      }, slots);
    };
  }
});
const IxProgress = Progress;
export { IxProgress, getThemeTokens as getProgressThemeTokens };
