import { defineComponent, ref, computed, createVNode, normalizeClass } from "vue";
import { useAccessorAndControl } from "@idux/cdk/forms";
import { convertNumber, callEmit } from "@idux/cdk/utils";
import { useGlobalConfig } from "@idux/components/config";
import { useFormSize, useFormElement, useFormItemRegister } from "@idux/components/form";
import { IxIcon } from "@idux/components/icon";
import { useThemeToken } from "@idux/components/theme";
import { IxTooltip } from "@idux/components/tooltip";
const rateProps = {
  control: {
    type: [String, Number, Object, Array],
    default: void 0
  },
  value: [Number, String],
  allowHalf: {
    type: Boolean,
    default: void 0
  },
  clearable: {
    type: Boolean,
    default: void 0
  },
  count: [Number, String],
  disabled: {
    type: Boolean,
    default: false
  },
  icon: String,
  tooltips: {
    type: Array,
    default: () => []
  },
  size: String,
  color: {
    type: String,
    default: void 0
  },
  "onUpdate:value": [Function, Array],
  onChange: [Function, Array],
  onBlur: [Function, Array],
  onFocus: [Function, Array],
  onKeyDown: [Function, Array]
};
const rateItemProps = {
  count: {
    type: Number,
    required: true
  },
  disabled: {
    type: Boolean,
    required: true
  },
  focused: {
    type: Boolean,
    required: true
  },
  index: {
    type: Number,
    required: true
  },
  prefixCls: {
    type: String,
    required: true
  },
  tooltip: String,
  value: {
    type: Number,
    required: true
  },
  color: String,
  onClick: {
    type: Function,
    required: true
  },
  onMouseMove: {
    type: Function,
    required: true
  }
};
var RateItem = /* @__PURE__ */ defineComponent({
  props: rateItemProps,
  setup(props, {
    slots
  }) {
    const liRef = ref();
    const handleClick = (evt) => props.onClick(evt, liRef.value, props.index);
    const handleKeyDown = (evt) => {
      if (evt.code === "Enter") {
        handleClick(evt);
      }
    };
    const handleMouseMove = (evt) => props.onMouseMove(evt, liRef.value, props.index);
    const classes = computed(() => {
      const {
        focused,
        index,
        prefixCls,
        value
      } = props;
      const itemValue = index + 1;
      const isHalf = value + 0.5 >= itemValue && value < itemValue;
      const isFull = !isHalf && itemValue <= value;
      const isZero = !isFull && !isHalf;
      return {
        [prefixCls]: true,
        [`${prefixCls}-focused`]: focused && (isHalf || value === itemValue || index === 0 && value === 0),
        [`${prefixCls}-full`]: isFull,
        [`${prefixCls}-half`]: isHalf,
        [`${prefixCls}-zero`]: isZero
      };
    });
    return () => {
      const {
        count,
        disabled,
        index,
        prefixCls,
        tooltip,
        value,
        color
      } = props;
      const iconNode = slots.default();
      const itemNode = createVNode("li", {
        "ref": liRef,
        "class": classes.value,
        "style": `color: ${color}`
      }, [createVNode("span", {
        "aria-checked": value > index,
        "aria-posinset": index + 1,
        "aria-setsize": count,
        "role": "radio",
        "tabindex": disabled ? -1 : 0,
        "onClick": disabled ? void 0 : handleClick,
        "onKeydown": disabled ? void 0 : handleKeyDown,
        "onMousemove": disabled ? void 0 : handleMouseMove
      }, [createVNode("span", {
        "class": `${prefixCls}-first`
      }, [iconNode]), createVNode("span", {
        "class": `${prefixCls}-second`
      }, [iconNode])])]);
      return tooltip ? createVNode(IxTooltip, {
        "title": tooltip
      }, {
        default: () => [itemNode]
      }) : itemNode;
    };
  }
});
function getDefaultThemeTokens(tokens, algorithms) {
  const { getColorPalette, getBaseColors } = algorithms;
  const { marginSizeSm } = tokens;
  const activeColor = getColorPalette(getBaseColors().brown).l10;
  return {
    activeColor,
    sizeSm: 14,
    sizeMd: 20,
    sizeLg: 28,
    itemMarginRight: marginSizeSm,
    itemScaleHover: 1.1,
    itemOutlineFocus: `1px dashed ${activeColor}`
  };
}
const getThemeTokens = (tokens, presetTheme, algorithms) => {
  return presetTheme === "default" ? getDefaultThemeTokens(tokens, algorithms) : getDefaultThemeTokens(tokens, algorithms);
};
var Rate = /* @__PURE__ */ defineComponent({
  name: "IxRate",
  props: rateProps,
  setup(props, {
    attrs,
    expose,
    slots
  }) {
    const common = useGlobalConfig("common");
    const {
      globalHashId,
      hashId,
      registerToken
    } = useThemeToken("rate");
    registerToken(getThemeTokens);
    const mergedPrefixCls = computed(() => `${common.prefixCls}-rate`);
    const config = useGlobalConfig("rate");
    const mergedSize = useFormSize(props, config);
    const {
      elementRef,
      focus,
      blur
    } = useFormElement();
    expose({
      focus,
      blur
    });
    const {
      accessor,
      control
    } = useAccessorAndControl();
    useFormItemRegister(control);
    const countRef = computed(() => {
      var _a;
      return convertNumber((_a = props.count) != null ? _a : config.count);
    });
    const iconRef = computed(() => {
      var _a;
      return (_a = props.icon) != null ? _a : config.icon;
    });
    const allowHalfRef = computed(() => {
      var _a;
      return (_a = props.allowHalf) != null ? _a : config.allowHalf;
    });
    const clearableRef = computed(() => {
      var _a;
      return (_a = props.clearable) != null ? _a : config.clearable;
    });
    const isDisabled = computed(() => accessor.disabled);
    const isFocused = ref(false);
    const hoverValue = ref();
    const changeValue = (value) => {
      accessor.setValue(value);
      callEmit(props.onChange, value);
    };
    const calcValue = (evt, element, index) => {
      let value = index + 1;
      if (allowHalfRef.value) {
        if (evt.offsetX < element.clientWidth / 2) {
          value -= 0.5;
        }
      }
      return value;
    };
    const handleItemClick = (evt, element, index) => {
      const currValue = accessor.value;
      const newValue = calcValue(evt, element, index);
      const isClear = clearableRef.value && currValue === newValue;
      changeValue(isClear ? 0 : newValue);
    };
    const handleItemMouseMove = (evt, element, index) => {
      const newValue = calcValue(evt, element, index);
      hoverValue.value = newValue;
    };
    const handleFocus = (evt) => {
      isFocused.value = true;
      callEmit(props.onFocus, evt);
    };
    const handleBlur = (evt) => {
      isFocused.value = false;
      accessor.markAsBlurred();
      callEmit(props.onBlur, evt);
    };
    const handleKeyDown = (evt) => {
      const currValue = accessor.value;
      const currCount = countRef.value;
      const currAllowHalf = allowHalfRef.value;
      const eventCode = evt.code;
      switch (eventCode) {
        case "ArrowRight":
        case "ArrowUp":
          {
            if (currValue < currCount) {
              changeValue(currValue + (currAllowHalf ? 0.5 : 1));
              evt.preventDefault();
            }
          }
          break;
        case "ArrowLeft":
        case "ArrowDown":
          if (currValue > 0) {
            changeValue(currValue - (currAllowHalf ? 0.5 : 1));
            evt.preventDefault();
          }
          break;
      }
    };
    const handleMouseLeave = () => {
      hoverValue.value = void 0;
    };
    const classes = computed(() => {
      const prefixCls = mergedPrefixCls.value;
      return normalizeClass({
        [globalHashId.value]: !!globalHashId.value,
        [hashId.value]: !!hashId.value,
        [prefixCls]: true,
        [`${prefixCls}-disabled`]: isDisabled.value,
        [`${prefixCls}-focused`]: isFocused.value,
        [`${prefixCls}-${mergedSize.value}`]: true
      });
    });
    return () => {
      var _a;
      const count = countRef.value;
      const disabled = isDisabled.value;
      const focused = isFocused.value;
      const itemValue = convertNumber((_a = hoverValue.value) != null ? _a : accessor.value);
      const itemPrefixCls = `${mergedPrefixCls.value}-item`;
      const {
        tooltips,
        color
      } = props;
      const children = [];
      for (let index = 0; index < count; index++) {
        children.push(createVNode(RateItem, {
          "key": index,
          "count": count,
          "disabled": disabled,
          "focused": focused,
          "index": index,
          "prefixCls": itemPrefixCls,
          "tooltip": tooltips[index],
          "color": color,
          "value": itemValue,
          "onClick": handleItemClick,
          "onMouseMove": handleItemMouseMove
        }, {
          default: () => {
            var _a2, _b;
            return [(_b = (_a2 = slots.icon) == null ? void 0 : _a2.call(slots, {
              disabled,
              focused,
              index
            })) != null ? _b : createVNode(IxIcon, {
              "name": iconRef.value
            }, null)];
          }
        }));
      }
      return createVNode("ul", {
        "ref": elementRef,
        "class": classes.value,
        "tabindex": disabled ? -1 : attrs.tabindex,
        "onFocus": disabled ? void 0 : handleFocus,
        "onBlur": disabled ? void 0 : handleBlur,
        "onKeydown": disabled ? void 0 : handleKeyDown,
        "onMouseleave": disabled ? void 0 : handleMouseLeave,
        "role": "radiogroup"
      }, [children]);
    };
  }
});
const IxRate = Rate;
export { IxRate, getThemeTokens as getRateThemeTokens };
