import { watch, computed, toRaw, inject, createVNode, defineComponent, mergeProps, ref, onMounted, provide, normalizeClass, watchEffect } from "vue";
import { isString, isNil, isFunction, toString, isBoolean } from "lodash-es";
import { useAccessorAndControl } from "@idux/cdk/forms";
import { useState, Logger, flattenNode, convertArray, callEmit, useControlledProp, convertCssPixel } from "@idux/cdk/utils";
import { \u0275Input as _Input } from "@idux/components/_private/input";
import { useGlobalConfig } from "@idux/components/config";
import { IxControlTrigger } from "@idux/components/control-trigger";
import { useFormItemRegister, useFormStatus } from "@idux/components/form";
import { IxSelector } from "@idux/components/selector";
import { IxSpin } from "@idux/components/spin";
import { useThemeToken } from "@idux/components/theme";
import { useMergedCommonControlProps } from "@idux/components/utils";
import { CdkDndSortableItem, CdkDndSortableHandle, useDndAutoScroll, CdkDndSortable } from "@idux/cdk/dnd";
import { CdkVirtualScroll } from "@idux/cdk/scroll";
import { \u0275Empty as _Empty } from "@idux/components/_private/empty";
import { IxCheckbox } from "@idux/components/checkbox";
import { IxIcon } from "@idux/components/icon";
function useActiveState(props, inputValue) {
  const [activeValue, setActiveValue] = useState(void 0);
  watch(
    inputValue,
    (value) => {
      if (!props.allowInput || !value) {
        return;
      }
      setActiveValue(value);
    },
    {
      immediate: true
    }
  );
  return {
    activeValue,
    setActiveValue
  };
}
function useGetOptionKey(props, config) {
  return computed(() => genGetKeyFn(props.labelKey || config.labelKey, props.getKey || config.getKey));
}
function usePanelGetOptionKey(props, config) {
  return computed(() => genGetKeyFn(props.labelKey || config.labelKey, props.getKey || config.getKey));
}
function genGetKeyFn(labelKey, getKeyFn) {
  return isString(getKeyFn) ? (record) => {
    let key = record[getKeyFn];
    if (key === void 0 && labelKey !== "value") {
      key = record["value"];
    }
    if (process.env.NODE_ENV !== "production" && key === void 0) {
      Logger.warn("components/select", "Each record in dataSource should have a unique `key` prop.");
    }
    return key;
  } : getKeyFn;
}
function useKeyboardEvents(props, inputValue, selectedValue, activeValue, overlayOpened, changeActiveIndex, changeSelected, handleRemove, clearInput, setOverlayOpened) {
  return (evt) => {
    switch (evt.code) {
      case "ArrowUp":
        evt.preventDefault();
        changeActiveIndex(-1);
        break;
      case "ArrowDown":
        evt.preventDefault();
        changeActiveIndex(1);
        break;
      case "Enter": {
        evt.preventDefault();
        const key = activeValue.value;
        if (!isNil(key) && (props.allowInput && inputValue.value || overlayOpened.value)) {
          changeSelected(key);
        }
        if (!props.multiple && (overlayOpened.value || props.allowInput && inputValue.value)) {
          setOverlayOpened(false);
          evt.stopImmediatePropagation();
        }
        (props.allowInput || !props.multiple) && clearInput();
        break;
      }
      case "Backspace": {
        const selectedLength = selectedValue.value.length;
        if (!inputValue.value && selectedLength) {
          handleRemove(selectedValue.value[selectedLength - 1]);
        }
        break;
      }
      case "Escape": {
        evt.preventDefault();
        setOverlayOpened(false);
        break;
      }
    }
  };
}
const optionKey = Symbol("IxSelectOption");
const SelectOption = () => {
};
SelectOption.displayName = "IxSelectOption";
SelectOption[optionKey] = true;
const optionGroupKey = Symbol("IxSelectOptionGroup");
const SelectOptionGroup = () => {
};
SelectOptionGroup.displayName = "IxSelectOptionGroup";
SelectOptionGroup[optionGroupKey] = true;
function flattenOptions(options, childrenKey, getKeyFn, labelKey) {
  const flattenedOptions = [];
  const appendOption = (item, index, parentKey) => {
    const parsedOption = parseOption(item, (item2) => {
      var _a;
      return (_a = getKeyFn(item2)) != null ? _a : index;
    }, childrenKey, labelKey, parentKey);
    flattenedOptions.push(parsedOption);
    return parsedOption.key;
  };
  options == null ? void 0 : options.forEach((item, index) => {
    const children = item[childrenKey];
    const optionKey2 = appendOption(item, index);
    if (children && children.length > 0) {
      children.forEach((child) => {
        appendOption(child, void 0, optionKey2);
      });
    }
  });
  return flattenedOptions;
}
function unFlattenOptions(flattenedOptions, childrenKey, getKeyFn) {
  const options = [];
  let currentGroupKeys;
  flattenedOptions.forEach((option) => {
    const data = option.rawData;
    if (option.type === "group") {
      options.push(data);
      currentGroupKeys = new Set(data[childrenKey].map((item) => getKeyFn(item)));
    } else if (!(currentGroupKeys == null ? void 0 : currentGroupKeys.has(option.key))) {
      options.push(data);
    }
  });
  return options;
}
function parseOption(option, getKey, childrenKey, labelKey, parentKey) {
  const children = option[childrenKey];
  return {
    key: getKey(option),
    parentKey,
    label: option[labelKey],
    disabled: !!option.disabled,
    type: children && children.length > 0 ? "group" : "item",
    rawData: toRaw(option)
  };
}
function generateOption(value) {
  const rawData = { key: value, label: value };
  return { key: value, label: value, type: "item", rawData };
}
function useConvertedOptions(props, slots) {
  return computed(() => {
    var _a, _b;
    return (_b = props.dataSource) != null ? _b : convertOptions((_a = slots.default) == null ? void 0 : _a.call(slots));
  });
}
function useOptionKeyMap(options) {
  return computed(() => {
    const map = /* @__PURE__ */ new Map();
    options.value.forEach((option) => {
      map.set(option.key, option);
    });
    return map;
  });
}
function useFlattenedOptions(options, mergedChildrenKey, getKey, mergedLabelKey) {
  return computed(() => flattenOptions(options.value, mergedChildrenKey.value, getKey.value, mergedLabelKey.value));
}
const filterKeys = [optionKey, optionGroupKey];
function convertOptions(nodes, parentKey) {
  const convertedOptions = [];
  flattenNode(nodes, { key: filterKeys }).forEach((node, index) => {
    var _a, _b;
    const type = node.type;
    const slots = (_a = node.children) != null ? _a : {};
    const props = (_b = node.props) != null ? _b : {};
    const isOption = type[optionKey];
    if (isOption) {
      const { key, disabled, label, value } = props;
      const { label: customLabel, default: customLabel2 } = slots;
      const _disabled = disabled || disabled === "";
      const option = {
        key: value != null ? value : key,
        disabled: _disabled,
        label,
        value,
        parentKey,
        customLabel: customLabel != null ? customLabel : customLabel2
      };
      convertedOptions.push(option);
    } else {
      const { key = index, label, children } = props;
      const { label: customLabel, default: defaultSlot } = slots;
      const _children = children != null ? children : convertOptions(defaultSlot == null ? void 0 : defaultSlot(), key);
      convertedOptions.push({ key, label, children: _children, customLabel });
    }
  });
  return convertedOptions;
}
function useFilteredOptions(props, options, inputValue, mergedLabelKey) {
  const searchFilter = useSearchFn(props, mergedLabelKey);
  return computed(() => {
    const _options = options.value;
    const searchValue = inputValue.value;
    if (!searchValue) {
      return _options;
    }
    const filter = searchFilter.value;
    const filteredOptions = filterOptions(_options, props.allowInput, searchValue, filter);
    return filteredOptions;
  });
}
function filterOptions(flattenedOptions, allowInput, searchValue, filter) {
  const filteredOptions = !filter ? flattenedOptions : flattenedOptions.filter((option) => filter(option.rawData, searchValue));
  if (allowInput) {
    const matchedOption = filteredOptions.find((option) => option.label === searchValue);
    if (!matchedOption) {
      return [generateOption(searchValue), ...filteredOptions];
    }
  }
  return filteredOptions;
}
function useSearchFn(props, mergedLabelKey) {
  return computed(() => {
    const searchFn = props.searchFn;
    if (isFunction(searchFn)) {
      return searchFn;
    }
    return searchFn ? getDefaultSearchFn(mergedLabelKey.value) : false;
  });
}
function getDefaultSearchFn(labelKey) {
  return (data, searchValue) => {
    const label = data[labelKey];
    const { children } = data;
    const hasChildrenMatch = children ? children.some((item) => matchRule(item.label, searchValue)) : false;
    return matchRule(String(label), searchValue) && !children || hasChildrenMatch;
  };
}
function matchRule(srcString, targetString) {
  return !isNil(srcString) && String(srcString).toLowerCase().includes(targetString.toLowerCase());
}
function usePanelProps(props, valueRef, activeValue, setActiveValue, onOptionClick) {
  return computed(() => ({
    activeValue: activeValue.value,
    selectedKeys: convertArray(valueRef.value),
    childrenKey: props.childrenKey,
    customAdditional: props.customAdditional,
    empty: props.empty,
    dndSortable: props.dndSortable,
    getKey: props.getKey,
    labelKey: props.labelKey,
    multiple: props.multiple,
    multipleLimit: props.multipleLimit,
    virtual: props.virtual,
    virtualItemHeight: props.virtualItemHeight,
    virtualScrollMode: props.virtualScrollMode,
    "onUpdate:activeValue": setActiveValue,
    onOptionClick,
    onScroll: props.onScroll,
    onScrolledChange: props.onScrolledChange,
    onScrolledBottom: props.onScrolledBottom,
    onDndSortReorder: props.onDndSortReorder,
    onDndSortChange: props.onDndSortChange,
    _virtualScrollHeight: props.overlayHeight
  }));
}
function useSelectedState$1(props, accessor, optionKeyMap) {
  const selectedValue = computed(() => convertArray(accessor.value));
  const selectedOptions = computed(
    () => selectedValue.value.map((value) => {
      var _a;
      return (_a = optionKeyMap.value.get(value)) != null ? _a : generateOption(value);
    })
  );
  const setSelectedValue = (value) => {
    const currValue = props.multiple ? value : value[0];
    const oldValue = toRaw(accessor.value);
    if (currValue !== oldValue) {
      accessor.setValue(currValue);
      callEmit(props.onChange, currValue, oldValue);
    }
  };
  const changeSelected = (key) => {
    const multiple = !!props.multiple;
    const currValue = selectedValue.value;
    const targetIndex = currValue.findIndex((item) => item === key);
    const isSelected = targetIndex > -1;
    if (!multiple) {
      !isSelected && setSelectedValue([key]);
      return;
    }
    if (isSelected) {
      setSelectedValue(currValue.filter((_, index) => targetIndex !== index));
      return;
    }
    if (selectedValue.value.length < props.multipleLimit) {
      setSelectedValue([...currValue, key]);
    }
  };
  const handleRemove = (key) => {
    setSelectedValue(selectedValue.value.filter((item) => key !== item));
  };
  const handleClear = (evt) => {
    setSelectedValue([]);
    callEmit(props.onClear, evt);
  };
  return {
    selectedValue,
    selectedOptions,
    changeSelected,
    handleRemove,
    handleClear
  };
}
const SELECT_PANEL_DATA_TOKEN = Symbol("SELECT_PANEL_DATA_TOKEN");
const selectPanelContext = Symbol("selectPanelContext");
function renderOptionLabel(slots, rawData, label) {
  var _a;
  const labelRender = (_a = rawData.customLabel) != null ? _a : "optionLabel";
  const labelSlot = isString(labelRender) ? slots[labelRender] : labelRender;
  return labelSlot ? labelSlot(rawData) : label;
}
const defaultStyle = {
  height: 0,
  width: 0,
  overflow: "hidden"
};
const ListBox = (_, {
  slots
}) => {
  const {
    selectedKeys,
    activeIndex,
    flattenedOptions
  } = inject(selectPanelContext);
  const currSelectedKeys = selectedKeys.value;
  return createVNode("div", {
    "role": "listbox",
    "style": defaultStyle
  }, [renderOption(slots, flattenedOptions.value[activeIndex.value - 1], currSelectedKeys), renderOption(slots, flattenedOptions.value[activeIndex.value], currSelectedKeys), renderOption(slots, flattenedOptions.value[activeIndex.value + 1], currSelectedKeys)]);
};
const renderOption = (slots, option, selectedValue) => {
  if (!option) {
    return void 0;
  }
  const {
    key,
    label,
    rawData
  } = option;
  const selected = selectedValue === key;
  return createVNode("div", {
    "key": key,
    "role": "option",
    "aria-label": label,
    "aria-selected": selected
  }, [renderOptionLabel(slots, rawData, label)]);
};
const selectPanelProps = {
  activeValue: { type: [String, Number, Symbol], default: void 0 },
  selectedKeys: { type: Array, default: void 0 },
  childrenKey: { type: String, default: void 0 },
  customAdditional: { type: Function, default: void 0 },
  dataSource: { type: Array, default: void 0 },
  dndSortable: { type: [Boolean, Object], default: false },
  empty: { type: [String, Object], default: "simple" },
  getKey: { type: [String, Function], default: void 0 },
  labelKey: { type: String, default: void 0 },
  multiple: { type: Boolean, default: false },
  multipleLimit: { type: Number, default: Number.MAX_SAFE_INTEGER },
  virtual: { type: Boolean, default: false },
  virtualScrollMode: { type: String, default: void 0 },
  virtualItemHeight: { type: Number, default: 32 },
  "onUpdate:activeValue": [Function, Array],
  onOptionClick: [Function, Array],
  onScroll: [Function, Array],
  onScrolledChange: [Function, Array],
  onScrolledBottom: [Function, Array],
  onDndSortReorder: [Function, Array],
  onDndSortChange: [Function, Array],
  _virtualScrollHeight: { type: [Number, String], default: 256 },
  _onDragStart: Function,
  _onDrop: Function
};
const selectProps = {
  control: {
    type: [String, Number, Object, Array],
    default: void 0
  },
  value: { type: [String, Number, Symbol, Array], default: void 0 },
  open: { type: Boolean, default: void 0 },
  allowInput: { type: Boolean, default: false },
  autocomplete: { type: String, default: "off" },
  autofocus: { type: Boolean, default: false },
  borderless: { type: Boolean, default: void 0 },
  childrenKey: { type: String, default: void 0 },
  clearable: { type: Boolean, default: false },
  clearIcon: { type: String, default: void 0 },
  customAdditional: { type: Object, default: void 0 },
  dataSource: { type: Array, default: void 0 },
  dndSortable: { type: [Boolean, Object], default: false },
  disabled: { type: Boolean, default: false },
  empty: { type: [String, Object], default: "simple" },
  getKey: { type: [String, Function], default: void 0 },
  labelKey: { type: String, default: void 0 },
  maxLabel: { type: [Number, String], default: Number.MAX_SAFE_INTEGER },
  multiple: { type: Boolean, default: false },
  multipleLimit: { type: Number, default: Number.MAX_SAFE_INTEGER },
  offset: Array,
  overlayClassName: { type: String, default: void 0 },
  overlayContainer: {
    type: [String, HTMLElement, Function],
    default: void 0
  },
  overlayTabindex: { type: Number, default: void 0 },
  overlayMatchWidth: { type: [Boolean, String], default: void 0 },
  overlayRender: { type: Function, default: void 0 },
  placeholder: { type: String, default: void 0 },
  readonly: { type: Boolean, default: false },
  searchable: { type: [Boolean, String], default: false },
  searchFn: { type: [Boolean, Function], default: true },
  searchPlaceholder: { type: String, default: void 0 },
  size: { type: String, default: void 0 },
  status: String,
  suffix: { type: String, default: void 0 },
  spin: { type: [Boolean, Object], default: void 0 },
  virtual: { type: Boolean, default: false },
  virtualScrollMode: { type: String, default: void 0 },
  virtualItemHeight: { type: Number, default: void 0 },
  "onUpdate:value": [Function, Array],
  "onUpdate:open": [Function, Array],
  onChange: [Function, Array],
  onClear: [Function, Array],
  onFocus: [Function, Array],
  onBlur: [Function, Array],
  onSearch: [Function, Array],
  onScroll: [Function, Array],
  onScrolledChange: [Function, Array],
  onScrolledBottom: [Function, Array],
  onDndSortReorder: [Function, Array],
  onDndSortChange: [Function, Array],
  overlayHeight: { type: Number, default: 256 }
};
const optionProps = {
  disabled: { type: Boolean, default: false },
  index: { type: Number, required: true },
  label: { type: [String, Number], default: void 0 },
  optionKey: { type: [String, Number, Symbol], required: true },
  rawData: { type: Object, required: true },
  parentKey: { type: [String, Number, Symbol], default: void 0 }
};
const optionGroupProps = {
  label: { type: [String, Number], default: void 0 },
  index: { type: Number, required: true },
  rawData: { type: Object, required: true }
};
var Option = /* @__PURE__ */ defineComponent({
  props: optionProps,
  setup(props, {
    slots
  }) {
    const {
      props: selectPanelProps2,
      mergedPrefixCls,
      mergedDndSortable,
      selectedKeys,
      selectedLimit,
      selectedLimitTitle,
      activeValue,
      setActiveIndex
    } = inject(selectPanelContext);
    const isActive = computed(() => activeValue.value === props.optionKey);
    const isSelected = computed(() => selectedKeys.value.some((selectedKey) => selectedKey === props.optionKey));
    const isDisabled = computed(() => props.disabled || !isSelected.value && selectedLimit.value);
    const classes = computed(() => {
      const {
        parentKey
      } = props;
      const prefixCls = `${mergedPrefixCls.value}-option`;
      return {
        [prefixCls]: true,
        [`${prefixCls}-active`]: isActive.value,
        [`${prefixCls}-with-drag-handle`]: mergedDndSortable.value && mergedDndSortable.value.dragHandle,
        [`${prefixCls}-disabled`]: isDisabled.value,
        [`${prefixCls}-grouped`]: !isNil(parentKey),
        [`${prefixCls}-selected`]: isSelected.value
      };
    });
    const handleMouseEnter = () => setActiveIndex(props.index);
    const handleClick = (evt) => {
      callEmit(selectPanelProps2.onOptionClick, props.rawData, evt);
    };
    return () => {
      const {
        label,
        rawData
      } = props;
      const {
        multiple
      } = selectPanelProps2;
      const selected = isSelected.value;
      const disabled = isDisabled.value;
      const prefixCls = `${mergedPrefixCls.value}-option`;
      const _label = toString(label);
      const title = disabled && selectedLimitTitle.value || _label;
      const customAdditional = selectPanelProps2.customAdditional ? selectPanelProps2.customAdditional({
        data: rawData,
        index: props.index
      }) : void 0;
      const optionContentNodes = [multiple && createVNode(IxCheckbox, {
        "checked": selected,
        "disabled": disabled
      }, null), createVNode("span", {
        "class": `${prefixCls}-label`
      }, [renderOptionLabel(slots, rawData, _label)])];
      const optionsProps = {
        class: classes.value,
        title,
        onMouseenter: disabled ? void 0 : handleMouseEnter,
        onClick: disabled ? void 0 : handleClick,
        "aria-label": _label,
        "aria-selected": selected,
        ...customAdditional
      };
      return mergedDndSortable.value ? createVNode(CdkDndSortableItem, mergeProps({
        "itemKey": props.optionKey,
        "canDrag": !isDisabled.value
      }, optionsProps), {
        default: () => [mergedDndSortable.value.dragHandle ? createVNode(CdkDndSortableHandle, {
          "class": `${prefixCls}-drag-handle`
        }, {
          default: () => [!isDisabled.value && createVNode(IxIcon, {
            "name": mergedDndSortable.value.dragHandle
          }, null)]
        }) : void 0, optionContentNodes]
      }) : createVNode("div", optionsProps, [optionContentNodes]);
    };
  }
});
var OptionGroup = /* @__PURE__ */ defineComponent({
  props: optionGroupProps,
  setup(props, {
    slots
  }) {
    const {
      props: selectProps2,
      mergedPrefixCls
    } = inject(selectPanelContext);
    return () => {
      var _a;
      const {
        label,
        rawData
      } = props;
      const _label = toString(label);
      const prefixCls = `${mergedPrefixCls.value}-option-group`;
      const labelRender = (_a = rawData.customLabel) != null ? _a : "optionGroupLabel";
      const labelSlot = isString(labelRender) ? slots[labelRender] : labelRender;
      const customAdditional = selectProps2.customAdditional ? selectProps2.customAdditional({
        data: rawData,
        index: props.index
      }) : void 0;
      return createVNode("div", mergeProps({
        "class": prefixCls,
        "title": _label,
        "aria-label": _label
      }, customAdditional), [createVNode("span", {
        "class": `${prefixCls}-label`
      }, [labelSlot ? labelSlot(rawData) : _label])]);
    };
  }
});
function getDefaultThemeTokens(tokens) {
  const {
    fontSizeSm,
    heightMd,
    marginSizeSm,
    marginSizeMd,
    paddingSizeXs,
    paddingSizeSm,
    paddingSizeMd,
    paddingSizeXl
  } = tokens;
  return {
    optionFontSize: fontSizeSm,
    optionHeight: heightMd,
    optionPadding: `${paddingSizeSm}px ${paddingSizeMd}px`,
    optionLabelMarginLeft: marginSizeSm,
    optionContainerPadding: `${paddingSizeXs}px 0`,
    optionGroupMargin: `0 ${marginSizeMd}px`,
    optionGroupPaddingLeft: 0,
    multipleOptionGroupedPaddingLeft: paddingSizeMd,
    optionGroupedPaddingLeft: paddingSizeXl,
    overlaySearchWrapperPadding: `${paddingSizeXs}px ${paddingSizeMd}px ${paddingSizeSm}px`
  };
}
const getThemeTokens = (tokens, presetTheme) => {
  return presetTheme === "default" ? getDefaultThemeTokens(tokens) : getDefaultThemeTokens(tokens);
};
function usePanelActiveState(props, flattenedOptions, selectedKeys, scrollTo) {
  const [activeValue, setActiveValue] = useControlledProp(props, "activeValue");
  const keyIndexMap = computed(() => {
    const map = /* @__PURE__ */ new Map();
    flattenedOptions.value.forEach((option, index) => {
      map.set(option.key, index);
    });
    return map;
  });
  const activeIndex = ref(0);
  const setActiveIndex = (index) => {
    var _a, _b;
    activeIndex.value = index;
    const key = (_a = flattenedOptions.value[index]) == null ? void 0 : _a.key;
    key !== activeValue.value && setActiveValue((_b = flattenedOptions.value[index]) == null ? void 0 : _b.key);
  };
  const scrollToActivated = () => {
    scrollTo({ index: activeIndex.value });
  };
  onMounted(() => {
    const options = flattenedOptions.value;
    const currValue = selectedKeys.value.length ? selectedKeys.value : [activeValue.value];
    const currIndex = options.findIndex((option) => currValue.some((value) => option.key === value));
    setActiveIndex(currIndex !== -1 ? getEnabledActiveIndex(options, currIndex, 1) : -1);
    watch(
      [() => props.activeValue, flattenedOptions],
      ([value, options2]) => {
        var _a;
        const targetIndex = isNil(value) ? -1 : (_a = keyIndexMap.value.get(value)) != null ? _a : -1;
        setActiveIndex(targetIndex !== -1 ? getEnabledActiveIndex(options2, targetIndex, 1) : -1);
      },
      {
        flush: "post"
      }
    );
    scrollToActivated();
  });
  const changeActiveIndex = (offset) => {
    const enabledIndex = getEnabledActiveIndex(
      flattenedOptions.value,
      activeIndex.value + offset,
      offset % 2
    );
    if (enabledIndex !== activeIndex.value) {
      setActiveIndex(enabledIndex);
      if (offset !== 0) {
        scrollToActivated();
      }
    }
  };
  return {
    activeValue,
    activeIndex,
    setActiveIndex,
    changeActiveIndex,
    scrollToActivated
  };
}
function getEnabledActiveIndex(options, currIndex, offset) {
  const length = options.length;
  for (let index = 0; index < length; index++) {
    const current = (currIndex + index * offset + length) % length;
    const { type, disabled } = options[current];
    if (type !== "group" && !disabled) {
      return current;
    }
  }
  return -1;
}
function useSelectedState(props, locale) {
  const selectedKeys = computed(() => convertArray(props.selectedKeys));
  const selectedLimit = computed(() => selectedKeys.value.length >= props.multipleLimit);
  const selectedLimitTitle = computed(() => {
    if (!selectedLimit.value) {
      return "";
    }
    return locale.select.limitMessage.replace("${0}", `${props.multipleLimit}`);
  });
  return {
    selectedKeys,
    selectedLimit,
    selectedLimitTitle
  };
}
var Panel = /* @__PURE__ */ defineComponent({
  name: "IxSelectPanel",
  props: selectPanelProps,
  setup(props, {
    slots,
    expose
  }) {
    const common = useGlobalConfig("common");
    const {
      globalHashId,
      hashId,
      registerToken
    } = useThemeToken("select");
    registerToken(getThemeTokens);
    const mergedPrefixCls = computed(() => `${common.prefixCls}-select`);
    const config = useGlobalConfig("select");
    const locale = useGlobalConfig("locale");
    const mergedDndSortable = computed(() => {
      var _a;
      const dndSortable = props.dndSortable;
      if (!dndSortable) {
        return false;
      }
      if (dndSortable === true) {
        return {
          autoScroll: true,
          dragHandle: false
        };
      }
      return {
        autoScroll: (_a = dndSortable.autoScroll) != null ? _a : true,
        dragHandle: dndSortable.dragHandle === true ? "holder" : false
      };
    });
    const {
      flattenedOptions,
      mergedChildrenKey,
      getKey
    } = useSelectPanelDataContext(props, config);
    const selectedStateContext = useSelectedState(props, locale);
    const virtualScrollRef = ref();
    const scrollTo = (options) => {
      var _a;
      return (_a = virtualScrollRef.value) == null ? void 0 : _a.scrollTo(options);
    };
    const activeStateContext = usePanelActiveState(props, flattenedOptions, selectedStateContext.selectedKeys, scrollTo);
    const autoScrollListRef = computed(() => {
      var _a;
      if (!mergedDndSortable.value) {
        return;
      }
      return mergedDndSortable.value.autoScroll ? (_a = virtualScrollRef.value) == null ? void 0 : _a.getHolderElement() : void 0;
    });
    useDndAutoScroll(autoScrollListRef, {
      canScroll: true,
      allowedAxis: "vertical"
    });
    expose({
      scrollTo,
      changeActiveIndex: activeStateContext.changeActiveIndex
    });
    provide(selectPanelContext, {
      props,
      mergedPrefixCls,
      flattenedOptions,
      mergedDndSortable,
      ...selectedStateContext,
      ...activeStateContext
    });
    const panelClasses = computed(() => {
      const prefixCls = `${mergedPrefixCls.value}-panel`;
      return normalizeClass({
        [globalHashId.value]: !!globalHashId.value,
        [hashId.value]: !!hashId.value,
        [prefixCls]: true,
        [`${prefixCls}-multiple`]: !!props.multiple
      });
    });
    const handleScrolledChange = (startIndex, endIndex, visibleOptions) => {
      const {
        onScrolledChange
      } = props;
      if (onScrolledChange) {
        callEmit(onScrolledChange, startIndex, endIndex, visibleOptions.map((item) => item.rawData));
      }
    };
    const handleSortReorder = (reorderInfo) => {
      callEmit(props.onDndSortReorder, reorderInfo);
    };
    const handleSortChange = (newData) => {
      const oldData = props.dataSource ? toRaw(props.dataSource) : [];
      const unFlattenedNewData = unFlattenOptions(newData, mergedChildrenKey.value, getKey.value);
      callEmit(props.onDndSortChange, unFlattenedNewData, oldData);
    };
    return () => {
      const {
        _virtualScrollHeight,
        virtualItemHeight,
        virtualScrollMode,
        virtual,
        onScroll,
        onScrolledBottom
      } = props;
      const options = flattenedOptions.value;
      const children = [createVNode(ListBox, null, slots)];
      if (options.length > 0) {
        const rowRender = ({
          item,
          index
        }) => {
          const {
            type,
            key,
            ...rest
          } = item;
          return type === "group" ? createVNode(OptionGroup, mergeProps({
            "index": index
          }, rest), slots) : createVNode(Option, mergeProps({
            "index": index,
            "optionKey": key
          }, rest), slots);
        };
        const list = createVNode(CdkVirtualScroll, {
          "ref": virtualScrollRef,
          "dataSource": options,
          "getKey": "key",
          "height": _virtualScrollHeight,
          "rowHeight": virtualItemHeight,
          "scrollMode": virtual ? virtualScrollMode : void 0,
          "rowRender": rowRender,
          "virtual": virtual,
          "onScroll": onScroll,
          "onScrolledBottom": onScrolledBottom,
          "onScrolledChange": handleScrolledChange
        }, null);
        children.push(mergedDndSortable.value ? createVNode(CdkDndSortable, {
          "dataSource": options,
          "onSortReorder": handleSortReorder,
          "onSortChange": handleSortChange,
          "onDragStart": props._onDragStart,
          "onDrop": props._onDrop
        }, {
          default: () => [list]
        }) : list);
      } else {
        children.push(createVNode(_Empty, {
          "empty": props.empty
        }, slots));
      }
      return createVNode("div", {
        "class": panelClasses.value
      }, [children]);
    };
  }
});
function useSelectPanelDataContext(props, config) {
  const dataContext = inject(SELECT_PANEL_DATA_TOKEN, null);
  const mergedChildrenKey = computed(() => {
    var _a;
    return (_a = props.childrenKey) != null ? _a : config.childrenKey;
  });
  const mergedLabelKey = computed(() => {
    var _a;
    return (_a = props.labelKey) != null ? _a : config.labelKey;
  });
  const getKey = usePanelGetOptionKey(props, config);
  const mergedFlattenedOptions = dataContext ? dataContext.flattenedOptions : useFlattenedOptions(computed(() => props.dataSource), mergedChildrenKey, getKey, mergedLabelKey);
  return {
    flattenedOptions: mergedFlattenedOptions,
    mergedChildrenKey,
    mergedLabelKey,
    getKey
  };
}
var Select = /* @__PURE__ */ defineComponent({
  name: "IxSelect",
  inheritAttrs: false,
  props: selectProps,
  setup(props, {
    attrs,
    expose,
    slots
  }) {
    const common = useGlobalConfig("common");
    const {
      globalHashId,
      hashId,
      registerToken
    } = useThemeToken("select");
    registerToken(getThemeTokens);
    const config = useGlobalConfig("select");
    const mergedPrefixCls = computed(() => `${common.prefixCls}-select`);
    const triggerRef = ref();
    const focus = () => {
      var _a;
      return (_a = triggerRef.value) == null ? void 0 : _a.focus();
    };
    const blur = () => {
      var _a;
      return (_a = triggerRef.value) == null ? void 0 : _a.blur();
    };
    const panelRef = ref();
    const scrollTo = (...params) => {
      var _a;
      return (_a = panelRef.value) == null ? void 0 : _a.scrollTo(...params);
    };
    const changeActiveIndex = (offset) => {
      var _a;
      return (_a = panelRef.value) == null ? void 0 : _a.changeActiveIndex(offset);
    };
    const [inputValue, _setInputValue] = useState("");
    const setInputValue = (input) => {
      _setInputValue(input);
      props.searchable && callEmit(props.onSearch, input);
    };
    const clearInput = () => {
      var _a;
      props.searchable === "overlay" ? setInputValue("") : (_a = triggerRef.value) == null ? void 0 : _a.clearInput();
    };
    expose({
      focus,
      blur,
      scrollTo
    });
    const [overlayOpened, setOverlayOpened] = useControlledProp(props, "open", false);
    const {
      accessor,
      control
    } = useAccessorAndControl();
    const commonControlProps = useMergedCommonControlProps(props, config);
    useFormItemRegister(control);
    const mergedStatus = useFormStatus(props, control);
    const getKey = useGetOptionKey(props, config);
    const {
      options,
      optionKeyMap
    } = useSelectOptions(props, config, slots, getKey, inputValue);
    provide(SELECT_PANEL_DATA_TOKEN, {
      flattenedOptions: options
    });
    const {
      selectedValue,
      selectedOptions,
      changeSelected,
      handleClear,
      handleRemove
    } = useSelectedState$1(props, accessor, optionKeyMap);
    const {
      activeValue,
      setActiveValue
    } = useActiveState(props, inputValue);
    const handleKeyDown = useKeyboardEvents(props, inputValue, selectedValue, activeValue, overlayOpened, changeActiveIndex, changeSelected, handleRemove, clearInput, setOverlayOpened);
    watch(overlayOpened, (opened) => {
      if (opened) {
        focus();
      }
    });
    const handleOptionClick = (option) => {
      var _a;
      changeSelected((_a = getKey.value(option)) != null ? _a : option.key);
      (props.allowInput || !props.multiple) && clearInput();
      if (!props.multiple) {
        setOverlayOpened(false);
      }
    };
    const onFocus = (evt) => {
      callEmit(props.onFocus, evt);
    };
    const onBlur = (evt) => {
      if (inputValue.value) {
        if (props.allowInput) {
          changeSelected(inputValue.value);
        }
        clearInput();
      }
      accessor.markAsBlurred();
      callEmit(props.onBlur, evt);
    };
    const handleItemRemove = (value) => {
      focus();
      handleRemove(value);
    };
    const panelProps = usePanelProps(props, selectedValue, activeValue, setActiveValue, handleOptionClick);
    const overlayClasses = computed(() => {
      const {
        overlayClassName
      } = props;
      const prefixCls = mergedPrefixCls.value;
      return normalizeClass({
        [globalHashId.value]: !!globalHashId.value,
        [hashId.value]: !!hashId.value,
        [`${prefixCls}-overlay`]: true,
        [overlayClassName || ""]: !!overlayClassName
      });
    });
    const selectorAllowInput = computed(() => {
      if (props.searchable === "overlay") {
        return false;
      }
      return props.searchable ? "searchable" : props.allowInput;
    });
    const renderTrigger = ({
      borderless,
      status,
      clearable,
      clearIcon,
      readonly,
      disabled,
      size,
      suffix,
      suffixRotate,
      focused,
      opened
    }) => [createVNode(IxSelector, {
      "ref": triggerRef,
      "class": mergedPrefixCls.value,
      "allowInput": selectorAllowInput.value,
      "autocomplete": props.autocomplete,
      "autofocus": props.autofocus,
      "borderless": borderless,
      "clearable": clearable,
      "clearIcon": clearIcon,
      "dataSource": selectedOptions.value,
      "disabled": disabled,
      "focused": focused,
      "maxLabel": props.maxLabel,
      "multiple": props.multiple,
      "opened": opened,
      "placeholder": props.placeholder,
      "readonly": readonly,
      "size": size,
      "status": status,
      "suffix": suffix,
      "suffixRotate": suffixRotate,
      "onClear": handleClear,
      "onInputValueChange": setInputValue,
      "onItemRemove": handleItemRemove,
      "onKeydown": handleKeyDown
    }, slots)];
    const renderLoading = (children) => {
      const {
        spin
      } = props;
      const spinProps = isBoolean(spin) ? {
        spinning: spin
      } : spin;
      return spinProps ? createVNode(IxSpin, spinProps, {
        default: () => [children]
      }) : children;
    };
    const renderContent = () => {
      var _a;
      const children = [renderLoading(createVNode(Panel, mergeProps({
        "ref": panelRef
      }, panelProps.value), slots))];
      const {
        searchable,
        overlayRender
      } = props;
      if (searchable === "overlay") {
        const value = inputValue.value;
        const clearIcon = (_a = props.clearIcon) != null ? _a : config.clearIcon;
        const handleSearchInput = (evt) => {
          const {
            value: value2
          } = evt.target;
          setInputValue(value2);
        };
        const handleSearchClear = () => {
          setInputValue("");
        };
        children.unshift(createVNode("div", {
          "class": `${mergedPrefixCls.value}-overlay-search-wrapper`
        }, [createVNode(_Input, {
          "clearable": true,
          "clearIcon": clearIcon,
          "clearVisible": !!value,
          "placeholder": props.searchPlaceholder,
          "size": "sm",
          "suffix": "search",
          "value": value,
          "onClear": handleSearchClear,
          "onInput": handleSearchInput
        }, null)]));
      }
      return [createVNode("div", null, [overlayRender ? overlayRender(children) : children])];
    };
    return () => {
      var _a, _b, _c, _d;
      const {
        suffix,
        borderless,
        clearIcon,
        size
      } = commonControlProps.value;
      const controlTriggerProps = {
        autofocus: props.autofocus,
        overlayClassName: overlayClasses.value,
        overlayContainer: (_a = props.overlayContainer) != null ? _a : config.overlayContainer,
        overlayContainerFallback: `.${mergedPrefixCls.value}-overlay-container`,
        overlayTabindex: (_b = props.overlayTabindex) != null ? _b : config.overlayTabindex,
        overlayMatchWidth: (_c = props.overlayMatchWidth) != null ? _c : config.overlayMatchWidth,
        class: mergedPrefixCls.value,
        borderless,
        value: selectedValue.value,
        offset: (_d = props.offset) != null ? _d : config.offset,
        open: overlayOpened.value,
        readonly: props.readonly,
        size,
        status: mergedStatus.value,
        suffix,
        clearable: props.clearable,
        clearIcon,
        disabled: accessor.disabled,
        onFocus,
        onBlur,
        "onUpdate:open": setOverlayOpened
      };
      const controlTriggerSlots = {
        trigger: renderTrigger,
        overlay: renderContent
      };
      return createVNode(IxControlTrigger, mergeProps(controlTriggerProps, attrs), controlTriggerSlots);
    };
  }
});
function useSelectOptions(props, config, slots, getKey, inputValue) {
  const mergedChildrenKey = computed(() => {
    var _a;
    return (_a = props.childrenKey) != null ? _a : config.childrenKey;
  });
  const mergedLabelKey = computed(() => {
    var _a;
    return (_a = props.labelKey) != null ? _a : config.labelKey;
  });
  const convertedOptions = useConvertedOptions(props, slots);
  const flattenedOptions = useFlattenedOptions(convertedOptions, mergedChildrenKey, getKey, mergedLabelKey);
  const filteredOptions = useFilteredOptions(props, flattenedOptions, inputValue, mergedLabelKey);
  const optionKeyMap = useOptionKeyMap(flattenedOptions);
  return {
    options: filteredOptions,
    optionKeyMap
  };
}
function useOverlayState(props, config, triggerRef) {
  const overlayRef = ref();
  const [overlayWidth, setOverlayWidth] = useState("");
  const overlayStyle = computed(() => {
    const { overlayMatchWidth = config.overlayMatchWidth } = props;
    return { [overlayMatchWidth ? "width" : "minWidth"]: overlayWidth.value };
  });
  const [overlayOpened, setOverlayOpened] = useControlledProp(props, "open", false);
  const updateOverlay = (rect) => {
    var _a, _b;
    if (rect) {
      const { width } = rect;
      if (width > 0) {
        setOverlayWidth(convertCssPixel(width));
        overlayOpened.value && ((_a = overlayRef.value) == null ? void 0 : _a.updatePopper());
      }
    } else {
      overlayOpened.value && ((_b = overlayRef.value) == null ? void 0 : _b.updatePopper());
    }
  };
  onMounted(() => {
    if (props.autofocus) {
      setOverlayOpened(true);
    }
    watchEffect(() => {
      const overlayInstance = overlayRef.value;
      if (overlayInstance && overlayOpened.value) {
        updateOverlay(triggerRef.value.getBoundingClientRect());
      }
    });
  });
  return { overlayRef, overlayStyle, updateOverlay, overlayOpened, setOverlayOpened };
}
const IxSelect = Select;
const IxSelectPanel = Panel;
const IxSelectOption = SelectOption;
const IxSelectOptionGroup = SelectOptionGroup;
export { IxSelect, IxSelectOption, IxSelectOptionGroup, IxSelectPanel, getThemeTokens as getSelectThemeTokens, useOverlayState as \u0275UseOverlayState };
