import { defineComponent, computed, createVNode, Transition, normalizeClass, normalizeStyle, provide, Fragment, ref, nextTick, inject } from "vue";
import { hasSlot, convertCssPixel, convertArray, addClass, removeClass, throwError } from "@idux/cdk/utils";
import { \u0275Loading as _Loading } from "@idux/components/_private/loading";
import { useGlobalConfig } from "@idux/components/config";
import { IxIcon } from "@idux/components/icon";
import { getAlphaColor, useThemeToken } from "@idux/components/theme";
import { CdkPortal } from "@idux/cdk/portal";
import { useResizeObserver } from "@idux/cdk/resize";
import { convertTarget } from "@idux/components/utils";
const spinProps = {
  strokeWidth: Number,
  radius: Number,
  duration: Number,
  spinning: {
    type: Boolean,
    default: true
  },
  rotate: {
    type: Boolean,
    default: true
  },
  icon: String,
  tip: String,
  tipAlign: String,
  size: String
};
const spinProviderProps = {};
function getDefaultThemeTokens(tokens, algorithms) {
  const { getBaseColors, getGreyColors } = algorithms;
  const { colorContainerBg, colorPrimary, fontSizeSm, fontSizeMd, fontSizeLg } = tokens;
  return {
    tipColor: colorPrimary,
    iconColor: colorPrimary,
    fontSizeSm,
    fontSizeMd,
    fontSizeLg,
    iconSizeSm: 20,
    iconSizeMd: 28,
    iconSizeLg: 48,
    bgCircleStroke: colorPrimary,
    fstArchStroke: getBaseColors().turquoise,
    sndArchStroke: getGreyColors().l20,
    maskBgColor: getAlphaColor(colorContainerBg, 0.7)
  };
}
const getThemeTokens = (tokens, _, algorithms) => {
  return getDefaultThemeTokens(tokens, algorithms);
};
const defaultStrokeWidth = {
  sm: 3,
  md: 3,
  lg: 4
};
const defaultRadius = {
  sm: 14,
  md: 14,
  lg: 24
};
var Spin = /* @__PURE__ */ defineComponent({
  name: "IxSpin",
  props: spinProps,
  setup(props, {
    slots
  }) {
    const common = useGlobalConfig("common");
    const {
      globalHashId,
      hashId,
      registerToken
    } = useThemeToken("spin");
    registerToken(getThemeTokens);
    const mergedPrefixCls = computed(() => `${common.prefixCls}-spin`);
    const spinConfig = useGlobalConfig("spin");
    const {
      size,
      strokeWidth,
      radius
    } = useSize(props, spinConfig);
    const hasDefaultSlot = computed(() => hasSlot(slots));
    const mergedIcon = computed(() => {
      var _a;
      return (_a = props.icon) != null ? _a : spinConfig.icon;
    });
    const mergedTip = computed(() => {
      var _a;
      return (_a = props.tip) != null ? _a : spinConfig.tip;
    });
    const {
      spinnerClassName,
      containerClassName
    } = useClasses(props, spinConfig, size, hasDefaultSlot, mergedPrefixCls);
    const renderContent = () => {
      if (!slots.default) {
        return null;
      }
      return createVNode("div", {
        "class": containerClassName.value
      }, [slots.default()]);
    };
    const renderTip = () => {
      if (!mergedTip.value) {
        return null;
      }
      return createVNode("div", {
        "class": `${mergedPrefixCls.value}-spinner-tip`
      }, [mergedTip.value]);
    };
    const renderIcon = () => {
      const iconCls = `${mergedPrefixCls.value}-spinner-icon`;
      if (slots.icon) {
        return createVNode("div", {
          "class": iconCls
        }, [slots.icon()]);
      }
      if (mergedIcon.value) {
        const iconStyle = normalizeStyle(props.duration && {
          animationDuration: `${props.duration}s`
        });
        return createVNode("div", {
          "class": [iconCls, props.rotate && `${iconCls}--rotate`],
          "style": iconStyle
        }, [createVNode(IxIcon, {
          "name": mergedIcon.value
        }, null)]);
      }
      return createVNode("div", {
        "class": iconCls
      }, [createVNode(_Loading, {
        "strokeWidth": strokeWidth.value,
        "radius": radius.value,
        "duration": props.duration
      }, null)]);
    };
    const renderSpinner = () => {
      if (!props.spinning) {
        return null;
      }
      return createVNode("div", {
        "class": spinnerClassName.value
      }, [renderIcon(), renderTip()]);
    };
    return () => createVNode("div", {
      "class": [mergedPrefixCls.value, globalHashId.value, hashId.value]
    }, [createVNode(Transition, {
      "name": `${common.prefixCls}-fade`,
      "appear": true
    }, {
      default: () => [renderSpinner()]
    }), renderContent()]);
  }
});
const useSize = (props, config) => {
  const size = computed(() => {
    var _a;
    return (_a = props.size) != null ? _a : config.size;
  });
  const strokeWidth = computed(() => {
    var _a, _b, _c;
    return (_c = (_b = props.strokeWidth) != null ? _b : (_a = config.strokeWidth) == null ? void 0 : _a[size.value]) != null ? _c : defaultStrokeWidth[size.value];
  });
  const radius = computed(() => {
    var _a, _b, _c;
    return (_c = (_b = props.radius) != null ? _b : (_a = config.radius) == null ? void 0 : _a[size.value]) != null ? _c : defaultRadius[size.value];
  });
  return {
    size,
    strokeWidth,
    radius
  };
};
const useClasses = (props, config, size, hasDefaultSlot, mergedPrefixCls) => {
  const prefixCls = mergedPrefixCls.value;
  const spinnerClassName = computed(() => {
    var _a;
    const tipAlign = (_a = props.tipAlign) != null ? _a : config.tipAlign;
    return normalizeClass([`${prefixCls}-spinner`, `${prefixCls}-spinner-tip-${tipAlign}`, `${prefixCls}-spinner-${size.value}`]);
  });
  const containerClassName = computed(() => {
    if (!hasDefaultSlot.value) {
      return [];
    }
    return normalizeClass([`${prefixCls}-container`, props.spinning ? `${prefixCls}-container-blur` : ""]);
  });
  return {
    spinnerClassName,
    containerClassName
  };
};
const SPIN_PROVIDER_TOKEN = Symbol("SPIN_PROVIDER_TOKEN");
const BASE_ZINDEX = 2e3;
var SpinProvider = /* @__PURE__ */ defineComponent({
  name: "IxSpinProvider",
  inheritAttrs: false,
  props: spinProviderProps,
  setup(_, {
    expose,
    slots
  }) {
    const common = useGlobalConfig("common");
    const mergedPrefixCls = computed(() => `${common.prefixCls}-spin`);
    const {
      spins,
      open,
      update,
      destroy,
      destroyAll
    } = useSpin$1(mergedPrefixCls);
    const apis = {
      open,
      update,
      destroy,
      destroyAll
    };
    provide(SPIN_PROVIDER_TOKEN, apis);
    expose(apis);
    return () => {
      var _a;
      const children = [];
      spins.value.forEach((spinOptions, index) => {
        const {
          spinning,
          target,
          width,
          height,
          isFullScreen,
          hasScroll,
          tip,
          zIndex
        } = spinOptions;
        const BaseZindex = zIndex != null ? zIndex : BASE_ZINDEX + index;
        const style = normalizeStyle({
          width: !isFullScreen && hasScroll ? convertCssPixel(width) : null,
          height: !isFullScreen && hasScroll ? convertCssPixel(height) : null,
          position: isFullScreen ? "fixed" : "",
          zIndex: BaseZindex
        });
        spinning && children.push(createVNode(CdkPortal, {
          "target": target
        }, {
          default: () => [createVNode(Spin, {
            "style": style,
            "tip": tip
          }, null)]
        }));
      });
      return createVNode(Fragment, null, [children, (_a = slots.default) == null ? void 0 : _a.call(slots)]);
    };
  }
});
function useSpin$1(mergedPrefixCls) {
  const spins = ref([]);
  const _convertTarget = (target) => {
    const targetElement = convertTarget(target);
    return targetElement === window ? null : targetElement;
  };
  const elementHasScroll = (element) => {
    return element.clientHeight < element.scrollHeight || element.clientWidth < element.scrollWidth;
  };
  const deleteClass = (targetElement) => {
    removeClass(targetElement, [`${mergedPrefixCls.value}-target-container`, `${mergedPrefixCls.value}-target-container-relative`, `${mergedPrefixCls.value}-target-container-has-scroll`]);
  };
  const getCurrIndex = (target) => {
    return spins.value.findIndex((spin) => spin.target === target);
  };
  const add = (item) => {
    const target = item.target;
    const currIndex = getCurrIndex(target);
    if (currIndex !== -1) {
      spins.value.splice(currIndex, 1, item);
    }
    spins.value.push(item);
  };
  const update = (options) => {
    const {
      target = "body"
    } = options;
    const currIndex = getCurrIndex(target);
    if (currIndex !== -1) {
      const newItem = {
        ...spins.value[currIndex],
        ...options
      };
      spins.value.splice(currIndex, 1, newItem);
    }
  };
  const destroy = (target = "body") => {
    const targets = convertArray(target);
    targets.forEach((target2) => {
      var _a;
      const currIndex = getCurrIndex(target2);
      if (currIndex !== -1) {
        const item = spins.value.splice(currIndex, 1)[0];
        const targetElement = _convertTarget(item.target);
        targetElement && deleteClass(targetElement);
        (_a = item.stopResizeWatch) == null ? void 0 : _a.call(item);
      }
    });
  };
  const destroyAll = () => {
    destroy(spins.value.map((item) => item.target));
    spins.value = [];
  };
  const open = (options) => {
    const {
      target = "body",
      ...reset
    } = options;
    if (getCurrIndex(target) === -1) {
      const targetElement = convertTarget(target);
      const hasScroll = elementHasScroll(targetElement);
      const isFullScreen = targetElement === document.body;
      const staticPosition = getComputedStyle(targetElement).position === "static";
      addClass(targetElement, [`${mergedPrefixCls.value}-target-container`, staticPosition ? `${mergedPrefixCls.value}-target-container-relative` : ""].filter(Boolean));
      let stopResizeWatch;
      if (!isFullScreen) {
        stopResizeWatch = useResizeObserver(targetElement, () => {
          const currIndex = getCurrIndex(target);
          spins.value[currIndex].width = 0;
          spins.value[currIndex].height = 0;
          nextTick(() => {
            const hasScroll2 = elementHasScroll(targetElement);
            spins.value[currIndex].hasScroll = hasScroll2;
            const hasScrollClsName = `${mergedPrefixCls.value}-target-container-has-scroll`;
            if (hasScroll2) {
              addClass(targetElement, hasScrollClsName);
              spins.value[currIndex].width = targetElement.scrollWidth;
              spins.value[currIndex].height = targetElement.scrollHeight;
            } else {
              removeClass(targetElement, hasScrollClsName);
            }
          });
        });
      }
      add({
        spinning: true,
        target,
        hasScroll,
        isFullScreen: targetElement === document.body,
        staticPosition,
        width: targetElement.scrollWidth,
        height: targetElement.scrollHeight,
        stopResizeWatch,
        ...reset
      });
    }
    const ref2 = {
      target,
      update: (options2) => update({
        target,
        ...options2
      }),
      destroy: () => destroy(target)
    };
    return ref2;
  };
  return {
    spins,
    open,
    update,
    destroy,
    destroyAll
  };
}
const useSpin = () => {
  const spinProviderRef = inject(SPIN_PROVIDER_TOKEN, null);
  if (spinProviderRef === null) {
    return throwError("components/spin", "<IxSpinProvider> not found.");
  }
  return spinProviderRef;
};
const IxSpin = Spin;
const IxSpinProvider = SpinProvider;
export { IxSpin, IxSpinProvider, SPIN_PROVIDER_TOKEN, getThemeTokens as getSpinThemeTokens, useSpin };
