import { defineComponent, computed, normalizeClass, provide, reactive, toRefs, createVNode, inject } from "vue";
import { useControlledProp, flattenNode, useKey, NoopFunction } from "@idux/cdk/utils";
import { useGlobalConfig } from "@idux/components/config";
import { useThemeToken } from "@idux/components/theme";
import { isSymbol } from "lodash-es";
import { IxIcon } from "@idux/components/icon";
import { IxProgress } from "@idux/components/progress";
import { convertStringVNode } from "@idux/components/utils";
const stepperToken = Symbol("stepperToken");
const stepperItemKey = Symbol("IDUX_STEPPER_ITEM_KEY");
const stepperProps = {
  activeKey: [String, Number, Symbol],
  clickable: {
    type: Boolean,
    default: void 0
  },
  labelPlacement: String,
  percent: {
    type: Number,
    validator: (value) => value >= 0 && value <= 100
  },
  size: String,
  status: {
    type: String,
    default: "process"
  },
  vertical: {
    type: Boolean,
    default: false
  },
  dot: {
    type: Boolean,
    default: false
  },
  "onUpdate:activeKey": [Function, Array]
};
const stepperItemProps = {
  description: String,
  disabled: {
    type: Boolean,
    default: false
  },
  icon: String,
  title: String,
  status: String
};
function getDefaultThemeTokens(tokens) {
  const { heightSm, heightMd, marginSizeSm, fontSizeSm, fontSizeMd, fontSizeLg, fontSizeHeaderSm, fontSizeHeaderMd } = tokens;
  return {
    iconSizeSm: heightSm,
    iconSizeMd: heightMd,
    iconFontSizeSm: fontSizeMd,
    iconFontSizeMd: fontSizeLg,
    iconBorderWidthSm: 2,
    iconBorderWidthMd: 2,
    iconMarginSm: `0 ${marginSizeSm}px 0 0`,
    iconMarginMd: `0 ${marginSizeSm}px 0 0`,
    titleFontSizeSm: fontSizeHeaderSm,
    titleFontSizeMd: fontSizeHeaderMd,
    descriptionFontSizeSm: fontSizeSm,
    descriptionFontSizeMd: fontSizeMd,
    contentMinHeightSm: 60,
    contentMinHeightMd: 80,
    lineType: "dashed"
  };
}
const getThemeTokens = (tokens, presetTheme) => {
  return presetTheme === "default" ? getDefaultThemeTokens(tokens) : getDefaultThemeTokens(tokens);
};
var Stepper = /* @__PURE__ */ defineComponent({
  name: "IxStepper",
  props: stepperProps,
  setup(props, {
    slots
  }) {
    const common = useGlobalConfig("common");
    const {
      globalHashId,
      hashId,
      registerToken
    } = useThemeToken("stepper");
    registerToken(getThemeTokens);
    const mergedPrefixCls = computed(() => `${common.prefixCls}-stepper`);
    const config = useGlobalConfig("stepper");
    const size = computed(() => {
      var _a;
      return (_a = props.size) != null ? _a : config.size;
    });
    const classes = computed(() => {
      const prefixCls = mergedPrefixCls.value;
      const {
        labelPlacement = config.labelPlacement,
        percent,
        vertical
      } = props;
      return normalizeClass({
        [globalHashId.value]: !!globalHashId.value,
        [hashId.value]: !!hashId.value,
        [prefixCls]: true,
        [`${prefixCls}-label-${labelPlacement}`]: true,
        [`${prefixCls}-${size.value}`]: true,
        [`${prefixCls}-vertical`]: vertical,
        [`${prefixCls}-with-percent`]: percent != null
      });
    });
    const [activeKey, setActiveKey] = useControlledProp(props, "activeKey");
    provide(stepperToken, {
      props: reactive({
        ...toRefs(props),
        size
      }),
      slots,
      activeKey,
      setActiveKey
    });
    return () => {
      var _a;
      const children = flattenNode((_a = slots.default) == null ? void 0 : _a.call(slots), {
        key: stepperItemKey
      }).map((item, index) => {
        if (item.key == null) {
          item.key = index + 1;
        }
        return item;
      });
      return createVNode("div", {
        "class": classes.value
      }, [children]);
    };
  }
});
var StepperItem = /* @__PURE__ */ defineComponent({
  name: "IxStepperItem",
  props: stepperItemProps,
  [stepperItemKey]: true,
  setup(props, {
    slots
  }) {
    const key = useKey();
    const common = useGlobalConfig("common");
    const mergedPrefixCls = computed(() => `${common.prefixCls}-stepper-item`);
    const {
      props: parentProps,
      activeKey,
      setActiveKey
    } = inject(stepperToken);
    const isActive = computed(() => activeKey.value === key);
    const status = computed(() => {
      if (props.status) {
        return props.status;
      }
      if (isActive.value) {
        return parentProps.status;
      }
      const currActiveKey = activeKey.value;
      if (!currActiveKey || isSymbol(currActiveKey) || isSymbol(key)) {
        return "wait";
      }
      return currActiveKey > key ? "finish" : "wait";
    });
    const classes = computed(() => {
      const prefixCls = mergedPrefixCls.value;
      const {
        disabled,
        icon
      } = props;
      const {
        dot
      } = parentProps;
      return normalizeClass({
        [prefixCls]: true,
        [`${prefixCls}-${status.value}`]: true,
        [`${prefixCls}-active`]: isActive.value,
        [`${prefixCls}-clickable`]: parentProps.clickable && !disabled,
        [`${prefixCls}-disabled`]: disabled,
        [`${prefixCls}-with-icon`]: icon || !!slots.icon,
        [`${prefixCls}-dot`]: dot
      });
    });
    const onClick = () => {
      if (isActive.value) {
        return;
      }
      setActiveKey(key);
    };
    return () => {
      const prefixCls = mergedPrefixCls.value;
      const clickable = parentProps.clickable && !props.disabled;
      const {
        dot
      } = parentProps;
      const iconNode = renderIcon(props, slots, parentProps, status, key);
      const titleNode = convertStringVNode(slots, props, "title");
      const descriptionNode = convertStringVNode(slots, props, "description");
      return createVNode("div", {
        "class": classes.value,
        "role": clickable ? "button" : void 0,
        "tabindex": clickable ? 0 : void 0,
        "onClick": clickable ? onClick : void 0
      }, [createVNode("div", {
        "class": `${prefixCls}-tail`
      }, null), createVNode("div", {
        "class": `${prefixCls}-icon`
      }, [dot ? "" : iconNode]), createVNode("div", {
        "class": `${prefixCls}-content`
      }, [createVNode("div", {
        "class": `${prefixCls}-title`
      }, [titleNode]), descriptionNode && createVNode("div", {
        "class": `${prefixCls}-description`
      }, [descriptionNode])])]);
    };
  }
});
const statusIcons = {
  finish: "check-filled",
  error: "exclamation"
};
function renderIcon(props, slots, parentProps, status, key) {
  const currStatus = status.value;
  let node;
  if (slots.icon) {
    node = slots.icon({
      key,
      status: currStatus
    });
  } else {
    const {
      icon = statusIcons[currStatus]
    } = props;
    if (icon) {
      node = createVNode(IxIcon, {
        "name": icon
      }, null);
    } else {
      node = createVNode("span", null, [key]);
    }
  }
  if (currStatus === "process" && parentProps.percent != null) {
    const width = parentProps.size === "sm" ? 32 : 40;
    return [createVNode(IxProgress, {
      "type": "circle",
      "percent": parentProps.percent,
      "width": width,
      "strokeWidth": 4,
      "format": NoopFunction
    }, null), node];
  }
  return node;
}
const IxStepper = Stepper;
const IxStepperItem = StepperItem;
export { IxStepper, IxStepperItem, getThemeTokens as getStepperThemeTokens };
