import { computed, ref, watch, Text, reactive, onBeforeUnmount, onMounted, defineComponent, inject, normalizeClass, createVNode, mergeProps, watchEffect, createTextVNode, provide, withDirectives, vShow } from "vue";
import { isObject, isString, isNumber, isNil, isSymbol, isPlainObject, isFunction, kebabCase, isBoolean } from "lodash-es";
import { convertArray, Logger, uniqueId, flattenNode, traverseTree, useControlledProp, callEmit, convertCssPixel, convertElement, useState, NoopArray, on, off, getOffset, isEmptyNode, isVisibleElement } from "@idux/cdk/utils";
import { \u0275Header as _Header } from "@idux/components/_private/header";
import { useGlobalConfig } from "@idux/components/config";
import { IxSpin } from "@idux/components/spin";
import { getAlphaColor, useThemeToken } from "@idux/components/theme";
import { useTreeCheckState, useGetKey } from "@idux/components/utils";
import { useResizeObserver, onResize, offResize } from "@idux/cdk/resize";
import { getScrollBarSize, scrollToTop, CdkVirtualScroll } from "@idux/cdk/scroll";
import { \u0275Empty as _Empty } from "@idux/components/_private/empty";
import { IxButton } from "@idux/components/button";
import { IxCheckbox } from "@idux/components/checkbox";
import { IxDropdown } from "@idux/components/dropdown";
import { IxIcon } from "@idux/components/icon";
import { IxMenu } from "@idux/components/menu";
import { IxRadio } from "@idux/components/radio";
import { IxTooltip } from "@idux/components/tooltip";
import { IxPagination } from "@idux/components/pagination";
function useColumnOffsets(fixedColumns, columnWidthsMap, columnCount) {
  const columnOffsets = computed(() => {
    const { fixedStartColumns, fixedEndColumns, fixedColumnIndexMap } = fixedColumns.value;
    return calculateOffsets(
      fixedStartColumns,
      fixedEndColumns.filter((column) => column.type !== "scroll-bar"),
      fixedColumnIndexMap,
      columnWidthsMap.value,
      columnCount.value - 1
    );
  });
  const columnOffsetsWithScrollBar = computed(() => {
    const { fixedStartColumns, fixedEndColumns, fixedColumnIndexMap } = fixedColumns.value;
    return calculateOffsets(
      fixedStartColumns,
      fixedEndColumns,
      fixedColumnIndexMap,
      columnWidthsMap.value,
      columnCount.value
    );
  });
  return { columnOffsets, columnOffsetsWithScrollBar };
}
function calculateOffsets(startColumns, endColumns, columnIndexMap, columnWidthsMap, columnCount) {
  var _a, _b, _c, _d, _e, _f;
  const startOffsets = {};
  const endOffsets = {};
  let startOffset = 0;
  let endOffset = 0;
  for (let index = 0; index < startColumns.length; index++) {
    const column = startColumns[index];
    const width = (_b = (_a = columnWidthsMap[column.key]) != null ? _a : column.width) != null ? _b : 0;
    startOffsets[column.key] = { index: (_c = columnIndexMap[column.key]) != null ? _c : index, offset: startOffset };
    startOffset += width;
  }
  for (let index = 0; index < endColumns.length; index++) {
    const column = endColumns[endColumns.length - index - 1];
    const width = (_e = (_d = columnWidthsMap[column.key]) != null ? _d : column.width) != null ? _e : 0;
    endOffsets[column.key] = { index: (_f = columnIndexMap[column.key]) != null ? _f : columnCount - index - 1, offset: endOffset };
    endOffset += width;
  }
  return {
    starts: startOffsets,
    ends: endOffsets
  };
}
function useColumnWidthMeasure(flattedColumns) {
  const measuredColumnWidthMap = ref({});
  const changeColumnWidth = (key, width) => {
    if (width === false) {
      delete measuredColumnWidthMap.value[key];
    } else {
      measuredColumnWidthMap.value[key] = width;
    }
  };
  watch(flattedColumns, (columns) => {
    const columnKeySet = new Set(columns.map((column) => column.key));
    Object.keys(measuredColumnWidthMap).forEach((key) => {
      if (!columnKeySet.has(key)) {
        delete measuredColumnWidthMap.value[key];
      }
    });
  });
  return {
    measuredColumnWidthMap,
    changeColumnWidth
  };
}
const tableColumnKey = Symbol("IxTableColumn");
const TableColumn = () => {
};
TableColumn.displayName = "IxTableColumn";
TableColumn[tableColumnKey] = true;
function getColTitle(ellipsis, children, title) {
  if (!ellipsis || isObject(ellipsis) && ellipsis.title === false) {
    return void 0;
  }
  if (isString(children) || isNumber(children)) {
    return children;
  }
  const textNode = flattenNode(children).find((node) => node.type === Text);
  if (textNode) {
    return textNode.children;
  }
  return title;
}
function getColumnKey(column) {
  if ("key" in column) {
    return column.key;
  }
  if ("dataKey" in column) {
    return convertArray(column.dataKey).join("-");
  }
  if ("type" in column) {
    return `__IDUX_table_column_key_${column.type}`;
  }
  process.env.NODE_ENV !== "production" && Logger.warn(
    "components/table",
    "Each column in table should have a unique `key`, `dataKey` or `type` prop.",
    column
  );
  return uniqueId("__IDUX_table_column_key_");
}
function modifyVirtualData(renderedRow, renderedCols, flattedColumns, flattedData, fixedStartColumns, fixedEndColumns, includeScrollBar = false) {
  const filterColumns = (columns) => columns.filter((column) => column.type !== "scroll-bar");
  const startColumns = includeScrollBar ? fixedStartColumns : filterColumns(fixedStartColumns);
  const endColumns = includeScrollBar ? fixedEndColumns : filterColumns(fixedEndColumns);
  if (!startColumns.length && !endColumns.length) {
    return;
  }
  const rowIndex = flattedData.findIndex((data) => data.rowKey === renderedRow.rowKey);
  const getAppendedColumn = (column) => {
    if (renderedCols.findIndex((col) => col.key === column.key) > -1) {
      return;
    }
    const index = flattedColumns.findIndex((col) => col.key === column.key);
    let poolKey;
    if (isString(column.key)) {
      poolKey = `fix-${rowIndex}-${column.key}`;
    } else {
      poolKey = `fix-${rowIndex}-${index}`;
    }
    return {
      data: column,
      index,
      poolKey
    };
  };
  return {
    start: startColumns.map(getAppendedColumn).filter(Boolean),
    end: endColumns.map(getAppendedColumn).filter(Boolean)
  };
}
function useColumns(props, slots, config, scrollBarSizeOnFixedHolder) {
  const mergedColumns = computed(() => {
    var _a;
    const { columns } = props;
    return mergeColumns(columns && columns.length > 0 ? columns : convertColumns((_a = slots.default) == null ? void 0 : _a.call(slots)), config);
  });
  const { flattedColumns, scrollBarColumn, flattedColumnsWithScrollBar } = useFlattedColumns(
    mergedColumns,
    scrollBarSizeOnFixedHolder
  );
  const { fixedColumns, fixedColumnKeys } = useFixedColumns(flattedColumnsWithScrollBar);
  const hasEllipsis = computed(
    () => !!props.ellipsis || flattedColumns.value.some((column) => column.ellipsis)
  );
  const hasFixed = computed(() => flattedColumns.value.some((column) => column.fixed));
  const mergedRows = computed(() => mergeRows(mergedColumns.value, scrollBarColumn.value));
  return {
    flattedColumns,
    scrollBarColumn,
    flattedColumnsWithScrollBar,
    fixedColumns,
    fixedColumnKeys,
    hasEllipsis,
    hasFixed,
    mergedRows
  };
}
function mergeColumns(columns, config) {
  return columns.map((column) => convertColumn(column, config));
}
function convertColumns(nodes) {
  const columns = [];
  flattenNode(nodes, { key: tableColumnKey }).forEach((node, index) => {
    const { props, children } = node;
    const { key = index, editable, ellipsis, ...newColumn } = props || {};
    newColumn.key = key;
    newColumn.editable = editable || editable === "";
    newColumn.ellipsis = ellipsis || ellipsis === "";
    const { default: defaultSlot, cell, title, expand, icon } = children || {};
    if (defaultSlot) {
      newColumn.children = convertColumns(defaultSlot());
    }
    if (cell) {
      newColumn.customCell = cell;
    }
    if (title) {
      newColumn.customTitle = title;
    }
    if (expand) {
      newColumn.customExpand = expand;
    }
    if (icon) {
      newColumn.customIcon = icon;
    }
    columns.push(newColumn);
  });
  return columns;
}
function resolveColumnAlign(align) {
  if (isString(align)) {
    return { title: align, cell: align };
  }
  return align;
}
function convertColumn(column, config) {
  var _a, _b;
  const { columnBase, columnExpandable, columnSelectable, columnIndexable } = config;
  const { align = columnBase.align } = column;
  const key = getColumnKey(column);
  if ("type" in column) {
    const { type } = column;
    if (type === "expandable") {
      const { showLine = columnExpandable.showLine, icon = columnExpandable.icon } = column;
      return { ...column, key, align: resolveColumnAlign(align), icon, showLine };
    }
    if (type === "selectable") {
      const { multiple = true, showIndex = columnSelectable.showIndex } = column;
      return { ...column, key, align: resolveColumnAlign(align), multiple, showIndex };
    }
    if (type === "indexable") {
      const align2 = (_b = (_a = column.align) != null ? _a : columnIndexable.align) != null ? _b : columnBase.align;
      return { ...columnIndexable, ...column, align: resolveColumnAlign(align2), key };
    }
    return { ...column, key, align: resolveColumnAlign(align) };
  } else {
    const { sortable, filterable, children } = column;
    const newColumn = { ...column, key, align: resolveColumnAlign(align) };
    if (sortable) {
      newColumn.sortable = { ...columnBase.sortable, ...sortable };
    }
    if (filterable) {
      newColumn.filterable = { ...columnBase.filterable, ...filterable };
    }
    if (children == null ? void 0 : children.length) {
      newColumn.children = mergeColumns(children, config);
    }
    return newColumn;
  }
}
function useFlattedColumns(mergedColumns, scrollBarSizeOnFixedHolder) {
  const flattedColumns = computed(() => flatColumns(mergedColumns.value));
  const scrollBarColumn = computed(() => {
    const scrollBarSize = scrollBarSizeOnFixedHolder.value;
    const columns = flattedColumns.value;
    if (scrollBarSize === 0 || columns.length === 0) {
      return void 0;
    }
    const lastColumn = columns[columns.length - 1];
    return {
      key: "__IDUX_table_column_key_scroll-bar",
      type: "scroll-bar",
      fixed: lastColumn && lastColumn.fixed,
      width: scrollBarSize
    };
  });
  const flattedColumnsWithScrollBar = computed(() => {
    const columns = flattedColumns.value;
    const scrollBar = scrollBarColumn.value;
    return scrollBar ? [...columns, scrollBar] : columns;
  });
  return { flattedColumns, scrollBarColumn, flattedColumnsWithScrollBar };
}
function flatColumns(columns) {
  const result = [];
  columns.forEach((column) => {
    const { fixed, children: subColumns } = column;
    if (subColumns == null ? void 0 : subColumns.length) {
      let subFlattedColumns = flatColumns(subColumns);
      if (fixed) {
        subFlattedColumns = subFlattedColumns.map((item) => ({ fixed, ...item }));
      }
      result.push(...subFlattedColumns);
    } else {
      result.push(column);
    }
  });
  return result;
}
function useFixedColumns(flattedColumnsWithScrollBar) {
  const fixedColumns = computed(() => {
    const fixedStartColumns = [];
    const fixedEndColumns = [];
    const fixedColumnIndexMap = {};
    flattedColumnsWithScrollBar.value.forEach((column, index) => {
      const { key, fixed } = column;
      if (fixed === "start") {
        fixedStartColumns.push(column);
        fixedColumnIndexMap[key] = index;
      } else if (fixed === "end") {
        fixedEndColumns.push(column);
        fixedColumnIndexMap[key] = index;
      }
    });
    return { fixedStartColumns, fixedEndColumns, fixedColumnIndexMap };
  });
  const fixedColumnKeys = computed(() => {
    var _a, _b;
    const { fixedStartColumns, fixedEndColumns } = fixedColumns.value;
    return { lastStartKey: (_a = fixedStartColumns[fixedStartColumns.length - 1]) == null ? void 0 : _a.key, firstEndKey: (_b = fixedEndColumns[0]) == null ? void 0 : _b.key };
  });
  return { fixedColumns, fixedColumnKeys };
}
function mergeRows(mergedColumns, scrollBarColumn) {
  const rows = [];
  const offsetIndexMap = {};
  function calculateColSpans(columns, colIndex, rowIndex) {
    var _a;
    (_a = rows[rowIndex]) != null ? _a : rows[rowIndex] = [];
    let colStart = colIndex;
    const titleColSpans = columns.map((column) => {
      var _a2;
      let titleColSpan = (_a2 = column.titleColSpan) != null ? _a2 : 1;
      let hasChildren = false;
      const subColumns = column.children;
      if (subColumns == null ? void 0 : subColumns.length) {
        hasChildren = true;
        const subColumnSpans = calculateColSpans(subColumns, colStart, rowIndex + 1);
        if (isNil(column.titleColSpan)) {
          titleColSpan = subColumnSpans.reduce((total, count) => total + count);
        }
      }
      const colEnd = colStart + titleColSpan - 1;
      rows[rowIndex].push({ ...column, titleColSpan, colStart, colEnd, hasChildren });
      offsetIndexMap[column.key] = { colStart, colEnd };
      colStart += titleColSpan;
      return titleColSpan;
    });
    return titleColSpans;
  }
  const rootColumns = scrollBarColumn ? [...mergedColumns, scrollBarColumn] : mergedColumns;
  calculateColSpans(rootColumns, 0, 0);
  const rowCount = rows.length;
  rows.forEach((columns, rowIndex) => {
    columns.forEach((col) => {
      if (!col.hasChildren) {
        col.titleRowSpan = rowCount - rowIndex;
      }
    });
  });
  return { rows, offsetIndexMap };
}
function useDataSource(props, mergedChildrenKey, mergedGetKey, activeSorters, activeFilters, expandedRowKeys, mergedPagination) {
  const mergedData = computed(() => {
    const childrenKey = mergedChildrenKey.value;
    const getKey = mergedGetKey.value;
    return props.dataSource.map(
      (record, idx) => convertMergeData(record, getKey, childrenKey, idx, props.dataSource.length)
    );
  });
  const mergedContext = computed(() => {
    const dataKeyMap = /* @__PURE__ */ new Map();
    const parentKeyMap2 = /* @__PURE__ */ new Map();
    const depthMap2 = /* @__PURE__ */ new Map();
    traverseTree(mergedData.value, "children", (item, parents) => {
      var _a;
      dataKeyMap.set(item.rowKey, item);
      parentKeyMap2.set(item.rowKey, (_a = parents[0]) == null ? void 0 : _a.rowKey);
      depthMap2.set(item.rowKey, parents.length);
    });
    return { dataKeyMap, parentKeyMap: parentKeyMap2, depthMap: depthMap2 };
  });
  const mergedMap = computed(() => mergedContext.value.dataKeyMap);
  const parentKeyMap = computed(() => mergedContext.value.parentKeyMap);
  const depthMap = computed(() => mergedContext.value.depthMap);
  const filteredData = computed(() => filterData(mergedData.value, activeFilters.value, expandedRowKeys.value));
  const sortedData = computed(() => sortData(filteredData.value, activeSorters.value, expandedRowKeys.value));
  const paginatedData = computed(() => {
    const pagination = mergedPagination.value;
    const data = sortedData.value;
    if (pagination === null || !pagination.pageSize) {
      return data;
    } else {
      const { total } = pagination;
      if (total && data.length < total) {
        return data;
      }
      const pageSize = pagination.pageSize;
      const startIndex = (pagination.pageIndex - 1) * pageSize;
      return data.slice(startIndex, startIndex + pageSize);
    }
  });
  const paginatedMap = computed(() => {
    const map = /* @__PURE__ */ new Map();
    traverseTree(paginatedData.value, "children", (item) => {
      map.set(item.rowKey, item);
    });
    return map;
  });
  const flattedData = computed(() => {
    const expandedKeys = expandedRowKeys.value;
    if (expandedKeys.length > 0) {
      return flatData(paginatedData.value, expandedKeys, 0);
    }
    return paginatedData.value;
  });
  const isTreeData = computed(() => mergedData.value.some((data) => {
    var _a;
    return (_a = data.children) == null ? void 0 : _a.length;
  }));
  return {
    isTreeData,
    filteredData,
    flattedData,
    mergedData,
    mergedMap,
    paginatedData,
    paginatedMap,
    parentKeyMap,
    depthMap
  };
}
function convertMergeData(record, getRowKey, childrenKey, index, total, parents) {
  var _a;
  const rowKey = getRowKey(record);
  const result = {
    record,
    rowKey,
    parentKey: parents == null ? void 0 : parents[0].rowKey,
    showLineIndentIndexList: (_a = [...parents != null ? parents : []].reverse()) == null ? void 0 : _a.reduce((res, parent, index2) => {
      if (parent.hasNextSibling) {
        res.push(index2);
      }
      return res;
    }, []),
    hasPrevSibling: index > 0,
    hasNextSibling: index < total - 1
  };
  const subData = record[childrenKey];
  if (subData) {
    result.children = subData.map(
      (subRecord, idx) => convertMergeData(subRecord, getRowKey, childrenKey, idx, subData.length, [result, ...parents != null ? parents : []])
    );
  }
  return result;
}
function sortData(mergedData, activeSorters, expandedRowKeys) {
  const sorters = activeSorters.filter((item) => item.sorter);
  const sorterLength = sorters.length;
  if (sorterLength === 0) {
    return mergedData;
  }
  const tempData = mergedData.slice();
  tempData.forEach((item) => {
    if (expandedRowKeys.includes(item.rowKey) && item.children && item.children.length > 0) {
      item.children = sortData(item.children, activeSorters, expandedRowKeys);
    }
  });
  return tempData.sort((curr, next) => {
    for (let index = 0; index < sorterLength; index++) {
      const { orderBy, sorter } = sorters[index];
      const sorterResult = sorter(curr.record, next.record);
      if (sorterResult !== 0) {
        return orderBy === "ascend" ? sorterResult : -sorterResult;
      }
    }
    return 0;
  });
}
function filterData(mergedData, activeFilters, expandedRowKeys) {
  const filters = activeFilters.filter((item) => item.filter);
  if (filters.length === 0) {
    return mergedData;
  }
  return mergedData.map((item) => {
    const valid = filters.every(({ filter, filterBy }) => filter(filterBy, item.record));
    const { rowKey, children } = item;
    const isExpanded = expandedRowKeys.includes(rowKey);
    let newItem = item;
    if (isExpanded && children && children.length) {
      const newChildren = filterData(children, activeFilters, expandedRowKeys);
      if (newChildren.length !== children.length) {
        newItem = { ...item, children: newChildren };
      }
    }
    return valid || isExpanded && newItem.children && newItem.children.length ? newItem : null;
  }).filter((item) => item !== null);
}
function flatData(mergedData, expandedRowKeys, level) {
  return mergedData.reduce((result, item) => {
    const { children, parentKey, record, rowKey, hasPrevSibling, hasNextSibling, showLineIndentIndexList } = item;
    const expanded = expandedRowKeys.includes(rowKey);
    result.push({
      children,
      parentKey,
      record,
      rowKey,
      level,
      expanded,
      hasPrevSibling,
      hasNextSibling,
      showLineIndentIndexList
    });
    if (expanded && item.children) {
      const childrenFlatData = flatData(item.children, expandedRowKeys, level + 1);
      result.push(...childrenFlatData);
    }
    return result;
  }, []);
}
function useExpandable(props, flattedColumns) {
  const expandable = computed(
    () => flattedColumns.value.find((column) => "type" in column && column.type === "expandable")
  );
  const [expandedRowKeys, setExpandedRowKeys] = useControlledProp(props, "expandedRowKeys", () => []);
  const handleExpandChange = (key, record) => {
    const { onChange, onExpand } = expandable.value || {};
    const tempKeys = [...expandedRowKeys.value];
    const index = tempKeys.indexOf(key);
    const expanded = index >= 0;
    if (expanded) {
      tempKeys.splice(index, 1);
    } else {
      tempKeys.push(key);
    }
    callEmit(onExpand, !expanded, record);
    setExpandedRowKeys(tempKeys);
    callEmit(onChange, tempKeys);
  };
  const checkExpandDisabled = (data) => {
    if (!expandable.value) {
      return true;
    }
    const { disabled, customExpand } = expandable.value;
    const { record } = data;
    if (disabled == null ? void 0 : disabled(record)) {
      return true;
    }
    return !(customExpand || data.children && data.children.length > 0);
  };
  return { expandable, expandedRowKeys, setExpandedRowKeys, checkExpandDisabled, handleExpandChange };
}
function useFilterable(flattedColumns) {
  const filterableColumns = computed(() => flattedColumns.value.filter((column) => column.filterable));
  const activeFilterByMap = reactive({});
  const activeFilters = ref([]);
  watch(
    filterableColumns,
    (currColumns, prevColumns) => {
      currColumns.forEach((currColumn) => {
        const { key, filterable } = currColumn;
        const currFilterBy = filterable.filterBy;
        if (currFilterBy || activeFilterByMap[key] === void 0) {
          activeFilterByMap[key] = currFilterBy || [];
          return;
        }
        const prevColumn = prevColumns ? prevColumns.find((column) => column.key === key) : void 0;
        const prevFilterBy = prevColumn == null ? void 0 : prevColumn.filterable.filterBy;
        if (prevFilterBy) {
          activeFilterByMap[key] = [];
        }
      });
      activeFilters.value = getActiveFilters(currColumns, activeFilterByMap);
    },
    { immediate: true }
  );
  const handleFilter = (activeKey, activeFilterable, filterBy) => {
    const { onChange } = activeFilterable;
    activeFilterByMap[activeKey] = filterBy;
    activeFilters.value = getActiveFilters(filterableColumns.value, activeFilterByMap);
    callEmit(onChange, filterBy, activeFilters.value);
  };
  return {
    activeFilters,
    activeFilterByMap,
    handleFilter
  };
}
function getActiveFilters(filterableColumns, activeFilterByMap) {
  return filterableColumns.map((column) => {
    const { key, filterable } = column;
    const { filter, filterBy = activeFilterByMap[key] } = filterable;
    return { key, filter, filterBy };
  }).filter((item) => item.filter && item.filterBy.length > 0);
}
function usePagination(props, config, mergedSize) {
  var _a, _b, _c, _d, _e;
  const paginationConfig = useGlobalConfig("pagination");
  const tempPagination = computed(() => {
    const { pagination } = props;
    if (pagination === false) {
      return null;
    }
    return pagination === true ? {} : pagination;
  });
  const tempIndex = ref((_b = (_a = tempPagination.value) == null ? void 0 : _a.pageIndex) != null ? _b : 1);
  const tempSize = ref(
    (_e = (_d = (_c = tempPagination.value) == null ? void 0 : _c.pageSize) != null ? _d : config.pagination.pageSize) != null ? _e : paginationConfig.pageSize
  );
  const handlePageChange = (pageIndex, pageSize) => {
    var _a2;
    tempIndex.value = pageIndex;
    tempSize.value = pageSize;
    callEmit((_a2 = tempPagination.value) == null ? void 0 : _a2.onChange, pageIndex, pageSize);
  };
  const mergedPagination = computed(() => {
    var _a2, _b2;
    const pagination = tempPagination.value;
    if (pagination === null) {
      return null;
    }
    return {
      ...config.pagination,
      size: mergedSize.value,
      ...pagination,
      pageIndex: (_a2 = pagination.pageIndex) != null ? _a2 : tempIndex.value,
      pageSize: (_b2 = pagination.pageSize) != null ? _b2 : tempSize.value,
      onChange: handlePageChange
    };
  });
  return { mergedPagination };
}
function useScroll(props, { setStickyScrollLeft }) {
  const virtualScrollRef = ref();
  const headerVirtualScrollRef = ref();
  const scrollHeadRef = ref();
  const scrollBodyRef = ref();
  const scrollContentRef = ref();
  const scrollFootRef = ref();
  watch(
    virtualScrollRef,
    (instance) => {
      if (instance) {
        scrollBodyRef.value = instance.getHolderElement();
      }
    },
    {
      immediate: true
    }
  );
  watch(
    headerVirtualScrollRef,
    (instance) => {
      if (instance) {
        scrollHeadRef.value = instance.getHolderElement();
        scrollHeadRef.value.style.overflow = "hidden";
      }
    },
    {
      immediate: true
    }
  );
  const resizeMark = useTableResized(scrollBodyRef, scrollContentRef);
  const { handleScroll, pingedStart, pingedEnd } = useScrollRef(
    props,
    virtualScrollRef,
    headerVirtualScrollRef,
    scrollHeadRef,
    scrollBodyRef,
    scrollFootRef,
    resizeMark,
    setStickyScrollLeft
  );
  const { scrollHorizontalOverflowed, scrollVerticalOverflowed } = useScrollOverflowed(scrollBodyRef, resizeMark);
  const scrollWidth = computed(() => {
    var _a;
    return convertCssPixel((_a = props.scroll) == null ? void 0 : _a.width);
  });
  const scrollHeight = computed(() => {
    var _a;
    return convertCssPixel((_a = props.scroll) == null ? void 0 : _a.height);
  });
  const scrollBarSize = computed(() => getScrollBarSize(convertElement(scrollBodyRef)));
  const scrollBarSizeOnFixedHolder = computed(() => scrollVerticalOverflowed.value ? scrollBarSize.value : 0);
  const mergedPingedStart = computed(() => pingedStart.value && scrollHorizontalOverflowed.value);
  const mergedPingedEnd = computed(() => pingedEnd.value && scrollHorizontalOverflowed.value);
  const scrollTo = (options) => {
    var _a;
    if (props.virtual) {
      (_a = virtualScrollRef.value) == null ? void 0 : _a.scrollTo(options);
    } else {
      if (typeof options === "number") {
        scrollToTop({ target: convertElement(scrollBodyRef), top: options, duration: 200 });
      } else {
        process.env.NODE_ENV !== "production" && Logger.warn("components/table", "the scrollTo argument must be a number, when virtual is not enabled.");
      }
    }
  };
  return {
    virtualScrollRef,
    headerVirtualScrollRef,
    scrollHeadRef,
    scrollBodyRef,
    scrollContentRef,
    scrollFootRef,
    handleScroll,
    scrollTo,
    pingedStart: mergedPingedStart,
    pingedEnd: mergedPingedEnd,
    scrollWidth,
    scrollHeight,
    scrollBarSize,
    scrollBarSizeOnFixedHolder,
    scrollHorizontalOverflowed,
    scrollVerticalOverflowed
  };
}
function useScrollRef(props, virtualScrollRef, headerVirtualScrollRef, scrollHeadRef, scrollBodyRef, scrollFootRef, resizeMark, setStickyScrollLeft) {
  const pingedStart = ref(false);
  const pingedEnd = ref(false);
  let timeout;
  const clearTimer = () => {
    if (timeout) {
      clearTimeout(timeout);
      timeout = void 0;
    }
  };
  onBeforeUnmount(() => clearTimer());
  const lockedScrollTargetRef = ref();
  const lockScrollTarget = (target) => {
    lockedScrollTargetRef.value = target;
    clearTimer();
    timeout = setTimeout(() => {
      lockedScrollTargetRef.value = void 0;
      timeout = void 0;
    }, 100);
  };
  const forceScroll = (scrollLeft, target) => {
    if (!target) {
      return;
    }
    if (target.scrollLeft !== scrollLeft) {
      target.scrollLeft = scrollLeft;
    }
  };
  const forceVirtualScroll = (scrollLeft, target) => {
    if (!target) {
      return;
    }
    target.scrollTo({ left: scrollLeft });
  };
  const changeStickyScrollLeft = (scrollLeft) => {
    const scrollBodyElement = convertElement(scrollBodyRef);
    if (!scrollBodyElement) {
      return;
    }
    const { clientWidth, scrollWidth } = scrollBodyElement;
    setStickyScrollLeft(scrollLeft / scrollWidth * clientWidth || 0);
  };
  const handleScroll = (evt, scrollLeft) => {
    const currentTarget = evt == null ? void 0 : evt.currentTarget;
    const mergedScrollLeft = scrollLeft != null ? scrollLeft : currentTarget.scrollLeft;
    const lockedTarget = lockedScrollTargetRef.value;
    if (!lockedTarget || lockedTarget === currentTarget) {
      lockScrollTarget(currentTarget);
      headerVirtualScrollRef.value ? forceVirtualScroll(mergedScrollLeft, headerVirtualScrollRef.value) : forceScroll(mergedScrollLeft, scrollHeadRef.value);
      virtualScrollRef.value ? forceVirtualScroll(mergedScrollLeft, virtualScrollRef.value) : forceScroll(mergedScrollLeft, convertElement(scrollBodyRef));
      forceScroll(mergedScrollLeft, scrollFootRef.value);
      changeStickyScrollLeft(mergedScrollLeft);
    }
    if (currentTarget) {
      const { scrollWidth, clientWidth } = currentTarget;
      pingedStart.value = mergedScrollLeft > 1;
      pingedEnd.value = scrollWidth - clientWidth - mergedScrollLeft > 1;
    }
    if ((evt == null ? void 0 : evt.type) === "scroll") {
      callEmit(props.onScroll, evt);
    }
  };
  watch(resizeMark, () => {
    const currentTarget = convertElement(scrollBodyRef);
    if (currentTarget) {
      handleScroll({ currentTarget });
    }
  });
  return { handleScroll, pingedStart, pingedEnd };
}
function useScrollOverflowed(scrollBodyRef, resizeMark) {
  const scrollHorizontalOverflowed = ref(false);
  const scrollVerticalOverflowed = ref(false);
  const calcScrollOverflowed = () => {
    const bodyEl = convertElement(scrollBodyRef.value);
    if (!bodyEl) {
      scrollHorizontalOverflowed.value = false;
      scrollVerticalOverflowed.value = false;
    } else {
      scrollHorizontalOverflowed.value = bodyEl.scrollWidth > bodyEl.clientWidth;
      scrollVerticalOverflowed.value = bodyEl.scrollHeight > bodyEl.clientHeight;
    }
  };
  watch(resizeMark, calcScrollOverflowed);
  return {
    scrollHorizontalOverflowed,
    scrollVerticalOverflowed
  };
}
function useTableResized(scrollBodyRef, scrollContentRef) {
  const resizeMark = ref(false);
  const handleResize = () => {
    resizeMark.value = !resizeMark.value;
  };
  let stopResizeObservers = [];
  const stopHandler = () => stopResizeObservers.forEach((stop) => stop());
  onMounted(() => {
    stopResizeObservers = [
      useResizeObserver(scrollBodyRef, handleResize),
      useResizeObserver(scrollContentRef, handleResize)
    ];
  });
  onBeforeUnmount(() => stopHandler());
  return resizeMark;
}
function useScrollOnChange(props, config, mergedPagination, activeSorters, activeFilters, scrollTo) {
  const mergedScrollToTopOnChange = computed(() => {
    var _a;
    return (_a = props.scrollToTopOnChange) != null ? _a : config.scrollToTopOnChange;
  });
  let watchStopHandlers = [];
  const startOnChangeWatch = () => {
    watchStopHandlers = [
      watch([() => {
        var _a;
        return (_a = mergedPagination.value) == null ? void 0 : _a.pageIndex;
      }, () => {
        var _a;
        return (_a = mergedPagination.value) == null ? void 0 : _a.pageSize;
      }], () => scrollTo(0)),
      watch([activeFilters, activeSorters], ([currentFilters, currentSorters], [formerFilter, formerSorters]) => {
        if (!compareFilters(currentFilters, formerFilter) || !compareSorters(currentSorters, formerSorters)) {
          scrollTo(0);
        }
      })
    ];
  };
  const stopOnChangeWatch = () => {
    watchStopHandlers.forEach((stop) => stop());
  };
  watch(
    mergedScrollToTopOnChange,
    (scrollToTopOnChange) => {
      stopOnChangeWatch();
      if (scrollToTopOnChange) {
        startOnChangeWatch();
      }
    },
    { immediate: true }
  );
}
function compareFilters(currentFilters, formerFilters) {
  return currentFilters.length === formerFilters.length && currentFilters.every((filter) => {
    const formerFilter = formerFilters.find((f) => f.key === filter.key);
    if (!formerFilter) {
      return false;
    }
    return filter.filter === formerFilter.filter && filter.filterBy.length === formerFilter.filterBy.length && filter.filterBy.every((value) => formerFilter.filterBy.includes(value));
  });
}
function compareSorters(currentSorters, formerSorters) {
  return currentSorters.length === formerSorters.length && currentSorters.every((sorter) => {
    const formerSorter = formerSorters.find((s) => s.key === sorter.key);
    if (!formerSorter) {
      return false;
    }
    return sorter.sorter === formerSorter.sorter && sorter.orderBy === formerSorter.orderBy && sorter.multiple === formerSorter.multiple;
  });
}
function useSelectable(props, locale, flattedColumns, { mergedData, mergedMap, paginatedMap, parentKeyMap, depthMap }) {
  const selectable = computed(
    () => flattedColumns.value.find((column) => "type" in column && column.type === "selectable")
  );
  const [selectedRowKeys, setSelectedRowKeys] = useControlledProp(props, "selectedRowKeys", () => []);
  const resolverContext = computed(() => {
    return {
      data: mergedData.value,
      dataMap: mergedMap.value,
      parentKeyMap: parentKeyMap.value,
      depthMap: depthMap.value
    };
  });
  const childrenKey = ref("children");
  const getKey = ref((item) => item.rowKey);
  const cascaderStrategy = computed(() => {
    var _a;
    return ((_a = selectable.value) == null ? void 0 : _a.multiple) ? props.cascaderStrategy : "off";
  });
  const isDisabled = computed(() => {
    var _a;
    const disabled = (_a = selectable.value) == null ? void 0 : _a.disabled;
    return (data) => !!(disabled == null ? void 0 : disabled(data.record));
  });
  const checkStateContext = useTreeCheckState(
    selectedRowKeys,
    resolverContext,
    childrenKey,
    getKey,
    cascaderStrategy,
    isDisabled
  );
  const { checkStateResolver, isCheckDisabled, isChecked, isIndeterminate, toggle, getDataByKeys } = checkStateContext;
  const currentPageRowKeys = computed(() => {
    const enabledRowKeys = [];
    const disabledRowKeys = [];
    paginatedMap.value.forEach((_, key) => {
      if (isCheckDisabled(key)) {
        disabledRowKeys.push(key);
      } else {
        enabledRowKeys.push(key);
      }
    });
    return { enabledRowKeys, disabledRowKeys };
  });
  const currentPageAllSelectState = computed(() => {
    let checked = !!paginatedMap.value.size;
    let allSelected = !!paginatedMap.value.size;
    let someSelected = false;
    for (const key of paginatedMap.value.keys()) {
      if (isChecked(key) || isIndeterminate(key)) {
        someSelected = true;
      } else {
        allSelected = false;
        if (!isCheckDisabled(key)) {
          checked = false;
        }
      }
    }
    if (!allSelected && !someSelected) {
      checked = false;
    }
    return { checked, allSelected, someSelected };
  });
  const currentPageAllSelected = computed(() => currentPageAllSelectState.value.checked);
  const currentPageSomeSelected = computed(
    () => !currentPageAllSelectState.value.allSelected && currentPageAllSelectState.value.someSelected
  );
  const emitChange = (tempRowKeys) => {
    setSelectedRowKeys(tempRowKeys);
    const { onChange } = selectable.value || {};
    if (onChange) {
      const selectedRecords = getDataByKeys(tempRowKeys).map((data) => data.record);
      callEmit(onChange, tempRowKeys, selectedRecords);
    }
  };
  const handleSelectChange = (key, record) => {
    const dataMap = mergedMap.value;
    const currData = dataMap.get(key);
    if (!currData) {
      return;
    }
    const { multiple, onSelect } = selectable.value || {};
    let tempRowKeys = [...selectedRowKeys.value];
    let _checked;
    if (multiple) {
      const { checked, checkedKeys } = toggle(currData);
      _checked = checked;
      tempRowKeys = checkedKeys;
    } else {
      _checked = !isChecked(key);
      tempRowKeys = _checked ? [key] : [];
    }
    callEmit(onSelect, _checked, record);
    emitChange(tempRowKeys);
  };
  const handleHeadSelectChange = () => {
    let newSelectedKeys;
    if (currentPageAllSelected.value) {
      const removedKeys = [];
      paginatedMap.value.forEach((_, key) => {
        if (isCheckDisabled(key)) {
          return;
        }
        if (isChecked(key)) {
          removedKeys.push(key);
        }
      });
      newSelectedKeys = checkStateResolver.removeKeys(selectedRowKeys.value, removedKeys);
    } else {
      const appendedKeys = [];
      paginatedMap.value.forEach((_, key) => {
        if (isCheckDisabled(key)) {
          return;
        }
        if (!isChecked(key)) {
          appendedKeys.push(key);
        }
      });
      newSelectedKeys = checkStateResolver.appendKeys(selectedRowKeys.value, appendedKeys);
    }
    emitChange(newSelectedKeys);
  };
  const mergedSelectableMenus = useMergedMenus(selectable, locale);
  const handleSelectableMenuClick = useMenuClickHandle(
    selectable,
    checkStateContext,
    mergedMap,
    paginatedMap,
    selectedRowKeys,
    emitChange
  );
  return {
    selectable,
    selectedRowKeys,
    isChecked,
    isIndeterminate,
    isCheckDisabled,
    currentPageRowKeys,
    currentPageAllSelected,
    currentPageSomeSelected,
    handleSelectChange,
    handleHeadSelectChange,
    mergedSelectableMenus,
    handleSelectableMenuClick
  };
}
const allMenuItemKey = Symbol("IDUX_TABLE_KEY_selectable-all");
const invertMenuItemKey = Symbol("IDUX_TABLE_KEY_selectable-invert");
const noneMenuItemKey = Symbol("IDUX_TABLE_KEY_selectable-none");
const pageInvertMenuItemKey = Symbol("IDUX_TABLE_KEY_selectable-pageInvert");
function useMergedMenus(selectable, locale) {
  return computed(() => {
    const { menus } = selectable.value || {};
    if (!menus || menus.length === 0) {
      return [];
    }
    const { selectAll, selectInvert, selectNone, selectPageInvert } = locale.table;
    return menus.map((item) => {
      if (isString(item)) {
        if (item === "all") {
          return { type: "item", key: allMenuItemKey, label: selectAll };
        }
        if (item === "invert") {
          return { type: "item", key: invertMenuItemKey, label: selectInvert };
        }
        if (item === "none") {
          return { type: "item", key: noneMenuItemKey, label: selectNone };
        }
        if (item === "pageInvert") {
          return { type: "item", key: pageInvertMenuItemKey, label: selectPageInvert };
        }
        return { type: "item", key: item, label: item };
      }
      return item;
    });
  });
}
function useMenuClickHandle(selectable, checkStateContext, mergedMap, paginatedMap, selectedRowKeys, emitChange) {
  const { checkDisabledKeySet, checkStateResolver, getAllCheckedKeys, isChecked, isCheckDisabled } = checkStateContext;
  const handleSelectAll = () => {
    const { onSelectAll } = selectable.value || {};
    const newSelectedKeys = getAllCheckedKeys([...checkDisabledKeySet.value].filter((key) => !isChecked(key)));
    callEmit(onSelectAll, newSelectedKeys);
    emitChange(newSelectedKeys);
  };
  const handleSelectInvert = () => {
    const { onSelectInvert } = selectable.value || {};
    const appendedKeys = [];
    const removedKeys = [];
    mergedMap.value.forEach((_, key) => {
      if (isCheckDisabled(key)) {
        return;
      }
      if (isChecked(key)) {
        removedKeys.push(key);
      } else {
        appendedKeys.push(key);
      }
    });
    const newSelectedKeys = checkStateResolver.removeKeys(
      checkStateResolver.appendKeys(selectedRowKeys.value, appendedKeys),
      removedKeys
    );
    emitChange(newSelectedKeys);
    callEmit(onSelectInvert, newSelectedKeys);
  };
  const handleSelectNone = () => {
    const { onSelectNone } = selectable.value || {};
    callEmit(onSelectNone);
    emitChange([]);
  };
  const handleSelectPageInvert = () => {
    const { onSelectPageInvert } = selectable.value || {};
    const appendedKeys = [];
    const removedKeys = [];
    paginatedMap.value.forEach((_, key) => {
      if (isCheckDisabled(key)) {
        return;
      }
      if (isChecked(key)) {
        removedKeys.push(key);
      } else {
        appendedKeys.push(key);
      }
    });
    const newSelectedKeys = checkStateResolver.removeKeys(
      checkStateResolver.appendKeys(selectedRowKeys.value, appendedKeys),
      removedKeys
    );
    emitChange(newSelectedKeys);
    callEmit(onSelectPageInvert, newSelectedKeys);
  };
  const menuClickHandles = /* @__PURE__ */ new Map([
    [allMenuItemKey, handleSelectAll],
    [invertMenuItemKey, handleSelectInvert],
    [noneMenuItemKey, handleSelectNone],
    [pageInvertMenuItemKey, handleSelectPageInvert]
  ]);
  return (options) => {
    const key = options.key;
    if (isSymbol(key) && menuClickHandles.has(key)) {
      const handle = menuClickHandles.get(key);
      handle();
      return;
    }
    const { onMenuClick } = selectable.value || {};
    if (!onMenuClick) {
      return;
    }
    const tempRowKeys = [];
    paginatedMap.value.forEach((_, key2) => {
      if (isCheckDisabled(key2)) {
        return;
      }
      tempRowKeys.push(key2);
    });
    callEmit(onMenuClick, options, tempRowKeys);
  };
}
function useSortable(flattedColumns) {
  const sortableColumns = computed(() => flattedColumns.value.filter((column) => column.sortable));
  const multipleEnabled = computed(() => sortableColumns.value.some((column) => column.sortable.multiple !== void 0));
  const activeOrderByMap = reactive({});
  const activeSorters = ref([]);
  watch(
    sortableColumns,
    (currColumns, prevColumns) => {
      currColumns.forEach((currColumn) => {
        const { key, sortable } = currColumn;
        const currOrderBy = sortable.orderBy;
        if (currOrderBy || activeOrderByMap[key] === void 0) {
          activeOrderByMap[key] = currOrderBy;
          return;
        }
        const prevColumn = prevColumns ? prevColumns.find((column) => column.key === key) : void 0;
        const prevOrderBy = prevColumn == null ? void 0 : prevColumn.sortable.orderBy;
        if (prevOrderBy) {
          activeOrderByMap[key] = void 0;
        }
      });
      activeSorters.value = getActiveSorters(currColumns, activeOrderByMap);
    },
    { immediate: true }
  );
  const handleSort = (activeKey, activeSortable) => {
    const { orders, onChange } = activeSortable;
    const currOrderBy = activeOrderByMap[activeKey];
    const nextOrderBy = currOrderBy ? getNextOrderBy(orders, currOrderBy) : getNextOrderBy(orders);
    if (multipleEnabled.value) {
      activeOrderByMap[activeKey] = nextOrderBy;
    } else {
      Object.keys(activeOrderByMap).forEach((key) => {
        if (activeKey === key) {
          activeOrderByMap[key] = nextOrderBy;
        } else {
          activeOrderByMap[key] = void 0;
        }
      });
    }
    activeSorters.value = getActiveSorters(sortableColumns.value, activeOrderByMap);
    callEmit(onChange, nextOrderBy, activeSorters.value);
  };
  return { activeSorters, activeOrderByMap, handleSort };
}
function getNextOrderBy(orders, currOrderBy) {
  if (!currOrderBy) {
    return orders[0];
  }
  return orders[orders.indexOf(currOrderBy) + 1];
}
function getActiveSorters(sortableColumns, activeOrderByMap) {
  return sortableColumns.map((column) => {
    const { key, sortable } = column;
    const { multiple = 0, orderBy = activeOrderByMap[key], sorter } = sortable;
    return { key, multiple, orderBy, sorter };
  }).filter((item) => item.orderBy).sort((c1, c2) => c2.multiple - c1.multiple);
}
function useSticky(props) {
  const isSticky = computed(() => !!props.sticky);
  const mergedSticky = computed(() => {
    const { sticky } = props;
    const {
      offsetTop = 0,
      offsetBottom = 0,
      offsetScroll = 0,
      container = window
    } = isPlainObject(sticky) ? sticky : {};
    return {
      offsetTop,
      offsetBottom,
      offsetScroll,
      container
    };
  });
  const [stickyScrollLeft, setStickyScrollLeft] = useState(0);
  return { isSticky, mergedSticky, stickyScrollLeft, setStickyScrollLeft };
}
function useTableLayout(props, { hasEllipsis, hasFixed }, { scrollWidth, scrollHeight }, isSticky, mergedVirtual, mergedAutoHeight) {
  return computed(() => {
    if (props.tableLayout) {
      return props.tableLayout;
    }
    if (scrollWidth.value && hasFixed.value) {
      return scrollWidth.value === "max-content" ? "auto" : "fixed";
    }
    if (scrollHeight.value || mergedAutoHeight.value || isSticky.value || hasEllipsis.value || mergedVirtual.value.horizontal || mergedVirtual.value.vertical) {
      return "fixed";
    }
    return "auto";
  });
}
const TABLE_TOKEN = Symbol("TABLE_TOKEN");
const tableBodyToken = Symbol("tableBodyToken");
const tableBodyRowToken = Symbol("tableBodyRowToken");
var ColGroup = /* @__PURE__ */ defineComponent({
  props: {
    isFixedHolder: Boolean,
    columns: Array
  },
  setup(props) {
    const {
      flattedColumns,
      flattedColumnsWithScrollBar,
      measuredColumnWidthMap,
      mergedSelectableMenus,
      mergedPrefixCls
    } = inject(TABLE_TOKEN);
    const resolvedColumns = computed(() => {
      const {
        isFixedHolder,
        columns
      } = props;
      if (!columns) {
        return isFixedHolder ? flattedColumnsWithScrollBar.value : flattedColumns.value;
      }
      return columns;
    });
    return () => {
      if (resolvedColumns.value.every((column) => !column.width && !column.type)) {
        return;
      }
      const prefixCls = mergedPrefixCls.value;
      const children = resolvedColumns.value.map((column) => {
        var _a;
        const {
          key,
          type
        } = column;
        const mergedWidth = props.isFixedHolder ? (_a = measuredColumnWidthMap.value[key]) != null ? _a : column.width : column.width;
        const className = type ? normalizeClass({
          [`${prefixCls}-col-${type}`]: true,
          [`${prefixCls}-col-with-dropdown`]: type === "selectable" && mergedSelectableMenus.value.length > 0
        }) : void 0;
        const style = {
          width: convertCssPixel(mergedWidth),
          minWidth: convertCssPixel(column.minWidth),
          maxWidth: convertCssPixel(column.maxWidth)
        };
        return createVNode("col", {
          "key": key,
          "class": className,
          "style": style
        }, null);
      });
      return createVNode("colgroup", null, [children]);
    };
  }
});
const tableProps = {
  expandedRowKeys: { type: Array, default: void 0 },
  selectedRowKeys: { type: Array, default: void 0 },
  autoHeight: { type: Boolean, default: void 0 },
  borderless: { type: Boolean, default: void 0 },
  childrenKey: { type: String, default: void 0 },
  cascaderStrategy: { type: String, default: "all" },
  columns: { type: Array, default: () => [] },
  customAdditional: { type: Object, default: void 0 },
  customTag: { type: Object, default: void 0 },
  dataSource: { type: Array, default: () => [] },
  ellipsis: { type: [Boolean, Object], default: false },
  empty: { type: [String, Object], default: "default" },
  emptyCell: {
    type: [String, Function],
    default: void 0
  },
  getKey: { type: [String, Function], default: void 0 },
  header: { type: [String, Object], default: void 0 },
  headless: { type: Boolean, default: void 0 },
  insetShadow: { type: Boolean, default: void 0 },
  pagination: { type: [Boolean, Object], default: true },
  scroll: { type: Object, default: void 0 },
  size: { type: String, default: void 0 },
  spin: { type: [Boolean, Object], default: void 0 },
  sticky: { type: [Boolean, Object], default: void 0 },
  scrollToTopOnChange: { type: Boolean, default: void 0 },
  tableLayout: { type: String, default: void 0 },
  virtual: { type: Boolean, default: false },
  virtualScrollMode: { type: String, default: void 0 },
  virtualHorizontal: { type: Boolean, default: false },
  virtualItemHeight: { type: Number, default: void 0 },
  virtualColWidth: { type: Number, default: void 0 },
  virtualBufferSize: { type: Number, default: void 0 },
  virtualBufferOffset: { type: Number, default: void 0 },
  "onUpdate:expandedRowKeys": [Function, Array],
  "onUpdate:selectedRowKeys": [Function, Array],
  onScroll: [Function, Array],
  onScrolledChange: [Function, Array],
  onScrolledBottom: [Function, Array]
};
const tableHeadRowProps = {
  columns: { type: Array, required: true }
};
const tableHeadCellProps = {
  column: { type: Object, required: true }
};
const tableBodyRowProps = {
  expanded: { type: Boolean, default: void 0 },
  rowIndex: { type: Number, required: true },
  level: { type: Number, default: void 0 },
  hasPrevSibling: { type: Boolean, default: void 0 },
  hasNextSibling: { type: Boolean, default: void 0 },
  showLineIndentIndexList: { type: Array, default: () => [] },
  hasChildren: { type: Boolean, default: void 0 },
  record: { type: Object, required: true },
  rowData: { type: Object, required: true },
  rowKey: { type: [String, Number, Symbol], required: true }
};
const tableBodyCellProps = {
  column: { type: Object, required: true },
  isLast: { type: Boolean, required: true },
  colSpan: Number,
  rowSpan: Number
};
const tableMeasureCellProps = {
  cellKey: { type: [String, Number, Symbol], required: true },
  changeColumnWidth: { type: Function, required: true }
};
const tableFilterableTriggerProps = {
  activeFilterBy: { type: Array, required: true },
  filterable: { type: Object, required: true },
  onUpdateFilterBy: { type: Function, required: true }
};
const tableSortableTriggerProps = {
  activeOrderBy: { type: String, default: void 0 },
  sortable: { type: Object, required: true }
};
var FilterableTrigger = /* @__PURE__ */ defineComponent({
  props: tableFilterableTriggerProps,
  setup(props) {
    const {
      slots,
      config,
      locale,
      mergedPrefixCls
    } = inject(TABLE_TOKEN);
    const multiple = computed(() => {
      var _a;
      return (_a = props.filterable.multiple) != null ? _a : config.columnBase.filterable.multiple;
    });
    const footer = computed(() => {
      var _a;
      return (_a = props.filterable.footer) != null ? _a : config.columnBase.filterable.footer;
    });
    const initSelectedKeys = props.activeFilterBy;
    const [selectedKeys, setSelectedKeys] = useState(() => initSelectedKeys);
    watch(() => props.activeFilterBy, (keys) => setSelectedKeys(keys));
    const [visible, setVisible] = useState(false);
    const onUpdateVisible = (value) => {
      setVisible(value);
      if (!value) {
        props.onUpdateFilterBy(selectedKeys.value);
      }
    };
    const handleConfirm = () => {
      setVisible(false);
      props.onUpdateFilterBy(selectedKeys.value);
    };
    const handleReset = () => {
      setSelectedKeys(initSelectedKeys);
    };
    const customLabel = (item) => {
      const prefixCls = `${mergedPrefixCls.value}-filterable-menu-label`;
      const Node = multiple.value ? IxCheckbox : IxRadio;
      return createVNode("span", {
        "class": prefixCls
      }, [createVNode(Node, {
        "checked": item.selected,
        "disabled": item.disabled
      }, null), createVNode("span", {
        "class": `${prefixCls}-content`
      }, [item.label])]);
    };
    const renderOverlay = () => {
      const children = [];
      const {
        customMenu,
        menus
      } = props.filterable;
      if (isFunction(customMenu)) {
        children.push(customMenu({
          selectedKeys: selectedKeys.value,
          setSelectedKeys
        }));
      } else if (isString(customMenu) && slots[customMenu]) {
        children.push(slots[customMenu]({
          selectedKeys: selectedKeys.value,
          setSelectedKeys
        }));
      } else {
        children.push(renderMenu(menus, multiple, selectedKeys, setSelectedKeys, customLabel));
      }
      if (footer.value) {
        children.push(renderFooter$1(locale, mergedPrefixCls, handleConfirm, handleReset));
      }
      return children;
    };
    const classes = computed(() => {
      const prefixCls = `${mergedPrefixCls.value}-filterable-trigger`;
      return normalizeClass({
        [prefixCls]: true,
        [`${prefixCls}-active`]: props.activeFilterBy.length > 0
      });
    });
    const renderTrigger2 = () => {
      const {
        customTrigger
      } = props.filterable;
      if (isFunction(customTrigger)) {
        return customTrigger();
      }
      if (isString(customTrigger) && slots[customTrigger]) {
        return slots[customTrigger]();
      }
      return createVNode("span", {
        "class": classes.value,
        "onClick": (evt) => evt.stopPropagation()
      }, [createVNode(IxIcon, {
        "name": "filter-filled"
      }, null)]);
    };
    return () => {
      const dropdownProps = {
        visible: visible.value,
        hideOnClick: !footer.value,
        placement: "bottomEnd",
        trigger: "click",
        "onUpdate:visible": onUpdateVisible
      };
      return createVNode(IxDropdown, dropdownProps, {
        default: renderTrigger2,
        overlay: renderOverlay
      });
    };
  }
});
const renderMenu = (menus, multiple, selectedKeys, setSelectedKeys, customLabel) => {
  const menuProps = {
    dataSource: menus,
    multiple: multiple.value,
    selectable: true,
    selectedKeys: selectedKeys.value,
    "onUpdate:selectedKeys": setSelectedKeys
  };
  const slots = {
    itemLabel: customLabel
  };
  return createVNode(IxMenu, menuProps, slots);
};
const renderFooter$1 = (locale, mergedPrefixCls, handleConfirm, handleReset) => {
  const {
    filterConfirm,
    filterReset
  } = locale.table;
  return createVNode("div", {
    "class": `${mergedPrefixCls.value}-filterable-trigger-footer`
  }, [createVNode(IxButton, {
    "mode": "primary",
    "size": "xs",
    "onClick": handleConfirm
  }, {
    default: () => [filterConfirm]
  }), createVNode(IxButton, {
    "size": "xs",
    "onClick": handleReset
  }, {
    default: () => [filterReset]
  })]);
};
var SelectableTrigger = /* @__PURE__ */ defineComponent({
  setup() {
    const {
      mergedPrefixCls,
      paginatedMap,
      selectable,
      currentPageRowKeys,
      currentPageAllSelected,
      currentPageSomeSelected,
      handleHeadSelectChange,
      mergedSelectableMenus,
      handleSelectableMenuClick
    } = inject(TABLE_TOKEN);
    const disabled = computed(() => {
      const dataCount = paginatedMap.value.size;
      return dataCount === 0 || dataCount === currentPageRowKeys.value.disabledRowKeys.length;
    });
    const [visible, setVisible] = useState(false);
    const dropdownProps = {
      "onUpdate:visible": setVisible,
      trigger: "click"
    };
    const renderOverlay = () => createVNode(IxMenu, {
      "dataSource": mergedSelectableMenus.value,
      "onClick": handleSelectableMenuClick
    }, null);
    const renderTrigger2 = () => createVNode(IxIcon, {
      "class": `${mergedPrefixCls.value}-selectable-trigger`,
      "name": "down",
      "rotate": visible.value ? 180 : 0
    }, null);
    const renderDropDown = () => {
      if (mergedSelectableMenus.value.length === 0) {
        return;
      }
      return createVNode(IxDropdown, dropdownProps, {
        default: renderTrigger2,
        overlay: renderOverlay
      });
    };
    return () => {
      const {
        multiple
      } = selectable.value;
      if (!multiple) {
        return;
      }
      return createVNode("span", {
        "class": `${mergedPrefixCls.value}-cell-triggers`
      }, [createVNode(IxCheckbox, {
        "checked": currentPageAllSelected.value,
        "indeterminate": currentPageSomeSelected.value,
        "disabled": disabled.value,
        "onChange": handleHeadSelectChange
      }, null), renderDropDown()]);
    };
  }
});
var SortableTrigger = /* @__PURE__ */ defineComponent({
  props: tableSortableTriggerProps,
  setup(props) {
    const {
      locale,
      mergedPrefixCls
    } = inject(TABLE_TOKEN);
    return () => {
      const {
        activeOrderBy,
        sortable
      } = props;
      const {
        orders,
        nextTooltip
      } = sortable;
      const title = nextTooltip && getNextTooltipTitle(locale.table, orders, activeOrderBy);
      const sortableTriggerNode = renderSortTrigger(mergedPrefixCls, orders, activeOrderBy);
      return title ? createVNode(IxTooltip, {
        "title": title
      }, {
        default: () => [sortableTriggerNode]
      }) : sortableTriggerNode;
    };
  }
});
function getNextTooltipTitle(locale, orders, activeOrderBy) {
  const nextOrderBy = activeOrderBy ? orders[orders.indexOf(activeOrderBy) + 1] : orders[0];
  const {
    sortCancel,
    sortAsc,
    sortDesc
  } = locale;
  if (!nextOrderBy) {
    return sortCancel;
  }
  return nextOrderBy === "ascend" ? sortAsc : sortDesc;
}
function renderSortTrigger(mergedPrefixCls, orders, activeOrderBy) {
  const prefixCls = mergedPrefixCls.value;
  const activeClassName = `${prefixCls}-sortable-trigger-active`;
  const upNode = orders.includes("ascend") && createVNode(IxIcon, {
    "name": "caret-up-filled",
    "class": activeOrderBy === "ascend" ? activeClassName : void 0
  }, null);
  const downNode = orders.includes("descend") && createVNode(IxIcon, {
    "name": "caret-down-filled",
    "class": activeOrderBy === "descend" ? activeClassName : void 0
  }, null);
  return createVNode("span", {
    "class": `${prefixCls}-sortable-trigger`
  }, [upNode, downNode]);
}
var HeadCell = /* @__PURE__ */ defineComponent({
  props: tableHeadCellProps,
  setup(props) {
    const {
      props: tableProps2,
      slots,
      mergedPrefixCls,
      fixedColumnKeys,
      columnOffsetsWithScrollBar,
      isSticky,
      handleSort,
      activeOrderByMap,
      activeFilterByMap,
      mergedRows,
      handleFilter
    } = inject(TABLE_TOKEN);
    const mergedEllipsis = computed(() => {
      const {
        type,
        ellipsis
      } = props.column;
      const _ellipsis = type ? ellipsis : ellipsis != null ? ellipsis : tableProps2.ellipsis;
      return isObject(_ellipsis) && _ellipsis.head === false ? void 0 : _ellipsis;
    });
    const classes = computed(() => {
      const {
        type,
        align,
        fixed,
        hasChildren,
        key,
        sortable,
        filterable
      } = props.column;
      const prefixCls = mergedPrefixCls.value;
      let classes2 = {
        [`${prefixCls}-cell`]: true,
        [`${prefixCls}-cell-${type}`]: !!type,
        [`${prefixCls}-cell-filterable`]: !!filterable,
        [`${prefixCls}-cell-sortable`]: !!sortable,
        [`${prefixCls}-cell-align-center`]: hasChildren || (align == null ? void 0 : align.title) === "center",
        [`${prefixCls}-cell-align-end`]: !hasChildren && (align == null ? void 0 : align.title) === "end",
        [`${prefixCls}-cell-ellipsis`]: !!mergedEllipsis.value
      };
      if (fixed) {
        const {
          lastStartKey,
          firstEndKey
        } = fixedColumnKeys.value;
        classes2 = {
          ...classes2,
          [`${prefixCls}-fix-start`]: fixed === "start",
          [`${prefixCls}-fix-start-last`]: lastStartKey === key,
          [`${prefixCls}-fix-end`]: fixed === "end",
          [`${prefixCls}-fix-end-first`]: firstEndKey === key,
          [`${prefixCls}-fix-sticky`]: isSticky.value
        };
      }
      return normalizeClass(classes2);
    });
    const style = computed(() => {
      var _a, _b;
      const {
        key,
        fixed
      } = props.column;
      const {
        offsetIndexMap
      } = mergedRows.value;
      if (!fixed) {
        return;
      }
      const {
        starts,
        ends
      } = columnOffsetsWithScrollBar.value;
      const offsets = Object.values(fixed === "start" ? starts : ends);
      const offsetIndex = (_a = offsetIndexMap[key]) == null ? void 0 : _a[fixed === "start" ? "colStart" : "colEnd"];
      if (isNil(offsetIndex)) {
        return {};
      }
      const fixedOffset = convertCssPixel((_b = offsets.find((offset) => offset.index === offsetIndex)) == null ? void 0 : _b.offset);
      return {
        position: "sticky",
        left: fixed === "start" ? fixedOffset : void 0,
        right: fixed === "end" ? fixedOffset : void 0
      };
    });
    const activeSortOrderBy = computed(() => activeOrderByMap[props.column.key]);
    const activeFilterBy = computed(() => activeFilterByMap[props.column.key] || NoopArray);
    const onUpdateFilterBy = (filterBy) => {
      const {
        key,
        filterable
      } = props.column;
      handleFilter(key, filterable, filterBy);
    };
    const onClick = () => {
      const {
        key,
        sortable
      } = props.column;
      if (sortable) {
        handleSort(key, sortable);
      }
    };
    return () => {
      var _a, _b, _c;
      const {
        column
      } = props;
      const {
        type,
        titleColSpan,
        titleRowSpan
      } = column;
      if (type === "scroll-bar") {
        return createVNode("th", {
          "class": classes.value,
          "style": style.value,
          "colspan": titleColSpan === 1 ? void 0 : titleColSpan,
          "rowspan": titleRowSpan === 1 ? void 0 : titleRowSpan,
          "onClick": onClick
        }, null);
      }
      const prefixCls = mergedPrefixCls.value;
      let _title;
      let children;
      if (type === "selectable") {
        children = createVNode(SelectableTrigger, null, null);
      } else {
        const {
          title,
          customTitle,
          sortable,
          filterable
        } = column;
        const ellipsis = mergedEllipsis.value;
        children = renderChildren$1(title, customTitle, slots);
        _title = getColTitle(ellipsis, children, title);
        const iconTriggers = renderTrigger(sortable, activeSortOrderBy, filterable, activeFilterBy, onUpdateFilterBy);
        if (iconTriggers.length > 0) {
          children = createVNode("span", {
            "class": `${prefixCls}-cell-triggers`
          }, [createVNode("span", {
            "class": `${prefixCls}-cell-title`
          }, [children]), iconTriggers]);
        } else if (ellipsis) {
          children = createVNode("span", {
            "class": `${prefixCls}-cell-title`
          }, [children]);
        }
      }
      const customAdditionalFn = (_a = tableProps2.customAdditional) == null ? void 0 : _a.headCell;
      const customAdditional = customAdditionalFn ? customAdditionalFn({
        column
      }) : void 0;
      const Tag = (_c = (_b = tableProps2.customTag) == null ? void 0 : _b.headCell) != null ? _c : "th";
      return createVNode(Tag, mergeProps({
        "class": classes.value,
        "style": style.value,
        "colSpan": titleColSpan === 1 ? void 0 : titleColSpan,
        "rowSpan": titleRowSpan === 1 ? void 0 : titleRowSpan,
        "title": _title
      }, customAdditional, {
        "onClick": onClick
      }), {
        default: () => [children]
      });
    };
  }
});
function renderChildren$1(title, customTitle, slots) {
  let children = title;
  if (isFunction(customTitle)) {
    children = customTitle({
      title
    });
  } else if (isString(customTitle) && slots[customTitle]) {
    children = slots[customTitle]({
      title
    });
  }
  return children;
}
function renderTrigger(sortable, activeSortOrderBy, filterable, activeFilterBy, onUpdateFilterBy) {
  const children = [];
  sortable && children.push(createVNode(SortableTrigger, {
    "activeOrderBy": activeSortOrderBy.value,
    "sortable": sortable
  }, null));
  if (filterable) {
    children.push(createVNode(FilterableTrigger, {
      "activeFilterBy": activeFilterBy.value,
      "filterable": filterable,
      "onUpdateFilterBy": onUpdateFilterBy
    }, null));
  }
  return children;
}
function renderHeaderCells(columns) {
  return columns.map((column) => renderHeaderCell(column)).filter(Boolean);
}
function renderHeaderCell(column) {
  if (column.titleColSpan === 0) {
    return;
  }
  return createVNode(HeadCell, {
    "key": column.key,
    "column": column
  }, null);
}
var HeaderRow = /* @__PURE__ */ defineComponent({
  props: tableHeadRowProps,
  setup(props, {
    slots
  }) {
    const {
      props: tableProps2,
      mergedPrefixCls
    } = inject(TABLE_TOKEN);
    return () => {
      var _a, _b, _c;
      const {
        columns
      } = props;
      const customAdditionalFn = (_a = tableProps2.customAdditional) == null ? void 0 : _a.headRow;
      const customAdditional = customAdditionalFn ? customAdditionalFn({
        columns
      }) : void 0;
      const Tag = (_c = (_b = tableProps2.customTag) == null ? void 0 : _b.headRow) != null ? _c : "tr";
      return createVNode(Tag, mergeProps({
        "class": `${mergedPrefixCls.value}-row`
      }, customAdditional), {
        default: () => {
          var _a2;
          return [(_a2 = slots.default) == null ? void 0 : _a2.call(slots)];
        }
      });
    };
  }
});
function renderHeaderRow(columns, index, headerCells) {
  return createVNode(HeaderRow, {
    "columns": columns,
    "key": index
  }, {
    default: () => [headerCells]
  });
}
var Head = /* @__PURE__ */ defineComponent({
  setup(_, {
    slots
  }) {
    const {
      props: tableProps2,
      mergedRows,
      mergedPrefixCls
    } = inject(TABLE_TOKEN);
    return () => {
      var _a, _b, _c;
      let children;
      const {
        rows
      } = mergedRows.value;
      const customAdditionalFn = (_a = tableProps2.customAdditional) == null ? void 0 : _a.head;
      const customAdditional = customAdditionalFn ? customAdditionalFn({
        rows
      }) : void 0;
      const Tag = (_c = (_b = tableProps2.customTag) == null ? void 0 : _b.head) != null ? _c : "thead";
      if (slots.default) {
        children = slots.default();
      } else {
        children = rows.map((columns, rowIndex) => {
          const cells = renderHeaderCells(columns);
          return renderHeaderRow(columns, rowIndex, cells);
        });
      }
      return createVNode(Tag, mergeProps({
        "class": `${mergedPrefixCls.value}-thead`
      }, customAdditional), {
        default: () => [children]
      });
    };
  }
});
var FixedHolder = /* @__PURE__ */ defineComponent({
  props: {
    offsetTop: {
      type: Number,
      default: void 0
    },
    offsetBottom: {
      type: Number,
      default: void 0
    }
  },
  setup(props) {
    const {
      props: tableProps2,
      mergedPrefixCls,
      mergedVirtual,
      mergedVirtualColWidth,
      getVirtualColWidth,
      headerVirtualScrollRef,
      handleScroll,
      scrollHeadRef,
      scrollWidth,
      flattedData,
      flattedColumns,
      fixedColumns,
      mergedRows,
      isSticky
    } = inject(TABLE_TOKEN);
    useScrollEvents(scrollHeadRef, handleScroll);
    const isMaxContent = computed(() => scrollWidth.value === "max-content");
    const hasData = computed(() => flattedData.value.length > 0);
    const classes = computed(() => {
      const prefixCls = mergedPrefixCls.value;
      return {
        [`${prefixCls}-fixed-holder`]: true,
        [`${prefixCls}-sticky-holder`]: isSticky.value
      };
    });
    const style = computed(() => {
      const sticky = isSticky.value;
      const {
        offsetTop,
        offsetBottom
      } = props;
      return {
        overflow: "hidden",
        top: sticky ? convertCssPixel(offsetTop) : void 0,
        bottom: sticky ? convertCssPixel(offsetBottom) : void 0
      };
    });
    const virtualData = computed(() => {
      if (!mergedVirtual.value.horizontal) {
        return [];
      }
      return mergedRows.value.rows.map((columns, rowIdx) => {
        return {
          key: rowIdx,
          data: columns
        };
      });
    });
    return () => {
      const showColGroup = hasData.value || !isMaxContent.value;
      if (mergedVirtual.value.horizontal) {
        const contentRender = (children, {
          renderedData
        }) => {
          let flattedColumns2 = [];
          if (showColGroup) {
            const columns = [];
            renderedData.forEach((row) => {
              columns.push(...row.data);
            });
            flattedColumns2 = flatColumns(columns);
          }
          return createVNode("table", {
            "style": {
              tableLayout: "fixed"
            }
          }, [showColGroup && createVNode(ColGroup, {
            "columns": flattedColumns2,
            "isFixedHolder": true
          }, null), createVNode(Head, null, {
            default: () => [children]
          })]);
        };
        const rowRender = ({
          item,
          index,
          children
        }) => renderHeaderRow(item.data, index, children);
        const colRneder = ({
          item
        }) => renderHeaderCell(item);
        const colModifier = (renderedRow, renderedCols) => {
          const {
            fixedStartColumns,
            fixedEndColumns
          } = fixedColumns.value;
          return modifyVirtualData(renderedRow, renderedCols, flattedColumns.value, flattedData.value, fixedStartColumns, fixedEndColumns, true);
        };
        const {
          virtualBufferSize,
          virtualBufferOffset
        } = tableProps2;
        return createVNode("div", {
          "class": classes.value,
          "style": style.value
        }, [createVNode(CdkVirtualScroll, {
          "ref": headerVirtualScrollRef,
          "dataSource": virtualData.value,
          "colModifier": colModifier,
          "getKey": "key",
          "height": 0,
          "width": "100%",
          "colWidth": mergedVirtualColWidth.value,
          "getColWidth": getVirtualColWidth,
          "rowRender": rowRender,
          "colRender": colRneder,
          "contentRender": contentRender,
          "virtual": {
            horizontal: true,
            vertical: false
          },
          "bufferSize": virtualBufferSize,
          "bufferOffset": virtualBufferOffset
        }, null)]);
      }
      return createVNode("div", {
        "class": classes.value,
        "style": style.value,
        "ref": scrollHeadRef
      }, [createVNode("table", {
        "style": {
          tableLayout: "fixed"
        }
      }, [showColGroup && createVNode(ColGroup, {
        "isFixedHolder": true
      }, null), createVNode(Head, null, null)])]);
    };
  }
});
function useScrollEvents(scrollHeadRef, handleScroll) {
  const onWheel = (evt) => {
    if (!evt.shiftKey) {
      return;
    }
    const delta = evt.deltaY;
    const currentTarget = evt.currentTarget;
    if (delta) {
      const scrollLeft = currentTarget.scrollLeft + delta;
      handleScroll(evt, scrollLeft);
    }
  };
  const bindWheelEvt = (el) => {
    on(el, "wheel", onWheel, {
      passive: true
    });
  };
  const unbindWheelEvt = (el) => {
    off(el, "wheel", onWheel);
  };
  onMounted(() => {
    watch(scrollHeadRef, (scrollHead, oldScrollHead) => {
      unbindWheelEvt(oldScrollHead);
      bindWheelEvt(scrollHead);
    }, {
      immediate: true
    });
  });
  onBeforeUnmount(() => unbindWheelEvt(scrollHeadRef.value));
}
var StickyScroll = /* @__PURE__ */ defineComponent({
  setup() {
    const {
      mergedPrefixCls,
      scrollBodyRef,
      handleScroll,
      mergedSticky,
      stickyScrollLeft,
      setStickyScrollLeft
    } = inject(TABLE_TOKEN);
    const isShow = ref(false);
    const isActive = ref(false);
    const delta = ref(0);
    const bodyClientWidth = ref(0);
    const bodyScrollWidth = ref(0);
    const scrollBarWidth = computed(() => {
      const clientWidth = bodyClientWidth.value;
      const scrollWidth = bodyScrollWidth.value;
      return scrollWidth && clientWidth * (clientWidth / scrollWidth);
    });
    watchEffect(() => {
      const scrollBodyElement = convertElement(scrollBodyRef);
      if (!scrollBodyElement) {
        return;
      }
      if (isShow.value) {
        const {
          clientWidth,
          scrollWidth,
          scrollLeft
        } = scrollBodyElement;
        setStickyScrollLeft(scrollLeft / scrollWidth * clientWidth);
        bodyClientWidth.value = clientWidth;
        bodyScrollWidth.value = scrollWidth;
      }
    });
    const style = computed(() => {
      return {
        height: getScrollBarSize(),
        width: bodyClientWidth.value,
        bottom: mergedSticky.value.offsetScroll
      };
    });
    const thumbClasses = computed(() => {
      const prefixCls = mergedPrefixCls.value;
      return {
        [`${prefixCls}-sticky-scrollbar-thumb`]: true,
        [`${prefixCls}-sticky-scrollbar-thumb-active`]: isActive.value
      };
    });
    const thumbStyle = computed(() => {
      return {
        width: `${scrollBarWidth.value}px`,
        transform: `translate3d(${stickyScrollLeft.value}px, 0, 0)`
      };
    });
    const handleMouseUp = () => isActive.value = false;
    const handleMouseDown = (evt) => {
      delta.value = evt.pageX - stickyScrollLeft.value;
      isActive.value = true;
      evt.preventDefault();
    };
    const handleMouseMove = (evt) => {
      if (!isActive.value) {
        return;
      }
      const {
        buttons
      } = evt || window.event;
      if (buttons === 0) {
        isActive.value = false;
        return;
      }
      let left = evt.pageX - delta.value;
      if (left <= 0) {
        left = 0;
      }
      const clientWidth = bodyClientWidth.value;
      const scrollWidth = bodyScrollWidth.value;
      const barWidth = scrollBarWidth.value;
      if (left + barWidth >= clientWidth) {
        left = clientWidth - barWidth;
      }
      const scrollLeft = left / clientWidth * (scrollWidth + 2);
      handleScroll(evt, scrollLeft);
    };
    const handleContainerScroll = () => {
      const scrollBodyElement = convertElement(scrollBodyRef);
      const offsetTop = getOffset(scrollBodyElement).top;
      const offsetBottom = offsetTop + scrollBodyElement.offsetHeight;
      const {
        container,
        offsetScroll
      } = mergedSticky.value;
      const offsetClient = container === window ? document.documentElement.scrollTop + window.innerHeight : getOffset(container).top + container.clientHeight;
      if (offsetBottom - getScrollBarSize() <= offsetClient || offsetTop >= offsetClient - offsetScroll) {
        isShow.value = false;
      } else {
        isShow.value = true;
      }
    };
    onMounted(() => {
      on(document.body, "mouseup", handleMouseUp);
      on(document.body, "mousemove", handleMouseMove);
      on(window, "resize", handleContainerScroll);
      watch(() => mergedSticky.value.container, (currContainer, oldContainer) => {
        off(oldContainer, "scroll", handleContainerScroll);
        on(currContainer, "scroll", handleContainerScroll);
      }, {
        immediate: true
      });
      handleContainerScroll();
    });
    onBeforeUnmount(() => {
      off(document.body, "mouseup", handleMouseUp);
      off(document.body, "mousemove", handleMouseMove);
      off(window, "resize", handleContainerScroll);
      off(mergedSticky.value.container, "scroll", handleContainerScroll);
    });
    return () => {
      if (!scrollBodyRef.value || !isShow.value || bodyScrollWidth.value <= bodyClientWidth.value || !scrollBarWidth.value) {
        return null;
      }
      return createVNode("div", {
        "class": `${mergedPrefixCls.value}-sticky-scrollbar`,
        "style": style.value
      }, [createVNode("div", {
        "class": thumbClasses.value,
        "style": thumbStyle.value,
        "onMousedown": handleMouseDown
      }, null)]);
    };
  }
});
var Body = /* @__PURE__ */ defineComponent({
  setup(_, {
    slots
  }) {
    const {
      props,
      mergedPrefixCls
    } = inject(TABLE_TOKEN);
    return () => {
      var _a, _b;
      const Tag = (_b = (_a = props.customTag) == null ? void 0 : _a.body) != null ? _b : "tbody";
      return createVNode(Tag, {
        "class": `${mergedPrefixCls.value}-tbody`
      }, {
        default: () => {
          var _a2;
          return [(_a2 = slots.default) == null ? void 0 : _a2.call(slots)];
        }
      });
    };
  }
});
var BodyRowSingle = /* @__PURE__ */ defineComponent({
  props: {
    columns: Array,
    isEmpty: Boolean
  },
  setup(props, {
    slots
  }) {
    const {
      clientWidth,
      mergedPrefixCls,
      hasFixed,
      scrollHorizontalOverflowed,
      scrollBarColumn
    } = inject(TABLE_TOKEN);
    const columnCount = computed(() => {
      var _a, _b;
      return (_b = (_a = props.columns) == null ? void 0 : _a.length) != null ? _b : 1;
    });
    return () => {
      let children = slots.default();
      if (props.isEmpty ? scrollHorizontalOverflowed.value : hasFixed.value) {
        const scrollBar = scrollBarColumn.value;
        children = createVNode("div", {
          "class": `${mergedPrefixCls.value}-fixed-row`,
          "style": {
            width: convertCssPixel(clientWidth.value - (scrollBar ? scrollBar.width : 0)),
            position: "sticky",
            left: 0,
            overflow: "hidden"
          }
        }, [children]);
      }
      return createVNode("tr", null, [createVNode("td", {
        "colspan": columnCount.value
      }, [children])]);
    };
  }
});
var MeasureCell = /* @__PURE__ */ defineComponent({
  props: tableMeasureCellProps,
  setup(props) {
    const cellRef = ref();
    const handleResize = (evt) => {
      const {
        contentRect: {
          width
        }
      } = evt;
      props.changeColumnWidth(props.cellKey, width);
    };
    useResizeObserver(cellRef, handleResize);
    onBeforeUnmount(() => {
      props.changeColumnWidth(props.cellKey, false);
    });
    return () => createVNode("td", {
      "ref": cellRef,
      "style": {
        padding: 0,
        border: 0,
        height: 0
      }
    }, [createVNode("div", {
      "style": {
        height: 0,
        overflow: "hidden"
      }
    }, [createTextVNode("\xA0")])]);
  }
});
var MeasureRow = /* @__PURE__ */ defineComponent({
  props: {
    columns: Array
  },
  setup(props) {
    const {
      mergedPrefixCls
    } = inject(TABLE_TOKEN);
    const {
      changeColumnWidth
    } = inject(tableBodyToken);
    return () => {
      var _a;
      const children = (_a = props.columns) == null ? void 0 : _a.map((column) => {
        const {
          key
        } = column;
        const cellProps = {
          key,
          cellKey: key,
          changeColumnWidth
        };
        return createVNode(MeasureCell, cellProps, null);
      });
      return createVNode("tr", {
        "class": `${mergedPrefixCls.value}-measure-row`,
        "style": {
          fontSize: 0,
          height: 0
        },
        "aria-hidden": true
      }, [children]);
    };
  }
});
var BodyCell = /* @__PURE__ */ defineComponent({
  props: tableBodyCellProps,
  setup(props) {
    const {
      props: tableProps2,
      slots,
      config,
      mergedPrefixCls,
      mergedEmptyCell,
      activeOrderByMap,
      fixedColumnKeys,
      columnOffsets,
      isSticky,
      isTreeData,
      expandable,
      selectable,
      mergedPagination
    } = inject(TABLE_TOKEN);
    const rowContext = inject(tableBodyRowToken);
    const rowProps = rowContext.props;
    const activeSortOrderBy = computed(() => activeOrderByMap[props.column.key]);
    const dataValue = useDataValue(rowContext, props);
    const isFixStartLast = computed(() => fixedColumnKeys.value.lastStartKey === props.column.key);
    const isFixEndFirst = computed(() => fixedColumnKeys.value.firstEndKey === props.column.key);
    const mergedEllipsis = computed(() => {
      const {
        type,
        ellipsis
      } = props.column;
      return type ? ellipsis : ellipsis != null ? ellipsis : tableProps2.ellipsis;
    });
    const classes = computed(() => {
      const {
        fixed,
        align,
        type
      } = props.column;
      const prefixCls = mergedPrefixCls.value;
      let classes2 = {
        [`${prefixCls}-cell`]: true,
        [`${prefixCls}-cell-sorted`]: !!activeSortOrderBy.value,
        [`${prefixCls}-cell-align-${align.cell}`]: !!align && align.cell != "start",
        [`${prefixCls}-cell-ellipsis`]: !!mergedEllipsis.value,
        [`${prefixCls}-cell-${type}`]: !!type,
        [`${prefixCls}-cell-last`]: !!props.isLast
      };
      if (fixed) {
        classes2 = {
          ...classes2,
          [`${prefixCls}-fix-start`]: fixed === "start",
          [`${prefixCls}-fix-start-last`]: isFixStartLast.value,
          [`${prefixCls}-fix-end`]: fixed === "end",
          [`${prefixCls}-fix-end-first`]: isFixEndFirst.value,
          [`${prefixCls}-fix-sticky`]: isSticky.value
        };
      }
      return normalizeClass(classes2);
    });
    const style = computed(() => {
      var _a;
      const {
        fixed
      } = props.column;
      if (!fixed) {
        return;
      }
      const {
        starts,
        ends
      } = columnOffsets.value;
      const offsets = fixed === "start" ? starts : ends;
      const fixedOffset = convertCssPixel((_a = offsets[props.column.key]) == null ? void 0 : _a.offset);
      if (isNil(fixedOffset)) {
        return {};
      }
      return {
        position: "sticky",
        left: fixed === "start" ? fixedOffset : void 0,
        right: fixed === "end" ? fixedOffset : void 0
      };
    });
    return () => {
      var _a, _b, _c;
      const {
        column
      } = props;
      const prefixCls = mergedPrefixCls.value;
      const {
        type
      } = column;
      let children;
      let title;
      if (type === "selectable") {
        children = renderSelectableChildren(rowContext, slots, selectable, config, mergedPagination, prefixCls);
      } else if (type === "indexable") {
        children = renderIndexableChildren(rowContext, slots, column, mergedPagination, prefixCls);
      } else {
        const text = dataValue.value;
        children = renderChildren(rowContext, props, slots, text);
        title = getColTitle(mergedEllipsis.value, children, text);
        if (!type && (isNil(children) || children === "")) {
          const emptyCellRender = slots.emptyCell || mergedEmptyCell.value;
          children = isFunction(emptyCellRender) ? emptyCellRender({
            column,
            record: rowProps.record,
            rowIndex: rowProps.rowIndex
          }) : emptyCellRender;
        }
      }
      if (props.column.fixed && mergedEllipsis.value && (isFixStartLast.value || isFixEndFirst.value)) {
        children = createVNode("span", {
          "class": `${prefixCls}-cell-content`
        }, [children]);
      }
      const customAdditionalFn = (_a = tableProps2.customAdditional) == null ? void 0 : _a.bodyCell;
      const customAdditional = customAdditionalFn ? customAdditionalFn({
        column,
        record: rowProps.record,
        rowIndex: rowProps.rowIndex
      }) : void 0;
      const Tag = (_c = (_b = tableProps2.customTag) == null ? void 0 : _b.bodyCell) != null ? _c : "td";
      const contentNode = type === "expandable" ? createVNode("div", {
        "class": `${prefixCls}-expandable-wrapper`
      }, [renderExpandableChildren(rowContext, slots, expandable, isTreeData, prefixCls), !isEmptyNode(children) && createVNode("span", {
        "class": `${prefixCls}-expandable-trigger-gap`
      }, null), children]) : children;
      return createVNode(Tag, mergeProps({
        "class": classes.value,
        "style": style.value,
        "title": title,
        "colSpan": props.colSpan,
        "rowSpan": props.rowSpan
      }, customAdditional), {
        default: () => [contentNode]
      });
    };
  }
});
function useDataValue(context, props) {
  return computed(() => {
    const {
      props: {
        record
      }
    } = context;
    const {
      column
    } = props;
    const dataKeys = convertArray(column.dataKey);
    if (dataKeys.length <= 0) {
      return void 0;
    }
    let value = record;
    for (let index = 0; index < dataKeys.length; index++) {
      if (!value) {
        break;
      }
      const key = dataKeys[index];
      value = value[key];
    }
    return value;
  });
}
function renderChildren(context, props, slots, value) {
  const {
    props: {
      record,
      rowIndex
    }
  } = context;
  const {
    column
  } = props;
  const {
    customCell
  } = column;
  const cellRender = isString(customCell) ? slots[customCell] : customCell;
  return cellRender ? cellRender({
    value,
    record,
    rowIndex
  }) : value;
}
function renderExpandableChildren(context, slots, expandable, isTreeData, prefixCls) {
  var _a;
  const {
    icon,
    customIcon,
    indent,
    showLine
  } = expandable.value;
  const {
    props: {
      record,
      expanded,
      level = 0,
      hasPrevSibling,
      hasNextSibling,
      showLineIndentIndexList
    },
    expandDisabled,
    handleExpend
  } = context;
  const hasParent = level > 0;
  const mergedShowLine = isTreeData.value && showLine && indent;
  let iconNode;
  const iconRender = (_a = isString(customIcon) ? slots[customIcon] : customIcon) != null ? _a : icon;
  if (isFunction(iconRender)) {
    iconNode = iconRender({
      expanded: !!expanded,
      record
    });
  } else {
    iconNode = isString(iconRender) ? createVNode(IxIcon, {
      "name": iconRender,
      "rotate": expanded ? 90 : 0
    }, null) : iconRender;
  }
  const indentStyle = indent ? `width: ${convertCssPixel(indent)}` : void 0;
  const triggerCls = {
    [`${prefixCls}-expandable-trigger`]: true,
    [`${prefixCls}-expandable-trigger-show-line`]: mergedShowLine,
    [`${prefixCls}-expandable-trigger-disabled`]: expandDisabled.value
  };
  const indents = [];
  if (indent) {
    for (let i = 0; i < level; i++) {
      indents.push(createVNode("div", {
        "class": {
          [`${prefixCls}-expandable-indent`]: true,
          [`${prefixCls}-expandable-indent-show-line`]: mergedShowLine && showLineIndentIndexList.includes(i)
        },
        "style": indentStyle
      }, null));
    }
  }
  const handleClick = (evt) => {
    evt.preventDefault();
    evt.stopImmediatePropagation();
    if (!expandDisabled.value) {
      handleExpend();
    }
  };
  return [...indents, createVNode("span", {
    "class": triggerCls,
    "style": indentStyle
  }, [createVNode("button", {
    "class": `${prefixCls}-expandable-trigger-button`,
    "type": "button",
    "onClick": handleClick
  }, [iconNode]), mergedShowLine && [createVNode("span", {
    "class": `${prefixCls}-expandable-trigger-line-right`
  }, null), (hasPrevSibling || hasParent) && createVNode("span", {
    "class": `${prefixCls}-expandable-trigger-line-top`
  }, null), hasNextSibling && createVNode("span", {
    "class": `${prefixCls}-expandable-trigger-line-bottom`
  }, null)]])].filter(Boolean);
}
function renderSelectableChildren(rowContext, slots, selectable, config, mergedPagination, prefixCls) {
  const {
    props: {
      record,
      rowIndex
    },
    isSelected,
    isIndeterminate,
    selectDisabled,
    isHover,
    handleSelect: onChange
  } = rowContext;
  const {
    showIndex,
    multiple,
    customCell
  } = selectable.value;
  const onClick = (evt) => {
    evt.stopPropagation();
    if (!multiple && isSelected.value && !selectDisabled.value && onChange) {
      onChange();
    }
  };
  if (!isSelected.value && !isHover.value && showIndex) {
    return renderIndexableChildren(rowContext, slots, config.columnIndexable, mergedPagination, prefixCls);
  }
  const customRender = isString(customCell) ? slots[customCell] : customCell;
  if (multiple) {
    const checkboxProps = {
      checked: isSelected.value,
      disabled: selectDisabled.value,
      indeterminate: isIndeterminate.value,
      onChange,
      onClick
    };
    return customRender ? customRender({
      ...checkboxProps,
      record,
      rowIndex
    }) : createVNode(IxCheckbox, checkboxProps, null);
  } else {
    const radioProps = {
      checked: isSelected.value,
      disabled: selectDisabled.value,
      onChange,
      onClick
    };
    return customRender ? customRender({
      ...radioProps,
      record,
      rowIndex
    }) : createVNode(IxRadio, radioProps, null);
  }
}
function renderIndexableChildren(context, slots, indexable, mergedPagination, prefixCls) {
  const {
    props: {
      record,
      rowIndex
    },
    selectDisabled
  } = context;
  const {
    customCell
  } = indexable;
  const {
    pageIndex = 1,
    pageSize = 0
  } = mergedPagination.value || {};
  const customRender = isString(customCell) ? slots[customCell] : customCell;
  if (!customRender) {
    process.env.NODE_ENV !== "production" && Logger.warn("components/table", "invalid customCell, please check the column is correct");
    return void 0;
  }
  const classes = [`${prefixCls}-indexable-label`, selectDisabled.value && `${prefixCls}-indexable-label-disabled`];
  return createVNode("span", {
    "class": classes
  }, [customRender({
    record,
    rowIndex,
    pageIndex,
    pageSize
  })]);
}
function renderBodyCells(columns, record, rowIndex) {
  const children = [];
  columns.forEach((column, colIndex) => {
    const cell = renderBodyCell(column, record, rowIndex, colIndex === columns.length - 1);
    if (cell) {
      children.push(cell);
    }
  });
  return children;
}
function renderBodyCell(column, record, rowIndex, isLast) {
  const {
    colSpan: getColSpan,
    rowSpan: getRowSpan
  } = column;
  const colSpan = getColSpan == null ? void 0 : getColSpan(record, rowIndex);
  const rowSpan = getRowSpan == null ? void 0 : getRowSpan(record, rowIndex);
  if (colSpan === 0 || rowSpan === 0) {
    return;
  }
  const colProps = {
    key: column.key,
    colSpan: colSpan === 1 ? void 0 : colSpan,
    rowSpan: rowSpan === 1 ? void 0 : rowSpan,
    isLast,
    column
  };
  return createVNode(BodyCell, colProps, null);
}
var BodyRow = /* @__PURE__ */ defineComponent({
  props: tableBodyRowProps,
  setup(props, {
    slots
  }) {
    const {
      props: tableProps2,
      mergedPrefixCls,
      expandable,
      isChecked,
      isIndeterminate,
      isCheckDisabled,
      handleExpandChange,
      checkExpandDisabled,
      selectable,
      handleSelectChange
    } = inject(TABLE_TOKEN);
    const {
      expandDisabled,
      handleExpend,
      selectDisabled,
      handleSelect,
      isHover,
      attrs
    } = useEvents(props, expandable, checkExpandDisabled, handleExpandChange, selectable, handleSelectChange, isCheckDisabled);
    const _isSelected = computed(() => isChecked(props.rowKey));
    const _isIndeterminate = computed(() => isIndeterminate(props.rowKey));
    const classes = computed(() => {
      const prefixCls = `${mergedPrefixCls.value}-row`;
      const {
        level,
        expanded
      } = props;
      return normalizeClass({
        [`${prefixCls}`]: true,
        [`${prefixCls}-level-${level}`]: !!level,
        [`${prefixCls}-selected`]: _isSelected.value,
        [`${prefixCls}-expanded`]: expanded
      });
    });
    provide(tableBodyRowToken, {
      props,
      expandDisabled,
      handleExpend,
      isSelected: _isSelected,
      isIndeterminate: _isIndeterminate,
      selectDisabled,
      handleSelect,
      isHover
    });
    return () => {
      var _a, _b, _c;
      const customAdditionalFn = (_a = tableProps2.customAdditional) == null ? void 0 : _a.bodyRow;
      const customAdditional = customAdditionalFn ? customAdditionalFn({
        record: props.record,
        rowIndex: props.rowIndex
      }) : void 0;
      const Tag = (_c = (_b = tableProps2.customTag) == null ? void 0 : _b.bodyRow) != null ? _c : "tr";
      return createVNode(Tag, mergeProps({
        "class": classes.value
      }, attrs.value, customAdditional), {
        default: () => {
          var _a2;
          return [(_a2 = slots.default) == null ? void 0 : _a2.call(slots)];
        }
      });
    };
  }
});
function useEvents(props, expandable, checkExpandDisabled, handleExpandChange, selectable, handleSelectChange, isCheckDisabled) {
  const expandDisabled = computed(() => checkExpandDisabled(props.rowData));
  const expendTrigger = computed(() => {
    var _a;
    return (_a = expandable.value) == null ? void 0 : _a.trigger;
  });
  const handleExpend = () => {
    const {
      rowKey,
      record
    } = props;
    handleExpandChange(rowKey, record);
  };
  const selectDisabled = computed(() => isCheckDisabled(props.rowKey));
  const selectTrigger = computed(() => {
    var _a;
    return (_a = selectable.value) == null ? void 0 : _a.trigger;
  });
  const showIndex = computed(() => {
    var _a;
    return (_a = selectable.value) == null ? void 0 : _a.showIndex;
  });
  const isHover = ref(false);
  const handleSelect = () => {
    const {
      rowKey,
      record
    } = props;
    handleSelectChange(rowKey, record);
  };
  const handleClick = () => {
    expendTrigger.value === "click" && handleExpend();
    selectTrigger.value === "click" && handleSelect();
  };
  const handleDblclick = () => {
    expendTrigger.value === "dblclick" && handleExpend();
    selectTrigger.value === "dblclick" && handleSelect();
  };
  const handleMouseenter = () => isHover.value = true;
  const handleMouseleave = () => isHover.value = false;
  const attrs = computed(() => {
    const _expandDisabled = expandDisabled.value;
    const _expendTrigger = expendTrigger.value;
    const _selectDisabled = selectDisabled.value;
    const _selectTrigger = selectTrigger.value;
    const clickEnabled = !_expandDisabled && _expendTrigger === "click" || !_selectDisabled && _selectTrigger === "click";
    const dblclickEnabled = !_expandDisabled && _expendTrigger === "dblclick" || !_selectDisabled && _selectTrigger === "dblclick";
    const mouseEnabled = !_selectDisabled && showIndex.value;
    let cursor = "";
    if (clickEnabled || dblclickEnabled) {
      cursor = "pointer";
    } else if (_expandDisabled && _expendTrigger || _selectDisabled && _selectTrigger) {
      cursor = "not-allowed";
    }
    return {
      style: cursor ? `cursor: ${cursor}` : void 0,
      onClick: clickEnabled ? handleClick : void 0,
      onDblclick: dblclickEnabled ? handleDblclick : void 0,
      onMouseenter: mouseEnabled ? handleMouseenter : void 0,
      onMouseleave: mouseEnabled ? handleMouseleave : void 0
    };
  });
  return {
    expandDisabled,
    handleExpend,
    selectDisabled,
    handleSelect,
    isHover,
    attrs
  };
}
function renderBodyRow(item, columns, rowIndex, slots, expandable, prefixCls, cols) {
  const {
    children,
    expanded,
    level,
    hasPrevSibling,
    hasNextSibling,
    showLineIndentIndexList,
    record,
    rowKey
  } = item;
  const rowProps = {
    key: rowKey,
    expanded,
    level,
    hasPrevSibling,
    hasNextSibling,
    showLineIndentIndexList,
    hasChildren: !!(children == null ? void 0 : children.length),
    record,
    rowData: item,
    rowIndex,
    rowKey
  };
  const rowNode = createVNode(BodyRow, rowProps, {
    default: () => [cols]
  });
  const expandedNode = expanded && renderExpandedContext(rowProps, slots, columns, expandable, prefixCls);
  return expandedNode ? [rowNode, expandedNode] : rowNode;
}
function renderExpandedContext(props, slots, columns, expandable, prefixCls) {
  const {
    customExpand
  } = expandable || {};
  const {
    record,
    rowIndex
  } = props;
  let expandedContext;
  if (isFunction(customExpand)) {
    expandedContext = customExpand({
      record,
      rowIndex
    });
  } else if (isString(customExpand) && slots[customExpand]) {
    expandedContext = slots[customExpand]({
      record,
      rowIndex
    });
  }
  return expandedContext && createVNode(BodyRowSingle, {
    "class": `${prefixCls}-expanded-row`,
    "columns": columns
  }, {
    default: () => [expandedContext]
  });
}
var MainTable = /* @__PURE__ */ defineComponent({
  setup() {
    const {
      props,
      slots,
      clientWidth,
      setClientWidth,
      expandable,
      mergedPrefixCls,
      mergedInsetShadow,
      mergedVirtual,
      mergedVirtualItemHeight,
      mergedVirtualColWidth,
      mergedAutoHeight,
      changeColumnWidth,
      flattedData,
      flattedColumns,
      fixedColumns,
      getVirtualColWidth,
      isSticky,
      mergedSticky,
      virtualScrollRef,
      scrollBodyRef,
      scrollContentRef,
      handleScroll,
      pingedStart,
      pingedEnd,
      scrollWidth,
      scrollHeight,
      scrollHorizontalOverflowed,
      scrollVerticalOverflowed,
      hasFixed,
      tableLayout
    } = inject(TABLE_TOKEN);
    const mainTableRef = ref();
    const showMeasure = computed(() => mergedAutoHeight.value || !!scrollWidth.value || isSticky.value || mergedVirtual.value.horizontal);
    const _changeColumnWidth = (key, width) => {
      if (isVisibleElement(mainTableRef.value)) {
        changeColumnWidth(key, width);
      }
    };
    provide(tableBodyToken, {
      changeColumnWidth: _changeColumnWidth
    });
    const triggerScroll = () => {
      const currentTarget = convertElement(scrollBodyRef);
      if (currentTarget) {
        handleScroll({
          currentTarget
        });
      }
    };
    const handleWrapperResize = (evt) => {
      const {
        offsetWidth
      } = evt.target;
      if (offsetWidth !== clientWidth.value) {
        setClientWidth(offsetWidth);
      }
    };
    onMounted(() => {
      triggerScroll();
      watch([() => props.dataSource, scrollWidth], ([, width]) => {
        if (width) {
          triggerScroll();
        }
      });
      onResize(mainTableRef.value, handleWrapperResize);
    });
    onBeforeUnmount(() => offResize(mainTableRef.value, handleWrapperResize));
    const classes = computed(() => {
      const prefixCls = mergedPrefixCls.value;
      return normalizeClass({
        [`${prefixCls}-container`]: true,
        [`${prefixCls}-ping-start`]: pingedStart.value,
        [`${prefixCls}-ping-end`]: pingedEnd.value,
        [`${prefixCls}-fixed-layout`]: tableLayout.value === "fixed",
        [`${prefixCls}-fixed-column`]: hasFixed.value,
        [`${prefixCls}-inset-shadow`]: mergedInsetShadow.value,
        [`${prefixCls}-scroll-horizontal`]: scrollHorizontalOverflowed.value,
        [`${prefixCls}-scroll-vertical`]: scrollVerticalOverflowed.value
      });
    });
    const contentStyle = computed(() => {
      var _a;
      const height = scrollHeight.value;
      const overflowX = mergedVirtual.value.horizontal ? "hidden" : "auto";
      const overflowY = mergedVirtual.value.vertical ? "hidden" : "auto";
      const fullHeight = (_a = props.scroll) == null ? void 0 : _a.fullHeight;
      return {
        overflowX,
        overflowY,
        [fullHeight ? "height" : "maxHeight"]: height
      };
    });
    const tableStyle = computed(() => {
      return {
        tableLayout: tableLayout.value,
        width: scrollWidth.value,
        minWidth: scrollWidth.value ? "100%" : void 0
      };
    });
    const handleScrolledChange = (startIndex, endIndex, visibleData) => {
      callEmit(props.onScrolledChange, startIndex, endIndex, visibleData.map((item) => item.record));
    };
    const virtualData = computed(() => {
      if (!mergedVirtual.value.vertical && !mergedVirtual.value.horizontal) {
        return [];
      }
      return flattedData.value.map((data) => {
        return {
          ...data,
          data: flattedColumns.value
        };
      });
    });
    const renderMeasureRow = (columns) => {
      if (!showMeasure.value || !columns) {
        return;
      }
      return createVNode(MeasureRow, {
        "columns": columns
      }, null);
    };
    const renderAlert = (columns) => {
      var _a;
      const alertNodes = (_a = slots.alert) == null ? void 0 : _a.call(slots);
      if (isEmptyNode(alertNodes)) {
        return;
      }
      return createVNode(BodyRowSingle, {
        "class": `${mergedPrefixCls.value}-alert-row`,
        "columns": columns
      }, {
        default: () => [alertNodes]
      });
    };
    const renderEmpty = (columns) => {
      if (flattedData.value.length > 0) {
        return;
      }
      return createVNode(BodyRowSingle, {
        "class": `${mergedPrefixCls.value}-empty-row`,
        "columns": columns,
        "isEmpty": true
      }, {
        default: () => [createVNode(_Empty, {
          "empty": props.empty
        }, slots)]
      });
    };
    const _renderBody = (columns, data, children) => {
      let contentNodes;
      if (children) {
        contentNodes = children;
      } else if (data == null ? void 0 : data.length) {
        const rows = [];
        data.forEach((item, rowIndex) => {
          const cells = renderBodyCells(columns != null ? columns : [], item.record, rowIndex);
          rows.push(...convertArray(renderBodyRow(item, columns, rowIndex, slots, expandable.value, mergedPrefixCls.value, cells)));
        });
        contentNodes = rows;
      }
      return createVNode(Body, null, {
        default: () => [renderAlert(columns), renderMeasureRow(columns), renderEmpty(columns), contentNodes]
      });
    };
    const renderBody = () => _renderBody(flattedColumns.value, flattedData.value);
    const renderVirtualBody = (columns, children) => _renderBody(columns, void 0, children);
    return () => {
      const prefixCls = mergedPrefixCls.value;
      const autoHeight = mergedAutoHeight.value;
      const virtual = mergedVirtual.value;
      const children = slots.default ? slots.default() : [];
      if (autoHeight || scrollHeight.value || isSticky.value || virtual.horizontal) {
        const {
          offsetTop
        } = mergedSticky.value;
        if (!props.headless) {
          children.push(createVNode(FixedHolder, {
            "offsetTop": offsetTop
          }, null));
        }
        if (virtual.vertical && (props.scroll || autoHeight) || !virtual.vertical && virtual.horizontal) {
          const rowRender = ({
            item,
            index,
            children: children2
          }) => renderBodyRow(item, item.data, index, slots, expandable.value, prefixCls, children2);
          const colRender = ({
            row,
            item,
            index,
            rowIndex
          }) => renderBodyCell(item, row.record, rowIndex, index === row.data.length - 1);
          const contentRender = (children2, {
            renderedData
          }) => {
            var _a;
            const columns = flattedData.value.length ? (_a = renderedData[0]) == null ? void 0 : _a.data : flattedColumns.value;
            return createVNode("table", {
              "ref": scrollContentRef,
              "class": `${prefixCls}-table`,
              "style": tableStyle.value
            }, [columns ? createVNode(ColGroup, {
              "columns": columns
            }, null) : void 0, renderVirtualBody(columns, children2), false]);
          };
          const colModifier = (renderedRow, renderedCols) => {
            const {
              fixedStartColumns,
              fixedEndColumns
            } = fixedColumns.value;
            return modifyVirtualData(renderedRow, renderedCols, flattedColumns.value, flattedData.value, fixedStartColumns, fixedEndColumns, false);
          };
          const {
            scroll,
            onScrolledBottom
          } = props;
          if (process.env.NODE_ENV !== "production" && virtual.vertical && !autoHeight && !isNumber(scroll == null ? void 0 : scroll.height)) {
            Logger.warn("components/table", "`scroll.height` must is a valid number when enable vertical virtual scroll");
          }
          children.push(createVNode(CdkVirtualScroll, {
            "ref": virtualScrollRef,
            "class": `${prefixCls}-body-virtual-scroll`,
            "dataSource": virtualData.value,
            "colModifier": colModifier,
            "fullHeight": scroll == null ? void 0 : scroll.fullHeight,
            "getKey": (item) => {
              var _a;
              return (_a = item.rowKey) != null ? _a : item.key;
            },
            "height": mergedAutoHeight.value ? "100%" : scroll == null ? void 0 : scroll.height,
            "width": "100%",
            "rowHeight": mergedVirtualItemHeight.value,
            "colWidth": mergedVirtualColWidth.value,
            "getColWidth": getVirtualColWidth,
            "rowRender": rowRender,
            "colRender": colRender,
            "contentRender": contentRender,
            "virtual": mergedVirtual.value,
            "scrollMode": props.virtualScrollMode,
            "bufferSize": props.virtualBufferSize,
            "bufferOffset": props.virtualBufferOffset,
            "onScroll": handleScroll,
            "onScrolledBottom": onScrolledBottom,
            "onScrolledChange": handleScrolledChange
          }, null));
        } else {
          children.push(createVNode("div", {
            "ref": scrollBodyRef,
            "class": `${prefixCls}-content`,
            "style": contentStyle.value,
            "onScroll": handleScroll
          }, [createVNode("table", {
            "ref": scrollContentRef,
            "class": `${prefixCls}-table`,
            "style": tableStyle.value
          }, [createVNode(ColGroup, null, null), renderBody(), false])]));
        }
        if (isSticky.value) {
          children.push(createVNode(StickyScroll, null, null));
        }
      } else {
        children.push(createVNode("div", {
          "ref": scrollBodyRef,
          "class": `${prefixCls}-content`,
          "style": contentStyle.value,
          "onScroll": handleScroll
        }, [createVNode("table", {
          "ref": scrollContentRef,
          "class": `${prefixCls}-table`,
          "style": tableStyle.value
        }, [createVNode(ColGroup, null, null), !props.headless && createVNode(Head, null, null), renderBody(), false])]));
      }
      return createVNode("div", {
        "ref": mainTableRef,
        "class": classes.value
      }, [children]);
    };
  }
});
function renderFooter(slots, prefixCls) {
  if (!slots.footer) {
    return null;
  }
  return createVNode("div", {
    "class": `${prefixCls}-footer`
  }, [slots.footer()]);
}
function renderPagination(slots, mergedPagination, filteredData, visible, prefixCls) {
  var _a, _b;
  let top = null;
  let bottom = null;
  if (mergedPagination !== null) {
    const {
      position
    } = mergedPagination;
    const [vertical, horizontal] = kebabCase(position).split("-");
    const className = `${prefixCls}-pagination ${prefixCls}-pagination-${horizontal}`;
    const node = (_b = (_a = slots.pagination) == null ? void 0 : _a.call(slots, {
      total: filteredData.length,
      ...mergedPagination
    })) != null ? _b : withDirectives(createVNode(IxPagination, mergeProps({
      "class": className,
      "total": filteredData.length
    }, mergedPagination), null), [[vShow, visible]]);
    top = vertical === "top" ? node : null;
    bottom = vertical === "bottom" ? node : null;
  }
  return [top, bottom];
}
function getDefaultThemeTokens(tokens, algorithms) {
  const { getGreyColors } = algorithms;
  const greyColors = getGreyColors();
  const {
    fontSizeSm,
    heightMd,
    paddingSizeSm,
    paddingSizeMd,
    paddingSizeLg,
    colorTextTitleSecondary,
    colorEmphasizedContainerBg,
    colorBorder,
    colorIconInfo,
    colorPrimary,
    colorContainerBg,
    colorContainerBgHover,
    colorInfoContainerBg
  } = tokens;
  return {
    fontSizeSm,
    fontSizeMd: fontSizeSm,
    fontSizeLg: fontSizeSm,
    headRowHeightSm: heightMd,
    headRowHeightMd: heightMd,
    headRowHeightLg: heightMd,
    cellPaddingHorizontalSm: paddingSizeSm,
    cellPaddingHorizontalMd: paddingSizeMd,
    cellPaddingHorizontalLg: paddingSizeLg,
    cellPaddingVerticalSm: 6,
    cellPaddingVerticalMd: 10,
    cellPaddingVerticalLg: 18,
    headColor: colorTextTitleSecondary,
    headBgColor: colorEmphasizedContainerBg,
    headSeparatorColor: colorBorder,
    headIconColor: colorIconInfo,
    headIconColorActive: colorPrimary,
    headIconBgColorHover: getGreyColors().l30,
    insetShadowStart: `inset 10px 0 8px -8px ${getAlphaColor(greyColors.d50, 0.12)}`,
    insetShadowEnd: `inset -10px 0 8px -8px ${getAlphaColor(greyColors.d50, 0.12)}`,
    bodyRowBgColorHover: colorContainerBgHover,
    bodyRowBgColorSelected: colorContainerBg,
    bodyRowBgColorExpanded: colorInfoContainerBg,
    bodyRowBgColorTreeExpanded: colorInfoContainerBg,
    bodyCellVerticalAlign: "middle",
    expandableIconColor: colorIconInfo
  };
}
function getDarkThemeTokens(tokens, algorithms) {
  const { getGreyColors } = algorithms;
  const greyColors = getGreyColors();
  const defaultTokens = getDefaultThemeTokens(tokens, algorithms);
  return {
    ...defaultTokens,
    insetShadowStart: `inset 10px 0 8px -8px ${greyColors.l50}`,
    insetShadowEnd: `inset -10px 0 8px -8px ${greyColors.l50}`
  };
}
const getThemeTokens = (tokens, presetTheme, algorithms) => {
  return presetTheme === "default" ? getDefaultThemeTokens(tokens, algorithms) : getDarkThemeTokens(tokens, algorithms);
};
const virtualItemHeight = {
  sm: 32,
  md: 40,
  lg: 56
};
const virtualColWidth = 150;
var Table = /* @__PURE__ */ defineComponent({
  name: "IxTable",
  props: tableProps,
  setup(props, {
    expose,
    slots
  }) {
    const common = useGlobalConfig("common");
    const {
      globalHashId,
      hashId,
      registerToken
    } = useThemeToken("table");
    registerToken(getThemeTokens);
    const mergedPrefixCls = computed(() => `${common.prefixCls}-table`);
    const locale = useGlobalConfig("locale");
    const config = useGlobalConfig("table");
    const [clientWidth, setClientWidth] = useState(0);
    const mergedAutoHeight = computed(() => {
      var _a;
      return (_a = props.autoHeight) != null ? _a : config.autoHeight;
    });
    const mergedChildrenKey = computed(() => {
      var _a;
      return (_a = props.childrenKey) != null ? _a : config.childrenKey;
    });
    const mergedGetKey = useGetKey(props, config, "components/table");
    const mergedEmptyCell = computed(() => {
      var _a;
      return (_a = props.emptyCell) != null ? _a : config.emptyCell;
    });
    const mergedInsetShadow = computed(() => {
      var _a;
      return (_a = props.insetShadow) != null ? _a : config.insetShadow;
    });
    const mergedSize = computed(() => {
      var _a;
      return (_a = props.size) != null ? _a : config.size;
    });
    const mergedVirtual = computed(() => {
      return {
        horizontal: props.virtualHorizontal,
        vertical: props.virtual
      };
    });
    const mergedVirtualItemHeight = computed(() => {
      var _a;
      return (_a = props.virtualItemHeight) != null ? _a : virtualItemHeight[mergedSize.value];
    });
    const mergedVirtualColWidth = computed(() => {
      var _a;
      return (_a = props.virtualColWidth) != null ? _a : virtualColWidth;
    });
    const {
      mergedPagination
    } = usePagination(props, config, mergedSize);
    const stickyContext = useSticky(props);
    const scrollContext = useScroll(props, stickyContext);
    const columnsContext = useColumns(props, slots, config, scrollContext.scrollBarSizeOnFixedHolder);
    const {
      flattedColumns,
      flattedColumnsWithScrollBar,
      fixedColumns
    } = columnsContext;
    const columnMeasureContext = useColumnWidthMeasure(flattedColumns);
    const {
      measuredColumnWidthMap
    } = columnMeasureContext;
    const columnCount = computed(() => flattedColumnsWithScrollBar.value.length);
    const columnOffsetsContext = useColumnOffsets(fixedColumns, measuredColumnWidthMap, columnCount);
    const sortableContext = useSortable(flattedColumns);
    const filterableContext = useFilterable(flattedColumns);
    const expandableContext = useExpandable(props, flattedColumns);
    const tableLayout = useTableLayout(props, columnsContext, scrollContext, stickyContext.isSticky, mergedVirtual, mergedAutoHeight);
    const {
      activeSorters
    } = sortableContext;
    const {
      activeFilters
    } = filterableContext;
    const columnMap = /* @__PURE__ */ new Map();
    const initColumnMap = () => {
      columnMap.clear();
      flattedColumns.value.forEach((col) => {
        columnMap.set(col.key, col);
      });
    };
    watch(flattedColumns, () => {
      initColumnMap();
    }, {
      immediate: true
    });
    const dataContext = useDataSource(props, mergedChildrenKey, mergedGetKey, activeSorters, activeFilters, expandableContext.expandedRowKeys, mergedPagination);
    const selectableContext = useSelectable(props, locale, columnsContext.flattedColumns, dataContext);
    useScrollOnChange(props, config, mergedPagination, activeSorters, activeFilters, scrollContext.scrollTo);
    const getVirtualColWidth = (rowKey, colKey) => {
      var _a, _b;
      return (_b = measuredColumnWidthMap.value[colKey]) != null ? _b : (_a = columnMap.get(colKey)) == null ? void 0 : _a.width;
    };
    const context = {
      props,
      slots,
      config,
      locale,
      clientWidth,
      setClientWidth,
      mergedPrefixCls,
      mergedEmptyCell,
      mergedInsetShadow,
      mergedVirtual,
      mergedVirtualItemHeight,
      mergedVirtualColWidth,
      mergedAutoHeight,
      getVirtualColWidth,
      ...columnsContext,
      ...columnMeasureContext,
      ...columnOffsetsContext,
      ...scrollContext,
      ...sortableContext,
      ...filterableContext,
      ...stickyContext,
      tableLayout,
      mergedPagination,
      ...expandableContext,
      ...dataContext,
      ...selectableContext
    };
    provide(TABLE_TOKEN, context);
    expose({
      scrollTo: scrollContext.scrollTo
    });
    const {
      flattedData,
      filteredData
    } = dataContext;
    const classes = computed(() => {
      const prefixCls = mergedPrefixCls.value;
      const {
        borderless = config.borderless,
        scroll
      } = props;
      return normalizeClass({
        [globalHashId.value]: !!globalHashId.value,
        [hashId.value]: !!hashId.value,
        [prefixCls]: true,
        [`${prefixCls}-auto-height`]: mergedAutoHeight.value,
        [`${prefixCls}-borderless`]: borderless,
        [`${prefixCls}-empty`]: flattedData.value.length === 0,
        [`${prefixCls}-full-height`]: scroll == null ? void 0 : scroll.fullHeight,
        [`${prefixCls}-${mergedSize.value}`]: true
      });
    });
    return () => {
      const prefixCls = mergedPrefixCls.value;
      const header = createVNode(_Header, {
        "header": props.header
      }, slots);
      const footer = renderFooter(slots, prefixCls);
      const [paginationTop, paginationBottom] = renderPagination(slots, mergedPagination.value, filteredData.value, flattedData.value.length > 0, prefixCls);
      const children = [header];
      const restChildren = [paginationTop, createVNode(MainTable, null, null), footer, paginationBottom].filter(Boolean);
      const spinProps = convertSpinProps(props.spin);
      if (spinProps) {
        children.push(createVNode(IxSpin, spinProps, {
          default: () => [restChildren]
        }));
      } else {
        children.push(...restChildren);
      }
      return createVNode("div", {
        "class": classes.value
      }, [children]);
    };
  }
});
function convertSpinProps(spin) {
  return isBoolean(spin) ? {
    spinning: spin
  } : spin;
}
const IxTable = Table;
const IxTableColumn = TableColumn;
export { IxTable, IxTableColumn, TABLE_TOKEN, getThemeTokens as getTableThemeTokens, getColumnKey as \u0275GetColumnKey, tableProps as \u0275TableProps };
