import { defineComponent, inject, computed, normalizeClass, createVNode, provide, mergeProps } from "vue";
import { isNil } from "lodash-es";
import { useKey, useControlledProp, useState, callEmit } from "@idux/cdk/utils";
import { useGlobalConfig } from "@idux/components/config";
import { IxIcon } from "@idux/components/icon";
import { getAlphaColor, useThemeToken } from "@idux/components/theme";
import { IxSpace } from "@idux/components/space";
const tagToken = Symbol("tag");
const tagProps = {
  bordered: { type: Boolean, default: void 0 },
  filled: { type: Boolean, default: void 0 },
  icon: String,
  number: Number,
  shape: String,
  status: { type: String, default: "normal" }
};
const tagGroupProps = {
  activeKeys: {
    type: Array,
    default: () => []
  },
  bordered: { type: Boolean, default: void 0 },
  clickable: {
    type: Boolean,
    default: false
  },
  closable: {
    type: Boolean,
    default: false
  },
  closeIcon: {
    type: String,
    default: "close"
  },
  filled: { type: Boolean, default: void 0 },
  dataSource: Array,
  gap: [Number, String],
  wrap: { type: Boolean, default: void 0 },
  shape: String,
  "onUpdate:activeKeys": [Function, Array],
  onClick: [Function, Array],
  onClose: [Function, Array]
};
function getDefaultThemeTokens(tokens, algrithms) {
  const {
    lineWidth,
    borderRadiusSm,
    tagCompColorAlpha,
    colorContainerBg,
    colorTextInfo,
    colorSuccessBg,
    colorInfoBg,
    colorWarningBg,
    colorRiskBg,
    colorErrorBg,
    colorFatalBg
  } = tokens;
  const { getGreyColors } = algrithms;
  const greyColors = getGreyColors();
  return {
    bgColorNormal: getAlphaColor(colorTextInfo, tagCompColorAlpha, colorContainerBg),
    bgColorFilled: greyColors.base,
    borderWidth: lineWidth,
    borderRadius: borderRadiusSm,
    minWidthRect: 40,
    minWidthRound: 48,
    minWidthNumeric: 64,
    successBgColor: getAlphaColor(colorSuccessBg, tagCompColorAlpha, colorContainerBg),
    infoBgColor: getAlphaColor(colorInfoBg, tagCompColorAlpha, colorContainerBg),
    warningBgColor: getAlphaColor(colorWarningBg, tagCompColorAlpha, colorContainerBg),
    riskBgColor: getAlphaColor(colorRiskBg, tagCompColorAlpha, colorContainerBg),
    errorBgColor: getAlphaColor(colorErrorBg, tagCompColorAlpha, colorContainerBg),
    fatalBgColor: getAlphaColor(colorFatalBg, tagCompColorAlpha, colorContainerBg)
  };
}
const getThemeTokens = (tokens, presetTheme, algrithms) => {
  return presetTheme === "default" ? getDefaultThemeTokens(tokens, algrithms) : getDefaultThemeTokens(tokens, algrithms);
};
var Tag = /* @__PURE__ */ defineComponent({
  name: "IxTag",
  props: tagProps,
  setup(props, {
    slots
  }) {
    const common = useGlobalConfig("common");
    const {
      globalHashId,
      hashId,
      registerToken
    } = useThemeToken("tag");
    registerToken(getThemeTokens);
    const config = useGlobalConfig("tag");
    const key = useKey();
    const {
      props: groupProps,
      mergedClosedKeys,
      handleTagClick,
      handleTagClose
    } = inject(tagToken, {});
    const mergedPrefixCls = computed(() => `${common.prefixCls}-tag`);
    const handleClick = (evt) => {
      if (groupProps == null ? void 0 : groupProps.clickable) {
        handleTagClick == null ? void 0 : handleTagClick(key, evt);
      }
    };
    const handleClose = (evt) => {
      if (groupProps == null ? void 0 : groupProps.closable) {
        evt.stopPropagation();
        handleTagClose == null ? void 0 : handleTagClose(key);
      }
    };
    const classes = computed(() => {
      const {
        shape = (groupProps == null ? void 0 : groupProps.shape) || config.shape,
        bordered = groupProps == null ? void 0 : groupProps.bordered,
        filled = groupProps == null ? void 0 : groupProps.filled,
        number,
        status
      } = props;
      const prefixCls = mergedPrefixCls.value;
      const isNumeric = !isNil(number);
      return normalizeClass({
        [globalHashId.value]: !!globalHashId.value,
        [hashId.value]: !!hashId.value,
        [`${prefixCls}`]: true,
        [`${prefixCls}-${status}`]: true,
        [`${prefixCls}-${shape}`]: !isNumeric && shape,
        [`${prefixCls}-bordered`]: bordered,
        [`${prefixCls}-filled`]: filled,
        [`${prefixCls}-numeric`]: isNumeric
      });
    });
    return () => {
      var _a;
      if (mergedClosedKeys == null ? void 0 : mergedClosedKeys.value.includes(key)) {
        return null;
      }
      const prefixCls = mergedPrefixCls.value;
      const {
        icon,
        number
      } = props;
      const {
        closable,
        closeIcon
      } = groupProps || {};
      const icoNode = slots.icon ? slots.icon() : icon && createVNode(IxIcon, {
        "name": icon
      }, null);
      let suffixNode = slots.suffix ? slots.suffix() : closable && createVNode(IxIcon, {
        "name": closeIcon
      }, null);
      if (closable) {
        suffixNode = createVNode("span", {
          "class": `${prefixCls}-close-icon`,
          "onClick": handleClose
        }, [suffixNode]);
      }
      return createVNode("span", {
        "class": classes.value,
        "onClick": handleClick
      }, [renderNumericPrefix(prefixCls, number), icoNode, createVNode("span", {
        "class": `${prefixCls}-content`
      }, [(_a = slots.default) == null ? void 0 : _a.call(slots)]), suffixNode]);
    };
  }
});
function renderNumericPrefix(prefixCls, number) {
  if (isNil(number)) {
    return null;
  }
  return createVNode("span", {
    "class": `${prefixCls}-numeric-prefix`
  }, [number > 9 ? "9+" : number]);
}
var TagGroup = /* @__PURE__ */ defineComponent({
  name: "IxTagGroup",
  props: tagGroupProps,
  setup(props, {
    slots
  }) {
    const common = useGlobalConfig("common");
    const config = useGlobalConfig("tagGroup");
    const mergedPrefixCls = computed(() => `${common.prefixCls}-tag-group`);
    const mergedGap = computed(() => {
      var _a;
      return (_a = props.gap) != null ? _a : config.gap;
    });
    const mergedWrap = computed(() => {
      var _a;
      return (_a = props.wrap) != null ? _a : config.wrap;
    });
    const [mergedActiveKeys, setActiveKeys] = useControlledProp(props, "activeKeys", () => []);
    const [mergedClosedKeys, setClosedKeys] = useState([]);
    const handleTagClick = (key, evt) => {
      callEmit(props.onClick, key, evt);
      const activeKeys = [...mergedActiveKeys.value];
      const targetIndex = activeKeys.findIndex((currentKey) => currentKey === key);
      targetIndex === -1 ? activeKeys.push(key) : activeKeys.splice(targetIndex, 1);
      setActiveKeys(activeKeys);
    };
    const handleTagClose = async (key) => {
      const result = await callEmit(props.onClose, key);
      if (result !== false) {
        const activeKeys = mergedActiveKeys.value;
        const targetIndex = activeKeys.findIndex((currentKey) => currentKey === key);
        if (targetIndex !== -1) {
          const tempActiveKeys = [...activeKeys];
          tempActiveKeys.splice(targetIndex, 1);
          setActiveKeys(tempActiveKeys);
        }
        setClosedKeys([...mergedClosedKeys.value, key]);
      }
    };
    provide(tagToken, {
      props,
      mergedActiveKeys,
      mergedClosedKeys,
      handleTagClick,
      handleTagClose
    });
    const classes = computed(() => {
      const prefixCls = mergedPrefixCls.value;
      const gap = mergedGap.value;
      return {
        [prefixCls]: true,
        [`${prefixCls}-clickable`]: props.clickable,
        [`${prefixCls}-compact`]: !gap || gap === "0"
      };
    });
    return () => {
      return createVNode(IxSpace, {
        "class": classes.value,
        "size": mergedGap.value,
        "wrap": mergedWrap.value
      }, {
        default: () => {
          var _a, _b;
          return [((_a = props.dataSource) == null ? void 0 : _a.map((data, index) => {
            const {
              key = index,
              label,
              ...reset
            } = data;
            return createVNode(Tag, mergeProps(reset, {
              "key": key
            }), {
              default: () => [label],
              suffix: slots.closeIcon
            });
          })) || ((_b = slots.default) == null ? void 0 : _b.call(slots))];
        }
      });
    };
  }
});
const IxTag = Tag;
const IxTagGroup = TagGroup;
export { IxTag, IxTagGroup, getThemeTokens as getTagThemeTokens };
