import { computed, h, onMounted, watch, nextTick, defineComponent, shallowRef, createVNode, normalizeClass, createTextVNode } from "vue";
import { isArray, isString, isObject, throttle, isFunction, isNumber, toInteger, isNaN, isBoolean } from "lodash-es";
import { useState, callEmit, Logger, useControlledProp, flattenNode, getFirstValidNode } from "@idux/cdk/utils";
import { useGlobalConfig } from "@idux/components/config";
import { useThemeToken } from "@idux/components/theme";
import { IxTooltip } from "@idux/components/tooltip";
import { useClipboard } from "@idux/cdk/clipboard";
import { IxIcon } from "@idux/components/icon";
import { useResizeObserver } from "@idux/cdk/resize";
function useCopyTooltip(props) {
  const { text } = useGlobalConfig("locale");
  const copyTooltip = computed(() => {
    const { copyTooltip: copyTooltip2 } = props;
    if (!copyTooltip2) {
      return;
    }
    if (isArray(copyTooltip2)) {
      return text.copyText.map((title, idx) => {
        const tooltip = copyTooltip2[idx];
        return isString(tooltip) ? { title: tooltip } : { title, ...tooltip };
      });
    }
    if (isObject(copyTooltip2)) {
      return text.copyText.map((title) => ({ title, ...copyTooltip2 }));
    }
    return text.copyText.map((title) => ({ title }));
  });
  return copyTooltip;
}
function useCopyable(props, config) {
  const copyIcon = computed(() => {
    var _a;
    return (_a = props.copyIcon) != null ? _a : config.copyIcon;
  });
  const [copied, setCopied] = useState(false);
  const { copy: _copy } = useClipboard();
  const copy = throttle((text) => {
    if (!text || copied.value) {
      return;
    }
    _copy(text).then((success) => {
      if (success) {
        setCopied(true);
        setTimeout(() => setCopied(false), 3e3);
      }
      callEmit(props.onCopy, success, text);
    });
  }, 300);
  const copyIconRenderer = ({ copied: _copied }) => {
    if (!copyIcon.value) {
      return;
    }
    if (isString(copyIcon.value)) {
      return h(IxIcon, { name: copyIcon.value });
    }
    if (isFunction(copyIcon.value)) {
      return copyIcon.value({ copied: _copied });
    }
    if (isArray(copyIcon.value)) {
      return h(IxIcon, { name: _copied ? copyIcon.value[1] : copyIcon.value[0] });
    }
  };
  return {
    copied,
    copy,
    copyIconRenderer
  };
}
function getNodeLength(node) {
  if (isString(node.children)) {
    return node.children.length;
  }
  return 1;
}
function getNodesLength(nodes) {
  if (!nodes) {
    return 0;
  }
  let count = 0;
  nodes.forEach((node) => {
    count += getNodeLength(node);
  });
  return count;
}
function sliceNode(node, start, end) {
  if (isString(node.children)) {
    return { ...node, children: node.children.slice(start, end) };
  }
  return node;
}
function sliceNodes(nodes, start, end) {
  if (!nodes) {
    return;
  }
  let currentIdx = 0;
  let currentNodeIdx = 0;
  const res = [];
  while (currentIdx < end && currentNodeIdx < nodes.length) {
    const currentNode = nodes[currentNodeIdx];
    const nodeLen = getNodeLength(currentNode);
    if (currentIdx < start) {
      if (currentIdx + nodeLen > start) {
        res.push(sliceNode(currentNode, start - currentIdx));
      }
    } else if (currentIdx + nodeLen <= end) {
      res.push(currentNode);
    } else {
      res.push(sliceNode(currentNode, 0, end - currentIdx));
    }
    currentIdx += nodeLen;
    currentNodeIdx++;
  }
  return res;
}
function useEllipsis(props, expandable, contentRef, innerRef, measureRef, rowMeasureRef) {
  const [contentNodesLength, setContentNodesLength] = useState(0);
  const [lastRenderIndex, setLastRenderIndex] = useState(0);
  const [measureLastRenderIndex, setMeasureLastRenderIndex] = useState(0);
  const [contentWidth, setContentWidth] = useState(0);
  const [isEllipsis, setIsEllipsis] = useState(false);
  const [measureStatus, setMeasureStatus] = useState("none");
  let currentWalk;
  const rows = computed(() => {
    const { ellipsis, lineClamp } = props;
    if (ellipsis === true) {
      return 1;
    }
    if (ellipsis && isNumber(ellipsis.rows)) {
      return ellipsis.rows;
    }
    if (lineClamp) {
      Logger.warn("components/text", "The `lineClamp` prop is deprecated, use `ellipsis` instead");
      const parsedRows = toInteger(lineClamp);
      return isNaN(parsedRows) ? 0 : parsedRows;
    }
    return 0;
  });
  const isSimple = computed(() => rows.value === 1 && !expandable.value);
  const onRender = (nodes) => {
    setContentNodesLength(getNodesLength(nodes));
  };
  const onMeasureRender = () => {
    if (measureStatus.value !== "none") {
      currentWalk == null ? void 0 : currentWalk();
    }
  };
  const renderClampedContent = (nodes, measure = false) => !measure ? sliceNodes(nodes, 0, lastRenderIndex.value) : measureStatus.value === "preparing" ? nodes : sliceNodes(nodes, 0, measureLastRenderIndex.value);
  const calculate = () => {
    let maxHeight = 0;
    let rowHeight = 0;
    let start = 0;
    let end = contentNodesLength.value;
    setMeasureLastRenderIndex(end);
    currentWalk = void 0;
    const next = (index) => {
      index && setMeasureLastRenderIndex(index);
      currentWalk = () => {
        nextTick(walk);
      };
    };
    const walk = () => {
      var _a, _b, _c, _d, _e, _f;
      const measureHeight = (_b = (_a = measureRef.value) == null ? void 0 : _a.offsetHeight) != null ? _b : 0;
      const conentHeight = (_d = (_c = contentRef.value) == null ? void 0 : _c.offsetHeight) != null ? _d : 0;
      if (measureStatus.value === "preparing") {
        if (!rowMeasureRef.value) {
          return;
        }
        rowHeight = (_f = (_e = rowMeasureRef.value) == null ? void 0 : _e.offsetHeight) != null ? _f : 0;
        maxHeight = rows.value * rowHeight;
        if (conentHeight <= maxHeight || measureHeight <= maxHeight) {
          setIsEllipsis(false);
          setLastRenderIndex(contentNodesLength.value);
          currentWalk = void 0;
          setMeasureStatus("none");
        } else {
          setMeasureStatus("measuring");
          next(Math.ceil(contentNodesLength.value / 2));
        }
      } else {
        if (measureHeight <= maxHeight) {
          start = measureLastRenderIndex.value;
        } else {
          end = measureLastRenderIndex.value;
        }
        if (start === end || start === end - 1) {
          if (measureHeight > maxHeight) {
            next(measureLastRenderIndex.value - 1);
          } else {
            currentWalk = void 0;
            setIsEllipsis(true);
            setLastRenderIndex(measureLastRenderIndex.value);
            setMeasureStatus("none");
          }
        } else {
          next(Math.ceil((start + end) / 2));
        }
      }
    };
    setMeasureStatus("preparing");
    next();
  };
  const startCalculate = () => {
    if (!contentNodesLength.value || !contentWidth.value) {
      return;
    }
    if (!rows.value) {
      setIsEllipsis(false);
      return;
    }
    if (isSimple.value) {
      setIsEllipsis(!!innerRef.value && innerRef.value.scrollWidth > innerRef.value.clientWidth);
      return;
    }
    calculate();
  };
  onMounted(() => {
    watch([contentNodesLength, rows], startCalculate, {
      flush: "post",
      immediate: true
    });
    useResizeObserver(contentRef, () => {
      var _a, _b;
      if (measureStatus.value !== "none") {
        return;
      }
      const width = (_b = (_a = contentRef.value) == null ? void 0 : _a.offsetWidth) != null ? _b : 0;
      if (width !== contentWidth.value) {
        setContentWidth(width);
        startCalculate();
      }
    });
  });
  return {
    isEllipsis,
    isSimple,
    measureStatus,
    onRender,
    onMeasureRender,
    renderClampedContent
  };
}
function useExpandable(props, config) {
  const [expanded, setExpanded] = useControlledProp(props, "expanded", false);
  const expandable = computed(() => {
    const _expandable = isObject(props.ellipsis) ? props.ellipsis.expandable : void 0;
    if (isBoolean(_expandable)) {
      return _expandable;
    }
    if (isBoolean(props.expandable)) {
      Logger.warn("components/text", "The `expandable` prop is deprecated, use `ellipsis` instead");
      return props.expandable;
    }
    return false;
  });
  const expandIcon = computed(() => {
    var _a;
    return (_a = props.expandIcon) != null ? _a : config.expandIcon;
  });
  const expandIconRenderer = computed(() => {
    if (!expandIcon.value) {
      return;
    }
    return ({ expanded: _expanded }) => {
      if (isString(expandIcon.value)) {
        return h(IxIcon, { name: expandIcon.value });
      }
      if (isFunction(expandIcon.value)) {
        return expandIcon.value({ expanded: _expanded });
      }
      if (isArray(expandIcon.value)) {
        return h(IxIcon, { name: _expanded ? expandIcon.value[1] : expandIcon.value[0] });
      }
    };
  });
  return {
    expandIconRenderer,
    expanded,
    expandable,
    setExpanded
  };
}
const textProps = {
  copyable: { type: Boolean, default: false },
  copyIcon: {
    type: [String, Function, Array],
    default: void 0
  },
  copyTooltip: {
    type: [Boolean, Array, Object],
    default: true
  },
  expanded: { type: Boolean, default: void 0 },
  expandable: { type: Boolean, default: void 0 },
  expandIcon: {
    type: [String, Function, Array],
    default: void 0
  },
  ellipsis: { type: [Boolean, Object], default: void 0 },
  lineClamp: { type: [String, Number], default: void 0 },
  tag: { type: [String, Object], default: "span" },
  tooltip: { type: [Boolean, String, Object], default: true },
  onCopy: [Function, Array],
  "onUpdate:expanded": [Function, Array]
};
const measureElementStyle = {
  display: "block",
  position: "absolute",
  visibility: "hidden",
  top: "-100px",
  left: "-100px"
};
const rowMeasureElementStyle = {
  ...measureElementStyle,
  wordBreak: "keep-all",
  whiteSpace: "nowrap"
};
var Text = /* @__PURE__ */ defineComponent({
  name: "IxText",
  props: textProps,
  setup(props, {
    slots
  }) {
    const common = useGlobalConfig("common");
    const {
      globalHashId
    } = useThemeToken();
    const config = useGlobalConfig("text");
    const mergedPrefixCls = computed(() => `${common.prefixCls}-text`);
    const contentRef = shallowRef();
    const innerRef = shallowRef();
    const measureRef = shallowRef();
    const rowMeasureRef = shallowRef();
    const {
      expanded,
      expandable,
      setExpanded,
      expandIconRenderer
    } = useExpandable(props, config);
    const {
      copied,
      copy,
      copyIconRenderer
    } = useCopyable(props, config);
    const copyTooltip = useCopyTooltip(props);
    const {
      isSimple,
      isEllipsis,
      measureStatus,
      onRender,
      onMeasureRender,
      renderClampedContent
    } = useEllipsis(props, expandable, contentRef, innerRef, measureRef, rowMeasureRef);
    const toggleExpanded = () => {
      const nextExpanded = !expanded.value;
      setExpanded(nextExpanded);
    };
    const renderEllipsisNode = () => {
      var _a, _b;
      return createVNode("span", {
        "class": `${mergedPrefixCls.value}-ellipsis`
      }, [(_b = (_a = slots.ellipsis) == null ? void 0 : _a.call(slots)) != null ? _b : "..."]);
    };
    const renderCopyNode = () => {
      var _a;
      if (!props.copyable) {
        return;
      }
      const node = createVNode("span", {
        "class": `${mergedPrefixCls.value}-copy-icon`,
        "onClick": () => copy(getStringBySlot(slots.default))
      }, [((_a = slots.copyIcon) != null ? _a : copyIconRenderer)({
        copied: copied.value
      })]);
      if (!copyTooltip.value) {
        return node;
      }
      return createVNode(IxTooltip, copied.value ? copyTooltip.value[1] : copyTooltip.value[0], {
        default: () => [node]
      });
    };
    const renderMeasureElement = (nodes) => {
      const {
        tag: Tag
      } = props;
      return createVNode(Tag, {
        "ref": measureRef,
        "class": `${mergedPrefixCls.value}-inner`,
        "style": measureElementStyle
      }, {
        default: () => {
          var _a;
          return [renderClampedContent(nodes, true), renderEllipsisNode(), (_a = slots.suffix) == null ? void 0 : _a.call(slots), renderCopyNode()];
        }
      });
    };
    const renderRowMeasureElement = () => {
      const {
        tag: Tag
      } = props;
      return createVNode(Tag, {
        "ref": rowMeasureRef,
        "class": `${mergedPrefixCls.value}-inner`,
        "style": rowMeasureElementStyle
      }, {
        default: () => [createTextVNode("lg")]
      });
    };
    return () => {
      var _a, _b, _c;
      const {
        tag: Tag,
        tooltip
      } = props;
      const prefixCls = mergedPrefixCls.value;
      const isNative = tooltip === "native";
      const titleSlot = slots.title || slots.default;
      const hasExpandIcon = !!slots.expandIcon || !!expandIconRenderer.value;
      const expandNode = expandable.value && hasExpandIcon ? createVNode("div", {
        "class": `${prefixCls}-expand-icon`,
        "onClick": toggleExpanded
      }, [((_a = slots.expandIcon) != null ? _a : expandIconRenderer.value)({
        expanded: expanded.value
      })]) : void 0;
      const contentNodes = flattenNode((_b = slots.default) == null ? void 0 : _b.call(slots));
      let node = createVNode(Tag, {
        "ref": innerRef,
        "class": `${prefixCls}-inner`,
        "title": isNative && isEllipsis.value ? getStringBySlot(titleSlot) : void 0,
        "onClick": expandable.value && !hasExpandIcon ? toggleExpanded : void 0
      }, {
        default: () => [!isSimple.value && isEllipsis.value && !expanded.value && measureStatus.value === "none" ? renderClampedContent(contentNodes) : contentNodes, !isSimple.value && isEllipsis.value && !expanded.value && renderEllipsisNode()]
      });
      if (isEllipsis.value && !expanded.value && tooltip && !isNative) {
        const tooltipProps = isObject(tooltip) ? tooltip : {};
        const _node = function() {
          return node;
        }();
        node = createVNode(IxTooltip, tooltipProps, {
          default: () => [_node],
          title: titleSlot
        });
      }
      const classes = normalizeClass({
        [globalHashId.value]: globalHashId.value,
        [prefixCls]: true,
        [`${prefixCls}-simple`]: isSimple.value,
        [`${prefixCls}-ellipsis`]: isEllipsis.value,
        [`${prefixCls}-expandable`]: expandable.value,
        [`${prefixCls}-has-expand-icon`]: hasExpandIcon
      });
      onRender(contentNodes);
      if (measureStatus.value !== "none") {
        onMeasureRender();
      }
      return createVNode("div", {
        "ref": contentRef,
        "class": classes
      }, [node, (_c = slots.suffix) == null ? void 0 : _c.call(slots), renderCopyNode(), isEllipsis.value && expandNode, measureStatus.value !== "none" && renderMeasureElement(contentNodes), measureStatus.value === "preparing" && renderRowMeasureElement()]);
    };
  }
});
function getStringBySlot(slot) {
  const validNode = getFirstValidNode(slot == null ? void 0 : slot());
  if (!validNode) {
    return "";
  }
  const {
    children
  } = validNode;
  if (isString(children)) {
    return children;
  }
  return "";
}
const IxText = Text;
export { IxText };
