import { customRef, watch, computed, onMounted, onUnmounted, nextTick, defineComponent, normalizeClass, createVNode, mergeProps } from "vue";
import { useGlobalConfig } from "@idux/components/config";
import { IxIcon } from "@idux/components/icon";
import { \u0275InputCommonProps as _InputCommonProps, \u0275UseInput as _UseInput } from "@idux/components/input";
import { numUnitTransform, useThemeToken } from "@idux/components/theme";
import { isBoolean, throttle, isObject, isNumber } from "lodash-es";
import { useResizeObserver } from "@idux/cdk/resize";
import { getBoxSizingData as getBoxSizingData$1, rAF } from "@idux/cdk/utils";
function getBoxSizingData(node) {
  const data = getBoxSizingData$1(node);
  const { paddingBottom, paddingTop, borderBottom, borderTop } = data;
  return {
    ...data,
    paddingSize: paddingTop + paddingBottom,
    borderSize: borderTop + borderBottom
  };
}
const HIDDEN_TEXTAREA_STYLE = {
  height: "0",
  "min-height": "0",
  "max-height": "none",
  visibility: "hidden",
  overflow: "hidden",
  transition: "none"
};
function measureTextarea(textarea, measureFn, cache = true) {
  let clonedTextarea;
  const _textarea = textarea;
  if (_textarea.__cdk_measure_textarea) {
    clonedTextarea = _textarea.__cdk_measure_textarea;
  } else {
    clonedTextarea = textarea.cloneNode(false);
    cache && (_textarea.__cdk_measure_textarea = clonedTextarea);
  }
  Object.keys(HIDDEN_TEXTAREA_STYLE).forEach((key) => {
    clonedTextarea.style.setProperty(key, HIDDEN_TEXTAREA_STYLE[key], "important");
  });
  textarea.parentNode.appendChild(clonedTextarea);
  const measureRes = measureFn(clonedTextarea);
  if (clonedTextarea.parentNode) {
    clonedTextarea.parentNode.removeChild(clonedTextarea);
  }
  return measureRes;
}
function useLineHeight(textareaRef) {
  let lineHeight = 0;
  return customRef((track, trigger) => {
    watch(textareaRef, () => {
      lineHeight = calcTextareaLineHeight(textareaRef.value);
      trigger();
    });
    return {
      get() {
        if (!textareaRef.value) {
          return 0;
        }
        const value = lineHeight || (lineHeight = calcTextareaLineHeight(textareaRef.value));
        track();
        return value;
      },
      set() {
      }
    };
  });
}
function calcTextareaLineHeight(textarea) {
  if (!textarea) {
    return 0;
  }
  if (!textarea.parentNode) {
    return 0;
  }
  return measureTextarea(
    textarea,
    (el) => {
      const rows = el.rows;
      const { paddingSize } = getBoxSizingData(el);
      el.rows = 1;
      el.value = "x";
      const lineHeight = el.scrollHeight - paddingSize;
      el.rows = rows;
      return lineHeight > 0 ? lineHeight : 0;
    },
    false
  );
}
const isAutoRowsObject = (value) => {
  return isObject(value) && (isNumber(value.minRows) || isNumber(value.maxRows));
};
function useAutoRows(textareaRef, autoRows) {
  let previousValue;
  let previousMinRows;
  let initialHeight;
  let isFunctioning = false;
  let currentOverflow;
  let cachedPlaceholderHeight;
  const enabled = computed(() => isAutoRowsObject(autoRows.value) || isBoolean(autoRows.value) && !!autoRows.value);
  const minRows = computed(() => isAutoRowsObject(autoRows.value) ? autoRows.value.minRows : void 0);
  const maxRows = computed(() => isAutoRowsObject(autoRows.value) ? autoRows.value.maxRows : void 0);
  const cachedLineHeight = useLineHeight(textareaRef);
  const boxSizingData = computed(() => {
    if (!textareaRef.value) {
      return { paddingSize: 0, borderSize: 0, boxSizing: "" };
    }
    return getBoxSizingData(textareaRef.value);
  });
  function getTextareaScrollHeight() {
    return measureTextarea(textareaRef.value, (textarea) => {
      textarea.value = textareaRef.value.value;
      void textarea.scrollHeight;
      return getHeightByScrollHeight(textarea.scrollHeight, boxSizingData.value);
    });
  }
  function getTextareaPlaceholderHeight() {
    if (!textareaRef.value) {
      return 0;
    }
    const textarea = textareaRef.value;
    if (!textarea.placeholder) {
      return 0;
    }
    if (!cachedPlaceholderHeight) {
      measureTextarea(textareaRef.value, (textarea2) => {
        textarea2.style.height = "";
        textarea2.value = textarea2.placeholder;
        void textarea2.scrollHeight;
        cachedPlaceholderHeight = getHeightByScrollHeight(textarea2.scrollHeight, boxSizingData.value);
      });
    }
    return cachedPlaceholderHeight;
  }
  function resizeToFitContent(force = false) {
    if (!enabled.value) {
      return;
    }
    if (!cachedLineHeight.value) {
      return;
    }
    const textarea = textareaRef.value;
    const value = textarea.value;
    if (!force && minRows.value === previousMinRows && value === previousValue) {
      return;
    }
    textarea.style.lineHeight = `${cachedLineHeight.value}px`;
    const height = Math.max(getTextareaScrollHeight(), getTextareaPlaceholderHeight());
    textarea.style.height = `${height}px`;
    currentOverflow = currentOverflow != null ? currentOverflow : textarea.style.overflow;
    textarea.style.overflow = "hidden";
    nextTick(
      () => rAF(() => {
        scrollToCaretPosition(textarea);
        textarea.style.overflow = currentOverflow != null ? currentOverflow : "";
        currentOverflow = void 0;
      })
    );
    previousValue = value;
    previousMinRows = minRows.value;
  }
  function setMinHeight() {
    if (!textareaRef.value) {
      return;
    }
    const contentHeight = minRows.value && cachedLineHeight.value ? minRows.value * cachedLineHeight.value : null;
    const minHeight = contentHeight ? `${getHeightByContentHeight(contentHeight, boxSizingData.value)}px` : "";
    textareaRef.value.style.minHeight = minHeight;
  }
  function setMaxHeight() {
    if (!textareaRef.value) {
      return;
    }
    const contentHeight = maxRows.value && cachedLineHeight.value ? maxRows.value * cachedLineHeight.value : null;
    const maxHeight = contentHeight ? `${getHeightByContentHeight(contentHeight, boxSizingData.value)}px` : "";
    textareaRef.value.style.maxHeight = maxHeight;
  }
  function scrollToCaretPosition(textarea) {
    const { selectionStart, selectionEnd } = textarea;
    if (isFunctioning && document.activeElement === textarea) {
      textarea.setSelectionRange(selectionStart, selectionEnd);
    }
  }
  function reset() {
    if (initialHeight !== void 0) {
      textareaRef.value && (textareaRef.value.style.height = initialHeight);
    }
  }
  let stopWatch;
  const onResize = throttle(() => {
    cachedPlaceholderHeight = void 0;
    resizeToFitContent(true);
  }, 16);
  onMounted(() => {
    const watchStopHandlers = [
      watch(
        textareaRef,
        () => {
          textareaRef.value && (initialHeight = textareaRef.value.style.height);
        },
        { immediate: true }
      ),
      watch(
        autoRows,
        () => {
          enabled.value ? resizeToFitContent(true) : reset();
        },
        { immediate: true }
      ),
      watch(
        minRows,
        () => {
          setMinHeight();
        },
        { immediate: true }
      ),
      watch(
        maxRows,
        () => {
          setMaxHeight();
        },
        { immediate: true }
      )
    ];
    const stopResizeObserver = useResizeObserver(textareaRef, onResize);
    stopWatch = () => {
      watchStopHandlers.forEach((stop) => stop());
      stopResizeObserver();
    };
    isFunctioning = true;
  });
  onUnmounted(() => {
    stopWatch == null ? void 0 : stopWatch();
    isFunctioning = false;
  });
  return {
    resizeToFitContent,
    lineHeight: cachedLineHeight,
    boxSizingData
  };
}
function getHeightByScrollHeight(scrollHeight, sizingData) {
  const { borderSize, paddingSize, boxSizing } = sizingData;
  if (boxSizing === "border-box") {
    return scrollHeight + borderSize;
  }
  return scrollHeight - paddingSize;
}
function getHeightByContentHeight(contentHeight, sizingData) {
  const { borderSize, paddingSize, boxSizing } = sizingData;
  if (boxSizing === "border-box") {
    return contentHeight + borderSize + paddingSize;
  }
  return contentHeight + paddingSize;
}
const textareaProps = {
  ..._InputCommonProps,
  autoRows: { type: [Boolean, Object], default: void 0 },
  computeCount: { type: Function, default: void 0 },
  maxCount: { type: [Number, String], default: void 0 },
  resize: { type: String, default: void 0 },
  showCount: { type: Boolean, default: void 0 }
};
function getDefaultThemeTokens(tokens) {
  const { colorTextPlaceholder } = tokens;
  return {
    countBottom: 1,
    countRight: 8,
    countOpacity: 0.9,
    countColor: colorTextPlaceholder
  };
}
const transforms = {
  countBottom: (value) => numUnitTransform(value, "px"),
  countRight: (value) => numUnitTransform(value, "px")
};
const getThemeTokens = (tokens, presetTheme) => {
  return presetTheme === "default" ? getDefaultThemeTokens(tokens) : getDefaultThemeTokens(tokens);
};
var Textarea = /* @__PURE__ */ defineComponent({
  name: "IxTextarea",
  inheritAttrs: false,
  props: textareaProps,
  setup(props, {
    slots,
    expose,
    attrs
  }) {
    const common = useGlobalConfig("common");
    const {
      globalHashId,
      hashId,
      registerToken
    } = useThemeToken("textarea");
    registerToken(getThemeTokens, transforms);
    const mergedPrefixCls = computed(() => `${common.prefixCls}-textarea`);
    const config = useGlobalConfig("textarea");
    const {
      elementRef,
      accessor,
      mergedSize,
      mergedStatus,
      clearable,
      clearIcon,
      clearVisible,
      isFocused,
      isComposing,
      focus,
      blur,
      handleInput,
      handleCompositionStart,
      handleCompositionEnd,
      handleClear,
      syncValue
    } = _UseInput(props, config);
    expose({
      focus,
      blur
    });
    const classes = computed(() => {
      const {
        showCount = config.showCount
      } = props;
      const status = mergedStatus.value;
      const prefixCls = mergedPrefixCls.value;
      const classes2 = {
        [globalHashId.value]: globalHashId.value,
        [hashId.value]: hashId.value,
        [prefixCls]: true,
        [`${prefixCls}-${mergedSize.value}`]: true,
        [`${prefixCls}-${status}`]: !!status,
        [`${prefixCls}-clearable`]: clearable.value,
        [`${prefixCls}-disabled`]: accessor.disabled,
        [`${prefixCls}-focused`]: isFocused.value,
        [`${prefixCls}-with-count`]: showCount
      };
      return normalizeClass([classes2, attrs.class]);
    });
    const dataCount = useDataCount(props, config, accessor);
    const autoRows = computed(() => {
      var _a;
      return (_a = props.autoRows) != null ? _a : config.autoRows;
    });
    const resize = computed(() => {
      var _a;
      const resize2 = (_a = props.resize) != null ? _a : config.resize;
      if (autoRows.value) {
        return resize2 === "horizontal" ? "horizontal" : "none";
      } else {
        return resize2;
      }
    });
    const textareaStyle = computed(() => ({
      resize: resize.value
    }));
    const {
      resizeToFitContent
    } = useAutoRows(elementRef, autoRows);
    onMounted(() => {
      syncValue();
      watch(() => accessor.value, resizeToFitContent, {
        immediate: true
      });
    });
    const _handleInput = (evt) => {
      handleInput(evt);
      if (isComposing.value) {
        resizeToFitContent();
      }
    };
    return () => {
      const {
        class: className,
        style,
        ...rest
      } = attrs;
      const prefixCls = mergedPrefixCls.value;
      return createVNode("span", {
        "class": classes.value,
        "style": style,
        "data-count": dataCount.value
      }, [createVNode("textarea", mergeProps(rest, {
        "ref": elementRef,
        "class": `${prefixCls}-inner`,
        "style": textareaStyle.value,
        "disabled": accessor.disabled,
        "readonly": props.readonly,
        "onInput": _handleInput,
        "onCompositionstart": handleCompositionStart,
        "onCompositionend": handleCompositionEnd
      }), null), clearable.value && createVNode("span", {
        "class": normalizeClass([`${prefixCls}-clear`, clearVisible.value ? "visible" : ""]),
        "onClick": handleClear
      }, [slots.clearIcon ? slots.clearIcon() : createVNode(IxIcon, {
        "name": clearIcon.value
      }, null)])]);
    };
  }
});
function useDataCount(props, config, accessor) {
  return computed(() => {
    var _a, _b, _c, _d;
    const showCount = (_a = props.showCount) != null ? _a : config.showCount;
    const computeCount = (_b = props.computeCount) != null ? _b : config.computeCount;
    const maxCount = (_c = props.maxCount) != null ? _c : config.maxCount;
    let dataCount = "";
    if (showCount) {
      const value = (_d = accessor.value) != null ? _d : "";
      dataCount = value.length;
      if (computeCount) {
        dataCount = computeCount(value);
      } else if (maxCount) {
        dataCount += " / " + maxCount;
      }
    }
    return dataCount;
  });
}
const IxTextarea = Textarea;
export { IxTextarea, getThemeTokens as getTextareaThemeTokens, getBoxSizingData as \u0275GetBoxSizingData, measureTextarea as \u0275MeasureTextarea, useAutoRows as \u0275UseAutoRows, useLineHeight as \u0275UseLineHeight };
