import { defineComponent, inject, getCurrentInstance, computed, normalizeClass, createVNode, provide } from "vue";
import { isBoolean } from "lodash-es";
import { hasSlot, flattenNode } from "@idux/cdk/utils";
import { useGlobalConfig } from "@idux/components/config";
import { IxIcon } from "@idux/components/icon";
import { useThemeToken } from "@idux/components/theme";
import { isPresetColor, isStatusColor, convertStringVNode } from "@idux/components/utils";
const timelineToken = Symbol("timelineToken");
const timelineItemKey = Symbol("IDUX_TIMELINE_ITEM_KEY");
const timelineProps = {
  pending: {
    type: [String, Boolean],
    default: false
  },
  pendingDot: String,
  reverse: {
    type: Boolean,
    default: false
  },
  placement: {
    type: String,
    default: "end"
  },
  labelPlacement: {
    type: String,
    default: "bottom"
  },
  both: {
    type: Boolean,
    default: true
  }
};
const timelineItemProps = {
  color: {
    type: String,
    default: "primary"
  },
  dot: String,
  label: String,
  placement: String
};
var TimelineItem = /* @__PURE__ */ defineComponent({
  name: "IxTimelineItem",
  props: timelineItemProps,
  [timelineItemKey]: true,
  setup(props, {
    slots
  }) {
    const {
      props: parentProps,
      mergedPrefixCls
    } = inject(timelineToken);
    const {
      vnode
    } = getCurrentInstance();
    const itemPlacement = computed(() => {
      const {
        placement: parentPlacement
      } = parentProps;
      const {
        placement
      } = props;
      const index = vnode.index;
      let _placement;
      if (parentPlacement === "alternate") {
        _placement = placement ? placement : index % 2 ? "start" : "end";
      } else {
        _placement = parentPlacement;
      }
      return _placement;
    });
    const isPresetOrStatusColor = computed(() => isPresetColor(props.color) || isStatusColor(props.color));
    const isTopLabel = computed(() => parentProps.labelPlacement === "top");
    const dotStyle = computed(() => {
      const {
        color
      } = props;
      if (isPresetOrStatusColor.value) {
        return {};
      }
      return {
        color,
        "border-color": color
      };
    });
    const dotInnerStyle = computed(() => isPresetOrStatusColor.value ? {} : {
      "background-color": props.color
    });
    const classes = computed(() => {
      const prefixCls = `${mergedPrefixCls.value}-item`;
      const placement = itemPlacement.value;
      return normalizeClass({
        [`${prefixCls}`]: true,
        [`${prefixCls}-${placement}`]: true
      });
    });
    const isCustomDot = computed(() => hasSlot(slots, "dot") || !!props.dot);
    const dotClass = computed(() => {
      const prefixCls = `${mergedPrefixCls.value}-item-head`;
      const {
        color
      } = props;
      return normalizeClass({
        [`${prefixCls}-dot`]: true,
        [`${prefixCls}-dot-custom`]: isCustomDot.value,
        [`${prefixCls}-dot-${color}`]: isPresetOrStatusColor.value
      });
    });
    return () => {
      const {
        label,
        dot
      } = props;
      const prefixCls = `${mergedPrefixCls.value}-item`;
      const dotNode = convertStringVNode(slots.dot, dot);
      const labelNode = convertStringVNode(slots.label, label);
      return createVNode("li", {
        "class": classes.value
      }, [createVNode("div", {
        "class": `${prefixCls}-head`
      }, [createVNode("div", {
        "class": `${prefixCls}-head-line`
      }, null), createVNode("div", {
        "class": dotClass.value,
        "style": dotStyle.value
      }, [dotNode, !isCustomDot.value && createVNode("div", {
        "class": `${prefixCls}-head-dot-inner`,
        "style": dotInnerStyle.value
      }, null)])]), isTopLabel.value ? createVNode("div", {
        "class": `${prefixCls}-content ${prefixCls}-content-top`
      }, [labelNode && createVNode("div", {
        "class": `${prefixCls}-content-label`
      }, [labelNode]), slots.default && createVNode("div", {
        "class": `${prefixCls}-content-desc`
      }, [slots.default()])]) : createVNode("div", {
        "class": `${prefixCls}-content`
      }, [slots.default && createVNode("div", {
        "class": `${prefixCls}-content-desc`
      }, [slots.default()]), labelNode && createVNode("div", {
        "class": `${prefixCls}-content-label`
      }, [labelNode])])]);
    };
  }
});
function getDefaultThemeTokens(tokens) {
  const { fontSizeSm, marginSizeSm, marginSizeLg, colorSeparator } = tokens;
  return {
    fontSize: fontSizeSm,
    dotSize: 12,
    dotBorderWidth: 2,
    contentLabelMarginTop: marginSizeSm,
    lineWidth: 2,
    lineBgColor: colorSeparator,
    pendingItemContentMinWidth: 40,
    contentMarginBottom: marginSizeLg
  };
}
const getThemeTokens = (tokens, presetTheme) => {
  return presetTheme === "default" ? getDefaultThemeTokens(tokens) : getDefaultThemeTokens(tokens);
};
var Timeline = /* @__PURE__ */ defineComponent({
  name: "IxTimeline",
  props: timelineProps,
  setup(props, {
    slots
  }) {
    const common = useGlobalConfig("common");
    const {
      globalHashId,
      hashId,
      registerToken
    } = useThemeToken("timeline");
    registerToken(getThemeTokens);
    const mergedPrefixCls = computed(() => `${common.prefixCls}-timeline`);
    const hasPendingNode = computed(() => props.pending !== false || hasSlot(slots, "pending"));
    const classes = computed(() => {
      const {
        placement,
        reverse,
        both
      } = props;
      const prefixCls = mergedPrefixCls.value;
      return normalizeClass({
        [globalHashId.value]: !!globalHashId.value,
        [hashId.value]: !!hashId.value,
        [`${prefixCls}`]: true,
        [`${prefixCls}-${placement}`]: true,
        [`${prefixCls}-reverse`]: reverse,
        [`${prefixCls}-not-both`]: !both,
        [`${prefixCls}-pending`]: hasPendingNode.value
      });
    });
    provide(timelineToken, {
      props,
      mergedPrefixCls
    });
    return () => {
      const itemNodes = useItems(slots, props, hasPendingNode, mergedPrefixCls);
      return createVNode("ul", {
        "class": classes.value
      }, [itemNodes.value]);
    };
  }
});
function convertItemNodes(itemNodes) {
  return flattenNode(itemNodes, {
    key: timelineItemKey
  }).map((node, index) => {
    node.index = index;
    return node;
  });
}
function useItems(slots, props, hasPendingNode, mergedPrefixCls) {
  return computed(() => {
    var _a, _b;
    const {
      pending,
      pendingDot,
      reverse
    } = props;
    const defaultSlots = ((_a = slots == null ? void 0 : slots.default) == null ? void 0 : _a.call(slots)) || [];
    let pendingNode = null;
    if (hasPendingNode.value) {
      const pendingDotNode = ((_b = slots == null ? void 0 : slots.pendingDot) == null ? void 0 : _b.call(slots)) || pendingDot || createVNode(IxIcon, {
        "name": "loading"
      }, null);
      pendingNode = createVNode(TimelineItem, {
        "class": `${mergedPrefixCls.value}-item-pending`
      }, {
        default: () => [convertStringVNode(slots.pending, isBoolean(pending) ? "" : pending)],
        dot: () => pendingDotNode
      });
    }
    const timelineItemNodes = reverse ? [pendingNode, ...defaultSlots.reverse()] : [...defaultSlots, pendingNode];
    return convertItemNodes(timelineItemNodes.filter(Boolean));
  });
}
const IxTimeline = Timeline;
const IxTimelineItem = TimelineItem;
export { IxTimeline, IxTimelineItem, getThemeTokens as getTimelineThemeTokens };
