/**
 * @license
 *
 * Use of this source code is governed by an MIT-style license that can be
 * found in the LICENSE file at https://github.com/IDuxFE/idux/blob/main/LICENSE
 */
import type { TransferOperationsContext } from './composables/useTransferOperations';
import type { ConvertToSlotParams } from './utils';
import type { VirtualScrollMode, VirtualScrollToFn } from '@idux/cdk/scroll';
import type { ExtractInnerPropTypes, ExtractPublicPropTypes, MaybeArray, VKey } from '@idux/cdk/utils';
import type { EmptyProps } from '@idux/components/empty';
import type { PaginationProps } from '@idux/components/pagination';
import type { GetKeyFn } from '@idux/components/utils';
import type { ComputedRef, DefineComponent, HTMLAttributes, PropType, Slots, VNode } from 'vue';
export interface SeparatedData<T extends TransferData = TransferData> {
    sourceData: T[];
    targetData: T[];
}
export interface TransferDataStrategy<T extends TransferData = TransferData, K = VKey> {
    genDataKeys: (data: T[], getKey: GetKeyFn) => Set<K>;
    genDataKeyMap: (dataSource: T[], getKey: GetKeyFn) => Map<K, T>;
    genDisabledKeys: (data: T[], getKey: GetKeyFn) => Set<K>;
    getAllSelectedKeys: (selected: boolean, data: T[], selectedKeySet: Set<K>, disabledKeySet: Set<K>, getKey: GetKeyFn) => K[];
    separateDataSource: (dataSource: T[], dataKeyMap: Map<K, T>, targetKeySet: Set<K>, getKey: GetKeyFn) => SeparatedData<T>;
    dataFilter: (data: T[], searchValue: string, searchFn: (item: T, searchValue: string) => boolean) => T[];
    append: (keys: K[], targetKeys: K[], getKey: GetKeyFn) => K[];
    remove: (keys: K[], targetKeys: K[], getKey: GetKeyFn) => K[];
}
export type TransferDataStrategyProp<T extends TransferData = TransferData> = Partial<TransferDataStrategy<T>>;
export type TransferMode = 'default' | 'immediate';
export interface TransferBindings<T extends TransferData = TransferData> {
    data: ComputedRef<T[]>;
    filteredData: ComputedRef<T[]>;
    paginatedData: ComputedRef<T[]>;
    dataSource: ComputedRef<T[]>;
    dataKeyMap: ComputedRef<Map<VKey, T>>;
    filteredDataSource: ComputedRef<T[]>;
    paginatedDataSource: ComputedRef<T[]>;
    targetKeySet: ComputedRef<Set<VKey>>;
    selectedKeys: ComputedRef<VKey[]>;
    selectedKeySet: ComputedRef<Set<VKey>>;
    disabledKeys: ComputedRef<Set<VKey>>;
    disabledDataSourceKeys: ComputedRef<Set<VKey>>;
    selectAllDisabled: ComputedRef<boolean>;
    selectAllStatus: ComputedRef<{
        checked: boolean;
        indeterminate: boolean;
    }>;
    showSelectAll: ComputedRef<boolean>;
    searchable: ComputedRef<boolean>;
    searchPlaceholder: ComputedRef<string>;
    pagination: ComputedRef<PaginationProps | undefined>;
    triggerAppend: (keys: VKey[]) => void;
    triggerRemove: (keys: VKey[]) => void;
    getKey: ComputedRef<GetKeyFn>;
    handleSelectChange: (keys: Set<VKey> | VKey[]) => void;
    selectAll: (selected?: boolean) => void;
    searchValue: ComputedRef<string>;
    handleSearchChange: (value: string) => void;
}
export type TransferListSlotParams<T extends TransferData = TransferData> = ConvertToSlotParams<TransferBindings<T>> & {
    isSource: boolean;
};
export type TransferOperationsSlotParams = ConvertToSlotParams<TransferOperationsContext>;
export type TransferSlot<T extends TransferData = TransferData> = (params: TransferListSlotParams<T> & {
    isSource: boolean;
}) => VNode[];
export interface TransferSlots<T extends TransferData = TransferData> extends Slots {
    default?: TransferSlot<T>;
    header?: TransferSlot<T>;
    footer?: TransferSlot<T>;
    headerLabel?: (params: {
        data: T[];
        isSource: boolean;
    }) => VNode[];
    headerSuffix?: (params: {
        isSource: boolean;
    }) => VNode[];
    operations?: (operations: TransferOperationsSlotParams) => VNode[];
    label?: (params: {
        item: T;
        isSource: boolean;
    }) => VNode[];
    empty?: (params: EmptyProps) => VNode[];
    clearIcon?: () => VNode[];
}
export interface TransferPaginationProps {
    pageIndex?: [number | undefined, number | undefined] | [number | undefined] | number;
    pageSize?: [number | undefined, number | undefined] | [number | undefined] | number;
    disabled?: boolean;
    total?: [number | undefined, number | undefined] | [number | undefined] | number;
    onChange?: (isSource: boolean, pageIndex: number, pageSize: number) => void;
}
export interface TransferScroll {
    height?: string | number;
    width?: string | number | {
        source?: string | number;
        target?: string | number;
    };
    fullHeight?: boolean;
}
export type SearchFn<T extends TransferData = TransferData> = (isSource: boolean, item: T, searchValue: string) => boolean;
export declare const transferProps: {
    readonly value: PropType<VKey[]>;
    readonly sourceSelectedKeys: PropType<VKey[]>;
    readonly targetSelectedKeys: PropType<VKey[]>;
    readonly sourceSearchValue: StringConstructor;
    readonly targetSearchValue: StringConstructor;
    readonly clearable: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly clearIcon: StringConstructor;
    readonly customAdditional: {
        readonly type: PropType<TransferCustomAdditional>;
        readonly default: undefined;
    };
    readonly dataSource: {
        readonly type: PropType<TransferData<VKey>[]>;
        readonly default: () => TransferData[];
    };
    readonly dataStrategy: PropType<Partial<TransferDataStrategy<TransferData<VKey>, VKey>>>;
    readonly defaultTargetData: PropType<TransferData<VKey>[]>;
    readonly disabled: {
        readonly type: BooleanConstructor;
        readonly default: false;
    };
    readonly empty: {
        readonly type: PropType<"default" | "simple" | EmptyProps>;
        readonly default: "simple";
    };
    readonly getKey: PropType<string | ((item: TransferData<any>) => any)>;
    readonly mode: {
        readonly type: PropType<TransferMode>;
        readonly default: "default";
    };
    readonly pagination: {
        readonly type: PropType<boolean | TransferPaginationProps>;
        readonly default: undefined;
    };
    readonly scroll: PropType<TransferScroll>;
    readonly searchable: {
        readonly type: PropType<boolean | {
            source: boolean;
            target: boolean;
        }>;
        readonly default: undefined;
    };
    readonly searchFn: PropType<SearchFn<TransferData<VKey>>>;
    readonly searchPlaceholder: PropType<string | string[]>;
    readonly spin: {
        readonly type: PropType<boolean | {
            source: boolean;
            target: boolean;
        }>;
        readonly default: undefined;
    };
    readonly showSelectAll: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly virtual: {
        readonly type: BooleanConstructor;
        readonly default: false;
    };
    readonly virtualScrollMode: {
        readonly type: PropType<VirtualScrollMode>;
        readonly default: undefined;
    };
    readonly virtualItemHeight: {
        readonly type: NumberConstructor;
        readonly default: undefined;
    };
    readonly 'onUpdate:value': PropType<MaybeArray<(keys: any[]) => void>>;
    readonly 'onUpdate:sourceSelectedKeys': PropType<MaybeArray<(keys: any[]) => void>>;
    readonly 'onUpdate:targetSelectedKeys': PropType<MaybeArray<(keys: any[]) => void>>;
    readonly 'onUpdate:sourceSearchValue': PropType<MaybeArray<(searchValue: string) => void>>;
    readonly 'onUpdate:targetSearchValue': PropType<MaybeArray<(searchValue: string) => void>>;
    readonly onChange: PropType<MaybeArray<(keys: any[], oldKeys: any[]) => void>>;
    readonly onSearch: PropType<MaybeArray<(isSource: boolean, searchValue: string) => void>>;
    readonly onSelectAll: PropType<MaybeArray<(isSource: boolean, selectAll: boolean) => void>>;
    readonly onClear: PropType<MaybeArray<() => void>>;
    readonly onScroll: PropType<MaybeArray<(isSource: boolean, evt: Event) => void>>;
    readonly onScrolledChange: PropType<MaybeArray<(isSource: boolean, startIndex: number, endIndex: number, visibleData: unknown[]) => void>>;
    readonly onScrolledBottom: PropType<MaybeArray<(isSource: boolean) => void>>;
};
export declare const transferContentProps: {
    isSource: {
        type: BooleanConstructor;
        required: boolean;
    };
};
export declare const transferBodyProps: {
    isSource: {
        type: BooleanConstructor;
        required: boolean;
    };
};
export declare const transferHeaderProps: {
    isSource: {
        type: BooleanConstructor;
        required: boolean;
    };
};
export declare const transferFooterProps: {
    isSource: {
        type: BooleanConstructor;
        required: boolean;
    };
};
export interface TransferApis {
    scrollTo: (isSource: boolean, ...params: Parameters<VirtualScrollToFn>) => ReturnType<VirtualScrollToFn>;
}
export type TransferProps = ExtractInnerPropTypes<typeof transferProps>;
export type TransferPublicProps = ExtractPublicPropTypes<typeof transferProps>;
export type TransferComponent = DefineComponent<Omit<HTMLAttributes, keyof TransferPublicProps> & TransferPublicProps, TransferApis>;
export type TransferInstance = InstanceType<DefineComponent<TransferProps, TransferApis>>;
export type TransferCustomAdditional = (options: {
    data: TransferData;
    index: number;
}) => Record<string, any> | undefined;
export interface TransferData<K = VKey> {
    key?: K;
    label?: string;
    disabled?: boolean;
    [key: string]: unknown;
}
export interface TransferListScroll {
    height?: string | number;
    fullHeight?: boolean;
}
export declare const transferListProps: {
    readonly dataSource: PropType<TransferData<VKey>[]>;
    readonly checkable: {
        readonly type: BooleanConstructor;
        readonly default: true;
    };
    readonly removable: {
        readonly type: BooleanConstructor;
        readonly default: false;
    };
    readonly checked: PropType<(item: TransferData) => boolean>;
    readonly customAdditional: {
        readonly type: PropType<TransferCustomAdditional>;
        readonly default: undefined;
    };
    readonly disabled: PropType<(item: TransferData) => boolean>;
    readonly getKey: PropType<(item: TransferData) => VKey>;
    readonly labelKey: StringConstructor;
    readonly scroll: PropType<TransferScroll>;
    readonly virtual: {
        readonly type: BooleanConstructor;
        readonly default: false;
    };
    readonly virtualScrollMode: {
        readonly type: PropType<VirtualScrollMode>;
        readonly default: undefined;
    };
    readonly virtualItemHeight: {
        readonly type: NumberConstructor;
        readonly default: 32;
    };
    readonly onCheckChange: PropType<MaybeArray<(item: TransferData, checked: boolean) => void>>;
    readonly onRemove: PropType<MaybeArray<(item: TransferData) => void>>;
    readonly onScroll: PropType<MaybeArray<(evt: Event) => void>>;
    readonly onScrolledChange: PropType<MaybeArray<(startIndex: number, endIndex: number, visibleData: unknown[]) => void>>;
    readonly onScrolledBottom: PropType<MaybeArray<() => void>>;
};
export declare const transferListItemProps: {
    readonly checked: {
        readonly type: BooleanConstructor;
        readonly default: false;
    };
    readonly checkable: {
        readonly type: BooleanConstructor;
        readonly default: true;
    };
    readonly removable: {
        readonly type: BooleanConstructor;
        readonly default: false;
    };
    readonly disabled: {
        readonly type: BooleanConstructor;
        readonly default: false;
    };
    readonly label: StringConstructor;
    readonly value: {
        readonly type: PropType<VKey>;
        readonly required: true;
    };
    readonly onCheckChange: PropType<MaybeArray<(checked: boolean) => void>>;
    readonly onRemove: PropType<MaybeArray<() => void>>;
};
export interface TransferListApi {
    scrollTo: VirtualScrollToFn;
}
export type TransferListProps = ExtractInnerPropTypes<typeof transferListProps>;
export type TransferListItemProps = ExtractInnerPropTypes<typeof transferListItemProps>;
export type TransferListPublicProps = ExtractPublicPropTypes<typeof transferListProps>;
export type TransferListComponent = DefineComponent<Omit<HTMLAttributes, keyof TransferListPublicProps> & TransferListPublicProps, TransferListApi>;
export type TransferListInstance = InstanceType<DefineComponent<TransferListProps, TransferListApi>>;
