import { computed, ref, watch, onBeforeUnmount, h, watchEffect, defineComponent, inject, createVNode, normalizeClass, mergeProps, provide, normalizeStyle } from "vue";
import { isNil, isString, isFunction, isArray } from "lodash-es";
import { CdkVirtualScroll } from "@idux/cdk/scroll";
import { callEmit, useControlledProp, NoopArray, convertCssPixel } from "@idux/cdk/utils";
import { \u0275Empty as _Empty } from "@idux/components/_private/empty";
import { useGlobalConfig } from "@idux/components/config";
import { useThemeToken } from "@idux/components/theme";
import { useTreeCheckState, useGetKey } from "@idux/components/utils";
import { IxIcon } from "@idux/components/icon";
import { IxCheckbox } from "@idux/components/checkbox";
function callChange(mergedNodeMap, newKeys, onChange) {
  if (onChange) {
    const selectedNodes = [];
    const nodeMap = mergedNodeMap.value;
    newKeys.forEach((key) => {
      const currNode = nodeMap.get(key);
      currNode && selectedNodes.push(currNode.rawNode);
    });
    callEmit(onChange, newKeys, selectedNodes);
  }
}
function getChildrenKeys(currNode, disabledKeys) {
  const keys = [];
  const { children } = currNode || {};
  children && children.forEach((item) => {
    {
      keys.push(item.key);
    }
    keys.push(...getChildrenKeys(item));
  });
  return keys;
}
function getParentKeys(nodeMap, currNode, disabledKeys) {
  const keys = [];
  while (currNode && !isNil(currNode.parentKey)) {
    const { parentKey } = currNode;
    {
      keys.push(parentKey);
    }
    currNode = nodeMap.get(parentKey);
  }
  return keys;
}
function useCheckable(props, mergedNodes, mergedNodeMap, parentKeyMap, depthMap) {
  const [checkedKeys, setCheckedKeys] = useControlledProp(props, "checkedKeys", () => []);
  const resolverContext = computed(() => {
    return {
      data: mergedNodes.value,
      dataMap: mergedNodeMap.value,
      parentKeyMap: parentKeyMap.value,
      depthMap: depthMap.value
    };
  });
  const childrenKey = ref("children");
  const getKey = ref((item) => item.key);
  const cascaderStrategy = computed(() => props.cascaderStrategy);
  const isDisabled = ref((data) => !!data.checkDisabled);
  const { checkStateResolver, isChecked, isIndeterminate, toggle } = useTreeCheckState(
    checkedKeys,
    resolverContext,
    childrenKey,
    getKey,
    cascaderStrategy,
    isDisabled
  );
  const handleCheck = (node) => {
    const { checked, checkedKeys: newCheckedKeys } = toggle(node);
    if (node.checkDisabled) {
      return;
    }
    handleChange(checked, node.rawNode, newCheckedKeys);
  };
  const handleChange = (checked, rawNode, newKeys) => {
    const { onCheck, onCheckedChange } = props;
    callEmit(onCheck, checked, rawNode);
    setCheckedKeys(newKeys);
    callChange(mergedNodeMap, newKeys, onCheckedChange);
  };
  watch(cascaderStrategy, () => {
    const newCheckedKeys = checkStateResolver.appendKeys([], checkedKeys.value);
    setCheckedKeys(newCheckedKeys);
    callChange(mergedNodeMap, newCheckedKeys, props.onCheckedChange);
  });
  return {
    checkedKeys,
    isChecked,
    isIndeterminate,
    handleCheck
  };
}
function useMergeNodes(props, mergedChildrenKey, mergedGetKey, mergedLabelKey) {
  const loadedNodes = ref({});
  const setLoadedNodes = (key, nodes) => {
    loadedNodes.value[key] = nodes;
  };
  const getLoadedNodes = (key) => {
    return loadedNodes.value[key];
  };
  const context = computed(() => {
    const dataKeyMap = /* @__PURE__ */ new Map();
    const parentKeyMap2 = /* @__PURE__ */ new Map();
    const depthMap2 = /* @__PURE__ */ new Map();
    const mergedNodes2 = convertMergeNodes(
      props.dataSource,
      getLoadedNodes,
      {
        props,
        childrenKey: mergedChildrenKey.value,
        getKey: mergedGetKey.value,
        labelKey: mergedLabelKey.value,
        parentKey: void 0,
        parentLevel: -1,
        parentsDisabled: []
      },
      {
        dataKeyMap,
        parentKeyMap: parentKeyMap2,
        depthMap: depthMap2
      }
    );
    return {
      mergedNodes: mergedNodes2,
      dataKeyMap,
      parentKeyMap: parentKeyMap2,
      depthMap: depthMap2
    };
  });
  const mergedNodes = computed(() => context.value.mergedNodes);
  const mergedNodeMap = computed(() => context.value.dataKeyMap);
  const parentKeyMap = computed(() => context.value.parentKeyMap);
  const depthMap = computed(() => context.value.depthMap);
  return { mergedNodes, mergedNodeMap, parentKeyMap, depthMap, setLoadedNodes };
}
function useFlattedNodes(mergedNodes, { expandedKeys }, props, searchedKeys) {
  const flattenedContext = computed(() => {
    const { searchValue } = props;
    const expandedKeysMap = new Map(expandedKeys.value.map((item, index) => [item, index]));
    const searchedKeysMap = new Map(searchedKeys.value.map((item, index) => [item, index]));
    const flattenedNodeMap2 = /* @__PURE__ */ new Map();
    if (searchValue && !searchedKeysMap.size) {
      return {
        flattedNodes: [],
        flattenedNodeMap: flattenedNodeMap2
      };
    }
    if (expandedKeysMap.size || searchedKeysMap.size) {
      const nodes = flatNode(mergedNodes.value, expandedKeysMap, searchedKeysMap, flattenedNodeMap2);
      return {
        flattedNodes: nodes,
        flattenedNodeMap: flattenedNodeMap2
      };
    }
    return {
      flattedNodes: mergedNodes.value.map((item) => {
        const node = { ...item, expanded: false, level: 0 };
        flattenedNodeMap2.set(node.key, node);
        return node;
      }),
      flattenedNodeMap: flattenedNodeMap2
    };
  });
  const flattedNodes = computed(() => flattenedContext.value.flattedNodes);
  const flattenedNodeMap = computed(() => flattenedContext.value.flattenedNodeMap);
  return {
    flattedNodes,
    flattenedNodeMap
  };
}
function convertMergeNodes(nodes, getLoadedNodes, options, maps) {
  const { props, childrenKey, getKey, labelKey, parentKey, parentLevel, parentsDisabled } = options;
  const { cascaderStrategy, disabled, loadChildren } = props;
  const { dataKeyMap, parentKeyMap, depthMap } = maps;
  const hasLoad = !!loadChildren;
  return nodes.map((node, index) => {
    var _a, _b, _c;
    const key = getKey(node);
    const nodeDisabled = convertDisabled(node, disabled);
    const subNodes = (_a = node[childrenKey]) != null ? _a : getLoadedNodes(key);
    const label = node[labelKey];
    const level = parentLevel + 1;
    const children = subNodes && convertMergeNodes(
      subNodes,
      getLoadedNodes,
      {
        props,
        childrenKey,
        getKey,
        labelKey,
        parentKey: key,
        parentLevel: level,
        parentsDisabled: [nodeDisabled, ...parentsDisabled != null ? parentsDisabled : []]
      },
      maps
    );
    const mergedDisabled = mergeDisabled(
      nodeDisabled,
      parentsDisabled != null ? parentsDisabled : [],
      (_b = children == null ? void 0 : children.map((child) => ({
        check: child.checkDisabled,
        drag: child.dragDisabled,
        drop: child.dropDisabled,
        select: child.selectDisabled
      }))) != null ? _b : [],
      cascaderStrategy !== "off"
    );
    const mergedNode = {
      label,
      key,
      children,
      isFirst: index === 0,
      isLeaf: (_c = node.isLeaf) != null ? _c : !((subNodes == null ? void 0 : subNodes.length) || hasLoad),
      isLast: index === nodes.length - 1,
      parentKey,
      expanded: false,
      level,
      rawNode: node,
      checkDisabled: mergedDisabled.check,
      dragDisabled: mergedDisabled.drag,
      dropDisabled: mergedDisabled.drop,
      selectDisabled: mergedDisabled.select
    };
    dataKeyMap.set(key, mergedNode);
    parentKeyMap.set(key, parentKey);
    depthMap.set(key, level);
    return mergedNode;
  });
}
function convertDisabled(node, disabled) {
  const nodeDisabled = node.disabled || {};
  if (nodeDisabled === true) {
    return { check: true, drag: true, drop: true, select: true };
  }
  let { check, drag, drop, select } = nodeDisabled;
  if (disabled) {
    const treeDisabled = disabled(node) || {};
    if (treeDisabled === true) {
      return { check: true, drag: true, drop: true, select: true };
    }
    check = check || treeDisabled.check;
    drag = drag || treeDisabled.drag;
    drop = drop || treeDisabled.drop;
    select = select || treeDisabled.select;
  }
  return { check, drag, drop, select };
}
function mergeDisabled(nodeDisabled, parentsDisabled, childrenDisabled, cascade) {
  const _mergeDisabled = (name) => {
    return nodeDisabled[name] || cascade && (parentsDisabled.some((disabled) => !!disabled[name]) || !!childrenDisabled.length && childrenDisabled.every((disabled) => !!disabled[name]));
  };
  return {
    check: _mergeDisabled("check"),
    drag: _mergeDisabled("drag"),
    drop: _mergeDisabled("drop"),
    select: _mergeDisabled("select")
  };
}
function hiddenIrrelevantNodes(mergedNodes = [], searchedKeysMap) {
  const result = [];
  const mergedNodesLength = mergedNodes.length;
  mergedNodes.forEach((node, index) => {
    node.isFirst = index === 0;
    node.isLast = index === mergedNodesLength - 1;
    if (searchedKeysMap.has(node.key) || searchedKeysMap.size == 0) {
      node.hidden = false;
      result.push(node);
    } else {
      node.hidden = true;
    }
    const obj = hiddenIrrelevantNodes(node.children, searchedKeysMap);
    if (obj.length > 0) {
      node.hidden = false;
      result.push(node);
    }
  });
  if (result.length) {
    result[0].isFirst = true;
    result[result.length - 1].isLast = true;
  }
  return result;
}
function flatNode(mergedNodes, expandedKeysMap, searchedKeysMap, nodeMap) {
  const flattedNodes = [];
  const stack = [];
  hiddenIrrelevantNodes(mergedNodes, searchedKeysMap);
  mergedNodes.forEach((node) => {
    stack.push({ ...node });
    while (stack.length) {
      const _node = stack.pop();
      if (_node) {
        const { children, key, hidden } = _node;
        const expanded = expandedKeysMap.has(key);
        _node.expanded = expanded;
        !hidden && flattedNodes.push(_node);
        nodeMap.set(_node.key, _node);
        if (children && expanded) {
          for (let i = children.length; i > 0; i--) {
            !children[i - 1].hidden && stack.push(children[i - 1]);
          }
        }
      }
    }
  });
  return flattedNodes;
}
function useDragDrop(props, config, { expandedKeys, setExpandedKeys }) {
  const draggableIcon = computed(() => {
    var _a;
    return (_a = props.draggableIcon) != null ? _a : config.draggableIcon;
  });
  const dragNodeRef = ref();
  const dragChildrenKeys = ref();
  const dropNodeRef = ref();
  const dropParentKey = ref();
  const dropType = ref();
  let dragTimer;
  const clearTimer = () => {
    if (dragTimer) {
      clearTimeout(dragTimer);
      dragTimer = void 0;
    }
  };
  onBeforeUnmount(() => clearTimer());
  const cleanDragState = () => {
    if (!isNil(dragNodeRef.value)) {
      dragNodeRef.value = void 0;
      dragChildrenKeys.value = void 0;
    }
  };
  const cleanDropState = () => {
    if (!isNil(dropNodeRef.value)) {
      dropNodeRef.value = void 0;
      dropParentKey.value = void 0;
      dropType.value = void 0;
    }
  };
  const getDragDropOptions = (evt, node) => {
    var _a, _b;
    return {
      evt,
      node: node.rawNode,
      dragNode: (_a = dragNodeRef.value) == null ? void 0 : _a.rawNode,
      dropNode: (_b = dropNodeRef.value) == null ? void 0 : _b.rawNode,
      dropType: dropType.value
    };
  };
  const handleWindowDragend = (evt) => {
    handleDragend(evt, void 0);
    window.removeEventListener("dragend", handleWindowDragend);
  };
  const handleDragstart = (evt, node) => {
    dragNodeRef.value = node;
    dragChildrenKeys.value = getChildrenKeys(node);
    delKey(node.key, expandedKeys.value, setExpandedKeys);
    window.addEventListener("dragend", handleWindowDragend);
    callEmit(props.onDragstart, getDragDropOptions(evt, node));
  };
  const handleDragend = (evt, node) => {
    if (node) {
      callEmit(props.onDragend, getDragDropOptions(evt, node));
    }
    cleanDragState();
    cleanDropState();
  };
  const handleDragenter = async (evt, node) => {
    const dragNode = dragNodeRef.value;
    if (dragNode) {
      clearTimer();
      if (dragNode.key !== node.key) {
        dragTimer = setTimeout(() => {
          var _a;
          if (dragNodeRef.value && ((_a = node.children) == null ? void 0 : _a.length)) {
            addKey(node.key, expandedKeys.value, setExpandedKeys);
          }
          dragTimer = void 0;
        }, 1e3);
      }
      const type = await calcDropType(node, dragNode, dragChildrenKeys.value, evt, props.droppable);
      if (type) {
        dropNodeRef.value = node;
        dropParentKey.value = type !== "inside" ? node.parentKey : void 0;
        dropType.value = type;
      } else {
        cleanDropState();
      }
    }
    callEmit(props.onDragenter, getDragDropOptions(evt, node));
  };
  const handleDragover = async (evt, node) => {
    const dragNode = dragNodeRef.value;
    if (dragNode) {
      const type = await calcDropType(node, dragNode, dragChildrenKeys.value, evt, props.droppable);
      if (type) {
        dropNodeRef.value = node;
        dropParentKey.value = type !== "inside" ? node.parentKey : void 0;
        dropType.value = type;
      } else {
        cleanDropState();
      }
    }
    callEmit(props.onDragover, getDragDropOptions(evt, node));
  };
  const handleDragleave = (evt, node) => {
    const dropNode = dropNodeRef.value;
    if ((dropNode == null ? void 0 : dropNode.key) === node.key && !evt.currentTarget.contains(evt.relatedTarget)) {
      cleanDropState();
    }
    callEmit(props.onDragleave, getDragDropOptions(evt, node));
  };
  const handleDrop = (evt, node) => {
    if (!dropType.value) {
      cleanDropState();
    }
    callEmit(props.onDrop, getDragDropOptions(evt, node));
    cleanDragState();
    cleanDropState();
  };
  return {
    draggableIcon,
    dragKey: computed(() => {
      var _a;
      return (_a = dragNodeRef.value) == null ? void 0 : _a.key;
    }),
    dropKey: computed(() => {
      var _a;
      return (_a = dropNodeRef.value) == null ? void 0 : _a.key;
    }),
    dropParentKey,
    dropType,
    handleDragstart,
    handleDragend,
    handleDragenter,
    handleDragover,
    handleDragleave,
    handleDrop
  };
}
function addKey(key, keys, setKeys) {
  const index = keys.indexOf(key);
  if (index === -1) {
    setKeys([...keys, key]);
  }
}
function delKey(key, keys, setKeys) {
  const index = keys.indexOf(key);
  if (index !== -1) {
    const tempKeys = [...keys];
    tempKeys.splice(index, 1);
    setKeys(tempKeys);
  }
}
async function calcDropType(dropNode, dragNode, dragChildrenKeys, evt, droppable) {
  const { key: dropKey, children: dropChildren = [] } = dropNode;
  if (dragNode.key === dropKey || dragChildrenKeys && dragChildrenKeys.includes(dropKey)) {
    return false;
  }
  const { clientY } = evt;
  const { top, height } = evt.target.getBoundingClientRect();
  const isTopHalf = clientY < top + height / 2;
  let dropType;
  if (droppable) {
    const dropOptions = { evt, isTopHalf, dragNode: dragNode.rawNode, dropNode: dropNode.rawNode };
    dropType = await droppable(dropOptions);
  }
  if (dropType === void 0 || dropType === true) {
    if (dropChildren.length > 0) {
      dropType = "inside";
    } else if (isTopHalf) {
      dropType = "before";
    } else {
      dropType = "after";
    }
  }
  return dropType;
}
function useEvents(props, mergedNodeMap, flattedNodes, { expandedKeys, handleExpand }, { activeKey, handleSelect }) {
  const focused = ref(false);
  const handleFocus = (evt) => {
    focused.value = true;
    callEmit(props.onFocus, evt);
  };
  const handleBlur = (evt) => {
    focused.value = true;
    callEmit(props.onBlur, evt);
  };
  const handleKeydown = (evt) => {
    switch (evt.code) {
      case "ArrowUp":
      case "ArrowDown":
        evt.preventDefault();
    }
    callEmit(props.onKeydown, evt);
  };
  const handleKeyup = (evt) => {
    const eventCode = evt.code;
    const _activeKey = activeKey.value;
    const _flattedNodes = flattedNodes.value;
    const nodeSize = _flattedNodes.length;
    if (_activeKey === void 0) {
      if (["ArrowDown", "ArrowUp", "ArrowLeft", "ArrowRight"].includes(eventCode)) {
        changeActiveKey(activeKey, _flattedNodes, 0, nodeSize);
      }
    } else {
      const _flattedNodes2 = flattedNodes.value;
      const currIndex = _flattedNodes2.findIndex((node) => node.key === _activeKey);
      if (currIndex === -1) {
        return;
      }
      switch (eventCode) {
        case "Enter":
        case "NumpadEnter":
          handleSelect(_activeKey);
          break;
        case "ArrowDown":
          changeActiveKey(activeKey, _flattedNodes2, currIndex + 1, nodeSize);
          break;
        case "ArrowUp":
          changeActiveKey(activeKey, _flattedNodes2, currIndex - 1, 0, true);
          break;
        case "ArrowLeft":
          {
            const { rawNode, key, parentKey } = _flattedNodes2[currIndex];
            if (rawNode.isLeaf || !expandedKeys.value.includes(key)) {
              const parentNode = parentKey ? mergedNodeMap.value.get(parentKey) : void 0;
              if (parentNode) {
                activeKey.value = parentNode.key;
              }
            } else {
              handleExpand(key, rawNode);
            }
          }
          break;
        case "ArrowRight":
          {
            const { rawNode, key } = _flattedNodes2[currIndex];
            if (!expandedKeys.value.includes(key)) {
              handleExpand(key, rawNode);
            } else {
              for (let index = currIndex + 1; index < nodeSize; index++) {
                const currNode = _flattedNodes2[index];
                activeKey.value = currNode.key;
                break;
              }
            }
          }
          break;
      }
    }
    callEmit(props.onKeyup, evt);
  };
  return {
    focused,
    handleFocus,
    handleBlur,
    handleKeydown,
    handleKeyup
  };
}
function changeActiveKey(activeKey, flattedNodes, start, end, isUp = false) {
  let index = start;
  while (isUp ? index >= end : index < end) {
    const currNode = flattedNodes[index];
    if (!currNode.selectDisabled) {
      activeKey.value = currNode.key;
      return;
    } else {
      isUp ? index-- : index++;
    }
  }
}
function useExpandable(props, config, mergedChildrenKey, mergedNodeMap, searchedKeys, setLoadedNodes) {
  const expandIcon = computed(() => {
    var _a;
    return (_a = props.expandIcon) != null ? _a : config.expandIcon;
  });
  const [expandedKeys, setExpandedKeys] = useControlledProp(props, "expandedKeys", () => []);
  const [loadedKeys, setLoadedKeys] = useControlledProp(props, "loadedKeys", () => []);
  const loadingKeys = ref([]);
  const expandIconRenderer = ({ key, expanded, node }) => {
    if (isString(expandIcon.value)) {
      return h(IxIcon, { name: expandIcon.value, rotate: expanded ? 90 : 0 });
    }
    if (isFunction(expandIcon.value)) {
      return expandIcon.value({ key, expanded, node });
    }
    if (isArray(expandIcon.value)) {
      return h(IxIcon, { name: expanded ? expandIcon.value[0] : expandIcon.value[1] });
    }
  };
  const setExpandWithSearch = (searchedKeys2) => {
    if (!searchedKeys2 || searchedKeys2.length <= 0) {
      return;
    }
    const { onExpandedChange } = props;
    const nodeMap = mergedNodeMap.value;
    const keySet = new Set(expandedKeys.value);
    searchedKeys2.forEach((key) => {
      getParentKeys(nodeMap, nodeMap.get(key)).forEach((parentKey) => keySet.add(parentKey));
    });
    const newKeys = [...keySet];
    setExpandedKeys(newKeys);
    callChange(mergedNodeMap, newKeys, onExpandedChange);
  };
  watch(searchedKeys, setExpandWithSearch);
  const handleExpand = async (key, rawNode) => {
    var _a;
    const { loadChildren } = props;
    if (loadingKeys.value.includes(key)) {
      return;
    }
    const childrenKey = mergedChildrenKey.value;
    if (!((_a = rawNode == null ? void 0 : rawNode[childrenKey]) == null ? void 0 : _a.length)) {
      if (!loadChildren || loadingKeys.value.includes(key) || loadedKeys.value.includes(key)) {
        return;
      }
      loadingKeys.value.push(key);
      const childrenNodes = await loadChildren(rawNode);
      loadingKeys.value.splice(loadingKeys.value.indexOf(key), 1);
      const nodeMap = mergedNodeMap.value;
      const currNode = nodeMap.get(key);
      if (childrenNodes == null ? void 0 : childrenNodes.length) {
        setLoadedNodes(key, childrenNodes);
        currNode.rawNode[childrenKey] = childrenNodes;
        const newLoadedKeys = [...loadedKeys.value, key];
        setLoadedKeys(newLoadedKeys);
        callEmit(props.onLoaded, newLoadedKeys, rawNode);
      } else {
        return;
      }
    }
    const index = expandedKeys.value.indexOf(key);
    const expanded = index >= 0;
    const tempKeys = [...expandedKeys.value];
    expanded ? tempKeys.splice(index, 1) : tempKeys.push(key);
    handleChange(expanded, rawNode, tempKeys);
  };
  const handleChange = (expanded, rawNode, newKeys) => {
    const { onExpand, onExpandedChange } = props;
    callEmit(onExpand, !expanded, rawNode);
    setExpandedKeys(newKeys);
    callChange(mergedNodeMap, newKeys, onExpandedChange);
  };
  const expandAll = () => {
    const expandAllKeys = [];
    const expendedAllNodes = [];
    mergedNodeMap.value.forEach((node) => {
      if (!node.isLeaf) {
        expandAllKeys.push(node.key);
        expendedAllNodes.push(node.rawNode);
      }
    });
    callEmit(props.onExpandedChange, expandAllKeys, expendedAllNodes);
    setExpandedKeys(expandAllKeys);
  };
  const collapseAll = () => {
    callEmit(props.onExpandedChange, [], []);
    setExpandedKeys([]);
  };
  if (searchedKeys.value.length) {
    setExpandWithSearch(searchedKeys.value);
  }
  return { expandIconRenderer, expandedKeys, setExpandedKeys, expandAll, collapseAll, handleExpand, loadingKeys };
}
function useSearchable(props, mergedNodeMap, mergedLabelKey) {
  const mergedSearchFn = useSearchFn(props, mergedLabelKey);
  const searchedKeys = computed(() => {
    const { searchValue } = props;
    if (!searchValue) {
      return NoopArray;
    }
    const searchFn = mergedSearchFn.value;
    const keys = [];
    mergedNodeMap.value.forEach((node) => {
      if (searchFn(node.rawNode, searchValue)) {
        keys.push(node.key);
      }
    });
    return keys;
  });
  return { searchedKeys };
}
function useSearchFn(props, mergedLabelKey) {
  return computed(() => {
    var _a;
    return (_a = props.searchFn) != null ? _a : getDefaultSearchFn(mergedLabelKey.value);
  });
}
function getDefaultSearchFn(labelKey) {
  return (data, searchValue) => {
    const label = data[labelKey];
    return label ? label.toLowerCase().includes(searchValue.toLowerCase()) : false;
  };
}
function useSelectable(props, mergedNodeMap) {
  const [selectedKeys, setSelectedKeys] = useControlledProp(props, "selectedKeys", () => []);
  const isMultiple = computed(() => props.selectable === "multiple");
  const activeKey = ref();
  watchEffect(() => {
    const currKeys = selectedKeys.value;
    const keySize = currKeys.length;
    activeKey.value = keySize > 0 ? currKeys[keySize - 1] : void 0;
  });
  const activeNode = computed(() => {
    const currKey = activeKey.value;
    return currKey !== void 0 ? mergedNodeMap.value.get(currKey) : void 0;
  });
  const handleSelect = (key) => {
    const nodeMap = mergedNodeMap.value;
    const currNode = nodeMap.get(key);
    if (!currNode) {
      return;
    }
    const index = selectedKeys.value.indexOf(key);
    const selected = index > -1;
    let tempKeys = [...selectedKeys.value];
    if (isMultiple.value) {
      selected ? tempKeys.splice(index, 1) : tempKeys.push(key);
    } else {
      tempKeys = selected ? [] : [key];
    }
    handleChange(selected, currNode.rawNode, tempKeys);
  };
  const handleChange = (selected, rawNode, newKeys) => {
    var _a;
    const { onSelect, onSelectedChange } = props;
    const realSelected = newKeys.includes((_a = rawNode.key) != null ? _a : "") ? true : !selected;
    callEmit(onSelect, realSelected, rawNode);
    setSelectedKeys(newKeys);
    callChange(mergedNodeMap, newKeys, onSelectedChange);
  };
  return { activeKey, activeNode, selectedKeys, handleSelect };
}
const treeToken = Symbol("treeToken");
const treeProps = {
  autoHeight: {
    type: Boolean,
    default: void 0
  },
  checkedKeys: Array,
  expandedKeys: Array,
  indeterminateKeys: Array,
  loadedKeys: Array,
  selectedKeys: Array,
  blocked: {
    type: Boolean,
    default: void 0
  },
  cascaderStrategy: { type: String, default: "off" },
  checkable: {
    type: Boolean,
    default: false
  },
  childrenKey: String,
  checkOnClick: {
    type: Boolean,
    default: false
  },
  customAdditional: { type: Function, default: void 0 },
  dataSource: {
    type: Array,
    default: () => []
  },
  disabled: Function,
  draggable: {
    type: Boolean,
    default: false
  },
  draggableIcon: { type: String, default: void 0 },
  droppable: Function,
  empty: { type: [String, Object], default: "simple" },
  expandIcon: {
    type: [String, Function, Array],
    default: void 0
  },
  getKey: { type: [String, Function], default: void 0 },
  height: Number,
  labelKey: String,
  leafLineIcon: String,
  loadChildren: Function,
  searchFn: Function,
  searchValue: String,
  selectable: {
    type: [Boolean, String],
    default: true
  },
  showLine: { type: Boolean, default: void 0 },
  virtual: { type: Boolean, default: false },
  virtualScrollMode: { type: String, default: void 0 },
  virtualItemHeight: { type: Number, default: 32 },
  "onUpdate:checkedKeys": [Function, Array],
  "onUpdate:expandedKeys": [Function, Array],
  "onUpdate:loadedKeys": [Function, Array],
  "onUpdate:selectedKeys": [Function, Array],
  onCheck: [Function, Array],
  onCheckedChange: [Function, Array],
  onDragstart: [Function, Array],
  onDragend: [Function, Array],
  onDragenter: [Function, Array],
  onDragleave: [Function, Array],
  onDragover: [Function, Array],
  onDrop: [Function, Array],
  onExpand: [Function, Array],
  onExpandedChange: [Function, Array],
  onLoaded: [Function, Array],
  onSelect: [Function, Array],
  onSelectedChange: [Function, Array],
  onFocus: [Function, Array],
  onBlur: [Function, Array],
  onKeydown: [Function, Array],
  onKeyup: [Function, Array],
  onNodeClick: [Function, Array],
  onNodeContextmenu: [Function, Array],
  onScroll: [Function, Array],
  onScrolledChange: [Function, Array],
  onScrolledBottom: [Function, Array]
};
const treeNodeProps = {
  node: {
    type: Object,
    required: true
  },
  isLeaf: {
    type: Boolean,
    required: true
  },
  isFirst: {
    type: Boolean,
    required: true
  },
  isLast: {
    type: Boolean,
    required: true
  },
  label: String,
  level: {
    type: Number,
    required: true
  },
  rawNode: {
    type: Object,
    required: true
  },
  expanded: {
    type: Boolean,
    required: true
  },
  children: Array,
  parentKey: [String, Number, Symbol],
  checkDisabled: {
    type: Boolean,
    default: void 0
  },
  dragDisabled: {
    type: Boolean,
    default: void 0
  },
  dropDisabled: {
    type: Boolean,
    default: void 0
  },
  selectDisabled: {
    type: Boolean,
    default: void 0
  }
};
const treeNodeCheckboxProps = {
  node: {
    type: Object,
    required: true
  },
  checkDisabled: {
    type: Boolean,
    default: void 0
  }
};
const treeNodeExpandProps = {
  expanded: {
    type: Boolean,
    required: true
  },
  hasTopLine: {
    type: Boolean,
    default: void 0
  },
  hasBottomLine: {
    type: Boolean,
    default: void 0
  },
  isLeaf: {
    type: Boolean,
    default: void 0
  },
  nodeKey: {
    type: [String, Number, Symbol],
    required: true
  },
  rawNode: {
    type: Object,
    required: true
  }
};
const treeNodeContentProps = {
  disabled: {
    type: Boolean,
    default: void 0
  },
  node: {
    type: Object,
    required: true
  },
  nodeKey: {
    type: [String, Number, Symbol],
    required: true
  },
  label: String,
  selected: {
    type: Boolean,
    default: void 0
  }
};
var Checkbox = /* @__PURE__ */ defineComponent({
  props: treeNodeCheckboxProps,
  setup(props) {
    const {
      mergedPrefixCls,
      isChecked: _isChecked,
      isIndeterminate: _isIndeterminate,
      handleCheck
    } = inject(treeToken);
    const isChecked = computed(() => {
      return _isChecked(props.node.key);
    });
    const isIndeterminate = computed(() => _isIndeterminate(props.node.key));
    const onChange = () => handleCheck(props.node);
    return () => createVNode(IxCheckbox, {
      "class": `${mergedPrefixCls.value}-node-checkbox`,
      "checked": isChecked.value,
      "disabled": props.checkDisabled,
      "indeterminate": isIndeterminate.value,
      "onChange": onChange
    }, null);
  }
});
var Content = /* @__PURE__ */ defineComponent({
  props: treeNodeContentProps,
  setup(props, {
    slots
  }) {
    const {
      props: treeProps2,
      mergedPrefixCls,
      mergedCheckOnClick,
      handleSelect,
      handleCheck,
      searchedKeys
    } = inject(treeToken);
    const searched = computed(() => searchedKeys.value.includes(props.nodeKey));
    const onClick = (evt) => {
      if (!props.disabled) {
        handleSelect(props.nodeKey);
        mergedCheckOnClick.value && handleCheck(props.node);
      }
      callEmit(treeProps2.onNodeClick, evt, props.node.rawNode);
    };
    const onContextmenu = (evt) => {
      callEmit(treeProps2.onNodeContextmenu, evt, props.node.rawNode);
    };
    return () => {
      var _a, _b;
      const {
        nodeKey,
        label,
        node,
        selected
      } = props;
      const {
        rawNode
      } = node;
      const {
        prefix,
        suffix
      } = rawNode;
      const iconProps = {
        key: nodeKey,
        selected,
        node: rawNode
      };
      const prefixIcon = ((_a = slots.prefix) == null ? void 0 : _a.call(slots, iconProps)) || prefix && createVNode(IxIcon, {
        "name": prefix
      }, null);
      const suffixIcon = ((_b = slots.suffix) == null ? void 0 : _b.call(slots, iconProps)) || suffix && createVNode(IxIcon, {
        "name": suffix
      }, null);
      const prefixCls = `${mergedPrefixCls.value}-node-content`;
      return createVNode("span", {
        "class": prefixCls,
        "onClick": onClick,
        "onContextmenu": onContextmenu
      }, [prefixIcon && createVNode("span", {
        "class": `${prefixCls}-prefix`
      }, [prefixIcon]), createVNode("span", {
        "class": `${prefixCls}-label`
      }, [renderLabel(slots.label, label, rawNode, treeProps2.searchValue, searched.value, prefixCls)]), suffixIcon && createVNode("span", {
        "class": `${prefixCls}-suffix`
      }, [suffixIcon])]);
    };
  }
});
function renderLabel(labelSlot, label, node, searchValue, searched, prefixCls) {
  if (labelSlot) {
    return labelSlot({
      node,
      searchValue,
      searched
    });
  }
  if (searched && label && searchValue) {
    const startIndex = label.toUpperCase().indexOf(searchValue.toUpperCase());
    if (startIndex > -1) {
      const endIndex = startIndex + searchValue.length;
      const beforeLabel = label.substring(0, startIndex);
      const afterLabel = label.substring(endIndex);
      const highlightLabel = createVNode("span", {
        "class": `${prefixCls}-label-highlight`
      }, [label.substring(startIndex, endIndex)]);
      return [beforeLabel, highlightLabel, afterLabel];
    }
  }
  return label;
}
var Expand = /* @__PURE__ */ defineComponent({
  props: treeNodeExpandProps,
  setup(props, {
    slots
  }) {
    const {
      mergedPrefixCls,
      expandIconRenderer,
      loadingKeys,
      handleExpand
    } = inject(treeToken);
    const isLoading = computed(() => loadingKeys.value.includes(props.nodeKey));
    const classes = computed(() => {
      const prefixCls = `${mergedPrefixCls.value}-node-expand`;
      return normalizeClass({
        [prefixCls]: true,
        [`${prefixCls}-noop`]: props.isLeaf
      });
    });
    const onClick = (evt) => {
      handleExpand(props.nodeKey, props.rawNode);
      evt.stopPropagation();
    };
    return () => {
      var _a, _b;
      const prefixCls = `${mergedPrefixCls.value}-node-expand`;
      let children;
      if (isLoading.value) {
        children = createVNode(IxIcon, {
          "name": "loading"
        }, null);
      } else if (!props.isLeaf) {
        const {
          expanded,
          nodeKey: key,
          rawNode: node
        } = props;
        children = (_b = (_a = slots.expandIcon) != null ? _a : expandIconRenderer) == null ? void 0 : _b({
          key,
          expanded,
          node
        });
      }
      return createVNode("span", {
        "class": classes.value,
        "onClick": onClick
      }, [props.hasTopLine && createVNode("div", {
        "class": `${prefixCls}-top-line`
      }, null), children, props.hasBottomLine && createVNode("div", {
        "class": `${prefixCls}-bottom-line`
      }, null)]);
    };
  }
});
const Indent = ({
  level,
  noopIdentUnitArr,
  prefixCls
}) => {
  const children = [];
  for (let index = 0; index < level; index++) {
    children.push(createVNode("span", {
      "key": index,
      "class": noopIdentUnitArr.includes(index) ? `${prefixCls}-node-indent-noop-unit` : `${prefixCls}-node-indent-unit`
    }, null));
  }
  return createVNode("span", {
    "aria-hidden": true,
    "class": `${prefixCls}-node-indent`
  }, [children]);
};
var LeafLine = /* @__PURE__ */ defineComponent({
  setup(_, {
    slots
  }) {
    const {
      props,
      mergedPrefixCls
    } = inject(treeToken);
    return () => {
      const prefixCls = `${mergedPrefixCls.value}-node-leaf`;
      let children;
      if (slots.leafLineIcon) {
        children = slots.leafLineIcon();
      } else if (props.leafLineIcon) {
        children = createVNode(IxIcon, {
          "name": props.leafLineIcon
        }, null);
      } else {
        children = createVNode("span", {
          "class": `${prefixCls}-line`
        }, null);
      }
      return createVNode("span", {
        "class": prefixCls
      }, [children]);
    };
  }
});
var TreeNode = /* @__PURE__ */ defineComponent({
  props: treeNodeProps,
  setup(props, {
    slots
  }) {
    const {
      props: treeProps2,
      flattenedNodeMap,
      mergedPrefixCls,
      mergedShowLine,
      activeKey,
      selectedKeys,
      draggableIcon,
      dragKey,
      dropKey,
      dropParentKey,
      dropType,
      handleDragstart,
      handleDragend,
      handleDragenter,
      handleDragover,
      handleDragleave,
      handleDrop
    } = inject(treeToken);
    const nodeKey = computed(() => props.node.key);
    const isActive = computed(() => activeKey.value === nodeKey.value);
    const isLast = computed(() => mergedShowLine.value && props.isLast);
    const hasTopLine = computed(() => mergedShowLine.value && !props.isLeaf && (!props.isFirst || props.level !== 0));
    const hasBottomLine = computed(() => mergedShowLine.value && !props.isLeaf && !props.isLast);
    const selected = computed(() => selectedKeys.value.includes(nodeKey.value));
    const disabled = computed(() => props.selectDisabled || !treeProps2.selectable);
    const dragging = computed(() => dragKey.value === nodeKey.value);
    const dropping = computed(() => dropKey.value === nodeKey.value);
    const dropParent = computed(() => dropParentKey.value === nodeKey.value);
    const dropBefore = computed(() => dropping.value && dropType.value === "before");
    const dropInside = computed(() => dropping.value && dropType.value === "inside");
    const dropAfter = computed(() => dropping.value && dropType.value === "after");
    const classes = computed(() => {
      const prefixCls = `${mergedPrefixCls.value}-node`;
      return {
        [prefixCls]: true,
        [`${prefixCls}-active`]: isActive.value,
        [`${prefixCls}-last`]: isLast.value,
        [`${prefixCls}-disabled`]: disabled.value,
        [`${prefixCls}-selected`]: selected.value,
        [`${prefixCls}-expanded`]: props.expanded,
        [`${prefixCls}-dragging`]: dragging.value,
        [`${prefixCls}-dropping`]: dropping.value,
        [`${prefixCls}-drop-parent`]: dropParent.value,
        [`${prefixCls}-drop-before`]: dropBefore.value,
        [`${prefixCls}-drop-inside`]: dropInside.value,
        [`${prefixCls}-drop-after`]: dropAfter.value
      };
    });
    const onDragstart = (evt) => {
      var _a;
      evt.stopPropagation();
      handleDragstart(evt, props.node);
      (_a = evt.dataTransfer) == null ? void 0 : _a.setData("text/plain", "");
    };
    const onDragend = (evt) => {
      evt.stopPropagation();
      handleDragend(evt, props.node);
    };
    const onDragenter = (evt) => {
      evt.preventDefault();
      evt.stopPropagation();
      handleDragenter(evt, props.node);
    };
    const onDragover = (evt) => {
      evt.preventDefault();
      evt.stopPropagation();
      handleDragover(evt, props.node);
    };
    const onDragleave = (evt) => {
      evt.stopPropagation();
      handleDragleave(evt, props.node);
    };
    const onDrop = (evt) => {
      evt.stopPropagation();
      handleDrop(evt, props.node);
    };
    return () => {
      var _a, _b;
      const nodeMap = flattenedNodeMap.value;
      const {
        isLeaf,
        label,
        level,
        rawNode,
        expanded,
        checkDisabled,
        dragDisabled,
        dropDisabled,
        node
      } = props;
      const {
        checkable,
        draggable
      } = treeProps2;
      const mergedDraggable = draggable && !dragDisabled;
      const draggableIconNode = (_b = (_a = slots.draggableIcon) == null ? void 0 : _a.call(slots)) != null ? _b : createVNode(IxIcon, {
        "name": draggableIcon.value
      }, null);
      const currNode = nodeMap.get(nodeKey.value);
      const noopIdentUnitArr = [];
      if (mergedShowLine.value) {
        getParentKeys(nodeMap, currNode).reverse().forEach((parentKey, index) => {
          var _a2;
          if ((_a2 = nodeMap.get(parentKey)) == null ? void 0 : _a2.isLast) {
            noopIdentUnitArr.push(index);
          }
        });
      }
      const customAdditional = treeProps2.customAdditional ? treeProps2.customAdditional({
        node: rawNode,
        level
      }) : void 0;
      return createVNode("div", mergeProps({
        "class": classes.value,
        "aria-grabbed": dragging.value || void 0,
        "aria-label": label,
        "aria-selected": selected.value,
        "draggable": mergedDraggable || void 0,
        "title": label,
        "onDragstart": mergedDraggable ? onDragstart : void 0,
        "onDragend": mergedDraggable ? onDragend : void 0,
        "onDragenter": mergedDraggable ? onDragenter : void 0,
        "onDragover": mergedDraggable ? onDragover : void 0,
        "onDragleave": mergedDraggable ? onDragleave : void 0,
        "onDrop": mergedDraggable && !dropDisabled ? onDrop : void 0
      }, customAdditional), [createVNode(Indent, {
        "level": level,
        "noopIdentUnitArr": noopIdentUnitArr,
        "prefixCls": mergedPrefixCls.value
      }, null), mergedDraggable ? createVNode("span", {
        "class": `${mergedPrefixCls.value}-node-draggable-icon`
      }, [draggableIconNode]) : draggable && createVNode("span", {
        "class": `${mergedPrefixCls.value}-node-draggable-icon-noop`
      }, null), isLeaf && mergedShowLine.value ? createVNode(LeafLine, null, slots) : createVNode(Expand, {
        "expanded": expanded,
        "hasTopLine": hasTopLine.value,
        "hasBottomLine": hasBottomLine.value,
        "isLeaf": isLeaf,
        "nodeKey": nodeKey.value,
        "rawNode": rawNode
      }, slots), checkable && createVNode(Checkbox, {
        "checkDisabled": checkDisabled,
        "node": node
      }, null), createVNode(Content, {
        "disabled": disabled.value,
        "node": node,
        "nodeKey": nodeKey.value,
        "label": label,
        "selected": selected.value
      }, slots)]);
    };
  }
});
function getDefaultThemeTokens(tokens) {
  const {
    heightMd,
    heightSm,
    marginSizeXs,
    colorPrimaryHover,
    colorPrimaryText,
    colorContainerBgHover,
    colorContainerBg,
    colorIconInfo
  } = tokens;
  const nodeIconWidth = heightSm;
  return {
    dropLineWidth: 2,
    dropLineColor: colorPrimaryHover,
    bgColor: colorContainerBg,
    nodeBgColorHover: colorContainerBgHover,
    nodeBgColorSelected: colorContainerBg,
    nodeColorSelected: colorPrimaryText,
    nodePaddingHorizontal: marginSizeXs,
    nodePaddingVertical: `0px`,
    nodeContentHeight: heightMd,
    nodeContentLabelHighlightColor: colorPrimaryText,
    nodeContentPrefixMarginRight: marginSizeXs,
    nodeCheckboxMarginRight: marginSizeXs,
    nodeIconWidth,
    expandIconColor: colorIconInfo
  };
}
const getThemeTokens = (tokens, presetTheme) => {
  return presetTheme === "default" ? getDefaultThemeTokens(tokens) : getDefaultThemeTokens(tokens);
};
const hiddenStyle = {
  width: 0,
  height: 0,
  display: "flex",
  overflow: "hidden",
  opacity: 0,
  border: 0,
  padding: 0,
  margin: 0
};
var Tree = /* @__PURE__ */ defineComponent({
  name: "IxTree",
  props: treeProps,
  setup(props, {
    attrs,
    expose,
    slots
  }) {
    const common = useGlobalConfig("common");
    const {
      globalHashId,
      hashId,
      registerToken
    } = useThemeToken("tree");
    registerToken(getThemeTokens);
    const mergedPrefixCls = computed(() => `${common.prefixCls}-tree`);
    const config = useGlobalConfig("tree");
    const autoHeight = computed(() => {
      var _a;
      return (_a = props.autoHeight) != null ? _a : config.autoHeight;
    });
    const mergedChildrenKey = computed(() => {
      var _a;
      return (_a = props.childrenKey) != null ? _a : config.childrenKey;
    });
    const mergedGetKey = useGetKey(props, config, "components/tree");
    const mergedLabelKey = computed(() => {
      var _a;
      return (_a = props.labelKey) != null ? _a : config.labelKey;
    });
    const mergedShowLine = computed(() => {
      var _a;
      return (_a = props.showLine) != null ? _a : config.showLine;
    });
    const mergedBlocked = computed(() => {
      var _a;
      return (_a = props.blocked) != null ? _a : config.blocked;
    });
    const mergedCheckOnClick = computed(() => props.checkable && props.checkOnClick);
    const {
      mergedNodes,
      mergedNodeMap,
      parentKeyMap,
      depthMap,
      setLoadedNodes
    } = useMergeNodes(props, mergedChildrenKey, mergedGetKey, mergedLabelKey);
    const {
      searchedKeys
    } = useSearchable(props, mergedNodeMap, mergedLabelKey);
    const expandableContext = useExpandable(props, config, mergedChildrenKey, mergedNodeMap, searchedKeys, setLoadedNodes);
    const {
      flattedNodes,
      flattenedNodeMap
    } = useFlattedNodes(mergedNodes, expandableContext, props, searchedKeys);
    const checkableContext = useCheckable(props, mergedNodes, mergedNodeMap, parentKeyMap, depthMap);
    const dragDropContext = useDragDrop(props, config, expandableContext);
    const selectableContext = useSelectable(props, mergedNodeMap);
    const {
      handleCheck
    } = checkableContext;
    provide(treeToken, {
      props,
      config,
      flattedNodes,
      flattenedNodeMap,
      mergedPrefixCls,
      mergedNodeMap,
      mergedGetKey,
      mergedShowLine,
      mergedCheckOnClick,
      searchedKeys,
      ...checkableContext,
      ...expandableContext,
      ...dragDropContext,
      ...selectableContext
    });
    const inputRef = ref();
    const virtualScrollRef = ref();
    const {
      activeKey
    } = selectableContext;
    const {
      focused,
      handleFocus,
      handleBlur,
      handleKeydown,
      handleKeyup
    } = useEvents(props, mergedNodeMap, flattedNodes, expandableContext, selectableContext);
    const classes = computed(() => {
      const prefixCls = mergedPrefixCls.value;
      return {
        [globalHashId.value]: !!globalHashId.value,
        [hashId.value]: !!hashId.value,
        [prefixCls]: true,
        [`${prefixCls}-active`]: !isNil(activeKey.value),
        [`${prefixCls}-blocked`]: mergedBlocked.value,
        [`${prefixCls}-virtual`]: props.virtual,
        [`${prefixCls}-auto-height`]: autoHeight.value,
        [`${prefixCls}-focused`]: focused.value,
        [`${prefixCls}-show-line`]: mergedShowLine.value,
        [`${prefixCls}-empty`]: !flattedNodes.value.length
      };
    });
    const accessibilityPath = computed(() => {
      var _a, _b;
      const _activeKey = activeKey.value;
      if (isNil(_activeKey)) {
        return "";
      }
      const nodeMap = mergedNodeMap.value;
      let path = String(_activeKey);
      let parentKey = (_a = nodeMap.get(_activeKey)) == null ? void 0 : _a.parentKey;
      while (parentKey) {
        path = `${String(parentKey)} > ${path}`;
        parentKey = (_b = nodeMap.get(parentKey)) == null ? void 0 : _b.parentKey;
      }
      return path;
    });
    const focus = (options) => {
      var _a;
      (_a = inputRef == null ? void 0 : inputRef.value) == null ? void 0 : _a.focus(options);
    };
    const blur = () => {
      var _a;
      (_a = inputRef == null ? void 0 : inputRef.value) == null ? void 0 : _a.blur();
    };
    const scrollTo = (option) => {
      var _a;
      (_a = virtualScrollRef.value) == null ? void 0 : _a.scrollTo(option);
    };
    const getNode = (key) => {
      var _a;
      return (_a = mergedNodeMap.value.get(key)) == null ? void 0 : _a.rawNode;
    };
    const _getFlattedNodes = () => {
      return flattedNodes.value;
    };
    const _handleCheck = (key) => {
      const node = mergedNodeMap.value.get(key);
      if (!node) {
        return;
      }
      handleCheck(node);
    };
    expose({
      focus,
      blur,
      expandAll: expandableContext.expandAll,
      collapseAll: expandableContext.collapseAll,
      scrollTo,
      getNode,
      _getFlattedNodes,
      _handleCheck
    });
    const handleScrolledChange = (startIndex, endIndex, visibleNodes) => {
      callEmit(props.onScrolledChange, startIndex, endIndex, visibleNodes.map((item) => item.rawNode));
    };
    return () => {
      var _a;
      const nodes = flattedNodes.value;
      let children;
      if (nodes.length > 0) {
        const rowRender = ({
          item
        }) => createVNode(TreeNode, mergeProps({
          "node": item
        }, item), slots);
        const {
          height,
          virtual,
          virtualItemHeight,
          virtualScrollMode,
          onScroll,
          onScrolledBottom
        } = props;
        children = virtual ? createVNode(CdkVirtualScroll, {
          "ref": virtualScrollRef,
          "class": `${mergedPrefixCls.value}-content`,
          "dataSource": nodes,
          "getKey": "key",
          "height": autoHeight.value ? "100%" : height,
          "rowHeight": virtualItemHeight,
          "rowRender": rowRender,
          "virtual": true,
          "scrollMode": virtualScrollMode,
          "onScroll": onScroll,
          "onScrolledBottom": onScrolledBottom,
          "onScrolledChange": handleScrolledChange
        }, null) : createVNode("div", {
          "class": `${mergedPrefixCls.value}-content`,
          "style": normalizeStyle(autoHeight.value ? void 0 : {
            maxHeight: convertCssPixel(height)
          })
        }, [createVNode("div", {
          "class": `${mergedPrefixCls.value}-content-inner`
        }, [nodes.map((item) => createVNode(TreeNode, mergeProps({
          "node": item
        }, item), slots))])]);
      } else {
        children = createVNode(_Empty, {
          "empty": props.empty
        }, slots);
      }
      return createVNode("div", {
        "class": classes.value,
        "role": "tree"
      }, [focused.value && createVNode("span", {
        "style": hiddenStyle,
        "aria-live": "assertive"
      }, [accessibilityPath.value]), createVNode("input", {
        "ref": inputRef,
        "style": hiddenStyle,
        "tabindex": (_a = attrs.tabIndex) != null ? _a : 0,
        "onFocus": handleFocus,
        "onBlur": handleBlur,
        "onKeydown": handleKeydown,
        "onKeyup": handleKeyup,
        "value": "",
        "aria-label": "for screen reader"
      }, null), children]);
    };
  }
});
const IxTree = Tree;
export { IxTree, getThemeTokens as getTreeThemeTokens };
