import { computed, isProxy, normalizeClass, shallowRef, h, defineComponent, inject, ref, createVNode, mergeProps, onBeforeUnmount, watchEffect, toRaw, watch, provide } from "vue";
import { useGlobalConfig } from "@idux/components/config";
import { IxIcon } from "@idux/components/icon";
import { IxProgress } from "@idux/components/progress";
import { IxTooltip } from "@idux/components/tooltip";
import { uniqueId, callEmit, useControlledProp, useState } from "@idux/cdk/utils";
import { isString, isNumber, isNil, isArray, isUndefined, isFunction } from "lodash-es";
import { IxImageViewer } from "@idux/components/image";
import { useThemeToken } from "@idux/components/theme";
function createUploadFile(file, options = {}) {
  const key = uniqueId();
  return {
    key,
    name: file.name,
    raw: file,
    percent: 0,
    ...options
  };
}
function getTargetFile(file, files) {
  const matchKey = file.key !== void 0 ? "key" : "name";
  return files.find((item) => item[matchKey] === file[matchKey]);
}
function getTargetFileIndex(file, files) {
  const matchKey = file.key !== void 0 ? "key" : "name";
  return files.findIndex((item) => item[matchKey] === file[matchKey]);
}
function isImage(file) {
  return !!file.type && file.type.startsWith("image/");
}
function filterFilesByAccept(filesSelected, accept) {
  if (!accept || accept.length === 0) {
    return [filesSelected, []];
  }
  const isMatch = (file, type) => {
    const ext = `.${file.name.split(".").pop()}`.toLowerCase();
    const baseType = file.type.replace(/\/.*$/, "").toLowerCase();
    const _type = type.toLowerCase();
    if (_type.startsWith(".")) {
      return ext === _type;
    }
    if (/\/\*$/.test(_type)) {
      return baseType === _type.replace(/\/\*$/, "");
    }
    if (/^[^/]+\/[^/]+$/.test(_type)) {
      return file.type.toLowerCase() === _type;
    }
    return false;
  };
  const accepted = [];
  const filtered = [];
  filesSelected.forEach((file) => {
    if (accept.some((type) => isMatch(file, type))) {
      accepted.push(file);
    } else {
      filtered.push(file);
    }
  });
  return [accepted, filtered];
}
function filterFilesByMaxCount(filesSelected, curFilesCount, maxCount) {
  if (!maxCount) {
    return [filesSelected, []];
  }
  if (maxCount === 1) {
    return [filesSelected.slice(0, 1), filesSelected.slice(1)];
  }
  const remainder = maxCount - curFilesCount;
  if (remainder <= 0) {
    return [[], filesSelected];
  }
  if (remainder >= filesSelected.length) {
    return [filesSelected, []];
  }
  return [filesSelected.slice(0, remainder), filesSelected.slice(remainder)];
}
function useCmpClasses() {
  const commonPrefix = useGlobalConfig("common");
  return computed(() => `${commonPrefix.prefixCls}-upload`);
}
function useListClasses(props, type) {
  const cpmClasses = useCmpClasses();
  return computed(
    () => normalizeClass([
      `${cpmClasses.value}-list`,
      `${cpmClasses.value}-list-${type}`,
      { [`${cpmClasses.value}-list-disabled`]: props.disabled }
    ])
  );
}
function useIcon(props) {
  const uploadFilesConfig = useGlobalConfig("uploadFiles");
  return computed(() => {
    var _a;
    return (_a = props.icon) != null ? _a : uploadFilesConfig.icon;
  });
}
function useSelectorVisible(props, listType) {
  const outerSelector = computed(
    () => props.dragable || (isProxy(listType) ? listType.value !== "imageCard" : listType !== "imageCard")
  );
  const imageCardSelector = computed(() => !outerSelector.value);
  return [outerSelector, imageCardSelector];
}
function useThumb() {
  const revokeList = shallowRef([]);
  const getThumbNode = (file) => {
    if (!file.thumbUrl && file.raw && isImage(file.raw)) {
      file.thumbUrl = window.URL.createObjectURL(file.raw);
      revokeList.value.push(() => {
        window.URL.revokeObjectURL(file.thumbUrl);
        Reflect.deleteProperty(file, "thumbUrl");
      });
    }
    if (!file.thumbUrl) {
      return null;
    }
    return h("img", {
      src: file.thumbUrl,
      alt: file.name,
      style: { height: "100%", width: "100%" }
    });
  };
  const revokeAll = () => {
    revokeList.value.forEach((revokeFn) => revokeFn());
    revokeList.value = [];
  };
  return {
    revokeList,
    getThumbNode,
    revokeAll
  };
}
const uploadToken = Symbol("UploadToken");
var FileSelector = /* @__PURE__ */ defineComponent({
  name: "IxUploadSelector",
  setup(_, {
    slots
  }) {
    const {
      props: uploadProps2,
      selectFiles,
      onDrop,
      onDragOver,
      onDragLeave,
      isDraggingOver
    } = inject(uploadToken);
    const cpmClasses = useCmpClasses();
    const config = useGlobalConfig("upload");
    const dir = useDir(uploadProps2, config);
    const multiple = useMultiple(uploadProps2, config);
    const dragable = useDragable(uploadProps2, config);
    const fileInputRef = ref(null);
    const inputClasses = computed(() => `${cpmClasses.value}-input`);
    const selectorClasses = useSelectorClasses(uploadProps2, cpmClasses, dragable, isDraggingOver);
    const onClick = () => {
      if (uploadProps2.disabled || !fileInputRef.value) {
        return;
      }
      fileInputRef.value.value = "";
      fileInputRef.value.click();
    };
    const onFileChange = () => {
      var _a, _b;
      const files = Array.prototype.slice.call((_b = (_a = fileInputRef.value) == null ? void 0 : _a.files) != null ? _b : []);
      selectFiles(files);
    };
    return () => {
      var _a;
      return createVNode("div", {
        "class": selectorClasses.value,
        "onClick": onClick,
        "onDragover": onDragOver,
        "onDrop": onDrop,
        "onDragleave": onDragLeave
      }, [createVNode("input", mergeProps(dir.value, {
        "class": inputClasses.value,
        "type": "file",
        "ref": fileInputRef,
        "accept": uploadProps2.accept,
        "multiple": multiple.value,
        "onClick": (e) => e.stopPropagation(),
        "onChange": onFileChange
      }), null), (_a = slots.default) == null ? void 0 : _a.call(slots)]);
    };
  }
});
function useSelectorClasses(props, cpmClasses, dragable, dragOver) {
  return computed(() => normalizeClass({
    [`${cpmClasses.value}-selector`]: true,
    [`${cpmClasses.value}-selector-drag`]: dragable.value,
    [`${cpmClasses.value}-selector-disabled`]: props.disabled,
    [`${cpmClasses.value}-selector-dragover`]: dragOver.value
  }));
}
function useDir(props, config) {
  const directoryCfg = {
    directory: "directory",
    webkitdirectory: "webkitdirectory"
  };
  return computed(() => {
    var _a;
    return ((_a = props.directory) != null ? _a : config.directory) ? directoryCfg : {};
  });
}
function useMultiple(props, config) {
  return computed(() => {
    var _a;
    return (_a = props.multiple) != null ? _a : config.multiple;
  });
}
function useDragable(props, config) {
  return computed(() => {
    var _a;
    return (_a = props.dragable) != null ? _a : config.dragable;
  });
}
function useOperation(listProps, context) {
  const { fileList, props: uploadProps2, abort: _abort, upload: upload2, setViewerVisible, updateFiles } = context;
  const abort = (file) => {
    _abort(file);
  };
  const retry = (file) => {
    if (uploadProps2.disabled) {
      return;
    }
    upload2(file);
    callEmit(listProps.onRetry, file);
  };
  const download = (file) => {
    if (uploadProps2.disabled) {
      return;
    }
    callEmit(listProps.onDownload, file);
  };
  const preview = (file) => {
    if (uploadProps2.disabled) {
      return;
    }
    if (!listProps.onPreview && file.thumbUrl) {
      setViewerVisible(true, file.thumbUrl);
      return;
    }
    callEmit(listProps.onPreview, file);
  };
  const remove = async (file) => {
    if (uploadProps2.disabled) {
      return;
    }
    const curFile = getTargetFile(file, fileList.value);
    if (!curFile) {
      return;
    }
    const allRemove = callEmit(listProps.onRemove, curFile);
    if (allRemove === false) {
      return;
    }
    if (allRemove instanceof Promise) {
      const result = await allRemove;
      if (result === false) {
        return;
      }
    }
    if (curFile.status === "uploading") {
      abort(curFile);
    }
    const preFiles = [...fileList.value];
    preFiles.splice(getTargetFileIndex(curFile, fileList.value), 1);
    updateFiles(preFiles, true);
  };
  return {
    abort,
    retry,
    download,
    preview,
    remove
  };
}
const uploadProps = {
  files: {
    type: Array,
    default: () => []
  },
  accept: String,
  action: {
    type: [String, Function],
    default: void 0
  },
  dragable: {
    type: Boolean,
    default: void 0
  },
  directory: {
    type: Boolean,
    default: void 0
  },
  disabled: {
    type: Boolean,
    default: void 0
  },
  maxCount: Number,
  multiple: {
    type: Boolean,
    default: void 0
  },
  progress: Object,
  name: String,
  customRequest: Function,
  withCredentials: {
    type: Boolean,
    default: void 0
  },
  requestData: [Object, Function],
  requestHeaders: Object,
  requestMethod: String,
  "onUpdate:files": [Function, Array],
  onSelect: [Function, Array],
  onMaxCountExceeded: [Function, Array],
  onBeforeUpload: [Function, Array],
  onFileStatusChange: [Function, Array],
  onRequestChange: [Function, Array]
};
const uploadFilesProps = {
  type: String,
  icon: Object,
  onDownload: [Function, Array],
  onPreview: [Function, Array],
  onRemove: [Function, Array],
  onRetry: [Function, Array]
};
function getIconNode(icon) {
  if (!icon) {
    return null;
  }
  if (isString(icon)) {
    return h(IxIcon, { name: icon });
  }
  return icon;
}
function renderIcon(icon, props) {
  if (!icon) {
    return null;
  }
  return h("span", props, [getIconNode(icon)]);
}
function renderOprIcon(file, icons, cpmClasses, fileOperation, locale) {
  const previewNode = renderIcon(icons.value.preview, {
    class: `${cpmClasses.value}-icon-opr ${cpmClasses.value}-icon-preview`,
    onClick: () => fileOperation.preview(file),
    title: locale.upload.preview
  });
  const retryNode = renderIcon(icons.value.retry, {
    class: `${cpmClasses.value}-icon-opr ${cpmClasses.value}-icon-retry`,
    onClick: () => fileOperation.retry(file),
    title: locale.upload.retry
  });
  const downloadNode = renderIcon(icons.value.download, {
    class: `${cpmClasses.value}-icon-opr ${cpmClasses.value}-icon-download`,
    onClick: () => fileOperation.download(file),
    title: locale.upload.download
  });
  const removeNode = renderIcon(icons.value.remove, {
    class: `${cpmClasses.value}-icon-opr ${cpmClasses.value}-icon-remove`,
    onClick: () => fileOperation.remove(file),
    title: locale.upload.remove
  });
  return {
    previewNode,
    retryNode,
    downloadNode,
    removeNode
  };
}
function showProgress(status, percent) {
  return status === "uploading" && isNumber(percent);
}
function showErrorTip(status, errorTip) {
  return status === "error" && !!errorTip;
}
function showRetry(status) {
  return status === "error";
}
function showDownload(status) {
  return status === "success";
}
function showPreview(status) {
  return status === "success";
}
var IxUploadImageCardList = /* @__PURE__ */ defineComponent({
  name: "IxUploadImageCardList",
  props: uploadFilesProps,
  setup(listProps) {
    const uploadContext = inject(uploadToken);
    const {
      props: uploadProps2,
      locale,
      fileList
    } = uploadContext;
    const icons = useIcon(listProps);
    const cpmClasses = useCmpClasses();
    const listClasses = useListClasses(uploadProps2, "imageCard");
    const [, imageCardVisible] = useSelectorVisible(uploadProps2, "imageCard");
    const showSelector = useShowSelector$1(uploadProps2, fileList, imageCardVisible);
    const {
      getThumbNode,
      revokeAll
    } = useThumb();
    const fileOperation = useOperation(listProps, uploadContext);
    const selectorNode = renderSelector(cpmClasses);
    onBeforeUnmount(revokeAll);
    return () => createVNode("ul", {
      "class": listClasses.value
    }, [showSelector.value && selectorNode, fileList.value.map((file) => renderItem$2(uploadProps2, file, icons, cpmClasses, fileOperation, locale, getThumbNode))]);
  }
});
function renderItem$2(uploadProps2, file, icons, cpmClasses, fileOperation, locale, getThumbNode) {
  const fileClasses = normalizeClass({
    [`${cpmClasses.value}-file`]: true,
    [`${cpmClasses.value}-file-${file.status}`]: !!file.status
  });
  const uploadStatusNode = renderUploadStatus(uploadProps2, file, locale, cpmClasses);
  const thumbNode = getThumbNode(file);
  const {
    retryNode,
    downloadNode,
    removeNode,
    previewNode
  } = renderOprIcon(file, icons, cpmClasses, fileOperation, locale);
  return createVNode(IxTooltip, {
    "title": showErrorTip(file.status, file.errorTip) ? file.errorTip : ""
  }, {
    default: () => [createVNode("li", {
      "class": fileClasses
    }, [showUploadStatus(file.status) ? uploadStatusNode : thumbNode, createVNode("div", {
      "class": `${cpmClasses.value}-icon`
    }, [showPreview(file.status) && previewNode, showRetry(file.status) && retryNode, showDownload(file.status) && downloadNode, removeNode])])]
  });
}
function renderUploadStatus(uploadProps2, file, locale, cpmClasses) {
  var _a;
  const statusTitle = {
    error: locale.upload.error,
    uploading: locale.upload.uploading
  };
  const curTitle = file.status && statusTitle[file.status];
  return createVNode("div", {
    "class": `${cpmClasses.value}-status`
  }, [curTitle && createVNode("div", {
    "class": `${cpmClasses.value}-status-title`
  }, [curTitle]), showProgress(file.status, file.percent) && createVNode(IxProgress, mergeProps({
    "class": `${cpmClasses.value}-progress`,
    "strokeColor": "#20CC94",
    "percent": file.percent,
    "strokeWidth": 3,
    "hide-info": true
  }, (_a = uploadProps2.progress) != null ? _a : {}), null)]);
}
function renderSelector(cpmClasses) {
  return createVNode(FileSelector, {
    "class": `${cpmClasses.value}-selector`
  }, {
    default: () => [createVNode(IxIcon, {
      "name": "plus"
    }, null)]
  });
}
function showUploadStatus(status) {
  return status && ["uploading", "error"].includes(status);
}
function useShowSelector$1(uploadProps2, files, imageCardVisible) {
  return computed(() => {
    const countLimit = !uploadProps2.maxCount || files.value.length < uploadProps2.maxCount;
    return countLimit && imageCardVisible.value;
  });
}
var IxUploadImageList = /* @__PURE__ */ defineComponent({
  name: "IxUploadImageList",
  props: uploadFilesProps,
  setup(listProps) {
    const uploadContext = inject(uploadToken);
    const {
      props: uploadProps2,
      locale,
      fileList
    } = uploadContext;
    const icons = useIcon(listProps);
    const cpmClasses = useCmpClasses();
    const listClasses = useListClasses(uploadProps2, "image");
    const {
      getThumbNode,
      revokeAll
    } = useThumb();
    const fileOperation = useOperation(listProps, uploadContext);
    onBeforeUnmount(revokeAll);
    return () => fileList.value.length > 0 && createVNode("ul", {
      "class": listClasses.value
    }, [fileList.value.map((file) => renderItem$1(uploadProps2, file, icons, cpmClasses, fileOperation, locale, getThumbNode))]);
  }
});
function renderItem$1(uploadProps2, file, icons, cpmClasses, fileOperation, locale, getThumbNode) {
  var _a;
  const fileClasses = normalizeClass({
    [`${cpmClasses.value}-file`]: true,
    [`${cpmClasses.value}-file-${file.status}`]: !!file.status
  });
  const fileNameClasses = normalizeClass([`${cpmClasses.value}-name`, `${cpmClasses.value}-name-pointer`]);
  const errorTipNode = renderIcon("exclamation-circle", {
    class: `${cpmClasses.value}-icon-error`
  });
  const {
    retryNode,
    downloadNode,
    removeNode
  } = renderOprIcon(file, icons, cpmClasses, fileOperation, locale);
  return createVNode("li", {
    "class": fileClasses
  }, [createVNode("div", {
    "class": `${cpmClasses.value}-thumb-info`
  }, [createVNode("span", {
    "class": `${cpmClasses.value}-thumb`
  }, [getThumbNode(file)]), createVNode("span", {
    "class": fileNameClasses,
    "onClick": () => showPreview(file.status) && fileOperation.preview(file),
    "title": file.name
  }, [file.name])]), createVNode("div", {
    "class": `${cpmClasses.value}-icon-wrap`
  }, [showErrorTip(file.status, file.errorTip) && createVNode(IxTooltip, {
    "title": file.errorTip
  }, {
    default: () => [errorTipNode]
  }), showRetry(file.status) && retryNode, showDownload(file.status) && downloadNode, removeNode]), showProgress(file.status, file.percent) && createVNode(IxProgress, mergeProps({
    "class": `${cpmClasses.value}-progress`,
    "percent": file.percent,
    "strokeColor": "#20CC94",
    "strokeWidth": 2,
    "hide-info": true
  }, (_a = uploadProps2.progress) != null ? _a : {}), null)]);
}
var IxUploadTextList = /* @__PURE__ */ defineComponent({
  name: "IxUploadTextList",
  props: uploadFilesProps,
  setup(listProps) {
    const uploadContext = inject(uploadToken);
    const {
      props: uploadProps2,
      locale,
      fileList
    } = uploadContext;
    const icons = useIcon(listProps);
    const cpmClasses = useCmpClasses();
    const listClasses = useListClasses(uploadProps2, "text");
    const fileOperation = useOperation(listProps, uploadContext);
    return () => fileList.value.length > 0 && createVNode("ul", {
      "class": listClasses.value
    }, [fileList.value.map((file) => renderItem(uploadProps2, file, icons, cpmClasses, fileOperation, locale))]);
  }
});
function renderItem(uploadProps2, file, icons, cpmClasses, fileOperation, locale) {
  var _a;
  const fileClasses = normalizeClass({
    [`${cpmClasses.value}-file`]: true,
    [`${cpmClasses.value}-file-${file.status}`]: !!file.status
  });
  const fileNameClasses = normalizeClass([`${cpmClasses.value}-name`, `${cpmClasses.value}-name-pointer`]);
  const errorTipNode = renderIcon("exclamation-circle", {
    class: `${cpmClasses.value}-icon-error`
  });
  const {
    retryNode,
    downloadNode,
    removeNode
  } = renderOprIcon(file, icons, cpmClasses, fileOperation, locale);
  return createVNode("li", {
    "class": fileClasses
  }, [createVNode("span", {
    "class": `${cpmClasses.value}-text-info`
  }, [renderIcon(icons.value.file, {
    class: `${cpmClasses.value}-icon-file`
  }), createVNode("span", {
    "class": fileNameClasses,
    "onClick": () => showPreview(file.status) && fileOperation.preview(file),
    "title": file.name
  }, [file.name])]), createVNode("span", {
    "class": `${cpmClasses.value}-icon-wrap`
  }, [showErrorTip(file.status, file.errorTip) && createVNode(IxTooltip, {
    "title": file.errorTip
  }, {
    default: () => [errorTipNode]
  }), showRetry(file.status) && retryNode, showDownload(file.status) && downloadNode, removeNode]), showProgress(file.status, file.percent) && createVNode(IxProgress, mergeProps({
    "class": `${cpmClasses.value}-progress`,
    "percent": file.percent,
    "strokeColor": "#20CC94",
    "strokeWidth": 2,
    "hide-info": true
  }, (_a = uploadProps2.progress) != null ? _a : {}), null)]);
}
const cpmMap = {
  text: IxUploadTextList,
  image: IxUploadImageList,
  imageCard: IxUploadImageCardList
};
var UploadFiles = /* @__PURE__ */ defineComponent({
  name: "IxUploadFiles",
  props: uploadFilesProps,
  setup(props) {
    const config = useGlobalConfig("uploadFiles");
    const listType = useListType(props, config);
    const {
      props: uploadProps2,
      setSelectorVisible
    } = inject(uploadToken);
    const [outerSelector] = useSelectorVisible(uploadProps2, listType);
    watchEffect(() => setSelectorVisible(outerSelector.value));
    return () => h(cpmMap[listType.value], {
      ...props
    });
  }
});
function useListType(props, config) {
  return computed(() => {
    var _a;
    return (_a = props.type) != null ? _a : config.type;
  });
}
function useDrag(props, selectFiles) {
  const isDraggingOver = ref(false);
  const allowDrag = computed(() => !!props.dragable && !props.disabled);
  function onDrop(e) {
    var _a, _b;
    e.preventDefault();
    if (!allowDrag.value) {
      return;
    }
    isDraggingOver.value = false;
    selectFiles(Array.prototype.slice.call((_b = (_a = e.dataTransfer) == null ? void 0 : _a.files) != null ? _b : []));
  }
  function onDragOver(e) {
    e.preventDefault();
    if (!allowDrag.value) {
      return;
    }
    isDraggingOver.value = true;
  }
  function onDragLeave(e) {
    e.preventDefault();
    if (!allowDrag.value) {
      return;
    }
    isDraggingOver.value = false;
  }
  return {
    allowDrag,
    isDraggingOver,
    onDrop,
    onDragOver,
    onDragLeave
  };
}
function useFileSelect(props, filesDataContext) {
  const { fileList, updateFiles } = filesDataContext;
  const accept = computed(
    () => {
      var _a;
      return (_a = props.accept) == null ? void 0 : _a.split(",").map((type) => type.trim()).filter(Boolean);
    }
  );
  const maxCount = computed(() => {
    var _a;
    return (_a = props.maxCount) != null ? _a : 0;
  });
  const filterSelectFiles = (files) => {
    const [acceptedFiles, acceptFilteredFiles] = filterFilesByAccept(files, accept.value);
    const [countAllowdFiles, countFilteredFiles] = filterFilesByMaxCount(
      acceptedFiles,
      fileList.value.length,
      maxCount.value
    );
    const filteredFiles = [
      ...acceptFilteredFiles.map((file) => ({ file, reason: "acceptNotMatch" })),
      ...countFilteredFiles.map((file) => ({ file, reason: "maxCountExceeded" }))
    ];
    return [countAllowdFiles, filteredFiles];
  };
  const selecFiles = async (files) => {
    const [allowdFiles, filteredFiles] = filterSelectFiles(files);
    const onSelectResult = props.onSelect && await callEmit(props.onSelect, allowdFiles, filteredFiles);
    const resolvedAllowedFiles = isNil(onSelectResult) || onSelectResult === true ? allowdFiles : onSelectResult === false ? [] : isArray(onSelectResult) ? onSelectResult : [];
    const uploadFiles = resolvedAllowedFiles.map((file) => createUploadFile(file, { status: "selected" }));
    if (filteredFiles.filter((file) => file.reason === "maxCountExceeded").length > 0 || props.maxCount === 1 && fileList.value.length > 0) {
      callEmit(props.onMaxCountExceeded);
    }
    if (props.maxCount === 1) {
      updateFiles(uploadFiles, true);
    } else {
      updateFiles(uploadFiles);
    }
  };
  return selecFiles;
}
function useFilesData(props) {
  const [fileList, setFileList] = useControlledProp(props, "files");
  const updateFiles = (files, replace = false) => {
    const newFileList = replace ? files : [...fileList.value];
    const statusChangeFiles = [];
    files.forEach((file) => {
      const index = getTargetFileIndex(file, fileList.value);
      const oldFile = fileList.value[index];
      if (index > -1 && !oldFile) {
        return;
      }
      if ((oldFile == null ? void 0 : oldFile.status) !== file.status) {
        statusChangeFiles.push(file);
      }
      if (replace) {
        return;
      }
      if (index === -1) {
        newFileList.push(file);
      } else {
        newFileList.splice(index, 1, file);
      }
    });
    setFileList(toRaw(newFileList));
    statusChangeFiles.forEach((file) => {
      callEmit(props.onFileStatusChange, file);
    });
  };
  return {
    fileList,
    updateFiles
  };
}
function getErrorMsg(option, xhr) {
  var _a;
  const errorMap = (/* @__PURE__ */ new Map([])).set((option2, xhr2) => !!xhr2.response, `${((_a = xhr.response) == null ? void 0 : _a.error) || xhr.response}`).set((option2, xhr2) => !!xhr2.responseText, `${xhr.responseText}`).set((option2) => !option2.action, "action not found.").set(() => true, `fail to ${option.requestMethod} ${option.action} ${xhr.status}`);
  for (const [testFn, errorMsg] of errorMap) {
    if (testFn(option, xhr)) {
      return errorMsg;
    }
  }
  return "";
}
function getError(option, xhr) {
  const msg = getErrorMsg(option, xhr);
  const err = new Error(msg);
  err.status = xhr.status;
  err.method = option.requestMethod;
  err.url = option.action;
  return err;
}
function getBody(xhr) {
  const text = xhr.responseText || xhr.response;
  if (!text) {
    return text;
  }
  try {
    return JSON.parse(text);
  } catch (e) {
    return text;
  }
}
function upload(option) {
  var _a;
  const xhr = new XMLHttpRequest();
  if (option.onProgress && xhr.upload) {
    xhr.upload.onprogress = function progress(e) {
      if (e.total > 0) {
        e.percent = e.loaded / e.total * 100;
      }
      option.onProgress(e);
    };
  }
  const formData = new FormData();
  if (option.requestData) {
    Object.keys(option.requestData).forEach((key) => {
      const value = option.requestData[key];
      if (Array.isArray(value)) {
        value.forEach((item) => {
          formData.append(`${key}[]`, item);
        });
        return;
      }
      formData.append(key, value);
    });
  }
  formData.append(option.name, option.file, option.file.name);
  xhr.onerror = function error() {
    var _a2;
    (_a2 = option.onError) == null ? void 0 : _a2.call(option, getError(option, xhr));
  };
  xhr.onload = function onload() {
    var _a2, _b;
    if (xhr.status < 200 || xhr.status >= 300) {
      return (_a2 = option.onError) == null ? void 0 : _a2.call(option, getError(option, xhr));
    }
    return (_b = option.onSuccess) == null ? void 0 : _b.call(option, getBody(xhr));
  };
  if (!option.action) {
    (_a = option.onError) == null ? void 0 : _a.call(option, getError(option, xhr));
    return null;
  }
  xhr.open(option.requestMethod, option.action, true);
  if (option.withCredentials && "withCredentials" in xhr) {
    xhr.withCredentials = true;
  }
  const headers = option.requestHeaders || {};
  if (headers["X-Requested-With"] !== null) {
    xhr.setRequestHeader("X-Requested-With", "XMLHttpRequest");
  }
  Object.keys(headers).forEach((h2) => {
    if (headers[h2] !== null) {
      xhr.setRequestHeader(h2, headers[h2]);
    }
  });
  xhr.send(formData);
  return {
    abort() {
      xhr.abort();
    }
  };
}
function useRequest(props, filesDataContext) {
  const fileUploading = ref([]);
  const aborts = /* @__PURE__ */ new Map([]);
  const config = useGlobalConfig("upload");
  const { fileList, updateFiles } = filesDataContext;
  const updateFileStatus = (file, status) => {
    updateFiles([{ ...file, status }]);
  };
  function abort(file) {
    var _a;
    (_a = aborts.get(file.key)) == null ? void 0 : _a();
    props.onRequestChange && callEmit(props.onRequestChange, {
      status: "abort",
      file: { ...file }
    });
    fileUploading.value.splice(getTargetFileIndex(file, fileUploading.value), 1);
    aborts.delete(file.key);
    const curFile = getTargetFile(file, fileList.value);
    if (curFile) {
      updateFileStatus(curFile, "abort");
    }
  }
  async function startUpload(file) {
    if (isUndefined(props.onBeforeUpload)) {
      await upload$1(file);
      return;
    }
    const before = callEmit(props.onBeforeUpload, file);
    if (before instanceof Promise) {
      try {
        const result = await before;
        if (result === true) {
          await upload$1(file);
          return;
        }
        if (typeof result === "object" && result) {
          await upload$1(result);
        }
      } catch (e) {
        updateFileStatus(file, "error");
      }
    } else if (before === true) {
      await upload$1(file);
    } else if (typeof before === "object") {
      await upload$1(before);
    } else {
      updateFileStatus(file, "cancel");
    }
  }
  async function upload$1(file) {
    var _a, _b, _c, _d, _e, _f, _g;
    if (!file.raw) {
      file.error = new Error("file error");
      updateFileStatus(file, "error");
    }
    const action = await getAction(props, file);
    const requestData = await getRequestData(props, file);
    const requestOption = {
      file: file.raw,
      name: (_a = props.name) != null ? _a : config.name,
      withCredentials: (_b = props.withCredentials) != null ? _b : config.withCredentials,
      action,
      requestHeaders: (_c = props.requestHeaders) != null ? _c : {},
      requestMethod: (_d = props.requestMethod) != null ? _d : config.requestMethod,
      requestData,
      onProgress: (event) => _onProgress(event, file),
      onError: (error) => _onError(error, file),
      onSuccess: (res) => _onSuccess(res, file)
    };
    const uploadHttpRequest = (_f = (_e = props.customRequest) != null ? _e : config.customRequest) != null ? _f : upload;
    props.onRequestChange && callEmit(props.onRequestChange, {
      status: "loadstart",
      file: { ...file }
    });
    updateFileStatus(file, "uploading");
    file.percent = 0;
    const requestHandler = await uploadHttpRequest(requestOption);
    aborts.set(file.key, (_g = requestHandler == null ? void 0 : requestHandler.abort) != null ? _g : () => {
    });
    fileUploading.value.push(file);
  }
  function _onProgress(event, file) {
    var _a;
    const curFile = getTargetFile(file, fileList.value);
    if (!curFile) {
      return;
    }
    curFile.percent = (_a = event.percent) != null ? _a : 0;
    props.onRequestChange && callEmit(props.onRequestChange, {
      status: "progress",
      file: { ...curFile },
      event
    });
  }
  function _onError(error, file) {
    const curFile = getTargetFile(file, fileList.value);
    if (!curFile) {
      return;
    }
    fileUploading.value.splice(getTargetFileIndex(curFile, fileUploading.value), 1);
    curFile.error = error;
    props.onRequestChange && callEmit(props.onRequestChange, {
      file: { ...curFile },
      status: "error",
      error
    });
    updateFileStatus(curFile, "error");
  }
  function _onSuccess(res, file) {
    const curFile = getTargetFile(file, fileList.value);
    if (!curFile) {
      return;
    }
    curFile.response = res;
    props.onRequestChange && callEmit(props.onRequestChange, {
      status: "loadend",
      file: { ...curFile },
      response: res
    });
    updateFileStatus(curFile, "success");
    fileUploading.value.splice(getTargetFileIndex(curFile, fileUploading.value), 1);
  }
  return {
    fileUploading,
    abort,
    startUpload,
    upload: upload$1
  };
}
async function getAction(props, file) {
  return isFunction(props.action) ? await props.action(file) : props.action;
}
async function getRequestData(props, file) {
  var _a;
  return isFunction(props.requestData) ? await props.requestData(file) : (_a = props.requestData) != null ? _a : {};
}
function useUploadControl(fileList, uploadRequest) {
  const { startUpload, abort } = uploadRequest;
  const [proxyedFileList, setProxyedFileList] = useState([...fileList.value]);
  watch(
    fileList,
    (_fileList) => {
      setProxyedFileList([..._fileList]);
    },
    {
      deep: true
    }
  );
  watch(
    [proxyedFileList, () => proxyedFileList.value.length],
    ([currentFileList, currentLength], [preFileList, preLength]) => {
      if (currentLength === preLength && currentLength !== 1 && preLength !== 1) {
        return;
      }
      currentFileList.forEach((file) => {
        const preFile = getTargetFile(file, preFileList);
        if (!preFile && file.status === "selected") {
          startUpload(file);
        }
      });
      preFileList.forEach((file) => {
        const currentFile = getTargetFile(file, currentFileList);
        if (!currentFile && file.status === "uploading") {
          abort(file);
        }
      });
    }
  );
}
function getDefaultThemeTokens(tokens) {
  const { controlLineWidth, colorBorder, colorPrimaryBorder } = tokens;
  return {
    selectorDragableBorder: `${controlLineWidth}px dashed ${colorBorder}`,
    selectorDragableBorderDragover: `${controlLineWidth}px dashed ${colorPrimaryBorder}`,
    listNameMaxWidth: `calc(100% - 74px)`,
    listImgThumbWidth: 48,
    listImgThumbHeight: 48,
    listImgCardHeight: 96,
    listImgCardWidth: 96
  };
}
const getThemeTokens = (tokens, presetTheme) => {
  return presetTheme === "default" ? getDefaultThemeTokens(tokens) : getDefaultThemeTokens(tokens);
};
var Upload = /* @__PURE__ */ defineComponent({
  name: "IxUpload",
  props: uploadProps,
  setup(props, {
    slots
  }) {
    const {
      globalHashId,
      hashId,
      registerToken
    } = useThemeToken("upload");
    registerToken(getThemeTokens);
    const locale = useGlobalConfig("locale");
    const cpmClasses = useCmpClasses();
    const [showSelector, setSelectorVisible] = useShowSelector();
    const filesDataContext = useFilesData(props);
    const selectFiles = useFileSelect(props, filesDataContext);
    const dragContext = useDrag(props, selectFiles);
    const uploadRequest = useRequest(props, filesDataContext);
    const {
      abort,
      upload: upload2
    } = uploadRequest;
    const {
      viewerVisible,
      images,
      setViewerVisible
    } = useImageViewer();
    useUploadControl(filesDataContext.fileList, uploadRequest);
    provide(uploadToken, {
      props,
      locale,
      selectFiles,
      setViewerVisible,
      setSelectorVisible,
      ...uploadRequest,
      ...filesDataContext,
      ...dragContext
    });
    return () => {
      var _a, _b;
      return createVNode("div", {
        "class": [cpmClasses.value, globalHashId.value, hashId.value]
      }, [showSelector.value && createVNode(FileSelector, null, {
        default: () => {
          var _a2;
          return [(_a2 = slots.default) == null ? void 0 : _a2.call(slots)];
        }
      }), (_a = slots.list) == null ? void 0 : _a.call(slots, {
        abort,
        upload: upload2
      }), createVNode(IxImageViewer, {
        "visible": viewerVisible.value,
        "onUpdate:visible": ($event) => viewerVisible.value = $event,
        "images": images.value
      }, null), createVNode("div", {
        "class": `${cpmClasses.value}-tip`
      }, [(_b = slots.tip) == null ? void 0 : _b.call(slots)])]);
    };
  }
});
function useShowSelector() {
  const showSelector = ref(true);
  function setSelectorVisible(isShow) {
    showSelector.value = isShow;
  }
  return [showSelector, setSelectorVisible];
}
function useImageViewer() {
  const viewerVisible = ref(false);
  const images = shallowRef([]);
  function setViewerVisible(visible, imageSrc) {
    images.value = visible && imageSrc ? [imageSrc] : [];
    viewerVisible.value = visible;
  }
  return {
    viewerVisible,
    images,
    setViewerVisible
  };
}
const IxUpload = Upload;
const IxUploadFiles = UploadFiles;
export { IxUpload, IxUploadFiles, getThemeTokens as getUploadThemeTokens };
