import { isFunction, isString, isArray, isObject, isNil, isBoolean } from "lodash-es";
import { Logger, convertArray, useState, useKey as useKey$1, traverseTree, getTreeKeys, mergeTree, filterTree, tryOnScopeDispose } from "@idux/cdk/utils";
import { createVNode, computed, shallowRef, watch, ref } from "vue";
import { IxIcon } from "@idux/components/icon";
import { useResizeObserver } from "@idux/cdk/resize";
import { getTreeCheckStateResolverContext as getTreeCheckStateResolverContext$1, useTreeCheckStateResolver as useTreeCheckStateResolver$1 } from "@idux/components/utils";
import { useSharedFocusMonitor } from "@idux/cdk/a11y";
import { useFormSize } from "@idux/components/form";
const presetColors = [
  "red",
  "orange",
  "brown",
  "yellow",
  "canary",
  "bud",
  "green",
  "turquoise",
  "cyan",
  "glacier",
  "blue",
  "indigo",
  "purple",
  "magenta"
];
function isPresetColor(color) {
  return presetColors.includes(color);
}
const statusColors = ["primary", "info", "pending", "success", "warning", "error"];
function isStatusColor(color) {
  return statusColors.includes(color);
}
function convertTarget(target) {
  const temp = isFunction(target) ? target() : target;
  if (isString(temp)) {
    const targetDom = document.querySelector(temp);
    if (targetDom) {
      return targetDom;
    } else {
      process.env.NODE_ENV !== "production" && Logger.warn("components/utils", `target does not exist: ${target}, default value are already used: window.`);
      return window;
    }
  }
  return temp || window;
}
function convertIconVNode(slots, props, keyOrParams, slotParams) {
  let iconSlot;
  let iconName;
  const isKey = isString(keyOrParams);
  const params = isKey ? slotParams : keyOrParams;
  if (isKey) {
    iconSlot = slots[keyOrParams];
    if (!iconSlot) {
      iconName = props[keyOrParams];
    }
  } else {
    iconSlot = slots;
    iconName = props;
  }
  if (iconSlot) {
    return iconSlot(params);
  }
  return isString(iconName) && iconName !== "" ? createVNode(IxIcon, { name: iconName }, null) : isFunction(iconName) ? iconName(params) : iconName;
}
function convertStringVNode(slots, props, keyOrParams, slotParams) {
  let labelSlot;
  let label;
  const isKey = isString(keyOrParams);
  const params = isKey ? slotParams : keyOrParams;
  if (isKey) {
    labelSlot = slots[keyOrParams];
    if (!labelSlot) {
      label = props[keyOrParams];
    }
  } else {
    labelSlot = slots;
    label = props;
  }
  return labelSlot ? labelSlot(params) : isFunction(label) ? label(params) : label;
}
const typeApplySequence = [
  "year",
  "month",
  "date",
  "hour",
  "minute",
  "second",
  "millisecond"
];
function convertToDate(dateConfig, value, format) {
  if (!value) {
    return void 0;
  }
  return dateConfig.convert(value, format);
}
function applyDateTime(dateConfig, sourceDate, targetDate, types) {
  const { set, get, endOf } = dateConfig;
  const typesArray = convertArray(types);
  return typesArray.reduce((date, type) => {
    const _type = type === "week" || type === "day" ? "date" : type;
    const typeIndex = typeApplySequence.indexOf(_type);
    const value = dateConfig.get(sourceDate, _type);
    if (typeIndex <= 0) {
      return dateConfig.set(date, value, _type);
    }
    const maxValue = get(endOf(targetDate, typeApplySequence[typeIndex - 1]), _type);
    return set(date, Math.min(maxValue, value), _type);
  }, targetDate);
}
function isSameDateTime(dateConfig, sourceDate, targetDate, types) {
  const typesArray = convertArray(types);
  return typesArray.every((type) => dateConfig.get(sourceDate, type) === dateConfig.get(targetDate, type));
}
function compareDateTime(dateConfig, v1, v2, type = "time") {
  if (!v1) {
    return 1;
  }
  if (!v2) {
    return 0;
  }
  if (type === "time") {
    return v1.valueOf() - v2.valueOf();
  }
  return dateConfig.startOf(v1, type).valueOf() - dateConfig.startOf(v2, type).valueOf();
}
function sortRangeValue(dateConfig, values, type = "time") {
  return values.sort((v1, v2) => compareDateTime(dateConfig, v1, v2, type));
}
function getTypeForSequenceSearch(type) {
  switch (type) {
    case "week":
      return "date";
    case "day":
      return "date";
    case "quarter":
      return "month";
    default:
      return type;
  }
}
function applyDateOfPropValue(dateConfig, value, propValue, type) {
  const typeForSequenceSearch = getTypeForSequenceSearch(type);
  const typesToApplay = typeApplySequence.slice(typeApplySequence.indexOf(typeForSequenceSearch) + 1);
  return applyDateTime(dateConfig, propValue, value, typesToApplay);
}
function adjustRangeBoundary(dateConfig, value, propValue, type, isStart) {
  if (!value) {
    return;
  }
  if (propValue) {
    return applyDateOfPropValue(dateConfig, value, propValue, type);
  }
  return isStart ? dateConfig.startOf(value, type) : dateConfig.endOf(value, type);
}
function adjustRangeValue(dateConfig, values, propValues, type) {
  if (!values) {
    return;
  }
  const [start, end] = values;
  return [
    adjustRangeBoundary(dateConfig, start, propValues == null ? void 0 : propValues[0], type, true),
    adjustRangeBoundary(dateConfig, end, propValues == null ? void 0 : propValues[1], type, false)
  ];
}
function usePortalTarget(props, config, common, mergedPrefix) {
  return computed(() => {
    var _a, _b, _c;
    const container = (_b = (_a = props.container) != null ? _a : config.container) != null ? _b : common.overlayContainer;
    return (_c = isFunction(container) ? container() : container) != null ? _c : `.${mergedPrefix.value}-container`;
  });
}
function useGetDisabled(props) {
  return computed(() => {
    const { disableData } = props;
    return (data) => {
      return data.disabled || (disableData ? disableData(data) : false);
    };
  });
}
function useElementSize(measureSpanRef) {
  const [size, setSize] = useState({ width: 0, height: 0 });
  useResizeObserver(measureSpanRef, ({ contentRect }) => {
    setSize({ width: contentRect.width, height: contentRect.height });
  });
  return size;
}
function useKey() {
  process.env.NODE_ENV !== "production" && Logger.warn("components/utils", "please use `import { useKey } from '@idux/cdk/utils'` instead");
  return useKey$1();
}
function useGetKey(props, config, location) {
  return computed(() => {
    const { getKey = config.getKey } = props;
    if (isString(getKey)) {
      return (data) => {
        const key = data[getKey];
        if (process.env.NODE_ENV !== "production" && key === void 0) {
          Logger.warn(location, "Each item in dataSource should have a unique `key` prop.");
        }
        return key;
      };
    }
    return getKey;
  });
}
function useTreeCheckStateResolver(dataOrContext, childrenKey, getKey, cascadeStrategy) {
  const _getContext = (dataOrContext2) => {
    if (!isArray(dataOrContext2)) {
      return dataOrContext2;
    }
    return getTreeCheckStateResolverContext(dataOrContext2, childrenKey.value, getKey.value);
  };
  const mergedCascadeStrategy = computed(() => {
    var _a;
    return (_a = cascadeStrategy == null ? void 0 : cascadeStrategy.value) != null ? _a : "all";
  });
  const _context = computed(() => _getContext(dataOrContext.value));
  const _getParents = (key, resolverContext) => {
    const { parentKeyMap, dataMap } = resolverContext;
    const parents = [];
    let currentKey = key;
    while (parentKeyMap.has(currentKey)) {
      const parentKey = parentKeyMap.get(currentKey);
      const parent = dataMap.get(parentKey);
      parent && parents.push(parent);
      currentKey = parentKey;
    }
    return parents;
  };
  const _getAllData = () => {
    const { data, parentKeyMap, dataMap } = _context.value;
    if (data) {
      return data;
    }
    const _data = [];
    for (const key of parentKeyMap.keys()) {
      if (!parentKeyMap.has(key) && dataMap.has(key)) {
        _data.push(dataMap.get(key));
      }
    }
    return _data;
  };
  const _append = (checkedKeys, appendedKeys, resolverContext) => {
    if (!appendedKeys.length) {
      return Array.from(checkedKeys);
    }
    if (mergedCascadeStrategy.value === "off") {
      return Array.from(/* @__PURE__ */ new Set([...checkedKeys, ...appendedKeys]));
    }
    const { dataMap } = resolverContext;
    const newKeySet = new Set(checkedKeys);
    const topMostCheckedKeys = /* @__PURE__ */ new Set();
    appendedKeys.forEach((key) => {
      if (!dataMap.has(key) || newKeySet.has(key)) {
        return;
      }
      const treeData = dataMap.get(key);
      const children = treeData[childrenKey.value];
      newKeySet.add(key);
      topMostCheckedKeys.add(key);
      if (children == null ? void 0 : children.length) {
        getTreeKeys(children, childrenKey.value, getKey.value, mergedCascadeStrategy.value === "child").forEach((key2) => {
          newKeySet.add(key2);
          topMostCheckedKeys.delete(key2);
        });
      }
    });
    if (mergedCascadeStrategy.value === "child") {
      return Array.from(newKeySet).filter((key) => {
        var _a, _b;
        return !((_b = (_a = dataMap.get(key)) == null ? void 0 : _a[childrenKey.value]) == null ? void 0 : _b.length);
      });
    }
    topMostCheckedKeys.forEach((key) => {
      _getParents(key, resolverContext).forEach((parent) => {
        var _a;
        if ((_a = parent[childrenKey.value]) == null ? void 0 : _a.every((child) => newKeySet.has(getKey.value(child)))) {
          newKeySet.add(getKey.value(parent));
        }
      });
    });
    if (mergedCascadeStrategy.value === "all") {
      return Array.from(newKeySet);
    }
    newKeySet.forEach((key) => {
      var _a;
      if (!newKeySet.has(key)) {
        return;
      }
      const children = (_a = dataMap.get(key)) == null ? void 0 : _a[childrenKey.value];
      if (children) {
        traverseTree(children, childrenKey.value, (item) => {
          newKeySet.delete(getKey.value(item));
        });
      }
    });
    return Array.from(newKeySet);
  };
  const _remove = (checkedKeys, removedKeys, resolverContext) => {
    if (!removedKeys.length) {
      return Array.from(checkedKeys);
    }
    const newKeySet = new Set(checkedKeys);
    if (mergedCascadeStrategy.value === "off") {
      removedKeys.forEach((key) => {
        newKeySet.delete(key);
      });
      return Array.from(newKeySet);
    }
    const { dataMap } = resolverContext;
    const deletedKeys = /* @__PURE__ */ new Set();
    const deleteKey = (key) => {
      deletedKeys.add(key);
      newKeySet.delete(key);
    };
    removedKeys.forEach((key) => {
      if (!dataMap.has(key)) {
        return;
      }
      getTreeKeys([dataMap.get(key)], childrenKey.value, getKey.value).forEach((key2) => {
        deleteKey(key2);
      });
      const parents = _getParents(key, resolverContext);
      if (mergedCascadeStrategy.value === "parent") {
        const keysInChain = [key, ...parents.map(getKey.value)];
        const selectedKeyIdx = keysInChain.findIndex((key2) => newKeySet.has(key2));
        if (selectedKeyIdx > -1) {
          parents.slice(0, selectedKeyIdx).forEach((parent) => {
            if (!parent[childrenKey.value]) {
              return;
            }
            parent[childrenKey.value].forEach((child) => {
              const childKey = getKey.value(child);
              if (!deletedKeys.has(childKey)) {
                newKeySet.add(childKey);
              }
            });
            deleteKey(getKey.value(parent));
          });
        }
      } else {
        parents.forEach((parent) => {
          deleteKey(getKey.value(parent));
        });
      }
    });
    return Array.from(newKeySet);
  };
  const appendKeys = (checkedKeys, appendedKeys) => _append(checkedKeys, appendedKeys, _context.value);
  const removeKeys = (checkedKeys, removedKeys) => _remove(checkedKeys, removedKeys, _context.value);
  const _getAllCheckedKeys = (data, defaultUnCheckedKeys) => {
    const _data = data != null ? data : _getAllData();
    const tempKeys = mergedCascadeStrategy.value === "parent" ? _data.map(getKey.value) : new Set(getTreeKeys(_data, childrenKey.value, getKey.value, mergedCascadeStrategy.value === "child"));
    return _remove(tempKeys, defaultUnCheckedKeys, data ? _getContext(data) : _context.value);
  };
  const _getAllUncheckedKeys = (data, defaultCheckedKeys) => {
    return _append([], defaultCheckedKeys, data ? _getContext(data) : _context.value);
  };
  const getAllCheckedKeys = (data, defaultUnCheckedKeys) => {
    const dataProvided = ((data2) => isObject(data2 == null ? void 0 : data2[0]))(data);
    return _getAllCheckedKeys(
      dataProvided ? data : void 0,
      defaultUnCheckedKeys != null ? defaultUnCheckedKeys : dataProvided ? [] : data != null ? data : []
    );
  };
  const getAllUncheckedKeys = (data, defaultCheckedKeys) => {
    const dataProvided = ((data2) => isObject(data2 == null ? void 0 : data2[0]))(data);
    return _getAllUncheckedKeys(
      dataProvided ? data : void 0,
      defaultCheckedKeys != null ? defaultCheckedKeys : dataProvided ? [] : data != null ? data : []
    );
  };
  return {
    appendKeys,
    removeKeys,
    getAllCheckedKeys,
    getAllUncheckedKeys
  };
}
function getTreeCheckStateResolverContext(data, childrenKey, getKey) {
  const dataMap = /* @__PURE__ */ new Map();
  const parentKeyMap = /* @__PURE__ */ new Map();
  const depthMap = /* @__PURE__ */ new Map();
  traverseTree(data, childrenKey, (item, parents) => {
    const key = getKey(item);
    const parent = parents[0];
    dataMap.set(key, item);
    depthMap.set(key, parents.length);
    if (parent) {
      parentKeyMap.set(key, getKey(parent));
    }
  });
  return {
    data,
    dataMap,
    parentKeyMap,
    depthMap
  };
}
function useTreeCheckState(checkedKeys, resolverContext, childrenKey, getKey, cascaderStrategy, isDisabled) {
  const {
    mergedResolverContext,
    cachedSelectedData,
    checkStateResolver,
    allCheckStateResolver,
    allCheckedKeySet,
    unexistedKeys
  } = useResolvers(checkedKeys, resolverContext, childrenKey, getKey, cascaderStrategy);
  const checkDisabledKeySet = computed(() => {
    const { dataMap } = mergedResolverContext.value;
    const disabledKeys = /* @__PURE__ */ new Set();
    dataMap.forEach((data, key) => {
      var _a;
      if ((_a = isDisabled.value) == null ? void 0 : _a.call(isDisabled, data)) {
        disabledKeys.add(key);
      }
    });
    return disabledKeys;
  });
  const indeterminateKeySet = computed(() => {
    const { parentKeyMap } = mergedResolverContext.value;
    const _checkedKeySet = allCheckedKeySet.value;
    if (_checkedKeySet.size === 0 || cascaderStrategy.value === "off") {
      return /* @__PURE__ */ new Set();
    }
    const keySet = /* @__PURE__ */ new Set();
    _checkedKeySet.forEach((key) => {
      let parentKey = parentKeyMap.get(key);
      if (!isNil(parentKey) && !_checkedKeySet.has(parentKey)) {
        while (!isNil(parentKey)) {
          keySet.add(parentKey);
          parentKey = parentKeyMap.get(parentKey);
        }
      }
    });
    return keySet;
  });
  const isCheckDisabled = (key) => {
    return checkDisabledKeySet.value.has(key);
  };
  const isChecked = (key) => {
    return allCheckedKeySet.value.has(key);
  };
  const isIndeterminate = (key) => {
    return indeterminateKeySet.value.has(key);
  };
  const appendUnexistedDataKeys = (newCheckedKeys) => {
    return unexistedKeys.value ? [...newCheckedKeys, ...unexistedKeys.value] : newCheckedKeys;
  };
  const toggle = (data) => {
    var _a;
    const currKey = getKey.value(data);
    if (isCheckDisabled(currKey)) {
      return {
        checked: isChecked(currKey),
        checkedKeys: checkedKeys.value
      };
    }
    const children = cascaderStrategy.value !== "off" ? (_a = data[childrenKey.value]) != null ? _a : [] : [];
    const checked = isChecked(currKey) || !!children.length && children.every((child) => {
      const childKey = getKey.value(child);
      return isChecked(childKey) || isIndeterminate(childKey) || isCheckDisabled(childKey);
    });
    const _checkedKeys = checkStateResolver.appendKeys([], checkedKeys.value);
    const newCheckedKeys = checkStateResolver[checked ? "removeKeys" : "appendKeys"](_checkedKeys, [currKey]);
    const disabledKeys = checked ? [...allCheckedKeySet.value].filter((key) => isCheckDisabled(key)) : (cascaderStrategy.value === "parent" ? allCheckStateResolver.appendKeys([], newCheckedKeys) : newCheckedKeys).filter((key) => isCheckDisabled(key) && !allCheckedKeySet.value.has(key));
    const resolvedCheckedKeys = appendUnexistedDataKeys(
      checkStateResolver[checked ? "appendKeys" : "removeKeys"](newCheckedKeys, disabledKeys)
    );
    return {
      checked: !checked,
      checkedKeys: resolvedCheckedKeys
    };
  };
  const resolveAllCheckedFnParams = (data, defaultKeysOrCached, cached) => {
    var _a;
    const dataProvided = ((data2) => isArray(data2) && isObject(data2 == null ? void 0 : data2[0]))(data);
    const _defaultKeys = (_a = dataProvided ? isBoolean(defaultKeysOrCached) ? [] : defaultKeysOrCached : data) != null ? _a : [];
    const _cached = dataProvided ? isBoolean(defaultKeysOrCached) ? defaultKeysOrCached : cached : void 0;
    const _data = dataProvided ? _cached ? mergeTree(data, cachedSelectedData.value, childrenKey.value, getKey.value) : data : [];
    return {
      dataProvided,
      data: _data,
      defaultKeys: _defaultKeys,
      cached: _cached
    };
  };
  const getAllCheckedKeys = (data, defaultUnCheckedKeysOrCached, cached) => {
    const params = resolveAllCheckedFnParams(data, defaultUnCheckedKeysOrCached, cached);
    const allCheckedKeys = params.dataProvided ? checkStateResolver.getAllCheckedKeys(params.data, params.defaultKeys) : checkStateResolver.getAllCheckedKeys(params.defaultKeys);
    return appendUnexistedDataKeys(allCheckedKeys);
  };
  const getAllUncheckedKeys = (data, defaultCheckedKeysOrCached, cached) => {
    const params = resolveAllCheckedFnParams(data, defaultCheckedKeysOrCached, cached);
    const allUnCheckedKeys = params.dataProvided ? checkStateResolver.getAllUncheckedKeys(params.data, params.defaultKeys) : checkStateResolver.getAllUncheckedKeys(params.defaultKeys);
    return appendUnexistedDataKeys(allUnCheckedKeys);
  };
  const getDataByKeys = (keys) => {
    const { dataMap } = mergedResolverContext.value;
    return keys.map((key) => dataMap.get(key)).filter(Boolean);
  };
  return {
    allCheckedKeySet,
    checkDisabledKeySet,
    checkStateResolver,
    isChecked,
    isCheckDisabled,
    isIndeterminate,
    toggle,
    getAllCheckedKeys,
    getAllUncheckedKeys,
    getDataByKeys
  };
}
function useResolvers(checkedKeys, resolverContext, childrenKey, getKey, cascaderStrategy) {
  const cachedSelectedData = shallowRef([]);
  const cachedSelectedDataReolverContext = computed(
    () => getTreeCheckStateResolverContext$1(cachedSelectedData.value, childrenKey.value, getKey.value)
  );
  const mergedResolverContext = shallowRef(resolverContext.value);
  const allCheckedKeySet = shallowRef(/* @__PURE__ */ new Set());
  const unexistedKeys = { value: [] };
  const checkStateResolver = useTreeCheckStateResolver$1(mergedResolverContext, childrenKey, getKey, cascaderStrategy);
  const allCheckStateResolver = useTreeCheckStateResolver$1(
    mergedResolverContext,
    childrenKey,
    getKey,
    computed(() => "all")
  );
  const updateMergedContext = () => {
    updateCachedSelectedData();
    const { data, dataMap, parentKeyMap, depthMap } = resolverContext.value;
    const {
      data: cachedData,
      dataMap: cachedDataMap,
      parentKeyMap: cachedParentMap,
      depthMap: cachedDepthMap
    } = cachedSelectedDataReolverContext.value;
    const mergedData = mergeTree(data != null ? data : [], cachedData != null ? cachedData : [], childrenKey.value, getKey.value);
    const mergedDataMap = new Map(dataMap);
    const _cachedDataMap = new Map(cachedDataMap);
    mergedDataMap.forEach((item, key) => {
      if (_cachedDataMap.has(key)) {
        const cachedItem = _cachedDataMap.get(key);
        mergedDataMap.set(key, mergeTree([item], [cachedItem], childrenKey.value, getKey.value)[0]);
        _cachedDataMap.delete(key);
      }
    });
    _cachedDataMap.forEach((item, key) => {
      mergedDataMap.set(key, item);
    });
    const mergedParentKeyMap = new Map([...cachedParentMap, ...parentKeyMap]);
    const mergedDepthMap = new Map([...cachedDepthMap, ...depthMap]);
    mergedResolverContext.value = {
      data: mergedData,
      dataMap: mergedDataMap,
      parentKeyMap: mergedParentKeyMap,
      depthMap: mergedDepthMap
    };
  };
  const updateCachedSelectedData = () => {
    const { data } = mergedResolverContext.value;
    const keySet = allCheckedKeySet.value;
    cachedSelectedData.value = filterTree(data != null ? data : [], childrenKey.value, (item) => keySet.has(getKey.value(item)), "or");
  };
  const updateAllSelectedKeySet = () => {
    const keys = cascaderStrategy.value === "off" ? checkedKeys.value : allCheckStateResolver.appendKeys([], checkedKeys.value);
    const keySet = new Set(keys);
    allCheckedKeySet.value = keySet;
    const newUnexistedKeys = checkedKeys.value.filter((key) => !keySet.has(key));
    unexistedKeys.value = newUnexistedKeys;
  };
  watch(resolverContext, updateMergedContext);
  watch([checkedKeys, mergedResolverContext, cascaderStrategy], updateAllSelectedKeySet, { immediate: true });
  return {
    mergedResolverContext,
    cachedSelectedData,
    checkStateResolver,
    allCheckStateResolver,
    allCheckedKeySet,
    unexistedKeys
  };
}
function useOverlayFocusMonitor(onFocus, onBlur) {
  const [focused, setFocused] = useState(false);
  const [overlayFocused, setOverlayFocused] = useState(false);
  const [triggerFocused, setTriggerFocused] = useState(false);
  const { monitor, stopMonitoring, focusVia: _focusVia, blurVia } = useSharedFocusMonitor();
  const { handleFocus: _handleFocus, handleBlur: _handleBlur } = useFocusHandlers(focused, setFocused, onFocus, onBlur);
  const handleFocus = (evt) => {
    _handleFocus(evt);
    setTriggerFocused(true);
  };
  const handleBlur = (evt) => {
    _handleBlur(evt);
    setTriggerFocused(false);
  };
  const monitorStops = /* @__PURE__ */ new Set();
  const _bindMonitor = (el, isOverlay = false) => {
    const stop = watch(monitor(el, true), (evt) => {
      const { origin, event } = evt;
      if (event) {
        if (origin) {
          _handleFocus(event, origin);
          isOverlay ? setOverlayFocused(true) : setTriggerFocused(true);
        } else {
          _handleBlur(event);
          isOverlay ? setOverlayFocused(false) : setTriggerFocused(false);
        }
      }
    });
    return () => {
      stop();
      stopMonitoring(el);
    };
  };
  const bindMonitor = (elRef) => {
    let stop;
    const stopWatch = watch(
      elRef,
      (el) => {
        stop == null ? void 0 : stop();
        if (!el) {
          return;
        }
        stop = _bindMonitor(el);
      },
      {
        immediate: true
      }
    );
    const stopMonitor = () => {
      stop == null ? void 0 : stop();
      stopWatch();
    };
    monitorStops.add(stopMonitor);
    return stopMonitor;
  };
  const bindOverlayMonitor = (overlayRef, overlayOpened) => {
    let stop;
    const stopWatch = watch(
      [() => {
        var _a;
        return (_a = overlayRef.value) == null ? void 0 : _a.getPopperElement();
      }, overlayOpened],
      ([el, opened]) => {
        stop == null ? void 0 : stop();
        if (!opened || !el) {
          return;
        }
        stop = _bindMonitor(el, true);
      },
      {
        immediate: true
      }
    );
    const stopMonitor = () => {
      stop == null ? void 0 : stop();
      stopWatch();
    };
    monitorStops.add(stopMonitor);
    return stopMonitor;
  };
  const unbindAllMonitor = () => {
    monitorStops.forEach((stop) => stop());
  };
  const focusVia = (elRef, origin, options) => {
    _focusVia(elRef, origin != null ? origin : "program", options);
  };
  tryOnScopeDispose(() => {
    unbindAllMonitor();
  });
  return {
    focused,
    overlayFocused,
    triggerFocused,
    bindMonitor,
    bindOverlayMonitor,
    focusVia,
    blurVia,
    handleFocus,
    handleBlur
  };
}
function useFocusHandlers(focused, setFocused, onFocus, onBlur) {
  let lastFocusEvtTime = 0;
  const checkSubsequentFocus = async (evt) => {
    await new Promise((resolve) => setTimeout(resolve));
    return lastFocusEvtTime >= evt.timeStamp;
  };
  const handleBlur = async (evt) => {
    if (await checkSubsequentFocus(evt)) {
      return;
    }
    setFocused(false);
    onBlur(evt);
  };
  const handleFocus = (evt, origin) => {
    lastFocusEvtTime = evt.timeStamp;
    if (focused.value) {
      return;
    }
    setFocused(true);
    onFocus(evt, origin != null ? origin : "program");
  };
  return {
    handleFocus,
    handleBlur
  };
}
function useMergedCommonControlProps(props, config) {
  const mergedSize = useFormSize(props, config);
  return computed(() => {
    var _a, _b, _c;
    return {
      borderless: (_a = props.borderless) != null ? _a : config.borderless,
      clearIcon: (_b = props.clearIcon) != null ? _b : config.clearIcon,
      size: mergedSize.value,
      suffix: (_c = props.suffix) != null ? _c : config.suffix
    };
  });
}
let zIndexCount = 0;
const useZIndex = (controlZIndex, configZIndex, visible) => {
  const getZIndex = () => {
    const zIndex = configZIndex.value;
    if (isFunction(zIndex)) {
      return zIndex();
    }
    return zIndex + zIndexCount++;
  };
  const innerZIndex = ref(0);
  watch(
    visible,
    (newVisible) => {
      if (newVisible && isNil(controlZIndex.value)) {
        innerZIndex.value = getZIndex();
      }
    },
    { immediate: true }
  );
  return computed(() => {
    var _a;
    return (_a = controlZIndex.value) != null ? _a : innerZIndex.value;
  });
};
export { adjustRangeValue, applyDateTime, compareDateTime, convertIconVNode, convertStringVNode, convertTarget, convertToDate, getTreeCheckStateResolverContext, isPresetColor, isSameDateTime, isStatusColor, presetColors, sortRangeValue, statusColors, useElementSize, useGetDisabled, useGetKey, useKey, useMergedCommonControlProps, useOverlayFocusMonitor, usePortalTarget, useTreeCheckState, useTreeCheckStateResolver, useZIndex };
