import { computed, ref, watchEffect, defineComponent, watch, createVNode } from "vue";
import { useGlobalConfig } from "@idux/components/config";
import { NoopFunction } from "@idux/cdk/utils";
import { isArray } from "lodash-es";
function useAntiTamper(wrapper, el) {
  const options = {
    childList: true,
    subtree: true,
    attributeFilter: ["style", "class"],
    attributeOldValue: true
  };
  const observer = new MutationObserver((records) => {
    records.filter((record) => record.target === el || record.target === wrapper).forEach((record) => {
      if (record.type === "attributes") {
        reset(() => {
          el.setAttribute(record.attributeName, record.oldValue);
        });
      }
      if (record.type === "childList") {
        reset(() => {
          wrapper.appendChild(record.removedNodes[0]);
        });
      }
    });
  });
  observer.observe(wrapper, options);
  const reset = (resume = NoopFunction) => {
    requestAnimationFrame(() => {
      observer.disconnect();
      resume();
      observer.observe(wrapper, options);
    });
  };
  return {
    stop: () => observer.disconnect()
  };
}
function useCalcBase64(props, densityData) {
  const ratio = Math.max(window.devicePixelRatio || 1, 2);
  const rotate = computed(() => Math.PI / 180 * props.rotate);
  const base64Ref = ref("");
  watchEffect(() => {
    var _a;
    if (!props.disabled) {
      const canvas = document.createElement("canvas");
      const { gapHorizontal, gapVertical, width, height, offsetLeft, offsetTop, gapContent } = densityData.value;
      const canvasWidth = `${(gapHorizontal + width) * ratio}px`;
      const canvasHeight = `${(gapVertical + height) * ratio}px`;
      const canvasOffsetLeft = offsetLeft || gapHorizontal / 2;
      const canvasOffsetTop = offsetTop || gapVertical / 2;
      canvas.setAttribute("width", canvasWidth);
      canvas.setAttribute("height", canvasHeight);
      const ctx = canvas.getContext("2d");
      if (ctx) {
        ctx.translate(canvasOffsetLeft * ratio, canvasOffsetTop * ratio);
        ctx.rotate(rotate.value);
        ctx.globalAlpha = props.opacity;
        const markWidth = width * ratio;
        const markHeight = height * ratio;
        const content = props.content;
        if (props.type === "image") {
          const img = new Image();
          img.crossOrigin = "anonymous";
          img.referrerPolicy = "no-referrer";
          img.src = content;
          img.onload = () => {
            ctx.drawImage(img, 0, 0, markWidth, markHeight);
            base64Ref.value = canvas.toDataURL();
          };
        } else if (content) {
          ctx.fillStyle = props.fontColor;
          const markSize = props.fontSize * ratio;
          ctx.font = props.font.replace(/\d*\.?\d+px/, `${markSize}px/${markHeight}px`);
          if (isArray(content)) {
            content.forEach((str, index) => {
              ctx.fillText(str, 0, index * markSize + gapContent * index);
            });
          } else {
            ctx.fillText(content, 0, 0);
          }
          (_a = props.renderFn) == null ? void 0 : _a.call(props, ctx);
          base64Ref.value = canvas.toDataURL();
        }
      } else {
        console.error("The current browser does not support canvas.");
      }
    }
  });
  return base64Ref;
}
const watermarkProps = {
  type: { type: String, default: "text" },
  content: { type: [String, Array], default: void 0, required: true },
  rotate: { type: Number, default: -22 },
  zIndex: { type: Number, default: 11 },
  opacity: { type: Number, default: 1 },
  density: { type: String, default: "low" },
  disabled: { type: Boolean, default: false },
  width: { type: Number, default: 120 },
  height: { type: Number, default: 64 },
  fontColor: { type: String, default: "rgba(0,0,0,.15)" },
  fontSize: { type: Number, default: 16 },
  font: { type: String, default: "16px sans-serif" },
  strict: { type: Boolean, default: true },
  renderFn: Function,
  gapHorizontal: { type: Number, default: 212 },
  gapVertical: { type: Number, default: 222 },
  gapContent: { type: Number, default: 22 },
  offsetLeft: { type: Number, default: void 0 },
  offsetTop: { type: Number, default: void 0 }
};
function calcDensity(props) {
  let { density, gapHorizontal, gapVertical, width, height, offsetLeft, offsetTop, gapContent } = props;
  switch (density) {
    case "mid":
      width /= 2;
      height /= 2;
      gapVertical /= 1.5;
      gapHorizontal /= 1.5;
      gapContent /= 1.5;
      break;
    case "high":
      width /= 2;
      height /= 2;
      gapVertical /= 3;
      gapHorizontal /= 2.5;
      gapContent /= 2;
      break;
  }
  return {
    gapHorizontal,
    gapVertical,
    width,
    height,
    offsetLeft,
    offsetTop,
    gapContent
  };
}
var Watermark = /* @__PURE__ */ defineComponent({
  name: "IxWatermark",
  props: watermarkProps,
  setup(props, {
    slots
  }) {
    const common = useGlobalConfig("common");
    const mergedPrefixCls = computed(() => `${common.prefixCls}-watermark`);
    const densityData = computed(() => calcDensity(props));
    const backgroundSize = computed(() => `${densityData.value.gapHorizontal + densityData.value.width}px`);
    const wrapperRef = ref();
    const canvasRef = ref();
    const base64Ref = useCalcBase64(props, densityData);
    let antiTamper = void 0;
    watch([wrapperRef, canvasRef, () => props.strict, base64Ref], ([wrapperEl, canvasEl, strict, base64]) => {
      if (strict && wrapperEl && canvasEl && base64 !== "") {
        if (antiTamper) {
          antiTamper.stop();
        }
        antiTamper = useAntiTamper(wrapperEl, canvasEl);
      }
    }, {
      immediate: true,
      flush: "post"
    });
    const canvasStyle = computed(() => {
      return {
        zIndex: props.zIndex,
        backgroundSize: backgroundSize.value,
        backgroundImage: `url(${base64Ref.value})`,
        position: "absolute",
        top: 0,
        left: 0,
        width: "100%",
        height: "100%",
        pointerEvents: "none",
        backgroundRepeat: "repeat"
      };
    });
    return () => {
      var _a;
      const prefixCls = mergedPrefixCls.value;
      return createVNode("div", {
        "ref": wrapperRef,
        "class": prefixCls
      }, [(_a = slots.default) == null ? void 0 : _a.call(slots), createVNode("div", {
        "ref": canvasRef,
        "style": canvasStyle.value
      }, null)]);
    };
  }
});
const IxWatermark = Watermark;
export { IxWatermark };
