import { cloneVNode, defineComponent, computed, ref, onMounted, onBeforeUnmount, watch, normalizeClass, createVNode } from "vue";
import { getFirstValidNode, Logger, convertCssPixel, isNumeric, convertArray } from "@idux/cdk/utils";
import { useGlobalConfig } from "@idux/components/config";
const iconProps = {
  name: String,
  iconfont: {
    type: Boolean,
    default: false
  },
  rotate: {
    type: [Boolean, Number, String],
    default: void 0
  },
  color: String,
  size: [String, Number]
};
const iconDefinitions = /* @__PURE__ */ new Map();
const iconCache = /* @__PURE__ */ new Map();
const scriptCache = /* @__PURE__ */ new Set();
function clearSVGElement(el) {
  var _a;
  const children = el.childNodes;
  const length = children.length;
  for (let i = length - 1; i >= 0; i--) {
    const child = children[i];
    if (((_a = child.tagName) == null ? void 0 : _a.toLowerCase()) === "svg") {
      child.remove();
    }
  }
}
async function loadSVGElement(iconName, loadIconDynamically) {
  if (process.env.NODE_ENV !== "production") {
    if (removeIcons.includes(iconName)) {
      Logger.warn("components/icon", `The icon [${iconName}]  has been deprecated, please use another icon.`);
    }
    if (Object.keys(renameIcons).includes(iconName)) {
      Logger.warn(
        "components/icon",
        `The icon [${iconName}]  has been deprecated, please use [${renameIcons[iconName]}] instead.`
      );
    }
  }
  const cached = iconCache.get(iconName);
  if (cached) {
    const svg2 = await cached;
    if (svg2) {
      return svg2.cloneNode(true);
    } else {
      iconCache.delete(iconName);
      return null;
    }
  }
  const svg = createSVGElement(iconName, loadIconDynamically);
  iconCache.set(iconName, svg);
  return svg;
}
async function loadSvgElementFormScript(iconName) {
  let svg = null;
  const cachedName = `IDUX_CACHED_ICON_FONT_` + iconName;
  const cached = iconCache.get(cachedName);
  if (cached) {
    svg = await cached;
  }
  if (!svg) {
    svg = convertSVGElement(`<svg><use xlink:href="#${iconName}"></svg>`);
    setSVGAttribute(svg, iconName);
    iconCache.set(cachedName, svg);
  }
  return svg.cloneNode(true);
}
async function createSVGElement(iconName, loadIconDynamically) {
  let svg = null;
  const icon = await loadIconDefinition(iconName, loadIconDynamically);
  if (icon) {
    svg = convertSVGElement(icon.svg);
    if (svg) {
      setSVGAttribute(svg, iconName);
      iconCache.set(iconName, svg);
    } else {
      process.env.NODE_ENV !== "production" && Logger.warn("components/icon", `The icon [${iconName}] create failed.`);
    }
  }
  return svg;
}
async function loadIconDefinition(iconName, loadIconDynamically) {
  let icon = iconDefinitions.get(iconName);
  if (!icon) {
    if (loadIconDynamically) {
      const svg = await loadIconDynamically(iconName);
      if (validSVGString(svg)) {
        icon = { name: iconName, svg };
        iconDefinitions.set(iconName, icon);
      } else {
        process.env.NODE_ENV !== "production" && Logger.warn("components/icon", `The dynamically loaded icon [${iconName}] is invalid.`);
        return;
      }
    } else {
      process.env.NODE_ENV !== "production" && Logger.warn("components/icon", `The icon [${iconName}] load failed.`);
    }
  }
  return icon;
}
function validSVGString(svg) {
  return !!svg && svg.includes("<svg");
}
function convertSVGElement(svg) {
  const div = document.createElement("div");
  div.innerHTML = svg;
  return div.querySelector("svg");
}
const defaultSVGAttributes = {
  viewBox: "0 0 1024 1024",
  focusable: "false",
  "aria-hidden": "true"
};
function setSVGAttribute(svg, iconName) {
  Object.entries(defaultSVGAttributes).forEach(([key, value]) => {
    if (!svg.hasAttribute(key)) {
      svg.setAttribute(key, value);
    }
  });
  svg.setAttribute("data-icon", iconName);
}
function convertSVGNode(child) {
  const node = getFirstValidNode(child);
  if (!node || node.type !== "svg") {
    Logger.warn("components/icon", "The children must is svg element");
    return;
  }
  return cloneVNode(node, { ...defaultSVGAttributes, ...node.props });
}
function createScriptElements(urls, index = 0) {
  const currentUrl = urls[index];
  if (!scriptCache.has(currentUrl)) {
    const scriptElement = document.createElement("script");
    scriptElement.setAttribute("src", currentUrl);
    if (urls.length > index + 1) {
      scriptElement.onload = () => createScriptElements(urls, index + 1);
      scriptElement.onerror = () => {
        Logger.error("components/icon", `The url ${currentUrl} failed to load`);
        createScriptElements(urls, index + 1);
      };
    }
    scriptCache.add(currentUrl);
    document.body.appendChild(scriptElement);
  }
}
const removeIcons = ["left-double", "right-double", "unexpand"];
const renameIcons = {
  bars: "view-list",
  "book-mark": "bookmark",
  card: "view-card",
  collect: "favourite",
  "delivered-procedure": "file-export",
  "dialog-close": "close-filled",
  environment: "location",
  exception: "file-alert",
  "file-gif": "gif",
  "for-screen": "projection",
  insurance: "safety",
  "layout-large": "grid-loose",
  "loading-split": "wait",
  loose: "layout-loose",
  normal: "layout-medium",
  menu: "layout-compact",
  "scan-security": "security-scan",
  "scan-virus": "radar",
  success: "check-filled",
  transmit: "send",
  "tree-expand": "expand-all",
  "tree-unexpand": "collapse-all",
  uncollapse: "expand"
};
var Icon = /* @__PURE__ */ defineComponent({
  name: "IxIcon",
  props: iconProps,
  setup(props, {
    slots
  }) {
    const common = useGlobalConfig("common");
    const mergedPrefixCls = computed(() => `${common.prefixCls}-icon`);
    const config = useGlobalConfig("icon");
    const root = ref();
    onMounted(() => appendChild(props, config, root.value));
    onBeforeUnmount(() => {
      if (root.value) {
        clearSVGElement(root.value);
        root.value = void 0;
      }
    });
    watch([() => props.name, () => props.iconfont], () => {
      const rootElement = root.value;
      if (!rootElement) {
        return;
      }
      clearSVGElement(rootElement);
      appendChild(props, config, rootElement);
    });
    const classes = computed(() => {
      const {
        name,
        rotate
      } = props;
      const prefixCls = mergedPrefixCls.value;
      return normalizeClass({
        [prefixCls]: true,
        [`${prefixCls}-${name}`]: !!name,
        [`${prefixCls}-spinning`]: rotate === true
      });
    });
    const style = computed(() => {
      const {
        rotate,
        color,
        size
      } = props;
      return {
        color,
        fontSize: convertCssPixel(size),
        transform: isNumeric(rotate) ? `rotate(${rotate}deg)` : void 0
      };
    });
    return () => {
      var _a;
      const {
        name
      } = props;
      if (name) {
        return createVNode("i", {
          "ref": root,
          "class": classes.value,
          "style": style.value,
          "role": "img",
          "aria-label": name
        }, null);
      } else {
        return createVNode("i", {
          "ref": root,
          "class": classes.value,
          "style": style.value,
          "role": "img"
        }, [convertSVGNode((_a = slots.default) == null ? void 0 : _a.call(slots))]);
      }
    };
  }
});
async function appendChild(props, config, rootElement) {
  const {
    name,
    iconfont
  } = props;
  if (name) {
    const svgElement = iconfont ? await loadSvgElementFormScript(name) : await loadSVGElement(name, config.loadIconDynamically);
    if (svgElement && name === props.name) {
      rootElement.appendChild(svgElement);
    }
  }
}
const addIconDefinitions = (icons) => {
  icons.forEach((icon) => iconDefinitions.set(icon.name, icon));
};
const fetchIconFormScript = (url) => {
  createScriptElements(convertArray(url));
};
const Alert = {
  name: "alert",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><g fill-rule="evenodd"><path d="M512 256c174.964 0 317.131 140.418 319.957 314.708L832 576v320h51.2c6.97 0 12.634 5.563 12.8 12.493V947.2c0 6.97-5.563 12.634-12.493 12.8H140.8c-6.967 0-12.634-5.563-12.8-12.493V908.8c0-6.97 5.566-12.634 12.493-12.796l.307-.004H192V576c0-176.73 143.27-320 320-320Zm0 64c-137.1 0-249.03 107.774-255.686 243.223L256 576v320h512V576c0-137.1-107.774-249.03-243.223-255.686L512 320Z"/><path d="m504.12 384.161.308-.01c4.101-.154 7.441 3.063 7.568 7.135l.004.24v48.887c0 4.173-3.213 7.429-7.343 7.79l-.33.023c-63.261 3.74-114.162 53.424-119.807 116.154l-.157 1.905-.018.323-.049 1.332c-.161 4.39-3.626 7.924-7.985 8.056l-.253.004h-47.953c-4.505 0-8.113-3.633-7.932-8.134l.046-1.112c4.687-98.854 84.133-178.135 183.056-182.56l.845-.033ZM178.355 197.103l72.544 72.544c4.999 4.999 5.05 12.916.05 17.915l-27.39 27.39c-4.998 5-12.916 4.949-17.915-.05L133.1 242.358c-4.999-5-5.05-12.917-.05-17.915l27.39-27.39c4.998-5 12.916-4.949 17.915.05ZM863.558 197.05l27.392 27.39c4.999 4.999 4.948 12.916-.05 17.915l-72.545 72.544c-4.998 4.999-12.915 5.05-17.913.05l-27.392-27.39c-4.999-4.998-4.948-12.916.05-17.915l72.545-72.544c4.998-4.999 12.915-5.05 17.913-.05ZM531.368 64C538.438 64 544 69.634 544 76.703v102.594c0 7.07-5.563 12.703-12.632 12.703h-38.736c-7.07 0-12.632-5.634-12.632-12.703V76.703C480 69.634 485.563 64 492.632 64h38.736Z"/></g></svg>'
};
const Apartment = {
  name: "apartment",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path d="M819.2 512c7.04 0 12.8 5.76 12.8 12.8V640h115.2c7.04 0 12.8 5.76 12.8 12.8v294.4a12.8 12.8 0 0 1-12.8 12.8H652.8a12.8 12.8 0 0 1-12.8-12.8V652.8c0-7.04 5.76-12.8 12.8-12.8H768v-64H256v64h115.2c7.04 0 12.8 5.76 12.8 12.8v294.4a12.8 12.8 0 0 1-12.8 12.8H76.8A12.8 12.8 0 0 1 64 947.2V652.8c0-7.04 5.76-12.8 12.8-12.8H192V524.8c0-7.04 5.76-12.8 12.8-12.8H480V384H364.8a12.8 12.8 0 0 1-12.8-12.8V76.8c0-7.04 5.76-12.8 12.8-12.8h294.4c7.04 0 12.8 5.76 12.8 12.8v294.4a12.8 12.8 0 0 1-12.8 12.8H544v128h275.2zM320 704H128v192h192V704zm576 0H704v192h192V704zM608 128H416v192h192V128z"/></svg>'
};
const ApplyAll = {
  name: "apply-all",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path fill-rule="evenodd" d="M771.208 564.021c4-3.998 9.987-4.798 14.78-2.398l3.322 2.4 162.876 162.91c3.997 3.998 4.797 9.983 2.398 14.777l-2.398 3.322-162.875 162.92c-3.998 4-9.985 4.8-14.78 2.402l-3.322-2.4-27.157-27.148c-4-4-4.8-9.986-2.401-14.78l2.399-3.323 94.673-94.7H588.8c-5.891 0-10.853-3.979-12.343-9.397l-.457-3.402v-38.4c0-5.891 3.98-10.853 9.397-12.343l3.403-.457h249.962l-94.711-94.73c-3.999-4-4.798-9.987-2.398-14.78l2.4-3.323 27.155-27.15ZM883.2 256c5.893 0 10.853 3.978 12.343 9.394l.457 3.403V499.2c0 5.891-3.978 10.853-9.396 12.343L883.2 512h-38.4c-5.893 0-10.853-3.98-12.343-9.397L832 499.2V320H256v448h243.2c5.891 0 10.853 3.98 12.343 9.397L512 780.8v38.4c0 5.891-3.98 10.853-9.397 12.343L499.2 832H204.797c-5.891 0-10.85-3.977-12.34-9.394l-.457-3.403V268.8c0-5.891 3.98-10.853 9.397-12.343L204.8 256h678.4Zm-128-128c5.893 0 10.853 3.98 12.343 9.397L768 140.8v38.4c0 5.891-3.978 10.853-9.396 12.343L755.2 192H128v499.2c0 5.891-3.98 10.853-9.397 12.343L115.2 704H76.8c-5.891 0-10.853-3.98-12.343-9.397L64 691.2V140.8c0-5.891 3.98-10.853 9.397-12.343L76.8 128h678.4Zm0 256c7.07 0 12.8 5.73 12.8 12.8v38.4c0 7.07-5.73 12.8-12.8 12.8H332.8c-7.07 0-12.8-5.73-12.8-12.8v-38.4c0-7.07 5.73-12.8 12.8-12.8h422.4Z"/></svg>'
};
const AppstoreAdd = {
  name: "appstore-add",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><g fill-rule="evenodd"><path d="M873.51 258.33a44.796 44.796 0 0 1 23.117 39.203v78.988c0 10.868-12.653 17.095-21.497 10.78a351.95 351.95 0 0 0-35.661-22.37c-4.199-2.294-6.9-6.674-6.9-11.456v-44.693L510.65 130.734 191.991 308.718v410.303l147.687 82.925c2.553 1.433 4.545 3.737 5.665 6.444a351.982 351.982 0 0 0 20.027 40.736c7.367 12.82-11.326 34.279-24.22 27.04L150.87 769.344A44.8 44.8 0 0 1 128 730.278V297.497a44.799 44.799 0 0 1 22.953-39.112L488.769 69.688a44.8 44.8 0 0 1 43.531-.09L873.51 258.33Z"/><path d="m271.105 316.459 149.372 86.24c7.544 4.355 8.57 14.874 2.382 21.004a351.995 351.995 0 0 0-24.64 27.09c-4.07 4.972-11.205 6.487-16.77 3.274l-142.344-82.183a12.797 12.797 0 0 1-4.685-17.485l19.2-33.255a12.803 12.803 0 0 1 17.485-4.685ZM670.586 385.773c159.059 0 288 128.943 288 288 0 159.059-128.941 288-288 288-159.058 0-288-128.941-288-288 0-159.057 128.942-288 288-288Zm0 64c-123.712 0-224 100.288-224 224s100.288 224 224 224 224-100.288 224-224-100.288-224-224-224Z"/><path d="M702.586 705.485v115.488c0 7.072-5.731 12.8-12.8 12.8h-38.4c-7.07 0-12.8-5.728-12.8-12.8V705.485h-115.2c-7.07 0-12.8-5.731-12.8-12.8v-38.4c0-7.07 5.73-12.8 12.8-12.8h115.2V526.572c0-7.069 5.73-12.8 12.8-12.8h38.4c7.069 0 12.8 5.731 12.8 12.8v114.911H819.77c7.072 0 12.8 5.731 12.8 12.8v38.4c0 7.07-5.728 12.8-12.8 12.8H702.586Z"/></g></svg>'
};
const Appstore = {
  name: "appstore",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path d="M490.24 67.904 152.32 256.64a44.8 44.8 0 0 0-22.912 39.04v432.832a44.8 44.8 0 0 0 22.848 39.04l337.856 189.632a44.8 44.8 0 0 0 43.712.128l341.184-189.696a44.8 44.8 0 0 0 23.04-39.168v-432.64a44.8 44.8 0 0 0-23.104-39.232l-341.248-188.8a44.8 44.8 0 0 0-43.52.128zM512 128.96l321.92 178.048v410.176L512 896.128l-318.592-178.88V306.944L512.064 128.96z"/><path d="m768.96 319.36 19.2 33.28a12.8 12.8 0 0 1-4.672 17.472l-239.552 138.24L544 819.2a12.8 12.8 0 0 1-12.8 12.8h-38.4a12.8 12.8 0 0 1-12.8-12.8V508.288L240.448 370.112a12.8 12.8 0 0 1-4.672-17.472l19.2-33.28a12.8 12.8 0 0 1 17.472-4.672l230.848 133.248h17.28l230.848-133.248a12.8 12.8 0 0 1 17.472 4.672z"/></svg>'
};
const ArrowDown = {
  name: "arrow-down",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path fill-rule="evenodd" d="M521.024 940.247c-5 5.005-13.107 5.004-18.106-.001l-389.17-389.667c-4.998-5.003-4.997-13.114.001-18.117l49.93-49.971c4.999-5.003 13.104-5.003 18.103 0l282.283 282.52v-688.2c0-7.078 5.732-12.811 12.801-12.811h70.407c7.07 0 12.801 5.733 12.801 12.812v688.746l282.427-282.665c4.999-5.003 13.108-5.003 18.107 0l49.643 49.684c4.999 5.003 4.999 13.115 0 18.119L521.024 940.247Z"/></svg>'
};
const ArrowLeft = {
  name: "arrow-left",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path fill-rule="evenodd" d="M83.753 521.024c-5.005-5-5.004-13.107.001-18.106l389.667-389.17c5.003-4.998 13.114-4.997 18.117.001l49.971 49.93c5.003 4.999 5.003 13.104 0 18.103l-282.52 282.283h688.2c7.078 0 12.811 5.732 12.811 12.801v70.407c0 7.07-5.733 12.801-12.812 12.801H258.442l282.665 282.427c5.003 4.999 5.003 13.108 0 18.107l-49.684 49.643c-5.003 4.999-13.115 4.999-18.119 0L83.753 521.024Z"/></svg>'
};
const ArrowRight = {
  name: "arrow-right",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path fill-rule="evenodd" d="M940.247 521.024c5.005-5 5.004-13.107-.001-18.106l-389.667-389.17c-5.003-4.998-13.114-4.997-18.117.001l-49.971 49.93c-5.003 4.999-5.003 13.104 0 18.103l282.52 282.283h-688.2c-7.078 0-12.811 5.732-12.811 12.801v70.407c0 7.07 5.733 12.801 12.812 12.801h688.746L482.893 842.501c-5.003 4.999-5.003 13.108 0 18.107l49.684 49.643c5.003 4.999 13.115 4.999 18.119 0l389.551-389.227Z"/></svg>'
};
const ArrowUp = {
  name: "arrow-up",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path fill-rule="evenodd" d="M521.024 83.753c-5-5.005-13.107-5.004-18.106.001l-389.17 389.667c-4.998 5.003-4.997 13.114.001 18.117l49.93 49.971c4.999 5.003 13.104 5.003 18.103 0l282.283-282.52v688.2c0 7.078 5.732 12.811 12.801 12.811h70.407c7.07 0 12.801-5.733 12.801-12.812V258.442l282.427 282.665c4.999 5.003 13.108 5.003 18.107 0l49.643-49.684c4.999-5.003 4.999-13.115 0-18.119L521.024 83.753Z"/></svg>'
};
const Audit = {
  name: "audit",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path fill-rule="evenodd" d="M896 499.253c0 7.07-5.728 12.8-12.8 12.8h-38.4c-7.072 0-12.8-5.73-12.8-12.8v-179.26H652.8c-7.07 0-12.8-5.73-12.8-12.8V127.74l-448-.003v768.32h179.2c7.07 0 12.8 5.734 12.8 12.8v38.4c0 7.071-5.73 12.8-12.8 12.8H140.8c-7.07 0-12.8-5.729-12.8-12.8V76.8c0-7.07 5.732-12.8 12.801-12.8l557.412.056a12.8 12.8 0 0 1 9.047 3.746l184.99 184.876c2.4 2.4 3.75 5.657 3.75 9.053v237.522Zm-90.746-243.03L703.744 154.77 704 255.993l101.254.23ZM256 499.252v-38.4c0-7.069 5.73-12.8 12.8-12.8h294.4c7.07 0 12.8 5.731 12.8 12.8v38.4c0 7.07-5.73 12.8-12.8 12.8H268.8c-7.07 0-12.8-5.73-12.8-12.8Zm384 199.31c-38.356-13.56-64-49.826-64-90.51 0-53.019 42.98-96 96-96s96 42.981 96 96c0 40.684-25.644 76.95-64 90.51v69.493h179.2c7.072 0 12.8 5.728 12.8 12.8v166.4c0 7.066-5.728 12.8-12.8 12.8H460.8c-7.07 0-12.8-5.734-12.8-12.8v-166.4c0-7.072 5.73-12.8 12.8-12.8H640v-69.493Zm32-58.51c17.673 0 32-14.326 32-32 0-17.673-14.327-32-32-32-17.673 0-32 14.327-32 32 0 17.674 14.327 32 32 32Zm32 192.003H512v64h320v-64H704ZM256 627.253v-38.4c0-7.069 5.73-12.8 12.8-12.8h230.4c7.07 0 12.8 5.731 12.8 12.8v38.4c0 7.07-5.73 12.8-12.8 12.8H268.8c-7.07 0-12.8-5.73-12.8-12.8Zm0 128.003v-38.4c0-7.072 5.73-12.8 12.8-12.8h102.4c7.07 0 12.8 5.728 12.8 12.8v38.4c0 7.066-5.73 12.8-12.8 12.8H268.8c-7.07 0-12.8-5.734-12.8-12.8Z"/></svg>'
};
const BarChart = {
  name: "bar-chart",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path d="M115.2 128c7.04 0 12.8 5.76 12.8 12.8V832h819.2c7.04 0 12.8 5.76 12.8 12.8v38.4a12.8 12.8 0 0 1-12.8 12.8H76.8A12.8 12.8 0 0 1 64 883.2V140.8c0-7.04 5.76-12.8 12.8-12.8h38.4z"/><path d="M268.8 320h38.4q12.8 0 12.8 12.8v422.4q0 12.8-12.8 12.8h-38.4q-12.8 0-12.8-12.8V332.8q0-12.8 12.8-12.8Zm192-128h38.4q12.8 0 12.8 12.8v550.4q0 12.8-12.8 12.8h-38.4q-12.8 0-12.8-12.8V204.8q0-12.8 12.8-12.8Zm192 128h38.4q12.8 0 12.8 12.8v422.4q0 12.8-12.8 12.8h-38.4q-12.8 0-12.8-12.8V332.8q0-12.8 12.8-12.8Zm192 0h38.4q12.8 0 12.8 12.8v422.4q0 12.8-12.8 12.8h-38.4q-12.8 0-12.8-12.8V332.8q0-12.8 12.8-12.8Z"/></svg>'
};
const Bars = {
  name: "bars",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path d="M320 140.8v166.4a12.8 12.8 0 0 1-12.8 12.8H140.8a12.8 12.8 0 0 1-12.8-12.8V140.8c0-7.04 5.76-12.8 12.8-12.8h166.4c7.04 0 12.8 5.76 12.8 12.8zM243.2 192h-38.4a12.8 12.8 0 0 0-12.8 12.8v38.4c0 7.04 5.76 12.8 12.8 12.8h38.4a12.8 12.8 0 0 0 12.8-12.8v-38.4a12.8 12.8 0 0 0-12.8-12.8zM320 716.8v166.4a12.8 12.8 0 0 1-12.8 12.8H140.8a12.8 12.8 0 0 1-12.8-12.8V716.8c0-7.04 5.76-12.8 12.8-12.8h166.4c7.04 0 12.8 5.76 12.8 12.8zM243.2 768h-38.4a12.8 12.8 0 0 0-12.8 12.8v38.4c0 7.04 5.76 12.8 12.8 12.8h38.4a12.8 12.8 0 0 0 12.8-12.8v-38.4a12.8 12.8 0 0 0-12.8-12.8zM320 428.8v166.4a12.8 12.8 0 0 1-12.8 12.8H140.8a12.8 12.8 0 0 1-12.8-12.8V428.8c0-7.04 5.76-12.8 12.8-12.8h166.4c7.04 0 12.8 5.76 12.8 12.8zM243.2 480h-38.4a12.8 12.8 0 0 0-12.8 12.8v38.4c0 7.04 5.76 12.8 12.8 12.8h38.4a12.8 12.8 0 0 0 12.8-12.8v-38.4a12.8 12.8 0 0 0-12.8-12.8zm153.6-352h486.4q12.8 0 12.8 12.8v38.4q0 12.8-12.8 12.8H396.8q-12.8 0-12.8-12.8v-38.4q0-12.8 12.8-12.8Zm0 576h486.4q12.8 0 12.8 12.8v38.4q0 12.8-12.8 12.8H396.8q-12.8 0-12.8-12.8v-38.4q0-12.8 12.8-12.8Zm0-288h486.4q12.8 0 12.8 12.8v38.4q0 12.8-12.8 12.8H396.8q-12.8 0-12.8-12.8v-38.4q0-12.8 12.8-12.8Zm0 128h486.4q12.8 0 12.8 12.8v38.4q0 12.8-12.8 12.8H396.8q-12.8 0-12.8-12.8v-38.4q0-12.8 12.8-12.8Zm0-288h486.4q12.8 0 12.8 12.8v38.4q0 12.8-12.8 12.8H396.8q-12.8 0-12.8-12.8v-38.4q0-12.8 12.8-12.8Zm0 576h486.4q12.8 0 12.8 12.8v38.4q0 12.8-12.8 12.8H396.8q-12.8 0-12.8-12.8v-38.4q0-12.8 12.8-12.8Z"/></svg>'
};
const Blacklist = {
  name: "blacklist",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path fill-rule="evenodd" d="M384 243.27V76.8c0-7.07 5.73-12.8 12.8-12.8h232.868c7.07 0 12.8 5.73 12.8 12.8v166.47c0 7.07-5.73 12.8-12.8 12.8H396.8c-7.07 0-12.8-5.73-12.8-12.8ZM140.8 64.07h166.4c7.07 0 12.8 5.731 12.8 12.8v38.4c0 7.07-5.73 12.8-12.8 12.8H192v768h640v-768H716.8c-7.07 0-12.8-5.73-12.8-12.8v-38.4c0-7.069 5.73-12.8 12.8-12.8h166.4c7.072 0 12.8 5.731 12.8 12.8v870.4c0 7.072-5.728 12.8-12.8 12.8H140.8c-7.07 0-12.8-5.728-12.8-12.8V76.87c0-7.069 5.73-12.8 12.8-12.8ZM448 192.01h130.432v-64.065H448v64.065Zm-73.334 557.212L521.376 597.3l153.228 147.974c5.086 4.908 13.189 4.768 18.1-.32l26.675-27.623c4.91-5.081 4.769-13.19-.317-18.098l-153.228-147.97 146.087-151.278c4.91-5.085 4.769-13.188-.317-18.099l-27.622-26.674c-5.085-4.911-13.188-4.77-18.1.315L519.797 506.804 369.274 361.446c-5.085-4.91-13.188-4.769-18.099.316L324.5 389.385c-4.91 5.085-4.77 13.188.316 18.1l150.522 145.357-146.71 151.92c-4.91 5.088-4.769 13.19.317 18.099l27.622 26.675c5.085 4.915 13.189 4.774 18.1-.314Z"/></svg>'
};
const BookMark = {
  name: "book-mark",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path d="M819.2 64c7.04 0 12.8 5.76 12.8 12.8v852.16a12.8 12.8 0 0 1-21.888 9.024L521.088 647.36a12.8 12.8 0 0 0-14.784-2.432l-3.392 2.432-289.024 290.624a12.8 12.8 0 0 1-21.888-8.96V76.8c0-7.04 5.76-12.8 12.8-12.8h614.4zM768 128H256v676.8l201.664-202.688a76.864 76.864 0 0 1 102.592-5.504l6.208 5.632L768 804.864V128z"/></svg>'
};
const Bookmark = {
  name: "bookmark",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path d="M819.2 64c7.04 0 12.8 5.76 12.8 12.8v852.16a12.8 12.8 0 0 1-21.888 9.024L521.088 647.36a12.8 12.8 0 0 0-14.784-2.432l-3.392 2.432-289.024 290.624a12.8 12.8 0 0 1-21.888-8.96V76.8c0-7.04 5.76-12.8 12.8-12.8h614.4zM768 128H256v676.8l201.664-202.688a76.864 76.864 0 0 1 102.592-5.504l6.208 5.632L768 804.864V128z"/></svg>'
};
const Branch = {
  name: "branch",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><g fill-rule="evenodd"><path d="m145.842 71.449 422.4 181.028A12.8 12.8 0 0 1 576 264.242v247.56h307.2c7.072 0 12.8 5.73 12.8 12.8v422.4c0 7.071-5.728 12.8-12.8 12.8H140.8c-7.07 0-12.8-5.729-12.8-12.8V83.213c0-9.19 9.396-15.385 17.842-11.765Zm46.162 98.889-.004.228v718.835a6.408 6.408 0 0 0 6.17 6.4H505.6a6.401 6.401 0 0 0 6.396-6.17l.004-.23V302.224a6.4 6.4 0 0 0-3.646-5.778l-.233-.105-307.2-131.657c-4.152-1.78-8.765 1.186-8.917 5.654ZM825.6 575.802H576v320h249.6a6.403 6.403 0 0 0 6.396-6.171l.004-.23V582.202a6.401 6.401 0 0 0-6.17-6.396l-.23-.004Z"/><path d="M441.6 639.802a6.4 6.4 0 0 1 6.396 6.17l.004.23v51.2a6.4 6.4 0 0 1-6.17 6.396l-.23.004H262.4a6.4 6.4 0 0 1-6.396-6.17l-.004-.23v-51.2a6.4 6.4 0 0 1 6.17-6.396l.23-.004h179.2ZM441.6 511.802a6.4 6.4 0 0 1 6.396 6.17l.004.23v51.2a6.4 6.4 0 0 1-6.17 6.396l-.23.004H262.4a6.4 6.4 0 0 1-6.396-6.17l-.004-.23v-51.2a6.4 6.4 0 0 1 6.17-6.396l.23-.004h179.2ZM441.6 383.802a6.4 6.4 0 0 1 6.396 6.17l.004.23v51.2a6.4 6.4 0 0 1-6.17 6.396l-.23.004H262.4a6.4 6.4 0 0 1-6.396-6.17l-.004-.23v-51.2a6.4 6.4 0 0 1 6.17-6.396l.23-.004h179.2Z"/></g></svg>'
};
const BugFilled = {
  name: "bug-filled",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path d="M304 280h416c4.4 0 8-3.6 8-8 0-40-8.8-76.7-25.9-108.1-17.2-31.5-42.5-56.8-74-74C596.7 72.8 560 64 520 64h-16c-40 0-76.7 8.8-108.1 25.9-31.5 17.2-56.8 42.5-74 74C304.8 195.3 296 232 296 272c0 4.4 3.6 8 8 8z"/><path d="M940 512H792V412c76.8 0 139-62.2 139-139 0-4.4-3.6-8-8-8h-60c-4.4 0-8 3.6-8 8 0 34.8-28.2 63-63 63H232c-34.8 0-63-28.2-63-63 0-4.4-3.6-8-8-8h-60c-4.4 0-8 3.6-8 8 0 76.8 62.2 139 139 139v100H84c-4.4 0-8 3.6-8 8v56c0 4.4 3.6 8 8 8h148v96c0 6.5.2 13 .7 19.3C164.1 728.6 116 796.7 116 876c0 4.4 3.6 8 8 8h56c4.4 0 8-3.6 8-8 0-44.2 23.9-82.9 59.6-103.7 6 17.2 13.6 33.6 22.7 49 24.3 41.5 59 76.2 100.5 100.5 28.9 16.9 61 28.8 95.3 34.5 4.4 0 8-3.6 8-8V484c0-4.4 3.6-8 8-8h60c4.4 0 8 3.6 8 8v464.2c0 4.4 3.6 8 8 8 34.3-5.7 66.4-17.6 95.3-34.5 41.5-24.3 76.2-59 100.5-100.5 9.1-15.5 16.7-31.9 22.7-49C812.1 793.1 836 831.8 836 876c0 4.4 3.6 8 8 8h56c4.4 0 8-3.6 8-8 0-79.3-48.1-147.4-116.7-176.7.4-6.4.7-12.8.7-19.3v-96h148c4.4 0 8-3.6 8-8v-56c0-4.4-3.6-8-8-8z"/></svg>'
};
const Bug = {
  name: "bug",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path fill-rule="evenodd" d="M755.222 320H269.086c-7.18 0-12.99-5.883-12.679-13.056.17-3.928.368-7.175.594-9.74C268.535 166.496 378.298 64 512.001 64c137.1 0 249.026 107.774 255.692 243.223.163 7.005-5.47 12.777-12.471 12.777Zm-62.167-64-3.706-9.737c-27.577-66.262-91.205-113.579-166.043-117.935L511.999 128c-79.15 0-147.114 47.894-176.493 116.282L330.88 256h362.176ZM192 436.876c-70.85-25.302-122.47-91.078-127.014-169.484-.178-6.173 4.777-11.275 10.954-11.275h41.21c6.398 0 11.454 5.05 12.013 11.423l.007.076c5.534 62.103 55.463 111.433 117.83 116.034l9.524.35h511.83c67.22-.225 122.222-52.311 127.092-118.339.198-5.326 4.57-9.544 9.9-9.544h43.022c6.284 0 11.462 5.077 11.097 11.353v.076c-4.614 78.507-56.422 144.328-127.462 169.492v138.333h115.2c7.065 0 12.8 5.731 12.8 12.8v38.4c0 7.07-5.735 12.8-12.8 12.8h-115.2V640c0 46.81-10.055 91.277-28.115 131.347 85.235 16.128 150.534 88.621 155.667 177.261.179 6.176-4.775 11.277-10.95 11.277h-42.49c-5.683 0-10.189-4.493-10.63-10.157l-.007-.07c-4.96-62.701-55.142-112.672-117.933-117.306l-9.523-.352C709.642 909.722 616.692 960 512 960c-104.693 0-197.643-50.278-256.024-128h-.224c-67.418 0-122.651 52.166-127.533 118.336a9.909 9.909 0 0 1-9.901 9.549H74.57c-5.886 0-10.73-4.749-10.412-10.63l.004-.071c4.89-88.98 70.4-161.805 155.932-177.875C202.044 731.245 192 686.794 192 640v-.63H76.8c-7.07 0-12.8-5.73-12.8-12.8v-38.4c0-7.068 5.73-12.8 12.8-12.8H192V436.877ZM256 448v192c0 130.547 97.718 238.272 224 254.022V588.8c0-7.07 5.73-12.8 12.8-12.8h38.4c7.069 0 12.8 5.73 12.8 12.8v305.222c122.138-15.232 217.558-116.505 223.69-241.245l.313-12.777V448H256Z"/></svg>'
};
const Bulb = {
  name: "bulb",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><g fill-rule="evenodd"><path d="M512 64c176.73 0 320 143.27 320 320 0 104.658-50.243 197.58-127.923 255.965L704 819.2c0 7.072-5.73 12.8-12.8 12.8H332.8c-7.07 0-12.8-5.728-12.8-12.8l-.012-179.186C242.27 581.633 192 488.687 192 384c0-176.73 143.27-320 320-320Zm128 640H384v64h256v-64ZM512 128c-141.385 0-256 114.615-256 256 0 75.744 33.052 145.892 89.018 194.057l1.703 1.453 11.706 9.334 20.432 15.348a12.8 12.8 0 0 1 5.107 9.886l.005.366-.035 25.556h256.128l.049-25.653a12.8 12.8 0 0 1 4.821-9.985l.288-.223 20.402-15.334C729.652 540.682 768 465.588 768 384c0-141.385-114.615-256-256-256Z"/><path d="M319.933 329.158c20.31-51.984 59.387-94.492 108.977-119.192l13.176-6.018c6.738-2.605 14.296.88 16.691 7.696l13.315 37.898c1.955 5.565-.892 11.538-6.315 13.856-38.505 16.5-69.19 47.616-85.12 86.274-2.204 5.948-8.722 9.083-14.745 7.093L327.21 343.98c-6.202-2.05-9.65-8.736-7.278-14.821ZM384 908.8v38.4c0 7.072 5.73 12.8 12.8 12.8h230.4c7.07 0 12.8-5.728 12.8-12.8v-38.4c0-7.072-5.73-12.8-12.8-12.8H396.8c-7.07 0-12.8 5.728-12.8 12.8Z"/></g></svg>'
};
const Calendar = {
  name: "calendar",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path d="M128 256h768v64H128zm128 192h128v64H256z"/><path d="M332.8 64h38.4c7.04 0 12.8 5.76 12.8 12.8V320h-64V76.8c0-7.04 5.76-12.8 12.8-12.8zm320 0h38.4c7.04 0 12.8 5.76 12.8 12.8V320h-64V76.8c0-7.04 5.76-12.8 12.8-12.8zM256 640h128v64H256zm192-192h128v64H448zm0 192h128v64H448zm192-192h128v64H640zm0 192h128v64H640z"/><path d="M960 140.8v742.4a12.8 12.8 0 0 1-12.8 12.8H76.8A12.8 12.8 0 0 1 64 883.2V140.8c0-7.04 5.76-12.8 12.8-12.8h870.4c7.04 0 12.8 5.76 12.8 12.8zM896 192H128v640h768V192z"/></svg>'
};
const Camera = {
  name: "camera",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><g fill-rule="evenodd"><path d="M696.089 128c4.849 0 9.28 2.74 11.45 7.076L768 256h179.2c7.072 0 12.8 5.73 12.8 12.8v614.4c0 7.072-5.728 12.8-12.8 12.8H76.8c-7.07 0-12.8-5.728-12.8-12.8V268.8c0-7.07 5.73-12.8 12.8-12.8H128v-51.2c0-7.07 5.73-12.8 12.8-12.8h38.4c7.07 0 12.8 5.73 12.8 12.8V256h63.936l60.526-120.924A12.8 12.8 0 0 1 327.911 128h368.178Zm-31.705 64H359.488l-64.017 128H128v512h768V320H728.448l-64.064-128Z"/><path d="M512 352c123.712 0 224 100.288 224 224S635.712 800 512 800 288 699.712 288 576s100.288-224 224-224Zm0 64c-88.365 0-160 71.635-160 160s71.635 160 160 160 160-71.635 160-160-71.635-160-160-160Z"/><path d="M499.101 480.859c-42.7 5.734-76.508 39.542-82.242 82.242-.94 7.007 4.872 12.899 11.941 12.899h38.4c7.07 0 12.608-6.196 15.412-12.685a32.119 32.119 0 0 1 16.703-16.703C505.805 543.808 512 538.27 512 531.2v-38.4c0-7.07-5.892-12.882-12.899-11.941Z"/></g></svg>'
};
const Card = {
  name: "card",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path d="M480 140.8v326.4a12.8 12.8 0 0 1-12.8 12.8H140.8a12.8 12.8 0 0 1-12.8-12.8V140.8c0-7.04 5.76-12.8 12.8-12.8h326.4c7.04 0 12.8 5.76 12.8 12.8zM416 192H192v224h224V192zm64 364.8v326.4a12.8 12.8 0 0 1-12.8 12.8H140.8a12.8 12.8 0 0 1-12.8-12.8V556.8c0-7.04 5.76-12.8 12.8-12.8h326.4c7.04 0 12.8 5.76 12.8 12.8zM416 608H192v224h224V608zm480-467.2v326.4a12.8 12.8 0 0 1-12.8 12.8H556.8a12.8 12.8 0 0 1-12.8-12.8V140.8c0-7.04 5.76-12.8 12.8-12.8h326.4c7.04 0 12.8 5.76 12.8 12.8zM832 192H608v224h224V192zm64 364.8v326.4a12.8 12.8 0 0 1-12.8 12.8H556.8a12.8 12.8 0 0 1-12.8-12.8V556.8c0-7.04 5.76-12.8 12.8-12.8h326.4c7.04 0 12.8 5.76 12.8 12.8zM832 608H608v224h224V608z"/></svg>'
};
const CaretDownFilled = {
  name: "caret-down-filled",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path d="M840.4 300H183.6c-19.7 0-30.7 20.8-18.5 35l328.4 380.8c9.4 10.9 27.5 10.9 37 0L858.9 335c12.2-14.2 1.2-35-18.5-35z"/></svg>'
};
const CaretDown = {
  name: "caret-down",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path d="M900.48 288c7.104 0 12.8 5.76 12.8 12.8v39.424a12.8 12.8 0 0 1-.192 1.408l.256-2.432a12.8 12.8 0 0 1-4.288 9.6L520.96 736.64a12.8 12.8 0 0 1-18.048 0L114.88 348.672a12.8 12.8 0 0 1-4.16-9.472v-38.4c0-7.04 5.76-12.8 12.8-12.8h777.088zm-85.376 64H208.768l294.208 294.144a12.8 12.8 0 0 0 14.72 2.432l3.328-2.432L815.104 352z"/></svg>'
};
const CaretLeftFilled = {
  name: "caret-left-filled",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path d="M689 165.1 308.2 493.5c-10.9 9.4-10.9 27.5 0 37L689 858.9c14.2 12.2 35 1.2 35-18.5V183.6c0-19.7-20.8-30.7-35-18.5z"/></svg>'
};
const CaretRightFilled = {
  name: "caret-right-filled",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path d="M715.8 493.5 335 165.1c-14.2-12.2-35-1.2-35 18.5v656.8c0 19.7 20.8 30.7 35 18.5l380.8-328.4c10.9-9.4 10.9-27.6 0-37z"/></svg>'
};
const CaretUpFilled = {
  name: "caret-up-filled",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path d="M858.9 689 530.5 308.2c-9.4-10.9-27.5-10.9-37 0L165.1 689c-12.2 14.2-1.2 35 18.5 35h656.8c19.7 0 30.7-20.8 18.5-35z"/></svg>'
};
const CaretUp = {
  name: "caret-up",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path d="M900.48 740.416a12.8 12.8 0 0 0 12.8-12.8v-39.424a12.8 12.8 0 0 0-.192-1.408l.256 2.432a12.8 12.8 0 0 0-4.288-9.6L520.96 291.84a12.8 12.8 0 0 0-18.048 0L114.88 679.68a12.8 12.8 0 0 0-4.224 9.536v38.4c0 7.04 5.76 12.8 12.8 12.8h777.088zm-85.376-64H208.768L503.04 382.272a12.8 12.8 0 0 1 14.72-2.432l3.328 2.432 294.08 294.144z"/></svg>'
};
const CheckCircleFilled = {
  name: "check-circle-filled",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path d="M512 0c282.752 0 512 229.248 512 512s-229.248 512-512 512S0 794.752 0 512 229.248 0 512 0zm238.336 366.933a17.067 17.067 0 0 0-24.15 0L466.689 626.432l-138.71-138.795a17.067 17.067 0 0 0-24.149 0l-36.224 36.224a17.067 17.067 0 0 0 0 24.15l187.051 187.05a17.067 17.067 0 0 0 24.15 0L786.56 427.307a17.067 17.067 0 0 0 0-24.15z"/></svg>'
};
const CheckCircle = {
  name: "check-circle",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><g fill-rule="evenodd"><path d="M512 64c247.424 0 448 200.577 448 448 0 247.424-200.576 448-448 448-247.423 0-448-200.576-448-448C64 264.577 264.577 64 512 64Zm0 64c-212.077 0-384 171.923-384 384s171.923 384 384 384 384-171.923 384-384-171.923-384-384-384Z"/><path d="M797.382 396.82 453.734 740.473c-4.999 4.998-12.935 4.934-17.934-.064L227.404 532.013c-4.999-4.999-5.064-12.93-.066-17.93l27.391-27.39c4.999-4.999 12.944-4.92 17.942.08l172.084 172.114 307.239-307.19c4.998-4.997 13.004-5.135 18.003-.137l27.392 27.39c4.998 5 4.998 12.87-.007 17.87Z"/></g></svg>'
};
const CheckFilled = {
  name: "check-filled",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path d="M264.384 443.52a25.6 25.6 0 0 1 36.224 0L463.552 606.4 762.24 307.712a25.6 25.6 0 0 1 36.16 0l54.336 54.272a25.6 25.6 0 0 1 0 36.224L481.6 769.28a25.6 25.6 0 0 1-36.16 0L210.112 534.016a25.6 25.6 0 0 1 0-36.224l54.272-54.336z"/></svg>'
};
const Check = {
  name: "check",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path fill-rule="evenodd" d="M977.15 287.8 452.51 812.44c-4.998 4.998-13.277 4.986-18.276-.013L67.857 446.05c-4.999-5-5.211-13.078-.212-18.077l27.39-27.39c4.999-4.999 12.65-5.214 17.649-.215l330.66 330.697 488.5-488.468c4.998-4.999 13.114-5.25 18.112-.251l27.392 27.39c4.998 4.999 4.8 13.066-.198 18.064Z"/></svg>'
};
const CheckingFilled = {
  name: "checking-filled",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path d="M512 0c282.752 0 512 229.248 512 512s-229.248 512-512 512S0 794.752 0 512 229.248 0 512 0zm25.6 213.333h-51.2a17.067 17.067 0 0 0-17.067 17.067v292.267a15.787 15.787 0 0 0 .512 4.053c.598 3.285 2.134 6.485 4.694 9.045l187.093 187.051a17.067 17.067 0 0 0 24.15 0l36.18-36.181a17.067 17.067 0 0 0 0-24.15L554.668 495.147V230.4a17.067 17.067 0 0 0-17.067-17.067z"/></svg>'
};
const Clear = {
  name: "clear",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path fill-rule="evenodd" d="M627.2 64c6.967 0 12.634 5.566 12.796 12.493l.004.307V320h307.2c6.97 0 12.634 5.566 12.8 12.493V499.2c0 6.967-5.563 12.634-12.493 12.796l-.307.004H848l108.026 432.096c1.993 7.985-3.952 15.726-12.14 15.904H144.394c-8.233 0-14.298-7.65-12.484-15.629l.066-.275L240 512h-99.2c-6.967 0-12.634-5.566-12.8-12.493V332.8c0-6.967 5.566-12.634 12.493-12.796l.307-.004H448V76.8c0-6.967 5.566-12.634 12.493-12.8H627.2Zm154.746 448H305.98l-96 384h117.726l58.477-219.857c1.817-6.832 8.828-10.896 15.66-9.08l37.11 9.87c6.831 1.818 10.897 8.83 9.08 15.66L393.93 896h484.015l-96-384ZM896 384H192v64h704v-64ZM576 128h-64v192h64V128Z"/></svg>'
};
const ClockCircle = {
  name: "clock-circle",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path d="M512 64a448 448 0 1 1 0 896 448 448 0 0 1 0-896zm0 64a384 384 0 1 0 0 768 384 384 0 0 0 0-768z"/><path d="M512 268.8v291.456c0 7.04 5.76 12.8 12.8 12.8h166.4c7.04 0 12.8 5.76 12.8 12.8v38.4a12.8 12.8 0 0 1-12.8 12.8H460.8a12.8 12.8 0 0 1-12.8-12.8V268.8c0-7.04 5.76-12.8 12.8-12.8h38.4c7.04 0 12.8 5.76 12.8 12.8z"/></svg>'
};
const CloseCircleFilled = {
  name: "close-circle-filled",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path d="M512 0c282.752 0 512 229.248 512 512s-229.248 512-512 512S0 794.752 0 512 229.248 0 512 0zm192 288.128a14.635 14.635 0 0 0-20.65 0l-170.923 170.88-168.278-168.277a14.635 14.635 0 0 0-20.693 0l-31.061 31.018a14.635 14.635 0 0 0 0 20.694l168.32 168.277-170.752 170.752a14.635 14.635 0 0 0 0 20.693l31.018 31.062a14.635 14.635 0 0 0 20.694 0l170.752-170.795 170.752 170.795a14.635 14.635 0 0 0 20.693 0l31.061-31.019a14.635 14.635 0 0 0 0-20.693L564.181 510.72l170.88-170.88a14.635 14.635 0 0 0 0-20.693l-31.018-31.019z"/></svg>'
};
const CloseCircle = {
  name: "close-circle",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path d="M512 64a448 448 0 1 1 0 896 448 448 0 0 1 0-896zm0 64a384 384 0 1 0 0 768 384 384 0 0 0 0-768zm195.2 215.232a12.8 12.8 0 0 1 0 18.112L557.632 510.848l149.44 149.44a12.8 12.8 0 0 1 0 18.112l-27.2 27.136a12.8 12.8 0 0 1-18.048 0L512.384 556.16 362.944 705.6a12.8 12.8 0 0 1-18.048 0l-27.2-27.136a12.8 12.8 0 0 1 0-18.112L467.2 510.848l-147.264-147.2a12.8 12.8 0 0 1 0-18.112l27.136-27.136a12.8 12.8 0 0 1 18.112 0l147.264 147.2 149.568-149.504a12.8 12.8 0 0 1 18.048 0l27.2 27.136z"/></svg>'
};
const CloseFilled = {
  name: "close-filled",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path fill-rule="evenodd" d="M916.741 210.766c4.999-4.999 4.999-13.103 0-18.102l-45.255-45.255c-4.999-4.999-13.103-4.999-18.102 0L532.073 468.721 210.766 147.414c-4.999-5-13.103-5-18.102 0l-45.255 45.254c-4.999 5-4.999 13.104 0 18.102l321.307 321.307-321.307 321.307c-5 5.001-4.999 13.103 0 18.102l45.255 45.255c4.999 4.999 13.101 5 18.102 0l321.307-321.307 321.311 321.312c5.001 5 13.103 4.999 18.102 0l45.255-45.255c4.999-4.999 5-13.101 0-18.102L595.43 532.077l321.31-321.311Z"/></svg>'
};
const Close = {
  name: "close",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path d="M282.24 236.928 512 466.688l229.76-229.76a12.8 12.8 0 0 1 18.176 0l27.136 27.136a12.8 12.8 0 0 1 0 18.112L557.248 512l229.76 229.888a12.8 12.8 0 0 1 0 18.112l-27.072 27.136a12.8 12.8 0 0 1-18.112 0L512 557.248l-229.76 229.76a12.8 12.8 0 0 1-18.176 0l-27.136-27.072a12.8 12.8 0 0 1 0-18.112L466.688 512l-229.76-229.76a12.8 12.8 0 0 1 0-18.176l27.136-27.136a12.8 12.8 0 0 1 18.112 0z"/></svg>'
};
const CloudDownload = {
  name: "cloud-download",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path fill-rule="evenodd" d="M64 640c0-94.779 51.506-177.528 128.056-221.784L192 416c0-159.058 128.942-288 288-288 150.921 0 274.726 116.087 287.002 263.84C877.9 419.851 960 520.337 960 640c0 137.1-107.776 249.03-243.226 255.686L704 896h-51.2c-7.07 0-12.8-5.728-12.8-12.8v-38.38c0-7.073 5.731-12.8 12.8-12.8h49.631l11.201-.257C815.514 826.758 896 742.461 896 640c0-84.559-55.149-158.125-133.805-183.02l-10.867-3.09-35.456-8.956a12.803 12.803 0 0 1-9.626-11.351l-3.026-36.446C693.627 281.637 596.704 192 480 192c-119.588 0-217.288 93.714-223.687 210.26l-.333 12.136.818 32.608a12.8 12.8 0 0 1-6.39 11.402l-26.32 15.217C165.066 507.745 128 570.609 128 640c0 102.252 79.93 185.837 180.719 191.674L320 832h51.2c7.07 0 12.8 5.734 12.8 12.8v38.4c0 7.072-5.73 12.8-12.8 12.8H320C178.615 896 64 781.382 64 640Zm325.368 84.005L501.916 889.39c5.08 7.462 16.083 7.462 21.164 0l112.547-165.384c5.783-8.497-.303-20.001-10.583-20.001H544V524.8c0-7.07-5.73-12.8-12.8-12.8h-38.4c-7.07 0-12.8 5.73-12.8 12.8v179.204h-80.05c-10.278 0-16.364 11.504-10.582 20.001Z"/></svg>'
};
const CloudServer = {
  name: "cloud-server",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path fill-rule="evenodd" d="M192.056 418.216C115.506 462.472 64 545.221 64 640c0 86.761 43.16 163.443 109.182 209.74 8.071 5.665 18.818-.32 18.818-10.182v-50.726c0-3.642-1.61-7.13-4.25-9.638C150.945 744.21 128 694.784 128 640c0-69.39 37.066-132.255 96.088-166.377l26.32-15.217a12.8 12.8 0 0 0 6.39-11.402l-.818-32.608.333-12.136C262.712 285.714 360.413 192 480 192c116.704 0 213.627 89.637 223.22 205.137l3.026 36.446a12.803 12.803 0 0 0 9.626 11.35l35.456 8.957 10.867 3.09C840.851 481.875 896 555.44 896 640c0 54.66-22.906 104.152-59.744 139.194-2.637 2.508-4.256 6.003-4.256 9.644v50.72c0 9.863 10.746 15.847 18.816 10.183C916.838 803.443 960 726.76 960 640c0-119.662-82.1-220.148-192.998-248.16C754.726 244.086 630.92 128 480 128c-159.058 0-288 128.942-288 288l.056 2.216ZM256 883.2V588.8c0-7.07 5.73-12.8 12.8-12.8h486.4c7.072 0 12.8 5.73 12.8 12.8v294.4c0 7.072-5.728 12.8-12.8 12.8H268.8c-7.07 0-12.8-5.728-12.8-12.8ZM320 640v192h384V640H320Zm192 76.8v38.4c0 7.07 5.73 12.8 12.8 12.8h102.4c7.07 0 12.8-5.73 12.8-12.8v-38.4c0-7.07-5.73-12.8-12.8-12.8H524.8c-7.07 0-12.8 5.73-12.8 12.8Z"/></svg>'
};
const CloudSync = {
  name: "cloud-sync",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path fill-rule="evenodd" d="M64 640c0-94.779 51.506-177.528 128.056-221.784L192 416c0-159.058 128.942-288 288-288 150.921 0 274.726 116.087 287.002 263.84C877.9 419.851 960 520.337 960 640c0 113.47-73.824 209.696-176.064 243.27-7.949 2.612-15.936-3.481-15.936-11.846v-41.28c0-5.42 3.494-10.24 8.518-12.294C846.746 789.197 896 720.156 896 640c0-84.559-55.149-158.125-133.805-183.02l-10.867-3.09-35.456-8.956a12.803 12.803 0 0 1-9.626-11.351l-3.026-36.446C693.627 281.637 596.704 192 480 192c-119.588 0-217.288 93.714-223.687 210.26l-.333 12.136.818 32.608a12.8 12.8 0 0 1-6.39 11.402l-26.32 15.217C165.066 507.745 128 570.609 128 640c0 80.385 49.4 149.229 119.496 177.837 5.023 2.054 8.504 6.867 8.504 12.288v41.299c0 8.365-7.99 14.458-15.937 11.846C137.825 849.696 64 753.47 64 640Zm603.969 175.974A192.018 192.018 0 0 0 704 704c0-106.038-85.962-192-192-192-35.204 0-69.63 9.677-99.57 27.836-6.751 4.095-7.33 13.271-1.746 18.855l28.453 28.453c4.151 4.152 10.475 4.935 15.729 2.315A128 128 0 0 1 512 576c70.692 0 128 57.308 128 128 0 8.24-.796 16.46-2.376 24.547h-28.33c-11.496 0-17.164 13.978-8.914 21.985l66.635 64.668c.047.051.127.032.151-.032l.803.806ZM430.378 670.46l-29.264-30.399.157-.271-11.211-11.212-33.734-35.042a.648.648 0 0 0-1.077.232l-.265-.266a192.002 192.002 0 0 0-34.587 98.156l-.17.48h.14a191.574 191.574 0 0 0-.367 11.863c0 106.042 85.962 192 192 192 34.515 0 68.294-9.3 97.834-26.797 6.837-4.045 7.525-13.331 1.906-18.95l-28.567-28.563c-4.103-4.11-10.452-4.858-15.681-2.343A128.054 128.054 0 0 1 512 832c-70.692 0-128-57.306-128-128 0-3.961.184-7.92.551-11.864h36.605c11.287 0 17.05-13.546 9.222-21.677Z"/></svg>'
};
const CloudUpload = {
  name: "cloud-upload",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path fill-rule="evenodd" d="M64 640c0-94.779 51.506-177.528 128.056-221.784L192 416c0-159.058 128.942-288 288-288 150.921 0 274.726 116.087 287.002 263.84C877.9 419.851 960 520.337 960 640c0 137.1-107.776 249.03-243.226 255.686L704 896h-51.2c-7.07 0-12.8-5.728-12.8-12.8v-38.38c0-7.073 5.731-12.8 12.8-12.8h49.631l11.201-.257C815.514 826.758 896 742.461 896 640c0-84.559-55.149-158.125-133.805-183.02l-10.867-3.09-35.456-8.956a12.803 12.803 0 0 1-9.626-11.351l-3.026-36.446C693.627 281.637 596.704 192 480 192c-119.588 0-217.288 93.714-223.687 210.26l-.333 12.136.818 32.608a12.8 12.8 0 0 1-6.39 11.402l-26.32 15.217C165.066 507.745 128 570.609 128 640c0 102.252 79.93 185.837 180.719 191.674L320 832h51.2c7.07 0 12.8 5.734 12.8 12.8v38.4c0 7.072-5.73 12.8-12.8 12.8H320C178.615 896 64 781.382 64 640Zm571.628 44.424L523.08 519.042c-5.08-7.465-16.084-7.465-21.164 0L389.37 684.424c-5.783 8.498.303 20.002 10.581 20.002h81.045v179.203c0 7.072 5.731 12.8 12.8 12.8h38.4c7.07 0 12.8-5.728 12.8-12.8V704.426h80.05c10.278 0 16.365-11.504 10.582-20.002Z"/></svg>'
};
const Cloud = {
  name: "cloud",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path fill-rule="evenodd" d="M64 640c0-94.779 51.506-177.528 128.056-221.784L192 416c0-159.058 128.942-288 288-288 150.921 0 274.726 116.087 287.002 263.84C877.9 419.851 960 520.337 960 640c0 137.1-107.776 249.03-243.226 255.686L704 896H320C178.615 896 64 781.382 64 640Zm687.328-186.11-35.456-8.956a12.803 12.803 0 0 1-9.626-11.351l-3.026-36.446C693.627 281.637 596.704 192 480 192c-119.588 0-217.288 93.714-223.687 210.26l-.333 12.136.818 32.608a12.8 12.8 0 0 1-6.39 11.402l-26.32 15.217C165.066 507.745 128 570.609 128 640c0 102.252 79.93 185.837 180.719 191.674L320 832l382.431.02 11.201-.257C815.514 826.758 896 742.461 896 640c0-84.559-55.149-158.125-133.805-183.02l-10.867-3.09Z"/></svg>'
};
const Cluster = {
  name: "cluster",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path d="M755.2 64c7.04 0 12.8 5.76 12.8 12.8v294.4a12.8 12.8 0 0 1-12.8 12.8H268.8a12.8 12.8 0 0 1-12.8-12.8V76.8c0-7.04 5.76-12.8 12.8-12.8h486.4zM704 128H320v192h384V128z"/><path d="M524.8 192h102.4q12.8 0 12.8 12.8v38.4q0 12.8-12.8 12.8H524.8q-12.8 0-12.8-12.8v-38.4q0-12.8 12.8-12.8ZM307.2 704c7.04 0 12.8 5.76 12.8 12.8v230.4a12.8 12.8 0 0 1-12.8 12.8H76.8A12.8 12.8 0 0 1 64 947.2V716.8c0-7.04 5.76-12.8 12.8-12.8h230.4zM256 768H128v128h128V768zm371.2-64c7.04 0 12.8 5.76 12.8 12.8v230.4a12.8 12.8 0 0 1-12.8 12.8H396.8a12.8 12.8 0 0 1-12.8-12.8V716.8c0-7.04 5.76-12.8 12.8-12.8h230.4zM576 768H448v128h128V768zm371.2-64c7.04 0 12.8 5.76 12.8 12.8v230.4a12.8 12.8 0 0 1-12.8 12.8H716.8a12.8 12.8 0 0 1-12.8-12.8V716.8c0-7.04 5.76-12.8 12.8-12.8h230.4zM896 768H768v128h128V768z"/><path d="M480 384h64v384h-64zM140.8 512h38.4q12.8 0 12.8 12.8v230.4q0 12.8-12.8 12.8h-38.4q-12.8 0-12.8-12.8V524.8q0-12.8 12.8-12.8Zm704 0h38.4q12.8 0 12.8 12.8v230.4q0 12.8-12.8 12.8h-38.4q-12.8 0-12.8-12.8V524.8q0-12.8 12.8-12.8Z"/><path d="M140.8 512h742.4q12.8 0 12.8 12.8v38.4q0 12.8-12.8 12.8H140.8q-12.8 0-12.8-12.8v-38.4q0-12.8 12.8-12.8Z"/></svg>'
};
const Code = {
  name: "code",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path fill-rule="evenodd" d="M140.8 896h742.4c7.072 0 12.8-5.728 12.8-12.8V140.8c0-7.07-5.728-12.8-12.8-12.8H140.8c-7.07 0-12.8 5.73-12.8 12.8v742.4c0 7.072 5.73 12.8 12.8 12.8Zm51.2-64V192h640v640H192Zm220.739-320.046-152.99 152.968a12.8 12.8 0 0 0-3.749 9.052v54.36c0 11.404 13.786 17.115 21.85 9.053L494.24 521.05c5-4.998 5-13.104.001-18.103l-216.39-216.393c-8.063-8.063-21.851-2.352-21.851 9.051v54.293a12.8 12.8 0 0 0 3.748 9.05l152.99 153.006ZM448 755.2v-38.4c0-7.07 5.73-12.8 12.8-12.8h230.4c7.07 0 12.8 5.73 12.8 12.8v38.4c0 7.07-5.73 12.8-12.8 12.8H460.8c-7.07 0-12.8-5.73-12.8-12.8Z"/></svg>'
};
const CollapseAll = {
  name: "collapse-all",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path fill-rule="evenodd" d="M192 256v-51.2c0-7.07-5.73-12.8-12.8-12.8h-38.4c-7.07 0-12.8 5.73-12.8 12.8v614.4c0 7.07 5.73 12.8 12.8 12.8h166.4c7.07 0 12.8-5.73 12.8-12.8v-38.4c0-7.07-5.73-12.8-12.8-12.8H192V576h115.2c7.07 0 12.8-5.73 12.8-12.8v-38.4c0-7.07-5.73-12.8-12.8-12.8H192V320h115.2c7.07 0 12.8-5.73 12.8-12.8v-38.4c0-7.07-5.73-12.8-12.8-12.8H192Zm704 51.2v-38.4c0-7.07-5.728-12.8-12.8-12.8H396.8c-7.07 0-12.8 5.73-12.8 12.8v38.4c0 7.07 5.73 12.8 12.8 12.8h486.4c7.072 0 12.8-5.73 12.8-12.8Zm0 256v-38.4c0-7.07-5.728-12.8-12.8-12.8H396.8c-7.07 0-12.8 5.73-12.8 12.8v38.4c0 7.07 5.73 12.8 12.8 12.8h486.4c7.072 0 12.8-5.73 12.8-12.8Zm0 256v-38.4c0-7.07-5.728-12.8-12.8-12.8H396.8c-7.07 0-12.8 5.73-12.8 12.8v38.4c0 7.07 5.73 12.8 12.8 12.8h486.4c7.072 0 12.8-5.73 12.8-12.8Z"/></svg>'
};
const Collapse = {
  name: "collapse",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path d="M819.2 128c7.04 0 12.8 5.76 12.8 12.8v742.4a12.8 12.8 0 0 1-12.8 12.8h-38.4a12.8 12.8 0 0 1-12.8-12.8V140.8c0-7.04 5.76-12.8 12.8-12.8h38.4zm-266.176 76.224 27.2 27.2a12.8 12.8 0 0 1 0 18.112l-262.528 262.4 262.528 262.528a12.8 12.8 0 0 1 0 18.112l-27.2 27.136a12.8 12.8 0 0 1-18.048 0L236.288 521.024a12.8 12.8 0 0 1 0-18.112l298.688-298.688a12.8 12.8 0 0 1 18.048 0z"/></svg>'
};
const Collect = {
  name: "collect",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path d="M883.2 192c7.04 0 12.8 5.76 12.8 12.8V448h-64V256H128v512l256-.064V832H76.8A12.8 12.8 0 0 1 64 819.2V192h819.2z"/><path d="m621.248 256-192-192H128a64 64 0 0 0-64 64v128h557.248zM402.752 128l64 64H128v-64h274.752zM651.52 343.232l79.552 161.28a12.8 12.8 0 0 0 9.6 7.04l177.984 25.856a12.8 12.8 0 0 1 7.104 21.76L796.992 684.8a12.8 12.8 0 0 0-3.712 11.328l30.4 177.28a12.8 12.8 0 0 1-18.56 13.44L645.952 803.2a12.8 12.8 0 0 0-11.904 0L474.88 886.848a12.8 12.8 0 0 1-18.56-13.504l30.4-177.28a12.8 12.8 0 0 0-3.712-11.328L354.24 559.232a12.8 12.8 0 0 1 7.04-21.824l177.984-25.856a12.8 12.8 0 0 0 9.664-7.04l79.616-161.28a12.8 12.8 0 0 1 22.912 0zm40 225.792L640 464.64l-51.52 104.448-115.264 16.768 83.392 81.28-19.776 114.752L640 727.68l103.104 54.144-19.712-114.752 83.328-81.28-115.2-16.768z"/></svg>'
};
const Comment = {
  name: "comment",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path fill-rule="evenodd" d="M512 896c-59.066 0-116.58-9.798-170.06-28.595l-231.968 67.117c-8.64 2.502-17.096-4.455-16.308-13.415l20.448-232.595-3.58-5.775c-27.356-47.229-43.162-99.031-46.05-152.76L64 512c0-213.372 201.65-384 448-384 246.349 0 448 170.628 448 384S758.349 896 512 896ZM173.796 663.518l5.983 9.238-15.811 179.468 180.48-52.128 9.948 3.744C403.486 822.31 456.893 832 512 832c213.152 0 384-144.564 384-320S725.152 192 512 192c-213.151 0-384 144.564-384 320 0 53.576 15.837 105.258 45.796 151.518ZM320.844 480c-26.51 0-48 21.49-48 48s21.49 48 48 48 48-21.49 48-48-21.49-48-48-48Zm192 0c-26.51 0-48 21.49-48 48s21.49 48 48 48 48-21.49 48-48-21.49-48-48-48Zm192 0c-26.51 0-48 21.49-48 48s21.49 48 48 48 48-21.49 48-48-21.49-48-48-48Z"/></svg>'
};
const Control = {
  name: "control",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path d="M896 140.8v742.4a12.8 12.8 0 0 1-12.8 12.8H140.8a12.8 12.8 0 0 1-12.8-12.8V140.8c0-7.04 5.76-12.8 12.8-12.8h742.4c7.04 0 12.8 5.76 12.8 12.8zM832 192H192v640h640V192z"/><path d="M332.8 256h38.4q12.8 0 12.8 12.8v486.4q0 12.8-12.8 12.8h-38.4q-12.8 0-12.8-12.8V268.8q0-12.8 12.8-12.8Zm320 0h38.4q12.8 0 12.8 12.8v486.4q0 12.8-12.8 12.8h-38.4q-12.8 0-12.8-12.8V268.8q0-12.8 12.8-12.8Z"/><path d="M352 512a96 96 0 1 1 0 192 96 96 0 0 1 0-192zm320-192a96 96 0 1 1 0 192 96 96 0 0 1 0-192z"/><path d="M352 512a96 96 0 1 1 0 192 96 96 0 0 1 0-192zm0 64a32 32 0 1 0 0 64 32 32 0 0 0 0-64zm320-256a96 96 0 1 1 0 192 96 96 0 0 1 0-192zm0 64a32 32 0 1 0 0 64 32 32 0 0 0 0-64z"/></svg>'
};
const Controlz = {
  name: "controlz",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path fill-rule="evenodd" d="M896 140.8v742.4c0 7.07-5.73 12.8-12.8 12.8H140.8c-7.07 0-12.8-5.73-12.8-12.8V140.8c0-7.07 5.73-12.8 12.8-12.8h742.4c7.07 0 12.8 5.73 12.8 12.8ZM832 192H192v640h640V192Zm-448.19 76.8.192 248.69 4.735 1.818a95.912 95.912 0 0 1 16.597 8.87l7.568 5.613 3.568 3.078 3.412 3.249a95.933 95.933 0 0 1 6.327 6.98l5.612 7.567a95.974 95.974 0 0 1 15.717 43.925l.346 4.7L448 608a95.99 95.99 0 0 1-16.18 53.334l-5.61 7.568-3.078 3.568-3.25 3.412a95.987 95.987 0 0 1-6.98 6.327l-7.567 5.612a96.097 96.097 0 0 1-8.08 4.843l-4.21 2.119-9.043 3.726L384 755.04c0 7.054-5.706 12.778-12.76 12.8l-38.24.12c-5.891.021-10.865-3.943-12.372-9.355l-.465-3.405-.163-56.69-4.738-1.818a95.911 95.911 0 0 1-16.597-8.871l-7.567-5.612-3.568-3.077-3.412-3.25a96.003 96.003 0 0 1-6.327-6.98l-5.612-7.567a96.05 96.05 0 0 1-12.045-25.467 95.962 95.962 0 0 1-3.672-37.278 95.964 95.964 0 0 1 10.874-35.844 95.927 95.927 0 0 1 4.843-8.08l5.612-7.568 3.078-3.568 3.249-3.412a95.95 95.95 0 0 1 6.98-6.327l7.567-5.612a95.927 95.927 0 0 1 8.08-4.843l4.21-2.119L320 517.49V268.96c0-7.054 5.706-12.778 12.76-12.8l38.21-.12c7.069-.022 12.818 5.69 12.84 12.76Zm320.03-.003.16 56.694 4.737 1.817a95.912 95.912 0 0 1 16.597 8.87l7.568 5.613 3.568 3.078 3.412 3.249a95.933 95.933 0 0 1 6.327 6.98l5.612 7.567a95.974 95.974 0 0 1 15.717 43.925 95.99 95.99 0 0 1-6.846 46.147 95.878 95.878 0 0 1-8.871 16.597l-5.612 7.568-3.077 3.568-3.25 3.412a95.987 95.987 0 0 1-6.98 6.327l-7.567 5.612a96.097 96.097 0 0 1-8.08 4.843l-4.21 2.119-9.044 3.726L704 755.04c0 7.054-5.706 12.778-12.76 12.8l-38.21.12c-5.89.018-10.865-3.946-12.372-9.359l-.468-3.431-.19-248.66-4.738-1.818a95.911 95.911 0 0 1-16.597-8.871l-7.567-5.612-3.568-3.077-3.412-3.25a96.003 96.003 0 0 1-6.327-6.98l-5.612-7.567a96.05 96.05 0 0 1-12.045-25.467 95.962 95.962 0 0 1-3.672-18.458l-.346-4.7L576 416a95.951 95.951 0 0 1 4.134-27.868 95.948 95.948 0 0 1 12.045-25.467l5.612-7.567 3.078-3.568 3.249-3.412a95.95 95.95 0 0 1 6.98-6.327l7.567-5.612a95.927 95.927 0 0 1 8.08-4.843l4.21-2.119 4.307-1.91A96.002 96.002 0 0 1 640 325.49v-56.53c0-7.054 5.706-12.778 12.76-12.8l38.24-.12c7.07-.025 12.818 5.688 12.84 12.757ZM352 576a31.92 31.92 0 0 0-6.243.615 31.941 31.941 0 0 0-16.384 8.758 31.941 31.941 0 0 0-8.758 16.384A31.917 31.917 0 0 0 320 608c0 2.101.205 4.182.615 6.243a31.92 31.92 0 0 0 4.778 11.535 31.867 31.867 0 0 0 3.98 4.85 31.941 31.941 0 0 0 16.384 8.758 31.989 31.989 0 0 0 12.486-.001 31.92 31.92 0 0 0 11.535-4.778 31.867 31.867 0 0 0 4.85-3.98 31.867 31.867 0 0 0 3.979-4.85 31.92 31.92 0 0 0 2.957-5.531 31.893 31.893 0 0 0 1.821-6.003c.41-2.06.615-4.142.615-6.243s-.205-4.182-.615-6.243a31.941 31.941 0 0 0-8.758-16.384 31.867 31.867 0 0 0-4.85-3.98 31.92 31.92 0 0 0-5.531-2.957 31.893 31.893 0 0 0-6.003-1.821A31.917 31.917 0 0 0 352 576Zm320-192a31.92 31.92 0 0 0-6.243.615 31.941 31.941 0 0 0-16.384 8.758 31.941 31.941 0 0 0-8.758 16.384A31.917 31.917 0 0 0 640 416c0 2.101.205 4.182.615 6.243a31.92 31.92 0 0 0 4.778 11.535 31.867 31.867 0 0 0 3.98 4.85 31.941 31.941 0 0 0 16.384 8.758 31.989 31.989 0 0 0 12.486-.001 31.92 31.92 0 0 0 11.535-4.778 31.867 31.867 0 0 0 4.85-3.98 31.867 31.867 0 0 0 3.979-4.85 31.92 31.92 0 0 0 2.957-5.531 31.893 31.893 0 0 0 1.821-6.003c.41-2.06.615-4.142.615-6.243s-.205-4.182-.615-6.243a31.941 31.941 0 0 0-8.758-16.384 31.867 31.867 0 0 0-4.85-3.98 31.92 31.92 0 0 0-5.531-2.957 31.893 31.893 0 0 0-6.003-1.821A31.917 31.917 0 0 0 672 384Z"/></svg>'
};
const Copy = {
  name: "copy",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path fill-rule="evenodd" d="M332.8 192h38.4c7.07 0 12.8-5.73 12.8-12.8V128h448v640h-51.2c-7.072 0-12.8 5.728-12.8 12.8v38.4c0 7.072 5.728 12.8 12.8 12.8h102.4c7.072 0 12.8-5.728 12.8-12.8V76.8c0-7.07-5.728-12.8-12.8-12.8H332.8c-7.07 0-12.8 5.73-12.8 12.8v102.4c0 7.07 5.73 12.8 12.8 12.8ZM128 947.2V268.8c0-7.07 5.73-12.8 12.8-12.8h550.4c7.07 0 12.8 5.73 12.8 12.8v678.4c0 7.072-5.73 12.8-12.8 12.8H140.8c-7.07 0-12.8-5.728-12.8-12.8ZM192 320v576h448V320H192Zm64 140.8v38.4c0 7.07 5.73 12.8 12.8 12.8h294.4c7.07 0 12.8-5.73 12.8-12.8v-38.4c0-7.07-5.73-12.8-12.8-12.8H268.8c-7.07 0-12.8 5.73-12.8 12.8Zm0 192v38.4c0 7.07 5.73 12.8 12.8 12.8h294.4c7.07 0 12.8-5.73 12.8-12.8v-38.4c0-7.07-5.73-12.8-12.8-12.8H268.8c-7.07 0-12.8 5.73-12.8 12.8Z"/></svg>'
};
const Copyright = {
  name: "copyright",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><g fill-rule="evenodd"><path d="M512 64c247.424 0 448 200.577 448 448 0 247.424-200.576 448-448 448-247.423 0-448-200.576-448-448C64 264.577 264.577 64 512 64Zm0 64c-212.077 0-384 171.923-384 384s171.923 384 384 384 384-171.923 384-384-171.923-384-384-384Z"/><path d="M300.8 466.61c0-111.439 87.225-202.506 197.127-208.654l11.86-.331h18.606c101.388 0 184.266 79.256 190.058 179.222.173 6.125-4.467 11.157-10.592 11.157h-40.455c-7.07 0-12.572-5.918-13.288-12.95-6.17-60.625-55.19-108.575-116.291-113.082l-9.432-.347h-18.607c-76.736 0-139.55 59.615-144.65 135.06l-.335 9.926v92.736c0 76.737 59.616 139.55 135.059 144.652l9.926.334h16.657c67.652 0 123.078-52.35 127.977-118.777a9.894 9.894 0 0 1 9.906-9.552h41.812c7.072 0 12.678 5.805 12.21 12.859-6.367 96.493-83.903 173.543-180.605 179.143l-11.3.327h-16.657c-111.44 0-202.507-87.225-208.654-197.127l-.331-11.859v-92.736Z"/></g></svg>'
};
const CreditCard = {
  name: "credit-card",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><g fill-rule="evenodd"><path d="M947.2 192c6.97 0 12.634 5.566 12.796 12.493l.004.307v614.4c0 6.967-5.563 12.634-12.493 12.796l-.307.004H76.8c-6.967 0-12.634-5.566-12.796-12.493L64 819.2V204.8c0-6.967 5.566-12.634 12.493-12.796L76.8 192h870.4ZM896 384H128v384h768V384Zm0-128H128v64h768v-64Z"/><path d="M819.2 640H652.8c-7.07 0-12.8 5.73-12.8 12.8v38.4c0 7.07 5.73 12.8 12.8 12.8h166.4c7.072 0 12.8-5.73 12.8-12.8v-38.4c0-7.07-5.728-12.8-12.8-12.8Z"/></g></svg>'
};
const CustomerService = {
  name: "customer-service",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path fill-rule="evenodd" d="M512 128c247.424 0 448 200.577 448 448v371.2c0 7.072-5.728 12.8-12.8 12.8H716.8c-7.072 0-12.8-5.728-12.8-12.8V588.8c0-7.07 5.728-12.8 12.8-12.8H896c0-207.258-164.198-376.164-369.604-383.735L512 192c-207.258 0-376.164 164.196-383.735 369.604L128 576h179.2c7.07 0 12.8 5.73 12.8 12.8v358.4c0 7.072-5.73 12.8-12.8 12.8H76.8c-7.07 0-12.8-5.728-12.8-12.8V576c0-247.423 200.577-448 448-448ZM128 896h128V640H128v256Zm768-256H768v256h128V640Z"/></svg>'
};
const Customization = {
  name: "customization",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path d="M499.2 640c7.07 0 12.8 5.73 12.8 12.8v166.4c0 7.07-5.73 12.8-12.8 12.8H332.8c-7.07 0-12.8-5.73-12.8-12.8V768H76.8c-7.07 0-12.8-5.73-12.8-12.8v-38.4c0-7.07 5.73-12.8 12.8-12.8H320v-51.2c0-7.07 5.73-12.8 12.8-12.8h166.4Zm448 64c7.07 0 12.8 5.73 12.8 12.8v38.4c0 7.07-5.73 12.8-12.8 12.8H588.8c-7.07 0-12.8-5.73-12.8-12.8v-38.4c0-7.07 5.73-12.8 12.8-12.8h358.4ZM448 704h-64v64h64v-64Zm371.2-320c7.07 0 12.8 5.73 12.8 12.8V448h115.2c7.07 0 12.8 5.73 12.8 12.8v38.4c0 7.07-5.73 12.8-12.8 12.8H832v51.2c0 7.07-5.73 12.8-12.8 12.8H652.8c-7.07 0-12.8-5.73-12.8-12.8V396.8c0-7.07 5.73-12.8 12.8-12.8h166.4Zm-256 64c7.07 0 12.8 5.73 12.8 12.8v38.4c0 7.07-5.73 12.8-12.8 12.8H76.8c-7.07 0-12.8-5.73-12.8-12.8v-38.4c0-7.07 5.73-12.8 12.8-12.8h486.4Zm204.8 0h-64v64h64v-64ZM371.2 128c7.07 0 12.8 5.73 12.8 12.8v166.4c0 7.07-5.73 12.8-12.8 12.8H204.8c-7.07 0-12.8-5.73-12.8-12.8V256H76.8c-7.07 0-12.8-5.73-12.8-12.8v-38.4c0-7.07 5.73-12.8 12.8-12.8H192v-51.2c0-7.07 5.73-12.8 12.8-12.8h166.4Zm576 64c7.07 0 12.8 5.73 12.8 12.8v38.4c0 7.07-5.73 12.8-12.8 12.8H460.8c-7.07 0-12.8-5.73-12.8-12.8v-38.4c0-7.07 5.73-12.8 12.8-12.8h486.4ZM320 192h-64v64h64v-64Z"/></svg>'
};
const Dashboard = {
  name: "dashboard",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path fill-rule="evenodd" d="M960 576c0 123.557-50.016 235.43-130.912 316.48-2.368 2.368-5.536 3.622-8.883 3.622H203.793c-3.348 0-6.516-1.254-8.881-3.622C114.019 811.43 64 699.557 64 576c0-247.423 200.577-448 448-448 247.424 0 448 200.577 448 448ZM805.216 823.98c54.202-63.97 86.31-144.073 90.349-229.579L896 576c0-212.077-171.923-384-384-384S128 363.923 128 576c0 92.38 32.71 179.44 90.784 247.98l7.2 8.084h571.968l7.264-8.083ZM480 307.2v-38.4c0-7.07 5.73-12.8 12.8-12.8h38.4c7.07 0 12.8 5.73 12.8 12.8v38.4c0 7.07-5.73 12.8-12.8 12.8h-38.4c-7.07 0-12.8-5.73-12.8-12.8Zm-130.11 52.015-19.2-33.255c-3.535-6.122-1.438-13.95 4.684-17.485l33.256-19.2a12.8 12.8 0 0 1 17.485 4.685l19.2 33.255c3.534 6.123 1.437 13.951-4.685 17.486l-33.256 19.2a12.8 12.8 0 0 1-17.485-4.686Zm268.8-32 19.2-33.255c3.534-6.122 11.362-8.22 17.484-4.685l33.256 19.2c6.122 3.535 8.22 11.363 4.685 17.485l-19.2 33.255c-3.535 6.123-11.363 8.22-17.485 4.686l-33.256-19.2a12.802 12.802 0 0 1-4.685-17.486ZM229.955 450.111l33.256 19.2a12.797 12.797 0 0 0 17.485-4.685l19.2-33.256a12.802 12.802 0 0 0-4.685-17.485l-33.256-19.2c-6.122-3.534-13.95-1.437-17.485 4.685l-19.2 33.256a12.8 12.8 0 0 0 4.685 17.485Zm498.831-36.222 33.255-19.2c6.124-3.534 13.952-1.437 17.484 4.685l19.2 33.256a12.785 12.785 0 0 1 1.28 9.712 12.78 12.78 0 0 1-5.964 7.773l-33.255 19.2a12.79 12.79 0 0 1-9.709 1.278 12.794 12.794 0 0 1-7.776-5.963l-19.2-33.256a12.815 12.815 0 0 1-1.28-9.713 12.825 12.825 0 0 1 5.965-7.772ZM560.675 651.972l139.314-139.315c4.997-4.998 4.997-13.103 0-18.101l-27.153-27.154c-4.999-4.998-13.103-4.998-18.102 0L514.12 608.017l-1.276-.017c-26.51 0-48 21.49-48 48s21.49 48 48 48 48-21.49 48-48c0-1.345-.057-2.689-.17-4.028ZM204.8 608h38.4c7.07 0 12.8-5.73 12.8-12.8v-38.4c0-7.07-5.73-12.8-12.8-12.8h-38.4c-7.07 0-12.8 5.73-12.8 12.8v38.4c0 7.07 5.73 12.8 12.8 12.8Zm576-64h38.4c7.072 0 12.8 5.73 12.8 12.8v38.4c0 7.07-5.728 12.8-12.8 12.8h-38.4c-7.072 0-12.8-5.73-12.8-12.8v-38.4c0-7.07 5.728-12.8 12.8-12.8ZM261.956 757.31l33.256-19.2c6.122-3.534 8.219-11.362 4.685-17.484l-19.2-33.256a12.804 12.804 0 0 0-17.485-4.685l-33.256 19.2c-6.122 3.535-8.22 11.363-4.685 17.485l19.2 33.256a12.802 12.802 0 0 0 17.485 4.685Zm498.831-74.62 33.255 19.2c6.124 3.533 8.224 11.362 4.684 17.484l-19.2 33.256c-3.532 6.122-11.36 8.22-17.484 4.685l-33.255-19.2a12.825 12.825 0 0 1-5.965-7.773 12.814 12.814 0 0 1 1.28-9.712l19.2-33.256a12.794 12.794 0 0 1 7.776-5.963 12.79 12.79 0 0 1 9.71 1.278Z"/></svg>'
};
const DataCenterStar = {
  name: "data-center-star",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path fill-rule="evenodd" d="M64 883.2V76.8C64 69.73 69.73 64 76.8 64h870.4c7.072 0 12.8 5.73 12.8 12.8v358.577c0 7.07-5.562 12.623-12.634 12.623h-38.732c-7.072 0-12.634-5.649-12.634-12.718V128H704.275v307.282c0 7.07-5.562 12.718-12.631 12.718h-38.736c-7.07 0-12.632-5.649-12.632-12.718V128H383.912v704h51.438c7.07 0 12.65 5.562 12.65 12.634v38.732c0 7.072-5.554 12.634-12.623 12.634H76.8c-7.07 0-12.8-5.728-12.8-12.8ZM128 128v704h192.052V128H128Zm64 76.8v102.4c0 7.07 5.73 12.8 12.8 12.8h38.4c7.07 0 12.8-5.73 12.8-12.8V204.8c0-7.07-5.73-12.8-12.8-12.8h-38.4c-7.07 0-12.8 5.73-12.8 12.8Zm288 0v102.4c0 7.07 5.73 12.8 12.8 12.8h38.4c7.07 0 12.8-5.73 12.8-12.8V204.8c0-7.07-5.73-12.8-12.8-12.8h-38.4c-7.07 0-12.8 5.73-12.8 12.8Zm288 0v102.4c0 7.07 5.728 12.8 12.8 12.8h38.4c7.072 0 12.8-5.73 12.8-12.8V204.8c0-7.07-5.728-12.8-12.8-12.8h-38.4c-7.072 0-12.8 5.73-12.8 12.8ZM478.103 658.705c-12.399 0-17.554 15.867-7.523 23.155l137.407 99.842-52.48 161.536c-3.831 11.796 9.665 21.6 19.697 14.31l137.417-99.826 137.414 99.827c10.029 7.29 23.527-2.515 19.693-14.31l-52.48-161.537 137.408-99.842c10.029-7.288 4.877-23.155-7.52-23.155l-169.85-.006-52.492-161.535c-3.834-11.792-20.519-11.792-24.35 0L647.954 658.7l-169.85.006Z"/></svg>'
};
const DataCenter = {
  name: "data-center",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path fill-rule="evenodd" d="M947.2 64c7.07 0 12.8 5.73 12.8 12.8v806.4c0 7.07-5.73 12.8-12.8 12.8H76.8c-7.07 0-12.8-5.73-12.8-12.8V76.8C64 69.73 69.73 64 76.8 64h870.4ZM320 128H128v704h192V128Zm320 0H384v704h256V128Zm256 0H704v704h192V128Zm-76.8 64c7.07 0 12.8 5.73 12.8 12.8v102.4c0 7.07-5.73 12.8-12.8 12.8h-38.4c-7.07 0-12.8-5.73-12.8-12.8V204.8c0-7.07 5.73-12.8 12.8-12.8h38.4Zm-288 0c7.07 0 12.8 5.73 12.8 12.8v102.4c0 7.07-5.73 12.8-12.8 12.8h-38.4c-7.07 0-12.8-5.73-12.8-12.8V204.8c0-7.07 5.73-12.8 12.8-12.8h38.4Zm-288 0c7.07 0 12.8 5.73 12.8 12.8v102.4c0 7.07-5.73 12.8-12.8 12.8h-38.4c-7.07 0-12.8-5.73-12.8-12.8V204.8c0-7.07 5.73-12.8 12.8-12.8h38.4Z"/></svg>'
};
const Database = {
  name: "database",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path fill-rule="evenodd" d="m127.848 768 .378 10.246C137.019 896.243 298.255 961.363 512 961.363c219.812 0 384.001-68.89 384.001-193.363V256l-.377-10.24C886.914 127.84 727.142 64 512 64c-221.33 0-384.152 67.546-384.152 192v512ZM512 385.364c-187.914 0-320.152-55.451-320.152-129.364 0-73.71 130.866-128 320.152-128 189.25 0 320.001 54.273 320.001 128 0 73.93-132.121 129.364-320 129.364ZM831.617 518.88c-7.93 70.464-137.61 122.489-319.617 122.489-187.915 0-320.153-55.452-320.153-129.365l-.034-139.83c67.512 50.384 183.04 77.195 320.187 77.195 137.12 0 252.596-26.808 320.072-77.196l-.032 139.831-.423 6.876Zm.385 249.127-.385 6.874c-7.93 70.464-137.61 122.49-319.617 122.49-187.915 0-320.153-55.456-320.153-129.364l-.034-139.834C259.325 678.557 374.853 705.37 512 705.37c137.12 0 252.596-26.81 320.072-77.198l-.07 139.834Z"/></svg>'
};
const Delete = {
  name: "delete",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path d="M435.456 448c7.04 0 12.8 5.76 12.8 12.8v294.4a12.8 12.8 0 0 1-12.8 12.8h-38.4a12.8 12.8 0 0 1-12.8-12.8V460.8c0-7.04 5.76-12.8 12.8-12.8h38.4zm204.8 12.8v294.4a12.8 12.8 0 0 1-12.8 12.8h-38.4a12.8 12.8 0 0 1-12.8-12.8V460.8c0-7.04 5.76-12.8 12.8-12.8h38.4c7.04 0 12.8 5.76 12.8 12.8z"/><path d="M256 332.8V896h512V332.8c0-7.04 5.76-12.8 12.8-12.8h38.4c7.04 0 12.8 5.76 12.8 12.8v614.08a12.8 12.8 0 0 1-12.8 12.8H204.8a12.8 12.8 0 0 1-12.8-12.8V332.8c0-7.04 5.76-12.8 12.8-12.8h38.4c7.04 0 12.8 5.76 12.8 12.8zM64 192h896v64H64z"/><path d="M755.2 64c7.04 0 12.8 5.76 12.8 12.8V256h-63.552V128h-383.68v128H256V76.8c0-7.04 5.76-12.8 12.8-12.8h486.4z"/></svg>'
};
const DeliveredProcedure = {
  name: "delivered-procedure",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path d="M703.488 64v192l192.512.384V371.2a12.8 12.8 0 0 1-12.8 12.8h-38.4a12.8 12.8 0 0 1-12.8-12.8v-51.264H653.504a12.8 12.8 0 0 1-12.8-12.8V123.52L192 127.616v768.32h177.6c7.04.064 12.8 5.76 12.8 12.864v38.4a12.8 12.8 0 0 1-12.8 12.8H140.8a12.8 12.8 0 0 1-12.8-12.8V76.8c0-7.04 5.76-12.8 12.8-12.8h562.688z"/><path d="M703.488 64 896 256.384l-45.248 45.248L658.24 109.248zM268.8 448h102.4q12.8 0 12.8 12.8v38.4q0 12.8-12.8 12.8H268.8q-12.8 0-12.8-12.8v-38.4q0-12.8 12.8-12.8Zm0 128h38.4q12.8 0 12.8 12.8v38.4q0 12.8-12.8 12.8h-38.4q-12.8 0-12.8-12.8v-38.4q0-12.8 12.8-12.8Zm0 128h38.4q12.8 0 12.8 12.8v38.4q0 12.8-12.8 12.8h-38.4q-12.8 0-12.8-12.8v-38.4q0-12.8 12.8-12.8ZM672 384a288 288 0 1 1 0 576 288 288 0 0 1 0-576zm0 64a224 224 0 1 0 0 448 224 224 0 0 0 0-448z"/><path d="M769.28 652.8v38.4a12.8 12.8 0 0 1-12.8 12.8h-230.4a12.8 12.8 0 0 1-12.8-12.8v-38.4c0-7.04 5.696-12.8 12.8-12.8h230.4c7.04 0 12.8 5.76 12.8 12.8z"/><path d="m637.44 776.064 94.976-95.04a12.8 12.8 0 0 0 0-18.048l-94.976-95.04a12.8 12.8 0 0 1 0-18.112l27.2-27.136a12.8 12.8 0 0 1 18.112 0L823.04 662.976a12.8 12.8 0 0 1 0 18.048L682.752 821.312a12.8 12.8 0 0 1-18.112 0l-27.2-27.136a12.8 12.8 0 0 1 0-18.112z"/></svg>'
};
const DesktopStar = {
  name: "desktop-star",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path fill-rule="evenodd" d="M896 128v371.2c0 7.07 5.728 12.8 12.8 12.8h38.4c7.072 0 12.8-5.73 12.8-12.8V76.8c0-7.07-5.728-12.8-12.8-12.8H76.8C69.73 64 64 69.73 64 76.8v614.4c0 7.072 5.73 12.8 12.8 12.8H384v128h-51.2c-7.07 0-12.8 5.728-12.8 12.8v38.4c0 7.072 5.73 12.8 12.8 12.8h102.4c7.07 0 12.8-5.728 12.8-12.8V652.8c0-7.07-5.73-12.8-12.8-12.8H128v-64h307.2c7.07 0 12.8-5.73 12.8-12.8v-38.4c0-7.07-5.73-12.8-12.8-12.8H128V128h768ZM478.103 658.705c-12.399 0-17.554 15.867-7.523 23.155l137.407 99.842-52.48 161.536c-3.831 11.796 9.665 21.6 19.697 14.31l137.417-99.826 137.414 99.827c10.029 7.29 23.527-2.515 19.693-14.31l-52.48-161.537 137.408-99.842c10.029-7.288 4.877-23.155-7.52-23.155l-169.85-.006-52.492-161.535c-3.834-11.792-20.519-11.792-24.35 0L647.954 658.7l-169.85.006Z"/></svg>'
};
const Desktop = {
  name: "desktop",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path fill-rule="evenodd" d="M947.2 64c6.97 0 12.634 5.566 12.796 12.493l.004.307v614.4c0 6.967-5.563 12.634-12.493 12.796l-.307.004H640v128h51.2c6.967 0 12.634 5.563 12.796 12.493l.004.307v38.4c0 6.97-5.566 12.634-12.493 12.796l-.307.004H332.8c-6.967 0-12.634-5.563-12.796-12.493L320 883.2v-38.4c0-6.97 5.566-12.634 12.493-12.796l.307-.004H384V704H76.8c-6.967 0-12.634-5.566-12.796-12.493L64 691.2V76.8c0-6.967 5.566-12.634 12.493-12.796L76.8 64h870.4ZM576 704H448v128h128V704Zm320-128H128v64h768v-64Zm0-448H128v384h768V128Z"/></svg>'
};
const DialogClose = {
  name: "dialog-close",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path fill-rule="evenodd" d="M916.741 210.766c4.999-4.999 4.999-13.103 0-18.102l-45.255-45.255c-4.999-4.999-13.103-4.999-18.102 0L532.073 468.721 210.766 147.414c-4.999-5-13.103-5-18.102 0l-45.255 45.254c-4.999 5-4.999 13.104 0 18.102l321.307 321.307-321.307 321.307c-5 5.001-4.999 13.103 0 18.102l45.255 45.255c4.999 4.999 13.101 5 18.102 0l321.307-321.307 321.311 321.312c5.001 5 13.103 4.999 18.102 0l45.255-45.255c4.999-4.999 5-13.101 0-18.102L595.43 532.077l321.31-321.311Z"/></svg>'
};
const Disconnect = {
  name: "disconnect",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path fill-rule="evenodd" d="m435.382 213.123-38.396.003c-7.07 0-12.802-5.74-12.802-12.808V97.92c0-7.07 5.73-12.8 12.799-12.8l38.402.003c7.07 0 12.794 5.725 12.794 12.794v102.398c0 7.07-5.727 12.808-12.797 12.808Zm-5.223 128.33 27.152 27.154c5 4.999 13.104 4.999 18.103 0L647.382 196.64c24.993-24.994 55.164-37.491 90.511-37.491 35.345 0 65.515 12.497 90.509 37.49 24.994 24.994 37.491 55.164 37.491 90.51 0 35.346-12.497 65.516-37.491 90.51l-171.97 171.968c-4.998 5-4.998 13.103 0 18.102l27.154 27.153c4.998 4.998 13.103 4.998 18.102 0l171.968-171.969c74.983-74.98 74.983-196.547 0-271.529-74.982-74.98-196.548-74.98-271.529 0L430.16 323.353c-5 4.998-5 13.102 0 18.1Zm-131.692 2.263L180.804 226.053c-4.999-4.998-4.999-13.103 0-18.102l27.153-27.152c4.999-5 13.103-5 18.102 0L343.721 298.46c5 4.999 5 13.103 0 18.102l-27.152 27.153c-5 4.998-13.104 4.999-18.102 0Zm-85.341 53.16-.007 38.39c0 7.068-5.727 12.805-12.796 12.805H97.927c-7.07 0-12.806-5.736-12.806-12.806l-.001-38.404c0-7.069 5.733-12.79 12.802-12.79H200.32c7.07 0 12.806 5.735 12.806 12.804Zm108.42 35.087L149.58 603.932c-74.981 74.98-74.981 196.545 0 271.528 74.98 74.982 196.548 74.982 271.529 0l171.968-171.968c4.999-4.998 4.999-13.102 0-18.101l-27.153-27.153c-4.999-5-13.103-5-18.102 0L375.853 830.205c-24.994 24.994-55.164 37.491-90.51 37.491-35.346 0-65.516-12.497-90.51-37.49-24.993-24.995-37.49-55.164-37.49-90.51 0-35.346 12.497-65.516 37.49-90.509l171.969-171.969c4.999-4.998 4.999-13.103 0-18.101l-27.153-27.154c-4.999-4.998-13.103-4.998-18.102 0Zm487.662 156.898v38.396c0 7.07 5.735 12.798 12.8 12.798h10.17l64 .006 28.23-.006c7.066 0 12.8-5.73 12.8-12.8V588.86c0-7.07-5.728-12.8-12.8-12.8l-102.406-.002c-7.066 0-12.794 5.734-12.794 12.804ZM678.606 723.859 796.267 841.52c4.999 4.999 13.108 4.999 18.106 0l27.149-27.148c4.998-5.005 4.998-13.108 0-18.106L723.86 678.604c-5-4.998-13.103-4.998-18.102 0l-27.153 27.153c-4.998 4.999-4.998 13.103 0 18.102Zm-51.235 85.335h-38.395c-7.069 0-12.803 5.722-12.803 12.794v102.4c0 7.065 5.734 12.806 12.803 12.806l38.395-.006c7.07 0 12.804-5.735 12.804-12.8v-102.4c0-7.066-5.734-12.794-12.804-12.794Z"/></svg>'
};
const DislikeFilled = {
  name: "dislike-filled",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path d="M256 608V128H140.8a12.8 12.8 0 0 0-12.8 12.8v454.4c0 7.04 5.76 12.8 12.8 12.8H256zm352 288a64 64 0 0 0 64-64V652.8c0-7.04 5.76-12.8 12.8-12.8h168.64a96 96 0 0 0 89.92-129.728l-120-320A96 96 0 0 0 733.44 128H320v480.512a128 128 0 0 1 100.608 65.28l92.288 166.272C532.096 874.624 568.512 896 608 896z"/></svg>'
};
const Dislike = {
  name: "dislike",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path d="M140.8 128a12.8 12.8 0 0 0-12.8 12.8v454.4c0 7.04 5.76 12.8 12.8 12.8h150.912a128 128 0 0 1 121.472 87.552l36.48 109.376A133.184 133.184 0 0 0 576 896a96 96 0 0 0 96-96V652.8c0-7.04 5.76-12.8 12.8-12.8H832a110.912 110.912 0 0 0 103.808-149.76l-104.64-279.168A128 128 0 0 0 711.296 128H140.8zM256 544h-64V192h64v352zm320 288c-27.072 0-51.392-15.744-62.72-39.872l-2.944-7.424-36.48-109.44a192 192 0 0 0-153.792-129.152L320 192h391.296A64 64 0 0 1 768 226.304l3.2 7.232L875.904 512.64a46.912 46.912 0 0 1-37.568 62.912L832 576H684.8a76.8 76.8 0 0 0-76.416 68.928L608 652.8V800a32 32 0 0 1-32 32z"/></svg>'
};
const Dns = {
  name: "dns",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path fill-rule="evenodd" d="M64 179.32V76.8C64 69.73 69.73 64 76.8 64h870.4c7.072 0 12.8 5.73 12.8 12.8v102.52c0 7.07-5.562 12.68-12.634 12.68h-38.732c-7.072 0-12.634-5.563-12.634-12.632V128H128v51.368c0 7.07-5.563 12.632-12.632 12.632H76.632C69.562 192 64 186.39 64 179.32Zm371.39 524.565h-38.4c-7.068 0-12.799-5.73-12.8-12.798l-.06-422.285c0-7.07 5.73-12.802 12.8-12.802h45.437a12.8 12.8 0 0 1 11.77 7.77l121.288 283.814.033-278.504c.001-7.068 5.732-12.798 12.8-12.798h38.4c7.07 0 12.8 5.73 12.8 12.8l-.032 422.12c0 7.068-5.731 12.798-12.8 12.798h-45.503a12.8 12.8 0 0 1-11.77-7.77L448.128 412.544l.06 278.538c.002 7.07-5.73 12.803-12.8 12.803Zm-114.404-159.04V416.007c0-88.366-71.634-160-160-160H76.8c-7.07 0-12.8 5.73-12.8 12.8v423.236c0 7.07 5.73 12.8 12.8 12.8h84.186c88.366 0 160-71.634 160-160ZM704 384.425c0-67.79 52.3-123.172 118.643-128.066l9.568-.352H915.2c7.072 0 12.8 5.73 12.8 12.8v38.4c0 7.07-5.728 12.8-12.8 12.8h-82.989c-32.928 0-60.07 24.787-63.776 56.721l-.435 7.488c0 33.016 24.704 60.08 56.538 63.779l7.462.43c67.482 0 122.765 52.217 127.648 118.448l.352 9.552c0 67.789-52.3 123.172-118.643 128.067l-9.568.352H748.8c-7.072 0-12.8-5.731-12.8-12.8v-38.4c0-7.07 5.728-12.8 12.8-12.8h82.989c32.928 0 60.07-24.788 63.776-56.721l.435-7.488c0-35.556-28.653-64.21-64-64.21-67.482 0-122.765-52.216-127.648-118.447l-.352-9.553ZM256.988 416.01v128.836c0 53.02-42.981 96-96 96l-33.05-.013V320l33.05.009c53.019 0 96 42.98 96 96ZM128 832h768v-51.2c0-7.072 5.728-12.8 12.8-12.8h38.4c7.072 0 12.8 5.728 12.8 12.8v102.4c0 7.072-5.728 12.8-12.8 12.8H76.8c-7.07 0-12.8-5.728-12.8-12.8V780.8c0-7.072 5.73-12.8 12.8-12.8h38.4c7.07 0 12.8 5.728 12.8 12.8V832Z"/></svg>'
};
const DotChart = {
  name: "dot-chart",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path d="M115.2 128c7.04 0 12.8 5.76 12.8 12.8V832h819.2c7.04 0 12.8 5.76 12.8 12.8v38.4a12.8 12.8 0 0 1-12.8 12.8H76.8A12.8 12.8 0 0 1 64 883.2V140.8c0-7.04 5.76-12.8 12.8-12.8h38.4z"/><path d="M192 704a64 64 0 1 0 128 0 64 64 0 1 0-128 0Zm192-128a64 64 0 1 0 128 0 64 64 0 1 0-128 0Zm256 32a96 96 0 1 0 192 0 96 96 0 1 0-192 0Zm320-288a64 64 0 1 0-128 0 64 64 0 1 0 128 0Z"/></svg>'
};
const DoubleLeft = {
  name: "double-left",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path fill-rule="evenodd" d="M502.936 774.465 249.51 521.038c-4.999-4.999-4.999-13.103 0-18.102L502.936 249.51c4.999-4.999 13.103-4.999 18.102 0l27.153 27.153c4.999 4.999 4.999 13.103 0 18.102L330.968 511.987 548.19 729.21c4.999 5 4.999 13.104 0 18.102l-27.153 27.153c-4.999 4.999-13.103 4.999-18.102 0Zm256 0L505.51 521.038c-4.999-4.999-4.999-13.103 0-18.102L758.936 249.51c4.999-4.999 13.103-4.999 18.102 0l27.153 27.153c4.999 4.999 4.999 13.103 0 18.102L586.968 511.987 804.19 729.21c4.999 5 4.999 13.104 0 18.102l-27.153 27.153c-4.999 4.999-13.103 4.999-18.102 0Z"/></svg>'
};
const DoubleRight = {
  name: "double-right",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path fill-rule="evenodd" d="M521.324 774.465 774.75 521.038c4.999-4.999 4.999-13.103 0-18.102L521.324 249.51c-4.999-4.999-13.103-4.999-18.102 0l-27.153 27.153c-4.999 4.999-4.999 13.103 0 18.102l217.223 217.223L476.07 729.21c-4.999 5-4.999 13.104 0 18.102l27.153 27.153c4.999 4.999 13.103 4.999 18.102 0Zm-256 0L518.75 521.038c4.999-4.999 4.999-13.103 0-18.102L265.324 249.51c-4.999-4.999-13.103-4.999-18.102 0l-27.153 27.153c-4.999 4.999-4.999 13.103 0 18.102l217.223 217.223L220.07 729.21c-4.999 5-4.999 13.104 0 18.102l27.153 27.153c4.999 4.999 13.103 4.999 18.102 0Z"/></svg>'
};
const DownCircle = {
  name: "down-circle",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path fill-rule="evenodd" d="M512 64a447.92 447.92 0 0 1 108.855 13.426 447.914 447.914 0 0 1 82.69 29.587 449.718 449.718 0 0 1 19.642 9.886 449.323 449.323 0 0 1 19.13 10.839l18.579 11.763a446.428 446.428 0 0 1 17.978 12.662 450.49 450.49 0 0 1 17.337 13.529 447.983 447.983 0 0 1 32.57 29.524 446.991 446.991 0 0 1 29.53 32.576 449.987 449.987 0 0 1 13.529 17.334 452.297 452.297 0 0 1 12.66 17.979 444.67 444.67 0 0 1 11.762 18.577 449.584 449.584 0 0 1 10.836 19.132 440.259 440.259 0 0 1 9.888 19.641 448.72 448.72 0 0 1 8.915 20.103 451.431 451.431 0 0 1 7.91 20.515 448.495 448.495 0 0 1 6.9 20.88 444.284 444.284 0 0 1 5.862 21.192 444.297 444.297 0 0 1 4.819 21.455 448.469 448.469 0 0 1 3.757 21.665 451.722 451.722 0 0 1 2.694 21.823 453.892 453.892 0 0 1 1.62 21.93 449.611 449.611 0 0 1 0 43.964 453.892 453.892 0 0 1-1.62 21.93 451.722 451.722 0 0 1-2.694 21.823 448.469 448.469 0 0 1-3.757 21.665 444.297 444.297 0 0 1-4.82 21.455 444.284 444.284 0 0 1-5.862 21.192 448.495 448.495 0 0 1-6.899 20.88 451.431 451.431 0 0 1-7.91 20.515 448.48 448.48 0 0 1-8.915 20.102 441.948 441.948 0 0 1-9.888 19.643 451.08 451.08 0 0 1-10.836 19.13 445.017 445.017 0 0 1-11.763 18.579 450.722 450.722 0 0 1-12.659 17.978 447.565 447.565 0 0 1-27.891 33.984 445.11 445.11 0 0 1-15.168 15.923 445.11 445.11 0 0 1-15.923 15.168 447.565 447.565 0 0 1-33.984 27.891 450.722 450.722 0 0 1-17.978 12.66 445.017 445.017 0 0 1-18.58 11.762 451.08 451.08 0 0 1-19.129 10.836 441.948 441.948 0 0 1-19.643 9.888 448.48 448.48 0 0 1-20.102 8.915 451.431 451.431 0 0 1-20.515 7.91 448.495 448.495 0 0 1-20.88 6.9 444.284 444.284 0 0 1-21.192 5.862 444.297 444.297 0 0 1-21.455 4.819 448.469 448.469 0 0 1-21.665 3.757 451.722 451.722 0 0 1-21.823 2.694 453.892 453.892 0 0 1-21.93 1.62 449.611 449.611 0 0 1-43.964 0 453.892 453.892 0 0 1-21.93-1.62 451.722 451.722 0 0 1-21.823-2.694 448.469 448.469 0 0 1-21.665-3.757 444.297 444.297 0 0 1-21.455-4.82 444.284 444.284 0 0 1-21.192-5.862 448.495 448.495 0 0 1-20.88-6.899 451.431 451.431 0 0 1-20.515-7.91 448.72 448.72 0 0 1-20.103-8.915 440.259 440.259 0 0 1-19.64-9.888 449.584 449.584 0 0 1-19.133-10.836 444.67 444.67 0 0 1-18.577-11.763 452.297 452.297 0 0 1-17.979-12.659 449.987 449.987 0 0 1-17.334-13.53 446.991 446.991 0 0 1-32.576-29.53 448.39 448.39 0 0 1-29.524-32.569 450.49 450.49 0 0 1-13.529-17.337 446.428 446.428 0 0 1-12.662-17.978l-11.763-18.58a449.369 449.369 0 0 1-10.839-19.129 447.922 447.922 0 0 1-48.05-145.452 447.94 447.94 0 0 1 0-131.47 447.993 447.993 0 0 1 58.889-164.583l11.763-18.577A447.973 447.973 0 0 1 263.105 139.5l18.577-11.763a447.884 447.884 0 0 1 100.27-44.447 447.914 447.914 0 0 1 108.067-18.75c7.323-.36 14.65-.54 21.982-.54Zm9.424 64.116L512 128a383.817 383.817 0 0 0-37.638 1.849l-18.706 2.307a383.544 383.544 0 0 0-36.96 7.352 383.802 383.802 0 0 0-36.062 10.94l-17.584 6.782-8.663 3.713-8.568 3.925a384.311 384.311 0 0 0-49.158 27.848 384.064 384.064 0 0 0-30.268 22.448l-7.211 6.068-13.966 12.657-13.327 13.327-6.414 6.906a383.817 383.817 0 0 0-34.76 44.54 384.048 384.048 0 0 0-19.373 32.322l-4.34 8.365-4.134 8.47a384.382 384.382 0 0 0-7.638 17.23l-6.783 17.585a383.126 383.126 0 0 0-10.94 36.062 383.666 383.666 0 0 0-7.35 36.96l-2.308 18.706a383.393 383.393 0 0 0-1.386 18.796l-.347 9.418L128 512a383.817 383.817 0 0 0 1.849 37.638l2.307 18.706a383.544 383.544 0 0 0 7.352 36.96 383.802 383.802 0 0 0 10.94 36.062l6.782 17.584 3.713 8.663 3.925 8.568a384.311 384.311 0 0 0 17.764 33.233 380.11 380.11 0 0 0 10.084 15.924 382.888 382.888 0 0 0 10.852 15.41 385.989 385.989 0 0 0 11.596 14.862l6.068 7.21 12.657 13.964 13.327 13.326 6.906 6.415a386.736 386.736 0 0 0 14.271 12.313 384.027 384.027 0 0 0 14.858 11.59 380.24 380.24 0 0 0 15.41 10.855 389.12 389.12 0 0 0 15.923 10.087 383.517 383.517 0 0 0 16.4 9.286l8.365 4.342 8.47 4.132a384.445 384.445 0 0 0 17.23 7.641l17.585 6.784a382.06 382.06 0 0 0 17.897 5.907 382.31 382.31 0 0 0 36.554 9.159 390.475 390.475 0 0 0 18.57 3.225l18.707 2.304c6.254.615 12.52 1.078 18.796 1.39a385.413 385.413 0 0 0 37.684 0 389.845 389.845 0 0 0 18.796-1.39l18.706-2.304a390.475 390.475 0 0 0 18.57-3.225 381.596 381.596 0 0 0 36.555-9.159 382.06 382.06 0 0 0 17.897-5.907l17.584-6.784 8.663-3.715 8.568-3.926a377.303 377.303 0 0 0 16.835-8.474 382.95 382.95 0 0 0 16.398-9.286 385.076 385.076 0 0 0 31.335-20.941 385.42 385.42 0 0 0 14.86-11.59l7.21-6.071 13.965-12.658 13.326-13.326 6.415-6.902c4.22-4.66 8.324-9.417 12.313-14.272a385.42 385.42 0 0 0 11.59-14.861 385.076 385.076 0 0 0 20.94-31.334 382.95 382.95 0 0 0 9.288-16.399l4.342-8.365 4.132-8.47a384.445 384.445 0 0 0 7.641-17.23l6.784-17.585a382.06 382.06 0 0 0 5.907-17.897 382.31 382.31 0 0 0 9.159-36.554 390.475 390.475 0 0 0 3.225-18.57l2.304-18.707a389.845 389.845 0 0 0 1.39-18.796 385.413 385.413 0 0 0 0-37.684 389.845 389.845 0 0 0-1.39-18.796l-2.304-18.706a390.475 390.475 0 0 0-3.225-18.57 381.596 381.596 0 0 0-9.159-36.555 382.06 382.06 0 0 0-5.907-17.897l-6.784-17.584-3.715-8.663-3.926-8.568a377.303 377.303 0 0 0-8.474-16.835 383.517 383.517 0 0 0-9.286-16.4 389.12 389.12 0 0 0-10.087-15.923 380.24 380.24 0 0 0-10.854-15.41 385.53 385.53 0 0 0-11.59-14.858l-6.071-7.211-12.658-13.966-13.326-13.327-6.902-6.414a388.396 388.396 0 0 0-14.272-12.31 385.989 385.989 0 0 0-14.861-11.597 382.888 382.888 0 0 0-15.411-10.852 383.949 383.949 0 0 0-32.321-19.373l-8.366-4.341-8.47-4.134a384.382 384.382 0 0 0-17.23-7.638l-17.585-6.783a383.126 383.126 0 0 0-36.062-10.94 383.666 383.666 0 0 0-36.96-7.35l-18.706-2.308a383.393 383.393 0 0 0-18.796-1.386l-9.418-.347Zm-255.61 261.736c3.998-3.998 9.972-4.66 14.761-2.207l3.32 2.42L512 618.085 736.356 393.66c3.999-4 9.876-4.844 14.627-2.462l3.304 2.392 27.39 27.39c5 5 4.76 13.105-.238 18.104L521.09 699.432c-4 4-10.023 4.916-14.832 2.563l-3.328-2.38-264.45-264.452c-5-4.999-5.055-12.922-.057-17.92l27.39-27.39Z"/></svg>'
};
const Down = {
  name: "down",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path d="M503.234 694.123 249.575 440.464c-4.998-4.998-5.001-12.869-.003-17.867l27.39-27.39c5-5 12.927-4.939 17.926.06l217.281 217.281 217.281-217.28c4.999-5 12.927-5.06 17.925-.062l27.39 27.39c5 5 4.996 12.87-.002 17.868L521.104 694.123c-4.998 4.998-12.872 4.998-17.87 0Z"/></svg>'
};
const Download = {
  name: "download",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path d="M128 780.8v102.4c0 7.04 5.76 12.8 12.8 12.8h742.4a12.8 12.8 0 0 0 12.8-12.8V780.8c0-7.04 5.76-12.8 12.8-12.8h38.4c7.04 0 12.8 5.76 12.8 12.8v166.4a12.8 12.8 0 0 1-12.8 12.8H76.8A12.8 12.8 0 0 1 64 947.2V780.8c0-7.04 5.76-12.8 12.8-12.8h38.4c7.04 0 12.8 5.76 12.8 12.8z"/><path d="M627.2 128c7.04 0 12.8 5.76 12.8 12.8V512h161.088a12.8 12.8 0 0 1 9.088 21.824L520.96 823.04a12.8 12.8 0 0 1-18.048 0L213.76 533.76a12.8 12.8 0 0 1 9.152-21.76L384 511.936V140.8c0-7.04 5.76-12.8 12.8-12.8h230.4zm-51.264 63.936h-128v384l-101.568-.064L512 741.504l165.504-165.568L576 576l-.064-384.064z"/></svg>'
};
const Edit = {
  name: "edit",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path d="M910.208 276.672a12.8 12.8 0 0 1 2.112 15.36l-2.176 2.752-512.32 508.992-245.632 83.968a12.8 12.8 0 0 1-16.256-16.064l80.832-248.96 512.448-508.992a12.8 12.8 0 0 1 18.048 0l162.944 162.944zm-81.6 8.832-90.496-90.496-465.28 462.208-44.352 136.768 134.72-46.08 465.408-462.4zM960 908.8v38.4a12.8 12.8 0 0 1-12.8 12.8H76.8A12.8 12.8 0 0 1 64 947.2v-38.4c0-7.04 5.76-12.8 12.8-12.8h870.4c7.04 0 12.8 5.76 12.8 12.8z"/></svg>'
};
const EllipsisVertical = {
  name: "ellipsis-vertical",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path d="M576 768c0 4.202-.41 8.364-1.23 12.486a63.86 63.86 0 0 1-3.642 12.006 63.829 63.829 0 0 1-5.914 11.064 63.878 63.878 0 0 1-7.959 9.699 63.878 63.878 0 0 1-9.699 7.96 63.829 63.829 0 0 1-11.064 5.913 63.827 63.827 0 0 1-12.006 3.642A63.83 63.83 0 0 1 512 832a63.83 63.83 0 0 1-12.486-1.23 63.835 63.835 0 0 1-12.006-3.642 63.839 63.839 0 0 1-11.064-5.914 63.857 63.857 0 0 1-9.699-7.959 63.866 63.866 0 0 1-7.96-9.699 63.833 63.833 0 0 1-5.913-11.064 63.839 63.839 0 0 1-3.642-12.006A63.847 63.847 0 0 1 448 768c0-4.202.41-8.364 1.23-12.486a63.839 63.839 0 0 1 3.642-12.006 63.833 63.833 0 0 1 5.914-11.064 63.866 63.866 0 0 1 7.96-9.699 63.857 63.857 0 0 1 9.698-7.96 63.839 63.839 0 0 1 11.064-5.913 63.868 63.868 0 0 1 12.006-3.643A63.861 63.861 0 0 1 512 704c4.202 0 8.364.41 12.486 1.23a63.86 63.86 0 0 1 12.006 3.642 63.829 63.829 0 0 1 11.064 5.914 63.878 63.878 0 0 1 9.699 7.959 63.878 63.878 0 0 1 7.96 9.699 63.829 63.829 0 0 1 5.913 11.064 63.86 63.86 0 0 1 3.643 12.006A63.862 63.862 0 0 1 576 768Zm0-512c0 4.202-.41 8.364-1.23 12.486a63.86 63.86 0 0 1-3.642 12.006 63.829 63.829 0 0 1-5.914 11.064 63.878 63.878 0 0 1-7.959 9.699 63.878 63.878 0 0 1-9.699 7.96 63.829 63.829 0 0 1-11.064 5.913 63.86 63.86 0 0 1-12.006 3.643A63.862 63.862 0 0 1 512 320c-4.202 0-8.364-.41-12.486-1.23a63.868 63.868 0 0 1-12.006-3.642 63.839 63.839 0 0 1-11.064-5.914 63.857 63.857 0 0 1-9.699-7.959 63.866 63.866 0 0 1-7.96-9.699 63.833 63.833 0 0 1-5.913-11.064 63.839 63.839 0 0 1-3.642-12.006A63.847 63.847 0 0 1 448 256c0-4.202.41-8.364 1.23-12.486a63.847 63.847 0 0 1 3.642-12.006 63.843 63.843 0 0 1 5.914-11.064 63.846 63.846 0 0 1 7.96-9.699 63.846 63.846 0 0 1 9.698-7.96 63.843 63.843 0 0 1 11.064-5.913 63.847 63.847 0 0 1 12.006-3.642A63.846 63.846 0 0 1 512 192c4.202 0 8.364.41 12.486 1.23a63.839 63.839 0 0 1 12.006 3.642 63.833 63.833 0 0 1 11.064 5.914 63.866 63.866 0 0 1 9.699 7.96 63.857 63.857 0 0 1 7.96 9.698 63.839 63.839 0 0 1 5.913 11.064 63.868 63.868 0 0 1 3.643 12.006A63.861 63.861 0 0 1 576 256Zm0 256c0 4.202-.41 8.364-1.23 12.486a63.86 63.86 0 0 1-3.642 12.006 63.829 63.829 0 0 1-5.914 11.064 63.878 63.878 0 0 1-7.959 9.699 63.878 63.878 0 0 1-9.699 7.96 63.829 63.829 0 0 1-11.064 5.913 63.827 63.827 0 0 1-12.006 3.642A63.83 63.83 0 0 1 512 576a63.83 63.83 0 0 1-12.486-1.23 63.835 63.835 0 0 1-12.006-3.642 63.839 63.839 0 0 1-11.064-5.914 63.857 63.857 0 0 1-9.699-7.959 63.866 63.866 0 0 1-7.96-9.699 63.833 63.833 0 0 1-5.913-11.064 63.839 63.839 0 0 1-3.642-12.006A63.847 63.847 0 0 1 448 512c0-4.202.41-8.364 1.23-12.486a63.839 63.839 0 0 1 3.642-12.006 63.833 63.833 0 0 1 5.914-11.064 63.866 63.866 0 0 1 7.96-9.699 63.857 63.857 0 0 1 9.698-7.96 63.839 63.839 0 0 1 11.064-5.913 63.868 63.868 0 0 1 12.006-3.643A63.861 63.861 0 0 1 512 448c4.202 0 8.364.41 12.486 1.23a63.86 63.86 0 0 1 12.006 3.642 63.829 63.829 0 0 1 11.064 5.914 63.878 63.878 0 0 1 9.699 7.959 63.878 63.878 0 0 1 7.96 9.699 63.829 63.829 0 0 1 5.913 11.064 63.86 63.86 0 0 1 3.643 12.006A63.862 63.862 0 0 1 576 512Z"/></svg>'
};
const Ellipsis = {
  name: "ellipsis",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><defs><style/></defs><path d="M768 448a64 64 0 1 0 0 128 64 64 0 1 0 0-128Zm-256 0a64 64 0 1 0 0 128 64 64 0 1 0 0-128Zm-256 0a64 64 0 1 0 0 128 64 64 0 1 0 0-128Z"/></svg>'
};
const Empty = {
  name: "empty",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path fill-rule="evenodd" d="M256 192V76.8c0-7.07 5.73-12.8 12.8-12.8h486.4c7.072 0 12.8 5.73 12.8 12.8V192h179.2c7.072 0 12.8 5.73 12.8 12.8v38.4c0 7.07-5.728 12.8-12.8 12.8H76.8c-7.07 0-12.8-5.73-12.8-12.8v-38.4c0-7.07 5.73-12.8 12.8-12.8H256Zm64.749 0h383.673v-64H320.75v64ZM204.8 959.693h614.4c7.072 0 12.8-5.69 12.8-12.755V332.893c0-7.068-5.562-12.893-12.634-12.893h-38.732c-7.072 0-12.634 5.562-12.634 12.631V896H256V332.631c0-7.069-5.563-12.631-12.632-12.631h-38.736c-7.07 0-12.632 5.825-12.632 12.893v614.045c0 7.065 5.73 12.755 12.8 12.755Z"/></svg>'
};
const Environment = {
  name: "environment",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path d="M512 64a384 384 0 0 1 286.208 640l-277.76 248.512a12.8 12.8 0 0 1-16.896 0L225.792 704A384 384 0 0 1 512 64zm0 64a320 320 0 0 0-249.344 520.576l10.816 12.736 12.736 14.272L511.872 874.56l200.576-177.088c4.992-4.032 9.792-8.192 15.104-12.992l8.384-7.808 4.032-3.648 10.56-11.712a318.336 318.336 0 0 0 81.024-196.288L832 448a320 320 0 0 0-320-320z"/><path d="M512 256a128 128 0 1 1 0 256 128 128 0 0 1 0-256zm0 64a64 64 0 1 0 0 128 64 64 0 0 0 0-128z"/></svg>'
};
const Exception = {
  name: "exception",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path d="M703.488 64v192l192.512.384V371.2a12.8 12.8 0 0 1-12.8 12.8h-38.4a12.8 12.8 0 0 1-12.8-12.8v-51.264H653.504a12.8 12.8 0 0 1-12.8-12.8V123.52L192 127.616v768.32h177.6c7.04.064 12.8 5.76 12.8 12.864v38.4a12.8 12.8 0 0 1-12.8 12.8H140.8a12.8 12.8 0 0 1-12.8-12.8V76.8c0-7.04 5.76-12.8 12.8-12.8h562.688z"/><path d="M703.488 64 896 256.384l-45.248 45.248L658.24 109.248zM268.8 448h102.4q12.8 0 12.8 12.8v38.4q0 12.8-12.8 12.8H268.8q-12.8 0-12.8-12.8v-38.4q0-12.8 12.8-12.8Zm0 128h38.4q12.8 0 12.8 12.8v38.4q0 12.8-12.8 12.8h-38.4q-12.8 0-12.8-12.8v-38.4q0-12.8 12.8-12.8Zm0 128h38.4q12.8 0 12.8 12.8v38.4q0 12.8-12.8 12.8h-38.4q-12.8 0-12.8-12.8v-38.4q0-12.8 12.8-12.8ZM672 384a288 288 0 1 1 0 576 288 288 0 0 1 0-576zm0 64a224 224 0 1 0 0 448 224 224 0 0 0 0-448z"/><path d="M652.8 512h38.4c7.04 0 12.8 5.76 12.8 12.8v198.4a12.8 12.8 0 0 1-12.8 12.8h-38.4a12.8 12.8 0 0 1-12.8-12.8V524.8c0-7.04 5.76-12.8 12.8-12.8zM672 768q48 0 48 48t-48 48q-48 0-48-48t48-48Z"/></svg>'
};
const ExclamationCircleFilled = {
  name: "exclamation-circle-filled",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path d="M512 1024c282.752 0 512-229.248 512-512S794.752 0 512 0 0 229.248 0 512s229.248 512 512 512zm25.6-405.333h-51.2a17.067 17.067 0 0 1-17.067-17.067V251.733c0-9.386 7.68-17.066 17.067-17.066h51.2c9.387 0 17.067 7.68 17.067 17.066V601.6a17.067 17.067 0 0 1-17.067 17.067zm0 170.666h-51.2a17.067 17.067 0 0 1-17.067-17.066v-51.2c0-9.387 7.68-17.067 17.067-17.067h51.2c9.387 0 17.067 7.68 17.067 17.067v51.2a17.067 17.067 0 0 1-17.067 17.066z"/></svg>'
};
const ExclamationCircle = {
  name: "exclamation-circle",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path d="M512 64c247.424 0 448 200.576 448 448S759.424 960 512 960 64 759.424 64 512 264.576 64 512 64Zm0 64c-212.077 0-384 171.923-384 384s171.923 384 384 384 384-171.923 384-384-171.923-384-384-384Zm19.2 576c6.967 0 12.634 5.566 12.796 12.493L544 755.2c0 6.967-5.566 12.634-12.493 12.796L492.8 768c-6.967 0-12.634-5.566-12.796-12.493L480 716.8c0-6.967 5.566-12.634 12.493-12.796L531.2 704Zm0-448c6.967 0 12.634 5.566 12.796 12.493L544 627.2c0 6.967-5.566 12.634-12.493 12.8H492.8c-6.967 0-12.634-5.566-12.796-12.493L480 268.8c0-6.967 5.566-12.634 12.493-12.8H531.2Z"/></svg>'
};
const Exclamation = {
  name: "exclamation",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path d="M550.4 768c14.08 0 25.6 11.52 25.6 25.6v76.8a25.6 25.6 0 0 1-25.6 25.6h-76.8a25.6 25.6 0 0 1-25.6-25.6v-76.8c0-14.08 11.52-25.6 25.6-25.6h76.8zm0-576c14.08 0 25.6 11.52 25.6 25.6v460.8a25.6 25.6 0 0 1-25.6 25.6h-76.8a25.6 25.6 0 0 1-25.6-25.6V217.6c0-14.08 11.52-25.6 25.6-25.6h76.8z"/></svg>'
};
const Exit = {
  name: "exit",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path fill-rule="evenodd" d="M128 880V208c0-8.837 7.163-16 16-16h352c8.836 0 16 7.163 16 16v32c0 8.837-7.164 16-16 16H192v576h304c8.836 0 16 7.162 16 16v32c0 8.838-7.164 16-16 16H144c-8.837 0-16-7.162-16-16Zm520.935-73.877 253.657-253.659c4.998-4.998 4.998-12.872 0-17.87L648.935 280.935c-4.998-4.998-12.868-5.002-17.867-.003l-27.39 27.39c-5 5-4.939 12.927.06 17.926L789.491 512H332.8c-7.07 0-12.8 5.73-12.8 12.8v38.4c0 7.07 5.73 12.8 12.8 12.8h455.75L603.738 760.81c-4.998 4.999-5.06 12.927-.06 17.925l27.39 27.39c4.999 5 12.869 4.996 17.867-.002Z"/></svg>'
};
const ExpandAll = {
  name: "expand-all",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path fill-rule="evenodd" d="M192 256v-51.2c0-7.07-5.73-12.8-12.8-12.8h-38.4c-7.07 0-12.8 5.73-12.8 12.8v358.4c0 7.07 5.73 12.8 12.8 12.8H256v243.2c0 7.07 5.73 12.8 12.8 12.8h166.4c7.07 0 12.8-5.73 12.8-12.8v-38.4c0-7.07-5.73-12.8-12.8-12.8H320V576h51.2c7.07 0 12.8-5.73 12.8-12.8v-38.4c0-7.07-5.73-12.8-12.8-12.8H192V320h51.2c7.07 0 12.8-5.73 12.8-12.8v-38.4c0-7.07-5.73-12.8-12.8-12.8H192Zm576 51.2v-38.4c0-7.07-5.73-12.8-12.8-12.8H332.8c-7.07 0-12.8 5.73-12.8 12.8v38.4c0 7.07 5.73 12.8 12.8 12.8h422.4c7.07 0 12.8-5.73 12.8-12.8Zm64 256v-38.4c0-7.07-5.728-12.8-12.8-12.8H460.8c-7.07 0-12.8 5.73-12.8 12.8v38.4c0 7.07 5.73 12.8 12.8 12.8h358.4c7.072 0 12.8-5.73 12.8-12.8Zm64 256v-38.4c0-7.07-5.728-12.8-12.8-12.8H524.8c-7.07 0-12.8 5.73-12.8 12.8v38.4c0 7.07 5.73 12.8 12.8 12.8h358.4c7.072 0 12.8-5.73 12.8-12.8Z"/></svg>'
};
const Expand = {
  name: "expand",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path d="M204.8 128a12.8 12.8 0 0 0-12.8 12.8v742.4c0 7.04 5.76 12.8 12.8 12.8h38.4a12.8 12.8 0 0 0 12.8-12.8V140.8a12.8 12.8 0 0 0-12.8-12.8h-38.4zm266.176 76.224-27.2 27.2a12.8 12.8 0 0 0 0 18.112l262.528 262.4-262.528 262.528a12.8 12.8 0 0 0 0 18.112l27.2 27.136a12.8 12.8 0 0 0 18.048 0l298.688-298.688a12.8 12.8 0 0 0 0-18.112L489.024 204.224a12.8 12.8 0 0 0-18.048 0z"/></svg>'
};
const Export = {
  name: "export",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path d="m840.768 135.936 45.248 45.248-566.4 568.32a12.8 12.8 0 0 1-18.048 0l-27.136-27.072a12.8 12.8 0 0 1 0-18.112L840.768 136z"/><path d="M896.192 140.8v358.4a12.8 12.8 0 0 1-12.8 12.8h-38.4a12.8 12.8 0 0 1-12.8-12.8l-.064-307.2H524.8a12.8 12.8 0 0 1-12.8-12.8v-38.4c0-7.04 5.76-12.8 12.8-12.8h358.592c7.04 0 12.8 5.76 12.8 12.8zm-520 0v37.888a12.8 12.8 0 0 1-12.8 12.8L140.288 192a12.8 12.8 0 0 0-12.8 12.8v680.32c0 7.04 5.76 12.8 12.8 12.8h691.648L832 652.8c0-7.04 5.76-12.8 12.8-12.8h38.4c7.04 0 12.8 5.76 12.8 12.8v264.384a44.8 44.8 0 0 1-38.72 44.416l-6.08.384-774.912-.384a12.8 12.8 0 0 1-12.8-12.8v-808c0-7.04 5.76-12.8 12.8-12.8h287.104c7.04 0 12.8 5.76 12.8 12.8z"/></svg>'
};
const EyeInvisible = {
  name: "eye-invisible",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path fill-rule="evenodd" d="M65.405 506.17a12.8 12.8 0 0 0 0 11.66l5.258 10.276.786 1.53a1153.52 1153.52 0 0 0 10.72 20.308l1.216 2.244 1.219 2.235 1.649 3.005c1.6 2.906 3.207 5.792 4.82 8.658l1.198 2.121a967.365 967.365 0 0 0 4.188 7.33l1.483 2.564 1.358 2.334 2.091 3.562a988.013 988.013 0 0 0 6.02 10.074l2.328 3.821a923.688 923.688 0 0 0 12.395 19.693l2.307 3.546c1.112 1.7 2.227 3.391 3.344 5.074l1.937 2.905a855.68 855.68 0 0 0 3.344 4.953l1.93 2.825 1.402 2.039 1.138 1.643 1.68 2.412c.627.897 1.255 1.791 1.884 2.683l.773 1.093c.82 1.16 1.643 2.314 2.466 3.465l1.183 1.647 1.796 2.486 1.168 1.605 1.336 1.825 2.255 3.058 1.898 2.547 2.232 2.97a757.394 757.394 0 0 0 6.051 7.894l1.824 2.338a765.88 765.88 0 0 0 2.713 3.44l1.686 2.116a690.26 690.26 0 0 0 3.384 4.196l1.287 1.58c1.613 1.973 3.23 3.93 4.853 5.87l1.131 1.348 2.014 2.385 1.27 1.492 1.798 2.096c4.283 4.974 8.6 9.836 12.949 14.585 4.85 5.295 13.105 5.363 18.183.285l27.24-27.24c4.866-4.866 5.014-12.706.338-17.752l-1.525-1.646a566.42 566.42 0 0 1-8.538-9.68 614.785 614.785 0 0 1-16.837-20.413c-11.119-14.077-22.026-29.09-32.722-45.046l-.885-1.324a809.053 809.053 0 0 1-5.678-8.627l-1.413-2.183c-.94-1.46-1.88-2.926-2.818-4.4l-.743-1.172-1.838-2.913a826.268 826.268 0 0 1-3.036-4.87l-1.757-2.85-1.517-2.481-1.523-2.51-1.692-2.809-1.146-1.916-.926-1.555-1.63-2.755-1.501-2.556-.796-1.364-1.186-2.04-1.74-3.014c-1-1.74-1.998-3.488-2.994-5.246l-.64-1.131-1.985-3.53-1.853-3.328-1.862-3.37-1.89-3.454-1.816-3.345-1.094-2.03L134.52 512l-.039-.066c.649-1.223 1.299-2.443 1.949-3.658l1.578-2.936a980.228 980.228 0 0 1 4.752-8.716l2.992-5.388c.117-.209.234-.417.35-.627l2.948-5.223.125-.22 1.624-2.847a973.896 973.896 0 0 1 4.896-8.444l2.982-5.05.34-.57 2.829-4.716.396-.656.747-1.229 2.292-3.754.224-.365 3.059-4.94.141-.226 1.66-2.647a854.987 854.987 0 0 1 4.997-7.846l2.838-4.37C271.989 294.05 382.197 224 509.495 224l15.09.332 1.495.069c46.174 2.242 90.377 13.905 132.635 35.144 5.183 2.605 11.365 1.525 15.332-2.709l27.443-29.292c5.601-5.98 4.245-15.61-2.964-19.511C639.532 176.105 576.506 160 509.494 160c-124.557 0-233.712 55.631-327.075 164.981l-.412.483-2.616 3.09a677.688 677.688 0 0 0-5.873 7.074l-2.082 2.555a717.91 717.91 0 0 0-13.63 17.348l-2.057 2.716-1.9 2.53-1.917 2.575a785.516 785.516 0 0 0-9.517 13.125l-2.069 2.93a826.07 826.07 0 0 0-5.548 7.993l-2.029 2.972-1.777 2.628-1.927 2.873-1.79 2.69-1.975 2.995-1.748 2.672-1.914 2.951-1.73 2.69-2.092 3.28a938.626 938.626 0 0 0-8.914 14.326l-1.708 2.812c-12.805 21.158-25.244 43.539-37.316 67.137l-4.473 8.744Zm748.63-324.768c-5.017-4.76-12.947-4.68-17.862.238L180.71 797.105l-.238.245c-4.76 5.012-4.68 12.941.238 17.856l27.153 27.15.245.242c5.014 4.756 12.939 4.679 17.857-.243l615.469-615.46.237-.245c4.761-5.015 4.678-12.94-.237-17.857l-27.155-27.153-.244-.238Zm60.525 181.815c-15.865-20.561-32.14-39.581-48.819-57.051-4.89-5.125-12.934-5.036-17.946-.026l-27.097 27.1c-4.986 4.988-4.954 13.098-.096 18.213 27.443 28.884 53.907 62.733 79.398 101.606a964.942 964.942 0 0 1 10.439 16.28l1.478 2.37 1.651 2.657a1001.366 1001.366 0 0 1 9.319 15.379l2.406 4.067c1.26 2.14 2.515 4.292 3.77 6.457l1.772 3.078.916 1.595 1.305 2.288 2.592 4.709.051.096a966.67 966.67 0 0 1-15.187 26.206l-1.721 2.855c-1.722 2.847-3.45 5.67-5.178 8.47-.46.738-.915 1.475-1.37 2.21l-.128.205-1.484 2.375-.058.095-.064.1-.224.357-1.74 2.765a869.983 869.983 0 0 1-5.242 8.2l-3.053 4.695C759.277 724.696 643.08 800 511.073 800l-15.052-.328c-46.76-2.062-91.17-13.8-133.26-35.385-5.064-2.597-11.332-1.78-15.356 2.244l-28.587 28.586c-5.862 5.862-4.59 15.731 2.652 19.77C380.068 847.526 443.29 864 511.072 864 642.934 864 759.367 801.645 860 679.082a708.249 708.249 0 0 0 6.611-8.168l1.888-2.369 1.876-2.382 2.003-2.562 1.85-2.38 2.092-2.722 1.818-2.383 2.06-2.72 1.818-2.42 2.298-3.088a861.623 861.623 0 0 0 12.934-17.942l2.119-3.028 1.779-2.567 2.016-2.927c1.85-2.705 3.699-5.433 5.542-8.183l1.895-2.84 1.747-2.638 2.22-3.386a983.907 983.907 0 0 0 12.737-19.994l1.824-2.95 1.715-2.788 1.952-3.204 1.734-2.86 1.92-3.206 1.722-2.887 1.9-3.205 1.652-2.818 2.157-3.695c1.715-2.956 3.426-5.934 5.132-8.933l16.352-29.703a12.811 12.811 0 0 0 .032-12.281l-4.953-9.125a1202.232 1202.232 0 0 0-3.655-6.676l-.364-.664-1.946-3.513a1023.498 1023.498 0 0 0-3.174-5.661l-2.016-3.566c-11.924-20.992-24.128-40.958-36.602-59.89l-1.338-2.027-1.856-2.787a670.634 670.634 0 0 0-3.718-5.526l-2.131-3.131a875.902 875.902 0 0 0-11.827-16.885l-2.72-3.768c-1.07-1.47-2.138-2.93-3.207-4.385l-1.568-2.124-1.165-1.573-1.612-2.153-2.19-2.914a928.998 928.998 0 0 0-2.649-3.477l-2.144-2.791Zm-296.985-31.384c-20.722-7.651-43.125-11.83-66.505-11.83-106.038 0-192 85.961-192 192 0 23.38 4.179 45.783 11.83 66.505 3.057 8.276 13.528 9.98 19.767 3.742l31.578-31.577c3.24-3.24 4.462-7.905 3.526-12.39a128.539 128.539 0 0 1-2.7-26.28c0-70.693 57.307-128 128-128 9.007 0 17.798.93 26.28 2.7 4.485.936 9.15-.287 12.39-3.527l31.576-31.577c6.239-6.238 4.534-16.71-3.742-19.766Zm-131.5 360.893c-8.269-2.974-10.17-13.41-4.05-19.717l30.95-31.886c3.223-3.321 7.963-4.597 12.5-3.676a128.609 128.609 0 0 0 25.594 2.56c70.692 0 128-57.308 128-128a128.6 128.6 0 0 0-2.412-24.856c-.856-4.353.308-8.877 3.387-12.071l31.62-32.805c6.25-6.25 16.799-4.386 19.829 3.916 7.49 20.528 11.576 42.695 11.576 65.815 0 106.039-85.961 192-192 192a192.668 192.668 0 0 1-42.42-4.702 190.65 190.65 0 0 1-22.573-6.578Z"/></svg>'
};
const Eye = {
  name: "eye",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path fill-rule="evenodd" d="M509.494 160c178.3 0 328.39 114.017 449.37 336.726l4.954 9.124a12.812 12.812 0 0 1-.032 12.28l-16.352 29.705C828.477 756.998 682.223 864 509.494 864c-178.58 0-325.497-114.353-439.616-337.426l-4.473-8.744a12.8 12.8 0 0 1 0-11.66l4.473-8.744C183.997 274.353 330.914 160 509.494 160ZM134.52 512l2.829 5.437C235.99 700.954 354.715 793.505 494.443 799.67l15.052.33c147.745 0 275.686-94.333 384.64-287.926l-.013-.042-.052-.096-2.592-4.708C786.832 323.372 664.78 230.516 524.586 224.332L509.495 224c-147.645 0-272.3 94.235-375.016 287.936l.04.064Zm374.973-192c106.04 0 192 85.962 192 192s-85.96 192-192 192c-106.038 0-192-85.962-192-192s85.962-192 192-192Zm0 64c-70.692 0-128 57.308-128 128 0 70.692 57.308 128 128 128 70.693 0 128-57.308 128-128 0-70.692-57.307-128-128-128Z"/></svg>'
};
const Favourite = {
  name: "favourite",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><g fill-rule="evenodd"><path d="M423.953 64a12.8 12.8 0 0 1 9.051 3.749L557.255 192h326.047c7.072 0 12.698 5.83 12.698 12.9v102.3c0 7.07-5.728 12.8-12.8 12.8h-38.4c-7.072 0-12.8-5.73-12.8-12.8V256H128v512h179.2c7.07 0 12.8 5.728 12.8 12.8v38.4c0 7.072-5.73 12.8-12.8 12.8H76.696C69.626 832 64 826.17 64 819.098V76.8C64 69.73 69.73 64 76.8 64h347.153Zm-21.201 64H128v64h338.752l-64-64Z"/><path d="M628.59 343.123c4.696-9.514 18.129-9.514 22.824 0l82.634 167.434 184.774 26.85c10.503 1.525 14.65 14.3 7.053 21.706l-133.702 130.33 31.565 184.029c1.792 10.458-9.076 18.349-18.47 13.414L640.001 800l-165.268 86.886c-9.39 4.935-20.258-2.956-18.465-13.414l31.564-184.03-133.704-130.329c-7.598-7.405-3.447-20.18 7.052-21.706l184.775-26.85Zm11.413 121.455-51.546 104.478-115.238 16.738L556.6 667.1l-19.765 114.723 103.168-54.125 103.101 54.125-19.7-114.723 83.316-81.307-115.172-16.738-51.545-104.478Z"/></g></svg>'
};
const FileAdd = {
  name: "file-add",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><g fill-rule="evenodd"><path d="m140.801 64 557.413.056a12.8 12.8 0 0 1 8.787 3.494l.26.252L892.25 252.678a12.837 12.837 0 0 1 3.75 8.69v109.885c0 6.967-5.563 12.634-12.493 12.797l-.307.003h-38.4c-6.97 0-12.634-5.566-12.796-12.493l-.004-.307v-51.26H652.8c-6.967 0-12.634-5.566-12.796-12.494l-.004-.306v-179.14l-448-.316v768.32h179.2c6.967 0 12.634 5.569 12.796 12.493l.004.306v38.4c0 6.97-5.566 12.634-12.493 12.8H140.8c-6.967 0-12.634-5.563-12.8-12.493V76.8c0-6.967 5.567-12.634 12.495-12.8h.306Zm562.942 90.77L704 255.993l101.254.23-101.51-101.453Z"/><path d="M672 384.053c159.06 0 288 128.942 288 288 0 159.056-128.94 288.003-288 288.003-159.058 0-288-128.947-288-288.003 0-159.058 128.942-288 288-288Zm0 64c-123.712 0-224 100.289-224 224 0 123.715 100.288 224.003 224 224.003s224-100.288 224-224.003c0-123.711-100.288-224-224-224ZM256 499.253v-38.4c0-7.069 5.73-12.8 12.8-12.8h102.4c7.07 0 12.8 5.731 12.8 12.8v38.4c0 7.07-5.73 12.8-12.8 12.8H268.8c-7.07 0-12.8-5.73-12.8-12.8Z"/><path d="M640 524.853c0-7.069 5.73-12.8 12.8-12.8h38.4c7.07 0 12.8 5.731 12.8 12.8v115.2h115.2c7.072 0 12.8 5.731 12.8 12.8v38.4c0 7.07-5.728 12.803-12.8 12.803H704v115.2c0 7.066-5.73 12.8-12.8 12.8h-38.4c-7.07 0-12.8-5.734-12.8-12.8v-115.2H524.8c-7.07 0-12.8-5.733-12.8-12.803v-38.4c0-7.069 5.73-12.8 12.8-12.8H640v-115.2ZM256 627.253v-38.4c0-7.069 5.73-12.8 12.8-12.8h38.4c7.07 0 12.8 5.731 12.8 12.8v38.4c0 7.07-5.73 12.8-12.8 12.8h-38.4c-7.07 0-12.8-5.73-12.8-12.8ZM256 755.256v-38.4c0-7.072 5.73-12.8 12.8-12.8h38.4c7.07 0 12.8 5.728 12.8 12.8v38.4c0 7.066-5.73 12.8-12.8 12.8h-38.4c-7.07 0-12.8-5.734-12.8-12.8Z"/></g></svg>'
};
const FileAlert = {
  name: "file-alert",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><g fill-rule="evenodd"><path d="M698.214 64c3.394 0 6.648 1.352 9.048 3.75L892.25 252.62c2.4 2.401 3.75 5.658 3.75 9.055V371.2c0 7.07-5.728 12.8-12.8 12.8h-38.4c-7.072 0-12.8-5.73-12.8-12.8v-51.264H653.524c-7.069 0-12.8-5.73-12.8-12.8L640 128l-448-.384v768.32l177.585.058c7.067.006 12.795 5.734 12.795 12.8V947.2c0 7.072-5.73 12.8-12.8 12.8H140.8c-7.07 0-12.8-5.728-12.8-12.8V76.8c0-7.07 5.73-12.8 12.8-12.8h557.414Zm5.3 90.487V256l101.772.196-101.772-101.709Z"/><path d="M672 384c159.06 0 288 128.942 288 288 0 159.06-128.94 288-288 288-159.058 0-288-128.94-288-288 0-159.058 128.942-288 288-288Zm0 64c-123.712 0-224 100.288-224 224s100.288 224 224 224 224-100.288 224-224-100.288-224-224-224Z"/><path d="M691.2 768c6.967 0 12.634 5.563 12.796 12.493l.004.307v38.4c0 6.97-5.566 12.634-12.493 12.796l-.307.004h-38.4c-6.967 0-12.634-5.563-12.796-12.493L640 819.2v-38.4c0-6.97 5.566-12.634 12.493-12.796l.307-.004h38.4ZM691.2 512c7.07 0 12.8 5.73 12.8 12.8v166.4c0 7.07-5.73 12.8-12.8 12.8h-38.4c-7.07 0-12.8-5.73-12.8-12.8V524.8c0-7.07 5.73-12.8 12.8-12.8h38.4ZM307.2 704c7.07 0 12.8 5.728 12.8 12.8v38.4c0 7.072-5.73 12.8-12.8 12.8h-38.4c-7.07 0-12.8-5.728-12.8-12.8v-38.4c0-7.072 5.73-12.8 12.8-12.8h38.4ZM307.2 576c7.07 0 12.8 5.73 12.8 12.8v38.4c0 7.07-5.73 12.8-12.8 12.8h-38.4c-7.07 0-12.8-5.73-12.8-12.8v-38.4c0-7.07 5.73-12.8 12.8-12.8h38.4ZM371.2 448c7.07 0 12.8 5.73 12.8 12.8v38.4c0 7.07-5.73 12.8-12.8 12.8H268.8c-7.07 0-12.8-5.73-12.8-12.8v-38.4c0-7.07 5.73-12.8 12.8-12.8h102.4Z"/></g></svg>'
};
const FileDisabled = {
  name: "file-disabled",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><g fill-rule="evenodd"><path d="M698.214 64c3.272 0 6.415 1.257 8.788 3.498l.26.253L892.25 252.62a12.837 12.837 0 0 1 3.75 8.69V371.2c0 6.967-5.563 12.634-12.493 12.796l-.307.004h-38.4c-6.97 0-12.634-5.566-12.796-12.494L832 371.2v-51.264H653.524c-6.967 0-12.633-5.566-12.796-12.494l-.004-.306V123.52L192 127.616v768.32l177.585.058c6.965.006 12.629 5.57 12.792 12.493l.003.307V947.2c0 6.97-5.565 12.634-12.493 12.8H140.8c-6.967 0-12.634-5.563-12.8-12.493V76.8c0-6.967 5.566-12.634 12.494-12.8h557.72Zm5.3 90.487V256l101.772.196-101.772-101.71Z"/><path d="M672 384c159.06 0 288 128.942 288 288 0 159.06-128.94 288-288 288-159.058 0-288-128.94-288-288 0-159.058 128.942-288 288-288ZM493.103 537.197A224 224 0 0 0 448 672c0 123.712 100.288 224 224 224a224 224 0 0 0 132.521-43.404l1.162-.859-312.58-314.54ZM672 448a224.001 224.001 0 0 0-132.315 43.255l-1.16.856 312.528 314.487.87-1.167A224.021 224.021 0 0 0 896 672c0-123.712-100.288-224-224-224ZM307.2 704c6.967 0 12.634 5.563 12.796 12.493l.004.307v38.4c0 6.97-5.566 12.634-12.493 12.796l-.307.004h-38.4c-6.967 0-12.634-5.563-12.796-12.493L256 755.2v-38.4c0-6.97 5.566-12.634 12.493-12.797l.307-.003h38.4ZM307.2 576c6.967 0 12.634 5.566 12.796 12.493l.004.307v38.4c0 6.967-5.566 12.634-12.493 12.796l-.307.004h-38.4c-6.967 0-12.634-5.566-12.796-12.494L256 627.2v-38.4c0-6.967 5.566-12.634 12.493-12.797l.307-.003h38.4ZM371.2 448c6.967 0 12.634 5.566 12.796 12.493l.004.307v38.4c0 6.967-5.566 12.634-12.493 12.796l-.307.004H268.8c-6.967 0-12.634-5.566-12.796-12.494L256 499.2v-38.4c0-6.967 5.566-12.634 12.493-12.797l.307-.003h102.4Z"/></g></svg>'
};
const FileDone = {
  name: "file-done",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><g fill-rule="evenodd"><path d="M698.214 64c3.272 0 6.415 1.257 8.788 3.498l.26.253L892.25 252.62a12.837 12.837 0 0 1 3.75 8.69V371.2c0 6.967-5.563 12.634-12.493 12.796l-.307.004h-38.4c-6.97 0-12.634-5.566-12.796-12.494L832 371.2v-51.264H653.476c-6.946 0-12.604-5.535-12.795-12.441l-.005-.31L640 128l-448-.384v768.32l177.585.058c6.965.006 12.629 5.57 12.792 12.493l.003.307V947.2c0 6.97-5.565 12.634-12.493 12.8H140.8c-6.967 0-12.634-5.563-12.8-12.493V76.8c0-6.967 5.566-12.634 12.494-12.8h557.72Zm5.3 90.487V256l101.772.196-101.772-101.71Z"/><path d="M672 384c159.06 0 288 128.942 288 288 0 159.06-128.94 288-288 288-159.058 0-288-128.94-288-288 0-159.058 128.942-288 288-288Zm0 64c-123.712 0-224 100.288-224 224s100.288 224 224 224 224-100.288 224-224-100.288-224-224-224ZM256 460.8v38.4c0 7.07 5.73 12.8 12.8 12.8h102.4c7.07 0 12.8-5.73 12.8-12.8v-38.4c0-7.07-5.73-12.8-12.8-12.8H268.8c-7.07 0-12.8 5.73-12.8 12.8Z"/><path d="m636.085 822.95 209.573-209.577c4.998-5 4.998-13.104 0-18.102l-27.15-27.153c-4.998-4.999-13.106-4.999-18.105 0L627.034 741.49l-81.459-81.46c-4.998-5-13.103-5-18.102 0l-27.152 27.152c-5 5-5 13.104 0 18.103L617.983 822.95c4.999 4.999 13.103 4.999 18.102 0ZM256 588.8v38.4c0 7.07 5.73 12.8 12.8 12.8h38.4c7.07 0 12.8-5.73 12.8-12.8v-38.4c0-7.07-5.73-12.8-12.8-12.8h-38.4c-7.07 0-12.8 5.73-12.8 12.8ZM256 755.2v-38.4c0-7.072 5.73-12.8 12.8-12.8h38.4c7.07 0 12.8 5.728 12.8 12.8v38.4c0 7.072-5.73 12.8-12.8 12.8h-38.4c-7.07 0-12.8-5.728-12.8-12.8Z"/></g></svg>'
};
const FileExport = {
  name: "file-export",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><g fill-rule="evenodd"><path d="M698.214 64c3.272 0 6.415 1.257 8.788 3.498l.26.253L892.25 252.62a12.837 12.837 0 0 1 3.75 8.69V371.2c0 6.967-5.563 12.634-12.493 12.796l-.307.004h-38.4c-6.97 0-12.634-5.566-12.796-12.494L832 371.2v-51.264H653.524c-6.967 0-12.633-5.566-12.796-12.494l-.004-.306L640 128l-448-.384v768.32l177.585.058c6.965.006 12.629 5.57 12.792 12.493l.003.307V947.2c0 6.97-5.565 12.634-12.493 12.8H140.8c-6.967 0-12.634-5.563-12.8-12.493V76.8c0-6.967 5.566-12.634 12.494-12.8h557.72Zm5.3 90.487V256l101.772.196-101.772-101.71Z"/><path d="M672 384c159.06 0 288 128.942 288 288 0 159.06-128.94 288-288 288-159.058 0-288-128.94-288-288 0-159.058 128.942-288 288-288Zm0 64c-123.712 0-224 100.288-224 224s100.288 224 224 224 224-100.288 224-224-100.288-224-224-224ZM256 460.8v38.4c0 7.07 5.73 12.8 12.8 12.8h102.4c7.07 0 12.8-5.73 12.8-12.8v-38.4c0-7.07-5.73-12.8-12.8-12.8H268.8c-7.07 0-12.8 5.73-12.8 12.8Z"/><path d="m822.95 662.948-140.29-140.29c-4.999-5-13.103-5-18.101 0l-27.154 27.152c-4.998 5-4.998 13.104 0 18.102L709.493 640H526.046c-7.07 0-12.8 5.73-12.8 12.8v38.4c0 7.07 5.73 12.8 12.8 12.8h183.445l-72.086 72.083c-4.998 4.999-4.998 13.107 0 18.106l27.154 27.149c4.998 4.998 13.102 4.998 18.101 0L822.95 681.05c4.999-4.999 4.999-13.104 0-18.102ZM256 588.8v38.4c0 7.07 5.73 12.8 12.8 12.8h38.4c7.07 0 12.8-5.73 12.8-12.8v-38.4c0-7.07-5.73-12.8-12.8-12.8h-38.4c-7.07 0-12.8 5.73-12.8 12.8ZM256 755.2v-38.4c0-7.072 5.73-12.8 12.8-12.8h38.4c7.07 0 12.8 5.728 12.8 12.8v38.4c0 7.072-5.73 12.8-12.8 12.8h-38.4c-7.07 0-12.8-5.728-12.8-12.8Z"/></g></svg>'
};
const FileGif = {
  name: "file-gif",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path d="M325.76 288a124.8 124.8 0 0 1 124.48 115.456l.32 9.344h-64a60.8 60.8 0 0 0-53.632-60.416L325.76 352H224a96 96 0 0 0-95.552 86.784L128 448v128a96 96 0 0 0 86.784 95.552L224 672h130.56a32 32 0 0 0 31.488-26.24l.512-5.76v-90.112H269.632v-64H450.56V640a96 96 0 0 1-86.72 95.552l-9.28.448H224A160 160 0 0 1 64.32 586.496L64 576V448a160 160 0 0 1 149.504-159.68L224 288h101.76zm183.872 0h64v448h-64zm448 0v64h-256v128h256v64h-256v192h-64V288h320zM896 832v115.2a12.8 12.8 0 0 1-12.8 12.8H140.8a12.8 12.8 0 0 1-12.8-12.8V832h64v64h640v-64h64zM883.2 64c7.04 0 12.8 5.76 12.8 12.8V192h-64v-64H192v64h-64V76.8c0-7.04 5.76-12.8 12.8-12.8h742.4z"/></svg>'
};
const FileImage = {
  name: "file-image",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path fill-rule="evenodd" d="M896 261.731v685.525c0 7.072-5.728 12.8-12.8 12.8H140.8c-7.07 0-12.8-5.728-12.8-12.8V76.8c0-7.07 5.732-12.8 12.801-12.8l557.412.056a12.8 12.8 0 0 1 9.047 3.746l184.99 184.876c2.4 2.4 3.75 5.657 3.75 9.053ZM192 896.056h640V319.993H652.8c-7.07 0-12.8-5.73-12.8-12.8V123.577l-448 4.16v768.32Zm613.254-639.834L703.744 154.77 704 255.993l101.254.23ZM384 400.053c-26.51 0-48 21.49-48 48s21.49 48 48 48 48-21.49 48-48-21.49-48-48-48Zm203.03 66.331 169.684 281.98c5.134 8.53-1.01 19.404-10.967 19.404H385.809l.089-.153h-105.95c-10.23 0-16.327-11.405-10.644-19.91L373.357 591.98c5.069-7.585 16.217-7.585 21.286 0l50.646 75.797L565.062 466.44c4.947-8.317 16.978-8.348 21.968-.056Z"/></svg>'
};
const FileProtect = {
  name: "file-protect",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path fill-rule="evenodd" d="M140.8 960h294.4c7.07 0 12.8-5.728 12.8-12.8v-38.4c0-7.072-5.729-12.8-12.797-12.8L192 895.936v-768.32L640 128l.676 179.185c.027 7.05 5.75 12.75 12.8 12.75H832v50.916c0 7.07 5.728 12.8 12.8 12.8h38.4c7.072 0 12.8-5.73 12.8-12.8V261.675c0-3.397-1.35-6.654-3.75-9.054L707.262 67.75a12.803 12.803 0 0 0-9.048-3.75H140.8c-7.07 0-12.8 5.73-12.8 12.8v870.4c0 7.072 5.73 12.8 12.8 12.8Zm562.714-805.513 101.772 101.709L703.514 256V154.487ZM256 460.8v38.4c0 7.07 5.73 12.8 12.8 12.8h102.4c7.07 0 12.8-5.73 12.8-12.8v-38.4c0-7.07-5.73-12.8-12.8-12.8H268.8c-7.07 0-12.8 5.73-12.8 12.8Zm192 139.966.347 16.564c5.766 137.57 83.076 262.996 205.151 332.263l13.086 7.072a12.791 12.791 0 0 0 12.288-.057l13.296-7.36 14.353-8.615C824.294 866.676 896 739.182 896 601.726v-103.78a12.802 12.802 0 0 0-10.586-12.607l-210.464-36.95a12.801 12.801 0 0 0-4.412-.003L458.602 485.33A12.8 12.8 0 0 0 448 497.941v102.825Zm384 .96-.34 14.934c-4.934 109.145-63.79 209.452-158.226 269.177l-.922.563-13.357-8.71C567.35 814.42 512 711.352 512 600.766v-59.774l160.832-28.032L832 540.929v60.796ZM256 588.8v38.4c0 7.07 5.73 12.8 12.8 12.8h38.4c7.07 0 12.8-5.73 12.8-12.8v-38.4c0-7.07-5.73-12.8-12.8-12.8h-38.4c-7.07 0-12.8 5.73-12.8 12.8Zm0 128v38.4c0 7.072 5.73 12.8 12.8 12.8h38.4c7.07 0 12.8-5.728 12.8-12.8v-38.4c0-7.072-5.73-12.8-12.8-12.8h-38.4c-7.07 0-12.8 5.728-12.8 12.8Z"/></svg>'
};
const FileSearch = {
  name: "file-search",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><g fill-rule="evenodd"><path d="m140.801 64 557.413.056a12.8 12.8 0 0 1 8.787 3.494l.26.252L892.25 252.678a12.837 12.837 0 0 1 3.75 8.69v237.885c0 6.967-5.563 12.634-12.493 12.797l-.307.003h-38.4c-6.97 0-12.634-5.566-12.796-12.493l-.004-.307v-179.26H652.8c-6.967 0-12.634-5.566-12.796-12.494l-.004-.306v-179.14l-448-.316v768.32h243.2c6.967 0 12.634 5.569 12.796 12.493l.004.306v38.4c0 6.97-5.566 12.634-12.493 12.8H140.8c-6.967 0-12.634-5.563-12.8-12.493V76.8c0-6.967 5.567-12.634 12.495-12.8h.306Zm562.942 90.77L704 255.993l101.254.23-101.51-101.453Z"/><path d="M256 499.253v-38.4c0-7.069 5.73-12.8 12.8-12.8h102.4c7.07 0 12.8 5.731 12.8 12.8v38.4c0 7.07-5.73 12.8-12.8 12.8H268.8c-7.07 0-12.8-5.73-12.8-12.8ZM608 448.053c123.712 0 224 100.289 224 224a223.991 223.991 0 0 1-44.602 134.135l94.061 94.067c4.999 4.998 4.999 13.1 0 18.099l-27.149 27.155c-4.998 4.999-13.107 4.999-18.105 0l-94.062-94.06A224.033 224.033 0 0 1 608 896.055c-123.712 0-224-100.288-224-224.003 0-123.711 100.288-224 224-224Zm0 64c-88.365 0-160 71.635-160 160 0 88.368 71.635 160.003 160 160.003s160-71.635 160-160.003c0-88.365-71.635-160-160-160ZM256 627.253v-38.4c0-7.069 5.73-12.8 12.8-12.8h38.4c7.07 0 12.8 5.731 12.8 12.8v38.4c0 7.07-5.73 12.8-12.8 12.8h-38.4c-7.07 0-12.8-5.73-12.8-12.8ZM256 755.256v-38.4c0-7.072 5.73-12.8 12.8-12.8h38.4c7.07 0 12.8 5.728 12.8 12.8v38.4c0 7.066-5.73 12.8-12.8 12.8h-38.4c-7.07 0-12.8-5.734-12.8-12.8Z"/></g></svg>'
};
const FileSync = {
  name: "file-sync",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><g fill-rule="evenodd"><path d="M698.189 64a12.8 12.8 0 0 1 8.787 3.493l.26.253L892.25 252.634a12.838 12.838 0 0 1 3.75 8.691V371.2c0 6.967-5.563 12.634-12.493 12.796l-.307.004h-38.4c-6.97 0-12.634-5.566-12.796-12.493L832 371.2v-51.264H653.504c-6.967 0-12.634-5.566-12.796-12.493l-.004-.307V123.52L192 127.616v768.32h177.6c6.936.063 12.63 5.594 12.796 12.552l.004.312v38.4c0 6.97-5.566 12.634-12.493 12.8H140.8c-6.967 0-12.634-5.563-12.8-12.493V76.8c0-6.938 5.594-12.633 12.495-12.8h557.694Zm5.299 90.466V256l101.805.203-101.805-101.737Z"/><path d="M931.558 547.152C883.578 447.411 782.682 383.99 672 384a286.788 286.788 0 0 0-168.69 54.527c-6.178 4.475-6.365 13.163-.967 18.554l27.672 27.633c4.501 4.495 11.554 5 16.827 1.443 77.882-52.54 180.858-51.015 257.35 4.868 79.469 58.06 111.738 161.192 79.52 254.191H820.8a12.807 12.807 0 0 0-11.795 7.949 12.813 12.813 0 0 0 2.835 13.939L877.248 832v-.064l9.658 9.69c5.395 5.408 14.278 4.985 18.745-1.21 61.504-85.286 71.815-197.824 25.907-293.264ZM371.2 448H268.8c-8.533 0-12.8 4.267-12.8 12.8v38.4c0 8.533 4.267 12.8 12.8 12.8h102.4c8.533 0 12.8-4.267 12.8-12.8v-38.4c0-8.533-4.267-12.8-12.8-12.8ZM488.115 533.363l-31.026-31.026c-5.395-5.395-14.345-4.859-18.81 1.327-82.265 113.928-70.409 271.747 29.95 372.106 100.36 100.358 258.18 112.217 372.11 29.952 6.183-4.468 6.72-13.415 1.325-18.81l-27.603-27.603c-4.506-4.506-11.725-4.903-16.999-1.338-88.404 59.693-207.482 48.839-283.646-27.321-61.813-61.818-80.606-151.896-53.584-230.903h64.328a12.8 12.8 0 0 0 8.96-21.888l-45.005-44.496ZM307.2 576h-38.4c-8.533 0-12.8 4.267-12.8 12.8v38.4c0 8.533 4.267 12.8 12.8 12.8h38.4c8.533 0 12.8-4.267 12.8-12.8v-38.4c0-8.533-4.267-12.8-12.8-12.8ZM307.2 704h-38.4c-8.533 0-12.8 4.267-12.8 12.8v38.4c0 8.533 4.267 12.8 12.8 12.8h38.4c8.533 0 12.8-4.267 12.8-12.8v-38.4c0-8.533-4.267-12.8-12.8-12.8Z"/></g></svg>'
};
const FileText = {
  name: "file-text",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path fill-rule="evenodd" d="M896 947.256V261.731c0-3.396-1.35-6.653-3.75-9.053L707.26 67.802a12.8 12.8 0 0 0-9.047-3.746L140.801 64C133.731 64 128 69.73 128 76.8v870.456c0 7.072 5.73 12.8 12.8 12.8h742.4c7.072 0 12.8-5.728 12.8-12.8m-64-51.2H192v-768.32l448-4.16v183.617c0 7.07 5.73 12.8 12.8 12.8H832v576.063ZM703.743 154.77l101.511 101.452L704 255.993l-.257-101.223ZM256 460.853v38.4c0 7.07 5.73 12.8 12.8 12.8h486.4c7.07 0 12.8-5.73 12.8-12.8v-38.4c0-7.069-5.73-12.8-12.8-12.8H268.8c-7.07 0-12.8 5.731-12.8 12.8Zm0 192v38.4c0 7.07 5.73 12.803 12.8 12.803h294.4c7.07 0 12.8-5.733 12.8-12.803v-38.4c0-7.069-5.73-12.8-12.8-12.8H268.8c-7.07 0-12.8 5.731-12.8 12.8Z"/></svg>'
};
const FileZip = {
  name: "file-zip",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><g fill-rule="evenodd"><path d="m140.801 64 557.412.056A12.8 12.8 0 0 1 707 67.55l.26.252 184.99 184.876a12.837 12.837 0 0 1 3.75 8.69v685.888c0 6.97-5.563 12.634-12.493 12.8H140.8c-6.967 0-12.634-5.563-12.8-12.493V76.8c0-7.07 5.732-12.8 12.801-12.8ZM192 127.737v768.32h640V319.992H652.8c-7.07 0-12.8-5.73-12.8-12.8v-179.14H320v51.2c0 7.07-5.73 12.8-12.8 12.8h-38.4c-7.07 0-12.8-5.73-12.8-12.8v-51.2l-64-.316Zm511.743 27.033L704 255.993l101.254.23-101.51-101.453Z"/><path d="M320 243.253v-38.4c0-7.069 5.73-12.8 12.8-12.8h38.4c7.07 0 12.8 5.731 12.8 12.8v38.4c0 7.07-5.73 12.8-12.8 12.8h-38.4c-7.07 0-12.8-5.73-12.8-12.8ZM256 307.253v-38.4c0-7.069 5.73-12.8 12.8-12.8h38.4c7.07 0 12.8 5.731 12.8 12.8v38.4c0 7.07-5.73 12.8-12.8 12.8h-38.4c-7.07 0-12.8-5.73-12.8-12.8ZM320 332.853v38.4c0 7.07 5.73 12.8 12.8 12.8h38.4c7.07 0 12.8-5.73 12.8-12.8v-38.4c0-7.069-5.73-12.8-12.8-12.8h-38.4c-7.07 0-12.8 5.731-12.8 12.8ZM256 435.253v-38.4c0-7.069 5.73-12.8 12.8-12.8h38.4c7.07 0 12.8 5.731 12.8 12.8v38.4c0 7.07-5.73 12.8-12.8 12.8h-38.4c-7.07 0-12.8-5.73-12.8-12.8ZM320 460.853v38.4c0 7.07 5.73 12.8 12.8 12.8h38.4c7.07 0 12.8-5.73 12.8-12.8v-38.4c0-7.069-5.73-12.8-12.8-12.8h-38.4c-7.07 0-12.8 5.731-12.8 12.8ZM268.8 512.053h38.4c7.07 0 12.8 5.731 12.8 12.8v51.2h115.2c7.07 0 12.8 5.731 12.8 12.8v166.403c0 7.066-5.73 12.8-12.8 12.8H268.8c-7.07 0-12.8-5.734-12.8-12.8V524.853c0-7.069 5.73-12.8 12.8-12.8Z"/><path d="M320 704.05631 320 640.05311 384 640.05311 384 704.05631z"/></g></svg>'
};
const File = {
  name: "file",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path fill-rule="evenodd" d="M896 261.731v685.525c0 7.072-5.728 12.8-12.8 12.8H140.8c-7.07 0-12.8-5.728-12.8-12.8V76.8c0-7.07 5.732-12.8 12.801-12.8l557.412.056a12.8 12.8 0 0 1 9.047 3.746l184.99 184.876c2.4 2.4 3.75 5.657 3.75 9.053m-90.746-5.509L703.744 154.77 704 255.993l101.254.23ZM192 896.056h640V319.993H652.8c-7.07 0-12.8-5.73-12.8-12.8V123.577l-448 4.16v768.32Z"/></svg>'
};
const FilterFilled = {
  name: "filter-filled",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path d="M259.776 323.776a12.8 12.8 0 0 1-3.776-9.088V268.8c0-7.04 5.76-12.8 12.8-12.8h486.4c7.04 0 12.8 5.76 12.8 12.8v45.888a12.8 12.8 0 0 1-3.776 9.088L576 512v248.064a12.8 12.8 0 0 1-7.04 11.52l-102.4 51.2a12.8 12.8 0 0 1-18.56-11.52V512L259.776 323.776z"/></svg>'
};
const Filter = {
  name: "filter",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path d="M883.2 128c7.04 0 12.8 5.76 12.8 12.8v140.16a12.8 12.8 0 0 1-6.016 10.816L640 448v376.064a12.8 12.8 0 0 1-7.04 11.52l-230.4 115.2a12.8 12.8 0 0 1-18.56-11.52V448L134.016 291.776a12.8 12.8 0 0 1-6.016-10.88V140.8c0-7.04 5.76-12.8 12.8-12.8h742.4zM832 191.936H192v60.608l256 160v443.84l128-64v-379.84L832 252.48v-60.544z"/></svg>'
};
const FitToView = {
  name: "fit-to-view",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path fill-rule="evenodd" d="M320 140.8v38.4c0 7.07-5.73 12.8-12.8 12.8H192v115.2c0 7.07-5.73 12.8-12.8 12.8h-38.4c-7.07 0-12.8-5.73-12.8-12.8V140.8c0-7.07 5.73-12.8 12.8-12.8h166.4c7.07 0 12.8 5.73 12.8 12.8M883.2 320h-38.4c-7.072 0-12.8-5.73-12.8-12.8V192H716.8c-7.07 0-12.8-5.73-12.8-12.8v-38.4c0-7.07 5.73-12.8 12.8-12.8h166.4c7.072 0 12.8 5.73 12.8 12.8v166.4c0 7.07-5.728 12.8-12.8 12.8m-256 256H396.8c-7.07 0-12.8-5.73-12.8-12.8V460.8c0-7.07 5.73-12.8 12.8-12.8h230.4c7.07 0 12.8 5.73 12.8 12.8v102.4c0 7.07-5.73 12.8-12.8 12.8M140.8 704h38.4c7.07 0 12.8 5.73 12.8 12.8V832h115.2c7.07 0 12.8 5.728 12.8 12.8v38.4c0 7.072-5.73 12.8-12.8 12.8H140.8c-7.07 0-12.8-5.728-12.8-12.8V716.8c0-7.07 5.73-12.8 12.8-12.8M704 883.2v-38.4c0-7.072 5.73-12.8 12.8-12.8H832V716.8c0-7.07 5.728-12.8 12.8-12.8h38.4c7.072 0 12.8 5.73 12.8 12.8v166.4c0 7.072-5.728 12.8-12.8 12.8H716.8c-7.07 0-12.8-5.728-12.8-12.8"/></svg>'
};
const FolderAdd = {
  name: "folder-add",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><g fill-rule="evenodd"><path d="M423.953 64a12.8 12.8 0 0 1 9.051 3.749L557.255 192h326.047c7.072 0 12.698 5.83 12.698 12.9v102.3c0 7.07-5.728 12.8-12.8 12.8h-38.4c-7.072 0-12.8-5.73-12.8-12.8V256H128v512h179.2c7.07 0 12.8 5.728 12.8 12.8v38.4c0 7.072-5.73 12.8-12.8 12.8H76.696C69.626 832 64 826.17 64 819.098V76.8C64 69.73 69.73 64 76.8 64h347.153Zm-21.201 64H128v64h338.752l-64-64Z"/><path d="M672 384c159.06 0 288 128.942 288 288 0 159.06-128.94 288-288 288-159.058 0-288-128.94-288-288 0-159.058 128.942-288 288-288Zm0 64c-123.712 0-224 100.288-224 224s100.288 224 224 224 224-100.288 224-224-100.288-224-224-224Z"/><path d="M652.8 512c-7.07 0-12.8 5.73-12.8 12.8V640H524.8c-7.07 0-12.8 5.73-12.8 12.8v38.4c0 7.07 5.73 12.8 12.8 12.8H640v115.2c0 7.072 5.73 12.8 12.8 12.8h38.4c7.07 0 12.8-5.728 12.8-12.8V704h115.2c7.072 0 12.8-5.73 12.8-12.8v-38.4c0-7.07-5.728-12.8-12.8-12.8H704V524.8c0-7.07-5.73-12.8-12.8-12.8h-38.4Z"/></g></svg>'
};
const FolderOpen = {
  name: "folder-open",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path d="M76.8 896A12.8 12.8 0 0 1 64 883.2V192a64 64 0 0 1 56.512-63.552L128 128h301.248l128 128H883.2c7.04 0 12.8 5.76 12.8 12.8V448h111.04a12.8 12.8 0 0 1 12.288 16.32l-120.704 422.4a12.8 12.8 0 0 1-12.288 9.28H76.8zm862.336-384h-698.88L148.8 832h698.88l91.456-320zM402.752 192H128v64h338.752l-64-64zM832 320H128v351.808l61.376-214.528A12.8 12.8 0 0 1 201.6 448H832V320z"/></svg>'
};
const Folder = {
  name: "folder",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path fill-rule="evenodd" d="m429.255 64 128 128H947.2c6.97 0 12.634 5.566 12.8 12.493V883.2c0 6.97-5.563 12.634-12.493 12.8H76.8c-6.967 0-12.634-5.563-12.8-12.493V76.8c0-6.967 5.566-12.634 12.493-12.8h352.762ZM896 256H128v576h768V256ZM402.752 128H128v64h338.752l-64-64Z"/></svg>'
};
const ForScreen = {
  name: "for-screen",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path d="M947.2 128c7.04 0 12.8 5.184 12.8 11.648v616.704c0 6.4-5.76 11.648-12.8 11.648H704v-64h192V192H128v512h192v64H76.8c-7.04 0-12.8-5.184-12.8-11.648V139.648c0-6.4 5.76-11.648 12.8-11.648h870.4z"/><path d="M517.76 593.152a12.8 12.8 0 0 1 5.696 5.76L694.72 941.504A12.8 12.8 0 0 1 683.264 960H340.736a12.8 12.8 0 0 1-11.52-18.56l171.328-342.528a12.8 12.8 0 0 1 17.152-5.76zM512 719.04 423.488 896h176.96L512 719.04z"/></svg>'
};
const Forward = {
  name: "forward",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path fill-rule="evenodd" d="m903.437 500.652-352.445-352.44c-4.998-4.999-4.999-12.866 0-17.865l27.39-27.39c4.998-5 12.85-5.014 17.849-.016l388.793 388.794c4.998 4.998 4.998 12.835 0 17.834L596.231 898.362c-4.999 4.998-12.85 4.985-17.85-.013l-27.39-27.392c-4.998-4.999-4.997-12.864.001-17.863l352.445-352.442ZM76.8 703.966l307.136.03.064 191.23c0 3.577 1.493 6.982 4.118 9.408 5.195 4.793 13.293 4.473 18.087-.724l353.782-383.262c4.525-4.903 4.525-12.46 0-17.364L406.205 120.022a12.8 12.8 0 0 0-9.405-4.118c-7.07 0-12.8 5.73-12.8 12.8l-.064 191.292-307.136-.03c-7.07 0-12.8 5.73-12.8 12.8v358.4c0 7.07 5.73 12.8 12.8 12.8Zm371.132-444.292 232.96 252.288-232.96 252.352-.015-124.31-319.92.022v-256l319.92-.022.015-124.33Z"/></svg>'
};
const FullscreenExit = {
  name: "fullscreen-exit",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path fill-rule="evenodd" d="M64 204.8v38.4c0 7.07 5.73 12.8 12.8 12.8h166.4c7.07 0 12.8-5.73 12.8-12.8V76.8c0-7.07-5.73-12.8-12.8-12.8h-38.4c-7.07 0-12.8 5.73-12.8 12.8V192H76.8c-7.07 0-12.8 5.73-12.8 12.8ZM819.2 64h-38.4c-7.072 0-12.8 5.73-12.8 12.8v166.4c0 7.07 5.728 12.8 12.8 12.8h166.4c7.072 0 12.8-5.73 12.8-12.8v-38.4c0-7.07-5.728-12.8-12.8-12.8H832V76.8c0-7.07-5.728-12.8-12.8-12.8ZM268.8 768h486.4c7.066 0 12.8-5.728 12.8-12.8V268.797c0-7.07-5.734-12.797-12.8-12.797H268.8c-7.068 0-12.8 5.727-12.8 12.797V755.2c0 7.072 5.732 12.8 12.8 12.8ZM320 320h384v384H320V320ZM76.8 768h166.4c7.07 0 12.8 5.728 12.8 12.8v166.4c0 7.072-5.73 12.8-12.8 12.8h-38.4c-7.07 0-12.8-5.728-12.8-12.8V832H76.8c-7.07 0-12.8-5.728-12.8-12.8v-38.4c0-7.072 5.73-12.8 12.8-12.8ZM960 819.2v-38.4c0-7.072-5.728-12.8-12.8-12.8H780.8c-7.072 0-12.8 5.728-12.8 12.8v166.4c0 7.072 5.728 12.8 12.8 12.8h38.4c7.072 0 12.8-5.728 12.8-12.8V832h115.2c7.072 0 12.8-5.728 12.8-12.8Z"/></svg>'
};
const Fullscreen = {
  name: "fullscreen",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path fill-rule="evenodd" d="M256 115.2V76.8c0-7.07-5.73-12.8-12.8-12.8H76.8C69.73 64 64 69.73 64 76.8v166.4c0 7.07 5.73 12.8 12.8 12.8h38.4c7.07 0 12.8-5.73 12.8-12.8V128h115.2c7.07 0 12.8-5.73 12.8-12.8ZM947.2 256h-38.4c-7.072 0-12.8-5.73-12.8-12.8V128H780.8c-7.072 0-12.8-5.73-12.8-12.8V76.8c0-7.07 5.728-12.8 12.8-12.8h166.4c7.072 0 12.8 5.73 12.8 12.8v166.4c0 7.07-5.728 12.8-12.8 12.8Zm-128 576H204.8c-7.07 0-12.8-5.728-12.8-12.8V204.8c0-7.07 5.73-12.8 12.8-12.8h614.4c7.072 0 12.8 5.73 12.8 12.8v614.4c0 7.072-5.728 12.8-12.8 12.8ZM768 256H256v512h512V256ZM115.2 768H76.8c-7.07 0-12.8 5.728-12.8 12.8v166.4c0 7.072 5.73 12.8 12.8 12.8h166.4c7.07 0 12.8-5.728 12.8-12.8v-38.4c0-7.072-5.73-12.8-12.8-12.8H128V780.8c0-7.072-5.73-12.8-12.8-12.8ZM768 947.2v-38.4c0-7.072 5.728-12.8 12.8-12.8H896V780.8c0-7.072 5.728-12.8 12.8-12.8h38.4c7.072 0 12.8 5.728 12.8 12.8v166.4c0 7.072-5.728 12.8-12.8 12.8H780.8c-7.072 0-12.8-5.728-12.8-12.8Z"/></svg>'
};
const Gif = {
  name: "gif",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path fill-rule="evenodd" d="M128 179.2V76.8c0-7.07 5.73-12.8 12.8-12.8h742.4c7.072 0 12.8 5.73 12.8 12.8v102.4c0 7.07-5.728 12.8-12.8 12.8h-38.4c-7.072 0-12.8-5.73-12.8-12.8V128H192v51.2c0 7.07-5.73 12.8-12.8 12.8h-38.4c-7.07 0-12.8-5.73-12.8-12.8ZM64 448c0-84.83 66.019-154.243 149.48-159.66L224 288h101.808c65.784 0 119.678 50.904 124.442 115.471.186 5.087-3.887 9.313-8.977 9.313h-41.881c-7.07 0-12.65-5.697-14.11-12.614-5.332-25.26-26.35-44.737-52.385-47.761l-7.089-.409H224c-49.9 0-90.909 38.073-95.56 86.755L128 448v128c0 49.9 38.073 90.909 86.755 95.56L224 672h130.592c15.71 0 28.775-11.32 31.484-26.248l.516-5.752-.055-90.112-104.124.029c-7.07.002-12.804-5.73-12.804-12.8V498.72c0-7.069 5.731-12.8 12.8-12.8h155.383c7.069 0 12.8 5.731 12.8 12.8V640c0 49.9-38.073 90.906-86.755 95.558l-9.245.442H224c-84.83 0-154.242-66.019-159.66-149.48L64 576V448Zm496.81-160h-38.4c-7.07 0-12.8 5.73-12.8 12.8v422.4c0 7.072 5.73 12.8 12.8 12.8h38.4c7.068 0 12.8-5.728 12.8-12.8V300.8c0-7.07-5.732-12.8-12.8-12.8Zm89.6 0h294.396c7.072 0 12.8 5.73 12.8 12.8v38.4c0 7.07-5.728 12.8-12.8 12.8H701.61v128h243.196c7.072 0 12.8 5.73 12.8 12.8v38.4c0 7.07-5.728 12.8-12.8 12.8H701.61v179.2c0 7.072-5.732 12.8-12.8 12.8h-38.4c-7.07 0-12.8-5.728-12.8-12.8V300.8c0-7.07 5.73-12.8 12.8-12.8ZM140.8 960h742.4c7.072 0 12.8-5.728 12.8-12.8V844.8c0-7.072-5.728-12.8-12.8-12.8h-38.4c-7.072 0-12.8 5.728-12.8 12.8V896H192v-51.2c0-7.072-5.73-12.8-12.8-12.8h-38.4c-7.07 0-12.8 5.728-12.8 12.8v102.4c0 7.072 5.73 12.8 12.8 12.8Z"/></svg>'
};
const Github = {
  name: "github",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path d="M511.6 76.3C264.3 76.2 64 276.4 64 523.5 64 718.9 189.3 885 363.8 946c23.5 5.9 19.9-10.8 19.9-22.2v-77.5c-135.7 15.9-141.2-73.9-150.3-88.9C215 726 171.5 718 184.5 703c30.9-15.9 62.4 4 98.9 57.9 26.4 39.1 77.9 32.5 104 26 5.7-23.5 17.9-44.5 34.7-60.8-140.6-25.2-199.2-111-199.2-213 0-49.5 16.3-95 48.3-131.7-20.4-60.5 1.9-112.3 4.9-120 58.1-5.2 118.5 41.6 123.2 45.3 33-8.9 70.7-13.6 112.9-13.6 42.4 0 80.2 4.9 113.5 13.9 11.3-8.6 67.3-48.8 121.3-43.9 2.9 7.7 24.7 58.3 5.5 118 32.4 36.8 48.9 82.7 48.9 132.3 0 102.2-59 188.1-200 212.9a127.5 127.5 0 0 1 38.1 91v112.5c.8 9 0 17.9 15 17.9 177.1-59.7 304.6-227 304.6-424.1 0-247.2-200.4-447.3-447.5-447.3z"/></svg>'
};
const Gitlab = {
  name: "gitlab",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path d="M913.9 552.2 805 181.4v-.1c-7.6-22.9-25.7-36.5-48.3-36.5-23.4 0-42.5 13.5-49.7 35.2l-71.4 213H388.8l-71.4-213c-7.2-21.7-26.3-35.2-49.7-35.2-23.1 0-42.5 14.8-48.4 36.6L110.5 552.2c-4.4 14.7 1.2 31.4 13.5 40.7l368.5 276.4c2.6 3.6 6.2 6.3 10.4 7.8l8.6 6.4 8.5-6.4c4.9-1.7 9-4.7 11.9-8.9l368.4-275.4c12.4-9.2 18-25.9 13.6-40.6zM751.7 193.4c1-1.8 2.9-1.9 3.5-1.9 1.1 0 2.5.3 3.4 3L818 394.3H684.5l67.2-200.9zm-487.4 1c.9-2.6 2.3-2.9 3.4-2.9 2.7 0 2.9.1 3.4 1.7l67.3 201.2H206.5l57.8-200zM158.8 558.7l28.2-97.3 202.4 270.2-230.6-172.9zm73.9-116.4h122.1l90.8 284.3-212.9-284.3zM512.9 776 405.7 442.3H620L512.9 776zm157.9-333.7h119.5L580 723.1l90.8-280.8zm-40.7 293.9 207.3-276.7 29.5 99.2-236.8 177.5z"/></svg>'
};
const Global = {
  name: "global",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path d="M512 64c136.96 0 259.584 61.44 341.76 158.336l1.472-.896 7.168 11.392A445.888 445.888 0 0 1 958.848 480H960v64h-1.152a446.016 446.016 0 0 1-103.04 255.296l-.576.704A448 448 0 0 1 65.088 544H64v-64h1.152A448 448 0 0 1 512 64zm-32 639.872c-54.08 2.944-107.84 12.608-160 29.184v111.552a381.44 381.44 0 0 0 160 50.048V703.872zm64-.256v191.04a381.632 381.632 0 0 0 160.128-50.112v-113.28A645.376 645.376 0 0 0 544 703.616zm224 51.008v43.584c8.64-7.68 16.96-15.872 24.832-24.32l5.12-5.568a624.565 624.565 0 0 0-29.888-13.696zM227.84 770.24c8.96 9.792 18.304 19.136 28.16 27.904V757.12c-9.472 4.096-18.816 8.512-28.16 13.12zM256 544H129.28c5.312 64 26.24 123.456 59.008 174.656A705.305 705.305 0 0 1 256 687.872V544zm638.72 0H768v141.632a705.985 705.985 0 0 1 69.12 30.72c31.936-50.56 52.352-109.312 57.6-172.288zM480 544H320v122.24a709.248 709.248 0 0 1 160-26.432V544zm224 0H544v95.552c53.952 2.176 107.648 10.56 160 25.024V544zM189.44 303.488l-6.72 10.88A381.632 381.632 0 0 0 129.344 480H256V333.632c-22.528-8.832-44.736-18.88-66.56-30.08zm354.56 78.4V480h160l.064-123.2a712.87 712.87 0 0 1-160 25.088zM320 355.2V480h160v-98.368A709.312 709.312 0 0 1 320 355.2zm515.84-49.6c-22.144 11.328-44.8 21.312-67.84 30.08V480h126.72a381.696 381.696 0 0 0-58.88-174.4zM544 129.344v188.48a645.312 645.312 0 0 0 160-27.648l.064-110.72a381.632 381.632 0 0 0-160-50.112zm-64 0a381.632 381.632 0 0 0-160 50.048v108.992a646.519 646.519 0 0 0 160 29.184V129.28zm288.064 96.512v40.96c9.472-4.096 18.88-8.384 28.224-12.928-8.96-9.792-18.304-19.2-28.224-28.032zM229.44 251.968c8.832 4.288 17.664 8.448 26.56 12.352v-38.528a394.187 394.187 0 0 0-26.56 26.176z"/></svg>'
};
const GridCompact = {
  name: "grid-compact",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path d="M907.84 713.024a12.8 12.8 0 0 0 0-18.048l-27.136-27.2a12.8 12.8 0 0 0-18.112 0l-126.72 126.72-126.72-126.72a12.8 12.8 0 0 0-18.048 0l-27.2 27.2a12.8 12.8 0 0 0 0 18.048l162.944 162.944a12.8 12.8 0 0 0 18.112 0l162.88-162.944zm0-402.048a12.8 12.8 0 0 1 0 18.048l-27.136 27.2a12.8 12.8 0 0 1-18.112 0l-126.72-126.72-126.72 126.72a12.8 12.8 0 0 1-18.048 0L563.84 328.96a12.8 12.8 0 0 1 0-18.048l162.944-162.944a12.8 12.8 0 0 1 18.112 0l162.88 162.944zM140.8 192h294.4q12.8 0 12.8 12.8v38.4q0 12.8-12.8 12.8H140.8q-12.8 0-12.8-12.8v-38.4q0-12.8 12.8-12.8Zm0 192h294.4q12.8 0 12.8 12.8v38.4q0 12.8-12.8 12.8H140.8q-12.8 0-12.8-12.8v-38.4q0-12.8 12.8-12.8Zm0 192h294.4q12.8 0 12.8 12.8v38.4q0 12.8-12.8 12.8H140.8q-12.8 0-12.8-12.8v-38.4q0-12.8 12.8-12.8Zm0 192h294.4q12.8 0 12.8 12.8v38.4q0 12.8-12.8 12.8H140.8q-12.8 0-12.8-12.8v-38.4q0-12.8 12.8-12.8Z"/></svg>'
};
const GridLoose = {
  name: "grid-loose",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path d="M907.84 713.024a12.8 12.8 0 0 0 0-18.048l-27.136-27.2a12.8 12.8 0 0 0-18.112 0l-126.72 126.72-126.72-126.72a12.8 12.8 0 0 0-18.048 0l-27.2 27.2a12.8 12.8 0 0 0 0 18.048l162.944 162.944a12.8 12.8 0 0 0 18.112 0l162.88-162.944zm0-402.048a12.8 12.8 0 0 1 0 18.048l-27.136 27.2a12.8 12.8 0 0 1-18.112 0l-126.72-126.72-126.72 126.72a12.8 12.8 0 0 1-18.048 0L563.84 328.96a12.8 12.8 0 0 1 0-18.048l162.944-162.944a12.8 12.8 0 0 1 18.112 0l162.88 162.944zM140.8 192h294.4q12.8 0 12.8 12.8v230.4q0 12.8-12.8 12.8H140.8q-12.8 0-12.8-12.8V204.8q0-12.8 12.8-12.8Zm0 384h294.4q12.8 0 12.8 12.8v230.4q0 12.8-12.8 12.8H140.8q-12.8 0-12.8-12.8V588.8q0-12.8 12.8-12.8Z"/></svg>'
};
const GridMedium = {
  name: "grid-medium",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path d="M907.84 713.024a12.8 12.8 0 0 0 0-18.048l-27.136-27.2a12.8 12.8 0 0 0-18.112 0l-126.72 126.72-126.72-126.72a12.8 12.8 0 0 0-18.048 0l-27.2 27.2a12.8 12.8 0 0 0 0 18.048l162.944 162.944a12.8 12.8 0 0 0 18.112 0l162.88-162.944zm0-402.048a12.8 12.8 0 0 1 0 18.048l-27.136 27.2a12.8 12.8 0 0 1-18.112 0l-126.72-126.72-126.72 126.72a12.8 12.8 0 0 1-18.048 0L563.84 328.96a12.8 12.8 0 0 1 0-18.048l162.944-162.944a12.8 12.8 0 0 1 18.112 0l162.88 162.944zM140.8 192h294.4q12.8 0 12.8 12.8v102.4q0 12.8-12.8 12.8H140.8q-12.8 0-12.8-12.8V204.8q0-12.8 12.8-12.8Zm0 256h294.4q12.8 0 12.8 12.8v102.4q0 12.8-12.8 12.8H140.8q-12.8 0-12.8-12.8V460.8q0-12.8 12.8-12.8Zm0 256h294.4q12.8 0 12.8 12.8v102.4q0 12.8-12.8 12.8H140.8q-12.8 0-12.8-12.8V716.8q0-12.8 12.8-12.8Z"/></svg>'
};
const Heart = {
  name: "heart",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path d="M354.368 63.424A288.512 288.512 0 0 0 66.624 334.08l-1.152 17.344a596.288 596.288 0 0 0 399.36 592.384L512 960.256l47.552-16.64 17.088-6.208A604.16 604.16 0 0 0 964.8 361.344l.192-7.872a288.128 288.128 0 0 0-288-288.64l-16.64.448a286.336 286.336 0 0 0-137.6 44.288l-6.144 4.096-6.4-4.416a286.848 286.848 0 0 0-155.776-45.824zm0 64c52.48 0 102.144 18.048 141.888 50.624l20.096 16.512 20.288-16.32a224 224 0 0 1 364.352 174.528l-.192 8.512a540.48 540.48 0 0 1-346.56 516.16l-16.192 5.888L512 892.352l-26.112-9.024a532.16 532.16 0 0 1-356.48-528.384l1.088-16.896a224.512 224.512 0 0 1 223.872-210.56z"/><path d="M320 192a32 32 0 0 1 0 64 64 64 0 0 0-64 64 32 32 0 0 1-64 0 128 128 0 0 1 128-128z"/></svg>'
};
const Heat = {
  name: "heat",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><defs><style/></defs><path d="M735.232 513.408C768.256 204.736 462.848 0 462.848 0 560.768 174.144 128 393.152 128 707.2c0 196.864 218.816 316.8 388.288 316.8 559.744 0 346.88-665.344 306.304-701.44-12.672 73.6-60.416 178.56-87.36 190.848zm-224.192-9.344s131.392 76.16 120.512 267.136c0 0 19.2 2.752 73.28-64.896 7.744 52.672-8.32 232.96-167.488 232.96s-308.672-146.88-26.304-435.2z"/></svg>'
};
const History = {
  name: "history",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path d="M512 64a448 448 0 0 1 377.344 689.6L731.2 595.392l-.64-.896A12.8 12.8 0 0 1 742.08 576h148.608A384 384 0 1 0 760 805.12l45.376 45.44A448 448 0 1 1 512 64z"/><path d="m544 270.08-.064 265.792L657.6 674.24a12.8 12.8 0 0 1-1.728 18.048l-29.696 24.32a12.8 12.8 0 0 1-17.984-1.728L480 558.912V270.08c0-7.04 5.76-12.8 12.8-12.8h38.4c7.04 0 12.8 5.76 12.8 12.8z"/></svg>'
};
const Holder = {
  name: "holder",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path d="M320 256a64 64 0 1 0 128 0 64 64 0 1 0-128 0Zm256 0a64 64 0 1 0 128 0 64 64 0 1 0-128 0ZM320 512a64 64 0 1 0 128 0 64 64 0 1 0-128 0Zm256 0a64 64 0 1 0 128 0 64 64 0 1 0-128 0ZM320 768a64 64 0 1 0 128 0 64 64 0 1 0-128 0Zm256 0a64 64 0 1 0 128 0 64 64 0 1 0-128 0Z"/></svg>'
};
const Home = {
  name: "home",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path fill-rule="evenodd" d="m520.055 70.705.26.217 413.4 354.34c8.928 7.657 3.66 22.224-7.978 22.515l-.355.004h-93.401v435.201c0 6.964-5.563 12.634-12.494 12.8H204.784c-6.967 0-12.634-5.569-12.796-12.493l-.004-.307v-435.2H98.586c-11.765 0-17.243-14.491-8.597-22.285l.266-.234L503.654 70.922a12.8 12.8 0 0 1 16.401-.217Zm-8.07 77.365L236.915 383.782h19.072v448h127.998V588.586c0-7.07 5.73-12.8 12.8-12.8h230.4c7.07 0 12.8 5.73 12.8 12.8v243.196h128.003v-448h19.008L511.985 148.07Zm63.999 491.716h-128v191.996h128V639.786Z"/></svg>'
};
const Import = {
  name: "import",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path d="M690.368 128c7.04 0 12.8 5.76 12.8 12.8v37.888a12.8 12.8 0 0 1-12.8 12.8l-550.08.512a12.8 12.8 0 0 0-12.8 12.8v680.384c0 7.04 5.76 12.8 12.8 12.8h691.648l.064-563.52c0-7.04 5.76-12.8 12.8-12.8h38.4c7.04 0 12.8 5.76 12.8 12.8v582.72a44.8 44.8 0 0 1-38.72 44.416l-6.08.384-774.912-.384a12.8 12.8 0 0 1-12.8-12.8v-808c0-7.04 5.76-12.8 12.8-12.8h614.08zm160.96 16.832 27.136 27.2a12.8 12.8 0 0 1 0 18.112l-513.28 513.792L627.2 704c7.04 0 12.8 5.76 12.8 12.8v38.4a12.8 12.8 0 0 1-12.8 12.8H268.8a12.8 12.8 0 0 1-12.8-12.8V396.8c0-7.04 5.76-12.8 12.8-12.8h38.4c7.04 0 12.8 5.76 12.8 12.8v261.888l513.216-513.856a12.8 12.8 0 0 1 18.112 0z"/></svg>'
};
const InfoCircleFilled = {
  name: "info-circle-filled",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path d="M512 0c282.752 0 512 229.248 512 512s-229.248 512-512 512S0 794.752 0 512 229.248 0 512 0zm25.6 405.333h-51.2a17.067 17.067 0 0 0-17.067 17.067v349.867c0 9.386 7.68 17.066 17.067 17.066h51.2a17.067 17.067 0 0 0 17.067-17.066V422.4a17.067 17.067 0 0 0-17.067-17.067zm0-170.666h-51.2a17.067 17.067 0 0 0-17.067 17.066v51.2c0 9.387 7.68 17.067 17.067 17.067h51.2a17.067 17.067 0 0 0 17.067-17.067v-51.2a17.067 17.067 0 0 0-17.067-17.066z"/></svg>'
};
const InfoCircle = {
  name: "info-circle",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path d="M512 960c247.424 0 448-200.576 448-448S759.424 64 512 64 64 264.576 64 512s200.576 448 448 448Zm0-64c-212.077 0-384-171.923-384-384s171.923-384 384-384 384 171.923 384 384-171.923 384-384 384Zm19.2-576c6.967 0 12.634-5.566 12.796-12.493L544 268.8c0-6.967-5.566-12.634-12.493-12.796L492.8 256c-6.967 0-12.634 5.566-12.796 12.493L480 307.2c0 6.967 5.566 12.634 12.493 12.796L531.2 320Zm0 448c6.967 0 12.634-5.566 12.796-12.493L544 396.8c0-6.967-5.566-12.634-12.493-12.8H492.8c-6.967 0-12.634 5.566-12.796 12.493L480 755.2c0 6.967 5.566 12.634 12.493 12.8H531.2Z"/></svg>'
};
const Info = {
  name: "info",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path d="M448 224a64 64 0 1 0 128 0 64 64 0 1 0-128 0zm96 168h-64c-4.4 0-8 3.6-8 8v464c0 4.4 3.6 8 8 8h64c4.4 0 8-3.6 8-8V400c0-4.4-3.6-8-8-8z"/></svg>'
};
const Insurance = {
  name: "insurance",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path d="M518.656 66.304H507.2L138.24 133.12a12.8 12.8 0 0 0-10.496 12.608V334.4c0 259.456 141.888 498.048 369.792 622.016l15.616 8.512 33.28-18.88a707.2 707.2 0 0 0 349.504-609.92V145.664a12.8 12.8 0 0 0-10.496-12.544L518.72 66.24zm-5.824 63.936 319.04 58.176v147.648l-.32 20.736a643.008 643.008 0 0 1-299.52 523.008L512.64 891.52l-2.176-1.28A644.032 644.032 0 0 1 191.68 334.4l-.064-145.92 321.216-58.24z"/><path d="m261.76 420.224 28.096-27.008a12.8 12.8 0 0 1 17.92.128l164.8 164.864a12.8 12.8 0 0 0 18.112 0l251.648-251.52a12.8 12.8 0 0 1 18.112 0l27.136 27.136a12.8 12.8 0 0 1 0 18.112L490.368 649.152a12.8 12.8 0 0 1-18.112 0L261.632 438.528a12.8 12.8 0 0 1 .128-18.304z"/></svg>'
};
const Ip = {
  name: "ip",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path fill-rule="evenodd" d="M64 76.8v806.4c0 7.072 5.73 12.8 12.8 12.8h870.4c7.072 0 12.8-5.728 12.8-12.8V76.8c0-7.07-5.728-12.8-12.8-12.8H76.8C69.73 64 64 69.73 64 76.8ZM128 832V128h768v704H128Zm192-512h-51.2c-7.07 0-12.8-5.73-12.8-12.8v-38.4c0-7.07 5.73-12.8 12.8-12.8h166.4c7.07 0 12.8 5.73 12.8 12.8v38.4c0 7.07-5.73 12.8-12.8 12.8H384v320h51.2c7.07 0 12.8 5.728 12.8 12.8v38.4c0 7.072-5.73 12.8-12.8 12.8H268.8c-7.07 0-12.8-5.728-12.8-12.8v-38.4c0-7.072 5.73-12.8 12.8-12.8H320V320Zm204.8-64c-7.07 0-12.8 5.73-12.8 12.8v422.4c0 7.072 5.73 12.8 12.8 12.8h38.4c7.07 0 12.8-5.728 12.8-12.8V512h64c70.694 0 128-57.308 128-128 0-70.692-57.306-128-128-128H524.8ZM640 320c35.347 0 64 28.654 64 64 0 35.346-28.653 64-64 64h-64V320h64Z"/></svg>'
};
const Jumpz = {
  name: "jumpz",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path fill-rule="evenodd" d="m896.006 140.792.48 806.402c0 7.072-5.728 12.806-12.8 12.806H532.61c-7.07 0-12.8-5.728-12.8-12.8v-37.907c0-7.06 5.717-12.787 12.779-12.8L832.512 896V190.016H128.044l-.042 309.186C128 506.27 122.27 512 115.202 512H76.8c-7.07 0-12.8-5.73-12.8-12.8V140.8c0-7.07 5.73-12.8 12.8-12.8h806.406c7.066 0 12.8 5.726 12.8 12.792M269.292 448h260.079L146.874 832.32c-4.978 4.998-4.97 13.088.02 18.074l27.15 27.161c5.008 5.011 13.13 5.005 18.126-.019L576.625 491.25l.037 263.952c.001 7.068 5.732 12.798 12.8 12.798h38.402c7.07 0 12.8-5.73 12.8-12.8V396.8c0-7.07-5.73-12.8-12.8-12.8H269.292c-7.07 0-12.8 5.73-12.8 12.8v38.4c0 7.07 5.73 12.8 12.8 12.8"/></svg>'
};
const Key = {
  name: "key",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path d="M512 512a227.556 227.556 0 1 1-321.82 321.82A227.556 227.556 0 0 1 512 512zm-40.22 40.22A170.667 170.667 0 1 0 230.4 793.6a170.667 170.667 0 0 0 241.38-241.38z"/><path d="M431.56 592.44a113.778 113.778 0 1 1-160.94 160.94 113.778 113.778 0 0 1 160.94-160.94zm-40.221 40.221a56.889 56.889 0 1 0-80.498 80.498 56.889 56.889 0 0 0 80.555-80.555zm422.343-462.563 40.22 40.22-362.04 362.041-40.221-40.22z"/><path d="m813.682 250.539-40.22 40.22L612.58 129.82 652.8 89.6zm-80.384 80.44L693.02 371.2 532.14 210.318l40.22-40.22zM652.8 411.42l-40.22 40.22-120.72-120.66 40.278-40.22z"/></svg>'
};
const LayoutCompact = {
  name: "layout-compact",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path d="M896 140.8v38.4a12.8 12.8 0 0 1-12.8 12.8H140.8a12.8 12.8 0 0 1-12.8-12.8v-38.4c0-7.04 5.76-12.8 12.8-12.8h742.4c7.04 0 12.8 5.76 12.8 12.8zm0 128v38.4a12.8 12.8 0 0 1-12.8 12.8H140.8a12.8 12.8 0 0 1-12.8-12.8v-38.4c0-7.04 5.76-12.8 12.8-12.8h742.4c7.04 0 12.8 5.76 12.8 12.8zm0 128v38.4a12.8 12.8 0 0 1-12.8 12.8H140.8a12.8 12.8 0 0 1-12.8-12.8v-38.4c0-7.04 5.76-12.8 12.8-12.8h742.4c7.04 0 12.8 5.76 12.8 12.8zm0 128v38.4a12.8 12.8 0 0 1-12.8 12.8H140.8a12.8 12.8 0 0 1-12.8-12.8v-38.4c0-7.04 5.76-12.8 12.8-12.8h742.4c7.04 0 12.8 5.76 12.8 12.8zm0 128v38.4a12.8 12.8 0 0 1-12.8 12.8H140.8a12.8 12.8 0 0 1-12.8-12.8v-38.4c0-7.04 5.76-12.8 12.8-12.8h742.4c7.04 0 12.8 5.76 12.8 12.8zm0 128v38.4a12.8 12.8 0 0 1-12.8 12.8H140.8a12.8 12.8 0 0 1-12.8-12.8v-38.4c0-7.04 5.76-12.8 12.8-12.8h742.4c7.04 0 12.8 5.76 12.8 12.8z"/></svg>'
};
const LayoutLarge = {
  name: "layout-large",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path d="M907.84 713.024a12.8 12.8 0 0 0 0-18.048l-27.136-27.2a12.8 12.8 0 0 0-18.112 0l-126.72 126.72-126.72-126.72a12.8 12.8 0 0 0-18.048 0l-27.2 27.2a12.8 12.8 0 0 0 0 18.048l162.944 162.944a12.8 12.8 0 0 0 18.112 0l162.88-162.944zm0-402.048a12.8 12.8 0 0 1 0 18.048l-27.136 27.2a12.8 12.8 0 0 1-18.112 0l-126.72-126.72-126.72 126.72a12.8 12.8 0 0 1-18.048 0L563.84 328.96a12.8 12.8 0 0 1 0-18.048l162.944-162.944a12.8 12.8 0 0 1 18.112 0l162.88 162.944zM140.8 192h294.4q12.8 0 12.8 12.8v230.4q0 12.8-12.8 12.8H140.8q-12.8 0-12.8-12.8V204.8q0-12.8 12.8-12.8Zm0 384h294.4q12.8 0 12.8 12.8v230.4q0 12.8-12.8 12.8H140.8q-12.8 0-12.8-12.8V588.8q0-12.8 12.8-12.8Z"/></svg>'
};
const LayoutLoose = {
  name: "layout-loose",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path fill-rule="evenodd" d="M883.2 512H140.8c-7.07 0-12.8-5.73-12.8-12.8V204.8c0-7.07 5.73-12.8 12.8-12.8h742.4c7.072 0 12.8 5.73 12.8 12.8v294.4c0 7.07-5.728 12.8-12.8 12.8ZM192 256v192h640V256H192Zm691.2 640H140.8c-7.07 0-12.8-5.728-12.8-12.8V588.8c0-7.07 5.73-12.8 12.8-12.8h742.4c7.072 0 12.8 5.73 12.8 12.8v294.4c0 7.072-5.728 12.8-12.8 12.8ZM192 640v192h640V640H192Z"/></svg>'
};
const LayoutMedium = {
  name: "layout-medium",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path fill-rule="evenodd" d="M883.2 384H140.8c-7.07 0-12.8-5.73-12.8-12.8V204.8c0-7.07 5.73-12.8 12.8-12.8h742.4c7.072 0 12.8 5.73 12.8 12.8v166.4c0 7.07-5.728 12.8-12.8 12.8ZM192 256v64h640v-64H192Zm691.2 384H140.8c-7.07 0-12.8-5.73-12.8-12.8V460.8c0-7.07 5.73-12.8 12.8-12.8h742.4c7.072 0 12.8 5.73 12.8 12.8v166.4c0 7.07-5.728 12.8-12.8 12.8ZM192 512v64h640v-64H192Zm691.2 384H140.8c-7.07 0-12.8-5.728-12.8-12.8V716.8c0-7.07 5.73-12.8 12.8-12.8h742.4c7.072 0 12.8 5.73 12.8 12.8v166.4c0 7.072-5.728 12.8-12.8 12.8ZM192 768v64h640v-64H192Z"/></svg>'
};
const Layout = {
  name: "layout",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path fill-rule="evenodd" d="M883.2 128c6.97 0 12.634 5.566 12.796 12.493l.004.307v742.4c0 6.97-5.563 12.634-12.493 12.796l-.307.004H140.8c-6.967 0-12.634-5.563-12.796-12.493L128 883.2V140.8c0-6.967 5.566-12.634 12.493-12.796l.307-.004h742.4ZM832 384H384v448h448V384ZM320 192H192v640h128V192Zm512 0H384v128h448V192Z"/></svg>'
};
const LeftCircle = {
  name: "left-circle",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><g fill-rule="evenodd"><path d="M512 64c247.424 0 448 200.577 448 448 0 247.424-200.576 448-448 448-247.423 0-448-200.576-448-448C64 264.577 264.577 64 512 64Zm0 64c-212.077 0-384 171.923-384 384s171.923 384 384 384 384-171.923 384-384-171.923-384-384-384Z"/><path d="M410.919 513.867 635.343 738.22c5 4.998 5.069 12.934.07 17.933l-27.39 27.392c-5 4.998-13.105 4.761-18.104-.237L329.57 522.957c-4.999-4.999-5.182-13.16-.183-18.159L593.84 240.346c4.998-4.999 12.922-5.055 17.92-.056l27.39 27.39c5 4.998 4.784 13.083-.213 18.082L410.919 513.867Z"/></g></svg>'
};
const LeftDouble = {
  name: "left-double",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path d="M533.504 265.344a32 32 0 0 1 0 45.312L342.336 501.76a12.8 12.8 0 0 0 0 18.112l192.512 192.512a32 32 0 0 1-45.248 45.248L265.344 533.504a32 32 0 0 1 0-45.248L488.32 265.344a32 32 0 0 1 45.248 0zm224 0a32 32 0 0 1 0 45.312L566.336 501.76a12.8 12.8 0 0 0 0 18.112l192.512 192.512a32 32 0 1 1-45.248 45.248L489.344 533.504a32 32 0 0 1 0-45.248l222.912-222.912a32 32 0 0 1 45.248 0z"/></svg>'
};
const LeftFilled = {
  name: "left-filled",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path d="M512 0c282.784 0 512 229.216 512 512s-229.216 512-512 512S0 794.784 0 512 229.216 0 512 0zm54.944 294.784-208.16 208.16a12.8 12.8 0 0 0-1.632 16.096l1.6 2.016 208.192 208.16a12.8 12.8 0 0 0 18.112 0l27.136-27.136a12.8 12.8 0 0 0 0-18.112L440.224 512l171.968-171.968a12.8 12.8 0 0 0 0-18.112l-27.136-27.136a12.8 12.8 0 0 0-18.112 0z"/></svg>'
};
const Left = {
  name: "left",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path d="m313.575 521.104 253.659 253.659c4.998 4.998 12.869 5.001 17.867.003l27.39-27.39c5-5 4.939-12.927-.06-17.926L395.15 512.17l217.28-217.281c5-4.999 5.06-12.927.062-17.925l-27.39-27.39c-5-5-12.87-4.996-17.868.002L313.575 503.234c-4.998 4.998-4.998 12.872 0 17.87Z"/></svg>'
};
const LikeFilled = {
  name: "like-filled",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path d="M256 416v480H140.8a12.8 12.8 0 0 1-12.8-12.8V428.8c0-7.04 5.76-12.8 12.8-12.8H256zm352-288a64 64 0 0 1 64 64v179.2c0 7.04 5.76 12.8 12.8 12.8h168.64a96 96 0 0 1 89.92 129.728l-120 320A96 96 0 0 1 733.44 896H320V415.488a128 128 0 0 0 100.608-65.28l92.288-166.272A108.8 108.8 0 0 1 608 128z"/></svg>'
};
const Like = {
  name: "like",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path d="M140.8 896a12.8 12.8 0 0 1-12.8-12.8V428.8c0-7.04 5.76-12.8 12.8-12.8h150.912a128 128 0 0 0 121.472-87.552l36.48-109.376A133.184 133.184 0 0 1 576 128a96 96 0 0 1 96 96v147.2c0 7.04 5.76 12.8 12.8 12.8H832a110.912 110.912 0 0 1 103.808 149.76l-104.64 279.168A128 128 0 0 1 711.296 896H140.8zM256 480h-64v352h64V480zm320-288c-27.072 0-51.392 15.744-62.72 39.872l-2.944 7.424-36.48 109.44a192 192 0 0 1-153.792 129.152L320 832h391.296A64 64 0 0 0 768 797.696l3.2-7.232L875.904 511.36a46.912 46.912 0 0 0-37.568-62.912L832 448H684.8a76.8 76.8 0 0 1-76.416-68.928L608 371.2V224a32 32 0 0 0-32-32z"/></svg>'
};
const LineChart = {
  name: "line-chart",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path fill-rule="evenodd" d="M64 883.2V140.8c0-7.07 5.73-12.8 12.8-12.8h38.4c7.07 0 12.8 5.73 12.8 12.8V832h819.2c7.072 0 12.8 5.728 12.8 12.8v38.4c0 7.072-5.728 12.8-12.8 12.8H76.8c-7.07 0-12.8-5.728-12.8-12.8Zm511.615-412.287L457.64 347.817c-4.98-5.195-13.46-5.402-18.529-.294L209.249 579.232c-4.978 5.019-5.046 12.986-.027 17.964l27.5 27.28c5.019 4.98 12.498 5.337 17.476.317l193.353-194.968 118.26 123.317c4.962 5.174 12.98 5.105 18.054.04l368.788-368.099c5.043-5.03 4.851-13.461-.23-18.449l-27.233-26.716c-5.01-4.914-13.132-4.742-18.099.214l-331.476 330.78ZM947.2 768H204.8c-7.07 0-12.8-5.73-12.8-12.8v-38.4c0-7.07 5.73-12.8 12.8-12.8H896V396.8c0-7.07 5.728-12.8 12.8-12.8h38.4c7.072 0 12.8 5.73 12.8 12.8v358.4c0 7.07-5.728 12.8-12.8 12.8Z"/></svg>'
};
const Link = {
  name: "link",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path fill-rule="evenodd" d="M412.746 592.975c-66.156-75.417-63.255-190.282 8.703-262.24l181.06-181.06c74.98-74.98 196.55-74.98 271.526 0 74.983 74.981 74.983 196.549 0 271.53L747.328 547.911c-4.998 5-13.1 5-18.1 0l-27.154-27.152c-4.998-5-4.998-13.104 0-18.103L828.781 375.95c49.99-49.987 49.99-131.032 0-181.018-49.984-49.988-131.03-49.988-181.018 0l-181.012 181.01c-46.941 46.942-49.802 121.27-8.582 171.55 4.482 5.467 4.53 13.522-.47 18.52l-27.2 27.2c-4.998 5-13.091 5.077-17.753-.237Zm-263.07 280.497c-74.981-74.982-74.981-196.548 0-271.529l126.681-126.682c5-4.999 13.103-4.999 18.102 0l27.153 27.153c4.999 4.998 4.999 13.103 0 18.102l-126.68 126.68c-49.987 49.987-49.987 131.031 0 181.022 49.987 49.984 131.032 49.984 181.019 0L556.93 647.236c46.94-46.939 49.802-121.264 8.586-171.543-4.481-5.466-4.53-13.522.468-18.521l27.196-27.202c4.998-5 13.09-5.08 17.753.235 66.156 75.418 63.256 190.283-8.703 262.24L421.204 873.472c-74.98 74.982-196.548 74.982-271.529 0Z"/></svg>'
};
const List = {
  name: "list",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path fill-rule="evenodd" d="M332.636 256h358.728c7.069 0 12.636-5.151 12.636-12.22V76.22c0-7.069-5.567-12.22-12.636-12.22H332.636C325.567 64 320 69.151 320 76.22v167.56c0 7.069 5.567 12.22 12.636 12.22Zm447.998-64H832v704H192V192h51.368c7.07 0 12.632-5.563 12.632-12.632v-38.736c0-7.07-5.634-12.632-12.703-12.632H140.703c-7.069 0-12.703 5.627-12.703 12.696v806.606c0 7.072 5.849 12.698 12.918 12.698h742.167c7.065 0 12.915-5.626 12.915-12.698V140.696c0-7.07-5.632-12.696-12.704-12.696H780.704c-7.072 0-12.704 5.563-12.704 12.632v38.736c0 7.07 5.562 12.632 12.634 12.632ZM384 192v-64h256v64H384ZM256 435.2v-38.4c0-7.07 5.73-12.8 12.8-12.8h486.4c7.07 0 12.8 5.73 12.8 12.8v38.4c0 7.07-5.73 12.8-12.8 12.8H268.8c-7.07 0-12.8-5.73-12.8-12.8Zm0 128v-38.4c0-7.07 5.73-12.8 12.8-12.8h486.4c7.07 0 12.8 5.73 12.8 12.8v38.4c0 7.07-5.73 12.8-12.8 12.8H268.8c-7.07 0-12.8-5.73-12.8-12.8Zm0 128v-38.4c0-7.07 5.73-12.8 12.8-12.8h294.4c7.07 0 12.8 5.73 12.8 12.8v38.4c0 7.07-5.73 12.8-12.8 12.8H268.8c-7.07 0-12.8-5.73-12.8-12.8Z"/></svg>'
};
const Loading3Quarters = {
  name: "loading-3-quarters",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path fill-rule="evenodd" d="M499.226 128C293.062 134.745 128 303.995 128 511.798c0 212.078 171.923 384 384 384 207.803 0 377.056-165.062 383.796-371.226.23-7.065-5.524-12.774-12.596-12.774h-38.4c-7.072 0-12.774 5.799-13.056 12.862C824.998 695.428 684.423 831.798 512 831.798c-176.73 0-320-143.269-320-320 0-172.423 136.37-312.996 307.138-319.746 7.064-.28 12.862-5.984 12.862-13.054v-38.4c0-7.069-5.709-12.822-12.774-12.598"/></svg>'
};
const LoadingSplit = {
  name: "loading-split",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path d="M492.8 64h38.4Q544 64 544 76.8v166.4q0 12.8-12.8 12.8h-38.4q-12.8 0-12.8-12.8V76.8Q480 64 492.8 64Zm226.572 50.43 33.256 19.2q11.085 6.4 4.685 17.485l-83.2 144.106q-6.4 11.085-17.485 4.685l-33.256-19.2q-11.085-6.4-4.685-17.485l83.2-144.106q6.4-11.085 17.485-4.685ZM890.37 271.372l19.2 33.256q6.4 11.085-4.685 17.485l-144.106 83.2q-11.085 6.4-17.485-4.685l-19.2-33.256q-6.4-11.085 4.685-17.485l144.106-83.2q11.085-6.4 17.485 4.685ZM960 492.8v38.4q0 12.8-12.8 12.8H780.8q-12.8 0-12.8-12.8v-38.4q0-12.8 12.8-12.8h166.4q12.8 0 12.8 12.8Zm-50.43 226.572-19.2 33.256q-6.4 11.085-17.485 4.685l-144.106-83.2q-11.085-6.4-4.685-17.485l19.2-33.256q6.4-11.085 17.485-4.685l144.106 83.2q11.085 6.4 4.685 17.485ZM752.628 890.37l-33.256 19.2q-11.085 6.4-17.485-4.685l-83.2-144.106q-6.4-11.085 4.685-17.485l33.256-19.2q11.085-6.4 17.485 4.685l83.2 144.106q6.4 11.085-4.685 17.485ZM531.2 960h-38.4q-12.8 0-12.8-12.8V780.8q0-12.8 12.8-12.8h38.4q12.8 0 12.8 12.8v166.4q0 12.8-12.8 12.8Zm-226.572-50.43-33.256-19.2q-11.085-6.4-4.685-17.485l83.2-144.106q6.4-11.085 17.485-4.685l33.256 19.2q11.085 6.4 4.685 17.485l-83.2 144.106q-6.4 11.085-17.485 4.685ZM133.63 752.628l-19.2-33.256q-6.4-11.085 4.685-17.485l144.106-83.2q11.085-6.4 17.485 4.685l19.2 33.256q6.4 11.085-4.685 17.485l-144.106 83.2q-11.085 6.4-17.485-4.685ZM64 531.2v-38.4Q64 480 76.8 480h166.4q12.8 0 12.8 12.8v38.4q0 12.8-12.8 12.8H76.8Q64 544 64 531.2Zm50.43-226.572 19.2-33.256q6.4-11.085 17.485-4.685l144.106 83.2q11.085 6.4 4.685 17.485l-19.2 33.256q-6.4 11.085-17.485 4.685l-144.106-83.2q-11.085-6.4-4.685-17.485ZM271.372 133.63l33.256-19.2q11.085-6.4 17.485 4.685l83.2 144.106q6.4 11.085-4.685 17.485l-33.256 19.2q-11.085 6.4-17.485-4.685l-83.2-144.106q-6.4-11.085 4.685-17.485Z"/></svg>'
};
const Loading = {
  name: "loading",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path d="m527.424 64.32 3.712.064a448 448 0 0 1 430.912 432.32v2.368a12.8 12.8 0 0 1-12.8 13.056h-38.336a12.8 12.8 0 0 1-12.8-13.632A384 384 0 0 0 528.64 128.384h-1.792a12.8 12.8 0 0 1-12.608-12.8V77.12a12.8 12.8 0 0 1 13.12-12.8z"/></svg>'
};
const Location = {
  name: "location",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path d="M512 64a384 384 0 0 1 286.208 640l-277.76 248.512a12.8 12.8 0 0 1-16.896 0L225.792 704A384 384 0 0 1 512 64zm0 64a320 320 0 0 0-249.344 520.576l10.816 12.736 12.736 14.272L511.872 874.56l200.576-177.088c4.992-4.032 9.792-8.192 15.104-12.992l8.384-7.808 4.032-3.648 10.56-11.712a318.336 318.336 0 0 0 81.024-196.288L832 448a320 320 0 0 0-320-320z"/><path d="M512 256a128 128 0 1 1 0 256 128 128 0 0 1 0-256zm0 64a64 64 0 1 0 0 128 64 64 0 0 0 0-128z"/></svg>'
};
const Lock = {
  name: "lock",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><g fill-rule="evenodd"><path d="M512 64c141.385 0 256 98.241 256 219.428V384h115.2c7.072 0 12.8 5.73 12.8 12.8v550.4c0 7.072-5.728 12.8-12.8 12.8H140.8c-7.07 0-12.8-5.728-12.8-12.8V396.8c0-7.07 5.73-12.8 12.8-12.8H256V283.428C256 162.241 370.615 64 512 64Zm320 384H192v448h640V448ZM512 128c-101.229 0-184.16 62.672-191.476 142.16l-.198 2.415L320 281.6V384h384V281.6c0-80.983-78.34-147.329-177.701-153.18l-3.018-.16L512 128Z"/><path d="M640 582.4v51.2c0 3.535-5.73 6.4-12.8 6.4H396.8c-7.07 0-12.8-2.865-12.8-6.4v-51.2c0-3.535 5.73-6.4 12.8-6.4h230.4c7.07 0 12.8 2.865 12.8 6.4Z"/></g></svg>'
};
const Login = {
  name: "login",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path fill-rule="evenodd" d="M512 960c247.424 0 448-200.576 448-448 0-247.423-200.576-448-448-448-166.228 0-318.179 91.962-395.628 237.798C111.964 310.1 118.067 320 127.467 320h44.59c4.572 0 8.749-2.39 11.11-6.305C252.56 198.606 377.254 128 512 128c212.077 0 384 171.923 384 384S724.077 896 512 896c-134.746 0-259.44-70.605-328.833-185.696-2.361-3.917-6.538-6.304-11.11-6.304h-44.59c-9.4 0-15.503 9.9-11.095 18.202C193.822 868.038 345.772 960 512 960Zm-44.419-298.74 220.733-137.213c8.056-5.008 8.056-16.305 0-21.313L467.58 365.522c-8.526-5.3-19.514 1.09-19.514 11.13V480H76.8c-7.07 0-12.8 5.73-12.8 12.8v38.4c0 7.07 5.73 12.8 12.8 12.8h371.267v106.13c0 10.039 10.988 16.43 19.514 11.13Z"/></svg>'
};
const Logout = {
  name: "logout",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path fill-rule="evenodd" d="M511.922 960c247.425 0 448.001-200.576 448.001-448 0-247.423-200.576-448-448.001-448-166.228 0-318.179 91.962-395.628 237.798C111.886 310.1 117.99 320 127.39 320h44.59c4.572 0 8.749-2.39 11.11-6.305C252.483 198.606 377.176 128 511.922 128 724 128 895.923 299.923 895.923 512S724 896 511.922 896c-134.746 0-259.44-70.605-328.833-185.696-2.361-3.917-6.538-6.304-11.11-6.304h-44.59c-9.4 0-15.503 9.9-11.095 18.202C193.744 868.038 345.694 960 511.922 960ZM290.388 365.548 70.042 502.52c-8.056 5.008-8.056 16.733 0 21.742l220.346 136.971c8.526 5.3 19.558-.831 19.558-10.87V544h381.176c7.069 0 12.8-5.73 12.8-12.8v-38.4c0-7.07-5.731-12.8-12.8-12.8H309.946V376.42c0-10.04-11.032-16.172-19.558-10.872Z"/></svg>'
};
const Loose = {
  name: "loose",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path d="M896 140.8v294.4a12.8 12.8 0 0 1-12.8 12.8H140.8a12.8 12.8 0 0 1-12.8-12.8V140.8c0-7.04 5.76-12.8 12.8-12.8h742.4c7.04 0 12.8 5.76 12.8 12.8zM819.2 192H204.8a12.8 12.8 0 0 0-12.8 12.8v166.4c0 7.04 5.76 12.8 12.8 12.8h614.4a12.8 12.8 0 0 0 12.8-12.8V204.8a12.8 12.8 0 0 0-12.8-12.8zM896 524.8v294.4a12.8 12.8 0 0 1-12.8 12.8H140.8a12.8 12.8 0 0 1-12.8-12.8V524.8c0-7.04 5.76-12.8 12.8-12.8h742.4c7.04 0 12.8 5.76 12.8 12.8zM819.2 576H204.8a12.8 12.8 0 0 0-12.8 12.8v166.4c0 7.04 5.76 12.8 12.8 12.8h614.4a12.8 12.8 0 0 0 12.8-12.8V588.8a12.8 12.8 0 0 0-12.8-12.8z"/></svg>'
};
const Mail = {
  name: "mail",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path fill-rule="evenodd" d="M76.8 832h870.4c7.072 0 12.8-5.73 12.8-12.8V204.8c0-7.07-5.728-12.8-12.8-12.8H76.8c-7.07 0-12.8 5.73-12.8 12.8v614.4c0 7.07 5.73 12.8 12.8 12.8Zm144.743-576h581.855l-291.4 152.898L221.544 256ZM128 279.085l363.14 191.163 6.71 2.862a44.804 44.804 0 0 0 34.975-2.835L896 279.69V768H128V279.085Z"/></svg>'
};
const MenuFold = {
  name: "menu-fold",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path d="M888.32 768c4.224 0 7.68 3.2 7.68 7.104v49.792c.064 3.84-3.328 7.04-7.68 7.104H135.68c-4.352-.064-7.744-3.264-7.68-7.104v-49.792c0-3.904 3.456-7.104 7.68-7.104h752.64zM736 392.576c0-7.168 7.872-11.136 13.248-6.72l143.68 119.488a8.832 8.832 0 0 1 0 13.44l-143.68 119.424c-5.376 4.48-13.248.384-13.248-6.784zM631.68 576c4.608 0 8.32 3.2 8.32 7.104v49.792c.064 3.84-3.584 7.04-8.32 7.104H136.32c-4.736-.064-8.384-3.264-8.32-7.104V583.04c0-3.84 3.712-7.104 8.32-7.104h495.36zm0-192c4.608 0 8.32 3.2 8.32 7.104v49.792c.064 3.84-3.584 7.04-8.32 7.104H136.32c-4.736-.064-8.384-3.264-8.32-7.104V391.04c0-3.84 3.712-7.104 8.32-7.104h495.36zm256.64-192c4.224 0 7.68 3.2 7.68 7.104v49.792c.064 3.84-3.328 7.04-7.68 7.104H135.68c-4.352-.064-7.744-3.264-7.68-7.104V199.04c0-3.84 3.456-7.04 7.68-7.04h752.64z"/></svg>'
};
const MenuUnfold = {
  name: "menu-unfold",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path d="M888.32 768c4.224 0 7.68 3.2 7.68 7.104v49.792c.064 3.84-3.328 7.04-7.68 7.104H135.68c-4.352-.064-7.744-3.264-7.68-7.104v-49.792c0-3.904 3.456-7.104 7.68-7.104h752.64zM896 392.576v238.848c0 7.168-7.872 11.264-13.248 6.784l-143.68-119.36a8.832 8.832 0 0 1 0-13.44l143.68-119.552c5.376-4.48 13.248-.448 13.248 6.72zM631.68 576c4.608 0 8.32 3.2 8.32 7.104v49.792c.064 3.84-3.584 7.04-8.32 7.104H136.32c-4.736-.064-8.384-3.264-8.32-7.104V583.04c0-3.84 3.712-7.104 8.32-7.104h495.36zm0-192c4.608 0 8.32 3.2 8.32 7.104v49.792c.064 3.84-3.584 7.04-8.32 7.104H136.32c-4.736-.064-8.384-3.264-8.32-7.104V391.04c0-3.84 3.712-7.104 8.32-7.104h495.36zm256.64-192c4.224 0 7.68 3.2 7.68 7.104v49.792c.064 3.84-3.328 7.04-7.68 7.104H135.68c-4.352-.064-7.744-3.264-7.68-7.104V199.04c0-3.84 3.456-7.04 7.68-7.04h752.64z"/></svg>'
};
const Menu = {
  name: "menu",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path d="M896 140.8v38.4a12.8 12.8 0 0 1-12.8 12.8H140.8a12.8 12.8 0 0 1-12.8-12.8v-38.4c0-7.04 5.76-12.8 12.8-12.8h742.4c7.04 0 12.8 5.76 12.8 12.8zm0 128v38.4a12.8 12.8 0 0 1-12.8 12.8H140.8a12.8 12.8 0 0 1-12.8-12.8v-38.4c0-7.04 5.76-12.8 12.8-12.8h742.4c7.04 0 12.8 5.76 12.8 12.8zm0 128v38.4a12.8 12.8 0 0 1-12.8 12.8H140.8a12.8 12.8 0 0 1-12.8-12.8v-38.4c0-7.04 5.76-12.8 12.8-12.8h742.4c7.04 0 12.8 5.76 12.8 12.8zm0 128v38.4a12.8 12.8 0 0 1-12.8 12.8H140.8a12.8 12.8 0 0 1-12.8-12.8v-38.4c0-7.04 5.76-12.8 12.8-12.8h742.4c7.04 0 12.8 5.76 12.8 12.8zm0 128v38.4a12.8 12.8 0 0 1-12.8 12.8H140.8a12.8 12.8 0 0 1-12.8-12.8v-38.4c0-7.04 5.76-12.8 12.8-12.8h742.4c7.04 0 12.8 5.76 12.8 12.8zm0 128v38.4a12.8 12.8 0 0 1-12.8 12.8H140.8a12.8 12.8 0 0 1-12.8-12.8v-38.4c0-7.04 5.76-12.8 12.8-12.8h742.4c7.04 0 12.8 5.76 12.8 12.8z"/></svg>'
};
const MinusCircle = {
  name: "minus-circle",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><g fill-rule="evenodd"><path d="M512 64c247.424 0 448 200.577 448 448 0 247.424-200.576 448-448 448-247.423 0-448-200.576-448-448C64 264.577 264.577 64 512 64Zm0 64c-212.077 0-384 171.923-384 384s171.923 384 384 384 384-171.923 384-384-171.923-384-384-384Z"/><path d="M753.926 545.414H267.238c-7.069 0-12.656-5.563-12.656-12.632v-38.736c0-7.07 5.587-12.632 12.656-12.632h486.688c7.066 0 12.653 5.563 12.653 12.632v38.736c0 7.069-5.587 12.632-12.653 12.632Z"/></g></svg>'
};
const MinusSquare = {
  name: "minus-square",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path d="M896 140.8v742.4a12.8 12.8 0 0 1-12.8 12.8H140.8a12.8 12.8 0 0 1-12.8-12.8V140.8c0-7.04 5.76-12.8 12.8-12.8h742.4c7.04 0 12.8 5.76 12.8 12.8zM832 192H192v640h640V192z"/><path d="M768.128 492.8v38.4a12.8 12.8 0 0 1-12.8 12.8H271.744a12.8 12.8 0 0 1-12.8-12.8v-38.4c0-7.04 5.76-12.8 12.8-12.8h483.584c7.04 0 12.8 5.76 12.8 12.8z"/></svg>'
};
const Minus = {
  name: "minus",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path fill-rule="evenodd" d="M140.915 512h742.167c7.069 0 12.918-5.563 12.918-12.632v-38.736c0-7.07-5.849-12.632-12.918-12.632H140.915C133.85 448 128 453.563 128 460.632v38.736c0 7.07 5.85 12.632 12.915 12.632Z"/></svg>'
};
const Mirror = {
  name: "mirror",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path fill-rule="evenodd" d="M499.2 960h-38.4c-7.07 0-12.8-5.728-12.8-12.8V76.8c0-7.07 5.73-12.8 12.8-12.8h38.4c7.07 0 12.8 5.73 12.8 12.8v870.4c0 7.072-5.73 12.8-12.8 12.8ZM374.305 827.411l-294.4 73.6C71.824 903.027 64 896.921 64 888.59V135.41c0-8.327 7.826-14.437 15.904-12.417l294.4 73.6A12.8 12.8 0 0 1 384 209.009v605.98a12.806 12.806 0 0 1-9.695 12.422Zm211.39 0 294.401 73.6c8.077 2.016 15.904-4.09 15.904-12.422v-753.18c0-8.327-7.827-14.437-15.904-12.417l-294.4 73.6A12.8 12.8 0 0 0 576 209.009v605.98a12.806 12.806 0 0 0 9.695 12.422Zm-265.758-52.435V248.961l-192-48v622.015l192-48Zm320.126 0V248.961l192.001-48v622.015l-192.001-48Z"/></svg>'
};
const Mobile = {
  name: "mobile",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path fill-rule="evenodd" d="M192 947.2c0 7.072 5.73 12.8 12.8 12.8h614.4c7.07 0 12.8-5.728 12.8-12.8V76.8c0-7.07-5.73-12.8-12.8-12.8H204.8c-7.07 0-12.8 5.73-12.8 12.8v870.4Zm64-51.2V128h512v768H256Zm256-64c35.346 0 64-28.653 64-64s-28.654-64-64-64c-35.346 0-64 28.653-64 64s28.654 64 64 64Z"/></svg>'
};
const Monitor = {
  name: "monitor",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><g fill-rule="evenodd"><path d="M448 96c212.077 0 384 171.923 384 384 0 82.976-26.88 163.72-76.608 230.142l154.182 154.184c4.999 4.999 4.999 13.101 0 18.1L882.42 909.58c-4.998 4.998-13.1 4.998-18.099 0L712.82 758.08A384.012 384.012 0 0 1 448 864C235.923 864 64 692.077 64 480S235.923 96 448 96Zm0 64c-176.73 0-320 143.27-320 320s143.27 320 320 320 320-143.27 320-320-143.27-320-320-320Z"/><path d="M470.628 547.885 343.914 421.172c-4.998-4.999-4.998-13.104 0-18.102l162.918-162.918c4.998-4.998 13.103-4.998 18.102 0l27.152 27.154c5 4.998 5 13.103 0 18.101L425.373 412.121l126.713 126.714c5 4.998 5 13.103 0 18.101L389.17 719.853c-4.999 5-13.103 5-18.101 0l-27.154-27.152c-4.998-4.999-4.998-13.104 0-18.102l126.714-126.714Z"/></g></svg>'
};
const More = {
  name: "more",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path fill-rule="evenodd" d="M890.24 768c3.168 0 5.761 3.186 5.761 7.111v49.778c.05 3.843-2.523 7.018-5.761 7.111H325.76c-3.241-.093-5.813-3.268-5.761-7.111v-49.778c0-3.925 2.594-7.111 5.762-7.111H890.24Zm-640 0c3.168 0 5.76 3.186 5.76 7.111v49.778c.051 3.843-2.522 7.018-5.76 7.111H133.76c-3.241-.093-5.813-3.268-5.761-7.111v-49.778c0-3.925 2.594-7.111 5.762-7.111H250.24Zm640-256c3.168 0 5.761 3.186 5.761 7.111v49.778c.05 3.843-2.523 7.018-5.761 7.111H325.76c-3.241-.093-5.813-3.268-5.761-7.111v-49.778c0-3.925 2.594-7.111 5.762-7.111H890.24Zm-640 0c3.168 0 5.76 3.186 5.76 7.111v49.778c.051 3.843-2.522 7.018-5.76 7.111H133.76c-3.241-.093-5.813-3.268-5.761-7.111v-49.778c0-3.925 2.594-7.111 5.762-7.111H250.24Zm640-256c3.168 0 5.761 3.186 5.761 7.111v49.778c.05 3.843-2.523 7.018-5.761 7.111H325.76c-3.241-.093-5.813-3.268-5.761-7.111V263.11c0-3.925 2.594-7.111 5.762-7.111H890.24Zm-640 0c3.168 0 5.76 3.186 5.76 7.111v49.778c.051 3.843-2.522 7.018-5.76 7.111H133.76c-3.241-.093-5.813-3.268-5.761-7.111V263.11c0-3.925 2.594-7.111 5.762-7.111H250.24Z"/></svg>'
};
const MoveFile = {
  name: "move-file",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path d="M384 672a288 288 0 1 1 576 0 288 288 0 0 1-576 0zm64 0a224 224 0 1 0 448 0 224 224 0 0 0-448 0z"/><path d="M576 704v-64h243.2c7.04 0 12.8 5.76 12.8 12.8v38.4a12.8 12.8 0 0 1-12.8 12.8H576z"/><path d="M650.176 785.92 545.28 680.96a12.8 12.8 0 0 1 0-18.048L650.24 558.08a12.8 12.8 0 0 1 21.824 9.088v209.664a12.8 12.8 0 0 1-21.824 9.088zM268.8 448h102.4q12.8 0 12.8 12.8v38.4q0 12.8-12.8 12.8H268.8q-12.8 0-12.8-12.8v-38.4q0-12.8 12.8-12.8Zm0 128h38.4q12.8 0 12.8 12.8v38.4q0 12.8-12.8 12.8h-38.4q-12.8 0-12.8-12.8v-38.4q0-12.8 12.8-12.8Zm0 128h38.4q12.8 0 12.8 12.8v38.4q0 12.8-12.8 12.8h-38.4q-12.8 0-12.8-12.8v-38.4q0-12.8 12.8-12.8Z"/><path d="M703.488 64 896 256.384v127.552h-64V320H652.8a12.8 12.8 0 0 1-12.352-9.408L640 307.2V123.52l-448 4.16V896h192v64H140.8a12.8 12.8 0 0 1-12.352-9.408L128 947.2V76.8l.448-3.456a12.8 12.8 0 0 1 8.96-8.96L140.8 64h562.688zm.256 90.688L704 256l101.184.192-101.44-101.44z"/></svg>'
};
const MoveFilez = {
  name: "move-filez",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><g fill-rule="evenodd"><path d="m140.8 64.001 557.415.056A12.8 12.8 0 0 1 707 67.551l.26.253 184.987 184.875a12.829 12.829 0 0 1 3.752 8.69v109.885c0 6.967-5.566 12.634-12.493 12.797l-.307.003h-38.4c-6.967 0-12.634-5.565-12.796-12.493l-.004-.307v-51.26H652.801c-6.967 0-12.633-5.566-12.796-12.493l-.004-.307V123.578l-448 4.16v768.32H371.2c6.967 0 12.634 5.565 12.796 12.493l.004.307v38.4c0 6.966-5.566 12.633-12.493 12.8H140.8c-6.967 0-12.634-5.566-12.8-12.494V76.8l.456-3.404c1.464-5.319 6.274-9.25 12.024-9.395h.32Zm562.943 90.77L704 255.994l101.254.23-101.51-101.453Z"/><path d="M672 384.054c159.06 0 288 128.942 288 288 0 159.056-128.94 288.004-288 288.004-159.058 0-288-128.948-288-288.004 0-159.058 128.942-288 288-288Zm0 64c-123.712 0-224 100.289-224 224 0 123.716 100.288 224.004 224 224.004S896 795.77 896 672.054c0-123.711-100.288-224-224-224ZM256 499.254v-38.4c0-7.068 5.73-12.8 12.8-12.8h102.4c7.07 0 12.8 5.732 12.8 12.8v38.4c0 7.07-5.73 12.8-12.8 12.8H268.8c-7.07 0-12.8-5.73-12.8-12.8Z"/><path d="m521.051 658.175 140.29-140.29c4-4 9.986-4.8 14.78-2.4l3.322 2.4 27.153 27.152c4.998 5 4.998 13.104 0 18.102l-76.915 76.915H819.2c7.07 0 12.8 5.731 12.8 12.8v38.404c0 7.069-5.73 12.8-12.8 12.8H639.34l67.256 67.256c4.998 4.999 4.998 13.103 0 18.102l-27.153 27.152c-3.999 4-9.985 4.799-14.78 2.4l-3.322-2.4-140.29-140.291c-4.999-5-4.999-13.104 0-18.102ZM256 627.254v-38.4c0-7.068 5.73-12.8 12.8-12.8h38.4c7.07 0 12.8 5.732 12.8 12.8v38.4c0 7.07-5.73 12.8-12.8 12.8h-38.4c-7.07 0-12.8-5.73-12.8-12.8ZM256 755.258v-38.4c0-7.072 5.73-12.8 12.8-12.8h38.4c7.07 0 12.8 5.728 12.8 12.8v38.4c0 7.065-5.73 12.8-12.8 12.8h-38.4c-7.07 0-12.8-5.735-12.8-12.8Z"/></g></svg>'
};
const MoveGroup = {
  name: "move-group",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><g fill-rule="evenodd"><path d="M672 384c159.06 0 288 128.942 288 288 0 159.06-128.94 288-288 288-159.058 0-288-128.94-288-288 0-159.058 128.942-288 288-288Zm0 64c-123.712 0-224 100.288-224 224s100.288 224 224 224 224-100.288 224-224-100.288-224-224-224Z"/><path d="M423.953 64a12.8 12.8 0 0 1 8.79 3.496l.261.253L557.255 192h326.047c6.97 0 12.535 5.663 12.698 12.593V307.2c0 6.967-5.563 12.634-12.493 12.796l-.307.004h-38.4c-6.97 0-12.634-5.566-12.796-12.493L832 307.2V256H128v512h179.2c6.967 0 12.634 5.563 12.796 12.493l.004.307v38.4c0 6.97-5.566 12.634-12.493 12.8H76.697c-6.968 0-12.534-5.663-12.697-12.596V76.8c0-6.967 5.566-12.634 12.493-12.8h347.46Zm-21.201 64H128v64h338.752l-64-64Z"/><path d="M659.183 522.538c4.92-4.918 12.85-4.763 17.866.008l.24.234 140.049 140.046c4.917 4.919 4.997 13.078.238 18.101l-.238.245-140.05 140.044c-4.919 4.92-12.85 5.233-17.865.477l-.24-.234-27.394-27.392c-4.918-4.924-4.993-12.623-.232-17.63l.238-.245 72.2-72.192H524.736c-6.965 0-12.57-5.4-12.733-12.321l-.004-.311v-38.736c0-6.965 5.502-12.468 12.426-12.628l.311-.004H704l-72.204-72.205c-4.92-4.918-5.004-12.618-.244-17.626l.238-.244 27.394-27.387Z"/></g></svg>'
};
const MoveList = {
  name: "move-list",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><g fill-rule="evenodd"><path d="M629.668 64c7.07 0 12.8 5.73 12.8 12.8v166.47c0 7.07-5.73 12.8-12.8 12.8H396.8c-7.07 0-12.8-5.73-12.8-12.8V76.8c0-7.07 5.73-12.8 12.8-12.8h232.868Zm-51.236 63.945H448v64.065h130.432v-64.065ZM140.8 64.07h166.4c7.07 0 12.8 5.731 12.8 12.8v38.4c0 7.07-5.73 12.8-12.8 12.8H192v768h179.2c7.07 0 12.8 5.728 12.8 12.8v38.4c0 7.072-5.73 12.8-12.8 12.8H140.8c-7.07 0-12.8-5.728-12.8-12.8V76.87c0-7.069 5.73-12.8 12.8-12.8ZM883.2 320.07h-38.4c-7.072 0-12.8-5.73-12.8-12.8v-179.2H716.8c-7.07 0-12.8-5.73-12.8-12.8v-38.4c0-7.069 5.73-12.8 12.8-12.8h166.4c7.072 0 12.8 5.731 12.8 12.8v230.4c0 7.07-5.728 12.8-12.8 12.8ZM672 384.07c159.06 0 288 128.942 288 288 0 159.06-128.94 288-288 288-159.058 0-288-128.94-288-288 0-159.058 128.942-288 288-288Zm0 64c-123.712 0-224 100.288-224 224s100.288 224 224 224 224-100.288 224-224-100.288-224-224-224Z"/><path d="m520.362 681.123 140.29 140.291c4.999 4.999 13.104 4.999 18.102 0l27.153-27.155c4.999-4.998 4.999-13.1 0-18.099l-72.087-72.09H819.2c7.072 0 12.8-5.73 12.8-12.8v-38.4c0-7.069-5.728-12.8-12.8-12.8H633.822l72.085-72.085c4.999-4.998 4.999-13.102 0-18.101l-27.153-27.153c-4.998-5-13.103-5-18.102 0l-140.29 140.29c-4.999 4.998-4.999 13.103 0 18.102Z"/></g></svg>'
};
const Move = {
  name: "move",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path fill-rule="evenodd" d="m309.325 255.809 27.15 27.15c5 5 13.107 4.998 18.105-.003l94.112-94.17-.237 261.9-261.782 1.859 96.923-96.868c5.001-4.999 5.002-13.105.003-18.105l-27.15-27.15c-5-4.999-13.104-4.999-18.103 0L75.43 473.34c-4.999 4.998-4.999 13.102 0 18.101l162.917 162.917c5 5 13.104 5 18.103 0l27.155-27.155c4.998-4.998 4.999-13.1.002-18.099l-92.534-92.59 257.325-1.829-.233 257.718-93.584-93.52c-4.999-4.996-13.101-4.994-18.099.003l-27.156 27.156c-4.998 4.998-4.998 13.1 0 18.105L472.243 887.06c4.999 4.999 13.103 4.999 18.102 0l162.917-162.912c5-5.005 5-13.107 0-18.105l-27.156-27.156c-4.997-4.997-13.1-4.998-18.098-.002l-96.482 96.418.235-261.066 262.972-1.868-96.705 95.623c-5.026 4.97-5.072 13.074-.103 18.101l26.998 27.309c4.97 5.027 13.074 5.074 18.104.104l163.93-162.06c5.043-4.99 5.069-13.135.051-18.154L723.104 309.387c-4.998-5-13.105-5-18.103 0l-27.15 27.15c-5 5-4.999 13.107.003 18.104l93.813 93.749-259.847 1.846.237-263.282 95.95 96.002c4.998 5.001 13.105 5.002 18.105.003l27.15-27.15c5-5 5-13.104 0-18.103L490.345 74.79c-5-4.999-13.103-4.999-18.102 0L309.325 237.706c-4.998 5-4.998 13.104 0 18.103Z"/></svg>'
};
const Network = {
  name: "network",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path fill-rule="evenodd" d="M512.192 64.098a447.92 447.92 0 0 1 108.855 13.426 447.914 447.914 0 0 1 82.69 29.587l9.88 4.822 9.76 5.064A447.84 447.84 0 0 1 796.4 165.79a447.409 447.409 0 0 1 32.576 29.525 447.733 447.733 0 0 1 43.052 49.91 447.973 447.973 0 0 1 12.662 17.978l11.764 18.578a447.884 447.884 0 0 1 44.447 100.27 447.914 447.914 0 0 1 18.751 108.066 447.919 447.919 0 0 1-8.069 109.382 447.88 447.88 0 0 1-25.493 84.042 447.922 447.922 0 0 1-29.636 58.875l-11.764 18.579a446.428 446.428 0 0 1-12.662 17.977A450.49 450.49 0 0 1 858.5 796.31a447.982 447.982 0 0 1-29.524 32.57 446.99 446.99 0 0 1-32.576 29.53 449.986 449.986 0 0 1-17.335 13.529 452.296 452.296 0 0 1-17.978 12.659 444.67 444.67 0 0 1-18.577 11.763l-9.5 5.535-9.632 5.3a440.259 440.259 0 0 1-19.641 9.888A448.719 448.719 0 0 1 683.634 926a451.431 451.431 0 0 1-20.516 7.91 448.494 448.494 0 0 1-20.879 6.9 444.284 444.284 0 0 1-21.192 5.862 444.297 444.297 0 0 1-21.455 4.819 448.468 448.468 0 0 1-21.665 3.757 451.721 451.721 0 0 1-21.824 2.694 453.89 453.89 0 0 1-21.93 1.62 449.611 449.611 0 0 1-43.963 0 453.89 453.89 0 0 1-21.93-1.62 451.721 451.721 0 0 1-21.823-2.694 448.468 448.468 0 0 1-21.666-3.757 444.297 444.297 0 0 1-21.454-4.82 444.284 444.284 0 0 1-21.193-5.862 448.494 448.494 0 0 1-20.879-6.899A451.431 451.431 0 0 1 340.75 926a448.479 448.479 0 0 1-20.103-8.916l-9.88-4.82-9.763-5.068a451.08 451.08 0 0 1-19.13-10.835 445.016 445.016 0 0 1-18.578-11.763 450.72 450.72 0 0 1-17.978-12.66 447.563 447.563 0 0 1-33.984-27.891 445.11 445.11 0 0 1-15.923-15.167 445.11 445.11 0 0 1-15.168-15.924 447.563 447.563 0 0 1-27.891-33.984 450.722 450.722 0 0 1-12.66-17.977 445.016 445.016 0 0 1-11.763-18.58 451.08 451.08 0 0 1-10.835-19.129 441.948 441.948 0 0 1-9.888-19.643 448.48 448.48 0 0 1-8.915-20.102 451.431 451.431 0 0 1-7.91-20.515 448.495 448.495 0 0 1-6.9-20.88 444.284 444.284 0 0 1-5.862-21.192 444.297 444.297 0 0 1-4.82-21.455 448.469 448.469 0 0 1-3.756-21.665 451.722 451.722 0 0 1-2.695-21.824 453.891 453.891 0 0 1-1.619-21.93 449.611 449.611 0 0 1 0-43.963c.363-7.324.903-14.633 1.62-21.93.72-7.296 1.619-14.57 2.694-21.823a448.469 448.469 0 0 1 3.757-21.666 444.297 444.297 0 0 1 4.819-21.454 444.285 444.285 0 0 1 5.862-21.193 448.495 448.495 0 0 1 6.9-20.879 451.431 451.431 0 0 1 7.91-20.515 448.72 448.72 0 0 1 8.915-20.103 440.26 440.26 0 0 1 9.888-19.641 449.584 449.584 0 0 1 10.835-19.132 444.671 444.671 0 0 1 11.763-18.578 452.297 452.297 0 0 1 12.66-17.978 449.988 449.988 0 0 1 13.53-17.334 446.991 446.991 0 0 1 29.529-32.576 448.391 448.391 0 0 1 32.57-29.525 450.49 450.49 0 0 1 17.337-13.528 446.429 446.429 0 0 1 36.557-24.426l9.5-5.536 9.63-5.302a447.922 447.922 0 0 1 145.452-48.05 447.94 447.94 0 0 1 65.735-4.85Zm-71.518 762.058a587.133 587.133 0 0 1-72.69 23.585l-26.91 6.122 6.937 3.364a384.445 384.445 0 0 0 17.23 7.642 391.306 391.306 0 0 0 17.585 6.784 382.06 382.06 0 0 0 17.896 5.907 382.308 382.308 0 0 0 36.555 9.159 390.473 390.473 0 0 0 18.57 3.225 382.65 382.65 0 0 0 18.706 2.304c6.254.615 12.52 1.078 18.796 1.389a385.413 385.413 0 0 0 37.685 0 389.844 389.844 0 0 0 18.796-1.389 382.52 382.52 0 0 0 18.706-2.304 390.473 390.473 0 0 0 18.57-3.225 381.596 381.596 0 0 0 36.555-9.159l8.985-2.843 11.77-4.171-2.063-.803a589.254 589.254 0 0 1-74.409-35.904c-27.458 20.912-65.294 26.242-98.738 10.647-11.004-5.132-20.579-12.084-28.532-20.33Zm342.308-307.599a95.314 95.314 0 0 1-30.295 13.25c-35.62 75.77-87.331 143.51-151.659 197.987a95.26 95.26 0 0 1 2.055 52.35 506.242 506.242 0 0 0 30.715 16.016l24.097 10.72 63.255 25.384 4.38-2.883a380.238 380.238 0 0 0 15.41-10.854c5.048-3.742 10-7.606 14.859-11.59a386.735 386.735 0 0 0 14.271-12.314 381.378 381.378 0 0 0 13.65-12.999l6.583-6.742 6.414-6.903a387.188 387.188 0 0 0 12.31-14.272 385.989 385.989 0 0 0 11.597-14.86l5.52-7.639 5.332-7.773a383.949 383.949 0 0 0 19.373-32.321l4.34-8.365 4.135-8.47a384.382 384.382 0 0 0 7.638-17.23l6.782-17.585 2.352-6.934-44.403-61.265a522.277 522.277 0 0 0-48.711-54.708ZM237.695 403.983c-36.473 13.64-71.969 31.642-105.739 54.098l-1.915 16.378a389.845 389.845 0 0 0-1.389 18.797 385.413 385.413 0 0 0 0 37.684 389.72 389.72 0 0 0 1.39 18.796l2.303 18.706a390.475 390.475 0 0 0 3.226 18.57 381.596 381.596 0 0 0 9.158 36.555 382.06 382.06 0 0 0 5.907 17.897l6.784 17.584 3.716 8.662 3.926 8.569a377.303 377.303 0 0 0 8.474 16.835 382.95 382.95 0 0 0 9.286 16.398l4.947 8.026 5.14 7.897a379.086 379.086 0 0 0 10.854 15.412 385.42 385.42 0 0 0 11.59 14.86l6.07 7.21 6.244 7.062a380.849 380.849 0 0 0 12.998 13.645 380.85 380.85 0 0 0 13.645 12.999l11.064 9.648 68.778-14.361 12.813-2.888a522.437 522.437 0 0 0 67.276-21.255c-1.233-12.99.246-26.34 4.663-39.261l4.144-10.185c-69.72-78.103-117.08-173.594-137.423-275.676l-7.348-2.964c-17.316-8.075-31.092-20.657-40.582-35.698Zm175.256-32.839c-1.358 7.517-3.744 14.912-7.097 22.103-11.273 24.173-31.33 41.447-54.597 49.792 19.016 84.858 59.252 164.191 117.085 229.954 25.05-12.058 54.964-13.129 82.007-.518a96.855 96.855 0 0 1 11.558 6.378c50.9-43.419 92.798-96.373 123.536-155.425-36.175-18.914-55.54-58.084-50.653-96.897l1.421-8.304-8.258-3.79c-67.826-29.494-141.238-44.221-215.002-43.293Zm394.99-104.053a580.406 580.406 0 0 1-6.782 106.97c22.435 24.656 30.873 60.02 20.59 92.947a587.59 587.59 0 0 1 59.037 65.13l13.5 18.176a389.196 389.196 0 0 0 1.443-19.374l.347-9.418.116-9.424a383.817 383.817 0 0 0-1.85-37.639l-2.306-18.706a383.544 383.544 0 0 0-7.352-36.96 383.802 383.802 0 0 0-10.94-36.061l-6.782-17.585-3.714-8.662-3.924-8.568a384.312 384.312 0 0 0-17.765-33.235l-4.943-8.023-5.14-7.9a384.066 384.066 0 0 0-10.852-15.41l-5.707-7.5-6.977-8.758Zm-518.993-67.533-5.505 4.108a385.989 385.989 0 0 0-14.86 11.596 388.397 388.397 0 0 0-14.273 12.31l-13.645 12.997-6.584 6.745-6.414 6.905a386.738 386.738 0 0 0-12.314 14.272 384.028 384.028 0 0 0-11.59 14.858l-5.523 7.638-5.332 7.772a389.12 389.12 0 0 0-10.086 15.923 383.517 383.517 0 0 0-9.286 16.4l-4.343 8.365-8.057 17.038-5.953 13.989a581.37 581.37 0 0 1 68.416-29.513c1.138-9.74 3.867-19.471 8.243-28.857 9.848-21.118 26.4-36.97 45.94-46.204a581.502 581.502 0 0 1 11.166-66.342Zm223.244-71.46a383.817 383.817 0 0 0-56.345 4.156 383.544 383.544 0 0 0-36.96 7.352 383.803 383.803 0 0 0-18.165 5.027l-8.984 2.846-8.912 3.066a384.21 384.21 0 0 0-14.656 5.584.572.572 0 0 0-.147.076l-.616 1.757c-11.727 33.652-19.789 67.754-24.385 101.853l7.832 2.346 8.565 3.509c19.56 9.12 34.601 23.992 44.052 41.662 91.85-2.524 183.46 16.613 266.827 56.037 19.655-15.65 45.05-23.125 70.394-20.3 5.409-46.17 4.574-93.85-3.144-141.907l-12.017-8.349a384.048 384.048 0 0 0-15.924-10.083l-16.399-9.29-16.835-8.474a384.382 384.382 0 0 0-34.816-14.42 383.128 383.128 0 0 0-36.061-10.94 383.667 383.667 0 0 0-36.96-7.352 384.113 384.113 0 0 0-56.344-4.156Z"/></svg>'
};
const Normal = {
  name: "normal",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path d="M896 140.8v166.4a12.8 12.8 0 0 1-12.8 12.8H140.8a12.8 12.8 0 0 1-12.8-12.8V140.8c0-7.04 5.76-12.8 12.8-12.8h742.4c7.04 0 12.8 5.76 12.8 12.8zM819.2 192H204.8a12.8 12.8 0 0 0-12.8 12.8v38.4c0 7.04 5.76 12.8 12.8 12.8h614.4a12.8 12.8 0 0 0 12.8-12.8v-38.4a12.8 12.8 0 0 0-12.8-12.8zM896 396.8v166.4a12.8 12.8 0 0 1-12.8 12.8H140.8a12.8 12.8 0 0 1-12.8-12.8V396.8c0-7.04 5.76-12.8 12.8-12.8h742.4c7.04 0 12.8 5.76 12.8 12.8zM819.2 448H204.8a12.8 12.8 0 0 0-12.8 12.8v38.4c0 7.04 5.76 12.8 12.8 12.8h614.4a12.8 12.8 0 0 0 12.8-12.8v-38.4a12.8 12.8 0 0 0-12.8-12.8zM896 652.8v166.4a12.8 12.8 0 0 1-12.8 12.8H140.8a12.8 12.8 0 0 1-12.8-12.8V652.8c0-7.04 5.76-12.8 12.8-12.8h742.4c7.04 0 12.8 5.76 12.8 12.8zM819.2 704H204.8a12.8 12.8 0 0 0-12.8 12.8v38.4c0 7.04 5.76 12.8 12.8 12.8h614.4a12.8 12.8 0 0 0 12.8-12.8v-38.4a12.8 12.8 0 0 0-12.8-12.8z"/></svg>'
};
const Notification = {
  name: "notification",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path d="M593.536 64c7.04 0 12.8 5.76 12.8 12.8v131.072A288.128 288.128 0 0 1 799.872 480v178.752l151.424 151.424A12.8 12.8 0 0 1 942.272 832H668.8a159.808 159.808 0 0 1-313.216 0H80.32a12.8 12.8 0 0 1-8.896-22.016l152.448-147.136V480c0-124.8 79.36-231.04 190.4-271.04V76.8c0-7.04 5.76-12.8 12.8-12.8h166.4zm9.472 768H421.952a96 96 0 0 0 181.056 0zm-91.136-576a224 224 0 0 0-224 224v210.112L207.104 768h611.52l-82.752-82.752V480a224 224 0 0 0-224-224zm32.384-128h-64.128v65.728c6.016-.64 12.096-1.152 18.176-1.408l13.568-.32c10.624 0 21.12.576 31.424 1.664l.96-65.664z"/></svg>'
};
const PaperClip = {
  name: "paper-clip",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path d="M812.373 178.802a228.693 228.693 0 0 1 7.851 315.164l-7.85 8.25-361.018 361.016a170.78 170.78 0 0 1-248.433-234.268l6.826-7.282 361.586-361.53a113.323 113.323 0 0 1 165.945 154l-5.746 6.143-313.23 313.23a11.378 11.378 0 0 1-16.043 0l-24.177-24.177a11.378 11.378 0 0 1 0-16.043l313.23-313.23a56.434 56.434 0 0 0-74.923-84.196l-4.892 4.437-361.53 361.53a113.892 113.892 0 0 0 154.852 166.911l6.258-5.802 361.017-361.017a171.804 171.804 0 0 0-235.634-249.856l-7.338 6.94-312.662 312.662a11.378 11.378 0 0 1-16.1 0l-24.12-24.121a11.378 11.378 0 0 1 0-16.1l312.661-312.661a228.693 228.693 0 0 1 323.413 0z"/></svg>'
};
const Paste = {
  name: "paste",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path fill-rule="evenodd" d="M319.95 755.2V76.8c0-7.07 5.73-12.8 12.8-12.8h550.399c7.072 0 12.8 5.73 12.8 12.8v678.4c0 7.072-5.728 12.8-12.8 12.8h-550.4c-7.069 0-12.8-5.728-12.8-12.8Zm64-627.2v576h447.999V128h-448ZM192 256v115.25c0 7.068-5.73 12.8-12.8 12.8h-38.4c-7.07 0-12.8-5.732-12.8-12.8V204.8c0-7.07 5.73-12.8 12.8-12.8h102.35c7.07 0 12.8 5.73 12.8 12.8v38.4c0 7.07-5.73 12.8-12.8 12.8H192Zm255.95 12.8v38.4c0 7.07 5.73 12.8 12.8 12.8h294.4c7.068 0 12.8-5.73 12.8-12.8v-38.4c0-7.07-5.732-12.8-12.8-12.8h-294.4c-7.07 0-12.8 5.73-12.8 12.8Zm0 192v38.4c0 7.07 5.73 12.8 12.8 12.8h294.4c7.068 0 12.8-5.73 12.8-12.8v-38.4c0-7.07-5.732-12.8-12.8-12.8h-294.4c-7.07 0-12.8 5.73-12.8 12.8Zm-255.95.047v166.4c0 7.07-5.73 12.8-12.8 12.8h-38.4c-7.07 0-12.8-5.73-12.8-12.8v-166.4c0-7.07 5.73-12.8 12.8-12.8h38.4c7.07 0 12.8 5.73 12.8 12.8ZM243.25 896H192V780.845c0-7.066-5.73-12.8-12.8-12.8h-38.4c-7.07 0-12.8 5.734-12.8 12.8V947.2c0 7.066 5.73 12.8 12.8 12.8h102.45c7.068 0 12.8-5.734 12.8-12.8v-38.4c0-7.072-5.732-12.8-12.8-12.8Zm396.748 32c-.027 0-.049-.02 0 0l-.049-51.2c0-7.072 5.731-12.8 12.8-12.8h38.4c7.07 0 12.8 5.728 12.8 12.8v70.4c0 7.072-5.73 12.8-12.8 12.8h-38.302c-7.07 0-12.8-5.728-12.8-12.8l-.049-19.2c.05.02.028 0 0 0Zm-63.947-19.2v38.4c0 7.072-5.731 12.8-12.8 12.8h-230.4c-7.07 0-12.8-5.728-12.8-12.8v-38.4c0-7.072 5.73-12.8 12.8-12.8h230.4c7.069 0 12.8 5.728 12.8 12.8Z"/></svg>'
};
const PauseCircle = {
  name: "pause-circle",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path d="M512 64a448 448 0 1 1 0 896 448 448 0 0 1 0-896zm0 64a384 384 0 1 0 0 768 384 384 0 0 0 0-768z"/><path d="M396.8 256h38.4q12.8 0 12.8 12.8v486.4q0 12.8-12.8 12.8h-38.4q-12.8 0-12.8-12.8V268.8q0-12.8 12.8-12.8Zm192 0h38.4q12.8 0 12.8 12.8v486.4q0 12.8-12.8 12.8h-38.4q-12.8 0-12.8-12.8V268.8q0-12.8 12.8-12.8Z"/></svg>'
};
const Pause = {
  name: "pause",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path d="M332.8 128h38.4q12.8 0 12.8 12.8v742.4q0 12.8-12.8 12.8h-38.4q-12.8 0-12.8-12.8V140.8q0-12.8 12.8-12.8Zm320 0h38.4q12.8 0 12.8 12.8v742.4q0 12.8-12.8 12.8h-38.4q-12.8 0-12.8-12.8V140.8q0-12.8 12.8-12.8Z"/></svg>'
};
const Phone = {
  name: "phone",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path fill-rule="evenodd" d="m660.174 387.125-139.822-139.82c-4.999-5-4.999-13.104 0-18.102L681.805 67.749c4.996-4.996 13.095-5 18.095-.007l274.32 273.894a12.804 12.804 0 0 1 2.074 15.408l-6.982 12.22A1612.601 1612.601 0 0 1 377.42 964.698l-20.125 11.385a12.81 12.81 0 0 1-15.354-2.093L67.75 699.798c-4.999-4.998-4.999-13.103 0-18.101L230.665 518.78c5-4.999 13.105-4.998 18.103.001L388.852 658.9a1447.895 1447.895 0 0 0 271.322-271.774ZM359.43 900.91l14.106-8.333a1548.56 1548.56 0 0 0 523.635-526.759l4.02-6.795-85.185-85.184-125.12-124.672-89.024 89.024 143.136 143.247-33.932 44.497a1512.034 1512.034 0 0 1-251.678 258.898l-31.64 24.89-44.52 34.073-143.478-143.62-90.496 90.56L359.43 900.908Z"/></svg>'
};
const PieChart = {
  name: "pie-chart",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path fill-rule="evenodd" d="m892.32 344.051-287.044 95.683c-10.007 3.336-19.864-6.006-16.527-16.012l97.86-293.498 10.846-32.527c2.312-6.933 9.495-10.596 16.241-7.787 11.405 4.602 28.211 11.6 38.157 16.731 77.005 39.738 138.835 104.76 174.521 184.087 2.906 6.457 6.836 16.64 9.856 24.716 2.586 6.702-.883 14.265-7.699 16.537l-36.211 12.07ZM480 960.098c229.754 0 416-186.252 416-416.002 0-48.558-8.32-95.173-23.648-138.518-2.278-6.431-9.44-9.592-15.917-7.449L512 512.096l115.482-343.885c2.173-6.472-.806-13.579-7.233-15.88-43.818-15.689-91.036-24.234-140.249-24.234-229.75 0-416 186.249-416 416 0 229.749 186.25 416.001 416 416.001Zm56.356-763.524 13.212 2.434-130.843 389.794c-3.356 10 6.142 19.538 16.156 16.226L825.28 475.872l2.336 12.477A354.887 354.887 0 0 1 832 544.096c0 194.402-157.596 352.002-352 352.002s-352-157.6-352-352.002c0-194.403 157.596-352 352-352l18.973.502a353.5 353.5 0 0 1 37.383 3.976Z"/></svg>'
};
const Pin = {
  name: "pin",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path fill-rule="evenodd" d="m688.994 195.511.244.238 162.915 162.918c4.918 4.917 4.997 12.842.238 17.857l-.238.244-126.711 126.714 39.126 39.127a12.799 12.799 0 0 1 2.842 13.785l-.127.308-124.903 291.44c-3.528 8.233-14.17 10.366-20.599 4.22l-.217-.213-190.273-190.27-171.972 171.972c-4.919 4.918-12.843 4.997-17.858.238l-.245-.238-27.153-27.152c-4.918-4.918-4.997-12.842-.238-17.858l.238-.244 171.973-171.973-190.284-190.283c-6.331-6.331-4.36-17.006 3.73-20.693l.28-.123 291.44-124.904a12.801 12.801 0 0 1 13.855 2.482l.238.232 39.128 39.128 126.713-126.714c4.918-4.918 12.843-4.997 17.858-.238Zm-8.807 81.696L544.423 412.972l-59.871-59.87-190.07 81.458 318.862 318.863 81.459-190.07-59.871-59.871 135.764-135.764-90.509-90.51Z"/></svg>'
};
const PlayCircle = {
  name: "play-circle",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><g fill-rule="evenodd"><path d="M512 64c247.424 0 448 200.577 448 448 0 247.424-200.576 448-448 448-247.423 0-448-200.576-448-448C64 264.577 264.577 64 512 64Zm0 64c-212.077 0-384 171.923-384 384s171.923 384 384 384 384-171.923 384-384-171.923-384-384-384Z"/><path d="M361.92 275.844 737.893 500.94c8.276 4.954 8.25 17.247 0 22.243L361.92 750.803c-8.53 5.165-19.522-.454-19.522-10.425V286.399c0-9.943 10.99-15.663 19.522-10.555Zm44.493 101.164v272L632.27 512.24 406.413 377.008Z"/></g></svg>'
};
const Play = {
  name: "play",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path d="M845.376 511.488a12.8 12.8 0 0 1-1.152 14.912l-3.072 2.688-621.44 379.648a12.8 12.8 0 0 1-19.008-7.488l-.448-3.456V142.72a12.8 12.8 0 0 1 15.616-12.48l3.84 1.536L841.088 507.2a12.8 12.8 0 0 1 4.288 4.288zM724.544 518.4l-454.4-274.56v552.128l454.4-277.568z"/></svg>'
};
const PlusCircle = {
  name: "plus-circle",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><g fill-rule="evenodd"><path d="M512 64c247.424 0 448 200.577 448 448 0 247.424-200.576 448-448 448-247.423 0-448-200.576-448-448C64 264.577 264.577 64 512 64Zm0 64c-212.077 0-384 171.923-384 384s171.923 384 384 384 384-171.923 384-384-171.923-384-384-384Z"/><path d="M544 755.437V544h211.52c7.072 0 12.589-5.563 12.589-12.632v-38.736c0-7.07-5.517-12.632-12.589-12.632H544V268.377c0-7.07-5.563-12.666-12.632-12.666h-38.736c-7.07 0-12.632 5.597-12.632 12.666V480H271.528c-7.07 0-12.587 5.563-12.587 12.632v38.736c0 7.07 5.518 12.632 12.587 12.632H480v211.437c0 7.065 5.563 12.665 12.632 12.665h38.736c7.07 0 12.632-5.6 12.632-12.665Z"/></g></svg>'
};
const PlusSquare = {
  name: "plus-square",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path d="M896 140.8v742.4a12.8 12.8 0 0 1-12.8 12.8H140.8a12.8 12.8 0 0 1-12.8-12.8V140.8c0-7.04 5.76-12.8 12.8-12.8h742.4c7.04 0 12.8 5.76 12.8 12.8zM832 192H192v640h640V192z"/><path d="M531.2 255.68c7.04 0 12.8 5.76 12.8 12.8V480l211.328.064c7.04 0 12.8 5.76 12.8 12.8v38.4a12.8 12.8 0 0 1-12.8 12.8H544v211.328a12.8 12.8 0 0 1-12.8 12.8h-38.4a12.8 12.8 0 0 1-12.8-12.8V543.936L271.744 544a12.8 12.8 0 0 1-12.8-12.8v-38.4c0-7.04 5.76-12.8 12.8-12.8H480V268.416c0-7.04 5.76-12.8 12.8-12.8h38.4z"/></svg>'
};
const Plus = {
  name: "plus",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path fill-rule="evenodd" d="M492.8 896c-7.07 0-12.8-5.293-12.8-11.814V544H139.815C133.29 544 128 538.27 128 531.2v-38.4c0-7.07 5.29-12.8 11.815-12.8H480V139.815c0-6.525 5.73-11.815 12.8-11.815h38.4c7.07 0 12.8 5.29 12.8 11.815V480h340.186c6.521 0 11.814 5.73 11.814 12.8v38.4c0 7.07-5.293 12.8-11.814 12.8H544v340.186c0 6.521-5.73 11.814-12.8 11.814h-38.4Z"/></svg>'
};
const Poweroff = {
  name: "poweroff",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path d="M960 512A448 448 0 1 1 384 82.56v67.2a384 384 0 1 0 256 0l.064-67.2A448.192 448.192 0 0 1 960 512z"/><path d="M492.8 64h38.4Q544 64 544 76.8v486.4q0 12.8-12.8 12.8h-38.4q-12.8 0-12.8-12.8V76.8Q480 64 492.8 64Z"/></svg>'
};
const Projection = {
  name: "projection",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path fill-rule="evenodd" d="M64 756.364V139.636C64 133.21 69.73 128 76.8 128h870.4c7.072 0 12.8 5.21 12.8 11.636v616.728c0 6.426-5.728 11.636-12.8 11.636H716.8c-7.072 0-12.8-5.73-12.8-12.8v-38.4c0-7.07 5.728-12.8 12.8-12.8H896V192H128v512h179.2c7.07 0 12.8 5.73 12.8 12.8v38.4c0 7.07-5.73 12.8-12.8 12.8H76.8c-7.07 0-12.8-5.21-12.8-11.636Zm265.261 185.114a12.809 12.809 0 0 0-1.35 5.722c0 7.072 5.73 12.8 12.8 12.8h342.577c1.988 0 3.947-.46 5.725-1.35 6.323-3.162 8.886-10.848 5.724-17.172l-171.29-342.58a12.798 12.798 0 0 0-5.723-5.724c-6.323-3.161-14.012-.598-17.174 5.725l-171.289 342.58ZM423.488 896 512 719.04 600.448 896h-176.96Z"/></svg>'
};
const Qrcode = {
  name: "qrcode",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path fill-rule="evenodd" d="M140.8 512h358.4c7.07 0 12.8-5.73 12.8-12.8V140.8c0-7.07-5.73-12.8-12.8-12.8H140.8c-7.07 0-12.8 5.73-12.8 12.8v358.4c0 7.07 5.73 12.8 12.8 12.8ZM576 499.2c0 7.07 5.73 12.8 12.8 12.8h358.4c7.072 0 12.8-5.73 12.8-12.8V140.8c0-7.07-5.728-12.8-12.8-12.8H588.8c-7.07 0-12.8 5.73-12.8 12.8v358.4ZM192 448V192h256v256H192Zm448 0V192h256v256H640Zm-320-76.8v-38.4c0-7.07 5.73-12.8 12.8-12.8h38.4c7.07 0 12.8 5.73 12.8 12.8v38.4c0 7.07-5.73 12.8-12.8 12.8h-38.4c-7.07 0-12.8-5.73-12.8-12.8Zm448-38.4v38.4c0 7.07 5.728 12.8 12.8 12.8h38.4c7.072 0 12.8-5.73 12.8-12.8v-38.4c0-7.07-5.728-12.8-12.8-12.8h-38.4c-7.072 0-12.8 5.73-12.8 12.8ZM140.8 960h358.4c7.07 0 12.8-5.728 12.8-12.8V588.8c0-7.07-5.73-12.8-12.8-12.8H140.8c-7.07 0-12.8 5.73-12.8 12.8v358.4c0 7.072 5.73 12.8 12.8 12.8Zm486.4 0h-38.4c-7.07 0-12.8-5.728-12.8-12.8V588.8c0-7.07 5.73-12.8 12.8-12.8h165.364c7.072 0 12.804 5.734 12.8 12.806L766.907 704h128.639l.019-115.202c0-7.068 5.728-12.798 12.8-12.798h38.4c7.065 0 12.8 5.73 12.8 12.8v166.4c0 7.07-5.735 12.8-12.8 12.8h-231c-7.067 0-12.797-5.727-12.8-12.794L702.906 640h-62.912L640 947.2c0 7.072-5.73 12.8-12.8 12.8ZM192 896V640h256v256H192Zm128-76.8v-38.4c0-7.072 5.73-12.8 12.8-12.8h38.4c7.07 0 12.8 5.728 12.8 12.8v38.4c0 7.072-5.73 12.8-12.8 12.8h-38.4c-7.07 0-12.8-5.728-12.8-12.8ZM716.8 832c-7.07 0-12.8 5.728-12.8 12.8v102.4c0 7.072 5.73 12.8 12.8 12.8h38.4c7.07 0 12.8-5.728 12.8-12.8V844.8c0-7.072-5.73-12.8-12.8-12.8h-38.4Zm192 128c-7.072 0-12.8-5.728-12.8-12.8V844.8c0-7.072 5.728-12.8 12.8-12.8h38.4c7.072 0 12.8 5.728 12.8 12.8v102.4c0 7.072-5.728 12.8-12.8 12.8h-38.4Z"/></svg>'
};
const QuestionCircleFilled = {
  name: "question-circle-filled",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path d="M512 64c247.424 0 448 200.576 448 448S759.424 960 512 960 64 759.424 64 512 264.576 64 512 64zm.853 656.939a48 48 0 1 0 0 96 48 48 0 0 0 0-96zM512 192a192.021 192.021 0 0 0-191.019 172.48 144.576 144.576 0 0 0-.469 6.72 12.8 12.8 0 0 0 12.8 12.8h38.57a12.8 12.8 0 0 0 12.758-11.819c.213-2.773.448-5.077.725-6.933A128.021 128.021 0 0 1 640 384a128 128 0 0 1-128 128h-32v115.2a12.8 12.8 0 0 0 12.8 12.8h38.4a12.8 12.8 0 0 0 12.8-12.8v-53.867A192 192 0 0 0 512 192z"/></svg>'
};
const QuestionCircle = {
  name: "question-circle",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path d="M512 64a448 448 0 1 1 0 896 448 448 0 0 1 0-896zm0 64a384 384 0 1 0 0 768 384 384 0 0 0 0-768zm0 64a192 192 0 1 1 0 384v-64a128 128 0 1 0-128-128h-64a192 192 0 0 1 192-192zm-32 320h64v115.2a12.8 12.8 0 0 1-12.8 12.8h-38.4a12.8 12.8 0 0 1-12.8-12.8V512zm32.832 208.96q48 0 48 48t-48 48q-48 0-48-48t48-48Z"/></svg>'
};
const Question = {
  name: "question",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path fill-rule="evenodd" d="M229.567 416h39.498c6.843 0 12.442-5.45 12.626-12.29C288.07 285.713 385.77 192 505.359 192c123.712 0 224 100.288 224 224s-100.288 224-224 224h-19.5c-7.07 0-12.8 5.73-12.8 12.8v102.4c0 7.07 5.73 12.8 12.8 12.8h38.4c7.069 0 12.8-5.73 12.8-12.8v-52.925c144.143-15.78 256.3-137.93 256.3-286.275 0-159.058-128.942-288-288-288-154.969 0-281.35 122.398-287.746 275.795a97.31 97.31 0 0 0-.003.081c-.276 6.716 5.236 12.124 11.957 12.124Zm276.335 432.928c-26.51 0-48 21.491-48 48s21.49 48 48 48 48-21.491 48-48-21.49-48-48-48Z"/></svg>'
};
const RadarChart = {
  name: "radar-chart",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path fill-rule="evenodd" d="M518.153 66.739 953.414 409.04a12.794 12.794 0 0 1 4.436 13.434L814.566 947.078a12.809 12.809 0 0 1-12.345 9.428H219.876a12.802 12.802 0 0 1-12.349-9.434L64.452 422.453a12.8 12.8 0 0 1 4.416-13.413l433.44-342.285a12.8 12.8 0 0 1 15.845-.016ZM647.356 799.66H375.288L300.1 892.506h422.44l-75.184-92.845ZM146.572 480.453l102.005 373.985 78.91-97.446-62.355-228.637-118.56-47.902Zm728.948.392-118.451 47.857-62.243 227.88 78.806 97.319L875.52 480.845ZM515.235 636.503h-7.826l-80.28 99.139h168.385l-80.28-99.139Zm-175.593-78.044 37.455 137.27 74.522-92.028-111.977-45.242Zm342.807.39-111.323 44.978 74.041 91.433 37.282-136.41ZM542.224 393.927v139.9l10.614 8.363 117.524-47.482-128.138-100.78Zm-64 .802-126.273 99.678 116.483 47.062 9.79-7.747V394.728Zm64-227.584V312.51l196.534 154.562 122.125-49.342-318.659-250.587Zm-64 .105L161.412 417.423l122.312 49.417 194.5-153.596V167.25Z"/></svg>'
};
const Radar = {
  name: "radar",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path fill-rule="evenodd" d="M700.864 105.519A446.09 446.09 0 0 0 512.234 64C224.8 64 11.709 330.794 75.286 611.068c63.557 280.299 370.88 429.093 630.174 305.099 254.432-121.664 333.46-444.213 167.898-669.262-4.845-6.586-14.663-6.798-19.84-.469l-25.248 30.886c-3.693 4.517-3.834 10.965-.506 15.756 149.664 215.338 43.584 515.934-210.546 588.183-257.328 73.159-508.733-132.646-487.84-399.353 20.638-263.456 294.482-426.836 535.18-322.5 5.359 2.323 11.478.826 15.174-3.694l25.319-30.958c5.17-6.324 3.225-15.795-4.186-19.237Zm87.725 42.963 29.568 24.512c5.408 4.514 6.15 12.553 1.664 17.984L572.253 489.78a63.993 63.993 0 0 1 3.981 22.22c0 35.346-28.653 64-64 64-35.346 0-64-28.654-64-64 0-35.345 28.654-64 64-64 3.611 0 7.215.306 10.774.914L770.541 150.21c4.518-5.452 12.601-6.198 18.048-1.728ZM486.525 766.676C637.297 781.895 768.218 663.538 768.237 512c0-37.808-8.23-73.812-23.002-106.206-3.77-8.257-14.694-9.134-20.435-2.107l-28.305 34.625c-2.894 3.54-3.66 8.309-2.203 12.642 36.914 109.76-31.47 227.931-146.355 249.67-117.304 22.198-226.21-66.909-227.681-186.285C318.814 397.425 421.105 306.997 536 321.444c4.538.57 9.06-1.127 11.956-4.666l28.288-34.566c5.748-7.024 2.52-17.616-6.32-19.673-138.958-32.34-279.543 56.082-308.531 198.302-30.265 148.485 74.361 290.616 225.132 305.835Z"/></svg>'
};
const Redo = {
  name: "redo",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path d="M512 64a448 448 0 0 0-377.344 689.6L292.8 595.392l.64-.896A12.8 12.8 0 0 0 281.92 576H133.376a384 384 0 1 1 130.688 229.12l-45.376 45.44A448 448 0 1 0 512 64z"/></svg>'
};
const Reload = {
  name: "reload",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path d="M512 64a448 448 0 0 1 377.344 689.6L731.2 595.392l-.64-.896A12.8 12.8 0 0 1 742.08 576h148.608A384 384 0 1 0 760 805.12l45.376 45.44A448 448 0 1 1 512 64z"/></svg>'
};
const RightCircle = {
  name: "right-circle",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><g fill-rule="evenodd"><path d="M512 64c247.424 0 448 200.577 448 448 0 247.424-200.576 448-448 448-247.423 0-448-200.576-448-448C64 264.577 264.577 64 512 64Zm0 64c-212.077 0-384 171.923-384 384s171.923 384 384 384 384-171.923 384-384-171.923-384-384-384Z"/><path d="M608.264 511.923 383.841 287.567c-5-5-5.07-12.933-.07-17.932l27.39-27.39c4.999-4.999 13.105-4.76 18.104.238l260.348 260.349c4.999 4.998 5.181 13.16.183 18.16L425.343 785.44c-4.998 5.005-12.921 5.056-17.92.058l-27.39-27.386c-4.998-4.998-4.784-13.088.213-18.086l228.018-228.103Z"/></g></svg>'
};
const RightDouble = {
  name: "right-double",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path d="M490.752 265.344a32 32 0 0 0 0 45.312L681.92 501.824a12.8 12.8 0 0 1 0 18.112L489.344 712.448a32 32 0 0 0 45.312 45.248l224.192-224.192a32 32 0 0 0 0-45.248L536 265.344a32 32 0 0 0-45.248 0zm-224 0a32 32 0 0 0 0 45.312L457.92 501.76a12.8 12.8 0 0 1 0 18.112L265.344 712.384a32 32 0 1 0 45.312 45.248L534.848 533.44a32 32 0 0 0 0-45.248L312 265.344a32 32 0 0 0-45.248 0z"/></svg>'
};
const RightFilled = {
  name: "right-filled",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path d="M512 0c282.784 0 512 229.216 512 512s-229.216 512-512 512S0 794.784 0 512 229.216 0 512 0zm-54.944 294.784a12.8 12.8 0 0 0-18.112 0l-27.136 27.136a12.8 12.8 0 0 0 0 18.112L583.776 512 411.808 683.968a12.8 12.8 0 0 0 0 18.112l27.136 27.136a12.8 12.8 0 0 0 18.112 0l208.16-208.16 1.632-2.016a12.8 12.8 0 0 0-1.6-16.096z"/></svg>'
};
const Right = {
  name: "right",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path d="M710.763 521.104 457.104 774.763c-4.998 4.998-12.869 5.001-17.867.003l-27.39-27.39c-5-5-4.939-12.927.06-17.926l217.281-217.28-217.28-217.281c-5-4.999-5.06-12.927-.062-17.925l27.39-27.39c5-5 12.87-4.996 17.868.002l253.659 253.659c4.998 4.998 4.998 12.872 0 17.87Z"/></svg>'
};
const RingChart = {
  name: "ring-chart",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path fill-rule="evenodd" d="M831.552 525.863c-9.114-181.382-155.515-326.579-337.395-333.773l-1.61-.032C485.578 191.92 480 186.23 480 179.26V76.74c0-7.12 5.85-12.892 12.969-12.737 2.595.061 4.802.127 6.621.2 250.868 10.07 452.173 212.677 460.16 464.072l.047 2.532c.107 7.148-5.653 13.003-12.802 13.003H844.64c-7.008 0-12.678-5.715-12.89-12.719a267 267 0 0 0-.198-5.228ZM480 961.397c-229.75 0-416-186.253-416-416.002 0-187.971 125.681-350.633 303.114-400.496l2.71-.706c6.932-1.806 13.998 2.43 15.672 9.396l8.955 37.25c1.637 6.814-2.352 13.602-9.113 15.442-1.823.496-3.4.94-4.73 1.33C232.56 251.035 128 387.702 128 545.394c0 194.402 157.596 352.002 352 352.002 158.05 0 295.02-105.043 338.106-253.711l.646-2.415c1.792-6.728 8.634-10.787 15.398-9.136l37.447 9.138c6.918 1.688 11.168 8.678 9.299 15.545a227.127 227.127 0 0 1-.845 3.02C829.664 836.488 667.384 961.398 480 961.398Z"/></svg>'
};
const Robot = {
  name: "robot",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><g fill-rule="evenodd"><path d="M320 179.2v-38.4c0-7.07 5.73-12.8 12.8-12.8h358.4c7.07 0 12.8 5.73 12.8 12.8v38.4c0 7.07-5.73 12.8-12.8 12.8H332.8c-7.07 0-12.8-5.73-12.8-12.8ZM819.2 256c6.97 0 12.634 5.566 12.796 12.493l.004.307V384h115.2c6.97 0 12.634 5.566 12.8 12.493V755.2c0 6.967-5.563 12.634-12.493 12.796l-.307.004H832v115.2c0 6.97-5.563 12.634-12.493 12.8H204.8c-6.967 0-12.634-5.563-12.796-12.493L192 883.2V768H76.8c-6.967 0-12.634-5.566-12.8-12.493V396.8c0-6.967 5.566-12.634 12.493-12.796L76.8 384H192V268.8c0-6.967 5.566-12.634 12.493-12.8H819.2ZM768 320H256v512h512V320ZM192 448h-64v256h64V448Zm704 0h-64v256h64V448Z"/><path d="M384 563.2V460.8c0-7.07 5.73-12.8 12.8-12.8h38.4c7.07 0 12.8 5.73 12.8 12.8v102.4c0 7.07-5.73 12.8-12.8 12.8h-38.4c-7.07 0-12.8-5.73-12.8-12.8ZM576 563.2V460.8c0-7.07 5.73-12.8 12.8-12.8h38.4c7.07 0 12.8 5.73 12.8 12.8v102.4c0 7.07-5.73 12.8-12.8 12.8h-38.4c-7.07 0-12.8-5.73-12.8-12.8Z"/></g></svg>'
};
const Rollback = {
  name: "rollback",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path d="M53.056 320h224.96V95.296a12.8 12.8 0 0 0-21.824-9.088L44.032 298.112A12.8 12.8 0 0 0 53.056 320z"/><path d="M641.472 256a320.768 320.768 0 0 1 13.952 641.28l-13.952.256h-234.24a12.8 12.8 0 0 1-12.8-12.8v-38.4c0-7.04 5.76-12.8 12.8-12.8h234.24a256.768 256.768 0 0 0 12.8-513.28l-12.8-.256H96v-64h545.472zm-384.64 564.16q48 0 48 48t-48 48q-48 0-48-48t48-48Z"/></svg>'
};
const RotateLeft = {
  name: "rotate-left",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path fill-rule="evenodd" d="M462.19 319.456h222.412c114.233 6.21 205.69 98.006 211.366 212.378v.074c.307 6.251 5.312 11.213 11.571 11.213h39.61c7.18 0 12.96-5.913 12.797-13.096l-.016-.461c-6.848-147.817-125.127-266.53-272.765-274.063V94.746c0-11.4-13.78-17.113-21.845-9.057L453.146 297.6c-8.071 8.062-2.363 21.857 9.045 21.857ZM76.8 959.872h678.4c7.072 0 12.8-5.734 12.8-12.8V396.671c0-7.07-5.728-12.8-12.8-12.8H76.8c-7.07 0-12.8 5.73-12.8 12.8v550.401c0 7.066 5.73 12.8 12.8 12.8Zm51.2-64V447.871h576v448.001H128Z"/></svg>'
};
const RotateRight = {
  name: "rotate-right",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path fill-rule="evenodd" d="M561.76 319.456H339.349c-114.231 6.21-205.69 98.006-211.365 212.378l-.004.074c-.306 6.251-5.308 11.213-11.567 11.213h-39.61c-7.185 0-12.962-5.913-12.8-13.096l.015-.461c6.85-147.817 125.13-266.53 272.767-274.063V94.746c0-11.4 13.78-17.113 21.846-9.057L570.805 297.6c8.07 8.062 2.362 21.857-9.045 21.857Zm385.443 640.416H268.805c-7.07 0-12.8-5.734-12.8-12.8V396.671c0-7.07 5.73-12.8 12.8-12.8h678.398c7.072 0 12.8 5.73 12.8 12.8v550.401c0 7.066-5.728 12.8-12.8 12.8Zm-51.2-64V447.871H320.005v448.001h575.998Z"/></svg>'
};
const Safety = {
  name: "safety",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path d="M518.656 66.304H507.2L138.24 133.12a12.8 12.8 0 0 0-10.496 12.608V334.4c0 259.456 141.888 498.048 369.792 622.016l15.616 8.512 33.28-18.88a707.2 707.2 0 0 0 349.504-609.92V145.664a12.8 12.8 0 0 0-10.496-12.544L518.72 66.24zm-5.824 63.936 319.04 58.176v147.648l-.32 20.736a643.008 643.008 0 0 1-299.52 523.008L512.64 891.52l-2.176-1.28A644.032 644.032 0 0 1 191.68 334.4l-.064-145.92 321.216-58.24z"/><path d="m261.76 420.224 28.096-27.008a12.8 12.8 0 0 1 17.92.128l164.8 164.864a12.8 12.8 0 0 0 18.112 0l251.648-251.52a12.8 12.8 0 0 1 18.112 0l27.136 27.136a12.8 12.8 0 0 1 0 18.112L490.368 649.152a12.8 12.8 0 0 1-18.112 0L261.632 438.528a12.8 12.8 0 0 1 .128-18.304z"/></svg>'
};
const ScanSecurity = {
  name: "scan-security",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path d="m518.656 66.304 366.72 66.88a12.8 12.8 0 0 1 10.496 12.544v190.336c0 43.008-3.84 85.376-11.456 126.848a287.616 287.616 0 0 0-53.056-100.736l.192-5.376.32-20.736V188.416l-319.04-58.176-321.216 58.24V334.4c0 229.248 121.856 440.64 318.848 555.84l2.176 1.28 19.392-11.712c23.36-14.72 45.504-30.72 66.496-48.064A287.36 287.36 0 0 0 710.4 813.312a704.128 704.128 0 0 1-164.032 132.736l-33.28 18.88-15.616-8.512A708.032 708.032 0 0 1 127.68 334.4V145.728a12.8 12.8 0 0 1 10.496-12.608L507.2 66.304h11.456z"/><path d="M608 320a224 224 0 0 1 179.392 358.144l94.08 94.08a12.8 12.8 0 0 1 0 18.112l-27.136 27.136a12.8 12.8 0 0 1-18.112 0l-94.08-94.08A224 224 0 1 1 608 320zm0 64a160 160 0 1 0 0 320 160 160 0 0 0 0-320z"/></svg>'
};
const ScanVirus = {
  name: "scan-virus",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path d="M512 64a446.08 446.08 0 0 1 202.88 48.448l-41.664 50.944a384 384 0 1 0 148.544 121.6l41.6-50.88A448 448 0 1 1 512 64zm256 448a256 256 0 1 1-179.968-244.48l-44.992 54.976a192 192 0 1 0 148.544 121.536l44.992-55.04C756.608 425.536 768 467.52 768 512z"/><path d="M448 512a64 64 0 1 0 128 0 64 64 0 1 0-128 0Z"/><path d="m788.352 148.48 29.568 24.512a12.8 12.8 0 0 1 1.664 17.984L536.64 532.48l-49.28-40.832 282.944-341.44a12.8 12.8 0 0 1 18.048-1.664z"/></svg>'
};
const Scan = {
  name: "scan",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path fill-rule="evenodd" d="M64 76.8v294.4c0 7.07 5.73 12.8 12.8 12.8h38.4c7.07 0 12.8-5.73 12.8-12.8V128h704v243.2c0 7.07 5.728 12.8 12.8 12.8h38.4c7.072 0 12.8-5.73 12.8-12.8V76.8c0-7.07-5.728-12.8-12.8-12.8H76.8C69.73 64 64 69.73 64 76.8Zm0 384v38.4c0 7.07 5.73 12.8 12.8 12.8h806.4c7.072 0 12.8-5.73 12.8-12.8v-38.4c0-7.07-5.728-12.8-12.8-12.8H76.8c-7.07 0-12.8 5.73-12.8 12.8ZM883.2 896H76.8c-7.07 0-12.8-5.728-12.8-12.8V588.8c0-7.07 5.73-12.8 12.8-12.8h38.4c7.07 0 12.8 5.73 12.8 12.8V832h704V588.8c0-7.07 5.728-12.8 12.8-12.8h38.4c7.072 0 12.8 5.73 12.8 12.8v294.4c0 7.072-5.728 12.8-12.8 12.8Z"/></svg>'
};
const Search = {
  name: "search",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path fill-rule="evenodd" d="M448 96c212.077 0 384 171.923 384 384 0 86.337-28.493 166.019-76.588 230.155l154.027 154.163c4.991 5.003 4.99 13.104-.004 18.106l-27.011 27.011c-5.002 4.993-13.103 4.995-18.106.004L712.82 758.076C643.925 823.709 550.668 864 448 864 235.923 864 64 692.077 64 480S235.923 96 448 96Zm0 64c-176.731 0-320 143.269-320 320s143.269 320 320 320 320-143.269 320-320-143.269-320-320-320Z"/></svg>'
};
const SecurityAlert = {
  name: "security-alert",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path fill-rule="evenodd" d="M896.224 333.57c0 250.686-132.99 482.467-349.528 609.415L519.57 958.34a12.815 12.815 0 0 1-12.425.108l-9.364-5.09C269.877 829.53 128 591.087 128 331.89V143.384a12.792 12.792 0 0 1 10.518-12.584l367.879-66.595a12.853 12.853 0 0 1 2.29-.205h9.136c.768 0 1.534.075 2.29.213l365.609 66.608a12.794 12.794 0 0 1 10.502 12.581V333.57ZM192 331.887c0 229.033 121.784 440.25 318.78 555.39l2.186 1.246 19.419-11.67c180.29-113.623 292.658-309.481 299.506-522.608l.333-20.678-.026-147.538-319.04-58.125-321.216 58.19.058 145.793ZM531.526 256h-38.4c-7.069 0-12.8 5.726-12.8 12.789v294.144c0 7.063 5.731 12.789 12.8 12.789h38.4c7.07 0 12.8-5.726 12.8-12.789V268.79c0-7.063-5.73-12.789-12.8-12.789m0 447.944h-38.4c-7.069 0-12.8-5.726-12.8-12.788V652.79c0-7.063 5.731-12.789 12.8-12.789h38.4c7.07 0 12.8 5.726 12.8 12.789v38.367c0 7.062-5.73 12.788-12.8 12.788"/></svg>'
};
const SecurityScan = {
  name: "security-scan",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><g fill-rule="evenodd"><path d="M546.696 943.75a708.559 708.559 0 0 0 126.706-94.816c9.83-9.241-.62-26.195-13.899-23.795a287.688 287.688 0 0 1-55.986 4.544c-3.12-.051-6.107.96-8.529 2.925a642.555 642.555 0 0 1-62.605 44.954l-19.418 11.68-2.185-1.248C313.782 772.755 192 561.354 192 332.122L191.94 186.2l321.216-58.24 319.041 58.176.026 147.667-.333 20.695-.019.596c-.102 3.12.858 6.079 2.79 8.533a288.341 288.341 0 0 1 30.042 46.89c6.15 12.014 26.323 10.003 27.718-3.42a708.78 708.78 0 0 0 3.802-73.294v-201.01L518.977 64h-11.445L128 132.764v199.358c0 259.424 141.877 498.073 369.782 622.01l15.606 8.486 33.308-18.868Z"/><path d="M608.328 317.723c123.712 0 223.998 100.288 223.998 224a224.019 224.019 0 0 1-43.737 132.974l-.864 1.165 94.06 94.064c4.925 4.918 5.004 12.847.239 17.862l-.238.244-27.15 27.149c-4.917 4.918-12.846 4.997-17.86.238l-.245-.238-94.063-94.061a224.021 224.021 0 0 1-134.14 44.602c-123.712 0-224-100.288-224-224 0-123.711 100.288-224 224-224Zm0 64c-88.366 0-160 71.634-160 160 0 88.365 71.634 160 160 160s159.998-71.635 159.998-160c0-88.366-71.632-160-159.998-160Z"/></g></svg>'
};
const Select = {
  name: "select",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path fill-rule="evenodd" d="M64 128v768c0 35.347 28.654 64 64 64h768c35.347 0 64-28.653 64-64V128c0-35.346-28.653-64-64-64H128c-35.346 0-64 28.654-64 64Zm832 768H128V128h768v768ZM774.598 440.294l-253.661 253.66c-4.998 4.998-12.872 4.998-17.87 0l-253.659-253.66c-4.999-4.998-5.002-12.868-.003-17.867l27.39-27.39c4.999-4.999 12.927-4.938 17.926.06l208.288 208.289a12.66 12.66 0 0 0 17.986 0L729.28 395.098c4.998-5 12.928-5.06 17.926-.061l27.392 27.39c4.999 4.999 4.999 12.869 0 17.867Z"/></svg>'
};
const Send = {
  name: "send",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path d="M922.624 80.256a44.608 44.608 0 0 1 57.408 49.344l-1.536 6.656-262.4 830.976a33.152 33.152 0 0 1-51.456 16.512l-3.648-3.072-252.16-252.16 45.056-45.056 216.576 216.576 236.288-748.16-750.208 234.176L322.56 552.064l-45.056 45.12L75.712 395.392a33.152 33.152 0 0 1 9.856-53.568l3.712-1.536L922.624 80.256z"/><path d="M961.088 102.208a35.072 35.072 0 0 0-49.024-7.68l-644.16 468.8v259.136a38.272 38.272 0 0 0 66.88 25.408l624.128-701.824a35.072 35.072 0 0 0 2.176-43.904zM734.4 302.592 331.648 755.52V595.776L734.4 302.592z"/></svg>'
};
const ServerStar = {
  name: "server-star",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path fill-rule="evenodd" d="M76.8 448h870.4c7.072 0 12.8-5.73 12.8-12.8V76.8c0-7.07-5.728-12.8-12.8-12.8H76.8C69.73 64 64 69.73 64 76.8v358.4c0 7.07 5.73 12.8 12.8 12.8ZM896 128v256H128V128h768ZM652.8 320h38.4c7.072 0 12.8-5.73 12.8-12.8V204.8c0-7.07-5.728-12.8-12.8-12.8h-38.4c-7.072 0-12.8 5.73-12.8 12.8v102.4c0 7.07 5.728 12.8 12.8 12.8ZM832 204.8v102.4c0 7.07-5.728 12.8-12.8 12.8h-38.4c-7.072 0-12.8-5.73-12.8-12.8V204.8c0-7.07 5.728-12.8 12.8-12.8h38.4c7.072 0 12.8 5.73 12.8 12.8ZM478.103 660.143c-12.399 0-17.554 15.866-7.523 23.155l137.407 99.838-52.48 161.542c-3.831 11.79 9.665 21.594 19.697 14.31l137.417-99.833 137.414 99.834c10.029 7.283 23.527-2.522 19.693-14.31l-52.48-161.543 137.408-99.838c10.029-7.289 4.877-23.155-7.52-23.155l-169.85-.007-52.492-161.534c-3.834-11.792-20.519-11.792-24.35 0l-52.491 161.534-169.85.007ZM128 576v256h307.2c7.07 0 12.8 5.728 12.8 12.8v38.4c0 7.072-5.73 12.8-12.8 12.8H76.8c-7.07 0-12.8-5.728-12.8-12.8V524.8c0-7.07 5.73-12.8 12.8-12.8h358.4c7.07 0 12.8 5.73 12.8 12.8v38.4c0 7.07-5.73 12.8-12.8 12.8H128Z"/></svg>'
};
const Server = {
  name: "server",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path d="M947.2 512c7.04 0 12.8 5.76 12.8 12.8v358.4a12.8 12.8 0 0 1-12.8 12.8H76.8A12.8 12.8 0 0 1 64 883.2V524.8c0-7.04 5.76-12.8 12.8-12.8h870.4zM896 576H128v256h768V576z"/><path d="M524.8 640h294.4q12.8 0 12.8 12.8v38.4q0 12.8-12.8 12.8H524.8q-12.8 0-12.8-12.8v-38.4q0-12.8 12.8-12.8ZM947.2 64c7.04 0 12.8 5.76 12.8 12.8v358.4a12.8 12.8 0 0 1-12.8 12.8H76.8A12.8 12.8 0 0 1 64 435.2V76.8C64 69.76 69.76 64 76.8 64h870.4zM896 128H128v256h768V128z"/><path d="M780.8 192h38.4q12.8 0 12.8 12.8v102.4q0 12.8-12.8 12.8h-38.4q-12.8 0-12.8-12.8V204.8q0-12.8 12.8-12.8Zm-128 0h38.4q12.8 0 12.8 12.8v102.4q0 12.8-12.8 12.8h-38.4q-12.8 0-12.8-12.8V204.8q0-12.8 12.8-12.8Z"/></svg>'
};
const Setting = {
  name: "setting",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path d="M627.2 64c7.04 0 12.8 5.76 12.8 12.8v141.824c22.08 9.6 42.88 21.76 62.016 35.84l122.88-70.912a12.8 12.8 0 0 1 17.472 4.672l115.2 199.552a12.8 12.8 0 0 1-4.672 17.472l-122.88 71.04a323.52 323.52 0 0 1 0 71.488l122.88 70.976a12.8 12.8 0 0 1 4.672 17.472l-115.2 199.552a12.8 12.8 0 0 1-17.472 4.672l-122.88-70.912c-19.2 14.08-39.936 26.24-61.952 35.84L640 947.2a12.8 12.8 0 0 1-12.8 12.8H396.8a12.8 12.8 0 0 1-12.8-12.8V805.376c-22.08-9.6-42.88-21.76-62.016-35.84l-122.88 70.912a12.8 12.8 0 0 1-17.472-4.672l-115.2-199.552a12.8 12.8 0 0 1 4.672-17.472l122.88-71.04a323.52 323.52 0 0 1 0-71.488L71.04 405.248a12.8 12.8 0 0 1-4.672-17.472l115.2-199.552a12.8 12.8 0 0 1 17.472-4.672l122.88 70.912c19.2-14.08 39.936-26.24 62.08-35.84V76.8c0-7.04 5.76-12.8 12.8-12.8h230.4zM576 128H447.936l.128 132.48-38.4 16.768a253.593 253.593 0 0 0-37.824 20.48l-11.776 8.192-33.728 24.96-114.944-66.368-64 110.912 114.816 66.304-4.608 41.6A259.52 259.52 0 0 0 256 512l.384 14.4 1.216 14.272 4.608 41.6-114.816 66.24 64 110.912 114.944-66.304 33.728 24.96a256 256 0 0 0 36.608 22.528l12.992 6.144 38.4 16.768L448 896h128V763.52l38.4-16.832c13.184-5.76 25.792-12.544 37.76-20.48l11.776-8.128 33.728-24.96 114.88 66.304 64-110.912-114.752-66.24 4.608-41.6c1.088-9.472 1.6-19.008 1.6-28.672l-.384-14.4-1.216-14.208-4.608-41.6 114.752-66.368-64-110.912-114.88 66.368-33.728-24.96a256 256 0 0 0-36.544-22.528l-12.992-6.08L576 260.48V128z"/><path d="M512 384a128 128 0 1 1 0 256 128 128 0 0 1 0-256zm0 64a64 64 0 1 0 0 128 64 64 0 0 0 0-128z"/></svg>'
};
const Shopping = {
  name: "shopping",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path fill-rule="evenodd" d="M307.567 478.298h-38.4c-7.07 0-12.8-5.73-12.8-12.8v-145.26c0-.774.004-1.547.01-2.32H192V829.92h640V317.918h-63.213l.058 2.32v145.26c0 7.07-5.732 12.8-12.801 12.8h-38.4c-7.07 0-12.8-5.73-12.8-12.8v-145.26c0-.774-.005-1.547-.014-2.32H320.434l-.067 2.32v145.26c0 7.07-5.73 12.8-12.8 12.8Zm-42.534-224.38C294.257 144.55 394.024 64 512.605 64c118.582 0 218.35 80.55 247.573 189.918H883.2c7.072 0 12.8 5.73 12.8 12.8V881.12c0 7.066-5.728 12.8-12.8 12.8H140.8c-7.07 0-12.8-5.734-12.8-12.8V266.718c0-7.07 5.73-12.8 12.8-12.8h124.233Zm428.064 0C666.084 180.426 595.467 128 512.605 128s-153.479 52.426-180.491 125.918h360.983Z"/></svg>'
};
const SortAscending = {
  name: "sort-ascending",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path d="M298.752 938.048 256.832 896V76.8c0-7.04 5.76-12.8 12.8-12.8h38.208c7.04 0 12.8 5.76 12.8 12.8v852.224a12.8 12.8 0 0 1-21.888 9.024z"/><path d="m299.264 940.8-167.04-150.464A12.8 12.8 0 0 1 140.8 768h179.84v163.2a12.8 12.8 0 0 1-21.376 9.6zM834.752 576c7.04 0 12.8 5.76 12.8 12.8v38.4l-.192 2.24a12.8 12.8 0 0 1-3.392 12.16L589.504 896h245.248c7.04 0 12.8 5.76 12.8 12.8v38.4a12.8 12.8 0 0 1-12.8 12.8H509.184a12.8 12.8 0 0 1-12.8-12.8v-38.4l.192-2.048a12.8 12.8 0 0 1 3.456-11.712L755.008 640H509.184a12.8 12.8 0 0 1-12.8-12.8v-38.4c0-7.04 5.76-12.8 12.8-12.8h325.568zM675.712 57.792c3.136-.064 5.76.64 8.192 2.176a12.8 12.8 0 0 1 3.968 4.288l207.296 359.04a12.8 12.8 0 0 1-4.672 17.408l-33.152 19.2a12.8 12.8 0 0 1-17.536-4.864l-60.16-107.2h-210.24L508.16 455.168a12.8 12.8 0 0 1-17.536 4.736l-33.216-19.2a12.8 12.8 0 0 1-4.672-17.472l207.68-359.808a12.8 12.8 0 0 1 15.296-5.632zm68.032 226.048-68.16-121.472L606.08 283.84h137.664z"/></svg>'
};
const SortDescending = {
  name: "sort-descending",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path d="M307.84 960h-38.208a12.8 12.8 0 0 1-12.8-12.8V127.04a12.8 12.8 0 0 1 4.288-9.536L299.264 83.2a12.8 12.8 0 0 1 21.376 9.536V947.2a12.8 12.8 0 0 1-12.8 12.8z"/><path d="m299.264 83.2-167.04 150.464A12.8 12.8 0 0 0 140.8 256h179.84V92.8a12.8 12.8 0 0 0-21.376-9.6zM834.752 576c7.04 0 12.8 5.76 12.8 12.8v38.4l-.192 2.24a12.8 12.8 0 0 1-3.392 12.16L589.504 896h245.248c7.04 0 12.8 5.76 12.8 12.8v38.4a12.8 12.8 0 0 1-12.8 12.8H509.184a12.8 12.8 0 0 1-12.8-12.8v-38.4l.192-2.048a12.8 12.8 0 0 1 3.456-11.712L755.008 640H509.184a12.8 12.8 0 0 1-12.8-12.8v-38.4c0-7.04 5.76-12.8 12.8-12.8h325.568zM675.712 57.792c3.136-.064 5.76.64 8.192 2.176a12.8 12.8 0 0 1 3.968 4.288l207.296 359.04a12.8 12.8 0 0 1-4.672 17.408l-33.152 19.2a12.8 12.8 0 0 1-17.536-4.864l-60.16-107.2h-210.24L508.16 455.168a12.8 12.8 0 0 1-17.536 4.736l-33.216-19.2a12.8 12.8 0 0 1-4.672-17.472l207.68-359.808a12.8 12.8 0 0 1 15.296-5.632zm68.032 226.048-68.16-121.472L606.08 283.84h137.664z"/></svg>'
};
const Sound = {
  name: "sound",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><g fill-rule="evenodd"><path d="M576 90.904v842.734c0 10.733-12.415 16.704-20.796 9.997L512 909.075 256 704.272H76.8c-7.07 0-12.8-5.73-12.8-12.8v-358.4c0-7.07 5.73-12.8 12.8-12.8H256L555.204 80.909c8.38-6.705 20.796-.738 20.796 9.995Zm-64 106.528-233.55 186.84H128v256h150.45L512 827.11V197.432ZM779.194 168.74c4.345-5.583 12.358-6.635 17.81-2.133C896.55 248.777 960 373.117 960 512.275c0 139.159-63.45 263.501-162.995 345.67-5.37 4.43-13.22 3.481-17.608-1.882l-.203-.255-23.47-30.176c-4.306-5.536-3.295-13.53 2.049-18.067a426.884 426.884 0 0 0 25.753-23.763C858.51 708.822 896 618.314 896 512.275c0-106.038-37.491-196.548-112.474-271.529a428.284 428.284 0 0 0-25.753-23.763c-5.26-4.466-6.322-12.277-2.25-17.8l.202-.265 23.469-30.178Z"/><path d="M661.312 320.303c4.342-5.583 12.45-6.58 17.806-1.96C733.548 365.288 768 434.758 768 512.275c0 77.517-34.451 146.987-88.882 193.932-5.273 4.549-13.215 3.653-17.603-1.705l-.203-.254-23.566-30.3c-4.305-5.535-3.415-13.403 1.794-18.097a218.132 218.132 0 0 0 8.224-7.811C685.254 610.55 704 565.295 704 512.275c0-53.02-18.745-98.274-56.236-135.764a218.821 218.821 0 0 0-8.224-7.811c-5.125-4.62-6.07-12.31-1.998-17.829l.204-.27 23.566-30.298Z"/></g></svg>'
};
const StarFilled = {
  name: "star-filled",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path d="M652.181 340.395 523.243 78.955a12.8 12.8 0 0 0-22.955 0L371.2 340.395a12.8 12.8 0 0 1-9.621 6.997L73.024 389.376A12.8 12.8 0 0 0 65.92 411.2l208.81 203.499a12.8 12.8 0 0 1 3.691 11.306L229.12 913.493a12.8 12.8 0 0 0 18.56 13.504l258.09-135.744a12.8 12.8 0 0 1 11.905 0l258.133 135.744a12.8 12.8 0 0 0 18.56-13.504l-49.323-287.381a12.8 12.8 0 0 1 3.691-11.328l208.853-203.563a12.8 12.8 0 0 0-7.104-21.824l-288.682-41.984a12.8 12.8 0 0 1-9.643-6.997z"/></svg>'
};
const Star = {
  name: "star",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path fill-rule="evenodd" d="M664.88 317.392 552.158 88.983a44.801 44.801 0 0 0-20.346-20.347c-22.188-10.95-49.051-1.84-60.002 20.347L359.024 317.392l-252.048 36.66a44.8 44.8 0 0 0-25.639 13.063l-4.652 5.613c-12.372 17.666-10.484 42.195 5.462 57.739l182.35 177.741L221.5 859.354a44.804 44.804 0 0 0 4.502 28.422l3.51 5.632c12.885 17.632 37.081 23.642 56.991 13.178l225.48-118.605 225.481 118.605a44.828 44.828 0 0 0 28.423 4.499l5.92-1.434c21.05-6.617 34.496-27.942 30.662-50.297L759.41 608.208l182.413-177.741a44.8 44.8 0 0 0 13.062-25.64l.47-6.074c.209-22.066-15.92-41.44-38.364-44.702L664.88 317.392Zm-510.2 94.403 223.59-32.489a44.8 44.8 0 0 0 33.732-24.507l99.99-202.588 99.99 202.588 3.158 5.41a44.805 44.805 0 0 0 30.574 19.097l223.527 32.49-161.73 157.703-4.169 4.674a44.802 44.802 0 0 0-8.715 34.98l38.166 222.61L532.84 726.701l-6.713-2.861a44.777 44.777 0 0 0-34.981 2.86L291.128 831.764l38.23-222.61.624-6.233a44.801 44.801 0 0 0-13.509-33.421L154.68 411.795Z"/></svg>'
};
const Stop = {
  name: "stop",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path d="M512 64a448 448 0 1 1 0 896 448 448 0 0 1 0-896zM128 512a384 384 0 0 0 630.976 294.08L217.92 264.96A382.464 382.464 0 0 0 128 512zm384-384a382.464 382.464 0 0 0-248.96 91.648L804.352 760.96A384 384 0 0 0 512 128z"/></svg>'
};
const Subscrib = {
  name: "subscrib",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path fill-rule="evenodd" d="M128 76.8v852.18c0 3.404 1.494 6.668 4.152 9.074 5.513 4.986 14.428 4.96 19.912-.05l317.953-290.66 3.716-2.442c5.281-2.386 11.864-1.569 16.251 2.441l317.952 290.66c2.643 2.413 6.234 3.776 9.984 3.776 7.776 0 14.08-5.734 14.08-12.8V76.8c0-7.07-6.304-12.8-14.08-12.8H142.08C134.304 64 128 69.73 128 76.8Zm413.263 525.414-6.997-5.616c-33.852-24.343-83.624-22.588-115.389 5.49L192 804.8V128h576v676.864l-226.737-202.65ZM460.8 192h38.4c7.07 0 12.8 5.73 12.8 12.8V320h115.2c7.07 0 12.8 5.73 12.8 12.8v38.4c0 7.07-5.73 12.8-12.8 12.8H512v115.2c0 7.07-5.73 12.8-12.8 12.8h-38.4c-7.07 0-12.8-5.73-12.8-12.8V384H332.8c-7.07 0-12.8-5.73-12.8-12.8v-38.4c0-7.07 5.73-12.8 12.8-12.8H448V204.8c0-7.07 5.73-12.8 12.8-12.8Z"/></svg>'
};
const Success = {
  name: "success",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path d="M264.384 443.52a25.6 25.6 0 0 1 36.224 0L463.552 606.4 762.24 307.712a25.6 25.6 0 0 1 36.16 0l54.336 54.272a25.6 25.6 0 0 1 0 36.224L481.6 769.28a25.6 25.6 0 0 1-36.16 0L210.112 534.016a25.6 25.6 0 0 1 0-36.224l54.272-54.336z"/></svg>'
};
const SwapVertical = {
  name: "swap-vertical",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path fill-rule="evenodd" d="M371.233 64c7.045 0 12.756 5.73 12.756 12.8v854.447c0 7.069-5.71 12.8-12.756 12.8a12.732 12.732 0 0 1-8.537-3.289L196.21 790.311c-5.235-4.73-5.657-12.824-.942-18.077a12.737 12.737 0 0 1 9.479-4.234h115.67V76.8c0-7.07 5.712-12.8 12.757-12.8h38.06Zm281.513 0c3.153 0 6.194 1.172 8.537 3.289l166.486 150.446c5.236 4.731 5.658 12.825.943 18.078a12.737 12.737 0 0 1-9.48 4.234h-115.67v691.2c0 7.069-5.711 12.8-12.757 12.8h-38.06c-7.044 0-12.756-5.731-12.756-12.8V76.8c0-7.07 5.712-12.8 12.757-12.8Z"/></svg>'
};
const Swap = {
  name: "swap",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path fill-rule="evenodd" d="M947.2 384H92.755c-11.085 0-16.928-13.084-9.51-21.29L233.69 196.245c7.846-8.685 22.31-3.152 22.31 8.535v115.655h691.2c7.07 0 12.8 5.712 12.8 12.756v38.054c0 7.044-5.73 12.755-12.8 12.755ZM76.8 640.031h854.447c11.084 0 16.929 13.084 9.511 21.29L790.312 827.787c-7.85 8.688-22.312 3.152-22.312-8.535V703.596H76.8c-7.072 0-12.8-5.711-12.8-12.756v-38.054c0-7.044 5.728-12.755 12.8-12.755Z"/></svg>'
};
const Sync = {
  name: "sync",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path d="M145.344 254.528 301.696 410.88a12.8 12.8 0 0 1-12.608 10.624h-150.4a384 384 0 0 0 588.48 408.576l46.016 45.888A448 448 0 0 1 73.152 421.504h-1.984l3.52-7.168a445.76 445.76 0 0 1 70.592-159.808zM512 64a448 448 0 0 1 353.472 723.328L712 633.792l-.64-.896a12.8 12.8 0 0 1 11.52-18.496h159.36a384 384 0 0 0-603.52-407.424l-45.568-45.632A446.08 446.08 0 0 1 512 64z"/></svg>'
};
const Tag = {
  name: "tag",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path fill-rule="evenodd" d="M67.656 615.21 411.7 959.25c4.998 4.998 13.082 4.812 18.081-.186L959.22 429.628c2.835-2.84 4.154-6.873 3.597-10.85l-41.959-299.943c-.787-5.636-5.12-9.934-10.752-10.754L608.173 64.138c-3.999-.582-8.034.697-10.892 3.554L67.844 597.129c-4.999 4.999-5.186 13.083-.188 18.081Zm352.998 262.44L149.215 606.212l475.086-475.085 238.54 34.71 33.082 236.547-475.27 475.266Zm351.813-670.666c-26.509 0-48 21.49-48 48s21.491 48 48 48 48-21.49 48-48-21.491-48-48-48Z"/></svg>'
};
const Tags = {
  name: "tags",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><g fill-rule="evenodd"><path d="m493.947 89.684.364.048 301.932 43.944c5.528.804 9.804 4.96 10.705 10.441l.047.312 10.983 78.517 104.755 15.246c5.528.804 9.804 4.96 10.705 10.442l.047.311 41.952 299.943c.54 3.857-.762 7.854-3.432 10.681l-.254.262-378.729 378.725c-4.918 4.924-12.654 5.016-17.663.257l-.244-.238-106.555-106.56-64.363 64.365c-4.92 4.92-12.858 5.214-17.874.458l-.24-.234L42.04 552.56c-4.92-4.92-4.784-12.859-.014-17.874l.234-.24L483.334 93.372c2.77-2.77 6.73-4.138 10.613-3.688Zm333.26 199.242 21.747 155.447c.557 3.976-.845 8.096-3.687 10.935L513.815 786.76l70.238 70.24 324.504-324.504-33.082-236.548-48.268-7.023ZM510.444 156.727 123.578 543.594 395.016 815.03 782.06 427.985l-33.082-236.548-238.535-34.71Z"/><path d="M694.611 247.587c-18.238-18.237-49.138-18.237-67.884.508-18.236 18.237-18.236 49.138.51 67.883 18.236 18.236 49.136 18.236 67.88-.51 18.24-18.235 18.24-49.136-.506-67.881Z"/></g></svg>'
};
const Team = {
  name: "team",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><g fill-rule="evenodd"><path d="M800 256a191.992 191.992 0 0 1-76.623 153.467c123.497 45.538 211.56 164.279 211.56 303.589 0 1.626-.019 3.38-.07 5.274l-.563 13.004c-1.357 23.706-20.973 42.221-44.755 42.24l-44.762-.038c-7.065-.006-12.787-5.734-12.787-12.8v-38.394c0-7.078 5.74-12.806 12.813-12.8l26.035.026-.224-9.462C868.397 654.8 854.547 612.55 832 576.274V576h-.173c-28.243-45.318-70.08-81.29-119.806-102.212-4.892-2.058-8.021-6.724-8.021-12.031V345.502c0-3.118 1.143-6.094 3.116-8.508A128 128 0 0 0 736 256c0-70.692-57.308-128-128-128-43.244 0-83.38 21.816-106.952 57.676-2.512 3.822-6.716 6.324-11.29 6.324h-44.675c-8.835 0-14.965-8.806-11.249-16.82C465.032 107.882 532.736 64 608 64c106.038 0 192 85.962 192 192Z"/><path d="M416 256c106.038 0 192 85.962 192 192a192 192 0 0 1-76.623 153.467c123.497 45.538 211.56 164.28 211.56 303.589 0 1.626-.019 3.38-.07 5.274l-.563 13.004c-1.357 23.706-20.972 42.221-44.756 42.24l-556.259-.499c-23.77-.025-43.39-18.605-44.69-42.31a502.627 502.627 0 0 1-.331-6.701L96 905.056c0-137.574 85.887-255.092 206.966-301.855A192.001 192.001 0 0 1 224 448c0-106.038 85.962-192 192-192Zm3.47 389.586c-143.302 0-259.47 116.168-259.47 259.47l.064-4 518.784.512-.227-9.466C672.084 759.11 565.414 652.44 432.42 645.903ZM416 320c-70.692 0-128 57.308-128 128 0 70.692 57.308 128 128 128 70.692 0 128-57.308 128-128 0-70.692-57.308-128-128-128Z"/></g></svg>'
};
const Thumbnail = {
  name: "thumbnail",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path fill-rule="evenodd" d="M883.2 128c5.893 0 10.853 3.98 12.343 9.397L896 140.8v742.4c0 5.893-3.978 10.853-9.396 12.343L883.2 896H140.8c-5.891 0-10.853-3.978-12.343-9.396L128 883.2V140.8c0-5.891 3.98-10.853 9.397-12.343L140.8 128h742.4ZM832 192H704v307.2c0 7.07-5.73 12.8-12.8 12.8H192v320h640V192Zm-192 0H192v256h448V192Z"/></svg>'
};
const Thunderbolt = {
  name: "thunderbolt",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path fill-rule="evenodd" d="m302.227 958.464 553.267-552.815c8.07-8.062 2.362-21.853-9.043-21.855l-253.862-.047L659.002 79.53c1.743-7.981-4.336-15.53-12.505-15.53h-349.65a12.8 12.8 0 0 0-12.495 10.023L176.31 560.216c-1.777 7.995 4.309 15.579 12.5 15.576l143.17-.045-51.48 371.907c-1.682 12.154 13.047 19.482 21.727 10.81Zm62.523-152.96 38.712-279.155c1.067-7.693-4.907-14.556-12.674-14.558l-138.23-.046 85.312-383.808h245.056l-66.352 304.33c-1.74 7.981 4.34 15.529 12.51 15.527l193.682-.049L364.75 805.504Z"/></svg>'
};
const Tool = {
  name: "tool",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path d="M925.68 214.657c2.535 4.344 4.662 7.92 6.2 10.816a320.128 320.128 0 0 1-429.83 435.532L229.163 933.892a12.8 12.8 0 0 1-18.102 0L93.4 816.229a12.8 12.8 0 0 1 0-18.102l272.886-272.886a320.128 320.128 0 0 1 446.303-423.586 12.8 12.8 0 0 1 2.444 20.003L568.53 368.161c-4.978 4.978-4.978 13.124 0 18.102l72.408 72.408a12.8 12.8 0 0 0 18.102 0l237.54-237.362 9.051-9.05a12.8 12.8 0 0 1 20.048 2.307zm-26.79 102.095-1.494-5.748-247.363 247.182-181.02-181.02 247.273-247.272-5.702-1.538a256.32 256.32 0 0 0-232.157 58.922l-9.458 8.915a256.128 256.128 0 0 0-51.817 287.594l5.792 11.948 21.723 41.634-269.855 269.674 45.3 45.3 269.855-269.764L531.6 604.3a256.128 256.128 0 0 0 367.2-287.55z"/></svg>'
};
const Transmit = {
  name: "transmit",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path d="M922.624 80.256a44.608 44.608 0 0 1 57.408 49.344l-1.536 6.656-262.4 830.976a33.152 33.152 0 0 1-51.456 16.512l-3.648-3.072-252.16-252.16 45.056-45.056 216.576 216.576 236.288-748.16-750.208 234.176L322.56 552.064l-45.056 45.12L75.712 395.392a33.152 33.152 0 0 1 9.856-53.568l3.712-1.536L922.624 80.256z"/><path d="M961.088 102.208a35.072 35.072 0 0 0-49.024-7.68l-644.16 468.8v259.136a38.272 38.272 0 0 0 66.88 25.408l624.128-701.824a35.072 35.072 0 0 0 2.176-43.904zM734.4 302.592 331.648 755.52V595.776L734.4 302.592z"/></svg>'
};
const TreeExpand = {
  name: "tree-expand",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path d="M179.2 192c7.04 0 12.8 5.76 12.8 12.8V256h51.2c7.04 0 12.8 5.76 12.8 12.8v38.4a12.8 12.8 0 0 1-12.8 12.8H192v192h179.2c7.04 0 12.8 5.76 12.8 12.8v38.4a12.8 12.8 0 0 1-12.8 12.8H320v192h115.2c7.04 0 12.8 5.76 12.8 12.8v38.4a12.8 12.8 0 0 1-12.8 12.8H268.8a12.8 12.8 0 0 1-12.8-12.8V576H140.8a12.8 12.8 0 0 1-12.8-12.8V204.8c0-7.04 5.76-12.8 12.8-12.8h38.4zm704 576c7.04 0 12.8 5.76 12.8 12.8v38.4a12.8 12.8 0 0 1-12.8 12.8H524.8a12.8 12.8 0 0 1-12.8-12.8v-38.4c0-7.04 5.76-12.8 12.8-12.8h358.4zm-64-256c7.04 0 12.8 5.76 12.8 12.8v38.4a12.8 12.8 0 0 1-12.8 12.8H460.8a12.8 12.8 0 0 1-12.8-12.8v-38.4c0-7.04 5.76-12.8 12.8-12.8h358.4zm-64-256c7.04 0 12.8 5.76 12.8 12.8v38.4a12.8 12.8 0 0 1-12.8 12.8H332.8a12.8 12.8 0 0 1-12.8-12.8v-38.4c0-7.04 5.76-12.8 12.8-12.8h422.4z"/></svg>'
};
const TreeUnexpand = {
  name: "tree-unexpand",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path d="M179.2 192c7.04 0 12.8 5.76 12.8 12.8V256h115.2c7.04 0 12.8 5.76 12.8 12.8v38.4a12.8 12.8 0 0 1-12.8 12.8H192v192h115.2c7.04 0 12.8 5.76 12.8 12.8v38.4a12.8 12.8 0 0 1-12.8 12.8H192v192h115.2c7.04 0 12.8 5.76 12.8 12.8v38.4a12.8 12.8 0 0 1-12.8 12.8H140.8a12.8 12.8 0 0 1-12.8-12.8V204.8c0-7.04 5.76-12.8 12.8-12.8h38.4zm704 576c7.04 0 12.8 5.76 12.8 12.8v38.4a12.8 12.8 0 0 1-12.8 12.8H396.8a12.8 12.8 0 0 1-12.8-12.8v-38.4c0-7.04 5.76-12.8 12.8-12.8h486.4zm0-256c7.04 0 12.8 5.76 12.8 12.8v38.4a12.8 12.8 0 0 1-12.8 12.8H396.8a12.8 12.8 0 0 1-12.8-12.8v-38.4c0-7.04 5.76-12.8 12.8-12.8h486.4zm0-256c7.04 0 12.8 5.76 12.8 12.8v38.4a12.8 12.8 0 0 1-12.8 12.8H396.8a12.8 12.8 0 0 1-12.8-12.8v-38.4c0-7.04 5.76-12.8 12.8-12.8h486.4z"/></svg>'
};
const Uncollapse = {
  name: "uncollapse",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path d="M204.8 128a12.8 12.8 0 0 0-12.8 12.8v742.4c0 7.04 5.76 12.8 12.8 12.8h38.4a12.8 12.8 0 0 0 12.8-12.8V140.8a12.8 12.8 0 0 0-12.8-12.8h-38.4zm266.176 76.224-27.2 27.2a12.8 12.8 0 0 0 0 18.112l262.528 262.4-262.528 262.528a12.8 12.8 0 0 0 0 18.112l27.2 27.136a12.8 12.8 0 0 0 18.048 0l298.688-298.688a12.8 12.8 0 0 0 0-18.112L489.024 204.224a12.8 12.8 0 0 0-18.048 0z"/></svg>'
};
const Unlock = {
  name: "unlock",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path fill-rule="evenodd" d="M763.34 241.542C740.53 140.39 636.674 64 512 64c-141.385 0-256 98.241-256 219.428V384H140.8c-7.07 0-12.8 5.73-12.8 12.8v550.4c0 7.072 5.73 12.8 12.8 12.8h742.4c7.072 0 12.8-5.728 12.8-12.8V396.8c0-7.07-5.728-12.8-12.8-12.8H320V281.6l.326-9.025C326.166 191.945 409.748 128 512 128l11.281.26c85.737 3.975 156.38 52.954 175.568 117.848 1.717 5.808 6.834 9.892 12.89 9.892h39.483c7.803 0 13.835-6.847 12.119-14.458ZM192 448h640v448H192V448Zm448 185.6v-51.2c0-3.535-5.73-6.4-12.8-6.4H396.8c-7.07 0-12.8 2.865-12.8 6.4v51.2c0 3.535 5.73 6.4 12.8 6.4h230.4c7.07 0 12.8-2.865 12.8-6.4Z"/></svg>'
};
const UpCircle = {
  name: "up-circle",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><g fill-rule="evenodd"><path d="M512 64c247.424 0 448 200.577 448 448 0 247.424-200.576 448-448 448-247.423 0-448-200.576-448-448C64 264.577 264.577 64 512 64Zm0 64c-212.077 0-384 171.923-384 384s171.923 384 384 384 384-171.923 384-384-171.923-384-384-384Z"/><path d="m520.99 328.025 264.45 264.452c4.998 4.998 5.056 12.921.058 17.92l-27.392 27.39c-4.999 4.999-13.082 4.784-18.08-.213L511.92 409.556 287.565 633.98c-5 5-12.933 5.068-17.932.07l-27.39-27.391c-4.999-4.999-4.76-13.104.238-18.103l260.35-260.349c4.997-4.999 13.16-5.181 18.158-.182Z"/></g></svg>'
};
const Up = {
  name: "up",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path d="M503.234 313.575 249.575 567.234c-4.998 4.998-5.001 12.869-.003 17.867l27.39 27.39c5 5 12.927 4.939 17.926-.06L512.169 395.15 729.45 612.43c4.999 5 12.927 5.06 17.925.062l27.39-27.39c5-5 4.996-12.87-.002-17.868L521.104 313.575c-4.998-4.998-12.872-4.998-17.87 0Z"/></svg>'
};
const Upload = {
  name: "upload",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path d="M128 780.8V896h768V780.8c0-7.04 5.76-12.8 12.8-12.8h38.4c7.04 0 12.8 5.76 12.8 12.8v166.4a12.8 12.8 0 0 1-12.8 12.8H76.8A12.8 12.8 0 0 1 64 947.2V780.8c0-7.04 5.76-12.8 12.8-12.8h38.4c7.04 0 12.8 5.76 12.8 12.8z"/><path d="M396.8 826.688a12.8 12.8 0 0 1-12.8-12.8v-371.2H222.912a12.8 12.8 0 0 1-9.088-21.824L503.04 131.776a12.8 12.8 0 0 1 18.048 0L810.24 420.864a12.8 12.8 0 0 1-9.088 21.824L640 442.752v371.2a12.8 12.8 0 0 1-12.8 12.8H396.8zm51.264-63.936h128v-384l101.568.064L512 213.184 346.496 378.752H448l.064 384z"/></svg>'
};
const UserAdd = {
  name: "user-add",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><g fill-rule="evenodd"><path d="m348.552 453.89-29.673 25.057c-5.4 4.561-13.582 3.801-17.876-1.814-35.05-45.826-54.464-102.01-54.464-161.111C246.539 169.412 365.39 50.56 512 50.56c142.168 0 258.233 111.757 265.142 252.212v.136c.175 7.19-5.604 13.114-12.797 13.114h-38.084c-7.068 0-12.755-5.678-13.195-12.733C706.497 197.957 618.988 114.56 512 114.56c-111.264 0-201.462 90.198-201.462 201.462 0 43.425 13.753 84.66 38.724 118.787l1.092 1.388c4.263 5.42 3.466 13.244-1.803 17.694ZM672 384.021c159.06 0 288 128.942 288 288 0 159.057-128.94 287.998-288 287.998-159.058 0-288-128.94-288-287.998s128.942-288 288-288Zm0 64c-123.712 0-224 100.288-224 224 0 123.71 100.288 223.998 224 223.998s224-100.288 224-223.998c0-123.712-100.288-224-224-224Z"/><path d="M640 704.02v115.2c0 7.071 5.73 12.8 12.8 12.8h38.4c7.07 0 12.8-5.729 12.8-12.8v-115.2h115.2c7.072 0 12.8-5.729 12.8-12.8V652.82c0-7.07-5.728-12.8-12.8-12.8H704v-115.2c0-7.07-5.73-12.8-12.8-12.8h-38.4c-7.07 0-12.8 5.73-12.8 12.8v115.2H524.8c-7.07 0-12.8 5.73-12.8 12.8v38.398c0 7.072 5.73 12.8 12.8 12.8H640ZM96.254 907.853 96 896.019c0-146.163 76.043-279.04 197.555-354.097l2.98-1.76c6.172-3.647 14.134-1.515 17.658 4.728l18.752 33.218c3.474 6.156 1.208 13.99-4.818 17.686-98.95 60.696-162.471 166.516-167.768 284.232L160 900.883l211.233.25c7.068.006 12.79 5.74 12.784 12.813l-.034 38.4c-.007 7.072-5.743 12.793-12.812 12.787l-229.484-.205c-23.692-.026-43.279-18.483-44.678-42.1l-.755-14.975Z"/></g></svg>'
};
const UserCircle = {
  name: "user-circle",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path d="M512 64a448 448 0 1 1 0 896 448 448 0 0 1 0-896zm0 64a384 384 0 1 0 0 768 384 384 0 0 0 0-768z"/><path d="M512 192a192 192 0 1 1 0 384 192 192 0 0 1 0-384zm0 64a128 128 0 1 0 0 256 128 128 0 0 0 0-256z"/><path d="M512 544c156.928 0 295.68 87.68 366.592 223.744l-18.816 25.216-29.312 16.896A352 352 0 0 0 512 608a351.936 351.936 0 0 0-309.952 184.96l-7.04 13.76-45.952-38.976c3.648-7.552-4.096-3.84 0-11.136A415.872 415.872 0 0 1 512 544z"/></svg>'
};
const UserDelete = {
  name: "user-delete",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><g fill-rule="evenodd"><path d="m348.552 453.89-29.673 25.057c-5.4 4.561-13.582 3.801-17.876-1.814-35.05-45.826-54.464-102.01-54.464-161.111C246.539 169.412 365.39 50.56 512 50.56c142.168 0 258.233 111.757 265.142 252.212v.136c.175 7.19-5.604 13.114-12.797 13.114h-38.084c-7.068 0-12.755-5.678-13.195-12.733C706.497 197.957 618.988 114.56 512 114.56c-111.264 0-201.462 90.198-201.462 201.462 0 43.425 13.753 84.66 38.724 118.787l1.092 1.388c4.263 5.42 3.466 13.244-1.803 17.694ZM672 384.021c159.06 0 288 128.942 288 288 0 159.057-128.94 287.998-288 287.998-159.058 0-288-128.94-288-287.998s128.942-288 288-288Zm0 64c-123.712 0-224 100.288-224 224 0 123.71 100.288 223.998 224 223.998s224-100.288 224-223.998c0-123.712-100.288-224-224-224ZM96.254 907.853 96 896.019c0-146.163 76.043-279.04 197.555-354.097l2.98-1.76c6.172-3.647 14.134-1.515 17.658 4.728l18.752 33.218c3.474 6.156 1.208 13.99-4.818 17.686-98.95 60.696-162.471 166.516-167.768 284.232L160 900.883l211.233.25c7.068.006 12.79 5.74 12.784 12.813l-.034 38.4c-.007 7.072-5.743 12.793-12.812 12.787l-229.484-.205c-23.692-.026-43.279-18.483-44.678-42.1l-.755-14.975Z"/><path d="M524.8 704.02h294.4c7.072 0 12.8-5.729 12.8-12.8V652.82c0-7.07-5.728-12.8-12.8-12.8H524.8c-7.07 0-12.8 5.73-12.8 12.8v38.398c0 7.072 5.73 12.8 12.8 12.8Z"/></g></svg>'
};
const UserDisabled = {
  name: "user-disabled",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><g fill-rule="evenodd"><path d="m318.88 478.947 29.672-25.056c5.27-4.45 6.066-12.274 1.803-17.694l-1.092-1.388c-24.971-34.127-38.724-75.362-38.724-118.787 0-111.264 90.198-201.462 201.462-201.462 106.987 0 194.497 83.397 201.066 188.729.44 7.055 6.127 12.733 13.195 12.733h38.084c7.193 0 12.972-5.924 12.797-13.114v-.136C770.233 162.317 654.17 50.56 512.001 50.56c-146.61 0-265.462 118.851-265.462 265.462 0 59.102 19.415 115.285 54.464 161.11 4.294 5.616 12.475 6.376 17.876 1.815ZM672 384.021c159.06 0 288 128.942 288 288 0 159.057-128.94 287.998-288 287.998-159.058 0-288-128.94-288-287.998s128.942-288 288-288Zm-192.238 173.02A224 224 0 0 0 448 672.02c0 123.71 100.288 223.998 224 223.998a223.993 223.993 0 0 0 113.938-31.142l1.044-.621-307.22-307.215ZM672 448.02a224 224 0 0 0-150.383 57.985l-1.245 1.136 316.505 316.506A223.997 223.997 0 0 0 896 672.021c0-123.712-100.288-224-224-224ZM96 896.02l.254 11.833.755 14.976c1.4 23.616 20.986 42.073 44.678 42.099l229.484.205c7.069.006 12.805-5.715 12.812-12.787l.034-38.4c.007-7.072-5.716-12.807-12.784-12.813L160 900.883l.359-20.857c5.297-117.716 68.819-223.536 167.768-284.232 6.026-3.696 8.292-11.53 4.818-17.686l-18.752-33.218c-3.524-6.243-11.486-8.375-17.657-4.729l-2.981 1.76C172.043 616.98 96 749.857 96 896.02Z"/></g></svg>'
};
const User = {
  name: "user",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path d="M512 64a256 256 0 1 1 0 512 256 256 0 0 1 0-512zm0 64a192 192 0 1 0 0 384 192 192 0 0 0 0-384z"/><path d="M512 512c-224.768 0-407.872 146.24-415.744 369.088l-.192 20.48.512 13.376.448 7.872a44.8 44.8 0 0 0 44.672 42.112l740.48.64a44.8 44.8 0 0 0 44.8-42.048l.768-15.36L928 896c0-229.76-186.24-384-416-384zm0 64c194.432 0 352 125.568 352 320l-.128 5.568L160 900.864V896c0-194.432 157.568-320 352-320z"/></svg>'
};
const Verify = {
  name: "verify",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path fill-rule="evenodd" d="M512 64c247.424 0 448 200.577 448 448 0 247.424-200.576 448-448 448-247.423 0-448-200.576-448-448C64 264.577 264.577 64 512 64Zm0 64c-212.077 0-384 171.923-384 384s171.923 384 384 384 384-171.923 384-384-171.923-384-384-384ZM377.541 320a12.8 12.8 0 0 1 10.748 5.85l1.4 2.923L510.044 711.25h4l124.032-382.516a12.796 12.796 0 0 1 8.929-8.327l3.208-.407h41.668c7.576 0 13.199 6.426 12.79 13.509l-.726 3.57-147.88 422.397a12.804 12.804 0 0 1-8.902 8.128l-3.162.396H480.02c-4.31 0-8.252-2.158-10.596-5.615l-1.44-2.823L320 337.164c-2.592-7.153 1.58-14.654 8.415-16.651l3.62-.513h45.506Z"/></svg>'
};
const VerticalAlignBottom = {
  name: "vertical-align-bottom",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path fill-rule="evenodd" d="M128 883.2v-38.4c0-1.697.325-3.33.974-4.898a12.678 12.678 0 0 1 2.775-4.153 12.68 12.68 0 0 1 4.153-2.775A12.679 12.679 0 0 1 140.8 832h678.4c1.698 0 3.33.325 4.896.974 1.57.65 2.955 1.575 4.154 2.775a12.624 12.624 0 0 1 2.777 4.153c.649 1.568.973 3.2.973 4.898v38.4c0 1.697-.324 3.33-.973 4.898a12.624 12.624 0 0 1-2.777 4.153 12.649 12.649 0 0 1-4.154 2.775c-1.566.65-3.198.974-4.896.974H140.8c-1.697 0-3.33-.325-4.898-.974a12.678 12.678 0 0 1-4.153-2.775 12.678 12.678 0 0 1-2.775-4.153A12.679 12.679 0 0 1 128 883.2Zm523.592-298.073L488.675 748.044c-4.999 4.999-13.104 4.999-18.102 0L307.655 585.127c-4.998-4.999-4.998-13.103 0-18.102l27.153-27.153c4.999-4.999 13.104-4.999 18.102 0l95.09 95.09V140.8c0-7.072 5.73-12.8 12.8-12.8h38.4c7.07 0 12.8 5.728 12.8 12.8v493.41l94.337-94.338c4.999-4.999 13.104-4.999 18.102 0l27.153 27.153c4.999 4.999 4.999 13.103 0 18.102Z"/></svg>'
};
const VerticalAlignCenter = {
  name: "vertical-align-center",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path d="M470.84 596.02c3.999-4 9.985-4.799 14.78-2.4l3.322 2.4 162.917 162.917c4 4 4.799 9.986 2.4 14.78l-2.4 3.322-27.153 27.153c-3.999 3.999-9.985 4.799-14.78 2.4l-3.322-2.4L512 709.568V947.2c0 7.07-5.73 12.8-12.8 12.8h-38.4c-7.07 0-12.8-5.73-12.8-12.8V709.312l-94.823 94.88c-3.999 3.999-9.986 4.799-14.78 2.4l-3.322-2.4-27.153-27.153c-3.999-3.999-4.799-9.986-2.4-14.78l2.4-3.322L470.84 596.02zM819.2 448c7.07 0 12.8 5.73 12.8 12.8v38.4c0 7.07-5.73 12.8-12.8 12.8H140.8c-7.07 0-12.8-5.73-12.8-12.8v-38.4c0-7.07 5.73-12.8 12.8-12.8h678.4zM499.2 0c5.891 0 10.853 3.98 12.343 9.397L512 12.8v237.504l94.59-94.53c4.998-5 13.103-5 18.101 0l27.153 27.152c4.999 4.999 4.999 13.104 0 18.102L488.927 363.945c-4.999 5-13.103 5-18.102 0L307.908 201.028c-5-4.999-5-13.103 0-18.102l27.152-27.153c5-4.999 13.104-4.999 18.102 0L448 250.56V12.8c0-5.891 3.98-10.853 9.397-12.343L460.8 0h38.4z"/></svg>'
};
const VerticalAlignTop = {
  name: "vertical-align-top",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path d="m488.96 276.032 162.88 162.944a12.8 12.8 0 0 1 0 18.048l-27.136 27.2a12.8 12.8 0 0 1-18.112 0L512 389.568V883.2a12.8 12.8 0 0 1-12.8 12.8h-38.4a12.8 12.8 0 0 1-12.8-12.8V389.312l-94.784 94.912a12.8 12.8 0 0 1-18.112 0L307.84 456.96a12.8 12.8 0 0 1 0-18.048l162.944-162.944a12.8 12.8 0 0 1 18.112 0zM819.2 128c7.04 0 12.8 5.76 12.8 12.8v38.4a12.8 12.8 0 0 1-12.8 12.8H140.8a12.8 12.8 0 0 1-12.8-12.8v-38.4c0-7.04 5.76-12.8 12.8-12.8h678.4z"/></svg>'
};
const ViewCard = {
  name: "view-card",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path d="M480 140.8v326.4a12.8 12.8 0 0 1-12.8 12.8H140.8a12.8 12.8 0 0 1-12.8-12.8V140.8c0-7.04 5.76-12.8 12.8-12.8h326.4c7.04 0 12.8 5.76 12.8 12.8zM416 192H192v224h224V192zm64 364.8v326.4a12.8 12.8 0 0 1-12.8 12.8H140.8a12.8 12.8 0 0 1-12.8-12.8V556.8c0-7.04 5.76-12.8 12.8-12.8h326.4c7.04 0 12.8 5.76 12.8 12.8zM416 608H192v224h224V608zm480-467.2v326.4a12.8 12.8 0 0 1-12.8 12.8H556.8a12.8 12.8 0 0 1-12.8-12.8V140.8c0-7.04 5.76-12.8 12.8-12.8h326.4c7.04 0 12.8 5.76 12.8 12.8zM832 192H608v224h224V192zm64 364.8v326.4a12.8 12.8 0 0 1-12.8 12.8H556.8a12.8 12.8 0 0 1-12.8-12.8V556.8c0-7.04 5.76-12.8 12.8-12.8h326.4c7.04 0 12.8 5.76 12.8 12.8zM832 608H608v224h224V608z"/></svg>'
};
const ViewList = {
  name: "view-list",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path d="M320 140.8v166.4a12.8 12.8 0 0 1-12.8 12.8H140.8a12.8 12.8 0 0 1-12.8-12.8V140.8c0-7.04 5.76-12.8 12.8-12.8h166.4c7.04 0 12.8 5.76 12.8 12.8zM243.2 192h-38.4a12.8 12.8 0 0 0-12.8 12.8v38.4c0 7.04 5.76 12.8 12.8 12.8h38.4a12.8 12.8 0 0 0 12.8-12.8v-38.4a12.8 12.8 0 0 0-12.8-12.8zM320 716.8v166.4a12.8 12.8 0 0 1-12.8 12.8H140.8a12.8 12.8 0 0 1-12.8-12.8V716.8c0-7.04 5.76-12.8 12.8-12.8h166.4c7.04 0 12.8 5.76 12.8 12.8zM243.2 768h-38.4a12.8 12.8 0 0 0-12.8 12.8v38.4c0 7.04 5.76 12.8 12.8 12.8h38.4a12.8 12.8 0 0 0 12.8-12.8v-38.4a12.8 12.8 0 0 0-12.8-12.8zM320 428.8v166.4a12.8 12.8 0 0 1-12.8 12.8H140.8a12.8 12.8 0 0 1-12.8-12.8V428.8c0-7.04 5.76-12.8 12.8-12.8h166.4c7.04 0 12.8 5.76 12.8 12.8zM243.2 480h-38.4a12.8 12.8 0 0 0-12.8 12.8v38.4c0 7.04 5.76 12.8 12.8 12.8h38.4a12.8 12.8 0 0 0 12.8-12.8v-38.4a12.8 12.8 0 0 0-12.8-12.8zm153.6-352h486.4q12.8 0 12.8 12.8v38.4q0 12.8-12.8 12.8H396.8q-12.8 0-12.8-12.8v-38.4q0-12.8 12.8-12.8Zm0 576h486.4q12.8 0 12.8 12.8v38.4q0 12.8-12.8 12.8H396.8q-12.8 0-12.8-12.8v-38.4q0-12.8 12.8-12.8Zm0-288h486.4q12.8 0 12.8 12.8v38.4q0 12.8-12.8 12.8H396.8q-12.8 0-12.8-12.8v-38.4q0-12.8 12.8-12.8Zm0 128h486.4q12.8 0 12.8 12.8v38.4q0 12.8-12.8 12.8H396.8q-12.8 0-12.8-12.8v-38.4q0-12.8 12.8-12.8Zm0-288h486.4q12.8 0 12.8 12.8v38.4q0 12.8-12.8 12.8H396.8q-12.8 0-12.8-12.8v-38.4q0-12.8 12.8-12.8Zm0 576h486.4q12.8 0 12.8 12.8v38.4q0 12.8-12.8 12.8H396.8q-12.8 0-12.8-12.8v-38.4q0-12.8 12.8-12.8Z"/></svg>'
};
const ViewThumbnail = {
  name: "view-thumbnail",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path fill-rule="evenodd" d="M320 716.8v166.4c0 7.07-5.73 12.8-12.8 12.8H140.8c-7.07 0-12.8-5.73-12.8-12.8V716.8c0-7.07 5.73-12.8 12.8-12.8h166.4c7.07 0 12.8 5.73 12.8 12.8Zm288 0v166.4c0 7.07-5.73 12.8-12.8 12.8H428.8c-7.07 0-12.8-5.73-12.8-12.8V716.8c0-7.07 5.73-12.8 12.8-12.8h166.4c7.07 0 12.8 5.73 12.8 12.8Zm288 0v166.4c0 7.07-5.73 12.8-12.8 12.8H716.8c-7.07 0-12.8-5.73-12.8-12.8V716.8c0-7.07 5.73-12.8 12.8-12.8h166.4c7.07 0 12.8 5.73 12.8 12.8ZM256 768h-64v64h64v-64Zm288 0h-64v64h64v-64Zm288 0h-64v64h64v-64ZM320 428.8v166.4c0 7.07-5.73 12.8-12.8 12.8H140.8c-7.07 0-12.8-5.73-12.8-12.8V428.8c0-7.07 5.73-12.8 12.8-12.8h166.4c7.07 0 12.8 5.73 12.8 12.8Zm288 0v166.4c0 7.07-5.73 12.8-12.8 12.8H428.8c-7.07 0-12.8-5.73-12.8-12.8V428.8c0-7.07 5.73-12.8 12.8-12.8h166.4c7.07 0 12.8 5.73 12.8 12.8Zm288 0v166.4c0 7.07-5.73 12.8-12.8 12.8H716.8c-7.07 0-12.8-5.73-12.8-12.8V428.8c0-7.07 5.73-12.8 12.8-12.8h166.4c7.07 0 12.8 5.73 12.8 12.8ZM256 480h-64v64h64v-64Zm288 0h-64v64h64v-64Zm288 0h-64v64h64v-64ZM320 140.8v166.4c0 7.07-5.73 12.8-12.8 12.8H140.8c-7.07 0-12.8-5.73-12.8-12.8V140.8c0-7.07 5.73-12.8 12.8-12.8h166.4c7.07 0 12.8 5.73 12.8 12.8Zm288 0v166.4c0 7.07-5.73 12.8-12.8 12.8H428.8c-7.07 0-12.8-5.73-12.8-12.8V140.8c0-7.07 5.73-12.8 12.8-12.8h166.4c7.07 0 12.8 5.73 12.8 12.8Zm288 0v166.4c0 7.07-5.73 12.8-12.8 12.8H716.8c-7.07 0-12.8-5.73-12.8-12.8V140.8c0-7.07 5.73-12.8 12.8-12.8h166.4c7.07 0 12.8 5.73 12.8 12.8ZM256 192h-64v64h64v-64Zm288 0h-64v64h64v-64Zm288 0h-64v64h64v-64Z"/></svg>'
};
const Wait = {
  name: "wait",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path d="M492.8 64h38.4Q544 64 544 76.8v166.4q0 12.8-12.8 12.8h-38.4q-12.8 0-12.8-12.8V76.8Q480 64 492.8 64Zm226.572 50.43 33.256 19.2q11.085 6.4 4.685 17.485l-83.2 144.106q-6.4 11.085-17.485 4.685l-33.256-19.2q-11.085-6.4-4.685-17.485l83.2-144.106q6.4-11.085 17.485-4.685ZM890.37 271.372l19.2 33.256q6.4 11.085-4.685 17.485l-144.106 83.2q-11.085 6.4-17.485-4.685l-19.2-33.256q-6.4-11.085 4.685-17.485l144.106-83.2q11.085-6.4 17.485 4.685ZM960 492.8v38.4q0 12.8-12.8 12.8H780.8q-12.8 0-12.8-12.8v-38.4q0-12.8 12.8-12.8h166.4q12.8 0 12.8 12.8Zm-50.43 226.572-19.2 33.256q-6.4 11.085-17.485 4.685l-144.106-83.2q-11.085-6.4-4.685-17.485l19.2-33.256q6.4-11.085 17.485-4.685l144.106 83.2q11.085 6.4 4.685 17.485ZM752.628 890.37l-33.256 19.2q-11.085 6.4-17.485-4.685l-83.2-144.106q-6.4-11.085 4.685-17.485l33.256-19.2q11.085-6.4 17.485 4.685l83.2 144.106q6.4 11.085-4.685 17.485ZM531.2 960h-38.4q-12.8 0-12.8-12.8V780.8q0-12.8 12.8-12.8h38.4q12.8 0 12.8 12.8v166.4q0 12.8-12.8 12.8Zm-226.572-50.43-33.256-19.2q-11.085-6.4-4.685-17.485l83.2-144.106q6.4-11.085 17.485-4.685l33.256 19.2q11.085 6.4 4.685 17.485l-83.2 144.106q-6.4 11.085-17.485 4.685ZM133.63 752.628l-19.2-33.256q-6.4-11.085 4.685-17.485l144.106-83.2q11.085-6.4 17.485 4.685l19.2 33.256q6.4 11.085-4.685 17.485l-144.106 83.2q-11.085 6.4-17.485-4.685ZM64 531.2v-38.4Q64 480 76.8 480h166.4q12.8 0 12.8 12.8v38.4q0 12.8-12.8 12.8H76.8Q64 544 64 531.2Zm50.43-226.572 19.2-33.256q6.4-11.085 17.485-4.685l144.106 83.2q11.085 6.4 4.685 17.485l-19.2 33.256q-6.4 11.085-17.485 4.685l-144.106-83.2q-11.085-6.4-4.685-17.485ZM271.372 133.63l33.256-19.2q11.085-6.4 17.485 4.685l83.2 144.106q6.4 11.085-4.685 17.485l-33.256 19.2q-11.085 6.4-17.485-4.685l-83.2-144.106q-6.4-11.085 4.685-17.485Z"/></svg>'
};
const Warning = {
  name: "warning",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path d="M492.8 416h38.4c7.04 0 12.8 5.76 12.8 12.8v262.4a12.8 12.8 0 0 1-12.8 12.8h-38.4a12.8 12.8 0 0 1-12.8-12.8V428.8c0-7.04 5.76-12.8 12.8-12.8zm20.032 319.04q48 0 48 48t-48 48q-48 0-48-48t48-48Z"/><path d="M953.408 944 524.096 145.344a12.8 12.8 0 0 0-22.528 0l-428.16 798.72a12.8 12.8 0 0 0 11.328 18.816H942.08A12.8 12.8 0 0 0 953.408 944zM512.832 259.648 856.448 898.88H170.176l342.656-639.232z"/></svg>'
};
const Whitelist = {
  name: "whitelist",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path fill-rule="evenodd" d="M384 76.8v166.47c0 7.07 5.73 12.8 12.8 12.8h232.868c7.07 0 12.8-5.73 12.8-12.8V76.8c0-7.07-5.73-12.8-12.8-12.8H396.8c-7.07 0-12.8 5.73-12.8 12.8Zm-76.8-12.73H140.8c-7.07 0-12.8 5.731-12.8 12.8v870.4c0 7.072 5.73 12.8 12.8 12.8h742.4c7.072 0 12.8-5.728 12.8-12.8V76.87c0-7.069-5.728-12.8-12.8-12.8H716.8c-7.07 0-12.8 5.731-12.8 12.8v38.4c0 7.07 5.73 12.8 12.8 12.8H832v768H192v-768h115.2c7.07 0 12.8-5.73 12.8-12.8v-38.4c0-7.069-5.73-12.8-12.8-12.8Zm271.232 127.94H448v-64.065h130.432v64.065Zm181.994 178.72 27.152 27.154c4.998 5 4.998 13.104 0 18.102L490.354 713.21c-5 5.004-13.103 5.004-18.102 0L261.623 502.584c-5.063-5.063-4.988-13.294.166-18.265l28.038-27.039c5.024-4.845 13.002-4.772 17.937.163l173.89 173.889 260.671-260.603c5-4.998 13.103-4.997 18.1.001Z"/></svg>'
};
const Wifi = {
  name: "wifi",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path fill-rule="evenodd" d="M905.952 348.559c-221.652-208.85-568.368-208.745-789.893.312l-3.134 3.044c-5.018 4.876-13.022 4.818-17.97-.13L67.74 324.57c-5.025-5.025-4.987-13.212.12-18.152a494.062 494.062 0 0 1 4.574-4.377c246.506-232.36 632.302-232.054 878.446.918l3.54 3.444c5.113 4.977 5.17 13.177.12 18.225l-27.09 27.095c-4.973 4.972-12.986 5.014-18.048.132a497.281 497.281 0 0 0-3.45-3.296Zm-655.529 137.5c146.31-135.909 372.792-136.636 519.958-2.18 1.037.945 2.248 2.076 3.635 3.393 5.094 4.822 13.088 4.788 18.048-.171l27.098-27.1c5.062-5.059 4.992-13.282-.154-18.254l-1.715-1.659C646.43 280.392 381.06 279.416 209.04 437.158a359.277 359.277 0 0 0-4.997 4.689c-5.154 4.916-5.13 13.061-.093 18.098l27.219 27.22c4.934 4.933 12.91 5.006 17.934.164l1.319-1.27Zm134.68 136.948c71.377-63.559 178.802-64.633 251.398-3.225.818.691 1.748 1.505 2.791 2.44 5.185 4.65 13.072 4.593 17.996-.332l27.355-27.354c4.954-4.955 4.855-13.019-.22-17.85-96.07-87.07-242.294-88.209-339.677-3.42a188.162 188.162 0 0 0-3.76 3.381c-5.266 4.846-5.196 13-.135 18.06l28.064 28.066c4.444 4.444 11.617 4.548 16.188.234ZM575.08 768.09a63.999 63.999 0 0 0-18.745-45.252c-24.994-24.994-65.516-24.994-90.51 0-24.993 24.993-24.993 65.514 0 90.506 24.994 24.998 65.516 24.998 90.51 0a63.987 63.987 0 0 0 18.745-45.254Z"/></svg>'
};
const ZoomIn = {
  name: "zoom-in",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path d="M448 96a384 384 0 0 1 307.392 614.208l154.24 154.112a12.8 12.8 0 0 1 0 18.112l-27.2 27.136a12.8 12.8 0 0 1-18.112 0L712.832 758.08A384 384 0 1 1 448 96zm0 64a320 320 0 1 0 0 640 320 320 0 0 0 0-640z"/><path d="M428.8 255.68h38.4c7.04 0 12.8 5.76 12.8 12.8v422.4a12.8 12.8 0 0 1-12.8 12.8h-38.4a12.8 12.8 0 0 1-12.8-12.8v-422.4c0-7.04 5.76-12.8 12.8-12.8z"/><path d="M672 460.48v38.4a12.8 12.8 0 0 1-12.8 12.8H236.8a12.8 12.8 0 0 1-12.8-12.8v-38.4c0-7.04 5.76-12.8 12.8-12.8h422.4c7.04 0 12.8 5.76 12.8 12.8z"/></svg>'
};
const ZoomOut = {
  name: "zoom-out",
  svg: '<svg viewBox="0 0 1024 1024" focusable="false"><path d="M448 96a384 384 0 0 1 307.392 614.208l154.24 154.112a12.8 12.8 0 0 1 0 18.112l-27.2 27.136a12.8 12.8 0 0 1-18.112 0L712.832 758.08A384 384 0 1 1 448 96zm0 64a320 320 0 1 0 0 640 320 320 0 0 0 0-640z"/><path d="M672 460.48v38.4a12.8 12.8 0 0 1-12.8 12.8H236.8a12.8 12.8 0 0 1-12.8-12.8v-38.4c0-7.04 5.76-12.8 12.8-12.8h422.4c7.04 0 12.8 5.76 12.8 12.8z"/></svg>'
};
const IDUX_ICON_DEPENDENCIES = [
  Bulb,
  CaretDownFilled,
  CaretUpFilled,
  Check,
  CheckCircle,
  CheckCircleFilled,
  CheckFilled,
  Clear,
  ClockCircle,
  Close,
  CloseCircle,
  CloseCircleFilled,
  CloseFilled,
  Collapse,
  CollapseAll,
  DoubleLeft,
  DoubleRight,
  Down,
  Ellipsis,
  Exclamation,
  ExclamationCircle,
  ExclamationCircleFilled,
  Expand,
  ExpandAll,
  FileImage,
  FilterFilled,
  GridCompact,
  GridLoose,
  GridMedium,
  Holder,
  InfoCircle,
  InfoCircleFilled,
  Left,
  Loading,
  MenuFold,
  MenuUnfold,
  Minus,
  Plus,
  QuestionCircle,
  QuestionCircleFilled,
  Right,
  RotateLeft,
  RotateRight,
  Search,
  StarFilled,
  User,
  VerticalAlignCenter,
  VerticalAlignTop,
  ZoomIn,
  ZoomOut
];
const IxIcon = Icon;
export { Alert, Apartment, ApplyAll, Appstore, AppstoreAdd, ArrowDown, ArrowLeft, ArrowRight, ArrowUp, Audit, BarChart, Bars, Blacklist, BookMark, Bookmark, Branch, Bug, BugFilled, Bulb, Calendar, Camera, Card, CaretDown, CaretDownFilled, CaretLeftFilled, CaretRightFilled, CaretUp, CaretUpFilled, Check, CheckCircle, CheckCircleFilled, CheckFilled, CheckingFilled, Clear, ClockCircle, Close, CloseCircle, CloseCircleFilled, CloseFilled, Cloud, CloudDownload, CloudServer, CloudSync, CloudUpload, Cluster, Code, Collapse, CollapseAll, Collect, Comment, Control, Controlz, Copy, Copyright, CreditCard, CustomerService, Customization, Dashboard, DataCenter, DataCenterStar, Database, Delete, DeliveredProcedure, Desktop, DesktopStar, DialogClose, Disconnect, Dislike, DislikeFilled, Dns, DotChart, DoubleLeft, DoubleRight, Down, DownCircle, Download, Edit, Ellipsis, EllipsisVertical, Empty, Environment, Exception, Exclamation, ExclamationCircle, ExclamationCircleFilled, Exit, Expand, ExpandAll, Export, Eye, EyeInvisible, Favourite, File, FileAdd, FileAlert, FileDisabled, FileDone, FileExport, FileGif, FileImage, FileProtect, FileSearch, FileSync, FileText, FileZip, Filter, FilterFilled, FitToView, Folder, FolderAdd, FolderOpen, ForScreen, Forward, Fullscreen, FullscreenExit, Gif, Github, Gitlab, Global, GridCompact, GridLoose, GridMedium, Heart, Heat, History, Holder, Home, IDUX_ICON_DEPENDENCIES, Import, Info, InfoCircle, InfoCircleFilled, Insurance, Ip, IxIcon, Jumpz, Key, Layout, LayoutCompact, LayoutLarge, LayoutLoose, LayoutMedium, Left, LeftCircle, LeftDouble, LeftFilled, Like, LikeFilled, LineChart, Link, List, Loading, Loading3Quarters, LoadingSplit, Location, Lock, Login, Logout, Loose, Mail, Menu, MenuFold, MenuUnfold, Minus, MinusCircle, MinusSquare, Mirror, Mobile, Monitor, More, Move, MoveFile, MoveFilez, MoveGroup, MoveList, Network, Normal, Notification, PaperClip, Paste, Pause, PauseCircle, Phone, PieChart, Pin, Play, PlayCircle, Plus, PlusCircle, PlusSquare, Poweroff, Projection, Qrcode, Question, QuestionCircle, QuestionCircleFilled, Radar, RadarChart, Redo, Reload, Right, RightCircle, RightDouble, RightFilled, RingChart, Robot, Rollback, RotateLeft, RotateRight, Safety, Scan, ScanSecurity, ScanVirus, Search, SecurityAlert, SecurityScan, Select, Send, Server, ServerStar, Setting, Shopping, SortAscending, SortDescending, Sound, Star, StarFilled, Stop, Subscrib, Success, Swap, SwapVertical, Sync, Tag, Tags, Team, Thumbnail, Thunderbolt, Tool, Transmit, TreeExpand, TreeUnexpand, Uncollapse, Unlock, Up, UpCircle, Upload, User, UserAdd, UserCircle, UserDelete, UserDisabled, Verify, VerticalAlignBottom, VerticalAlignCenter, VerticalAlignTop, ViewCard, ViewList, ViewThumbnail, Wait, Warning, Whitelist, Wifi, ZoomIn, ZoomOut, addIconDefinitions, fetchIconFormScript };
